<?php
/*******************************************************************************
 *	BDatabase.php		Database access class
 *	Copyright		Terry Barnaby,	BeamWeb,	2006-05-19
 *******************************************************************************
 */
require_once "BError.php";


function dbToInt($str){
	return intval($str);
}

function dbEscapeString($str){
	return mysqli_real_escape_string(BDatabase::$db_standard, $str);
}

class BDatabase {
	function __construct(&$sys){
		$this->state = $sys;
		$this->objClasses = array();
		$this->db = Null;
	}
	
	function __destruct(){
		mysqli_close($this->db);
	}

	function addObjectClasses($objClasses){
		$this->objClasses = array_merge($this->objClasses, $objClasses);
	}

	function getObjectClasses(){
		return $this->objClasses;
	}

	function open(){
		if(! $this->db = mysqli_connect($this->state->databaseHost, $this->state->databaseUser, $this->state->databasePassword, $this->state->databaseName))
			return new BError(1, mysqli_error($this->db));

		BDatabase::$db_standard = $this->db;
		
		// Disable strict mode so dates like "0000-00-00" can be used
		$this->query("set sql_mode = ''");

		return Null;
	}
	
	function reconnectIfNeeded(){
		if(mysqli_ping($this->db) == 0){
			mysqli_close($this->db);
			$this->open();
		}
	}
	
	function	getObjectListRaw($table, $fields = "", $sql = ""){
		$objList = array();
		$prefixedTable = $this->state->databasePrefix . $table;

		if($fields == "")
			$fields = "*";
		else
			$fields = dbEscapeString($fields);

		$sqlCmd = "select $fields from $prefixedTable";
		$sqlCmd = $sqlCmd . " " . $sql;

		if(self::$debug){		
			print "GetObjectList: SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		while($obj = mysqli_fetch_object($res)){
			$objList[] = $this->convertObject($table, $obj);
		}
		mysqli_free_result($res);
		return $objList;
	}

	function	getObjectList($table, $fields = "", $selection = "", $order = "", $limit = ""){
		$objList = array();
		$prefixedTable = $this->state->databasePrefix . $table;

		if($fields == "")
			$fields = "*";
		else
			$fields = dbEscapeString($fields);

		$sqlCmd = "select $fields from $prefixedTable";
//		print "getListObjects: Sel: $selection Order: $order<br>\n";
		
		if($selection != ""){
			$sqlCmd = $sqlCmd . " where " . $selection;
		} 
		if($order != ""){
			$sqlCmd = $sqlCmd . " order by " . dbEscapeString($this->orderProcess($order));
		}
		if($limit != ""){
			$sqlCmd = $sqlCmd . " limit " . dbEscapeString($limit);
		}

		if(self::$debug){		
			print "GetObjectList: SqlCmd: $sqlCmd<br>\n";
		}

		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		while($obj = mysqli_fetch_object($res)){
			$objList[] = $this->convertObject($table, $obj);
		}
		mysqli_free_result($res);
		return $objList;
	}

	function	getObjectFirst($table, $fields = "", $selection = "", $order = "", $limit = ""){
		$objList = $this->getObjectList($table, $fields, $selection, $order, "1");
		
		if(count($objList))
			return $objList[0];
		else
			return Null;
	}
	
	function	getObject($table, $id){
		$id = dbToInt($id);
		$prefixedTable = $this->state->databasePrefix . $table;
		$sqlCmd = "select * from $prefixedTable where id = $id";

		if(self::$debug){		
			print "SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		$obj = mysqli_fetch_object($res);
		if($obj)
			$obj = $this->convertObject($table, $obj);

		mysqli_free_result($res);

		return $obj;
	}

	function	getNewObject($table){
		if(isset($this->objClasses["$table"])){
			$obj = new $this->objClasses["$table"];
			$obj->id = 0;
		}
		else {
			$table = $this->state->databasePrefix . $table;
			$sqlCmd = "show columns from $table";

			if(self::$debug){		
				print "SqlCmd: $sqlCmd<br>\n";
			}
			$this->reconnectIfNeeded();
			$res = mysqli_query($this->db, $sqlCmd)
				or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

			$obj = new stdClass;

			while($o = mysqli_fetch_object($res)){
				$obj->{$o->Field} = "";
			}			
			mysqli_free_result($res);
		}
		
		return $obj;
	}
	
	function	appendObject($table, $obj){
		$table = $this->state->databasePrefix . $table;
		$s = "";
		foreach($obj as $k => $v){
			if(($k != "id") and ($k != "lastUpdate")){
				$v = dbEscapeString($v);
				if(strlen($s))
					$s = "$s, ";
				$s = "$s`$k`='$v'";  
			}
		}
		$sqlCmd = "insert into $table set $s";

		if(self::$debug){		
			print "SqlCmd: $sqlCmd<br>\n";
		}
//		if(!mysqli_query($this->db, $sqlCmd))
//			return False;
		$this->reconnectIfNeeded();
		mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		// Get Object id
		$obj->id = mysqli_insert_id($this->db);
		
		return $obj;
	}

	function	putObject($table, $obj){
		$obj->id = dbToInt($obj->id);
		$table = $this->state->databasePrefix . $table;
		$s = "";
		foreach($obj as $k => $v){
			if(($k != "id") and ($k != "lastUpdate")){
				$v = dbEscapeString($v);
				if(strlen($s))
					$s = "$s, ";
				$s = "$s`$k`='$v'";
			}
		}
		$sqlCmd = "update $table set $s where id = {$obj->id}";

		if(self::$debug){		
			print "SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);
		return $obj;
	}

	function	putObjectRaw($table, $obj){
		$obj->id = dbToInt($obj->id);
		$table = $this->state->databasePrefix . $table;
		$s = "";
		foreach($obj as $k => $v){
			if($k != "id"){
				$v = dbEscapeString($v);
				if(strlen($s))
					$s = "$s, ";
				$s = "$s`$k`='$v'";
			}
		}
		$sqlCmd = "update $table set $s where id = {$obj->id}";

		if(self::$debug){		
			print "SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);
		return $obj;
	}

	function	deleteObject($table, $id){
		$id = dbToInt($id);
		$prefixedTable = $this->state->databasePrefix . $table;
		$sqlCmd = "delete from $prefixedTable where id = '$id'";

		if(self::$debug){
			print "SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		return $res;
	}

	private function &convertObject($table, &$obj){
//		print "convertObject: $table "; print_r($this->objClasses); print "<br>\n";
		if(isset($this->objClasses["$table"])){
//			print "DoConvert for table: $table";
			$newObj = new $this->objClasses["$table"];
			$newObj->set($obj);
			return $newObj;
		}
		else {
			return $obj;
		}
	}

	function	getObjectNumber($table, $selection = ""){
		$prefixedTable = $this->state->databasePrefix . $table;

		$sqlCmd = "select count(id) from $prefixedTable";
		
		if($selection != ""){
			$sqlCmd = $sqlCmd . " where " . $selection;
		} 

		if(self::$debug){		
			print "GetObjectNumber: SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		$obj = mysqli_fetch_array($res);
		mysqli_free_result($res);
		return $obj[0];
	}

	function	query($sqlCmd){
		$rows = array();

		if(self::$debug){		
			print "Query: SqlCmd: $sqlCmd<br>\n";
		}
		
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd);

		if(is_a($res, "mysqli_result")){
			while($row = mysqli_fetch_row($res)){
				$rows[] = $row;
			}
			mysqli_free_result($res);
			return $rows;
		}
		else {
			return $res;
		}
	}
	
	function	queryObjs($table, $sqlCmd){
		$objList = array();

		if(self::$debug){		
			print "QueryObjs: SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		if(is_a($res, "mysqli_result")){
			while($obj = mysqli_fetch_object($res)){
				$objList[] = $this->convertObject($table, $obj);
			}
			mysqli_free_result($res);
			return $objList;
		}
		else {
			return $res;
		}
	}
	
	function transactionStart(){
		$sqlCmd = "start transaction";

		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd);
		
		return $res;
	}

	function transactionEnd($abort = 0){
		if($abort)
			$sqlCmd = "rollback";
		else
			$sqlCmd = "commit";

		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd);
		
		return $res;
	}

	function truncateTable($table){
		$prefixedTable = $this->state->databasePrefix . $table;
		$sqlCmd = "TRUNCATE TABLE " . $prefixedTable;

		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd);
		
		return $res;
	}

	function dropTable($table){
		$prefixedTable = $this->state->databasePrefix . $table;
		$sqlCmd = "DROP TABLE " . $prefixedTable;

		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd);
		
		return $res;
	}

	function createTable($table, $fields){
		$prefixedTable = $this->state->databasePrefix . $table;
		$sqlCmd = "CREATE TABLE " . $prefixedTable . " " . $fields . " ENGINE=MyISAM DEFAULT CHARSET=utf8;";

		if(self::$debug){
			print "SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);
		
		return $res;
	}

	function	tableExists($table){
		$prefixedTable = $this->state->databasePrefix . $table;

		$sqlCmd = "show tables like '{$prefixedTable}'";
		
		if(self::$debug){		
			print "TableExists: SqlCmd: $sqlCmd<br>\n";
		}
		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd)
			or trigger_error('SQL Error: '.mysqli_error($this->db), E_USER_ERROR);

		$list = mysqli_fetch_array($res);
		if((count($list) > 1) && ($list[1] = $prefixedTable))
			$r = 1;
		else
			$r = 0;

		return $r;
	}

	function getTableInfo($table){
		$prefixedTable = $this->state->databasePrefix . $table;
		$sqlCmd = "SHOW COLUMNS FROM " . $prefixedTable;

		$this->reconnectIfNeeded();
		$res = mysqli_query($this->db, $sqlCmd);
		
		if(is_a($res, "mysqli_result")){
			$rows = array();
			while($row = mysqli_fetch_row($res)){
				$rows[] = $row;
			}
			mysqli_free_result($res);
			return $rows;
		}
		else {
			return $res;
		}
	}
	
	function orderProcess($order){
		// Process order list with "!" symbols
		$ret = "";
		if($order != ""){
			$ol = explode(",", $order);
			foreach($ol as $o){
				// Make order component database safe
				$o = dbEscapeString(preg_replace("/[\s'`]/", "", $o));
				if($ret)
					$ret .= ",";
				if($o[0] == "!"){
					$ret .= "`" . substr($o, 1) . "`" . " desc";
				}
				else {
					$ret .= "`" . $o . "`";
				}
			}
		}
		
		return $ret;
	}


	
	public static	$debug = 0;
	public static	$db_standard;		
	private	$db;
	private	$sys;	
	private	$objClasses;
};
?>
