<?php
/*******************************************************************************
 *	BDate.php		Simple Date Object
 *	Copyright		Terry Barnaby,	BeamWeb,	2007-02-17
 *******************************************************************************
 */

class BDate {
	function __construct($str = "", $format = "%Y-%m-%d"){
		$this->set($str, $format);
	}
	
	function set($str = "", $format = "%Y-%m-%d"){
		if(is_array($str)){
			if($str[0] > 2037)
				$str[0] = 2037;
			$this->timeStamp = mktime(0, 0, 0, $str[1], $str[2], $str[0]);
		}
		else {
			if($str == ""){
				$this->timeStamp = 0;
			}
			else if($str == "0000-00-00"){
				$this->timeStamp = 0;
			}
			else if($str == "now"){
				$this->timeStamp = time();
			}
			else {
				$s = strptime($str, $format);
				if($s["tm_year"] > (2037 - 1900))
					$s["tm_year"] = (2037 - 1900);

				$this->timeStamp = mktime($s["tm_hour"], $s["tm_min"], $s["tm_sec"], $s["tm_mon"]+1, $s["tm_mday"], 1900 + $s["tm_year"]);
			}
		}
	}

	function setTime($hour, $minute, $second){
		$s = localtime($this->timeStamp, 1);
		$this->timeStamp = mktime($hour, $minute, $second, $s["tm_mon"]+1, $s["tm_mday"], 1900 + $s["tm_year"]);
	}
	
	function getDateTime($format = "%Y-%m-%d %H:%M:%S"){
		if($this->timeStamp == 0)
			return "";
		else
			return strftime($format, $this->timeStamp);
	}

	function getSeconds(){
		return $this->timeStamp;
	}
	
	function get($format = "%Y-%m-%d"){
		if($this->timeStamp == 0)
			return "";
		else
			return strftime($format, $this->timeStamp);
	}
	function getYear(){
		if($this->timeStamp == 0){
			return 0;
		}
		else {
			$s = localtime($this->timeStamp, 1);
			return $s["tm_year"] + 1900;
		}
	}
	function getMonth(){
		$s = localtime($this->timeStamp, 1);
		return $s["tm_mon"] + 1;
	}
	function getDay(){
		$s = localtime($this->timeStamp, 1);
		return $s["tm_mday"];
	}
	
	function addSeconds($sec){
		$this->timeStamp += $sec;
	}

	function addDays($days){
		$this->timeStamp += $days * 86400;
	}
	
	function addYears($years){
		$year = $this->getYear();
		while($years){
			if($this->isLeap($year))
				$this->addDays(366);
			else
				$this->addDays(365);
			$year++;
			$years--;
		}
	}

	static function monthName($m){
		$names = array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");

		return $names[$m-1];
	}

	static function isLeap($year){
		return (($year) % 4 == 0 && (($year) % 100 != 0 || ($year) % 400 == 0));
	}

	var	$timeStamp;
};

function bdateTest() {
	date_default_timezone_set("Europe/London");

	$d = new BDate("2005-03-20");
	$s = $d->get("%Y/%m/%d");
	print "Date: $s\n";
	print "Year: {$d->getYear()}, Mon: {$d->getMonth()}, Day: {$d->getDay()}\n";
	print "\n";
	
	$d->set();
	$s = $d->get("%Y/%m/%d");
	print "Date: $s\n";
	print "Year: {$d->getYear()}, Mon: {$d->getMonth()}, Day: {$d->getDay()}\n";
	print "\n";

	$d->set(array(2003, 11, 22));
	print "Date: {$d->get("%Y/%m/%d")}\n";
	print "Year: {$d->getYear()}, Mon: {$d->getMonth()}, Day: {$d->getDay()}\n";
	print "\n";

	$d->set(array(2999, 12, 31));
	print "Date: {$d->get("%Y/%m/%d")}\n";
	print "Year: {$d->getYear()}, Mon: {$d->getMonth()}, Day: {$d->getDay()}\n";
	print "\n";

	$d->set("2999-12-31");
	print "Date: {$d->get("%Y/%m/%d")}\n";
	print "Year: {$d->getYear()}, Mon: {$d->getMonth()}, Day: {$d->getDay()}\n";
	print "\n";

}

//bdateTest();

?>
