<?php
/*******************************************************************************
 *	BEdit.php		Edit objects
 *	Copyright:		Terry Barnaby,	BeamWeb,	2009-03-18
 *******************************************************************************
 */
require_once "BDate.php";

class BNameValue {
	function __construct($name = "", $value = ""){
		$this->name = $name;
		$this->value = $value;
	}
};

class BEditObj {
	function __construct($name = "", $title="", $saveUrl = "?cmd=save", $cancelUrl = "?cmd=cancel"){
		$this->name = $name;
		$this->title = $title;
		$this->saveUrl = $saveUrl;
		$this->cancelUrl = $cancelUrl;
		$this->saveButtonTitle = "Save";
		$this->cancelButtonTitle = "Cancel";
		$this->extra = array();
		$this->numColumns = 1;
		$this->namePrefix = "";
		$this->obj = new stdClass();
	}
	function setTitle($title){
		$this->title = $title;
	}
	function setSaveButtonTitle($title){
		$this->saveButtonTitle = $title;
	}
	function setCancelButtonTitle($title){
		$this->cancelButtonTitle = $title;
	}
	function setView($viewTypeList = array()){
		$this->viewTypeList = $viewTypeList;

		$a = array();
		foreach($this->viewTypeList as &$field){
			$a[$field->name] = "";
		}
		$this->obj = (object)$a;
	}
	function appendView($viewType){
		$this->viewTypeList[] = $viewType;

		$a = array();
		foreach($this->viewTypeList as &$field){
			$a[$field->name] = "";
		}
		$this->obj = (object)$a;
	}
	function setNumColumns($numColumns){
		$this->numColumns = $numColumns;
	}
	function setHelp($fieldName, $help){
		foreach($this->viewTypeList as &$field){
			if($field->name == $fieldName){
				$field->help = $help;
				break;
			}
		}
	}
	function setObj($obj){
		$this->obj = $obj;
	}
	function getObj(){
		return $this->obj;
	}
	
	function addExtra($text){
		$this->extra[] = $text;
	}
	function getField($fieldName){
		foreach($this->viewTypeList as &$field){
			if($field->name == $fieldName){
				return $field;
			}
		}
		return null;
	}
	
	function getHtml(){
		global	$sys;
		
		$viewTypeList = $this->viewTypeList;
		
		// Generate list of all object members
		if(count($this->obj) and count($viewTypeList) == 0){
			foreach($this->obj as $k => $v){
				$viewTypeList[] = new BViewType($k);
			}
		}
		$f = new BHtmlNode("form", "id='BEdit_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='{$this->saveUrl}' method='post'" );
		$t = &$f->appendNode("table", "class='BEdit' cellpadding='2' cellspacing='2' border='1'");
		
		if($this->title){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("th", "colspan=99 align=center", $this->title);
		}

		if(method_exists($this->obj, "validate"))
			$validateErrs = $this->obj->validate();

		if(count($this->obj)){
			// Do hidden objects first, they may be needed before other fields as for the "file" input
			foreach($viewTypeList as $field){
				$obj = $this->obj;
				$type = $field->type;
				$name = $this->namePrefix . $field->name;
				$value = $obj->{$field->name};
				if($type == "hidden"){
					$f->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
				}
			}
			
			$col = $this->numColumns;
			foreach($viewTypeList as $field){
				$obj = $this->obj;
				$type = $field->type;
				$name = $this->namePrefix . $field->name;
				$value = $obj->{$field->name};
				$style = "";
				$title = "";
				
				if($type == "hidden"){
					continue;
				}
				else if($type == ""){
					$type = "text";
				}
				
				if($col == $this->numColumns){
					$r = &$t->appendNode("tr");
					$col = 1;
				}
				else {
					$col += 1;
				}				
//				$r = &$t->appendNode("tr");

				if($field->title)
					$s = $field->title;
				else
					$s = ucfirst($field->name);

				if($field->help){
					$title = "title='{$field->help}'";
				}

				if($field->width){
					$style .= "width: {$field->width}px;";
				}
				if($field->height){
					$style .= "height: {$field->height}px;";
				}
				
				if($style != "")
					$style = "style='{$style}'";

				if($type != "empty"){
					if($type == "heading"){
						$r->appendNode("th", "align = 'left' colspan=2", $s);
					}
					else {
						$r->appendNode("th", "align = 'left'", $s);
						$c = &$r->appendNode("td");
					}
				}
					
				switch($type){
				case "empty":
					break;
				case "heading":
					break;
				case "link":
					eval("\$href = \"{$field->args[1]}\";");
					eval("\$value = \"{$field->args[0]}\";");
					$c->appendNode("a", "href=$href", $value);
					break;
				case 'textarea':
					$c->appendNode("textarea", "id='$name' name='$name' {$title} {$style}", $value);
					break;
				case 'textareaHtml':
					$args = array();
					if($field->width){
						$args['width'] = $field->width;
					}
					if($field->height){
						$args['height']	= $field->height;
					}
					$c->appendEditor($name, $value, $args);
					break;
				case 'textareaHtmlPics':
					$args = array();
					if($field->width){
						$args['width'] = $field->width;
					}
					if($field->height){
						$args['height']	= $field->height;
					}
					$args['type']	= "pics";
					$c->appendEditor($name, $value, $args);
					break;
				case 'textareaHtmlFull':
					$args = array();
					if($field->width){
						$args['width'] = $field->width;
					}
					if($field->height){
						$args['height']	= $field->height;
					}
					$args['type']	= "full";
					$c->appendEditor($name, $value, $args);
					break;
				case 'checkbox':
					if($value == "1")
						$c->appendNode("input", "type='checkbox' name='{$name}_checkbox' {$title}  value='$value' checked onchange='if(this.checked) document.{$this->name}.{$name}.value = 1; else document.{$this->name}.{$name}.value = 0;'");
					else
						$c->appendNode("input", "type='checkbox' name='{$name}_checkbox' {$title}  value='$value' onchange='if(this.checked) document.{$this->name}.{$name}.value = 1; else document.{$this->name}.{$name}.value = 0;'");
					$c->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
					break;
				case 'tristate':
					$c->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
					$c->appendNode("input", "type='checkbox' name='{$name}_checkbox' {$title} value='$value' onclick='editorTriStateUpdate(this, {$name}, 1);'");
					$c->appendNode("script", "", "editorTriStateUpdate(document.{$this->name}.{$name}_checkbox, document.{$this->name}.{$name}, 0);");
					break;
				case 'date':
					$c->appendNode("script", "", "editorDate('BEdit_{$this->name}','{$name}','{$value}', '{$field->help}');");
					break;
				case 'date1':
					$c->appendNode("script", "", "editorDate1('BEdit_{$this->name}','{$name}','{$value}');");
					break;
				case 'date2':
					$c->appendNode("script", "", "DateInput('{$name}', false, 'YYYY-MM-DD', '{$value}');");
					break;
				case 'select':
					$sel = &$c->appendNode("select", "id='$name' name='$name' {$title}  {$style}");
					foreach($field->args as $item){
						if($item == $value)
							$sel->appendNode("option", "value='{$item}' selected='selected'", "$item");
						else
							$sel->appendNode("option", "value='{$item}'", "$item");
					}
					break;

				case 'selectCombobox':
					$sel = &$c->appendNode("select", "name='{$name}_select' onchange='document.{$this->name}.{$name}.value = this.value;' {$title}  {$style}");
					foreach($field->args as $item){
						if($item == $value)
							$sel->appendNode("option", "value='{$item}' selected='selected'", "$item");
						else
							$sel->appendNode("option", "value='{$item}'", "$item");
					}
					$c->appendNode("input", "type='$type' name='{$name}_text' {$title}  value='' onchange='document.{$this->name}.{$name}.value = this.value;' size ='35' {$style}");
					$c->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
					break;

				case 'selectNameValue':
					$sel = &$c->appendNode("select", "id='$name' name='$name' {$title}  {$style}");
					foreach($field->args as $item){
						if($item->name == $value)
							$sel->appendNode("option", "value='{$item->name}' selected='selected'", $item->value);
						else
							$sel->appendNode("option", "value='{$item->name}'", $item->value);
					}
					break;
				case 'selectMulti':
					$sel = &$c->appendNode("select", "id='{$name}[]' name='{$name}[]' {$title} multiple property='vectList' size='4' {$style}");
					$v = explode(",", $value);
					foreach($field->args as $item){
						if(in_array($item, $v))
							$sel->appendNode("option", "value='{$item}' selected='selected'", $item);
						else
							$sel->appendNode("option", "value='{$item}'", $item);
					}
					break;
				case 'selectMultiHoriz':
					$o = implode(",", $field->args);
					$sel = &$c->appendNode("script", "", "editorSelectMultiHoriz('BEdit_{$this->name}', '{$name}', '$o', '{$value}', '{$field->help}');");
					break;
				case 'selectMultiCheckbox':
					$o = implode(",", $field->args);
					$sel = &$c->appendNode("script", "", "editorSelectMultiCheckbox('BEdit_{$this->name}', '{$name}', '$o', '{$value}', '{$field->help}');");
					break;
				case 'fileEdit':
					$c->appendNode("input", "type='text' id='$name' name='$name' value='$value' readonly='readonly' {$style}");
					$c->appendNode("input", "type='file' id='file_{$name}' name='file_{$name}' size ='35' {$style}");
					break;
				case 'fileUploader':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 0);");
					break;
				case 'fileUploaderLine':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 1);");
					break;
				case 'pictureUploaderLineLarge':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 1, 1024, 768);");
					break;
				case 'pictureUploaderLineMedium':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 1, 640, 480);");
					break;
				case 'pictureUploaderLineSmall':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 1, 320, 240);");
					break;
				case 'view':
					$c->appendNode("input", "type='$type' id='$name' name='$name' {$title}  value='$value' size ='35' {$style} readonly='readonly'");
					break;
				default:
					$c->appendNode("input", "type='$type' id='$name' name='$name' {$title}  value='$value' size ='35' {$style}");
					break;
				}

				if(isset($validateErrs["$name"])){
					$err = $validateErrs["$name"];
					$c->appendNode("span", "class='editError'", "*** $err ***");
				}
			}
		}
		
		// Add extra elements
		foreach($this->extra as $e){
			$f->appendText($e);
		}

		$f->appendNode("input", "type='submit' name='cmd' value='{$this->saveButtonTitle}'");
		if($this->cancelButtonTitle){
			if($this->cancelUrl != "")
				$f->appendNode("button", "type='button' name='cmd' onclick='location.href = \"{$this->cancelUrl}\";'", $this->cancelButtonTitle);
			else
				$f->appendNode("button", "type='button' name='cmd' onclick='window.history.go(-1); return false;'", $this->cancelButtonTitle);
		}
		return $f->render();
	}
	
	var	$name;							///< This Objects name
	var	$title;							///< The title to be displayed
	var	$obj;							///< The list of objects to display
	var	$viewTypeList;						///< The list of object fields to display
	var	$saveButtonTitle;					///< The save buttons title
	var	$cancelButtonTitle;					///< The save buttons title
	var	$namePrefix;
};



class BViewType1 {
	function __construct($name, $title = "", $type = "", $args = array(), $help=""){
		$this->name = $name;
		$this->title = $title;
		$this->type = $type;
		$this->args = $args;
		$this->selectType = "";
		$this->help = $help;
	}
	
	var	$name;		// Field name
	var	$title;		// Title to display, uses name if not set
	var	$editType;	// The type of editor to use
	var	$args;		// Aguments for viewType
	var	$selectType;	// The type of database selection to use for selections
	var	$help;		// Help information
};

class BViewTypeSelect {
	function __construct($name, $fieldName = "", $title = "", $selectType = "", $type = "", $args = array(), $help=""){
		$this->name = $name;
		$this->fieldName = $fieldName;
		$this->title = $title;
		$this->type = $type;
		$this->args = $args;
		$this->selectType = $selectType;
		$this->help = $help;
	}
	
	var	$name;		// Field name
	var	$title;		// Title to display, uses name if not set
	var	$fieldName;	// The actual field name if different from name.
	var	$editType;	// The type of editor to use
	var	$args;		// Aguments for viewType
	var	$selectType;	// The type of database selection to use for selections
	var	$help;		// Help information
};

class BEditObj1 {
	function __construct($name = "", $title="", $saveUrl = "?cmd=save", $cancelUrl = "?cmd=cancel"){
		global $sys;
		$this->sys = $sys;
		$this->name = $name;
		$this->title = $title;
		$this->saveUrl = $saveUrl;
		$this->cancelUrl = $cancelUrl;
		$this->saveButtonTitle = "Save";
		$this->cancelButtonTitle = "Cancel";
		$this->extra = array();
		$this->extraButtons = array();
		$this->numColumns = 1;
		$this->namePrefix = "";
		$this->pagerStart = 0;
		$this->pagerSize = 10;
		$this->pagerTotal = 0;
		$this->mode = "post";
		$this->obj = new stdClass();
	}
	function setFormMode($mode){
		$this->mode = $mode;
	}
	function setTitle($title){
		$this->title = $title;
	}
	function setSaveButtonTitle($title){
		$this->saveButtonTitle = $title;
	}
	function setCancelButtonTitle($title){
		$this->cancelButtonTitle = $title;
	}
	function setView($viewTypeList = array()){
		$this->viewTypeList = $viewTypeList;

		$a = array();
		foreach($this->viewTypeList as &$field){
			$a[$field->name] = "";
		}
		$this->obj = (object)$a;
	}
	function appendView($viewType){
		$this->viewTypeList[] = $viewType;

		$a = array();
		foreach($this->viewTypeList as &$field){
			$a[$field->name] = "";
		}
		$this->obj = (object)$a;
	}
	function addExtraButton($name, $title){
		$this->extraButtons[$name] = $title;
	}
	function setNumColumns($numColumns){
		$this->numColumns = $numColumns;
	}
	function setHelp($fieldName, $help){
		foreach($this->viewTypeList as &$field){
			if($field->name == $fieldName){
				$field->help = $help;
				break;
			}
		}
	}
	function setObj($obj){
		$this->obj = $obj;
	}
	function getObj(){
		return $this->obj;
	}
	
	function addExtra($text){
		$this->extra[] = $text;
	}
	function getField($fieldName){
		foreach($this->viewTypeList as &$field){
			if($field->name == $fieldName){
				return $field;
			}
		}
		return null;
	}
	
	function getHtml(){
		global	$sys;
		
		$viewTypeList = $this->viewTypeList;
		
		// Generate list of all object members
		if(count($this->obj) and count($viewTypeList) == 0){
			foreach($this->obj as $k => $v){
				$viewTypeList[] = new BViewType($k);
			}
		}
		if($this->mode == "get"){
			$f = new BHtmlNode("form", "id='BEdit_{$this->name}' name='{$this->name}' action='{$this->saveUrl}' method='get'" );
			if($this->saveUrl[0] = "?"){
				//print "BEditObj1::getHtml: '{$this->saveUrl}'</br>";
				foreach(preg_split("/[?&]+/", $this->saveUrl) as $arg){
					$nv = explode("=", $arg);
					$name = $nv[0];
					if(count($nv) == 2)
						$value = $nv[1];
					else
						$value = "";
					$f->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
				}
			}
		}
		else {
			$f = new BHtmlNode("form", "id='BEdit_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='{$this->saveUrl}' method='post'" );
		}
		$t = &$f->appendNode("table", "class='BEdit' cellpadding='2' cellspacing='2' border='1'");
		
		if($this->title){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("th", "colspan=99 align=center", $this->title);
		}

		if(method_exists($this->obj, "validate"))
			$validateErrs = $this->obj->validate();

		if(count($this->obj)){
			// Do hidden objects first, they may be needed before other fields as for the "file" input
			foreach($viewTypeList as $field){
				if($field->type == "empty")
					continue;

				$obj = $this->obj;
				$type = $field->type;
				$name = $this->namePrefix . $field->name;
				$value = $obj->{$field->name};
				if($type == "hidden"){
					$f->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
				}
			}
			
			$col = $this->numColumns;
			foreach($viewTypeList as $field){
				$obj = $this->obj;
				$type = $field->type;
				$name = $this->namePrefix . $field->name;
				$value = $obj->{$field->name};
				$style = "";
				$title = "";
				
//				print "BEditObj1::getHtml: do field: "; print_r($field); print "<br>";
				if($type == "hidden"){
					continue;
				}
				else if($type == ""){
					$type = "text";
				}
				
				if($col == $this->numColumns){
					$r = &$t->appendNode("tr");
					$col = 1;
				}
				else {
					$col += 1;
				}				
//				$r = &$t->appendNode("tr");

				if($field->title || $type == "empty")
					$s = $field->title;
				else
					$s = ucfirst($field->name);

				if($field->help){
					$title = "title='{$field->help}'";
				}

				if(isset($field->args["width"])){
					$style .= "width: {$field->args["width"]}px;";
				}
				if(isset($field->args["height"])){
					$style .= "height: {$field->args["height"]}px;";
				}
				
				if($style != "")
					$style = "style='{$style}'";

				if($type != "empty"){
					if($type == "heading"){
						$r->appendNode("th", "align = 'left' colspan=2", $s);
					}
					else {
						$r->appendNode("th", "align = 'left'", $s);
						$c = &$r->appendNode("td");
					}
				}
					
				switch($type){
				case "empty":
					break;
				case "heading":
					break;
				case "link":
					eval("\$href = \"{$field->args[1]}\";");
					eval("\$value = \"{$field->args[0]}\";");
					$c->appendNode("a", "href=$href", $value);
					break;
				case 'textarea':
					$c->appendNode("textarea", "id='$name' name='$name' {$title} {$style}", $value);
					break;
				case 'textareaHtml':
					$args = array();
					if(isset($field->args["width"])){
						$args['width'] = $field->args["width"];
					}
					if(isset($field->args["height"])){
						$args['height'] = $field->args["height"];
					}
					$c->appendEditor($name, $value, $args);
					break;
				case 'textareaHtmlFull':
					$args = array();
					if(isset($field->args["width"])){
						$args['width'] = $field->args["width"];
					}
					if(isset($field->args["height"])){
						$args['height'] = $field->args["height"];
					}
					$args['type']	= "full";
					$c->appendEditor($name, $value, $args);
					break;
				case 'checkbox':
					if($value == "1")
						$c->appendNode("input", "type='checkbox' name='{$name}_checkbox' {$title}  value='$value' checked onchange='if(this.checked) document.{$this->name}.{$name}.value = 1; else document.{$this->name}.{$name}.value = 0;'");
					else
						$c->appendNode("input", "type='checkbox' name='{$name}_checkbox' {$title}  value='$value' onchange='if(this.checked) document.{$this->name}.{$name}.value = 1; else document.{$this->name}.{$name}.value = 0;'");
					$c->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
					break;
				case 'tristate':
					$c->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
					$c->appendNode("input", "type='checkbox' name='{$name}_checkbox' {$title} value='$value' onclick='editorTriStateUpdate(this, {$name}, 1);'");
					$c->appendNode("script", "", "editorTriStateUpdate(document.{$this->name}.{$name}_checkbox, document.{$this->name}.{$name}, 0);");
					break;
				case 'date':
					$c->appendNode("script", "", "editorDate('BEdit_{$this->name}','{$name}','{$value}', '{$field->help}');");
					break;
				case 'date1':
					$c->appendNode("script", "", "editorDate1('BEdit_{$this->name}','{$name}','{$value}');");
					break;
				case 'date2':
					$c->appendNode("script", "", "DateInput('{$name}', false, 'YYYY-MM-DD', '{$value}');");
					break;
				case 'select':
					$sel = &$c->appendNode("select", "id='$name' name='$name' {$title}  {$style}");
					foreach($field->args as $item){
						if($item == $value)
							$sel->appendNode("option", "value='{$item}' selected='selected'", "$item");
						else
							$sel->appendNode("option", "value='{$item}'", "$item");
					}
					break;
				case 'selectMulti':
					$sel = &$c->appendNode("select", "id='{$name}[]' name='{$name}[]' {$title} multiple property='vectList' size='4' {$style}");
					
					if(is_array($value))
						$v = $value;
					else
						$v = explode(",", $value);

					foreach($field->args as $item){
						if(in_array($item, $v))
							$sel->appendNode("option", "value='{$item}' selected='selected'", $item);
						else
							$sel->appendNode("option", "value='{$item}'", $item);
					}
					break;
				case 'selectMulti8':
					$sel = &$c->appendNode("select", "id='{$name}[]' name='{$name}[]' {$title} multiple property='vectList' size='8' {$style}");
					
					if(is_array($value))
						$v = $value;
					else
						$v = explode(",", $value);

					foreach($field->args as $item){
						if(is_array($item)){
							$atrribs = "";
							if(in_array($item[0], $v))
								$atrribs .= " selected='selected'";
							if($item[1] != "")
								$atrribs .= " " . $item[1];

							$sel->appendNode("option", $atrribs, $item[0]);	
						}
						else {
							if(in_array($item, $v))
								$sel->appendNode("option", "value='{$item}' selected='selected'", $item);
							else
								$sel->appendNode("option", "value='{$item}'", $item);
						}
					}
					break;
				case 'selectMultiHoriz':
					$o = implode(",", $field->args);
					$sel = &$c->appendNode("script", "", "editorSelectMultiHoriz('BEdit_{$this->name}', '{$name}', '$o', '{$value}', '{$field->help}');");
					break;
				case 'selectMultiCheckbox':
					$o = implode(",", $field->args);
					$sel = &$c->appendNode("script", "", "editorSelectMultiCheckbox('BEdit_{$this->name}', '{$name}', '$o', '{$value}', '{$field->help}');");
					break;
				case 'fileUploader':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 0);");
					break;
				case 'fileUploaderLine':
					$c->appendNode("script", "", "new FileUploaderForm('$name', 1);");
					break;
				case 'pager':
					$c->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
					$start = 1 + intval($value) / $this->pagerSize;
					$c->appendText("{$start}");
					$total = intval(($this->pagerTotal + $this->pagerSize - 1) / $this->pagerSize);
					$c->appendText(" of {$total}");
					$c->appendText("&nbsp;");
					$prev = intval($value) - $this->pagerSize;
					if($prev < 0)
						$prev = 0;
					$next = intval($value) + $this->pagerSize;
					$c->appendNode("button", "type='button' name='pageStart' onclick='document.{$this->name}.{$name}.value = 0; document.{$this->name}.submit();'", "Start");
					$c->appendNode("button", "type='button' name='pagePrev' onclick='document.{$this->name}.{$name}.value = {$prev}; document.{$this->name}.submit();'", "Prev");
					$c->appendNode("button", "type='button' name='pageNext' onclick='document.{$this->name}.{$name}.value = {$next}; document.{$this->name}.submit();'", "Next");
					break;

				default:
					$c->appendNode("input", "type='$type' id='$name' name='$name' {$title}  value='$value' size ='18' {$style}");
					break;
				}

				if(isset($validateErrs["$name"])){
					$err = $validateErrs["$name"];
					$c->appendNode("span", "class='editError'", "*** $err ***");
				}
			}
		}
		
		// Add extra elements
		foreach($this->extra as $e){
			$f->appendText($e);
		}

		$f->appendNode("input", "type='submit' class='Button' name='select_cmd' value='{$this->saveButtonTitle}'");
		if($this->cancelButtonTitle){
			if($this->cancelUrl != "")
				$f->appendNode("button", "type='button' class='Button' name='cmd' onclick='location.href = \"{$this->cancelUrl}\";'", $this->cancelButtonTitle);
			else
				$f->appendNode("button", "type='button' class='Button' name='cmd' onclick='location.href = \"javascript:history.go(-1)\";'", $this->cancelButtonTitle);
		}

		foreach($this->extraButtons as $bn => $bt){
			$f->appendNode("input", "type='submit' class='Button' name='{$bn}' value='{$bt}'");
		}

		return $f->render();
	}
	
	var	$name;							///< This Objects name
	var	$title;							///< The title to be displayed
	var	$obj;							///< The list of objects to display
	var	$viewTypeList;						///< The list of object fields to display
	var	$saveButtonTitle;					///< The save buttons title
	var	$cancelButtonTitle;					///< The save buttons title
	var	$extraButtons;						///< Extra buttons list
	var	$namePrefix;
};


class BSelect extends BEditObj1 {
	function __construct($name = "", $title="Select", $saveUrl = "", $columns = 4){
		parent::__construct($name, $title, $saveUrl, "");
		$this->setNumColumns($columns);
		$this->setSaveButtonTitle("Search");
		$this->setCancelButtonTitle("");
		$this->setObj((object)array());
		$this->namePrefix = "sel_";
	}
	
	function update(){
		if($this->mode == "get"){
			foreach($_GET as $k => $v){
				if(!strncmp($k, $this->namePrefix, strlen($this->namePrefix))){
					$k = substr($k, strlen($this->namePrefix));
					if(is_array($v))
						$this->obj->{$k} = filterArray($v);
					else
						$this->obj->{$k} = filterString($v);
				}
			}
		}
		else {
			foreach($_POST as $k => $v){
				if(!strncmp($k, $this->namePrefix, strlen($this->namePrefix))){
					$k = substr($k, strlen($this->namePrefix));
					if(is_array($v))
						$this->obj->{$k} = filterArray($v);
					else
						$this->obj->{$k} = filterString($v);
				}
			}
		}
	}

	function postUpdate(){
		$this->update();
	}

	function pagerSet($start, $size, $total = 0){
		$this->pagerStart = $start;
		$this->pagerSize = $size;
		$this->pagerTotal = $total;
	}

	function pagerGetLimitString(){
		if($this->obj->pager == "")
			$this->obj->pager = 0;
		return "{$this->obj->pager},{$this->pagerSize}";
	}
	
	function getOrder(){
		if(isset($this->obj->order))
			return $this->obj->order;
		else
			return "";
	}
	
	function getSelectString(){
		$where = "";
		
		foreach($this->viewTypeList as $field){
			if($field->name == "")
				continue;

			if($field->fieldName == "none")
				continue;
				
			if($field->fieldName != "")
				$n = $field->fieldName;
			else
				$n = $field->name;
			$type = $field->selectType;
			$v = $this->obj->{$field->name};

			if(($type != "") && (strval($v) != "")){
				if($where != "")
					$where .= " and ";
				switch($type){
				case "equal":
					if($v[0] == "!"){
						$v = substr($v, 1);
						$where .= "`{$n}` != '{$v}'";
					}
					else {
						$where .= "`{$n}` = '{$v}'";
					}
					break;
				case "notEqual":
					$where .= "`{$n}` != '{$v}'";
					break;
				case "greaterThan":
					$where .= "`{$n}` > '{$v}'";
					break;
				case "greaterThanEqual":
					$where .= "`{$n}` >= '{$v}'";
					break;
				case "lessThan":
					$where .= "`{$n}` < '{$v}'";
					break;
				case "lessThanEqual":
					$where .= "`{$n}` <= '{$v}'";
					break;
				case "isset":
					if($v == "0")
						$where .= "`{$n}` = '0'";
					else if($v == "1")
						$where .= "`{$n}` > '0'";
					else
						$where .= "(1)";
					break;

				case "isZero":
					if($v == "1")
						$where .= "`{$n}` = '0'";
					else
						$where .= "'1' = '1'";
					break;

				case "isNotZero":
					if($v == "1")
						$where .= "`{$n}` != '0'";
					else
						$where .= "'1' = '1'";
					break;

				case "period":
					$date = new BDate("now");
					$date->setTime(0, 0, 0);
					$ed = $date;
					$ed->addSeconds(86400);
					$endDate = $ed->get();
					if($v == "Last Year"){
						$date->addSeconds(-86400 * 365);
						$startDate = $date->get();
					}
					else if($v == "Last 6 Months"){
						$date->addSeconds(-86400 * 183);
						$startDate = $date->get();
					}
					else if($v == "Last 2 Months"){
						$date->addSeconds(-86400 * 2 * 31);
						$startDate = $date->get();
					}
					else if($v == "Last Month"){
						$date->addSeconds(-86400 * 31);
						$startDate = $date->get();
					}
					else if($v == "Last Week"){
						$date->addSeconds(-86400 * 7);
						$startDate = $date->get();
					}
					else if($v == "Next Week"){
						$startDate = $date->get();;
						$date->addSeconds(86400 * 7);
						$endDate = $date->get();
					}
					else if($v == "Next Month"){
						$startDate = $date->get();
						$date->addSeconds(86400 * 31);
						$endDate = $date->get();
					}
					else if($v == "To Next Month"){
						$date->addSeconds(86400 * 31);
						$endDate = $date->get();
						$date->set("1970-01-01");
						$startDate = $date->get();
					}
					else if($v == "To Next Two Months"){
						$date->addSeconds(2 * 86400 * 31);
						$endDate = $date->get();
						$date->set("1970-01-01");
						$startDate = $date->get();
					}
					else if($v == "To Next Three Months"){
						$date->addSeconds(3 * 86400 * 31);
						$endDate = $date->get();
						$date->set("1970-01-01");
						$startDate = $date->get();
					}
					else if($v == "To Now"){
						$date->set("1970-01-01");
						$startDate = $date->get();
					}
					else if($v == "Over 1 Months ago"){
						$startDate = "0001-01-01";
						$date->addSeconds(-86400 * 1 * 31);
						$endDate = $date->get();
					}
					else if($v == "Over 2 Months ago"){
						$startDate = "0001-01-01";
						$date->addSeconds(-86400 * 2 * 31);
						$endDate = $date->get();
					}
					else if($v == "Over 3 Months ago"){
						$startDate = "0001-01-01";
						$date->addSeconds(-86400 * 3 * 31);
						$endDate = $date->get();
					}

					$where .= "(`$n` >= '{$startDate}' and `$n` <= '{$endDate}')";
					break;
					
				case "lastMonth":
					$endDate = new BDate("now");
					$startDate = clone $endDate;
					$startDate->addSeconds(-86400 * 31);
					if($v == "1"){
						$where .= "(`$n` >= '{$startDate->get()}' and `$n` <= '{$endDate->get()}')";
					}
					else {
						$where .= "(`$n` < '{$startDate->get()}' or `$n` > '{$endDate->get()}')";
					}
					break;

				case "last2Months":
					$endDate = new BDate("now");
					$startDate = clone $endDate;
					$startDate->addSeconds(-86400 * 2 * 31);
					if($v == "1"){
						$where .= "(`$n` >= '{$startDate->get()}' and `$n` <= '{$endDate->get()}')";
					}
					else {
						$where .= "(`$n` < '{$startDate->get()}' or `$n` > '{$endDate->get()}')";
					}
					break;

				case "inList":
					if($v == "ANY")
						$v = "_%";
					$where .= "((`{$n}` like '%{$v},%') or (`{$n}` like '%{$v}'))";
					break;

				case "like":
					$where .= "`{$n}` like '%{$v}%'";
					break;
				case "likeAfterStart":
					$where .= "`{$n}` like '{$v}%'";
					break;
				case "postcode":
					if(1 === preg_match('~[0-9\s]~', $v)){
						// Has digits
						$where .= "`{$n}` rlike '^{$v}'";
					}
					else {
						// Just characters
						$where .= "`{$n}` rlike '^{$v}[0-9]'";
					}
					break;
				case "pager":
				case "order":
					$where .= "(1)";
					break;
				case "userId":
					$v = $this->sys->user->id;
					$where .= "`{$n}` = '{$v}'";
					break;
				}
			}
		}
		return $where;
	}
};

class BEditList {
	function __construct($name = "", $title=""){
		$this->name = $name;
		$this->title = $title;
		$this->addUrl = "?cmd={$name}Add";
		$this->saveUrl = "?cmd={$name}Save";
		$this->cancelUrl = "";
		$this->addButtonTitle = "Add";
		$this->saveButtonTitle = "Save";
		$this->cancelButtonTitle = "Cancel";
		$this->mode = "post";
		$this->extra = array();
		$this->extraButtons = array();
		$this->selection = "";
		$this->order = "";
		$this->group = "";
		$this->extraRows = array();

		if(isset($_GET["{$this->name}_order"]))
			$this->order = filterStringParamList($_GET["{$this->name}_order"]);
	}
	function setName($name){
		$this->name = $name;
	}
	function setView($viewTypeList = array()){
		$this->viewTypeList = $viewTypeList;
	}
	function setList($list, $noOrder = 0){
		$this->list = $list;
		if(!$noOrder && $this->order){
			uasort($this->list, array("BViewList", "sortCompare"));
		}
	}
	function getList(){
		return $this->list;
	}
	function setSelection($selection){
		$this->selection = $selection;
	}
	function setOrder($order){
		$this->order = $order;
	}
	function setGroup($group){
		$this->group = $group;
	}
	function addExtraRow($obj){
		$this->extraRows[] = $obj;
	}
	
	function getHtml(){
		$line = 0;
		$viewTypeList = $this->viewTypeList;
			
		if($this->group){
			$groupCmd = "&group={$this->group}";
			$groupCmd1 = "?group={$this->group}";
		}
		else {
			$groupCmd = "";
			$groupCmd1 = "";
		}

		// Generate list of all object members
		if(count($this->list) and count($viewTypeList) == 0){
			foreach($this->list[0] as $k => $v){
				$viewTypeList[] = new BViewType($k);
			}
		}
		
		if($this->mode == "get")
			$f = new BHtmlNode("form", "id='BEdit_{$this->name}' name='{$this->name}' action='{$this->saveUrl}' method='get'" );
		else
			$f = new BHtmlNode("form", "id='BEdit_{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='{$this->saveUrl}' method='post'" );

		$t = &$f->appendNode("table", "class='BViewList' cellpadding='2' cellspacing='2' border='1'");

		if(count($viewTypeList)){
			if($this->order and ($this->order[0] == "!"))
				$order = substr($this->order, 1);
			else
				$order = $this->order;

			$r = &$t->appendNode("tr");
			$c = &$r->AppendNode("th", "", "Action");
			
			foreach($viewTypeList as $field){
				$c = &$r->AppendNode("th");
				if($field->title)
					$s = $field->title;
				else
					$s = ucfirst($field->name);
				
				if($order == $field->name){
					if($this->order[0] != "!")
						$o = "!{$order}";
					else
						$o = $order;
				}
				else {
					$o = $field->name;
				}
				$p = "href='?{$this->name}_order={$o}'";
				$c->appendNode("a", "$p", $s);
			}
		}

		foreach($this->list as $obj){
//			print "DoObj:"; print_r($obj);
			$r = &$t->appendNode("tr");
			
			$c = &$r->AppendNode("td");
			$c->appendNode("a", "href='?cmd={$this->name}Add&id={$obj->id}{$groupCmd}' title='Add Item'", "Add");
			$c->appendText("&nbsp;");
			
			$c->appendNode("input", "type='submit' class='ButtonLink' name='save1' value='Save'");
			//$c->appendNode("a", "href='?cmd={$this->name}Save&id={$obj->id}{$groupCmd}' title='Save Item' onclick='document.getElementById(\"BEdit_{$this->name}\").submit();'", "Save");
			$c->appendText("&nbsp;");
			$c->appendNode("a", "href='?cmd={$this->name}Delete&id={$obj->id}{$groupCmd}' title='Delete Item'", "Delete");

			$name = "objects[{$line}][id]";
			$c->appendNode("input", "type='hidden' id='$name' name='$name' value='{$obj->id}'");

			foreach($viewTypeList as $field){
				$type = $field->type;
				$name = "objects[{$line}][{$field->name}]";
				$value = strval($obj->{$field->name});
				$style = "";
				
				if(isset($field->args["width"])){
					$style .= "width: {$field->args["width"]}px;";
				}
				if(isset($field->args["height"])){
					$style .= "height: {$field->args["height"]}px;";
				}
				
				if($style != "")
					$style = "style='{$style}'";
				
				if($type != "empty"){
					$c = &$r->appendNode("td");
				}

				switch($type){
				case "empty":
					break;
				default:
					$c->appendNode("input", "type='$type' id='$name' name='$name' value='$value' size ='18' {$style}");
					break;
				}
			}
			$line++;
		}

		// Add extra elements
		foreach($this->extra as $e){
			$f->appendText($e);
		}

		$f->appendNode("button", "type='button' class='Button' name='cmd' onclick='location.href = \"{$this->addUrl}\";'", $this->addButtonTitle);
		$f->appendNode("input", "type='submit' class='Button' name='cmd' value='{$this->saveButtonTitle}'");
		if(0 && $this->cancelButtonTitle){
			if($this->cancelUrl != "")
				$f->appendNode("button", "type='button' class='Button' name='cmd' onclick='location.href = \"{$this->cancelUrl}\";'", $this->cancelButtonTitle);
			else
				$f->appendNode("button", "type='button' class='Button' name='cmd' onclick='location.href = \"javascript:history.go(-1)\";'", $this->cancelButtonTitle);
		}

		foreach($this->extraButtons as $bn => $bt){
			$f->appendNode("input", "type='submit' class='Button' name='{$bn}' value='{$bt}'");
		}

		return $f->render();
	}

	function sortCompare($a, $b){
		if($this->order[0] == "!"){
			$order = substr($this->order, 1);
			
			if($a->{$order} == $b->{$order})
				return 0;
			else
				return ($a->{$order} > $b->{$order}) ? -1 : 1;
		}
		else {
			$order = $this->order;

			if($a->{$order} == $b->{$order})
				return 0;
			else
				return ($a->{$order} < $b->{$order}) ? -1 : 1;
		}
	}
	
	private		$name;			// This Objects name
	private		$mode;
	protected	$list;			// The list of objects to display
	protected	$viewTypeList;		// The list of object fields to display
	protected	$filter;		// An expression to select objects based on field values
	protected	$selection;		// The order which to display
};

class BEditDatabase extends BEditList {
	function setTable($table){
		global $sys;

		if($this->limit){
			$limitStr = "{$this->limit}";
		}
		else {
			$limitStr = "";
		}

//		BDataBase::$debug = 1;
		$this->setList($sys->db->getObjectList($table, "", $this->selection, $this->order, $limitStr), 1);
	}
	
	function setLimit($limit){
		$this->limit = $limit;
	}
	
	function setRange($posStart, $posEnd){
		$this->posStart = $posStart;
		$this->posEnd = $posEnd;
	}
	
	var	$limit = 0;
};


?>
