<?php
/*******************************************************************************
 *	BHtml.php		Generates HTML
 *	Copyright		Terry Barnaby,	BeamWeb,	2004-12-16
 *******************************************************************************
 */
require_once("simple_html_dom.php");

function htmlEncode($str){
	return str_replace("'", "&#39;", $str);
}

class BHtmlNode {
	function __construct($tag, $args = "", $text = ""){
		$this->tag = $tag;
		$this->args = $args;
		$this->nodes = array();
		if($text != "")
			$this->appendText($text);
//		print("BHtmlNode: $tag, $args\n");
	}
	
	function &append($node){
		$this->nodes[] = &$node;
		return $node;
	}
	
	function &appendNode($tag, $args = "", $text = ""){
		$node = new BHtmlNode($tag, $args);
		$this->nodes[] = &$node;
		if($text != "")
			$node->appendText($text);
		return $node;
	}

	function &appendText($args){
		$node = new BHtmlNode("", $args);
		$this->nodes[] = &$node;
		return $node;
	}

	function appendTable($table, $sortUrl = ""){
		$t = new BHtmlNode("table", "cellpadding='2' cellspacing='2' border='1'");
		$r = &$t->appendNode("tr");
		foreach($table->colNames as $v){
			$c = &$r->appendNode("th");
			if($sortUrl != ""){
				$a = &$c->appendNode("a", "href=" . $sortUrl . "?tableSort=$v");
				$a->appendText($v);
			}
			else {
				$c->appendText($v);
			}
		}

		foreach($table->rows as $rd){
			$r = &$t->appendNode("tr");
			foreach($rd as $cd){
				$c = &$r->appendNode("td");
				$a = &$c->appendText($cd);
			}
		}
		$this->nodes[] = &$t;
	}
	
	function appendInput($name, $type, $value = "", $style="", $args = Null, $onchange = Null){
		global	$sys;

		$value = htmlEncode($value);

		switch($type){
		case 'select':
			if($onchange)
				$onchange = "onchange='{$onchange}'";
			$sel = &$this->appendNode("select", "id='$name' name='$name' {$onchange}");
			foreach($args as $a){
				if($a == $value)
					$sel->appendNode("option", "value='{$a}' selected='selected'", $a);
				else
					$sel->appendNode("option", "value='{$a}'", $a);
			}
			break;
		case 'selectCombobox':
			$sel = &$this->appendNode("select", "name='{$name}_select' onchange='document.getElementById(\"{$name}\").value = this.value;' {$title}  {$style}");
			foreach($args as $a){
				if($a == $value)
					$sel->appendNode("option", "value='{$a}' selected='selected'", "$a");
				else
					$sel->appendNode("option", "value='{$a}'", "$a");
			}
			$this->appendNode("input", "type='$type' name='{$name}_text' {$title}  value='' onchange='document.getElementById(\"{$name}\").value = this.value;' size ='35' {$style}");
			$this->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
			break;

		case 'checkbox':
			if($value == "1")
				$this->appendNode("input", "type='checkbox' name='{$name}_checkbox' value='$value' checked onchange='if(this.checked) document.getElementById(\"{$name}\").value = 1; else document.getElementById(\"{$name}\").value = 0; {$onchange}'");
			else
				$this->appendNode("input", "type='checkbox' name='{$name}_checkbox' value='$value' onchange='if(this.checked) document.getElementById(\"{$name}\").value = 1; else document.getElementById(\"{$name}\").value = 0; {$onchange}'");
			$this->appendNode("input", "type='hidden' id='{$name}' name='$name' value='$value'");
			break;
		case 'date':
			$this->appendNode("script", "", "editorDate('', '$name', '$value', '', '$onchange');");
			break;
		case 'textarea':
			if($onchange)
				$onchange = "onchange='{$onchange}'";
			$this->appendNode("textarea", "id='$name' name='$name' style='{$style}' {$onchange}", $value);
			break;
		case 'textareaHtml':
			$this->appendEditor($name, $value, $args);
			break;
		case 'file':
			$this->appendNode("input", "type='file' id='$name' name='$name' value='$value' size=40 style='{$style}'");
			break;
		case 'textView':
			$this->appendNode("div", "style='{$style}'", "{$value}");
			$this->appendNode("input", "type='hidden' id='$name' name='$name' value='$value'");
			break;
		case 'view':
			$this->appendNode("input", "type='text' id='$name' name='$name' readonly value='$value' style='{$style}'");
			break;
		case 'view_textarea':
			$this->appendNode("textarea", "type='text' id='$name' name='$name' readonly style='{$style}'", $value);
			break;
		case 'hidden':
			$this->appendNode("input", "type='hidden' id='$name' name='$name' value='$value' style='{$style}'");
			break;
		default:
			if($onchange)
				$onchange = "onchange='{$onchange}'";
			$this->appendNode("input", "type='text' id='$name' name='$name' value='$value' style='{$style}' {$onchange}");
			break;
		}
	}

	function appendTableNameValue($name, $value, $args=""){
		$row = &$this->appendNode("tr");
		$row->appendNode("th", "align = 'left'", $name);
		$c = $row->appendNode("td", $args, $value);
	}

	function appendTableInput($title, $name, $type, $value = "", $style="", $args = Null, $onchange = Null){
		$row = &$this->appendNode("tr");
		if($title != "")
			$row->appendNode("th", "align = 'left'", $title);
		$c = $row->appendNode("td");
		$c->appendInput($name, $type, $value, $style, $args, $onchange);
	}

	function appendTableRowInput($title, $name, $type, $value = "", $style="", $args = Null, $onchange = Null){
		if($title != "")
			$this->appendNode("th", "align = 'left'", $title);
		$c = $this->appendNode("td");
		$c->appendInput($name, $type, $value, $style, $args, $onchange);
	}
	
	function appendEditor($name, $value = "", $args = Null){
		global $sys;
		
		$h = &$this;
		$text = htmlspecialchars($value);

		if(isset($args["type"]) && ($args["type"] == "full")){
			$toolbar = "[ ['Save','Print','-','Templates'],['Cut','Copy','Paste','PasteText','PasteFromWord','-','Undo','Redo']";
			$toolbar .= ",['Find','Replace','-','SelectAll','-','SpellChecker', 'Scayt' ],['Bold','Italic','Underline','Strike','Subscript','Superscript','-','RemoveFormat' ]";
			$toolbar .= ",['TextColor','BGColor'],['Source', 'Maximize', 'ShowBlocks','-','About' ]";
			$toolbar .= ",['Styles','Format','Font','FontSize'],['NumberedList','BulletedList','-','Outdent','Indent','-','Blockquote','CreateDiv','-','JustifyLeft','JustifyCenter','JustifyRight','JustifyBlock']";
			$toolbar .= ",['Link','Unlink','Anchor'], ['Image','Table','HorizontalRule','Smiley','SpecialChar','PageBreak']";
			$toolbar .= "]";
		}
		else if(isset($args["type"]) && ($args["type"] == "pics")){
			$toolbar = "[ ['Format', 'Bold','Italic', '-', 'NumberedList','BulletedList', '-', 'Link','Unlink','Smiley']";
			$toolbar .= ",['Maximize', 'ShowBlocks','-','About']";
			$toolbar .= ",['SpellChecker', 'Scayt', 'Image', 'Table']";
			$toolbar .= "]";
		}
		else if(isset($args["type"]) && ($args["type"] == "source")){
			$toolbar = "[ ['Format', 'Bold','Italic', '-', 'NumberedList','BulletedList', '-', 'Link','Unlink','Smiley']";
			$toolbar .= ",['Source', 'Maximize', 'ShowBlocks','-','About']";
			$toolbar .= ",['SpellChecker', 'Scayt']";
			$toolbar .= "]";
		}
		else {
			$toolbar = "[ ['Format', 'Bold','Italic', '-', 'NumberedList','BulletedList', '-', 'Link','Unlink','Smiley']";
			$toolbar .= ",['Maximize', 'ShowBlocks','-','About']";
			$toolbar .= ",['SpellChecker', 'Scayt']";
			$toolbar .= "]";
		}
		$h->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/lib/ckeditor/ckeditor.js'");
		$h->appendNode("textarea", "name='{$name}' id='{$name}'", $text);
		if(isset($args["width"])){
			$v = $args["width"];
			$h->appendNode("script", "", "CKEDITOR.config.width = '{$v}';");
		}
		if(isset($args["height"])){
			$v = $args["height"];
			$h->appendNode("script", "", "CKEDITOR.config.height = '{$v}';");
		}
		$h->appendNode("script", "", "CKEDITOR.config.resize_dir = 'both';");
		$h->appendNode("script", "", "CKEDITOR.config.skin = 'kama';");
		$h->appendNode("script", "", "CKEDITOR.config.allowedContent = true;");
		$h->appendNode("script", "", "CKEDITOR.config.sourceAreaTabSize = 2;");
		$h->appendNode("script", "", "CKEDITOR.config.enterMode = 'CKEDITOR.ENTER_BR';");
		$h->appendNode("script", "", "CKEDITOR.config.contentCss = '';");
		$h->appendNode("script", "", "CKEDITOR.config.fullPage = false;");
		$h->appendNode("script", "", "CKEDITOR.config.toolbar = {$toolbar};");
		$h->appendNode("script", "", "CKEDITOR.config.filebrowserBrowseUrl = '{$sys->baseUrl}/lib/FileBrowser/fileBrowser.php?type=files&workDir={$sys->workDir}';");
		$h->appendNode("script", "", "CKEDITOR.config.filebrowserImageBrowseUrl = '{$sys->baseUrl}/lib/FileBrowser/fileBrowser.php?type=images&workDir={$sys->workDir}';");
		$h->appendNode("script", "", "CKEDITOR.replace('{$name}');");
	}

	function render($singleLine = 0){
		$str = "";
		$term = ">";

		if($singleLine)
			$end = "";
		else
			$end = "\n";
		
		$node = $this;
		
		if($node->tag == ""){
			$str = $str . $node->args;
		}
		else {
			switch($node->tag){
			case "br":
			case "link":
			case "input":
			case "img":
				$term = "/>";
				break;
			}
			if($node->args != ""){
				$str = $str . "<" . $node->tag . " " . $node->args . $term;
			}
			else {
				$str = $str . "<" . $node->tag . $term;
			}
//			if($node->tag == "table")
//				$str = $str . "<tbody>" . $end;
			if($node->tag == "div")
				$str = $str . $end;

			switch($node->tag){
			case "html":
			case "head":
			case "body":
				$str = $str . $end;
				break;
			}
			
			foreach($node->nodes as $n){
				$str = $str . $n->render($singleLine);
			}
//			if($node->tag == "table")
//				$str = $str . "</tbody>";

			switch($node->tag){
			case "br":
			case "link":
			case "input":
			case "img":
				break;
			default:			
				$str = $str . "</" . $node->tag . ">";
			}

			switch($node->tag){
			case "t":
			case "table":
			case "div":
			case "li":
			case "tr":
			case "head":
			case "body":
			case "style":
			case "script":
				$str = $str . $end;
				break;
			}
		}
		return $str;
	}	

	var	$tag;
	var	$args;
	var	$nodes;
}

class BHtmlPager {
	function __construct($number = 100, $total = 0){
		$this->start = 0;;
		$this->number = $number;
		$this->total = $total;
	}
	
	function setTotal($total = 0){
		$this->total = $total;
	}
	
	function getRange(){
		if(isset($_GET["start"]))
			$this->start = intval($_GET["start"]);
		else
			$this->start = 0;
			
		return "{$this->start},{$this->number}";
	}
	
	function getHtml($url){
		$page = "";
		if(isset($_GET["start"]))
			$this->start = intval($_GET["start"]);
		else
			$this->start = 0;
		$prev = $this->start - $this->number;
		if($prev < 0)
			$prev = 0;
		if(($this->start + $this->number) < $this->total)
			$next = $this->start + $this->number;
		else
			$next = $this->start;
		$last = $this->total - $this->number;
		if($last < 0)
			$last = 0;
	
		$h = new BHtmlNode("div", "class=Pager");
		$h->appendText("Pager: pos: {$this->start} of {$this->total}&nbsp;");
		$h->appendNode("button", "type='button' name='pageStart' onclick='window.location.href=\"{$url}&start=0\"'", "Start");
		$h->appendNode("button", "type='button' name='pagePrev' onclick='window.location.href=\"{$url}&start={$prev}\"'", "Prev");
		$h->appendNode("button", "type='button' name='pageNext' onclick='window.location.href=\"{$url}&start={$next}\"'", "Next");
		$h->appendNode("button", "type='button' name='pageLast' onclick='window.location.href=\"{$url}&start={$last}\"'", "Last");
		$page .= $h->render();

		return $page;
	}
};

function htmlParseOld($htmlFileName) {
	$html = array();

	$content = file_get_contents($htmlFileName);

	preg_match("/<body.*?>(.*)<\/body>/si", $content, $match);
	$html['body'] = array_key_exists("1",$match) ? $match[1] : "";

	preg_match("/<title.*?>(.*)<\/title>/si", $content, $match);
	$html['title'] = array_key_exists("1",$match) ? $match[1] : "";

	return $html;
}

function htmlParse($htmlFileName, $funcs = array()){
	$html = array();
	$styles = "";

	$h = file_get_html($htmlFileName);

	// Parse and run any HTML script functions
	foreach($h->find('script') as $e){
		if($e->getAttribute("language") == "php"){
			preg_match("/(.*)\\((.*)\\)/", $e->innertext, $a);
			
			$func = $a[1];
			if(array_value_exists($func, $funcs)){
				if(count($a) == 3){
//					$args = explode(",", $a[2]);
					$args = str_getcsv($a[2]);
					if(count($args) == 1)
						$e->outertext  = $func($args[0]);
					else if(count($args) == 2)
						$e->outertext  = $func($args[0], $args[1]);
					else if(count($args) == 3)
						$e->outertext  = $func($args[0], $args[1], $args[2]);
					else
						$e->outertext  = $func($args[0], $args[1], $args[2], $args[3]);
				}
				else {
					$e->outertext  = $func();
				}
			}
		}
	}
	
	if($h->find('body', 0) == Null){
		$html['body'] = "Error: Not a HTML file: {$htmlFileName}";
		$html['title'] = "";
	}
	else {
		if($h->find('head', 0)){
			// Add any styles
			$he = $h->find('head', 0);
			$html['head'] = $he->innertext;
			$t = count($he->children());
			foreach($he->children() as $e){
				if($e->tag == "style"){
					$styles .= $e->outertext . "\n";
				}
			}
		}
		else {
			$html['head'] = "";
		}

		$html['body'] = $styles . $h->find('body', 0)->innertext;

		if($h->find('title', 0))
			$html['title'] = $h->find('title', 0)->innertext;
		else
			$html['title'] = "";
	}

	return $html;
}

function htmlToText($text, $formatted = 0){
	if($formatted){
		$text = preg_replace(
			array(
				// Remove invisible content
				'@<head[^>]*?>.*?</head>@siu',
				'@<style[^>]*?>.*?</style>@siu',
				'@<script[^>]*?.*?</script>@siu',
				'@<object[^>]*?.*?</object>@siu',
				'@<embed[^>]*?.*?</embed>@siu',
				'@<applet[^>]*?.*?</applet>@siu',
				'@<noframes[^>]*?.*?</noframes>@siu',
				'@<noscript[^>]*?.*?</noscript>@siu',
				'@<noembed[^>]*?.*?</noembed>@siu',
				// Add line breaks before and after blocks
				'@</?((address)|(blockquote)|(center)|(del))@iu',
				'@</?((div)|(h[1-9])|(ins)|(isindex)|(p)|(pre))@iu',
				'@</?((dir)|(dl)|(dt)|(dd)|(li)|(menu)|(ol)|(ul))@iu',
				'@</?((table)|(th)|(td)|(caption))@iu',
				'@</?((form)|(button)|(fieldset)|(legend)|(input))@iu',
				'@</?((label)|(select)|(optgroup)|(option)|(textarea))@iu',
				'@</?((frameset)|(frame)|(iframe))@iu',
			),
			array(
				' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
				"\n\$0", "\n\$0", "\n\$0", "\n\$0", "\n\$0", "\n\$0",
				"\n\$0", "\n\$0",
			),
			$text);
		return iconv("UTF-8", "ASCII//TRANSLIT//IGNORE", html_entity_decode(strip_tags($text)));

	}
	else {
		$text = preg_replace(
			array(
				// Remove invisible content
				'@<head[^>]*?>.*?</head>@siu',
				'@<style[^>]*?>.*?</style>@siu',
				'@<script[^>]*?.*?</script>@siu',
				'@<object[^>]*?.*?</object>@siu',
				'@<embed[^>]*?.*?</embed>@siu',
				'@<applet[^>]*?.*?</applet>@siu',
				'@<noframes[^>]*?.*?</noframes>@siu',
				'@<noscript[^>]*?.*?</noscript>@siu',
				'@<noembed[^>]*?.*?</noembed>@siu',
			),
			array(
				' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
			),
			$text);
		return preg_replace('/\s+/', ' ', iconv("UTF-8", "ASCII//TRANSLIT//IGNORE", html_entity_decode(strip_tags($text))));
	}
}

?>
