<?php
/*******************************************************************************
 *	BMenu.php	Menu objects		
 *	Copyright	Terry Barnaby,	BeamWeb,	14/12/04
 *******************************************************************************
 */
class BMenu implements IteratorAggregate {
	function __construct($title, $url = "", $args = "", $subMenus = array()){
		$this->title = $title;
		$this->url = $url;
		$this->args = $args;
		$this->subMenus = array();

		foreach($subMenus as $k => $e){
			if(is_string($e))
				$this->subMenus[$k] = new BMenu($k, $e);
			else
				$this->subMenus[$k] = $e;
		}
	}
	
	function add($menu){
		$this->subMenus[$menu->title] = $menu;
	}
	
	function rebaseMenu($baseUrl = ""){
		global $sys;

		if($baseUrl == "")
			$baseUrl = $sys->baseUrl;

		if("https:" != substr($this->url, 0, 6) and "http:" != substr($this->url, 0, 5) and "/" == substr($this->url, 0, 1)){
			$this->url = $baseUrl . $this->url;
		}
		foreach($this->subMenus as &$m)
			$m->rebaseMenu($baseUrl);
	}
	
	function createFromFile($fileName){
		if($file = fopen($fileName, "r")){
			while(!feof($file)){
				$line = trim(fgets($file));
				if(($line != "") && ($line[0] != '#')){
					$param = explode(":", $line, 2);
					$menu = explode("/", trim($param[0]));
					$value = trim($param[1]);
					if($menu[0] == $this->title){
						$m = $this;
						for($level = 1; $level < count($menu); $level++){
							if(array_key_exists($menu[$level], $m->subMenus)){
								$m = $m->subMenus[$menu[$level]];
							}
							else {
								$m->add(new BMenu($menu[$level], $value));
								break;
							}
						}
					}
				}
			}
			fclose($file);
			return 0;
		
		}
		else {
			return False;
		}
	}
	
	// Required definition of interface IteratorAggregate
	public function getIterator() {
		return new ArrayIterator($this->subMenus);
	}

	// Variables
	var	$title;		///< Displayed title of menu entry
	var	$url;		///< The URL to call
	var	$args;		///< Arguments (as <a> options)
	var	$subMenus;	///< Submenu's
};

?>
