<?php
/*******************************************************************************
 *	BObj.php		Basic object with members
 *	Copyright		Terry Barnaby,	BeamWeb,	2006-06-02
 *******************************************************************************
 *
 * Notes:	Change of types to capatelised types
 */
require_once "Validate.php";

class BMember {
	function __construct($name, $type, $validateType = Null, $validateArgs = Null, $dbType = "", $dbArgs = ""){
		if($dbType == ""){
			if($name == "id"){
				$dbType = "int";
				$dbArgs = "NOT NULL auto_increment";
			}
			else {
				switch($type){
				case "Bool":		$dbType = "bool";	$dbArgs = "default 0";	break;
				case "Int":		$dbType = "int";	$dbArgs = "default 0";	break;
				case "Float":		$dbType = "float";	$dbArgs = "default 0";	break;
				case "Date":		$dbType = "date";	$dbArgs = "";		break;
				case "DateTime":	$dbType = "datetime";	$dbArgs = "";		break;
				case "String":		$dbType = "text";	$dbArgs = "";		break;
				default:		$dbType = "text";	$dbArgs = "";		break;
				}
			}
		}

		$this->name = $name;
		$this->type = $type;
		$this->validateType = $validateType;
		$this->validateArgs = $validateArgs;
		$this->dbType = $dbType;
		$this->dbArgs = $dbArgs;
		
	}
	
	public	$name;
	public	$type;
	public	$validateType;
	public	$validateArgs;
	public	$dbType;
	public	$dbArgs;
};

class BObj {
	function __construct($members){
		if($this->members == Null)
			$this->setMembers($members);

		foreach($members as $m){
			$this->{$m->name} = Null;
		}
	}

	function getMembers(){
		return $this->members;
	}
	
	function set1($values){
		foreach($values as $k => $v){
			if(!isset($this->members["$k"]))
				continue;
			$member = $this->members["$k"];
			if(is_array($v))
				$v = implode(",", $v);
			$v = stripslashes($v);
			switch($member->type){
			case "Bool":
					if($v == "on" or $v == "1" or $v == "True")
						$this->{$k} = "1";
					else
						$this->{$k} = "0";
					break;
			case "Date":
					if($v == "0000-00-00")
						$v = "";
					$this->{$k} = $v;
					break;
			case "DateTime":
					if(($v == "0000-00-00T00:00:00") || ($v == "0000-00-00 00:00:00"))
						$v = "";
					$this->{$k} = $v;
					break;
			default:	$this->{$k} = $v;	break;
			}
		}
	}
	
	function set($values){
		foreach($values as $k => $v){
			if(!isset($this->members["$k"]))
				continue;
			$member = $this->members["$k"];
			if(is_array($v))
				$v = implode(",", $v);
			$v = stripslashes($v);
			switch($member->type){
			case "Bool":
					if($v == "on" or $v == "1" or $v == "True")
						$this->{$k} = "1";
					else
						$this->{$k} = "0";
					break;

			case "Int":
					$this->{$k} = filterToInt($v);
					break;
					
			case "Float":
					$this->{$k} = filterToFloat($v);
					break;

			case "Date":
					$v = filterStringDate($v);
					if($v == "0000-00-00")
						$v = "";
					$this->{$k} = $v;
					break;

			case "DateTime":
					$v = filterStringDate($v);
					if(($v == "0000-00-00T00:00:00") || ($v == "0000-00-00 00:00:00"))
						$v = "";
					$this->{$k} = $v;
					break;

			case "String":
					$this->{$k} = filterString($v);
					break;

			case "StringHtml":
					$this->{$k} = $v;
					break;
			}
		}
	}
	
	function setDbArgs($dbArgs){
		$this->dbArgs = $dbArgs;
	}

	function getValues(){
		$values = array();
		foreach($this->members as $m){
			$values[$m->name] = $this->{$m->name};
		}
		return $values;
	}
	
	function validate(){
		$errs = array();
		
		$val = new Validate();
		foreach($this->members as $m){
//			print "Validate: {$m->name} Type: {$m->validateType} Args: "; print_r($m->validateArgs);
			$r = True;
			if(isset($this->{$m->name}) and $m->validateType){
				switch($m->validateType){
				case 'email':
				case 'Email':	$r = $val->email($this->{$m->name});	break;
				case 'emailOrNone':
					if(($this->{$m->name} != "") && !$val->email($this->{$m->name}))
						$r = False;
					break;
				case 'number':
				case 'Int':	$r = $val->number($this->{$m->name}, $m->validateArgs);	break;
				case 'Float':	$r = $val->number($this->{$m->name}, $m->validateArgs);	break;
				
				case 'string':
				case 'String':	$r = $val->string($this->{$m->name}, $m->validateArgs);	break;
				case 'stringNoSpaces':
				case 'StringNoSpaces':
						$r = $val->string($this->{$m->name}, array("format" => VALIDATE_ALPHA . VALIDATE_NUM . "_"));
						if(!$r)
							$errs["{$m->name}"] = "Error: No spaces allowed";
						break;
				case 'date':
				case 'Date':
					if(($this->{$m->name} != "0000-00-00") && ($this->{$m->name} != "")){
						if(! $a = $m->validateArgs)
							$a = array('format' => '%Y-%n-%j');
						$v = $this->{$m->name};
//						print("DoValidate: $v $a\n");
						$r = $val->date($this->{$m->name}, $a);	break;
					}
				case 'datetime':
				case 'Datetime':
					$n = $this->{$m->name};
					if(($this->{$m->name} != "0000-00-00T00:00:00") && ($this->{$m->name} != "0000-00-00 00:00:00") && ($this->{$m->name} != "")){
						$v = $this->{$m->name};
						if($a = $m->validateArgs){
							$r = $val->date($this->{$m->name}, $a);
							break;
						}
						else {
							$a1 = array('format' => '%Y-%n-%jT%G:%s:%s');
							$a2 = array('format' => '%Y-%n-%j %G:%s:%s');
							$r = $val->date($this->{$m->name}, $a1) || $val->date($this->{$m->name}, $a2);
							break;
						}
					}
				}
			}
			if(!$r && !isset($errs["{$m->name}"]))
				$errs["{$m->name}"] = "Error";
		}

		if(!count($errs))
			$errs = Null;
		return $errs;
	}

	function dbTableFields(){
		if($this->dbArgs)
			$args = "," . $this->dbArgs;
		else
			$args = "";
			
		$str = "(";
		foreach($this->members as $m){
			$str .= "`{$m->name}` {$m->dbType} {$m->dbArgs},";
		}
		$str .= "PRIMARY KEY (`id`){$args})";
		return $str;
	}

	function dbTableSetup($table){
		return "";
	}

	// Private functions
	function setMembers($members){
		$this->members = array();
		foreach($members as $m){
			$this->members["{$m->name}"] = $m;
//			$this->{$m->name} = Null;
		}
	}
	
	function addMember($member){
		$this->members["{$member->name}"] = $member;
	}

	function delMember($name){
		unset($this->{$name});
		unset($this->members[$name]);
		$this->members = array_values($this->members);
	}

	function getMemberNames(){
		return array_keys($this->members);
	}

	static function types(){
		return array("String", "StringNoSpaces", "StringList", "Email", "Int", "Float", "Bool", "Date", "DateTime");
	}
	
	// Variables
	private	$members = Null;
	private $dbArgs = "";
};

function stringListFromArray($a){
	return implode(',', $a);
}
function stringListToArray($str){
	return array_map('trim', explode(",", $str));
}
?>
