<?php
/*******************************************************************************
 *	BObjPicture.php		Picture handling
 *	Copyright:		Terry Barnaby,	BeamWeb,	2017-05-15
 *******************************************************************************
 */

global	$sys;
$sys->db->addObjectClasses(array("Picture" => "BObjPicture"));

class BObjPicture extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "String"),
			new BMember("good", "Int"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

function pictureSave($id, $fileName){
	global $sys;

//	debug_log("pictureSave: {$id} {$fileName}\n");

	$maxsize = 1024;
	$thumbSize = 128;
	$s = getimagesize($fileName);
	$aspect = (float)$s[0] / (float)$s[1];
	
//	debug_log("pictureSave: original: {$s[0]} x {$s[1]}\n");
	if($aspect > 1.0){
		$w = $s[0];
		if($w > $maxsize)
			$w = $maxsize;
		$h = (int)((float)$w/$aspect);
		$tw = $thumbSize;
		$th = (int)((float)$tw/$aspect);
	}
	else {
		$h = $s[1];
		if($h > $maxsize)
			$h = $maxsize;
		$w = (int)((float)$h * $aspect);
		$th = $thumbSize;
		$tw = (int)((float)$th * $aspect);
	}
//	debug_log("pictureSave: {$id} size: {$w} x {$h}\n");
	$image = imagecreatefromjpeg($fileName);
	$pic = imagecreatetruecolor($w, $h);
	$thumb = imagecreatetruecolor($tw, $th);
	imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $s[0], $s[1]);
	imagecopyresized($thumb, $image, 0, 0, 0, 0, $tw, $th, $s[0], $s[1]);
	imagejpeg($pic, $sys->baseDir . "/files/pictures/picture-{$id}.jpg");
	imagejpeg($thumb, $sys->baseDir . "/files/pictures/picture-{$id}-thumb.jpg");
}

function pictureSaveDir($filename, $type, $dirname, $id, $thumbWidth = 128, $thumbPng = 0){
	global $sys;

	//debug_log("pictureSaveDir: {$fileName} {$type} {$dirname} {$id}\n");

	$maxsize = 1024;
	$s = getimagesize($filename);
	$aspect = (float)$s[0] / (float)$s[1];
	
	//debug_log("pictureSave: original: {$s[0]} x {$s[1]}\n");
	if($aspect > 1.0){
		$w = $s[0];
		if($w > $maxsize)
			$w = $maxsize;
		$h = (int)((float)$w/$aspect);
		$tw = $thumbWidth;
		$th = (int)round((float)$tw/$aspect);
	}
	else {
		$h = $s[1];
		if($h > $maxsize)
			$h = $maxsize;
		$w = (int)((float)$h * $aspect);
		$tw = $thumbWidth;
		$th = (int)round((float)$tw/$aspect);
	}

	//debug_log("pictureSave: {$id} size: {$w} x {$h}\n");
	//print "Stype: '{$type}'<br>";

	if($type == "jpg")
		$image = imagecreatefromjpeg($filename);
	else if($type == "png")
		$image = imagecreatefrompng($filename);
	else if($type == "gif")
		$image = imagecreatefromgif($filename);
		
	$pic = imagecreatetruecolor($w, $h);
	$thumb = imagecreatetruecolor($tw, $th);
	imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $s[0], $s[1]);
	imagecopyresized($thumb, $image, 0, 0, 0, 0, $tw, $th, $s[0], $s[1]);
	imagejpeg($pic, $sys->baseDir . "/" . $dirname . "/picture-{$id}.jpg");
	
	if($thumbPng)
		imagepng($thumb, $sys->baseDir . "/" . $dirname . "/picture-{$id}-thumb.png");
	else
		imagejpeg($thumb, $sys->baseDir . "/" . $dirname . "/picture-{$id}-thumb.jpg");
}

function pictureUrl($id = 0, $category = "", $thumb = 0){
	global $sys;

	$select = "";

	if($id){
		$obj = $sys->db->getObject("Picture", $id);
	}
	else {
		if($category){
			$category = dbEscapeString($category);
			$select = "`category` = '{$category}'";
		}
		$list = $sys->db->getObjectList("Picture", "", $select, "");
		if(count($list)){
			$obj = $list[0];
		}
		else {
			$obj = new ObjLocal;
			$obj->id = 0;
			$obj->title = "No image";
		}
	}
	
	if($thumb)
		$html = "{$sys->baseUrl}/files/pictures/picture-{$obj->id}-thumb.jpg";
	else
		$html = "{$sys->baseUrl}/files/pictures/picture-{$obj->id}.jpg";

	return $html;
}

function pictureHtml($id = 0, $category = "", $thumb = 0, $args = ""){
	global $sys;

	$select = "";

	if($id){
		$obj = $sys->db->getObject("Picture", $id);
	}
	else {
		if($category){
			$category = dbEscapeString($category);
			$select = "`category` = '{$category}'";
		}
		$list = $sys->db->getObjectList("Picture", "", $select, "id", "1");
		if(count($list)){
			$obj = $list[0];
		}
		else {
			$obj = new ObjLocal;
			$obj->id = 0;
			$obj->title = "No image";
		}
	}
	
	if($thumb)
		$html = "<img alt='{$obj->title}' title='{$obj->title}' src='{$sys->baseUrl}/files/pictures/picture-{$obj->id}-thumb.jpg' {$args}>";
	else
		$html = "<img alt='{$obj->title}' title='{$obj->title}' src='{$sys->baseUrl}/files/pictures/picture-{$obj->id}.jpg' {$args}>";

	return $html;
}


?>
