<?php
/*******************************************************************************
 *	BView.php		Data View objects
 *	Copyright:		Terry Barnaby,	BeamWeb,	2009-03-18
 *******************************************************************************
 */

/// Object describing view type
/// ViewTypes include: "" - Standard, "link" - Display text with URL link
class BViewType {
	function __construct($name, $title = "", $type = "", $args = array(), $width=0, $height=0, $help=""){
		$this->name = $name;
		$this->title = $title;
		$this->type = $type;
		$this->args = $args;
		$this->width = $width;
		$this->height = $height;
		$this->help = $help;
	}
	
	var	$name;		// Field name
	var	$title;		// Title to display, uses name if not set
	var	$type;		// The type of editor to use
	var	$args;		// Aguments for viewType
	var	$width;		///< The width
	var	$height;	///< The height
	var	$help;		///< Help information
};

class BViewObj {
	function __construct($name = "", $mode = Null, $modeArgs = Null){
		$this->name = $name;
		$this->mode = $mode;
		$this->modeArgs = $modeArgs;
		$this->numColumns = 1;
		$this->extraRows = array();
	}
	function setView($viewTypeList = array()){
		$this->viewTypeList = $viewTypeList;
	}
	function setNumColumns($numColumns){
		$this->numColumns = $numColumns;
	}
	function setObj($obj){
		$this->obj = $obj;
	}
	function addExtraRow($obj){
		$this->extraRows[] = $obj;
	}
	function getHtml(){
		$viewTypeList = $this->viewTypeList;
		
		// Generate list of all object members
		if(count($this->obj) and count($viewTypeList) == 0){
			foreach($this->obj as $k => $v){
				$viewTypeList[] = new BViewType($k);
			}
		}
		$m = new BHtmlNode("div", "name='{$this->name}'");
		$m->appendNode("script", "document.ws.{$this->name} = {}");
		$t = &$m->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		if(count($this->obj)){
			$numColumns = $this->numColumns;
			foreach($viewTypeList as $field){
				$obj = $this->obj;
				$type = $field->type;
				$name = $field->name;
				$value = $obj->{$field->name};
				$style = "";
				$title = "";

				if($type == "hidden"){
					continue;
				}
				else if($type == ""){
					$type = "text";
				}

				if($numColumns == $this->numColumns){
					$r = &$t->appendNode("tr");
					$numColumns = 1;
				}
				else {
					$numColumns += 1;
				}

				if($field->title)
					$s = $field->title;
				else
					$s = ucfirst($field->name);

				if($field->help){
					$title = "title='{$field->help}'";
				}

				if($field->width){
					$style .= "width: {$field->width}px;";
				}
				if($field->height){
					$style .= "height: {$field->height}px;";
				}

				if($style != "")
					$style = "style='{$style}'";

				$r->appendNode("th", "align = 'left'", $s);

				switch($type){
				case "link":
					eval("\$href = \"{$field->args[1]}\";");
					eval("\$value = \"{$field->args[0]}\";");
					$c = &$r->appendNode("td", "{$style}", $value);
					$c->appendNode("a", "href=$href", $value);
					break;
				case "email":
					$args = "";
					if(count($field->args) > 0){
						$args = "?subject=" . rawurlencode($field->args[0]);
					}
					$c = &$r->appendNode("td", "{$style}");
					$c->appendNode("a", "href=mailto:{$value}{$args}", $value);
					break;
				case 'textarea':
					$c = &$r->appendNode("td", "{$style}", $value);
					break;
				case 'date':
					$r->appendNode("td", "{$style}", dateString($value));
					break;
				default:
					if($value == "")
						$value = "&nbsp;";
					$r->appendNode("td", "{$style}", $value);
					break;
				}
			}
		}
		
		// Add extra rows as needed
		foreach($this->extraRows as $obj){
			$r = &$t->appendNode("tr");
			foreach($obj as $f){
				$r->appendNode("td", "", $f);
			}
		}
		return $m->render();
	}
	
	private	$name;							// This Objects name
	private	$mode;
	private	$modeArgs;
	private	$obj;							// The list of objects to display
	private	$viewTypeList;						// The list of object fields to display
};

class BViewList {
	function __construct($name = "", $mode = Null, $modeArgs = Null){
		$this->name = $name;
		$this->mode = $mode;
		$this->modeArgs = $modeArgs;
		$this->selection = "";
		$this->order = "";
		$this->group = "";
		$this->extraRows = array();

		if(isset($_GET["{$this->name}_order"]))
			$this->order = filterStringParamList($_GET["{$this->name}_order"]);
	}
	function setName($name){
		$this->name = $name;
	}
	function setView($viewTypeList = array()){
		$this->viewTypeList = $viewTypeList;
	}
	function setList($list, $noOrder = 0){
		$this->list = $list;
		if(!$noOrder && $this->order){
			uasort($this->list, array("BViewList", "sortCompare"));
		}
	}
	function getList(){
		return $this->list;
	}
	function setSelection($selection){
		$this->selection = $selection;
	}
	function setOrder($order){
		$this->order = $order;
	}
	function setGroup($group){
		$this->group = $group;
	}
	function addExtraRow($obj){
		$this->extraRows[] = $obj;
	}
	
	function getHtml(){
		$line = 0;
		$viewTypeList = $this->viewTypeList;
			
			
		if($this->group){
			$groupCmd = "&group={$this->group}";
			$groupCmd1 = "?group={$this->group}";
		}
		else {
			$groupCmd = "";
			$groupCmd1 = "";
		}

		// Generate list of all object members
		if(count($this->list) and count($viewTypeList) == 0){
			foreach($this->list[0] as $k => $v){
				$viewTypeList[] = new BViewType($k);
			}
		}

		$m = new BHtmlNode("div", "name='{$this->name}'");
		$m->appendNode("script", "", "document.ws.{$this->name} = {}");
		$t = &$m->appendNode("table", "class='BViewList' cellpadding='2' cellspacing='2' border='1'");

		if(count($viewTypeList)){
			if($this->order and ($this->order[0] == "!"))
				$order = substr($this->order, 1);
			else
				$order = $this->order;

			$r = &$t->appendNode("tr");
			
			if($this->mode == "edit"){
				$c = &$r->AppendNode("th", "", "Function");
			}
			else if($this->mode == "view"){
				$c = &$r->AppendNode("th", "", "View");
			}
			else if($this->mode == "action"){
				$c = &$r->AppendNode("th", "", "Action");
			}
			else if(($this->mode == "select") or ($this->mode == "selectAction") or ($this->mode == "selectActionUrl")){
				$c = &$r->AppendNode("th", "", "ActionSel");
			}
			
			foreach($viewTypeList as $field){
				$c = &$r->AppendNode("th");
				if($field->title)
					$s = $field->title;
				else
					$s = ucfirst($field->name);
				
				if($order == $field->name){
					if($this->order[0] != "!")
						$o = "!{$order}";
					else
						$o = $order;
				}
				else {
					$o = $field->name;
				}
				$p = "href='?{$this->name}_order={$o}'";
				$c->appendNode("a", "$p", $s);
			}
		}

		foreach($this->list as $obj){
//			print "DoObj:"; print_r($obj);
			$r = &$t->appendNode("tr");
			

			if($this->mode == "edit"){
				$c = &$r->AppendNode("td");
				eval("\$href = \"{$this->modeArgs[1]}\";");
				eval("\$text = \"{$this->modeArgs[0]}\";");
				$c->appendNode("a", "href=$href", $text);
			}
			else if($this->mode == "view"){
				$c = &$r->AppendNode("td");
				$c->appendNode("a", "href='?cmd=view&id={$obj->id}' title='View Item'", "View");
			}
			else if($this->mode == "select"){
				$c = &$r->AppendNode("td");
				$c->appendNode("input", "type='radio' name='select' onClick='document.ws.{$this->name}.select=\"{$line}\"; document.ws.{$this->name}.objId=\"{$obj->id}\"'");
			}
			else if($this->mode == "action"){
				$c = &$r->AppendNode("td");
				if($this->name == "list"){
					$c->appendNode("a", "href='?cmd=view&id={$obj->id}{$groupCmd}' title='View Item'", "V");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd=edit&id={$obj->id}{$groupCmd}' title='Edit Item'", "E");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd=delete&id={$obj->id}{$groupCmd}' title='Delete Item'", "D");
				}
				else {
					$c->appendNode("a", "href='?cmd={$this->name}View&id={$obj->id}{$groupCmd}' title='View Item'", "V");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd={$this->name}Edit&id={$obj->id}{$groupCmd}' title='Edit Item'", "E");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd={$this->name}Delete&id={$obj->id}{$groupCmd}' title='Delete Item'", "D");
				}
			}
			else if($this->mode == "selectAction"){
				$c = &$r->AppendNode("td");
				if($this->name == "list"){
					$c->appendNode("input", "type='radio' name='select' onClick='document.ws.{$this->name}.select=\"{$line}\"; document.ws.{$this->name}.objId=\"{$obj->id}\"'");
					$c->appendNode("a", "href='?cmd=view&id={$obj->id}{$groupCmd}' title='View Item'", "V");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd=edit&id={$obj->id}{$groupCmd}' title='Edit Item'", "E");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd=delete&id={$obj->id}{$groupCmd}' title='Delete Item'", "D");
				}
				else {
					$c->appendNode("input", "type='radio' name='select' onClick='document.ws.{$this->name}.select=\"{$line}\"; document.ws.{$this->name}.objId=\"{$obj->id}\"'");
					$c->appendNode("a", "href='?cmd={$this->name}View&id={$obj->id}{$groupCmd}' title='View Item'", "V");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd={$this->name}Edit&id={$obj->id}{$groupCmd}' title='Edit Item'", "E");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='?cmd={$this->name}Delete&id={$obj->id}{$groupCmd}' title='Delete Item'", "D");
				}
			}
			else if($this->mode == "selectActionUrl"){
				global $sys;
				
				$c = &$r->AppendNode("td");
				if($this->name == "list"){
					$c->appendNode("input", "type='radio' name='select' onClick='document.ws.{$this->name}.select=\"{$line}\"; document.ws.{$this->name}.objId=\"{$obj->id}\"'");
					$c->appendNode("a", "href='{$sys->uri}/{$obj->id}{$groupCmd1}' title='View Item'", "V");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='{$sys->uri}/{$obj->id}{$groupCmd1}' title='Edit Item'", "E");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='{$sys->uri}/{$obj->id}{$groupCmd1}' title='Delete Item'", "D");
				}
				else {
					$c->appendNode("input", "type='radio' name='select' onClick='document.ws.{$this->name}.select=\"{$line}\"; document.ws.{$this->name}.objId=\"{$obj->id}\"'");
					$c->appendNode("a", "href='{$sys->uri}/{$obj->id}?cmd={$this->name}View&id={$obj->id}{$groupCmd}' title='View Item'", "V");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='{$sys->uri}/{$obj->id}?cmd={$this->name}Edit&id={$obj->id}{$groupCmd}' title='Edit Item'", "E");
					$c->appendText("&nbsp;");
					$c->appendNode("a", "href='{$sys->uri}/{$obj->id}?cmd={$this->name}Delete&id={$obj->id}{$groupCmd}' title='Delete Item'", "D");
				}
			}


			foreach($viewTypeList as $field){
				$value = strval($obj->{$field->name});
				if($field->type == "link"){
					$c = &$r->appendNode("td", "");
					eval("\$href = \"{$field->args[1]}\";");
					eval("\$text = \"{$field->args[0]}\";");
					$c->appendNode("a", "href=$href", $text);
				}
				else if($field->type == "code"){
					$text = eval($field->args);
					if($text == "")
						$text = "&nbsp;";
					$r->appendNode("td", "", $text);
				}
				else if($field->type == "date"){
					$text = dateString($obj->{$field->name});
					if($text == "")
						$text = "&nbsp;";
					$r->appendNode("td", "", $text);
				}
				else if($field->type == "email"){
					$args = "";
					if(count($field->args) > 0){
						$args = "?subject=" . rawurlencode($field->args[0]);
					}
					$c = &$r->appendNode("td");
					$c->appendNode("a", "href=mailto:{$value}{$args}", $value);
				}
				else {
					if($value == "")
						$value = "&nbsp;";
					$r->appendNode("td", "", $value);
				}
			}
			$line++;
		}

		// Add extra rows as needed
		foreach($this->extraRows as $obj){
			$r = &$t->appendNode("tr");
			foreach($obj as $f){
				$r->appendNode("td", "", $f);
			}
		}

		return $m->render();
	}

	function sortCompare($a, $b){
		if($this->order[0] == "!"){
			$order = substr($this->order, 1);
			
			if($a->{$order} == $b->{$order})
				return 0;
			else
				return ($a->{$order} > $b->{$order}) ? -1 : 1;
		}
		else {
			$order = $this->order;

			if($a->{$order} == $b->{$order})
				return 0;
			else
				return ($a->{$order} < $b->{$order}) ? -1 : 1;
		}
	}
	
	private		$name;			// This Objects name
	private		$mode;
	private		$modeArgs;
	protected	$list;			// The list of objects to display
	protected	$viewTypeList;		// The list of object fields to display
	protected	$filter;		// An expression to select objects based on field values
	protected	$selection;		// The order which to display
};

class BViewDatabase extends BViewList {
	function setTable($table){
		global $sys;

		if($this->limit){
			$limitStr = "{$this->limit}";
		}
		else {
			$limitStr = "";
		}

//		BDataBase::$debug = 1;
		$this->setList($sys->db->getObjectList($table, "", $this->selection, $this->order, $limitStr), 1);
	}
	
	function setLimit($limit){
		$this->limit = $limit;
	}
	
	function setRange($posStart, $posEnd){
		$this->posStart = $posStart;
		$this->posEnd = $posEnd;
	}
	
	var	$limit = 0;
};

/*
function testViewList(){
	$objList = array();

	// Get document list
	$list = $sys->db->getObjectList("Document");
	foreach($list as $id){
		$obj = $sys->db->getObject("Document", $id);
		$page = $page . $obj->title . "<br>\n";
		$objList[] = $obj;
	}

	$view = new BViewList();
	$view->setList($objList);
	$page = $view->getHtml();

	pageDisplay("Documents", $page);
}

function testViewDatabase(){
	$view = new BViewDatabase("document");
	$view->setView(array(new BViewType("id"), new BViewType("title", "Title"), new BViewType("html", "Html", "link", "document.php")));
	$view->setTable("Document");
	$page = $view->getHtml();

	pageDisplay("Documents", $page);
}
*/
?>
