//****************************************************************
//	FileBrowser
//****************************************************************
//
function fileExension(filename){
	return ( filename.indexOf('.') > 0 ) ? filename.split('.').pop().toLowerCase() : 'undefined';
}

function createObjDelegate(object, method, arg){
	return function(){
		if(arg !== undefined){
			arguments = [].slice.call(arguments);
			arguments.unshift(arg);
		}
		return method.apply(object, arguments);
	}
}

function inArray(array, elem){
	var len = array.length;
	for(var i = 0; i < len; i++){
		if(array[i] == elem)
			return true;
	}
	return false;
} 

function HtmlNode(p, type, attribs, text){
	node = document.createElement(type);
	p.appendChild(node);
	if(attribs !== undefined){
		for(k in attribs){
			if(k == "className")
				node.className = attribs[k];
			else
				node.setAttribute(k, attribs[k]);
		}
	}
	if(text !== undefined)
		node.appendChild(document.createTextNode(text));

	node.appendNode = function(type, attribs, text){
		return new HtmlNode(this, type, attribs, text);
	}
	node.clearNodes = function(){
		while(this.lastChild)
			this.removeChild(this.lastChild);
	}
	node.setSelected = function(on){
		if(on){
			this.style.color = "white";
			this.style.backgroundColor = "#5b9bda";
		}
		else {
			this.style.color = "";
			this.style.backgroundColor = "";
		}
//		this.className = this.className + " FileIconSelected";
	}

	return node;

}

function FileBrowser(){
	this.init = function(p, baseDir, workDir){
		bs = "box-sizing: border-box; -moz-box-sizing: border-box;";
		bs1 = bs + "border: 1px solid #adaba9; border-radius: 6px;background: white; padding: 4px;";

		this.top = new HtmlNode(p, "div", { id: "fileBrowser", style: "width: 100%; height: 100%; background: #E0DFDE;"});
		this.left = this.top .appendNode("div", { style: "float: left; width: 25%; height: 100%"});
		this.right = this.top .appendNode("div", { style: "float: left; width: 75%; height: 100%"});
		this.dirsTop = this.left.appendNode("div", {style: bs + "height: 70%; padding: 4px;"});
		this.dirs = this.dirsTop.appendNode("div", { style: bs1 + "height: 100%; overflow:auto;"});
		this.uploadTop = this.left.appendNode("div", {style: bs + "padding: 4px;"});
		this.upload = this.uploadTop.appendNode("div", {style: bs1});
		this.menuTop = this.right.appendNode("div", {style: bs + "height: 30px; padding: 4px;"});
		this.menu = this.menuTop.appendNode("div", { style: bs + "height: 100%;"});
		this.filesTop = this.right.appendNode("div", {style: bs + "height: 100%; padding: 4px;"});
		this.files = this.filesTop.appendNode("div", {style: bs1 + "height: 100%; overflow: auto;"});

		this.baseDir = baseDir;
		this.workDir = workDir;
		this.pathName = workDir;
		this.iconSize = {width: 100, height: 100};
		this.filesSelected = new Array();
		
//		console.log("WorkDir: " + this.workDir);

		this.menuInit();
		this.uploadInit();
		this.dirsUpdate();
		this.filesUpdate();
		
		this.uploader.setDir(this.workDir);

		window.onload = createObjDelegate(this, this.resize);
		window.onresize = createObjDelegate(this, this.resize);
	}

	this.resize = function(){
		this.dirsTop.style.height = (this.top.offsetHeight - this.uploadTop.offsetHeight) + "px";
		this.filesTop.style.height = (this.top.offsetHeight - this.menuTop.offsetHeight) + "px";
	}
	
	this.setCkEditor = function(ckEditor, ckFunction){
//		alert("setCkEditor: " + ckEditor + " Num: " + ckFunction);
//		this.ckEditor = document.getElementsByName(ckEditor)[0];
		this.ckEditor = ckEditor;
		this.ckFunction = ckFunction;
	}
	
	this.returnFile = function(filePath){
		filePath = this.baseDir + filePath;
//		console.log("FileBrowser::returnFile: " + filePath  + " Num: " + this.ckEditor);
//		alert("FileBrowser::returnFile: " + this.ckEditor  + " Num: " + this.ckFunction);
		if(this.ckEditor){
			window.opener.CKEDITOR.tools.callFunction(this.ckFunction, filePath, '');
			window.close();
		}
	}
	
	this.returnFileSelected = function(){
		if(this.fileSelected)
			this.returnFile(this.fileSelected.fileInfo.path);
		else
			alert("No file has been selected");
	}
	
	this.close = function(){
		window.close();
	}

	this.refresh = function(){
//		console.log("Refresh");
//		this.dirsUpdate();
		this.filesUpdate();
	}
	
	this.menuInit = function(){
		this.createButton(this.menu, "Refresh", "images/icons/refresh.png", this.refresh);
		this.settingListType = this.createCombobox(this.menu, ["Thumbnails", "List"], this.refresh);
		this.createButton(this.menu, "Ok", "images/icons/select.png", this.returnFileSelected);
		this.createButton(this.menu, "Cancel", "images/icons/close.png", this.close);
	}

	this.dirsAdd = function(p, fileInfo, expanded){
		d = null;
		if(fileInfo.type == "dir"){
			d = p.appendNode("div", {style: "margin-left: 16px;"});
			d.expanded = expanded;
			l = d.appendNode("div", {className: "DirList", style: ""});
			if(fileInfo.hasDirs){
				if(d.expanded)
					n = l.appendNode("div", { style: "float: left; width: 16px; height: 16px; background-image: url(images/tree/minus.png);"});
				else
					n = l.appendNode("div", { style: "float: left; width: 16px; height: 16px; background-image: url(images/tree/plus.png);"});
			}
			else {
					n = l.appendNode("div", { style: "float: left; width: 16px; height: 16px;"});
			}
			n.onclick = createObjDelegate(this, this.dirsExpand, d);

			f = l.appendNode("div", { style: "float: left;"});
			n = f.appendNode("div", { style: "float: left; width: 16px; height: 16px; background: no-repeat 2px center; background-image: url(images/tree/folder_current.png);"});
			n.onclick = createObjDelegate(this, this.dirsOpen, d);
			n = f.appendNode("div", {style: "float: left;"}, fileInfo.name);
			n.onclick = createObjDelegate(this, this.dirsOpen, d);
			l.appendNode("div", { style: "clear: both;"});
			d.fileInfo = fileInfo;
		}
		return d;
	}
	
	this.dirsUpdate = function(){
		this.dirs.clearNodes();
		d = this.dirsAdd(this.dirs, {type: "dir", name: "/", path: "/", hasDirs: 1}, 1);	// Create root dir
		this.dirsUpdateDir(d);
	}
	this.dirsUpdateDir = function(d){
		this.rpc("getDir", {dir: d.fileInfo.path}, this.dirsUpdateDirDo, d);
	}
	this.dirsUpdateDirDo = function(p, res){
//		console.log("dirsUpdateDirDo: " + p.fileInfo.path + " - " + this.workDir);
		if(p.fileInfo.path == this.workDir){
			this.dirsSelect(p);
		}

		for(var o in res){
			if(res[o].path == this.workDir){
				d = this.dirsAdd(p, res[o], 0);
				this.dirsSelect(d);
			}
			else if(res[o].hasDirs && (res[o].path == this.workDir.substring(0, res[o].path.length))){
				d = this.dirsAdd(p, res[o], 1);
				this.dirsUpdateDir(d);
			}
			else {
				d = this.dirsAdd(p, res[o], 0);
			}
		}
//		console.log("dirsUpdateDirDo: End");
	}
	
	this.dirsSelect = function(d){
		if(this.dirSelected){
			this.dirSelected.children[0].children[1].setSelected(0);
		}
		this.dirSelected = d;
		this.dirSelected.children[0].children[1].setSelected(1);
	}

	this.dirsExpand = function(d){
		if(d.expanded){
			d.children[0].children[0].style.backgroundImage = "url(images/tree/plus.png)";
			d.expanded = 0;
			for(var i = d.children.length - 1; i > 0; i--)
				d.removeChild(d.children[i]);
		}
		else{
			d.children[0].children[0].style.backgroundImage = "url(images/tree/minus.png)";
			d.expanded = 1;
			this.rpc("getDir", {dir: d.fileInfo.path}, this.dirsExpandDo, d);
		}
	}
	this.dirsExpandDo = function(d, res){
//		console.log("dirsExpandDo:", res);
		for(var o in res){
			this.dirsAdd(d, res[o], 0);
		}
		this.dirsSelect(d);
		this.pathName = d.fileInfo.path;
		this.uploader.setDir(d.fileInfo.path);
		this.filesUpdate();
	}

	this.dirsOpen = function(d){
//		console.log("this.dirsOpen:", d.fileInfo.path, d);
		this.dirsSelect(d);
		this.pathName = d.fileInfo.path;
		this.uploader.setDir(d.fileInfo.path);
		this.filesUpdate();
	}
	
	
	this.filesAdd = function(t, fileInfo){
		var images = ["jpg", "JPG", "png", "PNG", "gif", "GIF"];

		r = null;
		if(fileInfo.type == "file"){
			if(this.settingListType.options[this.settingListType.selectedIndex].text == "Thumbnails"){
				r = t.appendNode("div", {className: "FileIcon"});
				i = r.appendNode("div", {style: "width:" + this.iconSize.width + "px; height:" + this.iconSize.height + "px;"});
				ext = fileExension(fileInfo.name);
				if(inArray(images, ext)){
					i.appendNode("img", {src: "/files/.thumbnails/" + fileInfo.path, style: "width: 100%; height: auto;"});
				}
				else {
					i.appendNode("img", {src: "/lib/FileBrowser/images/files/big/" + ext + ".png", style: "width: 100%; height: auto;"});
				}
				r.appendNode("div", {}, fileInfo.name);
				r.style.width = this.iconSize.width + "px";
				r.fileInfo = fileInfo;
				r.onclick = createObjDelegate(this, this.filesSelect, r);
				r.ondblclick = createObjDelegate(this, this.filesAction, r);
			}
			else {
				if(fileInfo.size < 1024)
					size = (fileInfo.size) + " B";
				else if(fileInfo.size < 1024*1024)
					size = Math.round(fileInfo.size/1024) + " kB";
				else
					size = Math.round(fileInfo.size/(1024*1024)) + " MB";

				time = new Date(fileInfo.time * 1000);
				timeStr = time.toLocaleDateString() + " " + time.toLocaleTimeString();
				
				r = t.appendNode("tr", {className: "FileList"});
				c = r.appendNode("td", {style: "padding-left: 8px; border: 0px; margin: 0px;"});
				c.appendNode("span", {style: "display: block; float: left; width: 16px; height: 16px; cursor: pointer; background-image: url(" + this.fileIcon(fileInfo.name) + ");"});
				c.appendNode("span", {}, fileInfo.name);
				c = r.appendNode("td", {style: "padding-left: 8px;"}, timeStr);
				c = r.appendNode("td", {style: "padding-left: 8px;"}, size);
				r.fileInfo = fileInfo;
				r.onclick = createObjDelegate(this, this.filesSelect, r);
				r.ondblclick = createObjDelegate(this, this.filesAction, r);
			}
			
			if(inArray(this.filesSelected, r.fileInfo.name)){
				r.setSelected(1);
			}
		}
		return r;
	}

	this.filesUpdate = function(){
//		console.log("filesUpdate:", this.pathName);
		this.files.clearNodes();
		this.rpc("getDir", {dir: this.pathName}, this.filesUpdateDo);
	}
	this.filesUpdateDo = function(res){
//		console.log("filesUpdateDo: Selected: " + this.filesSelected.length);
		if(this.settingListType.options[this.settingListType.selectedIndex].text == "Thumbnails"){
			t = this.files.appendNode("div");

			for(var o in res){
				r = this.filesAdd(t, res[o]);
			}
		}
		else {
			t = this.files.appendNode("table", {border: "0", cellspacing: "0"});
			r = t.appendNode("tr");
			r.appendNode("th", {style: "padding-left: 8px;"}, "Name");
			r.appendNode("th", {style: "padding-left: 8px;"}, "Date");
			r.appendNode("th", {style: "padding-left: 8px;"}, "Size");

			for(var o in res){
				r = this.filesAdd(t, res[o]);
			}
		}
	}
	this.filesSelect = function(f){
		if(this.fileSelected){
			this.fileSelected.setSelected(0);
		}
		this.fileSelected = f;
		this.fileSelected.setSelected(1);
	}
	this.filesAction = function(f){
//		console.log("filesAction:", f.fileInfo.name, f.fileInfo.path);
		if (window.getSelection)
		        window.getSelection().removeAllRanges();
		else if (document.selection)
		        document.selection.empty();
		this.filesSelect(f);
		this.returnFile(f.fileInfo.path);
	}

	this.uploadInit = function(){
		this.uploader = new FileUploader(this.upload, "uploader", "fileBrowser.php?cmd=upload");
		this.uploader.setOnComplete(createObjDelegate(this, this.uploadComplete));
	}
	this.uploadComplete = function(files){
		this.filesSelected.length = 0;
		for(i in files)
			this.filesSelected.push(files[i]);
//		console.log("uploadCompleteDo: " + files.length + " : " + this.filesSelected.join(","));
		this.refresh();
	}


	this.createButton = function(p, name, image, callback){
		b = this.menu.appendNode("button", { style: "margin-left: 4px; margin-right: 4px; cursor: pointer;background: no-repeat 2px center; padding-left: 24px;background-image: url(" + image + ");"}, name);
		if(callback !== undefined)
			b.onclick = createObjDelegate(this, callback);
		return b;
 	}

	this.createCombobox = function(p, options, callback){
		b = this.menu.appendNode("select", { name: "select", style: "margin-left: 4px; margin-right: 4px;"});
		for(i in options){
			b.appendNode("option", {}, options[i]);
		}
		if(callback !== undefined)
			b.onclick = createObjDelegate(this, callback);
		return b;
 	}
	
	this.rpc = function(cmd, args, process, arg){
		var httpRequest = new XMLHttpRequest();
		var get = "?cmd=" + cmd;
		var obj;
		
		process = createObjDelegate(this, process, arg);

		if(args != null)
			get += "&args=" + JSON.stringify(args);

//		httpRequest.timeout = 4000;
		httpRequest.open("GET", get, true);

		httpRequest.onreadystatechange = function () {
//			console.log("Comms:", httpRequest.readyState, httpRequest.status);
			if(httpRequest.readyState == 4 && httpRequest.status == 200){
//				alert("Response: " + httpRequest.responseText);
				obj = JSON.parse(httpRequest.responseText);
				process(obj);
			}
		};

		httpRequest.send(null);
	}
	this.fileIcon = function(fileName){
		ext = (fileName.indexOf('.') > 0) ? fileName.split('.').pop().toLowerCase() : 'html';
		return "images/files/small/" + ext + ".png";
	}
};

//fileBrowser = new FileBrowser();
//fileBrowser.init(document.body, "/");
