<?php
/*******************************************************************************
 *	FileBrowser.php		A File browser
 *	Copyright:		Terry Barnaby,	BeamWeb,	2013-04-15
 *******************************************************************************
 */

set_include_path("../..");
require_once "config.php";
require_once "BSys.php";
require_once "BHtml.php";

$sys = new BSys($config);
$sys->init();

// Check login if needed. Could be done better
if($sys->loginRequired){
	$login = new BLogin($sys, "pageLogin", false);
	if(!$login->checkLogin()){
		print "Login required<br>";
		exit();
	}
}

function log_local($message){
	$f = fopen("log.txt","a"); 
	fputs($f, $message . "\n"); 
	fclose($f);
}

function arrayToString($a){
	$r = "";
	foreach($a as $k => $v)
	        $r .= $k . '=' . $v . ", ";
	return $r;
}

class FileBrowser {
	function __construct(){
		global $sys;
		
		$this->images = array("jpg", "png", "gif");

	        $this->get = &$_GET;
        	$this->post = &$_POST;
	        $this->cookie = &$_COOKIE;
		$this->files = &$_FILES;
		$this->filesUploaded = array();

		$this->thumbSize = 200;
		if($sys){
			$this->baseDir = $this->normalize($sys->baseDir);
			$this->topDir = $this->normalize($sys->baseDir . "/files");
			$this->topUrl = $this->normalize($sys->baseUrl . "/files");
			$this->workDir = $this->normalize($sys->workDir);
			if($this->workDir[0] != '/')
				$this->workDir = "/" . $this->workDir;

//			$this->workDir = "/files/t1";
//			$this->workDir = "/images";
//			$this->workDir = "/";

//			error_log("BaseDir: " . $this->baseDir);
//			error_log("TopDir: " . $this->topDir);
//			error_log("TopUrl: " . $this->topUrl);
//			error_log("WorkDir: " . $sys->workDir . " - " . $this->workDir);
		}
		else {
			$this->baseDir = "/home/terry/tandemClub/www_events/www";
			$this->topDir = $this->baseDir . "/files";
			$this->topUrl = "/files";
			$this->workDir = "/";
		}
	}
	
	function process(){
//		error_log("Cmd: " . arrayToString($this->get));
//		error_log("Post: " .arrayToString($this->post));
//		error_log("Files: " .arrayToString($this->files));

		$this->cmd = isset($this->get['cmd']) ? $this->get['cmd'] : "browser";
		$this->args = isset($this->get['args']) ? json_decode(stripslashes($this->get['args'])) : null;
		$this->workDir = isset($this->get['workDir']) ? $this->get['workDir'] : $this->workDir;
		
		switch($this->cmd){
		case "getDir":		return $this->getDir();	break;
		case "upload":		return $this->upload();	break;
		case "browse":
		default:		return $this->browse();
		}
	}
	
	function browse(){
		global $sys;

		$p = "<!DOCTYPE html>\n";
//		$p = "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN' 'http://www.w3.org/TR/html4/loose.dtd'>\n";
//		$p = "<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01//EN' 'http://www.w3.org/TR/html4/strict.dtd'>\n"; 
//		$p = "";
		$h = new BHtmlNode("html", "style='width:100%; height:100%;'");
		$e = &$h->appendNode("head");
		$e->appendNode("meta", "http-equiv='content-type' content='text/html; charset=UTF-8'");
		$b = &$h->appendNode("body", "style='margin: 0px; width:100%; height:100%; overflow:hidden;'");
		$b->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/lib/plupload/plupload.full.min.js'");
		$b->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/lib/javascript/main.js'");
		$b->appendNode("script", "type='text/javascript' src='{$sys->baseUrl}/lib/FileBrowser/fileBrowser.js'");
		$b->appendNode("link", "type='text/css' rel='stylesheet' href='{$sys->baseUrl}/lib/FileBrowser/fileBrowser.css'");

		$b->appendNode("script", "type='text/javascript'", "fileBrowser = new FileBrowser(); fileBrowser.init(document.body, '/files', '{$this->workDir}');");
		if(isset($this->get['CKEditor'])){
//			error_log("FileBrowser::browse: CKEditor: " . $this->get['CKEditor']);
			$ckEditor = $this->get['CKEditor'];
			$ckKEditorFuncNum = $this->get['CKEditorFuncNum'];
			$b->appendNode("script", "type='text/javascript'", "fileBrowser.setCkEditor('{$ckEditor}', {$ckKEditorFuncNum});");
		}
		
		print $p . $h->render();
	}
	
	function dirsHasDirs($dir){
		if(filetype($dir) == "dir"){
			$files = scandir($dir);
			foreach($files as $f){
				if($f[0] != "."){
					$path = $dir . "/" . $f;
					if(filetype($path) == "dir")
						return 1;
				}
			}
		}
		return 0;
	}
	
	function getDir(){
		$dir = $this->topDir . "/" . $this->args->dir;
		$files = scandir($dir);
		$res = array();
		foreach($files as $f){
			if($f[0] != "."){
				$path = $dir . "/" . $f;
				$relPath = $this->normalize($this->args->dir ."/" . $f);
				$res[$f] = array("name" => $f, "path" => $relPath, "type" => filetype($path), "size" => filesize($path),
					"time" => filemtime($path), "hasDirs" => $this->dirsHasDirs($path));
				$this->thumbnailCreate($relPath);
			}
		}
		print json_encode($res);
	}

	function upload(){
//		error_log("*** upload ***");
		
		$this->filesUploaded = array();
		
		// Set and validate path
		$dir = $this->post["dir"];
		$path = $this->normalize($this->topDir . "/" . $dir);
//		error_log("Upload: Path: " . $path);
		
		if(strncmp($this->topDir, $path, strlen($this->topDir))){
			error_log("FileBrowser: upload: Path Error: {$path} != {$this->topDir}");
			return;
		}
		
		// Create sensible array of upload files
		foreach($this->files["upload"] as $k => $all){
			foreach($all as $i => $v){
		        	$files[$i][$k] = $v;   
			}   
		}
//		error_log("Files: " . arrayToString($files));
		
		foreach($files as $file){
			// Get file extension
			$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
			
			// Modify filename
			$name = $this->getNewFilename($path, $file["name"]);
//			error_log("File: " . arrayToString($file) . " Name: " . $name . " Ext: " . $ext);
			
			if(in_array($ext, $this->images)){
//				error_log("imageSave: image: " . $file["tmp_name"] . " to: " . $path . "/" . $name);
				$this->imageSave($file["tmp_name"], $path . "/" . $name, 1024);
			}
			else {
//				error_log("move_uploaded_file: " . $file["tmp_name"] . " to: " . $path . "/" . $name);
				move_uploaded_file($file["tmp_name"], $path . "/" . $name);
			}
			$this->filesUploaded[] = $name;
		}
		print json_encode($this->filesUploaded);
	}
	
	function thumbnailCreate($path){
		$pathSource = $this->topDir . "/" . $path;
		$pathThumb = $this->topDir . "/.thumbnails/" . $path;
		$dir = pathinfo($path, PATHINFO_DIRNAME);
		$ext = strtolower(pathinfo($path, PATHINFO_EXTENSION));

//		error_log("thumbnailCreate: Source: {$pathSource} Dest: {$pathThumb} {$ext}");
		if(!file_exists($this->topDir . "/.thumbnails")){
			error_log("thumbnailCreate: mkdir: " . $this->topDir . "/.thumbnails");
			mkdir($this->topDir . "/.thumbnails", 0777, true);
		}

		if(!file_exists($pathThumb)){
			if(in_array($ext, $this->images)){
				$dirThumb = pathinfo($pathThumb, PATHINFO_DIRNAME);
				if(!file_exists($dirThumb))
					mkdir($dirThumb, 0777, true);

				$this->imageSave($pathSource, $pathThumb, $this->thumbSize);
			}
		}
	}

	function imageSave($source, $dest, $maxSize, $move=0){
		$e = strtolower(pathinfo($source, PATHINFO_EXTENSION));
		list($iw, $ih, $it) = getimagesize($source);

//		debug_error("imageSave: $source: $iw x $ih");
		
		if(($iw > $maxSize) || ($ih > $maxSize)){
//			error_log("imageSave: resize: $source to $dest");

			// Save out of memory on server
			if(($iw * $ih) >= (4000 * 3000))
				return;

			$aspect = (float)$iw / (float)$ih;
			if($aspect > 1.0){
				$w = $maxSize;
				$h = (int)((float)$w/$aspect);
			}
			else {
				$h = $maxSize;
				$w = (int)((float)$h * $aspect);
			}
			switch($it){
			case IMAGETYPE_GIF:	$image = imagecreatefromgif($source);	break;
			case IMAGETYPE_WBMP:	$image = imagecreatefromwbmp($source);	break;
			case IMAGETYPE_JPEG:	$image = imagecreatefromjpeg($source);	break;
			case IMAGETYPE_JPEG2000:	$image = imagecreatefromjpeg($source);	break;
			case IMAGETYPE_PNG:	$image = imagecreatefrompng($source);	break;
			case IMAGETYPE_XBM:	$image = imagecreatefromxbm($source);	break;
			default:	return;
			}

			$pic = imagecreatetruecolor($w, $h);
			imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $iw, $ih);
			
			switch($it){
			case IMAGETYPE_GIF:	imagegif($pic, $dest);	break;
			case IMAGETYPE_WBMP:	imagewbmp($pic, $dest);	break;
			case IMAGETYPE_JPEG:		imagejpeg($pic, $dest);	break;
			case IMAGETYPE_JPEG2000:	imagejpeg($pic, $dest);	break;
			case IMAGETYPE_PNG:	imagepng($pic, $dest);	break;
			case IMAGETYPE_XBM:	imagexbm($pic, $dest);	break;
			default:	return;
			}
		}
		else {
//			error_log("imageSave: move_uploaded_file: $source to $dest");
			if($move)
				move_uploaded_file($source, $dest);
			else
				copy($source, $dest);
		}
	}

	function getNewFilename($dir, $name){
		$n = pathinfo($name, PATHINFO_FILENAME);
		$s = "";
		$e = strtolower(pathinfo($name, PATHINFO_EXTENSION));
		$e = strlen($e) ? ".{$e}" : $e;

		$n = preg_replace('/\s+/', "_", $n);
		$path = "{$dir}/${n}${s}${e}";
		$i = 1;
		while(file_exists($path)){
			$s = "({$i})";
			$path = "{$dir}/${n}${s}${e}";
			$i += 1;
		}

		return "${n}${s}${e}";
	}

	function normalize($path){
		$path = preg_replace('/\/+/s', "/", $path);

		$path = "/$path";
		if (substr($path, -1) != "/")
			$path .= "/";

		$expr = '/\/([^\/]{1}|[^\.\/]{2}|[^\/]{3,})\/\.\.\//s';
		while (preg_match($expr, $path))
			$path = preg_replace($expr, "/", $path);

		$path = substr($path, 0, -1);
		$path = substr($path, 1);
		return $path;
	}

	public	$get;
	public	$post;
	public	$cookie;
	public	$files;
	
	public	$baseDir;
	public	$topDir;
	public	$topUrl;
};

$fileBrowser = new FileBrowser();
$fileBrowser->process();

?>
