<?php
/*******************************************************************************
 *	Calendar.php	Calendar display
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("Calendar" => "ObjCalendar"));

class ObjCalendar extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("description", "StringHtml"),
			new BMember("dateStart", "Date", "date", array('format' => '%Y-%m-%d')),
			new BMember("dateEnd", "Date"),
			new BMember("dateAdded", "Date"),
			));
	}
};


class Calendar extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "calendar";
		$this->title = "Calendar";
		$this->info = "Calendar module";
		
		$this->sys->htmlFuncs[] = "calendarList";
	}
	
	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminPta") || $this->sys->login->checkLogin("adminPupil");
	}

	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Calendar", "/{$this->name}", "", array("Calendar List" => "/{$this->name}", "Calendar Add" => "/{$this->name}?cmd=add"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->checkAccess()){
			switch($this->cmd){
			case "calendarView":		return $this->view();			break;
			case "add":			return $this->edit("0");		break;
			case "calendarEdit":		return $this->edit(filterToInt($_GET["id"]));	break;
			case "save":			return $this->save();			break;
			case "calendarDelete":		return $this->delete(filterToInt($_GET["id"]));	break;
			case "deleteDo":		return $this->deleteDo(filterToInt($_GET["id"]));	break;
			case "viewAllCopy":		return $this->viewAllCopy();		break;
			case "viewAll":			return $this->viewAll();		break;
			default: 			return $this->viewAllAdmin();		break;
			}
		}
		else {
			return $this->viewAll();
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$o = new BMenu("Calendar", "", "", array(
				new BMenu("View All", "", "href='calendar?cmd=viewAll'"),
				new BMenu("Edit List All", "", "href='calendar?cmd=viewAllAdmin'"),
//				new BMenu("View Calendar Entry", "", "onmouseover='this.href = \"?cmd=view&amp;id=\" + document.ws.calendar.objId'"),
				new BMenu("Add Calendar Entry", "", "onmouseover='this.href = \"?cmd=add\"'"),
//				new BMenu("Edit Calendar Entry", "", "onmouseover='this.href = \"?cmd=edit&amp;id=\" + document.ws.calendar.objId'"),
//				new BMenu("Delete Calendar Entry", "", "onmouseover='this.href = \"?cmd=delete&amp;id=\" + document.ws.calendar.objId'"),
				new BMenu("", "", ""),
				new BMenu("View Calendar For Newsletter", "", "href='calendar?cmd=viewAllCopy'"),
				));
			return $o;
		}
		else {
			return Null;
		}
	}

	function viewAll(){
		$page = "<h1>Calendar</h1>";

		// Display the next years Calendar
		$html = new BHtmlNode("div", "class='calendar'");

		$dateNow = new BDate("now");
		$yearStart = $dateNow->getYear();
		$monStart = $dateNow->getMonth();
		$yc = -1;
		$mc = -1;

	//	BDatabase::$debug = 1;

		$y = $yearStart;
		for($m = $mon = $monStart; $mon < $monStart + 24; $mon++){
			if($m > 12){
				$y += 1;
				$m = 1;
			};

			$startDate = new BDate(array($y, $m, 1));
			$endDate = new BDate(array($y, $m+1, 1));

			$objList = $this->sys->db->getObjectList("Calendar", "", "`dateStart` >= '{$startDate->get()}' and `dateStart` < '{$endDate->get()}'", "dateStart");
			if(count($objList) == 0){
				$m++;
				continue;
			}

			if($yc != $y){
				$html->appendNode("p");
				$t = &$html->appendNode("table", "border=0 width='100%' background='white'");
				$r = &$t->appendNode("tr");
				$r->appendNode("th", "id='calendarYear' colspan=3", $y);
				$yc = $y;
			}
			if($mc != $m){
				$r = &$t->appendNode("tr");
				$r->appendNode("th", "id='calendarMonth' colspan=3", BDate::monthName($m));
				$mc = $m;
			}


			foreach($objList as $obj){
				$r = &$t->appendNode("tr");

				$dateStart = new BDate($obj->dateStart);
				$dateEnd = new BDate($obj->dateEnd);
				$dateStr = $dateStart->get("%a %e");
				if($dateEnd->getYear()){
					if($dateEnd->getMonth() != $dateStart->getMonth())
						$dateStr .= " - " . $dateEnd->get("%a %e %b");
					else
						$dateStr .= " - " . $dateEnd->get("%a %e");
				}

				$r->appendNode("td width='150px'", "", $dateStr);
				$r->appendNode("td", "", $obj->title);
				$r->appendNode("td", "", $obj->description);
			}

			$m++;
		}
		$page = $page . $html->render();

		return $page;
	}


	function viewAllAdmin(){
		$page = "<h3>All Calendar Entries</h3>";

		$view = new BViewDatabase("calendar", "action");

		$view->setView(array(new BViewType("title", "Title"),
			new BViewType("category", "Category"),
			new BViewType("description", "Description", "textarea", "", 300, 80),
			new BViewType("dateStart", "Date Start", "date"),
			new BViewType("dateEnd", "Date End", "date"),
			));

		$startDate = new BDate("now");
		$view->setSelection("`dateStart` >= '{$startDate->get()}'");
		$view->setOrder("dateStart");
		$view->setTable("Calendar");
		$page = $page . $view->getHtml();

		return $page;
	}

	function viewAllCopy(){
		$page = "<h1>School calendar</h1>";


		// Display the next years Calendar
		$html = new BHtmlNode("div", "class='calendar'");
		$t = &$html->appendNode("table", "border=1");

		$dateNow = new BDate("now");
		$yearStart = $dateNow->getYear();
		$monStart = $dateNow->getMonth();
		$yc = -1;
		$mc = -1;

	//	BDatabase::$debug = 1;

		for($mon = $monStart; $mon < $monStart + 12; $mon++){
			$y = $yearStart;
			$m = $mon;
			if($mon > 12){
				$y += 1;
				$m = $m - 12;
			};

			$startDate = new BDate(array($y, $m, 1));
			$endDate = new BDate(array($y, $m+1, 1));

			$objList = $this->sys->db->getObjectList("Calendar", "", "`dateStart` >= '{$startDate->get()}' and `dateStart` < '{$endDate->get()}'", "dateStart");
			if(count($objList) == 0)
				continue;

			if($yc != $y){
				$r = &$t->appendNode("tr");
				$r->appendNode("th", "id='calendarYear' colspan=3", $y);
				$yc = $y;
			}
			if($mc != $m){
				$r = &$t->appendNode("tr");
				$r->appendNode("th", "id='calendarMonth' colspan=3", BDate::monthName($m));
				$mc = $m;
			}


			foreach($objList as $obj){
				$r = &$t->appendNode("tr");

				$dateStart = new BDate($obj->dateStart);
				$dateEnd = new BDate($obj->dateEnd);
				$dateStr = $dateStart->get("%e %a");
				if($dateEnd->getYear())
					$dateStr .= " - " . $dateEnd->get("%e %a");

				$r->appendNode("td", "", $dateStr);
				$r->appendNode("td", "", $obj->title);
			}
		}
		$page = $page . $html->render();

		return $page;
	}

	function view(){
		$page = "<h2>View Calendar</h2>";

		$obj = $this->sys->db->getObject("Calendar", filterToInt($_GET["id"]));
		$view = new BViewObj("calendar");
		$view->setView(array(new BViewType("title", "Title"),
			new BViewType("category", "Category"),
			new BViewType("description", "Description", "textarea", "", 300, 80),
			new BViewType("dateStart", "Date Start", "date"),
			new BViewType("dateEnd", "Date End", "date"),
			));

		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}

	function editObj($obj){
		$page = "<h2>Edit Calendar</h2>";

		$view = new BEditObj("Calendar", "Calendar Information", "?cmd=save", "calendarManagement.php");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category", "select", categoryGetList("calendar")),
			new BViewType("description", "Description", "textarea", "", 300, 80),
			new BViewType("dateStart", "Start Date", "date"),
			new BViewType("dateEnd", "End Date", "date"),
			));

		$view->setHelp("title", "Enter the single line title for the calendar entry.");
		$view->setHelp("category", "Enter the category  by selecting one from the list.");
		$view->setHelp("description", "Enter an optional description for this calendar entry, can conatin web links.");
		$view->setHelp("dateStart", "Enter the main start date for this calendar entry.");
		$view->setHelp("dateEnd", "Enter an optional end date for this calendar entry. This is used when the event occurs over several days.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}

	function edit($id){
		$page = "<h2>Edit Calendar</h2>";

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else if($id == 0){
			$obj = $this->sys->db->getNewObject("Calendar");
			$obj->dateAdded = strftime("%Y-%m-%d");
		}
		else {
			$obj = $this->sys->db->getObject("Calendar", $id);
		}

		return $this->editObj($obj);
	}

	function save(){
		$page = "";

		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new ObjCalendar();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
#				print_r($obj);
				return $this->editObj($obj);
			}

			if($obj->id)
				$this->sys->db->putObject("Calendar", $obj);
			else
				$this->sys->db->appendObject("Calendar", $obj);

			$page = "<h3>The Calendar has been saved.</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		return $page;
	}

	function delete($id){

		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("Calendar", $id);

		$m = new BHtmlNode("div", "name='dialogQuestion'");
		$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteDo&amp;id={$id}' method='post'");
		$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

		$f->appendNode("input", "type='submit' name='arg' value='Delete'");
		$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

		$page = $f->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		
		return $page;
	}

	function deleteDo($id){
		$page = "";
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else {
			$this->sys->db->deleteObject("Calendar", $id);
			$page .= "<h3>Calendar entry has been Deleted</h3>";	
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		return $page;
	}
};

function calendarList(){
	global	$sys;
	
	$dateNow = new BDate("now");

	$objList = $sys->db->getObjectList("Calendar", "", "`dateStart` >= '{$dateNow->get()}'", "dateStart", "3");

	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/calendar.php'")->appendNode("h2", "", "Calendar");

	foreach($objList as $obj){
		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->dateStart));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/calendar?cmd=view&amp;id={$obj->id}'", $obj->title);
		$d1->appendNode("div", "class='NewsSummaryContent'", $obj->description);
	}

	return $html->render();
}

?>
