<?php
/*******************************************************************************
 *	Category.php	Category Module
 *			T.Barnaby,	BEAM Ltd,	2008-01-08
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("Category" => "BObjCategory"));

class BObjCategory extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("group", "String"),
			new BMember("value", "String"),
			new BMember("description", "String"),
			));
	}

	function dbTableSetup($table){
		global	$sys;

		$obj = new BObjCategory();

		$obj->group = "Groups";
		$obj->value = "content";
		$sys->db->appendObject($table, $obj);

		$obj->group = "Groups";
		$obj->value = "calendar";
		$sys->db->appendObject($table, $obj);

		$obj->group = "Groups";
		$obj->value = "news";
		$sys->db->appendObject($table, $obj);

		$obj->group = "Groups";
		$obj->value = "classes";
		$sys->db->appendObject($table, $obj);

		return "";
	}

};

class Category extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "category";
		$this->title = "Category";
		$this->info = "Category module";
	}

	function checkAccess(){
		if($this->sys->login->checkLogin("admin")){
			return 1;
		}
		else {
			return 0;
		}		
	}
	
	// Admin Interface
	function getMenu(){
		return new BMenu("Category", "/{$this->name}", "", array(
			"Category List" => "/{$this->name}",
			"Category Add Category" => "/{$this->name}?cmd=add",
			"Category Add Group" => "/{$this->name}?cmd=groupAdd",
			));
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view(filterToInt($_GET["id"]));	break;
			case "add":			return $this->edit("0");		break;
			case "edit":			return $this->edit(filterToInt($_GET["id"]));	break;
			case "save":			return $this->save();			break;
			case "delete":			return $this->delete(filterToInt($_GET["id"]));	break;
			case "groupAdd":		return $this->groupAdd();		break;
			case "groupSave":		return $this->groupSave();		break;
			case "groupDelete":		return "Not Implemented";		break;
			default: 			return $this->viewList();		break;
			}
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to view this item."));
			exit();
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$o = new BMenu("Categorys", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList'"),
				new BMenu("", "", ""),
//				new BMenu("View", "",	"onmouseover='this.href = \"?cmd=view&id=\" + document.ws.list.objId'"),
				new BMenu("Add Category", "",	"onmouseover='this.href = \"?cmd=add\"'"),
//				new BMenu("Edit", "",	"onmouseover='this.href = \"?cmd=edit&id=\" + document.ws.list.objId'"),
//				new BMenu("Delete", "",	"onmouseover='this.href = \"?cmd=delete&id=\" + document.ws.list.objId'"),
				new BMenu("", "", ""),
				new BMenu("Add Group", "",	"onmouseover='this.href = \"?cmd=groupAdd\"'"),
				new BMenu("Delete Group", "",	"onmouseover='this.href = \"?cmd=groupDelete\"'"),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}


	function viewList(){
		$page = "<h2>Categorys</h2>\n";

		$view = new BViewDatabase("list", "action");
		$view->setView(array(
			new BViewType("group"),
			new BViewType("value"),
			new BViewType("description"),
			));

//		$view->setOrder("`group`,`value`");
		$view->setOrder("group,value");
		$view->setTable("Category");
		$page = $page . $view->getHtml();

		return $page;
	}

	function view($id){
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$obj = $this->sys->db->getObject("Category", $id);
		}

		$page = "<h2>Category View</h2>";

		$view = new BViewObj("user");
		$view->setView(array(
			new BViewType("group"),
			new BViewType("value"),
			new BViewType("description"),
			));
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}

	function edit($arg){
		$categoryGroups = categoryGetList("Groups");
		
		$page = "<h2>Edit Category</h2>";

		if($arg == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else if(is_object($arg)){
			$obj = $arg;
		}
		else if($arg == 0){
			$obj = $this->sys->db->getNewObject("Category");
		}
		else {
			$obj = $this->sys->db->getObject("Category", $arg);
		}

		$view = new BEditObj("user", "Category Information", "?cmd=save", "");
		$view->setView(array(new BViewType("id", "", "hidden"),
			new BViewType("group", "Group", "select", $categoryGroups),
			new BViewType("value"),
			new BViewType("description"),
			));

		$view->setHelp("group", "Enter the category group.");
		$view->setHelp("value", "Enter the category.");
		$view->setHelp("description", "Enter the description.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save(){
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjCategory();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				$this->edit($obj);
				return;
			}
			
			if($obj->id)
				$this->sys->db->putObject("Category", $obj);
			else
				$this->sys->db->appendObject("Category", $obj);
		}
		$page = "<h3>Category details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($id){
		$page = "";

		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete News items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else {
			$this->sys->db->deleteObject("Category", $id);
			$page .= "<h3>Category Item has been Deleted</h3>";	
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-1)\"'>Back to list</button>";
		}

		return $page;
	}

	function groupAdd(){
		$groups = categoryGetList("Groups");
		
		$page = "<h2>Add Category Group</h2>";
		
		$obj = $this->sys->db->getNewObject("Category");

		$view = new BEditObj("edit", "Category Groups", "?cmd=groupSave", "");
		$view->setView(array(new BViewType("group")));
		$view->setHelp("group", "Enter the category group to add.");
		$view->setObj($obj);

		$page = $page . $view->getHtml();
		
		return $page;
	}

	function groupSave(){
		if(count($_POST)){
			unset($_POST['cmd']);

			$obj = $this->sys->db->getNewObject("Category");
			$obj->group = "Groups";
			$obj->value = $_POST['group'];

			$this->sys->db->appendObject("Category", $obj);
		}
		$page = "<h3>Category details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

};

function categoryGetList($group){
	global	$sys;
	
	$a = array();
	$group = dbEscapeString($group);
	$objList = $sys->db->getObjectList("Category", "value", "`group` = '$group'", "value");
	
	foreach($objList as $obj){
		$a[] = $obj->value;
	}
	
	return $a;
}

function categoryGetDescription($group, $value){
	global	$sys;
	
	$group = dbEscapeString($group);
	$value = dbEscapeString($value);
	$objList = $sys->db->getObjectList("Category", "description", "`group` = '$group' and `value` = '$value'", "");
	
	if(count($objList))
		return $objList[0]->description;
	else
		return "";
}

?>
