<?php
/*******************************************************************************
 *	Config.php	Config Module
 *			T.Barnaby,	BEAM Ltd,	2013-11-24
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BObj.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjConfig extends BObj {
	function __construct($name = "", $type = "string", $value = "", $title = "", $description = ""){
		parent::__construct(array(
			new BMember("id", "Int", "Int"),
			new BMember("name", "String", "", "", "varchar(128)"),
			new BMember("type", "String", "", "", "varchar(32)"),
			new BMember("value", "String", "", "", "varchar(128)"),
			new BMember("title", "String"),
			new BMember("description", "String"),
			));
		$this->name = $name;
		$this->type = $type;
		$this->value = $value;
		$this->title= $title;
		$this->description = $description;
		$this->setDbArgs("KEY (`name`)");
	}
};

class Config extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "config";
		$this->title = "Config";
		$this->info = "Config module";
		$this->databasePresent = 0;

		// Add Objects
		$sys->db->addObjectClasses(array("Config" => "BObjConfig"));

		if($this->sys->db->tableExists("Config"))
			$this->databasePresent = 1;
	}

	function checkAccess(){
		if($this->sys->login->checkLogin("admin")){
			return 1;
		}
		else {
			return 0;
		}		
	}
	
	// Admin Interface
	function getMenu(){
		return new BMenu("Config", "/{$this->name}", "", array(
			"Config List" => "/{$this->name}",
			"Config Add Config" => "/{$this->name}?cmd=add",
			));
	}

	function getPage($uri){
		$this->parseUri($uri);

		switch($this->cmd){
		case "view":			return $this->view($uri);		break;
		case "add":			return $this->edit($uri);		break;
		case "edit":			return $this->edit($uri);		break;
		case "save":			return $this->save($uri);		break;
		case "delete":			return $this->delete($uri);		break;
		default: 			return $this->viewList($uri);		break;
		}
	}

	function getPageBlock($uri){
		$o = new BMenu("Configuration", "", "", array(
			new BMenu("List", "", "href='?cmd=viewList'"),
			new BMenu("", "", ""),
			new BMenu("Add Config", "", "href='?cmd=add'"),
			));
		return $o;
	}


	function viewList($uri){
		$page = "<h2>Configuration</h2>\n";

		$view = new BViewDatabase("list", "action");
		$view->setView(array(
			new BViewType("name"),
			new BViewType("type"),
			new BViewType("value"),
			new BViewType("title"),
			new BViewType("description"),
			));

		$view->setOrder("name");
		$view->setTable("Config");
		$page = $page . $view->getHtml();

		return $page;
	}

	function view($uri){
		$id = filterToInt($_GET["id"]);
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$obj = $this->sys->db->getObject("Config", $id);
		}

		$page = "<h2>Config View</h2>";

		$view = new BViewObj("config");
		$view->setView(array(
			new BViewType("name"),
			new BViewType("type"),
			new BViewType("value"),
			new BViewType("title"),
			new BViewType("description"),
			));
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}

	function edit($arg){
		$page = "<h2>Edit Config</h2>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Config</h2>";
			$obj = $this->sys->db->getObject("Config", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Config");
		}

		$view = new BEditObj("config", "Config Information", "?cmd=save", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("name"),
			new BViewType("type", "", "select", BObj::types()),
			new BViewType("value"),
			new BViewType("title"),
			new BViewType("description"),
			));

		$view->setHelp("name", "Enter the config name.");
		$view->setHelp("value", "Enter the config value.");
		$view->setHelp("description", "Enter the description.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save($uri){
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjConfig();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			if($obj->id)
				$this->sys->db->putObject("Config", $obj);
			else
				$this->sys->db->appendObject("Config", $obj);
		}
		$page = "<h3>Config details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($uri){
		$id = filterToInt($_GET["id"]);
		$page = "";

		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete Config items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("Config", $id);
			$page .= "<h3>Config Item has been Deleted</h3>";	
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {
			$obj = $this->sys->db->getObject("Config", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->name} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function read(){
		if($this->sys->db->tableExists("Config")){
			$objList = $this->sys->db->getObjectList("Config", "", "", "name");
			foreach($objList as $c){
//				print "Add: {$c->name} {$c->value}</br>";
				$this->sys->config[$c->name] = $c->value;
			}
		}
	}
};

function configGetList(){
	global	$sys;
	
	if($sys->modules["config"]->databasePresent)
		$objList = $sys->db->getObjectList("Config", "name,value", "", "name");
	else
		$objList = Null;
	
	return $objList;
}

function configGet($name){
	global	$sys;

	return $sys->config[$name];
}

function configAdd($base, $configs){
	global	$sys;

	if($sys->modules["config"]->databasePresent){
		foreach($configs as $c){
			$c->name = $base . "." . $c->name;
			if(!isset($sys->config[$c->name])){
				$sys->config[$c->name] = $c->value;
				$sys->db->appendObject("Config", $c);
			}
		}
//		print "configAdd: Config: "; print_r($sys->config); print "</br>";
	}
}

function configSet($base, $name, $value){
	global	$sys;
	
	if($sys->modules["config"]->databasePresent){
		$objList = $sys->db->getObjectList("Config", "", "`name` = '{$base}.{$name}'", "");
		if(count($objList)){
			$obj = $objList[0];
			$obj->value = $value;
			$sys->db->putObject("Config", $obj);
		}
		else {
			$obj = new BObjConfig();
			$obj->name = "{$base}.{$name}";
			$obj->value = $value;
			$sys->db->appendObject("Config", $obj);
		}
		$sys->config["{$base}.{$name}"] = $value;
	}

	return "";
}

?>
