<?php
/*******************************************************************************
 *	Content.php	Content Module
 *			T.Barnaby,	BEAM Ltd,	2008-03-15
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";
require_once "BObj.php";

class BObjContent extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("version", "Int", "Number"),
			new BMember("type", "String"),		/* html, doc, pdf */

			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "StringHtml"),
			new BMember("content", "String"),
			new BMember("filePath", "String"),
			new BMember("fileNameOriginal", "String"),
			
			new BMember("dateCreated", "Date", "date", array('format' => '%Y-%m-%d')),
			new BMember("dateModified", "Date", "date", array('format' => '%Y-%m-%d')),

			new BMember("uid", "Int"),
			new BMember("access", "String"),
			));
	}
	
	function filePath($type = ""){
		if($this->type == "gpx")
			$type = "gpx";
		if($type == "")
			$type = "pdf";

//		return "/files/{$this->filePath}-{$this->id}.{$type}";
//		return "/files/{$this->filePath}-{$this->id}-{$this->version}.{$type}";
		if($type == ""){
			return $this->filePath;
		}
		else {
			$pi = pathinfo($this->filePath);
			return $pi['dirname'] . "/" . $pi['filename'] . "." . $type;
		}
	}
};

class PageAddObj extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("pageName", "String"),
			));
	}
};


class BPopUp {
	function __construct($name, $uri = "", $width = 400, $height = 200){
		$this->name = $name;
		$this->uri = $uri;
		$this->width = $width;
		$this->height = $height;
	}
	
	function popUp(){
		$s .= "<a href='javascript:popup{$this->name}();'>PopUp</a>";
	}

	function getHtml(){
		$s = "<script language='javascript'>\n";
		$s .= "function popup{$this->name}(){\n";
		$s .= "win3 = window.open('{$this->uri}', '{$this->name}', 'screenX=200,screenY=400,width={$this->width},height={$this->height}');\n";
		$s .= "}\n";
		$s .= "</script>";

		return $s;
	}
};

class Content extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "content";
		$this->title = "Content";
		$this->info = "Content module. This module allows the management of Content items.";

		// Add Objects
		$sys->db->addObjectClasses(array("Content" => "BObjContent"));
		
		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminContent"));
	}

	function getMenu(){
		return Null;
	}
	
	function checkAdmin($uri){
		if(!strncmp($uri, "files/pta/", 10) && $this->sys->login->checkLogin("adminPta")){
			return 1;
		}
		if(!strncmp($uri, "files/pupils/", 13) && $this->sys->login->checkLogin("adminPupil")){
			return 1;
		}
		
		return $this->sys->login->checkLogin("admin,adminContent");
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->checkAdmin($uri)){
			switch($this->cmd){
			case "viewContent":		return $this->viewContent($uri);	break;
			case "edit":			return $this->edit($uri);	break;
			case "addPage":			return $this->addPage($uri);	break;
			case "addPage1":		return $this->addPage1($uri);	break;
			case "addDirectory":		return $this->addDirectory($uri);	break;
			case "addDirectory1":		return $this->addDirectory1($uri);	break;
			case "editFile":		return $this->editFile(filterToInt($_GET["id"]));	break;
			case "fileBrowser":		return $this->fileBrowser($uri);	break;
			case "uploadFile":		return $this->uploadFile($uri);	break;
			case "save":			return $this->save($uri);		break;
			case "delete":			return $this->delete($uri);	break;
			
			// Popups
			case "popupAddPage":		return $this->popupAddPage($uri);	break;
			case "popupAddPage1":		return $this->popupAddPage1($uri);	break;
			case "popupAddDirectory":	return $this->popupAddDirectory($uri);	break;
			case "popupAddDirectory1":	return $this->popupAddDirectory1($uri);	break;

			case "contentConvert":		return $this->contentConvert($uri);	break;

			case "view":
			default: 			return $this->view($uri);	break;
			}
		}
		else {
			switch($this->cmd){
			case "viewContent":		return $this->viewContent($uri);	break;
			case "contentConvert":		return $this->contentConvert($uri);	break;
			default:			return $this->view($uri);		break;
			}
		}
	}

	function getPageBlock($uri){
		$this->parseUri($uri);
		if($this->checkAdmin($uri)){
			if($this->cmd == 'edit'){
				$o = new BMenu("Content", "", "", array(
					new BMenu("View", "?cmd=view", "onclick=\"return confirm('Are you sure you want to lose the edits ?')\""),
					new BMenu("Add Page", "javascript:popupAddPage();"),
					new BMenu("Add Directory", "javascript:popupAddDirectory();"),
//					new BMenu("Upload File", "?cmd=editFile", ""),
//					new BMenu("Delete", "?cmd=delete", ""),
					new BMenu("", "", ""),
					));
			}
			else {
				$o = new BMenu("Content", "", "", array(
					new BMenu("View", "?cmd=view", ""),
					new BMenu("View Content", "?cmd=viewContent", ""),
					new BMenu("Edit", "?cmd=edit", ""),
					new BMenu("Add Page", "?cmd=addPage", ""),
					new BMenu("Add Directory", "?cmd=addDirectory", ""),
					new BMenu("FileBrowser", "?cmd=fileBrowser", ""),
					new BMenu("Upload File", "?cmd=editFile", ""),
//					new BMenu("Delete", "?cmd=delete", ""),
					new BMenu("", "", ""),
					));
			}
		}
		else {
			$o = new BMenu("Content", "", "", array(
				new BMenu("View", "?cmd=view", ""),
				new BMenu("View Content", "?cmd=viewContent", ""),
				));
		}
		return $o;
	}
	
	function view($uri){
		return $this->viewFile(filePathLocal($uri), filePathAbs($uri));
	}
		
	function viewFile($pathLocal){
		$pathAbs = $this->sys->baseDir . "/" . $pathLocal;

//		print "Content::viewFile: '{$pathLocal}' '{$pathAbs}'<br>"; return "";
		
		if(file_exists($pathAbs)){
			if((pathinfo($pathAbs, PATHINFO_EXTENSION) == "html") || (pathinfo($pathAbs, PATHINFO_EXTENSION) == "htm")){
				$html =	htmlParse($pathAbs, $this->sys->htmlFuncs);
				$this->sys->page->setTitle($html['title']);
				$page = $html['body'];
	
				// Add Date
				if($this->checkAdmin($pathLocal)){
					$fileDate = Date("F d Y H:i:s.", filemtime($pathAbs));
					$page .= "</br><div style='font: 8pt Verdana, Helvetica, Arial, sans-serif;'>Page was last modified on: {$fileDate}</div>";
				}
				return $page;
			}
			else {
				$ext = pathinfo($pathAbs, PATHINFO_EXTENSION);
				if($ext == "sch"){
					$mime = "application/octet-stream";
				}
				else if($ext == "gpx"){
					$mime = "application/gpx+xml";
				}
				else {
					$fi = new finfo(FILEINFO_MIME);
					$mime = $fi->file($pathAbs);
					if($mime == "")
						$mime = "text/plain";
				}
				
//				print "File: $uri Ext: $ext Content: $mime";
				header("Content-type: $mime");
				if (ob_get_level()) ob_end_clean();
				readfile($pathAbs);
				return null;
			}
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Page: {$pathLocal}: not found"));
			return "";
		}
	}
	
	function viewContent($uri){
		if(file_exists($this->filepathabs)){
			if (ob_get_level()) ob_end_clean();
			readfile($this->filepathabs);
			return null;
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Page: $uri: not found"));
			return "";
		}
	}
	
	function edit($uri){
		$filepath = $uri;

//		print "Edit: $uri $filepath {$this->filepathabs}<br>";
		if(file_exists(filePathAbs($filepath))){
			$text = file_get_contents(filePathAbs($filepath));
		}
		else{
			$text = "";
		}

		$page = "";
		$h = new BHtmlNode("form", "method='post' action='?cmd=save'");
		
		$text = htmlspecialchars($text);

		$toolbar = "[ ['Save','Print','-','Templates'],['Cut','Copy','Paste','PasteText','PasteFromWord','-','Undo','Redo']";
		$toolbar .= ",['Find','Replace','-','SelectAll','-','SpellChecker', 'Scayt' ],['Bold','Italic','Underline','Strike','Subscript','Superscript','-','RemoveFormat' ]";
		$toolbar .= ",['TextColor','BGColor'],['Source', 'Maximize', 'ShowBlocks','-','About' ]";
		$toolbar .= ",['Styles','Format','Font','FontSize'],['NumberedList','BulletedList','-','Outdent','Indent','-','Blockquote','CreateDiv','-','JustifyLeft','JustifyCenter','JustifyRight','JustifyBlock']";
		$toolbar .= ",['Link','Unlink','Anchor'], ['Image','Table','HorizontalRule','Smiley','SpecialChar','PageBreak']";
		$toolbar .= "]";
		$h->appendNode("script", "type='text/javascript' src='{$this->sys->baseUrl}/lib/ckeditor/ckeditor.js'");
		$h->appendNode("textarea", "name='editor' id='editor'", $text);
		$h->appendNode("script", "", "CKEDITOR.config.width = '100%';");
		$h->appendNode("script", "", "CKEDITOR.config.height = '600px';");
		$h->appendNode("script", "", "CKEDITOR.config.skin = 'kama';");
		$h->appendNode("script", "", "CKEDITOR.config.allowedContent = true;");
		$h->appendNode("script", "", "CKEDITOR.config.sourceAreaTabSize = 2;");
		$h->appendNode("script", "", "CKEDITOR.config.toolbar = {$toolbar};");
		$h->appendNode("script", "", "CKEDITOR.config.filebrowserBrowseUrl = '{$this->sys->baseUrl}/lib/FileBrowser/fileBrowser.php?type=files&workDir={$this->sys->workDir}';");
		$h->appendNode("script", "", "CKEDITOR.config.filebrowserImageBrowseUrl = '{$this->sys->baseUrl}/lib/FileBrowser/fileBrowser.php?type=images&workDir={$this->sys->workDir}';");
		$h->appendNode("script", "", "CKEDITOR.replace('editor');");
		$h->appendNode("script", "", "ckeditorAddText('{$text}');");
		
		$h->appendNode("input", "type='hidden' name='filepath' value='{$filepath}'");
		$h->appendNode("input", "type='submit' value='Submit'");
		$h->appendNode("input", "type='button' value='Cancel' onclick='location.href = \"javascript:history.go(-1)\";'");

		// Add popup
		$popUp = new BPopUp("AddPage", "?cmd=popupAddPage", 600, 200);
		$page .= $popUp->getHtml();
		$popUpDir = new BPopUp("AddDirectory", "?cmd=popupAddDirectory", 600, 200);
		$page .= $popUpDir->getHtml();
		$page .= "<script language='JavaScript'>function ckeditorAddText(text){e = CKEDITOR.instances.editor.insertHtml(text);}</script>\n";
		$page .= "<script language='JavaScript'>document.ws.ckeditorDirectory='" . $dirName . "';</script>\n";
		
		$page .= $h->render();
		return $page;
	}

	function save($uri){
		$filepath = filterString($_POST["filepath"]);
		$urlPath = urlPath($uri);
//		print "Content::save: uri: {$uri} filepath: {$filepath} urlPath: {$urlPath}<br>"; return "";
		
		if(preg_match("/.*\.html/", $filepath) || preg_match("/.*\.htm/", $filepath)){
			$text = "<html><head></head><body>\n";
			$text .= str_replace("\r", "", stripslashes($_POST["editor"]));
			$text .= "\n</body></html>\n";

			$filepathabs = filePathAbs($filepath);
//			print "SaveFile: {$uri} {$filepath} {$filepathabs}<br>"; return "";
			if(!file_put_contents($filepathabs, $text)){
				return "SaveFile: error: {$path}<br>";
			}
			else {
				header("Location: {$urlPath}");
				exit(0);
			}
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Can only save HTML files."));
		}
	}

	function delete(){
		return "Delete not implemented";
	}

	function addPage($uri){
		$page = "<h1>Add a new page</h1>";
	
		$obj = new PageAddObj();
		$view = new BEditObj("edit", "New Page", "?cmd=addPage1", "");
		$view->setView(array(
			new BViewType("pageName"),
			));
			
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		$page .= "This will create a new page, and add a link to this page at the end of the displayed page.";

		return $page;
	}

	function addPage1($uri){
		$this->parseUri($uri);

		if(!isset($_POST["pageName"]) || ($_POST["pageName"] == "")){
			$this->sys->pageDisplayError(new BError(1, "You need to set a name for the page"));
			return "";
		}
		$filename = filterString($_POST["pageName"]);
		
		// Add .html if required
		if(!preg_match("/.*\.html/", $filename))
			$filename .= ".html";

		$filepathabs = dirname($this->filepathabs) . "/" . $filename;
		
		// Check if file exists
		
		// Create new page
		$content = file_get_contents($this->sys->baseDir . "/lib/templates/Blank.html");
		file_put_contents($filepathabs, $content);
		
		// Add link to existing page
		$link = "<a href='{$filename}'>{$filename}</a><br>";
		$content = file_get_contents(filePathAbs($uri));

		$content = preg_replace("%</body>%", "{$link}\n</body>", $content);
		file_put_contents(filePathAbs($uri), $content);
		
		header ("Location: {$filename}?cmd=edit");
		exit(0);
	}

	function popupAddPage($uri){
		$page = "<h1>Add a new page</h1>";
	
		$obj = new PageAddObj();
		$view = new BEditObj("edit", "New Page", "?cmd=popupAddPage1", "");
		$view->setView(array(
			new BViewType("pageName"),
			));
			
		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		$page .= "This will create a new page, and add a link to this page at the current cursor position.";

		$this->sys->pageDisplayPopup("Add Page", $page);

		return Null;
	}

	function popupAddPage1($uri){
		$this->parseUri($uri);

		if(!isset($_POST["pageName"]) || ($_POST["pageName"] == "")){
			$this->sys->pageDisplayError(new BError(1, "You need to set a name for the page"));
			return "";
		}
		$filename = filterString($_POST["pageName"]);
		
		// Add .html if required
		if(!preg_match("/.*\.html/", $filename))
			$filename .= ".html";

		$filepathabs = dirname($this->filepathabs) . "/" . $filename;

		// Check if file exists
		
		// Create new page
		$content = file_get_contents($this->sys->baseDir . "/lib/templates/Blank.html");
		file_put_contents($filepathabs, $content);
		
		// Add link to existing page
		$link = "<a href='{$filename}'>{$filename}</a><br>";

		$page = "<script language='JavaScript'>self.opener.ckeditorAddText(\"{$link}\");window.close();</script>";
		print $page;
		return Null;
	}

	function addDirectory($uri){
		$page = "<h1>Add a new directory with index page</h1>";
	
		$obj = new PageAddObj();
		$view = new BEditObj("edit", "New Directory", "?cmd=addDirectory1", "");
		$view->setView(array(
			new BViewType("pageName", "Directory name"),
			));
			
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		$page .= "This will create a new directory, add an index.html file and add a link to this page at the end of the displayed page.";

		return $page;
	}

	function addDirectory1($uri){
		$this->parseUri($uri);

		if(!isset($_POST["pageName"]) || ($_POST["pageName"] == "")){
			$this->sys->pageDisplayError(new BError(1, "You need to set a name for the directory"));
			return "";
		}
		$dirname = filterString($_POST["pageName"]);
		$filename = $dirname . "/index.html";
		
		$dirpath = dirname($this->filepathabs) . "/" . $dirname;
		$filepath = dirname($this->filepathabs) . "/" . $filename;
		
		// Check if file exists
		
		// Create new dir
		mkdir($dirpath);
		
		// Create new page
		if($_SESSION["content.templateFile"])
			$content = file_get_contents($this->sys->baseDir . $_SESSION["content.templateFile"]);
		else
			$content = file_get_contents($this->sys->baseDir . "/lib/templates/Blank.html");
		file_put_contents($filepath, $content);
		
		// Add link to existing page
		$link = "<a href='{$filename}'>{$filename}</a><br>";
		$content = file_get_contents(filePathAbs($uri));
		$content = preg_replace("%</body>%", "{$link}\n</body>", $content);
		file_put_contents(filePathAbs($uri), $content);
		
		header ("Location: {$filename}?cmd=edit");
		exit(0);
	}

	function popupAddDirectory($uri){
		$page = "<h1>Add a new directory</h1>";
	
		$obj = new PageAddObj();
		$view = new BEditObj("edit", "New Directory", "?cmd=popupAddDirectory1", "");
		$view->setView(array(
			new BViewType("pageName", "DirectoryName"),
			));
			
		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		$page .= "This will create a new directory, add an index.html file and add a link to this page at the current cursor position.";

		$this->sys->pageDisplayPopup("Add Directory", $page);

		return Null;
	}

	function popupAddDirectory1($uri){
		$this->parseUri($uri);

		if(!isset($_POST["pageName"]) || ($_POST["pageName"] == "")){
			$this->sys->pageDisplayError(new BError(1, "You need to set a name for the page"));
			return "";
		}

		$dirname = filterString($_POST["pageName"]);
		$filename = $dirname . "/index.html";
		
		$dirpath = dirname($this->filepathabs) . "/" . $dirname;
		$filepath = dirname($this->filepathabs) . "/" . $filename;

		// Check if file exists
		
		// Create new dir
		mkdir($dirpath);
		
		// Create new page
		$content = file_get_contents($this->sys->baseDir . "/lib/templates/Blank.html");
		file_put_contents($filepath, $content);
		
		// Add link to existing page
		$link = "<a href='{$filename}'>{$filename}</a><br>";

		$page = "<script language='JavaScript'>self.opener.ckeditorAddText(\"{$link}\");window.close();</script>";
		print $page;
		return Null;
	}

	function editFile($obj){
		$page = "<h2>Upload documents and pictures</h2>";

		if($arg == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else if(is_object($arg)){
			$obj = $arg;
		}
		else if($arg == 0){
			$obj = $this->sys->db->getNewObject("Content");
			$obj->version = 0;
			$obj->dateCreated = strftime("%Y-%m-%d");
		}
		else {
			$obj = $this->sys->db->getObject("Content", $arg);
		}

		$obj->MAX_FILE_SIZE = (250*1024*1024);
		$obj->fileName = "";

		$view = new BEditObj("fileUpload", "Document Upload", "?cmd=uploadFile", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("type", "", "hidden"),
			new BViewType("version", "", "hidden"),
			new BViewType("filePath", "", "hidden"),
			new BViewType("MAX_FILE_SIZE", "", "hidden"),
			new BViewType("title", "", "text", "", 400, 0),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("category", "", "select", categoryGetList("content")),
			new BViewType("description", "", "textarea", "", 400, 100),
			new BViewType("dateCreated", "Date Created", "date"),
			new BViewType("fileName", "", "fileUploader", "", 400, 0),
//			new BViewType("fileName", "", "file", "", 400, 0),
			));

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Optionaly enter a description. This can be multiple lines long and contain web addresses etc");
		$view->setHelp("fileName", "Enter a local file name. You can use the Browse button to use a file selector dialog");

		$view->setObj($obj);

		return $page . $view->getHtml();
	}

	function fileBrowser($uri){
		header ("Location: {$this->sys->baseUrl}/lib/FileBrowser/fileBrowser.php?workDir={$this->sys->workDir}");
	}
	
	function uploadFile($uri){
//		print_r($_FILES);
		$page = "";
		$fileType = "";

		unset($_POST['cmd']);
		
		if($_POST['id'] == 0)
			$obj = new BObjContent();
		else
			$obj = $this->sys->db->getObject("Content", filterToInt($_POST['id']));

		$obj->set($_POST);

		if($err = $obj->validate()){
			$items = implode(",", array_keys($err));
			$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
			return $this->editFile($obj);
		}

		if(isset($_FILES["fileName"]) and ($_FILES["fileName"]["name"] != "")){
			$obj->version = $obj->version + 1;
			$obj->fileNameOriginal = $_FILES["fileName"]["name"];
			$pi = pathinfo($obj->fileNameOriginal);
			$obj->type = strtolower($pi['extension']);
		}
		
		// Modify file name as needed
		$n = pathinfo($obj->fileNameOriginal, PATHINFO_FILENAME);
		$e = strtolower(pathinfo($obj->fileNameOriginal, PATHINFO_EXTENSION));
		$e = strlen($e) ? ".{$e}" : $e;
		$obj->fileNameOriginal = "${n}${e}";

		if($obj->filePath == "")
			$obj->filePath = dirname($uri) . "/" . $obj->fileNameOriginal;
		
		// Add to database
		if($obj->id){
			$this->sys->db->putObject("Content", $obj);
		}
		else {
			$obj = $this->sys->db->appendObject("Content", $obj);
		}
		
//		print_r($_FILES);
		if(isset($_FILES["fileName"]) and ($_FILES["fileName"]["name"] != "")){
			// Move file
			if(!move_uploaded_file($_FILES["fileName"]["tmp_name"], filePathAbs($obj->filePath)))
				$page .= "Move Error";
			chmod(filePathLocal($obj->filePath), 0664);
		}

		// Add link to existing page
		$link = "<a href='{$obj->fileNameOriginal}'>{$obj->title}</a><br>";
		$filepath = $this->filepathabs;

		$content = file_get_contents($filepath);
		$content = preg_replace("%</body>%", "{$link}\n</body>", $content);
		file_put_contents($filepath, $content);
		
		$page .= "<h3>File has been saved</h3>";

		return $page;
	}

	function contentConvert($uri){
		
		$id = filterToInt($_GET["id"]);
		
		$obj = $this->sys->db->getObject("Content", $id);
		$fs = filePathAbs($obj->filePath);
		$fd = filePathAbs($obj->filePath("pdf"));

//		print "ConvertDo: Title: '{$obj->title}' Src: '{$fs}' Dst: '{$fd}'<br>";
		contentConvert($fs, $fd);
	}
	
	function parseUri($uri){
		parent::parseUri($uri);
		$this->filepathabs = filePathAbs($uri);
	}
};

function contentConvert($srcFileName, $destFileName){
//	printf("contentConvert '{$srcFileName}' '{$destFileName}'<br>\n");

	$postfields = array(
        	 "outType" => "PDF",
//        	 "fileName" => "@{$srcFileName}"
        	 "fileName" => new CurlFile($srcFileName)
	);

	$fp = fopen($destFileName, "w");

	$ch = curl_init("http://portal.beam.ltd.uk/bdocconv/bdocconv.py");
//	$ch = curl_init("http://portal.beam.ltd.uk/info.php");

	curl_setopt($ch, CURLOPT_FILE, $fp);
	curl_setopt($ch, CURLOPT_FAILONERROR, 1);
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);	// allow redirects
	curl_setopt($ch, CURLOPT_TIMEOUT, 240);		// times out after 4mins
	curl_setopt($ch, CURLOPT_POST, 1);		// set POST method
	curl_setopt($ch, CURLOPT_POSTFIELDS, $postfields); // add POST fields
	$result = curl_exec($ch);
	$error = curl_error($ch);

//	print "Curl Result: {$result} {$error}<br>\n";

	fclose($fp);
	return $result;
}

function contentView($title, $obj, $type = ""){
	global	$sys;
	
//	print "ContentView: Type: {$type} FilePath: {$obj->filePath}<br>";
	
	if($obj->content != "")
		return $obj->content;

	if($obj->filePath == ""){
		$sys->pageDisplayError(new BError(1, "No contents"));
		return "";
	}

	$filePath = filePathAbs($obj->filePath($type));
	$url = urlPath($obj->filePath($type));

//	print "View: {$filePath} {$cwd} {$url}<br>"; return "";
	
	if(!file_exists($filePath)){
		$sys->pageDisplayError(new BError(1, "File " . $filePath . " does not exist"));
		return "";
	}
	
	if($type == "html"){
		print file_get_contents($filePath);
		return Null;
	}
	else {
		header("Location: $url");
		return "";
	}
}

function contentCreate($filePath){
	global	$sys;
	
	$obj = $sys->db->getNewObject("Content");
	$obj->version = 1;
	$obj->dateCreated = $obj->dateCreated = strftime("%Y-%m-%d");
	$obj->filePath = $filePath;
	$obj = $sys->db->appendObject("Content", $obj);
	return $obj;
}

function contentGetObj($id){
	global	$sys;
	
	$obj = $sys->db->getObject("Content", $id);
	return $obj;
}

function contentGetObjTitled($title){
	global	$sys;
	
	$obj = Null;
	$title = dbEscapeString($title);
	$objList = $sys->db->getObjectList("Content", "", "title='{$title}'", "!version", "1");
	if(count($objList)){
		$obj = $objList[0];
	}
	
	return $obj;
}

function contentGetContent($obj, $type){
	global	$sys;
	
	$filePath = filePathLocal($obj->filePath($type));
	return file_get_contents($filePath);
}

function contentPutContent($obj, $type, $content){
	global	$sys;
	
	$filePath = filePathLocal($obj->filePath($type));
	file_put_contents($filePath, $content);
	contentConvert(filePathAbs($obj->filePath($type)), filePathAbs($obj->filePath("pdf")));
}

function contentDelete($obj){
	global	$sys;
	
	$r = unlink(filePathAbs($obj->filePath($obj->type)));
	if(file_exists(filePathAbs($obj->filePath("pdf"))))
		unlink(filePathAbs($obj->filePath("pdf")));
	
	$sys->db->deleteObject("Content", $obj->id);
}

?>
