<?php
/*******************************************************************************
 *	ContentDir.php		ContentDir Module
 *				T.Barnaby,	BEAM Ltd,	2015-05-15
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";

function urlEncodeFilename($name){
	return str_replace("&", "%26", $name);
}

class ContentDir extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "contentDir";
		$this->title = "ContentDir";
		$this->info = "ContentDir Module";
		
		// Add Objects
	}

	function sessionRead(){
//		if(isset($_SESSION["info.objId"]))
//			$this->objId = $_SESSION["info.objId"];
	}
	function sessionWrite(){
//		$_SESSION["category"] = $this->category;
//		$_SESSION["info.objId"] = $this->objId;
	}
	
	function checkAccess(){
		return ($this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminContent"));
	}

	// Standard interface
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("ContentDir", "/{$this->name}", "", array(
				"Directory List" => "/{$this->name}",
				));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		$this->sessionRead();

		if($this->checkAccess()){
			switch($this->cmd){
			
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;

			case "addDirectory":		$r = $this->addDirectory($uri);	break;
			case "addDirectory1":		$r = $this->addDirectory1($uri);	break;
			case "fileBrowser":		$r = $this->fileBrowser($uri);	break;
			default: 			$r = $this->view($uri);			break;
			}
		}
		else {
			$r = $this->view($uri);
		}

		$this->sessionWrite();
		return $r;
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			return new BMenu("ContentDir", "", "", array(
				new BMenu("Edit Info", "?cmd=edit", ""),
				new BMenu("Add Directory", "?cmd=addDirectory", ""),
				new BMenu("FileBrowser", "?cmd=fileBrowser", ""),
				));
		}
		else {
			return Null;
		}
	}
	
	function getIcon($fileName){
		$baseUrl = $this->sys->baseUrl;
		
		$ext = pathinfo($fileName, PATHINFO_EXTENSION);
		if(is_dir($fileName)){
			return $baseUrl . "/icons/filetypes/folder.gif";
		}
		else if($ext == "html"){
			return $baseUrl . "/icons/filetypes/fhtml.gif";
		}
		else if($ext == "pdf"){
			return $baseUrl . "/icons/filetypes/ffile.gif";
		}
		else if($ext == "rpm"){
			return $baseUrl . "/icons/filetypes/fzip.gif";
		}
		else if($ext == "gz"){
			return $baseUrl . "/icons/filetypes/fzip.gif";
		}
		else if($ext == "txt"){
			return $baseUrl . "/icons/filetypes/ffile.gif";
		}
		else if($ext == "odt"){
			return $baseUrl . "/icons/filetypes/fdoc.gif";
		}
		else if($ext == "doc"){
			return $baseUrl . "/icons/filetypes/fdoc.gif";
		}
		else if($ext == "docx"){
			return $baseUrl . "/icons/filetypes/fdoc.gif";
		}
		else if($ext == "png"){
			return $baseUrl . "/icons/filetypes/fimages.gif";
		}
		else if($ext == "gif"){
			return $baseUrl . "/icons/filetypes/fimages.gif";
		}
		else if($ext == "jpg"){
			return $baseUrl . "/icons/filetypes/fimages.gif";
		}
		else if($ext == "jpeg"){
			return $baseUrl . "/icons/filetypes/fimages.gif";
		}
		else if($ext == "mp3"){
			return $baseUrl . "/icons/filetypes/fsound.gif";
		}
		else {
			return $baseUrl . "/icons/filetypes/unknown.gif";
		}
	}
	
	function getDirListByName($path, $excludes){
		// Open the folder
		$dir = @opendir($path) or die("Unable to open $path");

		// Get list of files the files
		$dlist = array();
		while($file = readdir($dir)){
			if(!patternCompareArray($excludes, $file)){
				$dlist[$file] = $file;
			}
		}

		// Close
		closedir($dir);
		
		// Sort into date order
		asort($dlist);

		return $dlist;
	}

	function getDirListByDate($path, $excludes){
		// Open the folder
		$dir = @opendir($path) or die("Unable to open $path");

		// Get list of files the files
		$dlist = array();
		while($file = readdir($dir)){
			if(!patternCompareArray($excludes, $file)){
				$dlist[$file] = filemtime($path . $file);
			}
		}

		// Close
		closedir($dir);
		
		// Sort into date order
		arsort($dlist);

		return $dlist;
	}


	function view($uri){
		$excludes = array("\.", "\.\.", "index.php", "FileList.bp");
		$path = $this->filepathabs;
		
		$page = "";
		if(file_exists($path . "/info.html")){
			$html =	htmlParse($path . "/info.html", $this->sys->htmlFuncs);
			$this->sys->page->setTitle($html['title']);
			$page .= $html['body'];
		}
		
		$table = new BHtmlNode("table", "cellpadding='4' cellspacing='4' border='0'");
		
		// Add headers
		$r = &$table->appendNode("tr");
		$c = &$r->appendNode("th");
		$c->appendText("Type");
		$c = &$r->appendNode("th");
		$c->appendText("FileName");
		$c = &$r->appendNode("th");
		$c->appendText("Date");
		$c = &$r->appendNode("th");
		$c->appendText("Size");

		$r = &$table->appendNode("tr");
		$c = &$r->appendNode("td", "colspan=4");
		$c->appendNode("hr");
		
		// Get the directory list
		$dlist = $this->getDirListByName($path, $excludes);
//		$dlist = $this->getDirListByDate($path, $excludes);
		
		// Loop through the files
		foreach($dlist as $file){
			$filepath = $path . "/" . $file;
			$fileurl = urlPath($uri . "/" . rawurlencode($file));
			$icon = $this->getIcon($filepath);

//			print "AddFileUrl: {$fileurl} {$filepath}<br>";
			$r = &$table->appendNode("tr");
			$c = &$r->appendNode("td");
			$l = &$c->appendNode("img", "src=\"$icon\"");

			$c = &$r->appendNode("td");
			$l = &$c->appendNode("a", "href=\"$fileurl\"");
			$l->appendText($file);

			$c = &$r->appendNode("td");
			$c->appendText(date("Y-m-d h:ia", filemtime($filepath)));

			$c = &$r->appendNode("td", "align=right");
			$c->appendText(strval(intval(filesize($filepath) / 1024)) . "K");
		}

		$page .= $table->render();

		return $page;
	}
	
	function edit($uri){
		$filepath = $uri . "/info.html";
		$filepathabs = filePathAbs($filepath);
		$dirpath = "/" . dirname($filepath) . "/";
		
		if(file_exists($filepathabs)){
			$text = file_get_contents($filepathabs);
		}
		else{
			$text = "";
		}

		$page = "";
		$h = new BHtmlNode("form", "method='post' action='?cmd=save'");
		
		$text = htmlspecialchars($text);

		$toolbar = "[ ['Save','Print','-','Templates'],['Cut','Copy','Paste','PasteText','PasteFromWord','-','Undo','Redo']";
		$toolbar .= ",['Find','Replace','-','SelectAll','-','SpellChecker', 'Scayt' ],['Bold','Italic','Underline','Strike','Subscript','Superscript','-','RemoveFormat' ]";
		$toolbar .= ",['TextColor','BGColor'],['Source', 'Maximize', 'ShowBlocks','-','About' ]";
		$toolbar .= ",['Styles','Format','Font','FontSize'],['NumberedList','BulletedList','-','Outdent','Indent','-','Blockquote','CreateDiv','-','JustifyLeft','JustifyCenter','JustifyRight','JustifyBlock']";
		$toolbar .= ",['Link','Unlink','Anchor'], ['Image','Table','HorizontalRule','Smiley','SpecialChar','PageBreak']";
		$toolbar .= "]";
		$h->appendNode("script", "type='text/javascript' src='{$this->sys->baseUrl}/lib/ckeditor/ckeditor.js'");
		$h->appendNode("textarea", "name='editor' id='editor'", $text);
		$h->appendNode("script", "", "CKEDITOR.config.width = '100%';");
		$h->appendNode("script", "", "CKEDITOR.config.height = '600px';");
		$h->appendNode("script", "", "CKEDITOR.config.skin = 'kama';");
		$h->appendNode("script", "", "CKEDITOR.config.allowedContent = true;");
		$h->appendNode("script", "", "CKEDITOR.config.sourceAreaTabSize = 2;");
		$h->appendNode("script", "", "CKEDITOR.config.toolbar = {$toolbar};");
		$h->appendNode("script", "", "CKEDITOR.config.filebrowserBrowseUrl = '{$this->sys->baseUrl}/lib/FileBrowser/fileBrowser.php?type=files&workDir={$this->sys->workDir}';");
		$h->appendNode("script", "", "CKEDITOR.config.filebrowserImageBrowseUrl = '{$this->sys->baseUrl}/lib/FileBrowser/fileBrowser.php?type=images&workDir={$this->sys->workDir}';");
		$h->appendNode("script", "", "CKEDITOR.replace('editor');");
		$h->appendNode("script", "", "ckeditorAddText('{$text}');");
		
		$h->appendNode("input", "type='hidden' name='filepath' value='{$filepath}'");
		$h->appendNode("input", "type='submit' value='Submit'");
		$h->appendNode("input", "type='button' value='Cancel' onclick='location.href = \"javascript:history.go(-1)\";'");

		// Add popup
		$popUp = new BPopUp("AddPage", "?cmd=popupAddPage", 600, 200);
		$page .= $popUp->getHtml();
		$popUpDir = new BPopUp("AddDirectory", "?cmd=popupAddDirectory", 600, 200);
		$page .= $popUpDir->getHtml();
		$page .= "<script language='JavaScript'>function ckeditorAddText(text){e = CKEDITOR.instances.editor.insertHtml(text);}</script>\n";
		$page .= "<script language='JavaScript'>document.ws.ckeditorDirectory='" . $dirName . "';</script>\n";
		
		$page .= $h->render();
		return $page;
	}

	function save($uri){
		$filepath = filterString($_POST["filepath"]);
		if(preg_match("/.*\.html/", $filepath) || preg_match("/.*\.htm/", $filepath)){
			$text = "<html><head></head><body>\n";
			$text .= str_replace("\r", "", stripslashes($_POST["editor"]));
			$text .= "\n</body></html>\n";

			$filepathabs = filePathAbs($filepath);
//			print "SaveFile: {$uri} {$filepath} {$filepathabs}<br>"; return "";
			if(!file_put_contents(filePathAbs($filepath), $text)){
				return "SaveFile: error: {$path}<br>";
			}
			else {
				header(sprintf("Location: %s", urlPath($uri)));
				exit(0);
			}
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Can only save HTML files."));
		}
	}

	function addDirectory($uri){
		$page = "<h1>Add a new directory with index page</h1>";
	
		$obj = new PageAddObj();
		$view = new BEditObj("edit", "New Directory", "?cmd=addDirectory1", "");
		$view->setView(array(
			new BViewType("pageName", "Directory name"),
			));
			
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		$page .= "This will create a new directory, add an index.html file and add a link to this page at the end of the displayed page.";

		return $page;
	}

	function addDirectory1($uri){
		if(!isset($_POST["pageName"]) || ($_POST["pageName"] == "")){
			$this->sys->pageDisplayError(new BError(1, "You need to set a name for the directory"));
			return "";
		}
		$dirname = filterString($_POST["pageName"]);
		$filename = $dirname . "/info.html";
		$fileurl = urlPath($uri . "/". $filename);
		
		$dirpath = $this->filepathabs . "/" . $dirname;
		$filepath = $this->filepathabs . "/" . $filename;
		
//		print "addDirectory1: {$filename} {$dirpath} {$filepath}<br>";
		
		// Check if file exists
		
		// Create new dir
		mkdir($dirpath);
		
		// Create new page
		if($_SESSION["content.templateFile"])
			$content = file_get_contents($this->sys->baseDir . $_SESSION["content.templateFile"]);
		else
			$content = file_get_contents($this->sys->baseDir . "/lib/templates/Blank.html");
		file_put_contents($filepath, $content);
		
		// Add link to existing page
		$link = "<a href='{$filename}'>{$filename}</a><br>";
		$content = file_get_contents(filePathAbs($uri));
		$content = preg_replace("%</body>%", "{$link}\n</body>", $content);
		file_put_contents(filePathAbs($uri), $content);
		
		header ("Location: {$fileurl}?cmd=edit");
		exit(0);
	}

	function fileBrowser($uri){
		header ("Location: {$this->sys->baseUrl}/lib/FileBrowser/fileBrowser.php?workDir={$this->sys->workDir}");
	}
	
	function parseUri($uri){
		parent::parseUri($uri);
		$this->filepathabs = filePathAbs($uri);
	}
};

?>
