<?php
/*******************************************************************************
 *	Diary.php	Diary Module
 *			T.Barnaby,	BEAM Ltd,	2012-10-23
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjDiary extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "StringHtml"),
			new BMember("blog", "Int"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

class BObjDiaryEntry extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("diary", "Int", "Number"),
			new BMember("who", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "StringHtml"),
			new BMember("date", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

class Diary extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "diary";
		$this->title = "Diary";
		$this->info = "Diary module. This module allows the management of Diary items.";
		$this->group = "";
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("Diarys" => "BObjDiary"));
		$this->sys->db->addObjectClasses(array("DiaryEntrys" => "BObjDiaryEntry"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "diaryList";
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminGroup" . $this->group);
	}

	function checkFileAccess($uri){
		if(!strncmp($uri, "diary/", 6)){
			return 1;
		}
		
		return 0;
	}

	
	function getMenu(){
		if(isset($_GET['group']))
			$this->group = $_GET['group'];

		if($this->checkAccess()){
			return new BMenu("Diary", "/{$this->name}", "", array("Diary List" => "/{$this->name}", "Diary Add" => "/{$this->name}?cmd=add&amp;group={$this->group}"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		// Check if file access
		if($this->checkFileAccess($uri)){
			$module = $this->sys->modules["content"];
			return $module->getPage($uri);
		}

		$this->parseUri($uri);
		$_GET['group'] = $this->group;
		$_SESSION["group"] = $this->group;

		if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "entryView":		return $this->entryView($uri);		break;
			case "entryAdd":		return $this->entryEdit($uri);		break;
			case "entryEdit":		return $this->entryEdit($uri);		break;
			case "entrySave":		return $this->entrySave($uri);		break;
			case "entryDelete":		return $this->entryDelete($uri);	break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else {
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
	}

	function getPageBlock($uri){
		$this->parseUri($uri);
		$_GET['group'] = $this->group;

		if($this->checkAccess()){
			$help = $this->sys->baseUrl . "/modules/diary/info.html";

			$m = array();
			$m[] = new BMenu("Add Entry", "", "onmouseover='this.href = \"?cmd=entryAdd&amp;diaryId=\" + document.ws.list.objId'");
			$m[] = new BMenu("", "", "");
			$m[] = new BMenu("Add a new Diary", "", "href='?cmd=add'");
			$m[] = new BMenu("Help", "",		"href='{$help}'");
			$o = new BMenu("Diary", "", "", $m);
		}
		else {
			$o = Null;
		}
		return $o;
	}

	function viewList($uri){
		$page = "<h2>Diaries</h2>\n";

		if($this->checkAccess()){
			$view = new BViewDatabase("list", "selectAction");
		}
		else {
			$view = new BViewDatabase("list", "view");
		}

		if($this->group)
			$where = "`group`='" . dbEscapeString($this->group) . "'";
		else
			$where = "";
		
		$view->setView(array(
			new BViewType("id", "ID"),
			new BViewType("dateAdded", "Date Published", "date"),
			new BViewType("category", "Category"),
			new BViewType("group", "Group"),
			new BViewType("title", "Title"),
			new BViewType("description", "Description")
			));

		$view->setSelection($where);
		$view->setLimit(20);
	
		if($view->order == "")
			$view->setOrder("dateAdded");

		$view->setTable("Diarys");
		$page = $page . $view->getHtml();

#		$page .= "<div class='help'>";
#		$page .= "This lists all of the users of the system. ";
#		$page .= "In the left hand bar there are links to the functions you can perform.";
#		$page .= "</div>";

		return $page;
	}

	function edit($uri){
		if(is_object($uri)){
			$obj = $uri;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Diary</h2>";
			$obj = $this->sys->db->getObject("Diarys", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Diarys");
			$obj->group = $this->group;
			$obj->dateAdded = strftime("%Y-%m-%d");
		}

		$page = "<h2>Edit Diary</h2>";

		$view = new BEditObj("diary", "Diary Information", "?cmd=save&amp;group={$this->group}", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("dateAdded", "", "hidden"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category", "select", categoryGetList("diary")),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("description", "Description", "textareaHtml", "", 500, 280),
			new BViewType("blog", "Blog", "checkbox"),
			));

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save($uri){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjDiary();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			if($obj->id)
				$this->sys->db->putObject("Diarys", $obj);
			else
				$this->sys->db->appendObject("Diarys", $obj);
		}

		$page .= "<h3>Diary details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($uri){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("Diarys", $id);
			$this->sys->db->deleteObject("Diarys", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {
			$obj = $this->sys->db->getObject("Diarys", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
		}
			
		return $page;
	}

	function view($uri){
		if(isset($_GET["id"]))
			$this->diary = filterToInt($_GET["id"]);

		if($this->diary == 0)
			return $this->viewList($uri);

		$diary = $this->sys->db->getObject("Diarys", $this->diary);

		$page = new BHtmlNode("div", "class=Diary");
		$page->appendNode("h1", "", "Diary: " . $diary->title);

		$where = "`diary` = '" . dbEscapeString($this->diary) . "'";
		if($diary->blog)
			$order = "!date";
		else
			$order = "date";

		$list = $this->sys->db->getObjectList("DiaryEntrys", "", $where, $order);

		foreach($list as $obj){
			$date = explode(" ", $obj->date);
			$date = $date[0];
			$entry = &$page->appendNode("div", "class=DiaryEntry");
			$entry->appendNode("h1", "", $date . ": " . $obj->title);
			if($this->checkAccess()){
				$actions = $entry->appendNode("div");
				$actions->appendNode("a", "href='?cmd=entryEdit&amp;id={$obj->id}' title='Edit Item'", "Edit");
				$actions->appendText("&nbsp");
				$actions->appendNode("a", "href='?cmd=entryDelete&amp;id={$obj->id}' title='Delete Item'", "Delete");
			}
			$entry->appendNode("div", "", $obj->description);
		}
		$page->appendNode("script", "language='JavaScript'", "document.ws.list = {}; document.ws.list.objId = '{$this->diary}'");

		return $page->render();
	}

	function entryEdit($uri){
		if(is_object($uri)){
			$obj = $uri;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Diary Entry</h2>";
			$obj = $this->sys->db->getObject("DiaryEntrys", filterToInt($_GET["id"]));
		}
		else {
			if(isset($_GET["diaryId"])){
				$obj = $this->sys->db->getNewObject("DiaryEntrys");
				$obj->diary = intval($_GET["diaryId"]);
				$obj->who = $this->login->id;
				$obj->group = $this->group;
				$obj->dateAdded = strftime("%Y-%m-%d");
//				$obj->date = strftime("%Y-%m-%d %H:%M:%S");
				$obj->date = strftime("%Y-%m-%d");
			}
			else {
				return "Need to select a Diary to add items to";
			}
		}

		$page = "";

		$view = new BEditObj("diary", "Diary Edit Entry", "?cmd=entrySave&amp;group={$this->group}", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("dateAdded", "", "hidden"),
			new BViewType("who", "", "hidden"),
			new BViewType("diary", "Diary", "hidden"),
			new BViewType("date", "Date", "date"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category", "select", categoryGetList("diary")),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("description", "Description", "textareaHtmlFull", "", 800, 500),
			));

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function entrySave($uri){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjDiaryEntry();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			if($obj->id)
				$this->sys->db->putObject("DiaryEntrys", $obj);
			else
				$this->sys->db->appendObject("DiaryEntrys", $obj);
		}

		$page .= "<h3>Diary details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function entryDelete($uri){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("DiaryEntrys", $id);
			$this->sys->db->deleteObject("DiaryEntrys", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {
			$obj = $this->sys->db->getObject("DiaryEntrys", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=entryDelete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
		}
			
		return $page;
	}

	function parseUri($uri){
		parent::parseUri($uri);

		$this->group = "";
		$this->arg = "";
		$this->diary = 0;

		if(strpos($uri, "/")){
			$this->group = substr($uri, strpos($uri, "/") + 1);
			$this->group = filterPath($this->group, $this->sys->baseDir);
			if(strpos($this->group, "/")){
				$this->arg = substr($this->group, strpos($this->group, "/") + 1);
				$this->group = substr($this->group, 0, strpos($this->group, "/"));
			}
		}
		if(isset($_GET["diary"]))
			$this->diary = intval($_GET["diary"]);
	}
};

function diaryList($diary=0, $num=3){
	global	$sys;
	
	$diary = intval($diary);
	$num = intval($num);
	
	if($diary){
		$where = "`diary`='{$diary}'";
	}
	else {
		$diary = 1;
		if(isset($_GET['group'])){
			$group = filterString($_GET['group']);
			$where = "`group`='{$group}'";
		}
		else {
			$where = "";
		}
	}

	$diaryObj = $sys->db->getObject("Diarys", $diary);
	$objList = $sys->db->getObjectList("DiaryEntrys", "", $where, "!date", "{$num}");

	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/diary?cmd=view&amp;id={$diary}'")->appendNode("h2", "", "Diary: " . $diaryObj->title);

	foreach($objList as $obj){
		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->dateAdded));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/diary?cmd=view&amp;id={$obj->diary}'", $obj->title);
//		$d1->appendNode("div", "class='NewsSummaryCategory'", "Category: General");
//		$d1->appendNode("div", "class='NewsSummaryAuthor'", "Posted by: terry");
		$d1->appendNode("div", "class='NewsSummaryContent'", $obj->description);
	}

	return $html->render();
}

?>
