<?php
/*******************************************************************************
 *	EventBookingForm.php	BookingForm Module
 *				T.Barnaby,	BEAM Ltd,	2013-01-29
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("EventBookingTcn2013" => "BObjEventBooking"));

// Overall booking options
// If event has TShirts
global	$tshirtSizes;
$tshirtSizes = array("S", "M", "L", "XL", "XXL");

class BObjEventBooking extends BObj {
	function __construct(){
		global $tshirtSizes;

		$m = array();

		$m[] = new BMember("id", "Int", "Number");
		$m[] = new BMember("deleted", "Int", "Number");

		for($n = 0; $n < 6; $n++){
			$m[] = new BMember("name{$n}", "String");
			$m[] = new BMember("forename{$n}", "String");
			$m[] = new BMember("surname{$n}", "String");
			$m[] = new BMember("name{$n}", "String");
			$m[] = new BMember("sex{$n}", "Int");
			$m[] = new BMember("dateOfBirth{$n}", "Date", "date", array('format' => '%Y-%n-%j'));
			$m[] = new BMember("age{$n}", "Int");
		}
		
		$m[] = new BMember("member", "String");
		$m[] = new BMember("address", "String");
		$m[] = new BMember("postcode", "String");
		$m[] = new BMember("email", "String");
		$m[] = new BMember("telephone", "String");
		$m[] = new BMember("mobileTelephone", "String");
		$m[] = new BMember("arrivalDate", "Date");
		$m[] = new BMember("departureDate", "Date");
		$m[] = new BMember("accommodation", "String");
		$m[] = new BMember("currency", "String");
		$m[] = new BMember("help", "String");
		$m[] = new BMember("notes", "String");

		$m[] = new BMember("adults", "Int");
		$m[] = new BMember("children", "Int");
		$m[] = new BMember("infants", "Int");
		$m[] = new BMember("campingAdults", "Int");
		$m[] = new BMember("campingChildren", "Int");

		for($n = 0; $n < count($tshirtSizes); $n++){
			$m[] = new BMember("tshirt{$n}", "Int");
		}

		// Extra bits
		$m[] = new BMember("electricity", "Int");
		$m[] = new BMember("coachTripAdult", "Int");
		$m[] = new BMember("coachTripChild", "Int");

		for($n = 0; $n < 12; $n++){
			$m[] = new BMember("q{$n}", "Int");
			$m[] = new BMember("e{$n}", "Float");
			$m[] = new BMember("t{$n}", "Float");
		}
		$m[] = new BMember("total", "Float");
		$m[] = new BMember("discount", "Float");
		$m[] = new BMember("paid", "Float");

		$m[] = new BMember("addedDate", "Date");
		$m[] = new BMember("addedUserName", "String");

		parent::__construct($m);
	}

	function dbTableFields(){
		global $tshirtSizes;

		$s = "(`id` int NOT NULL auto_increment,";
		$s .= "`deleted` int default 0,";
		
		for($n = 0; $n < 6; $n++){
			$s .= "`name{$n}` text,";
			$s .= "`forename{$n}` text,";
			$s .= "`surname{$n}` text,";
			$s .= "`sex{$n}` int,";
			$s .= "`dateOfBirth{$n}` date,";
			$s .= "`age{$n}` int,";
		}

		$s .= "`member` text,";
		$s .= "`address` text,";
		$s .= "`postcode` text,";
		$s .= "`email` text,";
		$s .= "`telephone` text,";
		$s .= "`mobileTelephone` text,";
		$s .= "`arrivalDate` date,";
		$s .= "`departureDate` date,";
		$s .= "`accommodation` text,";
		$s .= "`currency` text,";
		$s .= "`help` text,";
		$s .= "`notes` text,";

		$s .= "`adults` int,";
		$s .= "`children` int,";
		$s .= "`infants` int,";
		$s .= "`campingAdults` int,";
		$s .= "`campingChildren` int,";

		for($n = 0; $n < count($tshirtSizes); $n++){
			$s .= "`tshirt{$n}` int,";
		}

		// Extra bits
		$s .= "`electricity` int,";
		$s .= "`coachTripAdult` int,";
		$s .= "`coachTripChild` int,";

		for($n = 0; $n < 12; $n++){
			$s .= "`q{$n}` float,";
			$s .= "`e{$n}` float,";
			$s .= "`t{$n}` float,";
		}
		$s .= "`total` float,";
		$s .= "`discount` float,";
		$s .= "`paid` float,";

		$s .= "`addedDate` text,";
		$s .= "`addedUserName` text,";
		$s .= "PRIMARY KEY  (`id`))";

		return $s;
	}
};

class Price {
	function __construct($number, $item, $unit, $pricePounds, $priceEuros){
		$this->number = $number;
		$this->item = $item;
		$this->unit = $unit;
		$this->pricePounds = $pricePounds;
		$this->priceEuros = $priceEuros;
	}
};

class EventBookingForm {
	// Basic Interface routines
	function __construct(&$sys, $event){
		global $tshirtSizes;

		$this->tshirtSizes = $tshirtSizes;
		$this->sys = $sys;
		$this->event = $event;
		$this->name = $this->event->name;
		$this->eventTitle = ucfirst($this->event->title);
		$this->eventStartDate = new BDate($this->event->startDate);
		$this->eventEndDate = new BDate($this->event->endDate);
		$this->accommodationStartDate = new BDate($this->event->startDate);
		$this->accommodationStartDate->timeStamp -= 0 * 24 * 3600;
		$this->accommodationEndDate = new BDate($this->event->endDate);
		$this->accommodationEndDate->timeStamp += 0 * 24 * 3600;
		$this->bookingStartDate = new BDate($this->event->bookingStartDate);
		$this->bookingEndDate = new BDate($this->event->bookingEndDate);
		$this->extrasEndDate = new BDate($this->event->extrasEndDate);
		$this->bookingTable = "EventBooking" . ucfirst($this->name);
		$this->ageAdult = 16;
		$this->ageChild = 5;
		$this->ageAdultCamping = 13;
		$this->ageChildCamping = 5;
		$this->ageInfantCamping = 0;
		$this->ageChildCampingMax = $this->ageAdultCamping - 1;
		
		$this->prices = array();
		$this->prices[] = new Price(0, "Rally fee adult ({$this->ageAdult}+)", "Per adult", 16.0, 0.0);
		$this->prices[] = new Price(1, "Camp fee", "Per pitch per night (up to 4 people)", 13.50, 0.0);
		$this->prices[] = new Price(2, "Additional adults", "Per adult per night", 2.0, 0.0);
		$this->prices[] = new Price(3, "Additional children", "Per child per night", 1.0, 0.0);
		$this->prices[] = new Price(4, "Electricity", "Per night", 2.0, 0.0);

		// Extras
		$this->prices[] = new Price(5, "Cottage1", "Per week", 295.0, 0.0);
		$this->prices[] = new Price(6, "Cottage2", "Per week", 395.0, 0.0);
		$this->prices[] = new Price(7, "Cottage3", "Per week", 395.0, 0.0);
		$this->prices[] = new Price(8, "Cottage4", "Per week", 420.0, 0.0);
		$this->prices[] = new Price(9, "Coach trip", "Per adult", 6.0, 0.0);
		$this->prices[] = new Price(10, "Coach trip", "Per child", 4.0, 0.0);

		if(count($this->tshirtSizes)){
			$this->prices[] = new Price(11, "T-Shirts", "Each", 7.0, 0.0);
		}
		
		for($n = 0; $n < count($this->prices); $n++){
			$this->prices[$n]->priceEuros = ceil($this->prices[$n]->pricePounds * 1.20);
		}
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin");
	}
		
	function getPage($uri){
		if(0){
			printf("Uri: %s<br>\n", $uri);
			print("Get: "); print_r($_GET); print("<br>\n");
			print("Post: "); print_r($_POST); print("<br>\n");
			print("Post: "); print_r($_SERVER); print("<br>\n");
			if($_SERVER["REMOTE_ADDR"] == "82.152.34.172")
				print "Terry";
		}

		// Check if before booking start date
		$dateNow = new BDate("now");

		if(! $this->checkAccess()){
			if($dateNow->timeStamp < $this->bookingStartDate->timeStamp){
				$str = "<center><H2>Bookings for this event are not open yet</H2></center>";
				$str .= "<center><H2>They open on {$this->bookingStartDate->getDateTime("%A %e %B %Y")}</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
			if($dateNow->timeStamp > $this->bookingEndDate->timeStamp){
				$str = "<center><H2>Bookings for this event are now closed</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
		}

		if(isset($_POST["buttonBook"])){
			$obj = new BObjEventBooking();
			$obj->set($_POST);
			
			$editing = 0;
			if($obj->id == 0){
				$obj->addedDate = strftime("%Y-%m-%d");
				$obj->addedUserName = $this->sys->user->name;
			}
			else {
				$editing = 1;
			}

/*			
			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
*/
//			BDatabase::$debug = 1;
			if($obj->id)
				$this->sys->db->putObject($this->bookingTable, $obj);
			else
				$obj = $this->sys->db->appendObject($this->bookingTable, $obj);

			$h = new BHtmlNode("h2", "style='text-align: center;'", "Thank you, {$obj->name0}, your Booking for {$this->eventTitle} has been accepted");
			$h->appendNode("h2", "style='text-align: center;'", "Your BookingRef is: {$obj->id}");
			$h->appendNode("h2", "style='text-align: center;'", "An email has been sent confirming this booking, with a copy of your booking form that you can print");
			$h->appendNode("h2", "style='text-align: center;'", "We hope you enjoy the event - Happy Tandeming !");
			
			if($obj->paid == 0){
				$h->appendNode("h3", "", "Please pay in one of the following ways, (Please use the reference: \"{$this->name} {$obj->id}\" )");
				$r = &$h->appendNode("ol");
				$r->appendNode("li", "", "For people in the UK you can pay in pounds sterling using direct bank transfer to the account: Bank: Barclays Bank, AccountName: The Tandem Club, Sortcode: 20-71-03, AccountNumber: 73018431, Reference: \"{$this->name} {$obj->id}\"");
				$r->appendNode("li", "", "For people in the UK you can post a cheque in pounds sterling for total amount, with name of Party Leader and BookingRef on the reverse, payable to: \"The Tandem Club\" to: Linda Stinchcombe, 17 Stockwell Ave. Mangotsfield, Bristol BS16 9DR, United Kingdom.");
				$r->appendNode("li", "", "For European bookings pay in euros by direct bank transfer to the following bank account: F.E.R. Oudhof, IBAN: NL58INGB0006385650, BIC: INGBNL2A use the reference \"{$this->name} {Name} \"(Substitute \"{Name}\" with the party leaders surname and initials, if they will fit!). For more information contact: Eelco Oudhof, Lichtboei 26 NL-1186 DE Amstelveen, The Netherlands, email: administratie@tandemclub.nl.");
				$r->appendNode("li", "", "International bookings outside Europe pay by direct money transfer to the euros account given above, or in Euro's cash on arrival on the rally site.");
			}
			$h->appendNode("p", "", "If you have any problems with the booking form or system please email: <a href='mailto:{$this->name}@events.tandem-club.org.uk'>{$this->name}@events.tandem-club.org.uk</a>");
			
			$page = $h->render();
			$page .= $this->view($obj);
			
			if($editing == 0){
				// Send an email
				$h = new BHtmlNode("html");
				$b = &$h->appendNode("head");
				$b = &$h->appendNode("body", "style='background-color: #CCDDDD;'");
				$b->appendNode("style", "type='text/css'", "table { empty-cells: show; }");
				$b->appendText($page);
//				sendEmailTo($obj->email, "{$this->sys->siteTitle}: Event booking: {$obj->id}", $h->render(), 1);
				sendEmailFromTo("{$this->name}@events.tandem-club.org.uk", $obj->email, "{$this->sys->siteTitle}: Event booking: {$obj->id}", $h->render(), 1);
			}
		}
		else {
			$page = "";
			$page .= $this->header();
			$page .= $this->form(0);
		}

		return $page;
	}

	function header(){
		$startDate = $this->eventStartDate->getDateTime("%A %e %B %Y");
		$endDate = $this->eventEndDate->getDateTime("%A %e %B %Y");
		$bookingEndDate = $this->bookingEndDate->getDateTime("%A %e %B %Y");
		$text = <<< End_Of_Text
<h1 style="text-align: center;">{$this->eventTitle}</h1>
<h2 style="text-align: center;">{$startDate} - ${endDate}</h2>
<h1 style="text-align: center;">Booking Form</h1>
<h4 style="text-align: center;">Bookings to be received by {$bookingEndDate}</h4>

<h3>Important Notes</h3>
<ul>
<li>This event is open to UK or overseas Tandem Club Members or, in the case of overseas people, members of the following organisations:
"Tandem Club of America", "Tandem Club de France", "Tandem Club NL (Netherlands)", "Tandem ClubVlaanderen (Belgium)" and "Tandem-fahren.de (Germany)".
You can join the Tandem Club using the forms on the TC website. <a href='http://www.tandem-club.org.uk/_subscriptiondetails.htm'>Joining the TC</a> (About 10 UKP per year)    </li>
<li>All bookings to be received by Rally organisers by: {$bookingEndDate}. Subject to availability.</li>
<li>The Rally fee covers the costs incurred in arranging the rally. This includes equipment costs, evening event costs,
Postage and printing of route instructions plus	all the other bits and pieces to make it all happen.</li>
<li>Tandem Club Membership numbers are on the mailing label for your Journal or on any renewal forms. Please enter your cycling clubs name for overseas non TC members.
If you don't know your membership number enter "Unknown" in the membership number box and we will find it for you or email us
at: <a href="mailto:{$this->name}@events.tandem-club.org.uk">{$this->name}@events.tandem-club.org.uk</a> so we can find it for you.</li>

<li>Camping Child rates apply to those between {$this->ageChildCamping} to {$this->ageChildCampingMax} inclusive on {$startDate}.</li>
<li>Camping Children/infants below {$this->ageChildCamping} on {$startDate} are free.</li>
<li>The Cottages need to be booked with the camp site direct. Please book these and then fill in this form indicating this accommodation type on the form.</li>

</ul>
<h3>Booking procedure</h3>
<ol>
<li>Please complete this form and then use the "Calculate" button at the end to calculate the payments.
Make sure you choose the currency type you will be using to pay.</li>
<li>Dates of birth only need to be entered (changed from the default) for people below the age of 18.</li>
<li>Check the payment values and if Ok use the "Book" button to book for the event. You will be sent
and email with the booking information and will be given a "BookingRef" number.</li>

<li>Please then pay in one of the following ways, (Please use the reference: "{$this->name} {BookingRef}"
substituting "{BookingRef}" with the booking reference number you have been given), the account
number details and address to post cheques will be given after you book:<br>
<ul>
<li>For people in the UK you can pay in pounds sterling using direct bank transfer to the Tandem Clubs
account.
<li>For people in the UK you can post a cheque in pounds sterling for total amount, with name of
Party Leader and BookingRef on the reverse, payable to: "The Tandem Club".

<li>For European bookings pay in euros by direct bank transfer to a european bank.

<li>International bookings outside Europe pay by direct money transfer to the euros account given above, or
in Euro's cash on arrival on the rally site.</li> 
</ul>
<li>Note that the TAB key can be used to move to the next field</li>
<li>If you have any problems with the booking form or system please email:
<a href="mailto:{$this->name}@events.tandem-club.org.uk">{$this->name}@events.tandem-club.org.uk</a></li>
</ol>
End_Of_Text;

		return $text;
	}

	function setDataFromPost(&$booking){
		// Sets the data from POST info
		foreach($booking as $k => $v){
			if(isset($_POST[$k])){
				$booking->$k = $_POST[$k];
			}
		}
		for($n = 0; $n < 6; $n++){
			if($booking->{"surname$n"}){
				$booking->{"name$n"} = $booking->{"forename$n"} . " " . $booking->{"surname$n"};
			}
		}
	}

	function calculate(&$booking){
		$adultsTc = 0;
		$adults = 0;
		$children = 0;
		$infants = 0;
		$campingAdults = 0;
		$campingChildren = 0;
		$camping = 0;
		$campingNumber = 0;
		
		switch($booking->accommodation){
		case "Camping":
		case "Caravan":
		case "MotorHome":
			$camping = 1;
			break;
		}

		for($n = 0; $n < count($this->prices); $n++){
			if($booking->currency == "Pounds")
				$booking->{"e$n"} = $this->prices[$n]->pricePounds;
			else
				$booking->{"e$n"} = $this->prices[$n]->priceEuros;
		}

		for($n = 0; $n < 6; $n++){
			if($booking->{"name$n"} != ""){
				$dateOfBirth = new BDate($booking->{"dateOfBirth$n"});
				$age = $this->eventStartDate->getYear() - $dateOfBirth->getYear();
				if($this->eventStartDate->getMonth() < $dateOfBirth->getMonth()){
					$age--;
				}
				else if(($this->eventStartDate->getMonth() == $dateOfBirth->getMonth()) &&
					($this->eventStartDate->getDay() < $dateOfBirth->getDay())){
					$age--;
				}
				$booking->{"age$n"} = $age;
				
//				print("Age: {$age}<br>\n");
				
				if($age >= $this->ageAdult){
					$adults++;
				}
				else if($age >= $this->ageChild){
					$children++;
				}
				else {
					$infants++;
				}
		
				if($camping){		
					if($age >= $this->ageAdultCamping){
						$campingAdults++;
					}
					else if($age >= $this->ageChildCamping){
						$campingChildren++;
					}
				}
			}
		}
		
		$booking->adults = $adults;
		$booking->children = $children;
		$booking->infants = $infants;
		$booking->campingAdults = $campingAdults;
		$booking->campingChildren = $campingChildren;

		$arrivalDate = new BDate($booking->arrivalDate);
		$departureDate = new BDate($booking->departureDate);
		
		$days = ($departureDate->timeStamp - $arrivalDate->timeStamp) / 86400;
		
		$booking->q0 = $adults;
		$booking->q1 = 0;
		$booking->q2 = 0;
		$booking->q3 = 0;
		$booking->q4 = 0;
		$booking->q5 = 0;
		$booking->q6 = 0;
		$booking->q7 = 0;
		$booking->q8 = 0;
		$booking->q9 = 0;
		$booking->q10 = 0;
		$booking->q11 = 0;
		$booking->q12 = 0;
		$booking->q13 = 0;

		switch($booking->accommodation){
		case "Camping":
		case "Caravan":
		case "MotorHome":
			$booking->q1 = $days;
			if($campingAdults > 4)
				$booking->q2 = ($campingAdults - 4) * $days;
			if($campingChildren){
				$booking->q3 = ($campingChildren - (($campingAdults < 4) ? (4 - $campingAdults):0)) * $days;
				if($booking->q3 < 0)
					$booking->q3 = 0;
			}
			break;
		case "Cottage1":
//			$booking->q5 = 1;
			break;
		case "Cottage2":
//			$booking->q6 = 1;
			break;
		case "Cottage3":
//			$booking->q7 = 1;
			break;
		case "Cottage4":
//			$booking->q8 = 1;
			break;
		}

		$booking->q4 = $booking->electricity * $days;
		$booking->q9 = intval($booking->coachTripAdult);
		$booking->q10 = intval($booking->coachTripChild);

		$booking->q11 = 0;
		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$booking->q11 += intval($booking->{"tshirt$n"});
		}

		$total = 0;
		for($n = 0; $n < count($this->prices); $n++){
			$booking->{"t$n"} = $booking->{"q$n"} * $booking->{"e$n"};
			$total += $booking->{"t$n"};
		}
		$total = round($total, 2);
		$booking->total = $total;
	}
	
	function validate(&$booking, $editing, &$errors){
		$err = 0;
		
		if($booking->name0 == ""){
			$err = 1;
			$errors[] = "We need the party leaders name";
		}
		
		// Email
//		if(!preg_match("/.+@.+/", $booking->email)){
		if(!filter_var($booking->email, FILTER_VALIDATE_EMAIL)){
			$err = 1;
			$errors[] = "No email or incorrect email format";
		}

		$this->findTotals();
		$arrivalDate = new BDate($booking->arrivalDate);
		$departureDate = new BDate($booking->departureDate);
		$accommodationStartDate = $this->eventStartDate;
		$accommodationEndDate = $this->eventEndDate;

		if(!$editing){
			switch($booking->accommodation){
			case "Camping":
			case "Caravan":
			case "MotorHome":
			case "Off-site";
				$accommodationStartDate = $this->accommodationStartDate;
				$accommodationEndDate = $this->accommodationEndDate;
				break;
			case "Cottage1":
				if($this->totalCottage1 >= 50){
					$err = 1;
					$errors[] = "All cottage1 have gone";
				}
				break;
			case "Cottage2":
				if($this->totalCottage2 >= 50){
					$err = 1;
					$errors[] = "All cottage2 have gone";
				}
				break;
			case "Cottage3":
				if($this->totalCottage3 >= 50){
					$err = 1;
					$errors[] = "All cottage3 have gone";
				}
				break;
			case "Cottage4":
				if($this->totalCottage4 >= 50){
					$err = 1;
					$errors[] = "All cottage4 have gone";
				}
				break;
			}

			if($booking->electricity && ($this->electricNumber >= 24)){
				$err = 1;
				$errors[] = "All the electricity hookups have gone";
			}
			
			if($booking->coachTripChild || $booking->coachTripAdult){
				if(($this->coachTripNumber + intval($booking->coachTripChild) + intval($booking->coachTripAdult)) > 50){
					$err = 1;
					$errors[] = "All the coach trip seats have gone";
				}
			}
		}
		
		if($arrivalDate->timeStamp < ($accommodationStartDate->timeStamp)){
			$err = 1;
			$errors[] = "Arrival date before event. Extra nights can be booked direct with campsite.";
		}
		if($departureDate->timeStamp > ($accommodationEndDate->timeStamp)){
			$err = 1;
			$errors[] = "Departure date after event. Extra nights can be booked direct with campsite.";
		}
		
		return $err;
	}


	function form($booking){
		$dateNow = new BDate("now");
		$helpSelect = array("None", "Arrival Desk", "Childrens Club", "Bar", "Games", "Rally Event", "End of Rally Clear up", "Anything");
		$accommodation = array("Camping", "Caravan", "MotorHome", "Cottage (Booked with campsite)", "Off-site");

		if($this->checkAccess()){
//			$accommodation = array_merge($accommodation, array("Cottage1", "Cottage2", "Cottage3", "Cottage4"));
		}

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='/events/{$this->name}/bookingForm#prices' method='post'");

		if($booking == 0){
			$editing = 0;
			$booking = new BObjEventBooking();
			for($n = 0; $n < 6; $n++){
				$booking->{"name$n"} = "";
				$booking->{"forename$n"} = "";
				$booking->{"surname$n"} = "";
				$booking->{"sex$n"} = "0";
				$booking->{"dateOfBirth$n"} = "1960-01-01";
			}
			$booking->member = "";
			$booking->address = "";
			$booking->postcode = "";
			$booking->email = "";
			$booking->telephone = "";
			$booking->mobileTelephone = "";
			$booking->arrivalDate = $this->eventStartDate->get();
			$booking->departureDate = $this->eventEndDate->get();
			$booking->accommodation = "";
			$booking->currency = "Pounds";
			$booking->help = "";
			$booking->notes = "";
			
			$booking->coachTripAdult = 0;
			$booking->coachTripChild = 0;
			for($n = 0; $n < count($this->tshirtSizes); $n++){
				$booking->{"tshirt$n"} = 0;
			}

			for($n = 0; $n < count($this->prices); $n++){
				$booking->{"q$n"} = 0;
				$booking->{"e$n"} = 0;
				$booking->{"t$n"} = 0;
			}
			$booking->total = 0;
			$booking->discount = 0;
			$booking->paid = 0;

//			print("Data: "); print_r($booking); print("<br>\n");
		}
		else {
			$editing = 1;
			// Booking Info
			$f->appendNode("h2", "", "Editing Booking");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$t->appendTableInput("Party Leader", "info0", "view", $booking->name0, "width:300px;"); 
			$t->appendTableInput("Booking Id", "info1", "view", $booking->id, "width:300px;"); 
			$t->appendTableInput("Added By", "info2", "view", $booking->addedUserName, "width:300px;"); 
			$t->appendTableInput("Added Date", "info3", "view", $booking->addedDate, "width:300px;"); 
		}
		
		if(isset($_POST["editing"]))
			$editing = $_POST["editing"];
		$this->setDataFromPost($booking);
		$this->calculate($booking);
//		print_r($booking);
		
		$f->appendNode("input", "type='hidden' name='id' value='{$booking->id}'");
		$f->appendNode("input", "type='hidden' name='editing' value='{$editing}'");

		// People
		$f->appendNode("h3", "", "Names of All Attending");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Forename");
		$r->appendNode("th", "", "Surname");
		$r->appendNode("th", "", "Male/Female (Male ticked)");
		$r->appendNode("th", "", "Date of Birth");

		for($n = 0; $n < 6; $n++){
			$r = &$t->appendNode("tr");
			if($n == 0)
				$r->appendNode("td", "", "Name " . $n . ": Party Leader");
			else
				$r->appendNode("td", "", "Name " . $n);

			if($booking->{"surname$n"})
				$booking->{"name$n"} = $booking->{"forename$n"} . " " . $booking->{"surname$n"};
			else
				$booking->{"name$n"} = "";
			$r->appendTableRowInput("", "forename$n", "text", $booking->{"forename$n"}, "width: 150px;");
			$r->appendTableRowInput("", "surname$n", "text", $booking->{"surname$n"}, "width: 150px;");
			$r->appendTableRowInput("", "sex$n", "checkbox", $booking->{"sex$n"});
			$r->appendTableRowInput("", "dateOfBirth$n", "date", $booking->{"dateOfBirth$n"});
			$r->appendNode("input", "type='hidden' name='name$n' value='{$booking->{"name$n"}}'");
			$r->appendNode("input", "type='hidden' name='age$n' value='{$booking->{"age$n"}}'");
		}
		
		$f->appendNode("h3", "", "General");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("TC Membership Number or Other Cycling organisation name", "member", "text", $booking->member, "width:300px;"); 
		$t->appendTableInput("Postal Address", "address", "textarea", $booking->address, "height:100px; width:400px"); 
		$t->appendTableInput("Postcode", "postcode", "text", $booking->postcode, "width:300px;"); 
		$t->appendTableInput("Email Address", "email", "text", $booking->email, "width:300px;"); 
		$t->appendTableInput("Telephone (with code)", "telephone", "text", $booking->telephone, "width:300px;"); 
		$t->appendTableInput("Mobile Telephone (with code)", "mobileTelephone", "text", $booking->mobileTelephone, "width:300px;"); 
		$t->appendTableInput("Arrival Date", "arrivalDate", "date", $booking->arrivalDate, "width:300px;"); 
		$t->appendTableInput("Departure Date", "departureDate", "date", $booking->departureDate, "width:300px;"); 
		$t->appendTableInput("Accommodation", "accommodation", "select", $booking->accommodation, "", $accommodation); 
		$t->appendTableInput("Electricity (Camping)", "electricity", "checkbox", $booking->electricity, "width:50px;"); 

		$t->appendTableInput("Payment Currency", "currency", "select", $booking->currency, "", array("Pounds", "Euros")); 
		$t->appendTableInput("Happy to help with", "help", "select", $booking->help, "", $helpSelect); 
		$t->appendTableInput("Additional Information", "notes", "textarea", $booking->notes, "height:100px; width:400px;"); 

		if($this->checkAccess() || ($dateNow->timeStamp < $this->extrasEndDate->timeStamp)){
			for($n = 0; $n < count($this->tshirtSizes); $n++){
				$size = $this->tshirtSizes[$n];
				$t->appendTableInput("T-Shirts {$size} (Number of)", "tshirt{$n}", "text", $booking->{"tshirt$n"}, "width:50px;"); 
			}
		}
		$t->appendTableInput("Coach Trip (Number of adults)", "coachTripAdult", "text", $booking->coachTripAdult, "width:50px;"); 
		$t->appendTableInput("Coach Trip (Number of children)", "coachTripChild", "text", $booking->coachTripChild, "width:50px;"); 

		// Prices
		$f->appendNode("a", "name=prices", "");
		$f->appendNode("h3", "", "Prices");
		$f->appendNode("i", "", "This section is automatically filled in from the above information. Please press the \"Calculate\" button to make sure this is updated.");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Unit");
		$r->appendNode("th", "", "Quantity");
		$r->appendNode("th", "", "Cost");
		$r->appendNode("th", "", "Total");
		
		for($n = 0; $n < count($this->prices); $n++){
//			if(($n < 5) || ($n > 8) || $this->checkAccess()){
			if(($n < 5) || ($n > 8)){
				$price = $this->prices[$n];
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $price->item);
				$r->appendNode("td", "", $price->unit);
				$r->appendTableRowInput("", "q{$n}", "textView", $booking->{"q$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "e{$n}", "textView", $booking->{"e$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "t{$n}", "textView", $booking->{"t$n"}, "background-color:yellow");
			}
		}

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendTableRowInput("", "currencyDisplay", "view", $booking->currency, "background-color:yellow;"); 
		$r->appendTableRowInput("", "total", "textView", $booking->total, "background-color:yellow;"); 

		if($this->checkAccess()){
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Discount");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "discount", "text", $booking->discount, "background-color:yellow; width:100px;"); 

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Paid");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "paid", "text", $booking->paid, "background-color:yellow; width:100px;"); 
		}
		
		// Add hidden info
		$f->appendNode("input", "type='hidden' name='adults' value='{$booking->adults}'");
		$f->appendNode("input", "type='hidden' name='children' value='{$booking->children}'");
		$f->appendNode("input", "type='hidden' name='infants' value='{$booking->infants}'");
		$f->appendNode("input", "type='hidden' name='campingAdults' value='{$booking->campingAdults}'");
		$f->appendNode("input", "type='hidden' name='campingChildren' value='{$booking->campingChildren}'");
		$f->appendNode("input", "type='hidden' name='addedUserName' value='{$booking->addedUserName}'");
		$f->appendNode("input", "type='hidden' name='addedDate' value='{$booking->addedDate}'");


		// Buttons
		$f->appendNode("h3", "", "Book");
		$r = &$f->appendNode("ul");
		$r->appendNode("li", "", "First use the Calculate button to calculate the cost and validate the booking. Once this has been done please use the Book button.");
		$r->appendNode("li", "", "Please do not use the back button to edit and re-submit once sent. Contact the administrators to change the booking.");
		$r->appendNode("li", "", "Please see the Booking procedure info at the top of this form for details on how to pay.");
		$r->appendNode("li", "", "The rally fee will not be refunded after the closing date. Refund of other components will depend on what the Club can recover from the campsite and/or other suppliers.");
		$r->appendNode("li", "", "Photographs taken at the event may be published on the Tandem Club Website and/or The Tandem Club Journal.");
		
		$f->appendNode("p", "", "If you participate in any ride or event organised by the Tandem Club, you do so entirely at your own risk. The club, its officers and ride organisers accept no responsibility for loss, damage to property, accident, injury, or death, that may occur, however caused, whether by negligence, deliberate act or otherwise.");

		$err = 0;		
		if(isset($_POST["buttonCalculate"])){
			$errors = array();
			if($err = $this->validate($booking, $editing, $errors)){
				$f->appendNode("h3", "style='color: red'; font-size: 150%;", "There are errors in the form data");
				$r = &$f->appendNode("ul");
				foreach($errors as $e){
					$r->appendNode("li")->appendNode("span", "style='color: red; font-size: 150%;'", $e);
				}
				$f->appendText("Please correct these errors and click on the Calculate button again<br><br>");
			}
		}

		if(($err == 1) || !isset($_POST["buttonCalculate"]))
			$disable = "disabled='disabled'";
		else
			$disable = "";

		$f->appendNode("input", "type='submit' name='buttonCalculate' style='background-color: lightgreen; font-size: 200%;' value='Calculate'");
		$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
//		$f->appendNode("button", "type='button' name='buttonPrint' $disable style='background-color: lightgreen; font-size: 200%;' onclick='window.print();'", "Print Form");
//		$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
		if($editing)
			$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Save Changes' onclick='this.value=\"Saving, please wait...\";'");
		else
			$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Book' onclick='this.value=\"Booking, please wait...\";'");

		return $f->render();
	}

	function view($booking){
		$f = new BHtmlNode("div");
		
		// Booking Info
		$f->appendNode("h3", "", "Booking Info");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("Party Leader", "info0", "view", $booking->name0, "width:300px;"); 
		$t->appendTableInput("Booking Id", "info1", "view", $booking->id, "width:300px;"); 
		$t->appendTableInput("Added By", "info2", "view", $booking->addedUserName, "width:300px;"); 
		$t->appendTableInput("Added Date", "info3", "view", $booking->addedDate, "width:300px;"); 

		// People
		$f->appendNode("h3", "", "Names of All Attending");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Name");
		$r->appendNode("th", "", "Male/Female (Male 1)");
		$r->appendNode("th", "", "Date of Birth");

		for($n = 0; $n < 6; $n++){
			$r = &$t->appendNode("tr");
			if($n == 0)
				$r->appendNode("td", "", "Name " . $n . ": Party Leader");
			else
				$r->appendNode("td", "", "Name " . $n);

			$r->appendTableRowInput("", "name$n", "view", $booking->{"name$n"}, "width: 300px;"); 
			$r->appendTableRowInput("", "sex$n", "view", $booking->{"sex$n"}); 
			$r->appendTableRowInput("", "dateOfBirth$n", "view", $booking->{"dateOfBirth$n"}); 
		}
		
		$f->appendNode("h3", "", "General");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("TC Membership Number or Other Cycling organisation name", "member", "view", $booking->member, "width:300px;"); 
		$t->appendTableInput("Postal Address", "address", "view", $booking->address, "height:100px; width:400px"); 
		$t->appendTableInput("Postcode", "postcode", "view", $booking->postcode, "width:300px;"); 
		$t->appendTableInput("Email Address", "email", "view", $booking->email, "width:300px;"); 
		$t->appendTableInput("Telephone (with code)", "telephone", "view", $booking->telephone, "width:300px;"); 
		$t->appendTableInput("Mobile Telephone (with code)", "mobileTelephone", "view", $booking->mobileTelephone, "width:300px;"); 
		$t->appendTableInput("Arrival Date", "arrivalDate", "view", $booking->arrivalDate, "width:300px;"); 
		$t->appendTableInput("Departure Date", "departureDate", "view", $booking->departureDate, "width:300px;"); 
		$t->appendTableInput("Accommodation", "accommodation", "view", $booking->accommodation, "", array("Camping", "Caravan", "MotorHome", "Off-site")); 
		$t->appendTableInput("Electricity", "electricity", "view", $booking->electricity, "width:50px;"); 

		$t->appendTableInput("Payment Currency", "currency", "view", $booking->currency, "", array("Pounds", "Euros")); 
		$t->appendTableInput("Happy to help with", "help", "view", $booking->help, "", $helpSelect); 
		$t->appendTableInput("Additional Information", "notes", "view", $booking->notes, "height:100px; width:400px;"); 

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$size = $this->tshirtSizes[$n];
			$t->appendTableInput("T-Shirts {$size} (Number of)", "tshirt{$n}", "view", $booking->{"tshirt$n"}, "width:80px;"); 
		}
		$t->appendTableInput("Coach Trip (Number of adults)", "coachTripAdult", "view", $booking->coachTripAdult, "width:50px;"); 
		$t->appendTableInput("Coach Trip (Number of children)", "coachTripChild", "view", $booking->coachTripChild, "width:50px;"); 

		// Prices
		$f->appendNode("h3", "", "Prices");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Unit");
		$r->appendNode("th", "", "Quantity");
		$r->appendNode("th", "", "Cost");
		$r->appendNode("th", "", "Total");
		
		for($n = 0; $n < count($this->prices); $n++){
			$price = $this->prices[$n];
			if(($n < 5) || ($n > 8) || $booking->{"q$n"}){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $price->item);
				$r->appendNode("td", "", $price->unit);
				$r->appendTableRowInput("", "q{$n}", "view", $booking->{"q$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "e{$n}", "view", $booking->{"e$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "t{$n}", "view", $booking->{"t$n"}, "background-color:yellow");
			}
		}

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendTableRowInput("", "currencyDisplay", "view", $booking->currency, "background-color:yellow;"); 
		$r->appendTableRowInput("", "total", "view", $booking->total, "background-color:yellow;"); 

		if($this->checkAccess()){
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Discount");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "discount", "view", $booking->discount, "background-color:yellow; width:100px;"); 

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Paid");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "paid", "view", $booking->paid, "background-color:yellow; width:100px;"); 
		}

		$f->appendNode("p", "", "The rally fee will not be refunded after the closing date. Refund of other components will depend on what the Club can recover from the campsite and/or other suppliers.");
		$f->appendNode("p", "", "Photographs taken at the event may be published on the Tandem Club Website and/or The Tandem Club Journal.");
		$f->appendNode("p", "", "If you participate in any ride or event organised by the Tandem Club, you do so entirely at your own risk. The club, its officers and ride organisers accept no responsibility for loss, damage to property, accident, injury, or death, that may occur, however caused, whether by negligence, deliberate act or otherwise.");

		return $f->render();
	}
	
	function bookingTotals($uri){
		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='get'");
		$f->appendNode("h3", "", "Event Totals");
		
		$bookings = 0;
		$adults = 0;
		$children = 0;
		$infants = 0;
		$campingAdults = 0;
		$campingChildren = 0;
		$campingUnits = 0;
		$cottage1 = 0;
		$cottage2 = 0;
		$cottage3 = 0;
		$cottage4 = 0;

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$tshirt{$n} = 0;
		}
		$electricity = 0;
		$coachTrip = 0;
		
		$due["Pounds"] = 0;
		$due["Euros"] = 0;
		$paid["Pounds"] = 0;
		$paid["Euros"] = 0;

		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		foreach($data as $obj){
			$camping = 0;
			
			switch($obj->accommodation){
			case "Camping":
			case "Caravan":
			case "MotorHome":
				$camping = 1;
				break;
			case "Cottage (Booked with campsite)":	$cottage1 += 1;		break;
			case "Cottage2":	$cottage2 += 1;		break;
			case "Cottage3":	$cottage3 += 1;		break;
			case "Cottage4":	$cottage4 += 1;		break;
			}

			for($n = 0; $n < 6; $n++){
				if($obj->{"name$n"} != ""){
					$age = $obj->{"age$n"};
					if($age >= $this->ageAdult){
						$adults++;
					}
					else if($age >= $this->ageChild){
						$children++;
					}
					else {
						$infants++;
					}
					if($camping){
						if($age >= $this->ageAdultCamping){
							$campingAdults++;
						}
						else if($age >= $this->ageChildCamping){
							$campingChildren++;
						}
					}
				}
			}
			
			if($camping){
				$campingUnits++;
			}
			
			for($n = 0; $n < count($this->tshirtSizes); $n++){
				$tshirt{$n} += $obj->{"tshirt$n"};
			}
			$electricity += $obj->electricity;
			$coachTrip += intval($obj->coachTripAdult) + intval($obj->coachTripChild);

			
			$due["{$obj->currency}"] += $obj->total - $obj->discount;
			$paid["{$obj->currency}"] += $obj->paid;
			
			$bookings++;
		}
		
		$people = $adults + $children + $infants;

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Value");

		$t->appendTableInput("Bookings", "", "view", $bookings, ""); 
		$t->appendTableInput("People", "", "view", $people, ""); 
		$t->appendTableInput("Adults", "", "view", $adults, ""); 
		$t->appendTableInput("Children", "", "view", $children, ""); 
		$t->appendTableInput("Infants", "", "view", $infants, ""); 
		$t->appendTableInput("CampingAdults", "", "view", $campingAdults, ""); 
		$t->appendTableInput("CampingChildren", "", "view", $campingChildren, ""); 
		$t->appendTableInput("CampingUnits", "", "view", $campingUnits, ""); 
		$t->appendTableInput("Cottage (Booked with campsite)", "", "view", $cottage1, ""); 
//		$t->appendTableInput("Cottage2", "", "view", $cottage2, ""); 
//		$t->appendTableInput("Cottage3", "", "view", $cottage3, ""); 
//		$t->appendTableInput("Cottage4", "", "view", $cottage4, ""); 

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$size = $this->tshirtSizes[$n];
			$t->appendTableInput("T-Shirts {$size}", "", "view", $tshirt{$n}, ""); 
		}
		$t->appendTableInput("Electricity", "", "view", $electricity, ""); 
		$t->appendTableInput("CoachTrip", "", "view", $coachTrip, ""); 


		$t->appendTableInput("Due Pounds", "", "view", $due["Pounds"], ""); 
		$t->appendTableInput("Paid Pounds", "", "view", $paid["Pounds"], ""); 
		$t->appendTableInput("Due Euros", "", "view", $due["Euros"], ""); 
		$t->appendTableInput("Paid Euros", "", "view", $paid["Euros"], ""); 

		
		return $f->render();
	}
	
	function findTotals(){
		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		$this->electricNumber = 0;
		$this->coachTripNumber = 0;
		$this->cottage1 = 0;
		$this->cottage2 = 0;
		$this->cottage3 = 0;
		$this->cottage4 = 0;
		foreach($data as $obj){
			switch($obj->accommodation){
			case "Cottage1":	$this->cottage1 += 1;	break;
			case "Cottage2":	$this->cottage2 += 1;	break;
			case "Cottage3":	$this->cottage3 += 1;	break;
			case "Cottage4":	$this->cottage4 += 1;	break;
			}
			if($obj->electricity)
				$this->electricNumber += 1;
			if($obj->coachTripAdult)
				$this->coachTripNumber += intval($obj->coachTripAdult);
			if($obj->coachTripChild)
				$this->coachTripNumber += intval($obj->coachTripChild);
		}
	}	

};
?>
