<?php
/*******************************************************************************
 *	EventBookingFormExtras.php	BookingForm Module
 *				T.Barnaby,	BEAM Ltd,	2013-04-26
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("EventBookingTcn2013Extra" => "BObjEventBookingExtra"));

class BObjEventBookingExtra extends BObj {
	function __construct(){
		$m = array();

		$m[] = new BMember("id", "Int", "Number");
		$m[] = new BMember("deleted", "Int", "Number");
		$m[] = new BMember("bookingId", "Int", "Number");
		$m[] = new BMember("name", "String");
		$m[] = new BMember("email", "String");
		$m[] = new BMember("currency", "String");
		$m[] = new BMember("extra1", "Int");
		$m[] = new BMember("extra2", "Int");

		for($n = 0; $n < 2; $n++){
			$m[] = new BMember("q{$n}", "Int");
			$m[] = new BMember("e{$n}", "Float");
			$m[] = new BMember("t{$n}", "Float");
		}
		$m[] = new BMember("total", "Float");
		$m[] = new BMember("paid", "Float");

		$m[] = new BMember("addedDate", "Date");
		$m[] = new BMember("addedUserName", "String");

		parent::__construct($m);
	}

	function dbTableFields(){
		$s = "(`id` int NOT NULL auto_increment,";

		$s .= "`deleted` int default 0,";
		$s .= "`bookingId` int,";
		$s .= "`name` text,";
		$s .= "`email` text,";
		$s .= "`currency` text,";
		$s .= "`extra1` int,";
		$s .= "`extra2` int,";

		for($n = 0; $n < 2; $n++){
			$s .= "`q{$n}` float,";
			$s .= "`e{$n}` float,";
			$s .= "`t{$n}` float,";
		}
		$s .= "`total` float,";
		$s .= "`paid` float,";

		$s .= "`addedDate` text,";
		$s .= "`addedUserName` text,";
		$s .= "PRIMARY KEY  (`id`))";

		return $s;
	}
};

class EventBookingFormExtra {
	// Basic Interface routines
	function __construct(&$sys, $event){
		$this->sys = $sys;
		$this->event = $event;
		$this->name = $this->event->name;
		$this->eventTitle = ucfirst($this->event->title);
		$this->eventStartDate = new BDate($this->event->startDate);
		$this->eventEndDate = new BDate($this->event->endDate);
		$this->bookingStartDate = new BDate($this->event->bookingStartDate);
		$this->bookingEndDate = new BDate($this->event->bookingEndDate);
		$this->extrasEndDate = new BDate($this->event->extrasEndDate);
		$this->bookingTable = "EventBooking" . ucfirst($this->name) . "Extra";
		
		$this->prices = array();
		$this->prices[] = new Price(0, "Skittle evening on Weds", "Per adult", 9.0, 0.0);
		$this->prices[] = new Price(0, "Rotisserie Chicken on Friday", "Per adult", 7.5, 0.0);
		for($n = 0; $n < count($this->prices); $n++){
			$this->prices[$n]->priceEuros = ceil($this->prices[$n]->pricePounds * 1.20);
		}
	}

	function header(){
		$startDate = $this->eventStartDate->getDateTime("%A %e %B %Y");
		$endDate = $this->eventEndDate->getDateTime("%A %e %B %Y");
		$bookingEndDate = $this->bookingEndDate->getDateTime("%A %e %B %Y");
		$text = <<< End_Of_Text
<h1 style="text-align: center;">{$this->eventTitle}</h1>
<h2 style="text-align: center;">{$startDate} - ${endDate}</h2>
<h1 style="text-align: center;">Booking Extras Form</h1>
<h4 style="text-align: center;">Bookings to be received by {$bookingEndDate}</h4>
<h3>Booking procedure</h3>
<ol>
<li>Please complete this form and then use the "Calculate" button at the end to calculate the payments.
<li>Check the payment values and if Ok use the "Book" button to book for the event. You will be sent
and email with the booking information and will be given a "BookingRef" number.</li>

<li>Please then pay in one of the following ways, (Please use the reference: "{$this->name}E {BookingRef}"
substituting "{BookingRef}" with the booking reference number you have been given), the account
number details and address to post cheques will be given after you book:<br>
<ul>
<li>For people in the UK you can pay in pounds sterling using direct bank transfer to the Tandem Clubs
account.</li>
<li>For people in the UK you can post a cheque in pounds sterling for total amount, with name of
Party Leader and BookingRef on the reverse, payable to: "The Tandem Club".</li>

<li>For International/European bookings please pay in pounds cash on arrival at the rally.</li>

<li>Note that the TAB key can be used to move to the next field</li>
<li>If you have any problems with the booking form or system please email:
<a href="mailto:{$this->name}@events.tandem-club.org.uk">{$this->name}@events.tandem-club.org.uk</a></li>
</ol>
End_Of_Text;
		return $text;
	}
	
	function checkAccess(){
		return $this->sys->login->checkLogin("admin");
	}
		
	function getPage($uri){
		if(0){
			printf("Uri: %s<br>\n", $uri);
			print("Get: "); print_r($_GET); print("<br>\n");
			print("Post: "); print_r($_POST); print("<br>\n");
			print("Post: "); print_r($_SERVER); print("<br>\n");
			if($_SERVER["REMOTE_ADDR"] == "82.152.34.172")
				print "Terry";
		}

		// Check if before booking start date
		$dateNow = new BDate("now");

		if(! $this->checkAccess()){
			if($dateNow->timeStamp < $this->bookingStartDate->timeStamp){
				$str = "<center><H2>Bookings for this event are not open yet</H2></center>";
				$str .= "<center><H2>They open on {$this->bookingStartDate->getDateTime("%A %e %B %Y")}</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
			if($dateNow->timeStamp > $this->bookingEndDate->timeStamp){
				$str = "<center><H2>Bookings for this event are now closed</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
		}

		if(isset($_POST["buttonBook"])){
			$obj = new BObjEventBookingExtra();
			$obj->set($_POST);
			
			$editing = 0;
			if($obj->id == 0){
				$obj->addedDate = strftime("%Y-%m-%d");
				$obj->addedUserName = $this->sys->user->name;
			}
			else {
				$editing = 1;
			}

//			BDatabase::$debug = 1;
			if($obj->id)
				$this->sys->db->putObject($this->bookingTable, $obj);
			else
				$obj = $this->sys->db->appendObject($this->bookingTable, $obj);

			$h = new BHtmlNode("h2", "style='text-align: center;'", "Thank you, {$obj->name}, your Extras Booking for {$this->eventTitle} has been accepted");
			$h->appendNode("h2", "style='text-align: center;'", "Your BookingRef is: {$obj->id}");
//			$h->appendNode("h2", "style='text-align: center;'", "An email has been sent confirming this booking, with a copy of your booking form that you can print");
			$h->appendNode("h2", "style='text-align: center;'", "We hope you enjoy the event - Happy Tandeming !");
			
			if($obj->paid == 0){
				$h->appendNode("h3", "", "Please pay in one of the following ways, (Please use the reference: \"{$this->name}E {$obj->id}\" )");
				$r = &$h->appendNode("ol");
				$r->appendNode("li", "", "For people in the UK you can pay in pounds sterling using direct bank transfer to the account: Bank: Barclays Bank, AccountName: The Tandem Club, Sortcode: 20-71-03, AccountNumber: 73018431, Reference: \"{$this->name}E {$obj->id}\"");
				$r->appendNode("li", "", "For people in the UK you can post a cheque in pounds sterling for total amount, with name of Party Leader and BookingRef on the reverse, payable to: \"The Tandem Club\" to: Linda Bailey, Tandem Club, 63 Silverberry Road, Worle, Weston-super-Mare, BS22 6RY.");
				$r->appendNode("li", "", "For International/European bookings please pay in pounds cash on arrival to the rally.");
			}
			$h->appendNode("p", "", "If you have any problems with the booking form or system please email: <a href='mailto:{$this->name}@events.tandem-club.org.uk'>{$this->name}@events.tandem-club.org.uk</a>");
			
			$page = $h->render();
			$page .= $this->view($obj);
			
			if($editing == 0){
				// Send an email
				$h = new BHtmlNode("html");
				$b = &$h->appendNode("head");
				$b = &$h->appendNode("body", "style='background-color: #CCDDDD;'");
				$b->appendNode("style", "type='text/css'", "table { empty-cells: show; }");
				$b->appendText($page);
				sendEmailFromTo("{$this->name}@events.tandem-club.org.uk", $obj->email, "{$this->sys->siteTitle}: Event booking: {$obj->id}", $h->render(), 1);
//				sendEmailFromTo("{$this->name}@events.tandem-club.org.uk", $obj->email, "{$this->sys->siteTitle}: Event booking: {$obj->id}", $page, 1);
			}
		}
		else {
			$page = "";
			$page .= $this->header();
			$page .= $this->form(0);
		}

		return $page;
	}
	
	function setDataFromPost(&$booking){
		// Sets the data from POST info
		foreach($booking as $k => $v){
			if(isset($_POST[$k])){
				$booking->$k = $_POST[$k];
			}
		}
	}

	function validate(&$booking, $editing, &$errors){
		$err = 0;
		
		if($booking->name == ""){
			$err = 1;
			$errors[] = "We need the party leaders name";
		}
		
		// Email
		if(!preg_match("/.+@.+/", $booking->email)){
			$err = 1;
			$errors[] = "No email or incorrect email format";
		}
		
		$extra1Total = 0;
		$extra1Total = 1;
		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");
		foreach($data as $obj){
			if($obj->extra1) $extra1Total += 1;
			if($obj->extra2) $extra2Total += 1;
		}

		if($extra1Total >= 50){
			$err = 1;
			$errors[] = "All extra1 have gone";
		}
		
		return $err;
	}

	function calculate(&$booking){
		for($n = 0; $n < count($this->prices); $n++){
			$booking->{"q$n"} = 0;
			if($booking->currency == "Pounds")
				$booking->{"e$n"} = $this->prices[$n]->pricePounds;
			else
				$booking->{"e$n"} = $this->prices[$n]->priceEuros;
		}

		$booking->q0 = intval($booking->extra1);
		$booking->q1 = intval($booking->extra2);

		$total = 0;
		for($n = 0; $n < count($this->prices); $n++){
			$booking->{"t$n"} = $booking->{"q$n"} * $booking->{"e$n"};
			$total += $booking->{"t$n"};
		}
		$total = round($total, 2);
		$booking->total = $total;
	}

	function view($booking){
		return $this->form($booking, 1);
	}
	
	function form($booking, $view = 0){
		if($booking == 0){
			$editing = 0;
			$booking = new BObjEventBookingExtra();
			$booking->currency = "Pounds";
		}
		else {
			$editing = 1;
		}
		
		if(isset($_POST["editing"]))
			$editing = $_POST["editing"];
		$this->setDataFromPost($booking);
		$this->calculate($booking);

		if($view){
			$f = new BHtmlNode("dev");
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='/events/{$this->name}/bookingFormExtra#prices' method='post'");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			$t->appendTableInput("Rally BookingRef", "bookingId", "view", $booking->bookingId, "width:200px;"); 
			$t->appendTableInput("Name", "name", "view", $booking->name, "width: 200px;");
			$t->appendTableInput("Email", "email", "view", $booking->email, "width: 200px;");
//			$t->appendTableInput("Payment Currency", "currency", "view", $booking->currency, "", array("Pounds", "Euros")); 
			$t->appendTableInput("Skittle evening on Weds (Number)", "extra1", "view", $booking->extra1, "width: 50px;");
			$t->appendTableInput("Rotisserie Chicken on Friday (Number)", "extra2", "view", $booking->extra2, "width: 50px;");

			// Prices
			$f->appendNode("a", "name=prices", "");
			$f->appendNode("h3", "", "Prices");
			$f->appendNode("i", "", "This section is automatically filled in from the above information. Please press the \"Calculate\" button to make sure this is updated.");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Item");
			$r->appendNode("th", "", "Unit");
			$r->appendNode("th", "", "Quantity");
			$r->appendNode("th", "", "Cost");
			$r->appendNode("th", "", "Total");

			for($n = 0; $n < count($this->prices); $n++){
				$price = $this->prices[$n];
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $price->item);
				$r->appendNode("td", "", $price->unit);
				$r->appendTableRowInput("", "q{$n}", "view", $booking->{"q$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "e{$n}", "view", $booking->{"e$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "t{$n}", "view", $booking->{"t$n"}, "background-color:yellow");
			}

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Total");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "currencyDisplay", "view", $booking->currency, "background-color:yellow;"); 
			$r->appendTableRowInput("", "total", "view", $booking->total, "background-color:yellow;"); 
		}
		else {
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='/events/{$this->name}/bookingFormExtra#prices' method='post'");
			$f->appendNode("input", "type='hidden' name='id' value='{$booking->id}'");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			$t->appendTableInput("Rally BookingRef (Leave blank if you cannot find this)", "bookingId", "text", $booking->bookingId, "width:200px;"); 
			$t->appendTableInput("Name", "name", "text", $booking->name, "width: 200px;");
			$t->appendTableInput("Email", "email", "text", $booking->email, "width: 200px;");
//			$t->appendTableInput("Payment Currency", "currency", "select", $booking->currency, "", array("Pounds", "Euros")); 
			$f->appendNode("input", "type='hidden' name='currency' value='{$booking->currency}'");
			$t->appendTableInput("Skittle evening on Weds (Number)", "extra1", "text", $booking->extra1, "width: 50px;");
			$t->appendTableInput("Rotisserie Chicken on Friday (Number)", "extra2", "text", $booking->extra2, "width: 50px;");

			// Prices
			$f->appendNode("a", "name=prices", "");
			$f->appendNode("h3", "", "Prices");
			$f->appendNode("i", "", "This section is automatically filled in from the above information. Please press the \"Calculate\" button to make sure this is updated.");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Item");
			$r->appendNode("th", "", "Unit");
			$r->appendNode("th", "", "Quantity");
			$r->appendNode("th", "", "Cost");
			$r->appendNode("th", "", "Total");

			for($n = 0; $n < count($this->prices); $n++){
				$price = $this->prices[$n];
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $price->item);
				$r->appendNode("td", "", $price->unit);
				$r->appendTableRowInput("", "q{$n}", "textView", $booking->{"q$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "e{$n}", "textView", $booking->{"e$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "t{$n}", "textView", $booking->{"t$n"}, "background-color:yellow");
			}

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Total");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "currencyDisplay", "view", $booking->currency, "background-color:yellow;"); 
			$r->appendTableRowInput("", "total", "textView", $booking->total, "background-color:yellow;"); 

			if($this->checkAccess()){
				$r = &$t->appendNode("tr");
				$r->appendNode("th", "align='left'", "Paid");
				$r->appendNode("td", "", "");
				$r->appendNode("td", "", "");
				$r->appendNode("td", "", "");
				$r->appendTableRowInput("", "paid", "text", $booking->paid, "background-color:yellow; width:100px;"); 
			}

			$f->appendNode("h3", "", "Book");
			$r = &$f->appendNode("ul");
			$r->appendNode("li", "", "First use the Calculate button to calculate the cost and validate the booking. Once this has been done please use the Book button.");
			$r->appendNode("li", "", "Please do not use the back button to edit and re-submit once sent. Contact the administrators to change the booking.");
			$r->appendNode("li", "", "Please see the Booking procedure info at the top of this form for details on how to pay.");
			$r->appendNode("li", "", "Photographs taken at the event may be published on the Tandem Club Website and/or The Tandem Club Journal.");

			$f->appendNode("p", "", "If you participate in any ride or event organised by the Tandem Club, you do so entirely at your own risk. The club, its officers and ride organisers accept no responsibility for loss, damage to property, accident, injury, or death, that may occur, however caused, whether by negligence, deliberate act or otherwise.");

			$err = 0;		
			if(isset($_POST["buttonCalculate"])){
				$errors = array();
				if($err = $this->validate($booking, $editing, $errors)){
					$f->appendNode("h3", "style='color: red'; font-size: 150%;", "There are errors in the form data");
					$r = &$f->appendNode("ul");
					foreach($errors as $e){
						$r->appendNode("li")->appendNode("span", "style='color: red; font-size: 150%;'", $e);
					}
					$f->appendText("Please correct these errors and click on the Calculate button again<br><br>");
				}
			}

			if(($err == 1) || !isset($_POST["buttonCalculate"]))
				$disable = "disabled='disabled'";
			else
				$disable = "";

			$f->appendNode("input", "type='submit' name='buttonCalculate' style='background-color: lightgreen; font-size: 200%;' value='Calculate'");
			$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
	//		$f->appendNode("button", "type='button' name='buttonPrint' $disable style='background-color: lightgreen; font-size: 200%;' onclick='window.print();'", "Print Form");
	//		$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
			if($editing)
				$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Save Changes' onclick='this.value=\"Saving, please wait...\";'");
			else
				$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Book' onclick='this.value=\"Booking, please wait...\";'");
		}
		return $f->render();
	}
};
?>
