<?php
/*******************************************************************************
 *	EventBookingFormExtras.php	BookingForm Module
 *				T.Barnaby,	BEAM Ltd,	2016-04-24
 *******************************************************************************
 *
 * Testing
 *	Make sure the Nochex account has test transcations enabled by logging in and setting it.
 *	The Payments class has a test variable that then uses this.
 *	Set the "test" variable of the Membership class to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for Full payment test accessing Nochex, but no actual payments being made.
 *	Set to 2 for Completely local testing (No Nochex interaction).
 *	Set to 3 for no payments at all.
 *
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("EventBookingTcn2016Extra" => "BObjEventBookingExtra"));

global $weekDays;
$weekDays = [ "Saturday", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday" ];

class BObjEventBookingExtra extends BObj {
	function __construct(){
		$m = array();

		$m[] = new BMember("id", "Int", "Number");
		$m[] = new BMember("deleted", "Int", "Number");
		$m[] = new BMember("bookingId", "Int", "Number");
		$m[] = new BMember("name", "String");
		$m[] = new BMember("address", "String");
		$m[] = new BMember("city", "String");
		$m[] = new BMember("postcode", "String");
		$m[] = new BMember("country", "String");
		$m[] = new BMember("email", "String");
		$m[] = new BMember("telephone", "String");

		for($n = 0; $n < 6; $n++){
			$name = "day_{$n}_breakfast";
			$m[] = new BMember($name, "Int");
			$name = "day_{$n}_meat";
			$m[] = new BMember($name, "Int");
			$name = "day_{$n}_veg";
			$m[] = new BMember($name, "Int");
		}
		$m[] = new BMember("day_5_lunch", "Int");
		$m[] = new BMember("day_6_breakfast", "Int");
		$m[] = new BMember("day_6_hogroast", "Int");
		$m[] = new BMember("day_7_breakfast", "Int");

		for($n = 0; $n < 5; $n++){
			$m[] = new BMember("q{$n}", "Int");
			$m[] = new BMember("e{$n}", "Float");
			$m[] = new BMember("t{$n}", "Float");
		}
		$m[] = new BMember("total", "Float", "", "", "decimal(8,2)");

		$m[] = new BMember("paymentType", "String");
		$m[] = new BMember("paymentId", "Int");
		$m[] = new BMember("paymentDate", "Date");
		$m[] = new BMember("paymentCurrency", "String");
		$m[] = new BMember("paymentAmount", "Float", "", "", "decimal(8,2)");
		$m[] = new BMember("paymentAmountPaid", "Float", "", "", "decimal(8,2)");

		$m[] = new BMember("modifiedDate", "Date");
		$m[] = new BMember("modifiedUserName", "String");

		parent::__construct($m);
	}
};

class EventBookingFormExtra {
	// Basic Interface routines
	function __construct(&$sys, $event){
		$this->sys = $sys;
		$this->event = $event;
		$this->name = $this->event->name;
		$this->eventTitle = ucfirst($this->event->title);
		$this->eventStartDate = new BDate($this->event->startDate);
		$this->eventEndDate = new BDate($this->event->endDate);
		$this->bookingStartDate = new BDate($this->event->bookingStartDate);
		$this->bookingEndDate = new BDate($this->event->bookingEndDate);
		$this->extrasEndDate = new BDate($this->event->extrasEndDate);
		$this->bookingTable = "EventBooking" . ucfirst($this->name) . "Extra";
		$this->payments = $sys->modules["payments"];

		$this->test = 0;
		
		$this->prices = array();
		$this->prices[] = new Price(0, "Breakfast", "Per meal", 5.0, 0.0);
		$this->prices[] = new Price(0, "Lunch", "Per meal", 4.50, 0.0);
		$this->prices[] = new Price(0, "Evening meat meals", "Per meal", 8, 0.0);
		$this->prices[] = new Price(0, "Evening vegetarian meals", "Per meal", 8, 0.0);
		$this->prices[] = new Price(0, "Hog roast (If not already booked for event)", "Per meal", 9.95, 0.0);

		for($n = 0; $n < count($this->prices); $n++){
			$this->prices[$n]->priceEuros = ceil($this->prices[$n]->pricePounds * 1.38);
		}
	}

	function header(){
		$startDate = $this->eventStartDate->getDateTime("%A %e %B %Y");
		$endDate = $this->eventEndDate->getDateTime("%A %e %B %Y");
		$bookingEndDate = $this->bookingEndDate->getDateTime("%A %e %B %Y");
		$text = <<< End_Of_Text
<h1 style="text-align: center;">{$this->eventTitle}</h1>
<h2 style="text-align: center;">{$startDate} - ${endDate}</h2>
<h1 style="text-align: center;">Booking Extras Form</h1>
<h4 style="text-align: center;">Bookings to be received by {$bookingEndDate}</h4>
<h3>Booking procedure</h3>
<ol>
<li>Please complete this form and then use the "Calculate" button at the end to calculate the payments.
<li>Check the payment values and if Ok use the "Book" button to book for the event. You will be sent
and email with the booking information and will be given a "BookingRef" number.</li>

<li>Please then pay in one of the following ways, (Please use the reference: "{$this->name}E {BookingRef}"
substituting "{BookingRef}" with the booking reference number you will have been given and you submit this form), the account
number details and address to post cheques will be given after you book:<br>
<ul>
<li>For card payments in GB Pounds click on the "Pay on Credit or Debit Card" button that is displayed after the "Book" button is pressed and you will be taken to a secure payments page.</li>
<li>For people in the UK you can pay in pounds sterling using direct bank transfer to the Tandem Clubs account.</li>
<li>For people in the UK you can post a cheque in pounds sterling for total amount, with name of
Party Leader and BookingRef on the reverse, payable to: "The Tandem Club".</li>

<li>For international bookings outside Europe pay by direct money transfer to the euros account given after booking, or in cash on arrival on the rally site.</li> 

<li>Note that the TAB key can be used to move to the next field</li>
<li>If you have any problems with the booking form or system please email:
<a href="mailto:{$this->name}@events.tandem-club.org.uk">{$this->name}@events.tandem-club.org.uk</a></li>
</ol>
End_Of_Text;
		return $text;
	}
	
	function checkAccess(){
		return $this->sys->login->checkLogin("admin,adminEvents");
	}
		
	function getPage($uri){
		if(0){
			printf("Uri: %s<br>\n", $uri);
			print("Get: "); print_r($_GET); print("<br>\n");
			print("Post: "); print_r($_POST); print("<br>\n");
			print("Post: "); print_r($_SERVER); print("<br>\n");
			if($_SERVER["REMOTE_ADDR"] == "82.152.34.172")
				print "Terry";
		}
		
		$page = "";
		if($this->test){
			$page .= "<h1><font color='red'>System is in test mode: {$this->test} Do not use</font></h1>\n";
		}

		// Check if before booking start date
		$dateNow = new BDate("now");

		if(! $this->checkAccess()){
			if($dateNow->timeStamp < $this->bookingStartDate->timeStamp){
				$str = "<center><H2>Bookings for this event are not open yet</H2></center>";
				$str .= "<center><H2>They open on {$this->bookingStartDate->getDateTime("%A %e %B %Y")}</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
			if($dateNow->timeStamp > $this->bookingEndDate->timeStamp){
				$str = "<center><H2>Bookings for this event are now closed</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
		}

		if(isset($_POST["buttonBook"])){
			$obj = new BObjEventBookingExtra();
			$obj->set($_POST);
			
			$editing = 0;
			if($obj->id == 0){
				$obj->modifiedDate = strftime("%Y-%m-%d");
				$obj->modifiedUserName = $this->sys->user->name;
			}
			else {
				$editing = 1;
			}

//			BDatabase::$debug = 1;
			if($obj->id)
				$this->sys->db->putObject($this->bookingTable, $obj);
			else
				$obj = $this->sys->db->appendObject($this->bookingTable, $obj);

			$bookingRef = $this->name . "Extra " . $obj->id;

			$h = new BHtmlNode("h2", "style='text-align: center;'", "Thank you, {$obj->name}, your Extras Booking for {$this->eventTitle} has been accepted");
			$h->appendNode("h2", "style='text-align: center;'", "Your BookingRef is: {$bookingRef}");
//			$h->appendNode("h2", "style='text-align: center;'", "An email has been sent confirming this booking, with a copy of your booking form that you can print");
			$h->appendNode("h2", "style='text-align: center;'", "We hope you enjoy the event - Happy Tandeming !");
			$h->appendNode("p", "", "If you have any problems with the booking form or system please email: <a href='mailto:{$this->name}@events.tandem-club.org.uk'>{$this->name}@events.tandem-club.org.uk</a>");
			$hEmail = clone $h;

			if(!$editing && ($obj->paymentAmountPaid == 0)){
				$payment = $this->payments->payGetNew();
				$payment->type = $obj->paymentType;
				$payment->source = "events";
				$payment->reference = $bookingRef;
				$payment->name = $obj->name;
				$payment->address = $obj->address;
				$payment->city = $obj->city;
				$payment->postcode = $obj->postcode;
				$payment->country = $obj->country;
				$payment->telephone = $obj->telephone;
				$payment->email = $obj->email;
				$payment->amount = $obj->total;
				$payment->currency = $obj->paymentCurrency;
				$payment->title = $this->eventTitle . "Extra";
				$payment->test = $this->test;

				if($this->test == 3){
					$h->appendNode("p", "", "<b>*** Warning: Payments disabled as in test mode 3 ***</b>");
				}
				else {
					$this->payments->test = $this->test;
					if($obj->paymentType == "card"){
						$payment->type = "card_event";
						$h->appendText($this->payments->payDo($payment));
					}
					else {
						if($obj->paymentType == "bacs"){
							if($obj->paymentCurrency == "Euros")
								$payment->type = "bacs_euros";
							else
								$payment->type = "bacs_pounds";
						}
						$n = new BHtmlNode("div");
						$n->appendNode("h3", "", "Please pay in one of the following ways, (Please use the reference: '{$this->name} {$obj->id}' )");
						$r = &$n->appendNode("ol");
						$r->appendNode("li", "", "For people in the UK you can pay in pounds sterling using direct bank transfer to the account: Bank: Barclays Bank, AccountName: The Tandem Club, Sortcode: 20-71-03, AccountNumber: 73018431, Reference: '{$this->name} {$obj->id}'");
						$r->appendNode("li", "", "For people in the UK you can post a cheque in pounds sterling for total amount, with name of Party Leader and '{$this->name} {$obj->id}' on the reverse, payable to: \"The Tandem Club\" to: {$this->chequeAddress}.");
						$r->appendNode("li", "", "For European bookings pay in euros by direct bank transfer to the following bank account: F.E.R. Oudhof, IBAN: NL58INGB0006385650, BIC: INGBNL2A use the reference '{$this->name} {$obj->id}'. For more information contact: Eelco Oudhof, Lichtboei 26 NL-1186 DE Amstelveen, The Netherlands, email: administratie@tandemclub.nl.");
						$r->appendNode("li", "", "International bookings outside Europe pay by direct money transfer to the euros account given above, or in Euro's cash on arrival on the rally site.");

						$h->appendText($n->render());
						$hEmail->appendText($n->render());
						$this->payments->payDo($payment);
					}
				}
			}

			$page .= $h->render();
			$page .= $this->view($obj);
			$pageEmail = $hEmail->render();
			$pageEmail .= $this->view($obj, 1);

			if($editing == 0){
				// Send an email
				$h = new BHtmlNode("html");
				$b = &$h->appendNode("head");
				$b = &$h->appendNode("body", "style='background-color: #CCDDDD;'");
				$b->appendNode("style", "type='text/css'", "table { empty-cells: show; }");
				$b->appendText($pageEmail);
				sendEmailFromTo("{$this->name}@tandem-club.org.uk", $obj->email, "{$this->sys->siteTitle}: Event Extras booking: {$obj->id}", $h->render(), 1);
			}
		}
		else {
			$page .= $this->header();
			$page .= $this->form(0);
		}

		return $page;
	}
	
	function setDataFromPost(&$booking){
		// Sets the data from POST info
		foreach($booking as $k => $v){
			if(isset($_POST[$k])){
				$booking->$k = $_POST[$k];
			}
		}
	}

	function validate(&$booking, $editing, &$errors){
		$err = 0;
		
		if($booking->name == ""){
			$err = 1;
			$errors[] = "We need the party leaders name";
		}
		
		// Email
		if(!filter_var($booking->email, FILTER_VALIDATE_EMAIL)){
			$err = 1;
			$errors[] = "No email or incorrect email format";
		}

		if(($booking->paymentType == "cheque") && ($booking->paymentCurrency != "Pounds")){
			$err = 1;
			$errors[] = "Sorry, only able to accept cheques in Pounds";
		}
		
		if(($booking->paymentType == "card") && ($booking->paymentCurrency != "Pounds")){
			$err = 1;
			$errors[] = "Card payments can only be in Pounds.";
		}

		return $err;
	}

	function calculate(&$booking){
		for($n = 0; $n < count($this->prices); $n++){
			$booking->{"q$n"} = 0;
			if($booking->paymentCurrency == "Pounds")
				$booking->{"e$n"} = $this->prices[$n]->pricePounds;
			else
				$booking->{"e$n"} = $this->prices[$n]->priceEuros;
		}

		// Caculate totals
		for($n = 0; $n < 6; $n++){
			$booking->q0 += $booking->{"day_{$n}_breakfast"};
			$booking->q2 += $booking->{"day_{$n}_meat"};
			$booking->q3 += $booking->{"day_{$n}_veg"};
		}
		
		$booking->q1 += $booking->day_5_lunch;
		$booking->q0 += $booking->day_6_breakfast;
		$booking->q4 += $booking->day_6_hogroast;
		$booking->q0 += $booking->day_7_breakfast;

		$total = 0;
		for($n = 0; $n < count($this->prices); $n++){
			$booking->{"t$n"} = $booking->{"q$n"} * $booking->{"e$n"};
			$total += $booking->{"t$n"};
		}
		$total = round($total, 2);
		$booking->total = $total;
		$booking->paymentAmount = $total;
	}

	function view($booking){
		return $this->form($booking, 1);
	}
	
	function form($booking, $view = 0){
		global $weekDays;
		
		if($booking == 0){
			$editing = 0;
			$booking = new BObjEventBookingExtra();
			$booking->paymentCurrency = "Pounds";
			$booking->country = "United Kingdom";
			$booking->paymentType = "card";
			
			if($this->sys->user){
				$a = explode(" ", $this->sys->user->name);
				if(count($a) == 2){
					$fn = $a[0];
					$sn = $a[1];
				}
				$booking->name = $this->sys->user->name;
				$booking->address = $this->sys->user->postalAddress;
				$booking->city = $this->sys->user->city;
				$booking->postcode = $this->sys->user->postcode;
				$booking->country = $this->sys->user->country;
				$booking->email = $this->sys->user->emailAddress;
				$booking->telephone = $this->sys->user->telephoneHome;
			}

		}
		else {
			$editing = 1;
		}
		
		if(isset($_POST["editing"]))
			$editing = $_POST["editing"];
		$this->setDataFromPost($booking);
		$this->calculate($booking);

		if($view){
			$f = new BHtmlNode("dev");
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='/events/{$this->name}/bookingFormExtra#prices' method='post'");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			$t->appendTableInput("Rally BookingRef", "bookingId", "view", $booking->bookingId, "width:200px;"); 
			$t->appendTableInput("Name", "name", "view", $booking->name, "width: 200px;");
			$t->appendTableInput("Postal Address (first lines)", "address", "view", $booking->address, "height:50px; width:400px"); 
			$t->appendTableInput("City or Town", "city", "view", $booking->city, "width:300px;"); 
			$t->appendTableInput("Postcode", "postcode", "view", $booking->postcode, "width:300px;"); 
			$t->appendTableInput("Country", "country", "view", $booking->country); 
			$t->appendTableInput("Email", "email", "view", $booking->email, "width: 200px;");
			$t->appendTableInput("Home Telephone (with code)", "telephone", "view", $booking->telephone, "width:300px;"); 

			$t->appendTableInput("Payment Type", "paymentType", "view", $booking->paymentType); 
			$t->appendTableInput("Payment Currency", "paymentCurrency", "view", $booking->paymentCurrency); 

			// Meals
			$f->appendNode("h3", "", "Meals");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Day");
			$r->appendNode("th", "", "Breakfast");
			$r->appendNode("th", "", "Lunch");
			$r->appendNode("th", "", "Evening meat");
			$r->appendNode("th", "", "Evening vegetarian");
			for($n = 0; $n < 6; $n++){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $weekDays[$n]);
				$r->appendTableRowInput("", "day_{$n}_breakfast", "view", $booking->{"day_{$n}_breakfast"}); 
				if($n == 5)
					$r->appendTableRowInput("", "day_{$n}_lunch", "view", $booking->{"day_{$n}_lunch"}); 
				else
					$r->appendNode("td");
				$r->appendTableRowInput("", "day_{$n}_meat", "view", $booking->{"day_{$n}_meat"}); 
				$r->appendTableRowInput("", "day_{$n}_veg", "view", $booking->{"day_{$n}_veg"}); 

			}
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $weekDays[$n % 7]);
			$r->appendTableRowInput("", "day_{$n}_breakfast", "view", $booking->{"day_{$n}_breakfast"}); 
			$r->appendNode("td");
			$r->appendTableRowInput("", "day_{$n}_hogroast", "view", $booking->{"day_{$n}_hogroast"}); 
			$r->appendNode("td");

			$n += 1;
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $weekDays[$n % 7]);
			$r->appendTableRowInput("", "day_{$n}_breakfast", "view", $booking->{"day_{$n}_breakfast"}); 

			
			// Prices
			$f->appendNode("a", "name=prices", "");
			$f->appendNode("h3", "", "Prices");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Item");
			$r->appendNode("th", "", "Unit");
			$r->appendNode("th", "", "Quantity");
			$r->appendNode("th", "", "Cost");
			$r->appendNode("th", "", "Total");

			for($n = 0; $n < count($this->prices); $n++){
				$price = $this->prices[$n];
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $price->item);
				$r->appendNode("td", "", $price->unit);
				$r->appendTableRowInput("", "q{$n}", "view", $booking->{"q$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "e{$n}", "view", $booking->{"e$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "t{$n}", "view", $booking->{"t$n"}, "background-color:yellow");
			}

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Total");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "currencyDisplay", "view", $booking->paymentCurrency, "background-color:yellow;"); 
			$r->appendTableRowInput("", "total", "view", $booking->total, "background-color:yellow;"); 
		}
		else {
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='/events/{$this->name}/bookingFormExtra' method='post'");
			$f->appendNode("input", "type='hidden' name='id' value='{$booking->id}'");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			$t->appendTableInput("Rally BookingRef (Leave blank if you cannot find this)", "bookingId", "text", $booking->bookingId, "width:200px;"); 
			$t->appendTableInput("Name", "name", "text", $booking->name, "width: 200px;");
			$t->appendTableInput("Postal Address (first lines)", "address", "textarea", $booking->address, "height:50px; width:400px"); 
			$t->appendTableInput("City or Town", "city", "text", $booking->city, "width:300px;"); 
			$t->appendTableInput("Postcode", "postcode", "text", $booking->postcode, "width:300px;"); 
			$t->appendTableInput("Country", "country", "select", $booking->country, "", $this->countryList(), "calc();"); 
			$t->appendTableInput("Email", "email", "text", $booking->email, "width: 200px;");
			$t->appendTableInput("Home Telephone (with code)", "telephone", "text", $booking->telephone, "width:300px;"); 

			$t->appendTableInput("Payment Type", "paymentType", "select", $booking->paymentType, "", array("card", "bacs", "cheque", "standingOrder")); 
			$t->appendTableInput("Payment Currency", "paymentCurrency", "select", $booking->paymentCurrency, "", array("Pounds", "Euros")); 

			// Meals
			$f->appendNode("h3", "", "Meals");
			$f->appendNode("i", "", "Please enter the number of meals required. See the events information as to what meals are provided.");
			$f->appendNode("br");
			$f->appendNode("i", "", "Note the evening meal on Friday is the Hogroast. You may have already booked this with the main event booking..");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Day");
			$r->appendNode("th", "", "Breakfast");
			$r->appendNode("th", "", "Lunch");
			$r->appendNode("th", "", "Evening meat");
			$r->appendNode("th", "", "Evening vegetarian");
			for($n = 0; $n < 6; $n++){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $weekDays[$n]);
				$r->appendTableRowInput("", "day_{$n}_breakfast", "text", $booking->{"day_{$n}_breakfast"}); 
				if($n == 5)
					$r->appendTableRowInput("", "day_{$n}_lunch", "text", $booking->{"day_{$n}_lunch"}); 
				else
					$r->appendNode("td");
				$r->appendTableRowInput("", "day_{$n}_meat", "text", $booking->{"day_{$n}_meat"}); 
				$r->appendTableRowInput("", "day_{$n}_veg", "text", $booking->{"day_{$n}_veg"}); 

			}
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $weekDays[$n % 7]);
			$r->appendTableRowInput("", "day_{$n}_breakfast", "text", $booking->{"day_{$n}_breakfast"}); 
			$r->appendNode("td");
			$r->appendTableRowInput("", "day_{$n}_hogroast", "text", $booking->{"day_{$n}_hogroast"}); 
			$r->appendNode("td");

			$n += 1;
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $weekDays[$n % 7]);
			$r->appendTableRowInput("", "day_{$n}_breakfast", "text", $booking->{"day_{$n}_breakfast"}); 

			// Prices
			$f->appendNode("a", "name=prices", "");
			$f->appendNode("h3", "", "Prices");
			$f->appendNode("i", "", "This section is automatically filled in from the above information. Please press the \"Calculate\" button to make sure this is updated.");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Item");
			$r->appendNode("th", "", "Unit");
			$r->appendNode("th", "", "Quantity");
			$r->appendNode("th", "", "Cost");
			$r->appendNode("th", "", "Total");

			for($n = 0; $n < count($this->prices); $n++){
				$price = $this->prices[$n];
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $price->item);
				$r->appendNode("td", "", $price->unit);
				$r->appendTableRowInput("", "q{$n}", "textView", $booking->{"q$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "e{$n}", "textView", $booking->{"e$n"}, "background-color:yellow"); 
				$r->appendTableRowInput("", "t{$n}", "textView", $booking->{"t$n"}, "background-color:yellow");
			}

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Total");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "currencyDisplay", "view", $booking->paymentCurrency, "background-color:yellow;"); 
			$r->appendTableRowInput("", "total", "textView", $booking->paymentAmount, "background-color:yellow;"); 

			if(0 && $this->checkAccess()){
				$r = &$t->appendNode("tr");
				$r->appendNode("th", "align='left'", "Paid");
				$r->appendNode("td", "", "");
				$r->appendNode("td", "", "");
				$r->appendNode("td", "", "");
				$r->appendTableRowInput("", "paid", "text", $booking->paid, "background-color:yellow; width:100px;"); 
			}

			$f->appendNode("h3", "", "Admin");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

			if($this->checkAccess()){
				$t->appendTableInput("PaymentId", "paymentId", "view", $booking->paymentId, "width:300px;"); 
				$t->appendNode("input", "type='hidden' name='paymentId' value='{$booking->paymentId}'");
				$t->appendTableInput("PaymentDate", "paymentDate", "view", $booking->paymentDate, "width:300px;"); 
				$t->appendNode("input", "type='hidden' name='paymentDate' value='{$booking->paymentDate}'");
				$t->appendTableInput("PaymentAmount", "paymentAmount", "view", $booking->paymentAmount, "width:300px;"); 
				$t->appendNode("input", "type='hidden' name='paymentAmount' value='{$booking->paymentAmount}'");
				$t->appendTableInput("PaymentAmountPaid", "paymentAmountPaid", "view", $booking->paymentAmountPaid, "width:300px;"); 
				$t->appendNode("input", "type='hidden' name='paymentAmountPaid' value='{$booking->paymentAmountPaid}'");
			}
			else {
				$t->appendNode("input", "type='hidden' name='paymentId' value='{$booking->paymentId}'");
				$t->appendNode("input", "type='hidden' name='paymentDate' value='{$booking->paymentDate}'");
				$t->appendNode("input", "type='hidden' name='paymentAmount' value='{$booking->paymentAmount}'");
				$t->appendNode("input", "type='hidden' name='paymentAmountPaid' value='{$booking->paymentAmountPaid}'");
			}
			$t->appendNode("input", "type='hidden' name='total' value='{$booking->total}'");

			$f->appendNode("h3", "", "Book");
			$r = &$f->appendNode("ul");
			$r->appendNode("li", "", "First use the Calculate button to calculate the cost and validate the booking. Once this has been done please use the Book button.");
			$r->appendNode("li", "", "Please do not use the back button to edit and re-submit once sent. Contact the administrators to change the booking.");
			$r->appendNode("li", "", "Please see the Booking procedure info at the top of this form for details on how to pay.");
			$r->appendNode("li", "", "Photographs taken at the event may be published on the Tandem Club Website and/or The Tandem Club Journal.");

			$f->appendNode("p", "", "If you participate in any ride or event organised by the Tandem Club, you do so entirely at your own risk. The club, its officers and ride organisers accept no responsibility for loss, damage to property, accident, injury, or death, that may occur, however caused, whether by negligence, deliberate act or otherwise.");

			$err = 0;		
			if(isset($_POST["buttonCalculate"])){
				$errors = array();
				if($err = $this->validate($booking, $editing, $errors)){
					$f->appendNode("h3", "style='color: red'; font-size: 150%;", "There are errors in the form data");
					$r = &$f->appendNode("ul");
					foreach($errors as $e){
						$r->appendNode("li")->appendNode("span", "style='color: red; font-size: 150%;'", $e);
					}
					$f->appendText("Please correct these errors and click on the Calculate button again<br><br>");
				}
			}

			if(($err == 1) || !isset($_POST["buttonCalculate"]))
				$disable = "disabled='disabled'";
			else
				$disable = "";

			$f->appendNode("input", "type='submit' name='buttonCalculate' style='background-color: lightgreen; font-size: 200%;' value='Calculate'");
			$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
	//		$f->appendNode("button", "type='button' name='buttonPrint' $disable style='background-color: lightgreen; font-size: 200%;' onclick='window.print();'", "Print Form");
	//		$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
			if($editing)
				$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Save Changes' onclick='this.value=\"Saving, please wait...\";'");
			else
				$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Book' onclick='this.value=\"Booking, please wait...\";'");
		}
		return $f->render();
	}
	
	function countryList(){
		$l = array("");
		if($maps = $this->sys->modules["maps"]){
			$l = array_merge($l, $maps->getCountryList());
		}
		return $l;
	}

};
?>
