<?php
/*******************************************************************************
 *	EventBookingForm.php	BookingForm Module
 *				T.Barnaby,	BEAM Ltd,	2018-01-19
 *******************************************************************************
 *
 * Testing
 *	Make sure the Nochex account has test transcations enabled by logging in and setting it.
 *	The Payments class has a test variable that then uses this.
 *	Set the "test" variable of the Membership class to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for Full payment test accessing Nochex, but no actual payments being made.
 *	Set to 2 for Completely local testing (No Nochex interaction).
 *	Set to 3 for no payments at all.
 *
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("EventBookingTcn2018" => "BObjEventBooking"));

// Overall booking options
// If event has TShirts
global	$tshirtSizes, $tshirtSizesChild;
//$tshirtSizes = array("S", "M", "L", "XL", "XXL");
//$tshirtSizesChild = array("24", "26", "28", "30", "32", "34");
$tshirtSizes = array();
$tshirtSizesChild = array();

function addressGetCity($address){
	$l = explode("\n", $address);
	$n = count($l);
	if($n)
		return $l[$n - 1];
	else
		return "";
}

class BObjEventBooking extends BObj {
	function __construct(){
		global $tshirtSizes, $tshirtSizesChild;

		$m = array();

		$m[] = new BMember("id", "Int", "Number");
		$m[] = new BMember("deleted", "Int", "Number");

		for($n = 0; $n < 6; $n++){
			$m[] = new BMember("forename{$n}", "String");
			$m[] = new BMember("surname{$n}", "String");
			$m[] = new BMember("name{$n}", "String");
			$m[] = new BMember("sex{$n}", "Int");
			$m[] = new BMember("dateOfBirth{$n}", "Date", "date", array('format' => '%Y-%n-%j'));
			$m[] = new BMember("age{$n}", "Int");
		}
		
		$m[] = new BMember("member", "String");
		$m[] = new BMember("address", "String");
		$m[] = new BMember("city", "String");
		$m[] = new BMember("postcode", "String");
		$m[] = new BMember("country", "String");
		$m[] = new BMember("email", "String");
		$m[] = new BMember("telephone", "String");
		$m[] = new BMember("mobileTelephone", "String");
		$m[] = new BMember("vehicleReg", "String");
		$m[] = new BMember("arrivalDate", "Date");
		$m[] = new BMember("departureDate", "Date");
		$m[] = new BMember("accommodation", "String");
		$m[] = new BMember("paymentType", "String");
		$m[] = new BMember("currency", "String");
		$m[] = new BMember("help", "String");
		$m[] = new BMember("notes", "String");
		$m[] = new BMember("adminNotes", "String");

		$m[] = new BMember("adults", "Int");
		$m[] = new BMember("children", "Int");
		$m[] = new BMember("infants", "Int");
		$m[] = new BMember("campingAdults", "Int");
		$m[] = new BMember("campingChildren", "Int");

		for($n = 0; $n < count($tshirtSizes); $n++){
			$m[] = new BMember("tshirt{$n}", "Int");
		}
		for($n = 0; $n < count($tshirtSizesChild); $n++){
			$m[] = new BMember("tshirtChild{$n}", "Int");
		}

		// Extra bits
		$m[] = new BMember("electricity", "Int");
//		$m[] = new BMember("dogs", "Int");
//		$m[] = new BMember("hogRoast", "Int");
//		$m[] = new BMember("trip1Adult", "Int");
//		$m[] = new BMember("trip1Concession", "Int");
//		$m[] = new BMember("trip1Child", "Int");
//		$m[] = new BMember("trip1Infant", "Int");
//		$m[] = new BMember("wineTasting", "Int");

//		$m[] = new BMember("length", "Float");
//		$m[] = new BMember("lengthWithoutTowbar", "Float");
//		$m[] = new BMember("width", "Float");

		// Totals
		for($n = 0; $n < 12; $n++){
			$m[] = new BMember("q{$n}", "Int");
			$m[] = new BMember("e{$n}", "Float");
			$m[] = new BMember("t{$n}", "Float");
		}
		$m[] = new BMember("total", "Float");
		$m[] = new BMember("adjustTitle", "String");
		$m[] = new BMember("adjustTotal", "Float");
		$m[] = new BMember("paymentId", "Int");
		$m[] = new BMember("paid", "Float");


		$m[] = new BMember("addedDate", "Date");
		$m[] = new BMember("addedUserName", "String");
		$m[] = new BMember("gdprAcceptance", "Bool");

		parent::__construct($m);
	}
};

class Price {
	function __construct($number, $item, $unit, $pricePounds, $priceEuros){
		$this->number = $number;
		$this->item = $item;
		$this->unit = $unit;
		$this->pricePounds = $pricePounds;
		$this->priceEuros = $priceEuros;
	}
};

class EventBookingForm {
	// Basic Interface routines
	function __construct(&$sys, $event){
		global $tshirtSizes, $tshirtSizesChild;

		$this->payments = $sys->modules["payments"];

		$this->tshirtSizes = $tshirtSizes;
		$this->tshirtSizesChild = $tshirtSizesChild;
		$this->sys = $sys;
		$this->event = $event;
		$this->name = $this->event->name;
		$this->eventTitle = ucfirst($this->event->title);
		$this->eventStartDate = new BDate($this->event->startDate);
		$this->eventEndDate = new BDate($this->event->endDate);
		$this->accommodationStartDate = new BDate($this->event->startDate);
		//$this->accommodationStartDate->timeStamp -= 5 * 24 * 3600;
		$this->accommodationEndDate = new BDate($this->event->endDate);
		//$this->accommodationEndDate->timeStamp += 6 * 24 * 3600;
		$this->bookingStartDate = new BDate($this->event->bookingStartDate);
		$this->bookingEndDate = new BDate($this->event->bookingEndDate);
		$this->extrasEndDate = new BDate($this->event->extrasEndDate);
		$this->bookingTable = "EventBooking" . ucfirst($this->name);
		$this->ageAdult = 19;
		$this->ageChild = 0;
		$this->ageAdultCamping = 16;
		$this->ageChildCamping = 3;
		$this->ageInfantCamping = 0;
		$this->ageChildCampingMax = $this->ageAdultCamping - 1;
		$this->chequeAddress = "Tony Prichard, 1 Poltondale, Swindon, Wilts, SN3 5BN, United Kingdom";
		$this->test = 0;
		
		$this->prices = array();
		$this->prices[] = new Price(0, "Rally fee adult", "Per adult ({$this->ageAdult}+)", 10.0, 0.0);
		$this->prices[] = new Price(1, "Rally fee child", "Per child ({$this->ageChild}+)", 5.0, 0.0);
		$this->prices[] = new Price(2, "Campsite pitch", "Per week", 50.0, 0.0);
		$this->prices[] = new Price(3, "Campsite pitch", "Per day", 8.0, 0.0);
		$this->prices[] = new Price(4, "Electricity", "Per week", 20.0, 0.0);

		// Extras
		if(count($this->tshirtSizes)){
			$this->prices[] = new Price(3, "T-Shirts Adult", "Each", 9.0, 10.0);
		}
		if(count($this->tshirtSizesChild)){
			$this->prices[] = new Price(4, "T-Shirts Child", "Each", 8.0, 9.0);
		}
		
		for($n = 0; $n < count($this->prices); $n++){
			$this->prices[$n]->priceEuros = ceil($this->prices[$n]->pricePounds * 1.2);
//			$this->prices[$n]->pricePounds = ceil($this->prices[$n]->priceEuros * 0.9);
		}
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin,adminEvents");
	}
		
	function getPage($uri){
		$page = "";
		if($this->test){
			$page .= "<h1><font color='red'>System is in test mode: {$this->test}: No payments will be taken</font></h1>\n";
		}

		if(0){
			printf("Uri: %s<br>\n", $uri);
			print("Get: "); print_r($_GET); print("<br>\n");
			print("Post: "); print_r($_POST); print("<br>\n");
			print("Post: "); print_r($_SERVER); print("<br>\n");
			if($_SERVER["REMOTE_ADDR"] == "82.152.34.172")
				print "Terry";
		}
		
		// Check if before booking start date
		$dateNow = new BDate("now");

		if(! $this->checkAccess()){
			if($dateNow->timeStamp < $this->bookingStartDate->timeStamp){
				$str = "<center><H2>Bookings for this event are not open yet</H2></center>";
				$str .= "<center><H2>They open on {$this->bookingStartDate->getDateTime("%A %e %B %Y")}</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
			if($dateNow->timeStamp > $this->bookingEndDate->timeStamp){
				$str = "<center><H2>Bookings for this event are now closed</H2></center>";
				$str .= "<center><H2>Sorry for any inconvenience</H2></center>";
				return $str;
			}
		}

		if(isset($_POST["buttonBook"])){
			$obj = new BObjEventBooking();
			$obj->set($_POST);
			
			$editing = 0;
			if($obj->id == 0){
				$obj->addedDate = strftime("%Y-%m-%d");
				$obj->addedUserName = $this->sys->user->name;
			}
			else {
				$editing = 1;
			}

/*			
			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
*/
//			BDatabase::$debug = 1;
			if($obj->id)
				$this->sys->db->putObject($this->bookingTable, $obj);
			else
				$obj = $this->sys->db->appendObject($this->bookingTable, $obj);

			$bookingRef = $this->name . " " . $obj->id;

			$h = new BHtmlNode("h2", "style='text-align: center;'", "Thank you, {$obj->name0}, your Booking for {$this->eventTitle} has been accepted");
			$h->appendNode("h2", "style='text-align: center;'", "Your BookingRef is: '{$bookingRef}'");
			$h->appendNode("h2", "style='text-align: center;'", "An email has been sent confirming this booking, with a copy of your booking form that you can print");
			$h->appendNode("h2", "style='text-align: center;'", "We hope you enjoy the event - Happy Tandeming !");
			$h->appendNode("p", "", "If you have any problems with the booking form or system please email: <a href='mailto:{$this->name}@tandem-club.org.uk'>{$this->name}@tandem-club.org.uk</a>");
			$hEmail = clone $h;
			
			if(!$editing && ($obj->paid == 0)){
				$payment = $this->payments->payGetNew();
				$payment->type = $obj->paymentType;
				$payment->source = "events";
				$payment->reference = $bookingRef;
				$payment->name = $obj->name0;
				$payment->address = $obj->address;
				$payment->city = $obj->city;
				$payment->postcode = $obj->postcode;
				$payment->country = $obj->country;
				$payment->telephone = $obj->telephone;
				$payment->emailAddress = $obj->email;
				$payment->amount = $obj->total;
				$payment->currency = $obj->currency;
				$payment->title = $this->eventTitle;
				$payment->test = $this->test;

				if($this->test == 3){
					$h->appendNode("p", "", "<b>*** Warning: Payments disabled as in test mode 3 ***</b>");
				}
				else {
					$this->payments->test = $this->test;
					if($obj->paymentType == "card"){
						$payment->type = "card_event";
						$h->appendText($this->payments->payDo($payment));
					}
					else {
						if($obj->paymentType == "bacs"){
							if($obj->currency == "Euros")
								$payment->type = "bacs_euros";
							else
								$payment->type = "bacs_pounds";
						}
						$n = new BHtmlNode("div");
						$n->appendNode("h3", "", "Please pay in one of the following ways, (Please use the reference: '{$this->name} {$obj->id}' )");
						$r = &$n->appendNode("ol");
						$r->appendNode("li", "", "For people in the UK you can pay in pounds sterling using direct bank transfer to the account: Bank: Barclays Bank, AccountName: The Tandem Club, Sortcode: 20-71-03, AccountNumber: 73018431, Reference: '{$this->name} {$obj->id}'");
						$r->appendNode("li", "", "For people in the UK you can post a cheque in pounds sterling for total amount, with name of Party Leader and '{$this->name} {$obj->id}' on the reverse, payable to: \"The Tandem Club\" to: {$this->chequeAddress}.");
						$r->appendNode("li", "", "For European bookings pay in euros by direct bank transfer to the following bank account: F.E.R. Oudhof, IBAN: NL58INGB0006385650, BIC: INGBNL2A use the reference '{$this->name} {$obj->id}'. For more information contact: Eelco Oudhof, Lichtboei 26 NL-1186 DE Amstelveen, The Netherlands, email: administratie@tandemclub.nl.");
						$r->appendNode("li", "", "International bookings outside Europe pay by direct money transfer to the euros account given above, or in Euro's cash on arrival on the rally site.");

						$h->appendText($n->render());
						$hEmail->appendText($n->render());
						$this->payments->payDo($payment);
					}
				}
				// Set paymentId
				$obj->paymentId = $payment->id;
				$this->sys->db->putObject($this->bookingTable, $obj);
			}
			
			$page .= $h->render();
			$page .= $this->view($obj, 1);
			$pageEmail = $hEmail->render();
			$pageEmail .= $this->view($obj, 1);
			
			if($editing == 0){
				// Send an email
				$h = new BHtmlNode("html");
				$b = &$h->appendNode("head");
				$b = &$h->appendNode("body", "style='background-color: #CCDDDD;'");
				$b->appendNode("style", "type='text/css'", "table { empty-cells: show; }");
				$b->appendText($pageEmail);
				emailListFrom("{$this->name}@tandem-club.org.uk", $obj->email, "{$this->sys->siteTitle}: Event booking: {$obj->id}", $h->render(), 1);

				// Send admin email
				$subject = "{$this->sys->siteTitle}: Event Admin: {$this->name}: New booking: Number: {$obj->id}";
				$message = "{$this->sys->siteTitle}: Event Admin: {$this->name}: New booking: Number: {$obj->id}\n\n";
				$message .= "Name:        {$obj->forename0} {$obj->surname0}\n";
				$message .= "Email:       {$obj->email}\n";
				$message .= "Postcode:    {$obj->postcode}\n";
				$message .= "PaymentType: {$obj->paymentType}\n\n";
				$message .= "View: {$this->sys->baseUrl}/events/{$this->name}?cmd=bookingView&id={$obj->id}\n";
				$message .= "Edit: {$this->sys->baseUrl}/events/{$this->name}?cmd=bookingEdit&id={$obj->id}\n";
				$message .= "Event information: {$this->sys->baseUrl}/events/{$this->name}\n";
				emailListFrom($this->sys->siteEmailAddress, "{$this->name}@tandem-club.org.uk", $subject, $message);
			}
		}
		else {
			$page .= $this->header();
			$page .= $this->form(0);
		}

		return $page;
	}

	function header(){
		$startDate = $this->eventStartDate->getDateTime("%A %e %B %Y");
		$endDate = $this->eventEndDate->getDateTime("%A %e %B %Y");
		$bookingEndDate = $this->bookingEndDate->getDateTime("%A %e %B %Y");
		$text = <<< End_Of_Text
<h1 style="text-align: center;">{$this->eventTitle}</h1>
<h2 style="text-align: center;">{$startDate} - ${endDate}</h2>
<h1 style="text-align: center;">Booking Form</h1>
<h4 style="text-align: center;">Bookings to be received by {$bookingEndDate}</h4>

<h3>Important Notes</h3>
<ul>
<li>This event is open to UK or overseas Tandem Club Members or, in the case of overseas people, members of the following organisations:
"Tandem Club of America", "Tandem Club de France", "Tandem Club NL (Netherlands)", "Tandem ClubVlaanderen (Belgium)" and "Tandem-fahren.de (Germany)".
You can join the Tandem Club using the forms on the TC website. <a href='/files/membershipJoin.html'>Joining the TC</a> (About 10 UKP per year)    </li>
<li>All bookings to be received by Rally organisers by: {$bookingEndDate}. Subject to availability.</li>
<li>The Rally fee covers the costs incurred in arranging the rally. This includes equipment costs, evening event costs,
Postage and printing of route instructions plus	all the other bits and pieces to make it all happen.</li>
<li>Tandem Club Membership numbers are on the mailing label for your Journal or on any renewal forms. Please enter your cycling clubs name for overseas non TC members.
If you don't know your membership number enter "Unknown" in the membership number box and we will find it for you or email us
at: <a href="mailto:{$this->name}@tandem-club.org.uk">{$this->name}@tandem-club.org.uk</a> so we can find it for you.</li>
<li>Data privacy: The information you provide on this form will be added to the Club event database to be used by the Club for purposes only in connection with the
running of the event. The data will be available as needed to facilitate the event. It will never be disclosed for marketing purposes.
You have a right to complain to the ICO if you believe there is a problem with the Club's handling of your data. Further details are given in the Club's Data Privacy Policy, available on the Club website
or on request from the Club Secretary, <a href='mailto:secretary@tandem-club.org.uk'>secretary@tandem-club.org.uk</a>. Please indicate on the form below that you have read and accept these terms.</li>

</ul>
<h3>Booking procedure</h3>
<ol>
<li>Please complete this form and then use the "Calculate" button at the end to calculate the payments.
Make sure you choose the payment type (card, bacs or cheque) and currency type you will be using to pay.</li>
<li>Dates of birth only need to be entered (changed from the default) for people below the age of 18.</li>
<li>Check the payment values and if Ok use the "Book" button to book for the event. You will be sent
an email with the booking information and will be given a "BookingRef".</li>

<li>Please then pay in one of the following ways:
<ul>
<li>For card payments click on the "Pay on Credit or Debit Card" button that is displayed after the "Book" button is pressed and you will be taken to a secure payments page.</li>
<li>For people in the UK you can pay in pounds sterling using BACS (direct bank transfer) to the Tandem Clubs account.</li>
<li>For people in the UK you can post a cheque in pounds sterling for total amount, with name of Party Leader and booking reference on the reverse, payable to: "The Tandem Club".</li>
<li>For european bookings you can pay in euros by direct bank transfer to a european bank.</li>
<li>For international bookings outside Europe pay by direct money transfer to the euros account given above, or in Euro's cash on arrival on the rally site.</li> 
</ul>
Please use the booking reference provided as the BACS payment reference and on the back of cheques. The booking reference, account number details and address to post cheques will be given after you book.
</li>
<li>Note that the TAB key can be used to move to the next field</li>
<li>If you have any problems with the booking form or system please email:
<a href="mailto:{$this->name}@tandem-club.org.uk">{$this->name}@tandem-club.org.uk</a></li>
</ol>
End_Of_Text;

		return $text;
	}

	function setDataFromPost(&$booking){
		// Sets the data from POST info
		if(0){
			foreach($booking as $k => $v){
				if(isset($_POST[$k])){
					$booking->$k = $_POST[$k];
				}
			}
		}
		$booking->set($_POST);
	
		// Work around a bug in a version of the Google Chrome web browser
		if(($booking->surname5 == $booking->surname0) && ($booking->forename5 == $booking->forename0)){
			$booking->surname5 = "";
			$booking->forename5 = "";
		}
				
		for($n = 0; $n < 6; $n++){
			if($booking->{"forename$n"} || $booking->{"surname$n"}){
				$booking->{"name$n"} = $booking->{"forename$n"} . " " . $booking->{"surname$n"};
			}
		}
	}

	function calculate(&$booking){
		$adultsTc = 0;
		$adults = 0;
		$children = 0;
		$infants = 0;
		$campingAdults = 0;
		$campingChildren = 0;
		$camping = 0;
		$campingNumber = 0;
		$roomAdults = 0;
		$roomChildren = 0;
		
		switch($booking->accommodation){
		case "Tent":
		case "TrailerTent":
		case "Caravan":
		case "MotorHome":
			$camping = 1;
			break;
		}

		for($n = 0; $n < 6; $n++){
			if($booking->{"name$n"} != ""){
				$dateOfBirth = new DateTime($booking->{"dateOfBirth$n"});
				$startTime = new DateTime($this->eventStartDate->getDateTime());
				$age = $startTime->diff($dateOfBirth)->y;
				//print "Arge: {$age}<br>";
				
				$booking->{"age$n"} = $age;
				
//				print("Age: {$age}<br>\n");
				
				if($age >= $this->ageAdult){
					$adults++;
				}
				else if($age >= $this->ageChild){
					$children++;
				}
				else {
					$infants++;
				}
		
				if($camping){		
					if($age >= $this->ageAdultCamping){
						$campingAdults++;
					}
					else if($age >= $this->ageChildCamping){
						$campingChildren++;
					}
				}
				
				if($age >= $this->ageAdultCamping){
					$roomAdults++;
				}
				else if($age >= $this->ageChildCamping){
					$roomChildren++;
				}
			}
		}
		
		$booking->adults = $adults;
		$booking->children = $children;
		$booking->infants = $infants;
		$booking->campingAdults = $campingAdults;
		$booking->campingChildren = $campingChildren;

		$arrivalDate = new BDate($booking->arrivalDate);
		$departureDate = new BDate($booking->departureDate);
		
		$days = ($departureDate->timeStamp - $arrivalDate->timeStamp) / 86400;

		if(0 && ($days >= 5)){
			// Apply discount
			$this->prices[1]->pricePounds *= 0.75;
			$this->prices[2]->pricePounds *= 0.75;
			$this->prices[4]->pricePounds *= 0.75;
			$this->prices[1]->priceEuros = ceil($this->prices[1]->pricePounds * 1.15);
			$this->prices[2]->priceEuros = ceil($this->prices[2]->pricePounds * 1.15);
			$this->prices[4]->priceEuros = ceil($this->prices[4]->pricePounds * 1.15);
		}

		for($n = 0; $n < count($this->prices); $n++){
			if($booking->currency == "Pounds")
				$booking->{"e$n"} = $this->prices[$n]->pricePounds;
			else
				$booking->{"e$n"} = $this->prices[$n]->priceEuros;
		}

		$booking->q0 = $adults;
		$booking->q1 = $children;
		$booking->q2 = 0;
		$booking->q3 = 0;
		$booking->q4 = 0;
		$booking->q5 = 0;
		$booking->q6 = 0;
		$booking->q7 = 0;
		$booking->q8 = 0;
		$booking->q9 = 0;
		$booking->q10 = 0;
		$booking->q11 = 0;
		$booking->q12 = 0;
		$booking->q13 = 0;

		switch($booking->accommodation){
		case "Tent":
		case "TrailerTent":
		case "Caravan":
		case "MotorHome":
			if($days >= 7)
				$booking->q2 = 1;
			else
				$booking->q3 = $days;

			if(0){
				if($campingAdults > 2)
					$booking->q4 = $campingDays * ($campingAdults - 2);
				if($campingChildren){
					$c = $campingChildren;
					if($campingAdults < 2){
						$c -= (2 - $campingAdults);
						if($c < 0)
							$c = 0;
					}
					$booking->q5 = $campingDays * $c;
				}
			}
//			print "campingAdults: {$campingAdults}, campingChildren: {$campingChildren}, {$booking->q2}, {$booking->q3}</br>\n";
			break;

		case "HallRoom":
//			$booking->q3 = $roomAdults * $days;
//			$booking->q4 = $roomChildren * $days;
		}

		$booking->q4 = $booking->electricity;
//		$booking->q4 = $booking->dogs * $days;
//		$booking->q4 = $booking->food;
//		$booking->q5 = intval($booking->hogRoast);
//		$booking->q1 = intval($booking->trip1Adult);
//		$booking->q7 = intval($booking->trip1Concession);
//		$booking->q2 = intval($booking->trip1Child);
//		$booking->q9 = intval($booking->trip1Infant);

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$booking->q4 += intval($booking->{"tshirt$n"});
		}
		for($n = 0; $n < count($this->tshirtSizesChild); $n++){
			$booking->q5 += intval($booking->{"tshirtChild$n"});
		}

		$total = 0;
		for($n = 0; $n < count($this->prices); $n++){
			$booking->{"t$n"} = $booking->{"q$n"} * $booking->{"e$n"};
			$total += $booking->{"t$n"};
		}
		
		$total += $booking->adjustTotal;
		$total = round($total, 2);
		$booking->total = $total;
	}
	
	function validate(&$booking, $editing, &$errors){
		$err = 0;
		
		if($booking->name0 == ""){
			$err = 1;
			$errors[] = "We need the party leaders name";
		}
		
		// Email
//		if(!preg_match("/.+@.+/", $booking->email)){
		if(!filter_var($booking->email, FILTER_VALIDATE_EMAIL)){
			$err = 1;
			$errors[] = "No email or incorrect email format";
		}

		if(!$booking->gdprAcceptance){
			$err = 1;
			$errors[] = "The data privacy checkbox is not ticked";		
		}

		$this->findTotals();
		$arrivalDate = new BDate($booking->arrivalDate);
		$departureDate = new BDate($booking->departureDate);

//		$accommodationStartDate = $this->eventStartDate;
//		$accommodationEndDate = $this->eventEndDate;
		$accommodationStartDate = $this->accommodationStartDate;
		$accommodationEndDate = $this->accommodationEndDate;
		$days = ($accommodationEndDate->timeStamp - $accommodationStartDate->timeStamp) / 86400;

		if(($booking->paymentType == "cheque") && ($booking->currency != "Pounds")){
			$err = 1;
			$errors[] = "Sorry, only able to accept cheques in Pounds";
		}

		if(0){
			switch($booking->accommodation){
			case "Caravan":
			case "MotorHome":
			case "TrailerTent":
				if((intval($booking->length) == 0) || (intval($booking->lengthWithoutTowbar) == 0) || (intval($booking->width) == 0)){
					$err = 1;
					$errors[] = "You need to provide the length and width information in cm";
				}
			}
		}

		if(!$editing){
			if(0){
				switch($booking->accommodation){
				case "On-site Caravan":
					if($this->totals["On-site Caravan"] >= 4){
						$err = 1;
						$errors[] = "All the On-site Caravan's have gone";
					}
					break;
				}
			}

			if(0){
				switch($booking->accommodation){
				case "Lodge":
					if(($this->totals["Lodge"] + $booking->q1) >= 48){
						$err = 1;
						$errors[] = "All the Lodge places are gone";
					}
					break;
				}
			}
			if(0){
				switch($booking->accommodation){
				case "HallRoom":
					$num += (($booking->q3 / $days) + ($booking->q4 / $days));
					if(($this->totals["HallRoom"] + $num) > 33){
						$err = 1;
						$errors[] = "All the Lodge places are gone";
					}
					break;
				}
			}

//			print "Total camping:"; print $this->totals["camping"]; print "<br>";
			if(($booking->accommodation == "Tent") || ($booking->accommodation == "TrailerTent") || ($booking->accommodation == "Caravan") || ($booking->accommodation == "MotorHome")){
				if(0 && ($this->totals["camping"] >= 100)){
					$err = 1;
					$errors[] = "All the camping pitches have gone";
				}
				if(1 && $booking->electricity && ($this->totals["electricity"] >= 40)){
					$err = 1;
					$errors[] = "All the electricity hookups have gone";
				}
			}
			else {
				if($booking->electricity){
					$err = 1;
					$errors[] = "Electricity is only for campers";
				}
				if($booking->dogs){
					$err = 1;
					$errors[] = "Dogs cost is only for campers";
				}
			}
		}

		if($arrivalDate->timeStamp < ($accommodationStartDate->timeStamp)){
			$err = 1;
			$errors[] = "Arrival date before event.";
		}
		if($departureDate->timeStamp > ($accommodationEndDate->timeStamp)){
			$err = 1;
			$errors[] = "Departure date after event.";
		}	
		if(($booking->paymentType == "card") && ($booking->currency != "Pounds")){
			$err = 1;
			$errors[] = "Card payments can only be in Pounds.";
		}
		
		return $err;
	}

	function form($booking){
		$dateNow = new BDate("now");
		$helpSelect = array("None", "Arrival Desk", "Childrens Club", "Bar", "Games", "Rally Event", "End of Rally Clear up", "Anything");
		$accommodation = array("Tent", "Caravan", "MotorHome", "TrailerTent", "Off site");

		if($this->checkAccess()){
//			$accommodation = array_merge($accommodation, array("Cottage1", "Cottage2", "Cottage3", "Cottage4"));
		}

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='/events/{$this->name}/bookingForm#prices' method='post'");

		if($booking == 0){
			$editing = 0;
			$booking = new BObjEventBooking();
			for($n = 0; $n < 6; $n++){
				$booking->{"name$n"} = "";
				$booking->{"forename$n"} = "";
				$booking->{"surname$n"} = "";
				$booking->{"sex$n"} = "0";
				$booking->{"dateOfBirth$n"} = "1960-01-01";
			}
			$booking->member = "";
			$booking->address = "";
			$booking->city = "";
			$booking->postcode = "";
			$booking->country = "United Kingdom";
			$booking->email = "";
			$booking->telephone = "";
			$booking->mobileTelephone = "";
			$booking->vehicleReg = "";
			$booking->arrivalDate = $this->eventStartDate->get();
			$booking->departureDate = $this->eventEndDate->get();
			$booking->accommodation = "Lodge";
			$booking->paymentType = "card";
			$booking->currency = "Pounds";
			$booking->help = "";
			$booking->notes = "";
			$booking->dogs = 0;
			
			for($n = 0; $n < count($this->tshirtSizes); $n++){
				$booking->{"tshirt$n"} = 0;
			}
			for($n = 0; $n < count($this->tshirtSizesChild); $n++){
				$booking->{"tshirtChild$n"} = 0;
			}

			for($n = 0; $n < count($this->prices); $n++){
				$booking->{"q$n"} = 0;
				$booking->{"e$n"} = 0;
				$booking->{"t$n"} = 0;
			}
			$booking->total = 0;
			$booking->adjustTotal = 0;
			$booking->paid = 0;
			
			if($this->sys->user){
				$a = explode(" ", $this->sys->user->name);
				if(count($a) == 2){
					$fn = $a[0];
					$sn = $a[1];
				}
				$booking->name0 = $this->sys->user->name;
				$booking->forename0 = $fn;
				$booking->surname0 = $sn;
				$booking->address = $this->sys->user->postalAddress;
				$booking->city = $this->sys->user->city;
				$booking->postcode = $this->sys->user->postcode;
				$booking->country = $this->sys->user->country;
				$booking->email = $this->sys->user->emailAddress;
				$booking->telephone = $this->sys->user->telephoneHome;
				$booking->mobileTelephone = $this->sys->user->telephoneMobile;
			}

//			print("Data: "); print_r($booking); print("<br>\n");
		}
		else {
			$editing = 1;
			// Booking Info
			$f->appendNode("h2", "", "Editing Booking");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$t->appendTableInput("Party Leader", "info0", "view", $booking->name0, "width:300px;"); 
			$t->appendTableInput("Booking Id", "info1", "view", $booking->id, "width:300px;"); 
			$t->appendTableInput("Added By", "info2", "view", $booking->addedUserName, "width:300px;"); 
			$t->appendTableInput("Added Date", "info3", "view", $booking->addedDate, "width:300px;"); 
		}
		
		if(isset($_POST["editing"]))
			$editing = filterToInt($_POST["editing"]);
		$this->setDataFromPost($booking);
		$this->calculate($booking);
//		print_r($booking);
		
		$f->appendNode("input", "type='hidden' name='id' value='{$booking->id}'");
		$f->appendNode("input", "type='hidden' name='editing' value='{$editing}'");

		// People
		$f->appendNode("h3", "", "Names of All Attending");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Forename");
		$r->appendNode("th", "", "Surname");
		$r->appendNode("th", "", "Male/Female (Male ticked)");
		$r->appendNode("th", "", "Date of Birth");

		for($n = 0; $n < 6; $n++){
			$r = &$t->appendNode("tr");
			if($n == 0)
				$r->appendNode("td", "", "Name " . $n . ": Party Leader");
			else
				$r->appendNode("td", "", "Name " . $n);

			if($booking->{"surname$n"})
				$booking->{"name$n"} = $booking->{"forename$n"} . " " . $booking->{"surname$n"};
			else
				$booking->{"name$n"} = "";
			$r->appendTableRowInput("", "forename$n", "text", $booking->{"forename$n"}, "width: 150px;");
			$r->appendTableRowInput("", "surname$n", "text", $booking->{"surname$n"}, "width: 150px;");
			$r->appendTableRowInput("", "sex$n", "checkbox", $booking->{"sex$n"});
			$r->appendTableRowInput("", "dateOfBirth$n", "date", $booking->{"dateOfBirth$n"});
			$r->appendNode("input", "type='hidden' name='name$n' value='{$booking->{"name$n"}}'");
			$r->appendNode("input", "type='hidden' name='age$n' value='{$booking->{"age$n"}}'");
		}
		
		$f->appendNode("h3", "", "General");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("TC Membership Number or Other Cycling organisation name", "member", "text", $booking->member, "width:300px;"); 
		$t->appendTableInput("Postal Address (First line or 2)", "address", "textarea", $booking->address, "height:50px; width:400px"); 
		$t->appendTableInput("Town/City", "city", "text", $booking->city, "width:300px"); 
		$t->appendTableInput("Postcode", "postcode", "text", $booking->postcode, "width:300px;"); 
		$t->appendTableInput("Country", "country", "select", $booking->country, "", $this->countryList()); 
		$t->appendTableInput("Email Address", "email", "text", $booking->email, "width:300px;"); 
		$t->appendTableInput("Telephone (with code)", "telephone", "text", $booking->telephone, "width:300px;"); 
		$t->appendTableInput("Mobile Telephone (with code)", "mobileTelephone", "text", $booking->mobileTelephone, "width:300px;"); 
		$t->appendTableInput("Vehicle Registration", "vehicleReg", "text", $booking->vehicleReg, "width:300px;"); 
		$t->appendTableInput("Arrival Date", "arrivalDate", "date", $booking->arrivalDate, "width:300px;"); 
		$t->appendTableInput("Departure Date", "departureDate", "date", $booking->departureDate, "width:300px;"); 
		$t->appendTableInput("Accommodation", "accommodation", "select", $booking->accommodation, "", $accommodation); 
		$t->appendTableInput("Electricity", "electricity", "checkbox", $booking->electricity, "width:50px;"); 

//		$t->appendTableInput("Caravan/MotorHome/TrailerTent: Length with towbar (cm)", "length", "text", $booking->length, "width:50px;"); 
//		$t->appendTableInput("Caravan/MotorHome/TrailerTent: Length without towbar (cm)", "lengthWithoutTowbar", "text", $booking->lengthWithoutTowbar, "width:50px;"); 
//		$t->appendTableInput("Caravan/MotorHome/TrailerTent: Width (cm)", "width", "text", $booking->width, "width:50px;"); 

		$t->appendTableInput("Payment Type", "paymentType", "select", $booking->paymentType, "", array("card", "bacs", "cheque")); 
		$t->appendTableInput("Payment Currency", "currency", "select", $booking->currency, "", array("Pounds", "Euros")); 
//		$t->appendTableInput("Happy to help with", "help", "select", $booking->help, "", $helpSelect); 
		$t->appendTableInput("I have read the data privacy note above and I consent to it.", "gdprAcceptance", "checkbox", $booking->gdprAcceptance); 

		$t->appendTableInput("Additional Information", "notes", "textarea", $booking->notes, "height:100px; width:400px;"); 
		if($this->checkAccess()){
			$t->appendTableInput("Admin Information", "adminNotes", "textarea", $booking->adminNotes, "height:100px; width:400px; background-color:LightSalmon;"); 
		}

//		$t->appendTableInput("Last night meal (Number of)", "food", "text", $booking->food, "width:50px;"); 
//		$t->appendTableInput("Coach trip adult (Number of)", "trip1Adult", "text", $booking->trip1Adult, "width:50px;"); 
//		$t->appendTableInput("Beamish trip concession (Number of)", "trip1Concession", "text", $booking->trip1Concession, "width:50px;"); 
//		$t->appendTableInput("Coach trip child (up to 18 Number of)", "trip1Child", "text", $booking->trip1Child, "width:50px;"); 
//		$t->appendTableInput("Beamish trip infant (0-5) Number of)", "trip1Infant", "text", $booking->trip1Infant, "width:50px;"); 
//		$t->appendTableInput("Interested in fish and chips", "food", "checkbox", $booking->food, "width:50px;"); 
//		$t->appendTableInput("Dogs (Number of)", "dogs", "text", $booking->dogs, "width:50px;"); 
//		$t->appendTableInput("Hogroast Salad (Number of people)", "hogRoastSaladAdult", "text", $booking->hogRoastSaladAdult, "width:50px;"); 
//		$t->appendTableInput("Hogroast (Number of children)", "hogRoastChild", "text", $booking->hogRoastChild, "width:50px;"); 
//		$t->appendTableInput("Hogroast Salad (Number of children)", "hogRoastSaladChild", "text", $booking->hogRoastSaladChild, "width:50px;"); 
//		$t->appendTableInput("Interested in wine tasting session", "wineTasting", "checkbox", $booking->wineTasting, "width:50px;"); 

		if($this->checkAccess() || ($dateNow->timeStamp < $this->extrasEndDate->timeStamp)){
			for($n = 0; $n < count($this->tshirtSizes); $n++){
				$size = $this->tshirtSizes[$n];
				$t->appendTableInput("T-Shirts {$size} (Number of)", "tshirt{$n}", "text", $booking->{"tshirt$n"}, "width:50px;"); 
			}
			for($n = 0; $n < count($this->tshirtSizesChild); $n++){
				$size = $this->tshirtSizesChild[$n];
				$t->appendTableInput("T-Shirts Child {$size} (Number of)", "tshirtChild{$n}", "text", $booking->{"tshirtChild$n"}, "width:50px;"); 
			}
		}

		// Prices
		$f->appendNode("a", "name=prices", "");
		$f->appendNode("h3", "", "Prices");
		$f->appendNode("i", "", "This section is automatically filled in from the above information. Please press the \"Calculate\" button to make sure this is updated.");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Unit");
		$r->appendNode("th", "", "Quantity");
		$r->appendNode("th", "", "Cost");
		$r->appendNode("th", "", "Total");
		
		for($n = 0; $n < count($this->prices); $n++){
//			if(($n < 5) || ($n > 8) || $this->checkAccess()){
			$price = $this->prices[$n];
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $price->item);
			$r->appendNode("td", "", $price->unit);
			$r->appendTableRowInput("", "q{$n}", "textView", $booking->{"q$n"}, "background-color:yellow"); 
			$r->appendTableRowInput("", "e{$n}", "textView", $booking->{"e$n"}, "background-color:yellow"); 
			$r->appendTableRowInput("", "t{$n}", "textView", $booking->{"t$n"}, "background-color:yellow");
		}

		if($this->checkAccess()){
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Adjust");
			$r->appendTableRowInput("", "adjustTitle", "text", $booking->adjustTitle, "width:300px;"); 
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendTableRowInput("", "adjustTotal", "text", $booking->adjustTotal, "width:100px;"); 
		}
		
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendTableRowInput("", "currencyDisplay", "view", $booking->currency, "background-color:yellow;"); 
		$r->appendTableRowInput("", "total", "textView", $booking->total, "background-color:yellow;"); 

		if($this->checkAccess()){
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "Paid");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			if($editing)
				$r->appendTableRowInput("", "paid", "text", $booking->paid, "background-color:yellow; width:100px;"); 
			else
				$r->appendTableRowInput("", "paid", "textView", $booking->paid, "background-color:yellow; width:100px;"); 

			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align='left'", "PaymentId");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "");
			$r->appendNode("td", "", "<a href='/payments?cmd=view&id={$booking->paymentId}'>{$booking->paymentId}</a>");
		}
		
		// Add hidden info
		$f->appendNode("input", "type='hidden' name='adults' value='{$booking->adults}'");
		$f->appendNode("input", "type='hidden' name='children' value='{$booking->children}'");
		$f->appendNode("input", "type='hidden' name='infants' value='{$booking->infants}'");
		$f->appendNode("input", "type='hidden' name='campingAdults' value='{$booking->campingAdults}'");
		$f->appendNode("input", "type='hidden' name='campingChildren' value='{$booking->campingChildren}'");
		$f->appendNode("input", "type='hidden' name='addedUserName' value='{$booking->addedUserName}'");
		$f->appendNode("input", "type='hidden' name='addedDate' value='{$booking->addedDate}'");


		// Buttons
		$f->appendNode("h3", "", "Book");
		$r = &$f->appendNode("ul");
		$r->appendNode("li", "", "First use the Calculate button to calculate the cost and validate the booking. Once this has been done please use the Book button.");
		$r->appendNode("li", "", "Please do not use the back button to edit and re-submit once sent. Contact the administrators to change the booking.");
		$r->appendNode("li", "", "Please see the Booking procedure info at the top of this form for details on how to pay.");
		$r->appendNode("li", "", "The rally fee will not be refunded after the closing date. Refund of other components will depend on what the Club can recover from the campsite and/or other suppliers.");
		$r->appendNode("li", "", "Photographs taken at the event may be published on the Tandem Club Website and/or The Tandem Club Journal.");
		
		$f->appendNode("p", "", "If you participate in any ride or event organised by the Tandem Club, you do so entirely at your own risk. The club, its officers and ride organisers accept no responsibility for loss, damage to property, accident, injury, or death, that may occur, however caused, whether by negligence, deliberate act or otherwise.");

		$err = 0;		
		if(isset($_POST["buttonCalculate"])){
			$errors = array();
			if($err = $this->validate($booking, $editing, $errors)){
				$f->appendNode("h3", "style='color: red'; font-size: 150%;", "There are errors in the form data");
				$r = &$f->appendNode("ul");
				foreach($errors as $e){
					$r->appendNode("li")->appendNode("span", "style='color: red; font-size: 150%;'", $e);
				}
				$f->appendText("Please correct these errors and click on the Calculate button again<br><br>");
			}
		}

		if(($err == 1) || !isset($_POST["buttonCalculate"]))
			$disable = "disabled='disabled'";
		else
			$disable = "";

		$f->appendNode("input", "type='submit' name='buttonCalculate' style='background-color: lightgreen; font-size: 200%;' value='Calculate'");
		$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
//		$f->appendNode("button", "type='button' name='buttonPrint' $disable style='background-color: lightgreen; font-size: 200%;' onclick='window.print();'", "Print Form");
//		$f->appendText("&nbsp;&nbsp;->&nbsp;&nbsp;");
		if($editing)
			$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Save Changes' onclick='this.value=\"Saving, please wait...\";'");
		else
			$f->appendNode("input", "type='submit' name='buttonBook' {$disable} style='background-color: lightgreen; font-size: 200%;' value='Book' onclick='this.value=\"Booking, please wait...\";'");

		return $f->render();
	}

	function view($booking, $forEmail = 0){
		$f = new BHtmlNode("div");
		
		// Booking Info
		$f->appendNode("h3", "", "Booking Info");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("Party Leader", "info0", "view", $booking->name0, "width:300px;"); 
		$t->appendTableInput("Booking Id", "info1", "view", $booking->id, "width:300px;"); 
		$t->appendTableInput("Added By", "info2", "view", $booking->addedUserName, "width:300px;"); 
		$t->appendTableInput("Added Date", "info3", "view", $booking->addedDate, "width:300px;"); 

		// People
		$f->appendNode("h3", "", "Names of All Attending");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Name");
		$r->appendNode("th", "", "Male/Female (Male 1)");
		$r->appendNode("th", "", "Date of Birth");

		for($n = 0; $n < 6; $n++){
			$r = &$t->appendNode("tr");
			if($n == 0)
				$r->appendNode("td", "", "Name " . $n . ": Party Leader");
			else
				$r->appendNode("td", "", "Name " . $n);

			$r->appendTableRowInput("", "name$n", "view", $booking->{"name$n"}, "width: 300px;"); 
			$r->appendTableRowInput("", "sex$n", "view", $booking->{"sex$n"}); 
			$r->appendTableRowInput("", "dateOfBirth$n", "view", $booking->{"dateOfBirth$n"}); 
		}
		
		$f->appendNode("h3", "", "General");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("TC Membership Number or Other Cycling organisation name", "member", "view", $booking->member, "width:300px;"); 
		$t->appendTableInput("Postal Address", "address", "view_textarea", $booking->address, "height:50px; width:400px"); 
		$t->appendTableInput("Town/City", "city", "view", $booking->city, "width:300px"); 
		$t->appendTableInput("Postcode", "postcode", "view", $booking->postcode, "width:300px;"); 
		$t->appendTableInput("Country", "country", "view", $booking->country); 
		$t->appendTableInput("Email Address", "email", "view", $booking->email, "width:300px;"); 
		$t->appendTableInput("Telephone (with code)", "telephone", "view", $booking->telephone, "width:300px;"); 
		$t->appendTableInput("Mobile Telephone (with code)", "mobileTelephone", "view", $booking->mobileTelephone, "width:300px;"); 
		$t->appendTableInput("Vehicle Registration", "vehicleReg", "view", $booking->vehicleReg, "width:300px;"); 
		$t->appendTableInput("Arrival Date", "arrivalDate", "view", $booking->arrivalDate, "width:300px;"); 
		$t->appendTableInput("Departure Date", "departureDate", "view", $booking->departureDate, "width:300px;"); 
		$t->appendTableInput("Accommodation", "accommodation", "view", $booking->accommodation, "", array("Tent", "Caravan", "MotorHome", "Off-site")); 
		$t->appendTableInput("Electricity", "electricity", "view", $booking->electricity, "width:50px;"); 

//		$t->appendTableInput("Caravan/MotorHome/TrailerTent: Length with towbar (cm)", "length", "view", $booking->length, "width:50px;"); 
//		$t->appendTableInput("Caravan/MotorHome/TrailerTent: Length without towbar (cm)", "lengthWithoutTowbar", "view", $booking->lengthWithoutTowbar, "width:50px;"); 
//		$t->appendTableInput("Caravan/MotorHome/TrailerTent: Width (cm)", "width", "view", $booking->width, "width:50px;"); 

		$t->appendTableInput("Payment Type", "paymentType", "view", $booking->paymentType, "", array("card", "bacs", "cheque")); 
		$t->appendTableInput("Payment Currency", "currency", "view", $booking->currency, "", array("Pounds", "Euros")); 
//		$t->appendTableInput("Happy to help with", "help", "view", $booking->help, "", $helpSelect); 
		$t->appendTableInput("GDPR Accepted", "gdprAccepted", "view", $booking->gdprAcceptance); 
		$t->appendTableInput("Additional Information", "notes", "view", $booking->notes, "height:100px; width:400px;"); 
		if(!$forEmail && $this->checkAccess()){
			$t->appendTableInput("Admin Information", "adminNotes", "view", $booking->adminNotes, "height:100px; width:400px; background-color:LightSalmon;"); 
		}

//		$t->appendTableInput("Last night meal (Number of)", "food", "view", $booking->food, "width:50px;"); 
//		$t->appendTableInput("Coach trip adult (Number of)", "trip1Adult", "view", $booking->trip1Adult, "width:50px;"); 
//		$t->appendTableInput("Beamish trip concession (Number of)", "trip1Concession", "view", $booking->trip1Concession, "width:50px;"); 
//		$t->appendTableInput("Coach trip child (up to 18 Number of)", "trip1Child", "view", $booking->trip1Child, "width:50px;"); 
//		$t->appendTableInput("Beamish trip infant (0-5 Number of)", "trip1Infant", "view", $booking->trip1Infant, "width:50px;"); 
//		$t->appendTableInput("Interested in fish and chips", "food", "view", $booking->food, "width:50px;"); 
//		$t->appendTableInput("Dogs (Number of)", "dogs", "view", $booking->dogs, "width:50px;"); 
//		$t->appendTableInput("Hogroast (Number of people)", "hogRoastAdult", "view", $booking->hogRoastAdult, "width:50px;"); 
//		$t->appendTableInput("Hogroast Salad (Number of people)", "hogRoastSaladAdult", "view", $booking->hogRoastSaladAdult, "width:50px;"); 
//		$t->appendTableInput("Hogroast (Number of children)", "hogRoastChild", "view", $booking->hogRoastChild, "width:50px;"); 
//		$t->appendTableInput("Hogroast Salad (Number of children)", "hogRoastSaladChild", "view", $booking->hogRoastSaladChild, "width:50px;"); 
//		$t->appendTableInput("Interested in wine tasting session", "wineTasting", "view", $booking->wineTasting, "width:50px;"); 

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$size = $this->tshirtSizes[$n];
			$t->appendTableInput("T-Shirts {$size} (Number of)", "tshirt{$n}", "view", $booking->{"tshirt$n"}, "width:80px;"); 
		}

		for($n = 0; $n < count($this->tshirtSizesChild); $n++){
			$size = $this->tshirtSizesChild[$n];
			$t->appendTableInput("T-Shirts Child {$size} (Number of)", "tshirtChild{$n}", "view", $booking->{"tshirtChild$n"}, "width:80px;"); 
		}

		// Prices
		$f->appendNode("h3", "", "Prices");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Unit");
		$r->appendNode("th", "", "Quantity");
		$r->appendNode("th", "", "Cost");
		$r->appendNode("th", "", "Total");
		
		for($n = 0; $n < count($this->prices); $n++){
			$price = $this->prices[$n];
//			if(($n < 5) || ($n > 8) || $booking->{"q$n"}){
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $price->item);
			$r->appendNode("td", "", $price->unit);
			$r->appendTableRowInput("", "q{$n}", "view", $booking->{"q$n"}, "background-color:yellow"); 
			$r->appendTableRowInput("", "e{$n}", "view", $booking->{"e$n"}, "background-color:yellow"); 
			$r->appendTableRowInput("", "t{$n}", "view", $booking->{"t$n"}, "background-color:yellow");
		}

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Adjust");
		$r->appendTableRowInput("", "adjustTitle", "view", $booking->adjustTitle, "background-color:yellow;"); 
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendTableRowInput("", "adjustTotal", "view", $booking->adjustTotal, "background-color:yellow; width:100px;"); 

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendTableRowInput("", "currencyDisplay", "view", $booking->currency, "background-color:yellow;"); 
		$r->appendTableRowInput("", "total", "view", $booking->total, "background-color:yellow;"); 

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Paid");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendTableRowInput("", "paid", "view", $booking->paid, "background-color:yellow; width:100px;"); 

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "PaymentId");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "");
		$r->appendNode("td", "", "<a href='/payments?cmd=view&id={$booking->paymentId}'>{$booking->paymentId}</a>");

		$f->appendNode("p", "", "The rally fee will not be refunded after the closing date. Refund of other components will depend on what the Club can recover from the campsite and/or other suppliers.");
		$f->appendNode("p", "", "Photographs taken at the event may be published on the Tandem Club Website and/or The Tandem Club Journal.");
		$f->appendNode("p", "", "If you participate in any ride or event organised by the Tandem Club, you do so entirely at your own risk. The club, its officers and ride organisers accept no responsibility for loss, damage to property, accident, injury, or death, that may occur, however caused, whether by negligence, deliberate act or otherwise.");

		return $f->render();
	}
	
	function bookingTotals($uri){
		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='get'");
		$f->appendNode("h3", "", "Event Totals");
		
		$bookings = 0;
		$adults = 0;
		$children = 0;
		$infants = 0;
		$campingAdults = 0;
		$campingChildren = 0;
		$campingUnits = 0;
		$camping1Units = 0;
		$cottage1 = 0;
		$cottage2 = 0;
		$cottage3 = 0;
		$cottage4 = 0;
		$lodge = 0;
		$hallrooms = 0;
		$hogRoast = 0;
		$food = 0;
		$trip1Adult = 0;
		$trip1Concession = 0;
		$trip1Child = 0;
		$trip1Infant = 0;

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$tshirt{$n} = 0;
		}
		for($n = 0; $n < count($this->tshirtSizesChild); $n++){
			$tshirtChild{$n} = 0;
		}
		$electricity = 0;
		$dogs = 0;
		$coachTrip = 0;
		$wineTasting = 0;
		
		$due["Pounds"] = 0;
		$due["Euros"] = 0;
		$paid["Pounds"] = 0;
		$paid["Euros"] = 0;

		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		foreach($data as $obj){
			$camping = 0;
			$camping1 = 0;

			$arrivalDate = new BDate($obj->arrivalDate);
			$departureDate = new BDate($obj->departureDate);
			$days = ($departureDate->timeStamp - $arrivalDate->timeStamp) / 86400;
			
			$people = $obj->adults + $obj->children + $obj->infants;
			switch($obj->accommodation){
			case "Tent":
			case "TrailerTent":
			case "Caravan":
			case "MotorHome":
				$camping = 1;
				break;
			case "Cottage (Booked with campsite)":	$cottage1 += 1;		break;
			case "Cottage2":	$cottage2 += 1;		break;
			case "Cottage3":	$cottage3 += 1;		break;
			case "Cottage4":	$cottage4 += 1;		break;
			case "Lodge":		$lodge += $people;	break;
			case "HallRoom":	$hallRooms += (($obj->q3 / $days) + ($obj->q4 / $days)); break;
			}

			for($n = 0; $n < 6; $n++){
				if($obj->{"name$n"} != ""){
					$age = $obj->{"age$n"};
					if($age >= $this->ageAdult){
						$adults++;
					}
					else if($age >= $this->ageChild){
						$children++;
					}
					else {
						$infants++;
					}
					if($camping || $camping1){
						if($age >= $this->ageAdultCamping){
							$campingAdults++;
						}
						else if($age >= $this->ageChildCamping){
							$campingChildren++;
						}
					}
				}
			}
			
			if($camping){
				$campingUnits++;
			}
			
			if($camping1){
				$camping1Units++;
			}
			
			for($n = 0; $n < count($this->tshirtSizes); $n++){
				$tshirt{$n} += $obj->{"tshirt$n"};
			}
			for($n = 0; $n < count($this->tshirtSizesChild); $n++){
				$tshirtChild{$n} += $obj->{"tshirtChild$n"};
			}
			$electricity += $obj->electricity;
			$dogs += $obj->dogs;
			$hogRoast += intval($obj->hogRoast);
			$trip1Adult += intval($obj->trip1Adult);
			$trip1Concession += intval($obj->trip1Concession);
			$trip1Child += intval($obj->trip1Child);
			$trip1Infant += intval($obj->trip1Infant);
			$coachTrip += intval($obj->coachTripAdult) + intval($obj->coachTripChild);
			if($obj->food)
				$food += $obj->food;
			if($obj->wineTasting)
				$wineTasting += $obj->wineTasting;
				
			$due["{$obj->currency}"] += $obj->total;
			$paid["{$obj->currency}"] += $obj->paid;
			
			$bookings++;
		}
		
		$people = $adults + $children + $infants;

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Value");

		$t->appendTableInput("Bookings", "", "view", $bookings, ""); 
		$t->appendTableInput("People", "", "view", $people, ""); 
		$t->appendTableInput("Adults", "", "view", $adults, ""); 
		$t->appendTableInput("Children", "", "view", $children, ""); 
		$t->appendTableInput("Infants", "", "view", $infants, ""); 
		$t->appendTableInput("CampingAdults", "", "view", $campingAdults, ""); 
		$t->appendTableInput("CampingChildren", "", "view", $campingChildren, ""); 
		$t->appendTableInput("CampingUnits", "", "view", $campingUnits, ""); 
//		$t->appendTableInput("Camping1Units", "", "view", $camping1Units, ""); 
//		$t->appendTableInput("Cottage (Booked with campsite)", "", "view", $cottage1, ""); 
//		$t->appendTableInput("Cottage2", "", "view", $cottage2, ""); 
//		$t->appendTableInput("Cottage3", "", "view", $cottage3, ""); 
//		$t->appendTableInput("Cottage4", "", "view", $cottage4, ""); 
//		$t->appendTableInput("Lodge beds", "", "view", $lodge, ""); 
//		$t->appendTableInput("Lodge beds left", "", "view", 48 - $lodge, ""); 
//		$t->appendTableInput("HallRooms", "", "view", $hallRooms, ""); 

		for($n = 0; $n < count($this->tshirtSizes); $n++){
			$size = $this->tshirtSizes[$n];
			$t->appendTableInput("T-Shirts {$size}", "", "view", $tshirt{$n}, ""); 
		}
		for($n = 0; $n < count($this->tshirtSizesChild); $n++){
			$size = $this->tshirtSizesChild[$n];
			$t->appendTableInput("T-Shirts Child {$size}", "", "view", $tshirtChild{$n}, ""); 
		}
//		$t->appendTableInput("Dogs", "", "view", $dogs, ""); 
		$t->appendTableInput("Electricity", "", "view", $electricity, ""); 
//		$t->appendTableInput("CoachTrip", "", "view", $coachTrip, ""); 
//		$t->appendTableInput("HogRoast", "", "view", $hogRoast, ""); 
		$t->appendTableInput("Trip1Adult", "", "view", $trip1Adult, "");
//		$t->appendTableInput("Trip1Concession", "", "view", $trip1Concession, "");
		$t->appendTableInput("Trip1Child", "", "view", $trip1Child, "");
//		$t->appendTableInput("Trip1Infant", "", "view", $trip1Infant, "");
//		$t->appendTableInput("Number of last night meals", "", "view", $food, ""); 
		$t->appendTableInput("Number of wine tastings", "", "view", $wineTasting, ""); 


		$t->appendTableInput("Due Pounds", "", "view", $due["Pounds"], ""); 
		$t->appendTableInput("Paid Pounds", "", "view", $paid["Pounds"], ""); 
		$t->appendTableInput("Due Euros", "", "view", $due["Euros"], ""); 
		$t->appendTableInput("Paid Euros", "", "view", $paid["Euros"], ""); 

		
		return $f->render();
	}
	
	function findTotals(){
		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		$this->totals["electricity"] = 0;
		$this->totals["camping"] = 0;
		$this->totals["camping1"] = 0;
		$this->totals["Booked with Campsite Already"] = 0;
		$this->totals["Lodge"] = 0;
		$this->totals["HallRoom"] = 0;

		foreach($data as $obj){
			$arrivalDate = new BDate($obj->arrivalDate);
			$departureDate = new BDate($obj->departureDate);
			$days = ($departureDate->timeStamp - $arrivalDate->timeStamp) / 86400;

			switch($obj->accommodation){
			case "Tent":
			case "TrailerTent":
			case "Caravan":
			case "MotorHome":
			case "Booked with Campsite Already":
				$this->totals["camping"] += 1;
				break;
			case "Canoe Club Tent":
				$this->totals["camping1"] += 1;
				break;
			case "Booked with Campsite Already":
				$this->totals["Booked with Campsite Already"] += 1;
				break;
			case "Lodge":
				$this->totals["Lodge"] += 1;
				break;
			case "HallRoom":
				$this->totals["HallRoom"] += (($obj->q3 / $days) + ($obj->q4 / $days));
				break;
			}

			if($obj->electricity)
				$this->totals["electricity"] += 1;
		}
	}

	function countryList(){
		$l = array("");
		if($maps = $this->sys->modules["maps"]){
			$l = array_merge($l, $maps->getCountryList());
		}
		return $l;
	}

	

};
?>
