<?php
/*******************************************************************************
 *	Events.php	Events Module
 *			T.Barnaby,	BEAM Ltd,	2011-12-07
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("Events" => "BObjEvent"));

class BObjEvent extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("name", "String"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("description", "StringHtml"),
			new BMember("email", "String"),
			new BMember("startDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("endDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("bookingStartDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("extrasEndDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("bookingEndDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("active", "Int"),
			new BMember("hasBookingForm", "Int"),
			new BMember("hasBookingFormExtra", "Int"),
			new BMember("nonMemberAccess", "String"),
			));
	}
};

function addressOneLine($str){
	$strRet = "";
	
	for($i = 0; $i < strlen($str); $i++){
		if($str[$i] == "\r"){
		}
		else if($str[$i] == "\n"){
			if($i){
				if($str[$i - 1] == ",")
					$strRet .= " ";
				else
					$strRet .= ", ";
			}
		}
		else {
			$strRet .= $str[$i];
		}
	}

	return $strRet;
}

class Events extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "events";
		$this->title = "Events";
		$this->info = "Events module. This module allows the management of Events.";
		$this->event = "";
		$this->eventArg = "";
		$this->eventObj = Null;
		
		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminEvents"));
		$this->sys->modules["user"]->appendRoles(array("viewEvents"));
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin,adminEvents");
	}

	function checkAccessView(){
		return $this->sys->login->checkLogin("viewEvents");
	}
		
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Events", "/{$this->name}", "", array("Events List" => "/{$this->name}", "Events Add" => "/{$this->name}?cmd=infoAdd", "Booking form" => "/tc/modules/events/bookingForm.php"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);
		if($this->event == "old"){
			$module = $this->sys->modules["content"];
			$page = $module->getPage("files/events/" . $this->event . "/index.html");
			return $page;
		}

		if($this->event != ""){
			// Get Event object
			$list = $this->sys->db->getObjectList("Events", "", "`name` = '{$this->event}'", "", "");
			if(count($list) != 1){
				$this->sys->pageDisplayError(new BError(1, "Unknown event name: '" . $this->event . "'"));
				return "";
			}
			$this->eventObj = $list[0];
		}

		
//		printf("Event: %s Arg: %s<br>\n", $this->event, $this->eventArg);
		
		$page = "";
		
		if($this->event == ""){
			switch($this->cmd){
			case "eventView":	$page = $this->eventEdit($uri); break;
			case "eventEdit":	$page = $this->eventEdit($uri); break;
			case "eventSave":	$page = $this->eventSave($uri); break;
			case "eventDelete":	$page = $this->eventDelete($uri); break;
			default:		$page = $this->eventList($uri); break;
			}
		}
		else {
			if($this->eventObj->hasBookingForm){
				// Import booking form code
				if(file_exists($this->sys->baseDir . "/modules/events/BookingForm" . ucfirst($this->event) . ".php"))
					require_once "BookingForm" . ucfirst($this->event) . ".php";
				else
					return "No event booking form available";
			}
			if($this->eventObj->hasBookingFormExtra){
				// Import booking form code
				if(file_exists($this->sys->baseDir . "/modules/events/BookingForm" . ucfirst($this->event) . "Extra.php"))
					require_once "BookingForm" . ucfirst($this->event) . "Extra.php";
				else
					return "No event booking form extra available";
			}

			$this->sys->pageTitle = ucfirst($this->event);
			$this->bookingTable = "EventBooking" . ucfirst($this->event);
			$this->bookingTableExtra = "EventBooking" . ucfirst($this->event) . "Extra";

			if($this->eventArg == "bookingForm"){
				if($this->eventObj->hasBookingForm){
					$bookingForm = new EventBookingForm($this->sys, $this->eventObj);
					$page .= $bookingForm->getPage($uri);
				}
				else {
					$this->sys->pageDisplayError(new BError(1, "This event has no booking form"));
					return "";
				}
			}
			else if($this->eventArg == "bookingFormExtra"){
				if($this->eventObj->hasBookingFormExtra){
					$bookingFormExtra = new EventBookingFormExtra($this->sys, $this->eventObj);
					$page .= $bookingFormExtra->getPage($uri);
				}
				else {
					$this->sys->pageDisplayError(new BError(1, "This event has no booking extra form"));
					return "";
				}
			}
			else {
				if($this->checkAccess()){
					switch($this->cmd){
					case "bookingTotals":
						$page = $this->bookingTotals($uri);
						break;
					case "bookingListFull":
						$page = $this->bookingListFull($uri);
						break;
					case "bookingListRaw":
						$page = $this->bookingListRaw($uri);
						break;
					case "bookingListSpreadsheet":
						$page = $this->bookingListSpreadsheet($uri);
						break;
					case "bookingList":
						$page = $this->bookingList($uri);
						break;
					case "bookingView":
						$page = $this->bookingView($uri);
						break;
					case "bookingEdit":
						$page = $this->bookingEdit($uri);
						break;
					case "bookingDelete":
						$page = $this->bookingDelete($uri);
						break;
					case "bookingListExtra":
						$page = $this->bookingListExtra($uri);
						break;
					case "bookingExtraEdit":
						$page = $this->bookingExtraEdit($uri);
						break;
					case "bookingExtraDelete":
						$page = $this->bookingExtraDelete($uri);
						break;
					case "bookingListExtraSpreadsheet":
						$page = $this->bookingListExtraSpreadsheet($uri);
						break;
					case "email":
						$page = $this->email($uri);
						break;
					case "emailAll":
						$page = $this->emailAll($uri);
						break;
					case "eventEditInfo":
						$page = $this->eventEditInfo($uri);
						break;
					case "eventAddPage":
						$page = $this->eventAddPage($uri);
						break;
					case "addPage1":
						$page = $this->eventAddPage1($uri);
						break;
					case "eventEditReport":
						$page = $this->eventEditReport($uri);
						break;
					case "save":
						$page = $this->save($uri);
						break;
					case "eventAddWhatsOn":
						$page = $this->eventAddWhatsOn($uri);
						break;
					case "createTable":
						$page = $this->createTable($uri);
						break;
					default:
						if(basename($this->eventArg) == "info.html"){
							$module = $this->sys->modules["contentDir"];
							$page = $module->getPage("files/events/" . $this->event . "/" . dirname($this->eventArg));
						}
						else {
							$module = $this->sys->modules["content"];
							if($this->eventArg == ""){
								header ("Location: /events/" . $this->event . "/index.html");
								return;
							}
							$page = $module->getPage("files/events/" . $this->event . "/" . $this->eventArg);
						}
					}
				}
				else if($this->checkAccessView()){
					switch($this->cmd){
					case "bookingTotals":
						$page = $this->bookingTotals($uri);
						break;
					case "bookingListFull":
						$page = $this->bookingListFull($uri);
						break;
					case "bookingListRaw":
						$page = $this->bookingListRaw($uri);
						break;
					case "bookingListSpreadsheet":
						$page = $this->bookingListSpreadsheet($uri);
						break;
					case "bookingList":
						$page = $this->bookingList($uri);
						break;
					case "bookingView":
						$page = $this->bookingView($uri);
						break;
					case "bookingListExtra":
						$page = $this->bookingListExtra($uri);
						break;
					case "bookingListExtraSpreadsheet":
						$page = $this->bookingListExtraSpreadsheet($uri);
						break;
					case "email":
						$page = $this->email($uri);
						break;
					case "emailAll":
						$page = $this->emailAll($uri);
						break;
					default:
						if(basename($this->eventArg) == "info.html"){
							$module = $this->sys->modules["contentDir"];
							$page = $module->getPage("files/events/" . $this->event . "/" . dirname($this->eventArg));
						}
						else {
							$module = $this->sys->modules["content"];
							if($this->eventArg == ""){
								header ("Location: /events/" . $this->event . "/index.html");
								return;
							}
							$page = $module->getPage("files/events/" . $this->event . "/" . $this->eventArg);
						}
					}
				}
				else {
					if(basename($this->eventArg) == "info.html"){
						$module = $this->sys->modules["contentDir"];
						$page = $module->getPage("files/events/" . $this->event . "/" . dirname($this->eventArg));
					}
					else {
						$module = $this->sys->modules["content"];
						if($this->eventArg == ""){
							header ("Location: /events/" . $this->event . "/index.html");
							return;
						}
						$page = $module->getPage("files/events/" . $this->event . "/" . $this->eventArg);
					}
				}
			}
		}

		return $page;
	}

	function getPageBlock($uri){
		$this->parseUri($uri);
		if($this->event == "old"){
			return "";
		}
		if($this->event != ""){
			// Get Event object
			$list = $this->sys->db->getObjectList("Events", "", "`name` = '{$this->event}'", "", "");
			if(count($list) != 1){
				$this->sys->pageDisplayError(new BError(1, "Unknown event name"));
				return "";
			}
			$this->eventObj = $list[0];
		}

		$help = $this->sys->baseUrl . "/modules/events/info.html";

		if($this->checkAccess()){
			if($this->event == ""){
				$o = new BMenu("Events", "", "", array(
					new BMenu("Events List All", "", "href='?cmd=eventList'"),
					new BMenu("", "", ""),
					new BMenu("Event Add", "",	"href='?cmd=eventEdit'"),
					new BMenu("", "", ""),
					new BMenu("Help", "",		"href='{$help}'"),
					));
			}
			else {
				$m = array();
				if($this->eventObj->hasBookingForm){
					$m[] = new BMenu("Event Home", "",	"href='/events/{$this->event}/index.html'");
					$m[] = new BMenu("Booking Form", "",	"href='/events/{$this->event}/bookingForm'");
					if($this->eventObj->hasBookingFormExtra){
						$m[] = new BMenu("Booking Form Extra", "",	"href='/events/{$this->event}/bookingFormExtra'");
					}
					$m[] = new BMenu("", "", "");
					$m[] = new BMenu("Event List Totals", "", "href='/events/{$this->event}?cmd=bookingTotals'");
					$m[] = new BMenu("Event List Bookings", "", "href='/events/{$this->event}?cmd=bookingList'");
					$m[] = new BMenu("Event List Bookings Full", "", "href='/events/{$this->event}?cmd=bookingListFull'");
					$m[] = new BMenu("Event List Bookings Spreadsheet", "", "href='/events/{$this->event}?cmd=bookingListSpreadsheet'");
					$m[] = new BMenu("Email all", "", "href='/events/{$this->event}?cmd=emailAll'");

					if($this->eventObj->hasBookingFormExtra){
						$m[] = new BMenu("", "", "");
						$m[] = new BMenu("Event List Extra Bookings", "", "href='/events/{$this->event}?cmd=bookingListExtra'");
						$m[] = new BMenu("Event List Extra Bookings Spreadsheet", "", "href='/events/{$this->event}?cmd=bookingListExtraSpreadsheet'");
					}
					
					$m[] = new BMenu("", "", "");
					$m[] = new BMenu("Edit page", "", "href='?cmd=eventEditInfo'");
					$m[] = new BMenu("Add Page", "", "href='/events/{$this->event}/index.html?cmd=eventAddPage'");
					$m[] = new BMenu("FileManager", "", "href='/lib/FileBrowser/fileBrowser.php?workDir={$this->sys->workDir}'");
					$m[] = new BMenu("Edit report", "", "href='/events/{$this->event}?cmd=eventEditReport'");
					$m[] = new BMenu("Add to Whats' On", "", "href='/events/{$this->event}?cmd=eventAddWhatsOn'");
					$m[] = new BMenu("Create Database Table", "", "href='/events/{$this->event}?cmd=createTable'");
				}
				else {
					$m[] = new BMenu("Edit page", "", "href='?cmd=eventEditInfo'");
					$m[] = new BMenu("Add Page", "", "href='/events/{$this->event}/index.html?cmd=eventAddPage'");
					$m[] = new BMenu("FileManager", "", "href='/lib/FileBrowser/fileBrowser.php?workDir={$this->sys->workDir}'");
					$m[] = new BMenu("Edit report", "", "href='/events/{$this->event}?cmd=eventEditReport'");
					$m[] = new BMenu("Add to Whats' On", "", "href='/events/{$this->event}?cmd=eventAddWhatsOn'");
					$m[] = new BMenu("", "", "");
				}
				$m[] = new BMenu("Help", "",		"href='{$help}'");

				$o = new BMenu("Event", "", "", $m);
			}
		}
		else if($this->checkAccessView()){
			if($this->event != ""){
				$m = array();
				if($this->eventObj->hasBookingForm){
					$m[] = new BMenu("Event Home", "",	"href='/events/{$this->event}/index.html'");
					$m[] = new BMenu("Booking Form", "",	"href='/events/{$this->event}/bookingForm'");
					if($this->eventObj->hasBookingFormExtra){
						$m[] = new BMenu("Booking Form Extra", "",	"href='/events/{$this->event}/bookingFormExtra'");
					}
					$m[] = new BMenu("", "", "");
					$m[] = new BMenu("Event List Totals", "", "href='/events/{$this->event}?cmd=bookingTotals'");
					$m[] = new BMenu("Event List Bookings", "", "href='/events/{$this->event}?cmd=bookingList'");
					$m[] = new BMenu("Event List Bookings Full", "", "href='/events/{$this->event}?cmd=bookingListFull'");
					$m[] = new BMenu("Event List Bookings Spreadsheet", "", "href='/events/{$this->event}?cmd=bookingListSpreadsheet'");
					$m[] = new BMenu("Email all", "", "href='/events/{$this->event}?cmd=emailAll'");
					if($this->eventObj->hasBookingFormExtra){
						$m[] = new BMenu("", "", "");
						$m[] = new BMenu("Event List Extra Bookings", "", "href='/events/{$this->event}?cmd=bookingListExtra'");
						$m[] = new BMenu("Event List Extra Bookings Spreadsheet", "", "href='/events/{$this->event}?cmd=bookingListExtraSpreadsheet'");
					}
				}
				else {
					$m[] = new BMenu("Edit page", "", "href='?cmd=eventEditInfo'");
					$m[] = new BMenu("Add Page", "", "href='/events/{$this->event}?cmd=eventAddPage'");
					$m[] = new BMenu("Edit report", "", "href='/events/{$this->event}?cmd=eventEditReport'");
					$m[] = new BMenu("", "", "");
				}
				$m[] = new BMenu("Help", "",		"href='{$help}'");

				$o = new BMenu("Event", "", "", $m);
			}
			else {
				$o = null;
			}
		}
		else {
			if($this->event != ""){
				if($this->eventObj->hasBookingForm){
					$m = array();
					$m[] = new BMenu("Event Home", "",	"href='/events/{$this->event}/index.html'");
					$m[] = new BMenu("Booking Form", "",	"href='/events/{$this->event}/bookingForm'");
					if($this->eventObj->hasBookingFormExtra){
						$m[] = new BMenu("Booking Form Extra", "",	"href='/events/{$this->event}/bookingFormExtra'");
					}
					$m[] = new BMenu("Help", "",		"href='{$help}'");
					$o = new BMenu("Event", "", "", $m);
				}
				else {
					$m = array();
					$m[] = new BMenu("Event Home", "",	"href='/events/{$this->event}/index.html'");
					$m[] = new BMenu("Help", "",		"href='{$help}'");
					$o = new BMenu("Event", "", "", $m);
				}
			}
			else {
				$o = null;
			}
		}
		return $o;
	}
	
	function eventList($arg){
		$p = new BHtmlNode("div");
		if($this->checkAccess() || $this->checkAccessView()){
			$view = new BViewDatabase("event", "action");
			$view->setView(array(
				new BViewType("id"),
				new BViewType("name"),
				new BViewType("title", "Title", "link", array("\$obj->title", "/events/\$obj->name/index.html")),
				new BViewType("category", "Category"),
				new BViewType("startDate"),
				new BViewType("active"),
				new BViewType("hasBookingForm"),
				new BViewType("hasBookingFormExtra"),
				));

			$view->setLimit(30);

			if($view->order == "")
				$view->setOrder("!startDate");

			$view->setTable("Events");
			
			$p->appendNode("h2", "", "Tandem Club Major Events");
			$p->appendText($view->getHtml());
		}
		else {
			$p->appendNode("h2", "", "Tandem Club Major Events");

			$t = &$p->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");
			
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "class='heading'", "Date");
			$r->appendNode("th", "class='heading'", "Event");

			$list = $this->sys->db->getObjectList("Events", "", "`startDate` > CURDATE()", "!startDate", "");

			foreach($list as $event){
				if($event->active){
					$r = &$t->appendNode("tr");
					$sd = new BDate($event->startDate);
					$ed = new BDate($event->endDate);
					
					$str = $event->title . ". " . $event->description;
					$str .= "<a href='/events/{$event->name}'>Details and booking arrangements</a>";
					$r->appendNode("td", "", $sd->getDateTime("%Y %A %e %B") . " to " . $ed->getDateTime("%A %e %B"));
					$r->appendNode("td", "", $str);
				}
				else{
					$r = &$t->appendNode("tr");
					$sd = new BDate($event->startDate);
					$ed = new BDate($event->endDate);
					
					$str = $event->title . ". " . $event->description;
					$str .= "This is preliminary information, you cannot book yet. details will follow when available.";
					$r->appendNode("td", "", $sd->getDateTime("%Y %A %e %B") . " to " . $ed->getDateTime("%A %e %B"));
					$r->appendNode("td", "", $str);
				}
			}
		}
		
		$p->appendNode("h2", "", "Tandem Club Regional Events");
		$p->appendText(whatsOnList());

		$p->appendNode("h2", "", "Tandem Club Past Events");

		$list = $this->sys->db->getObjectList("Events", "", "`startDate` <= CURDATE()", "!startDate", "");
		if(count($list)){
			$d = &$p->appendNode("div", "style='display:inline-block; overflow:hidden; overflow-y: scroll; height: 200px;'");
			$t = &$d->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

			$th = &$t->appendNode("thead");
			$r = &$th->appendNode("tr");
			$r->appendNode("th", "class='heading'", "Date");
			$r->appendNode("th", "class='heading'", "Past Event");


			$tb = &$t->appendNode("tbody");
			foreach($list as $event){
				if($event->active){
					$r = &$tb->appendNode("tr");
					$sd = new BDate($event->startDate);
					$ed = new BDate($event->endDate);

					$str = $event->title . $event->description;
					$str .= " <a href='/files/events/{$event->name}/report.html'>Report on the event</a>";
					$r->appendNode("td", "", $sd->getDateTime("%Y %A %e %B") . " to " . $ed->getDateTime("%A %e %B"));
					$r->appendNode("td", "", $str);
				}
			}
		}

		$page = $p->render();

		$page .= "<h2>Tandem Club Archive Events</h2>";
		$page .= "<a href='/files/events/old'>Old archived events</a>";

		return $page;
	}
	
	function eventEdit($arg){
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Event</h2>";
			$obj = $this->sys->db->getObject("Events", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Events");
			$obj->name = "tc";
			$obj->email = substr($this->sys->siteEmailAddress, strpos($this->sys->siteEmailAddress, "@"));
			$obj->nonMemberAccess = "non_members_" . makeRandomPassword();
		}

		$view = new BEditObj("event", "Event Information", "?cmd=eventSave", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("name", "Name"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category", "select", categoryGetList("events")),
			new BViewType("description", "Description", "textareaHtml", "", 500, 280),
			new BViewType("email", "Email"),
			new BViewType("startDate", "StartDate", "date"),
			new BViewType("endDate", "EndDate", "date"),
			new BViewType("bookingStartDate", "Booking Start Date", "date"),
			new BViewType("extrasEndDate", "Extras End Date", "date"),
			new BViewType("bookingEndDate", "Booking End Date", "date"),
			new BViewType("active", "Active", "checkbox"),
			new BViewType("hasBookingForm", "Has Booking Form", "checkbox"),
			new BViewType("hasBookingFormExtra", "Has Booking Extra Form", "checkbox"),
			new BViewType("nonMemberAccess", "", ""),
			));

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");
		$view->setHelp("active", "If checked will make event viewable to non admin users.");
		$view->setHelp("hasBookingForm", "If checked will make an event booking form.");
		$view->setHelp("hasBookingFormExtra", "If checked will make an event booking extra form.");

		$view->setObj($obj);
		return $view->getHtml();
	}
	
	function eventSave(){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjEvent();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->eventEdit($obj);
			}

			if($obj->id){
				$this->sys->db->putObject("Events", $obj);
			}
			else {
				$this->sys->db->appendObject("Events", $obj);
				$dir = $this->sys->baseDir . "/files/events/" . $obj->name;

				if(!file_exists($dir . "/index.html")){
					mkdir($dir);

					$p = new BHtmlNode("html");
					$p->appendNode("head")->appendNode("title", "", $obj->title);
					$body = &$p->appendNode("body");
					$body->appendNode("h1", "", $obj->title);
					$body->appendNode("p", "", $obj->description);
					$body->appendNode("a", "href='members/info.html'", "Event information");

					file_put_contents($dir . "/index.html", $p->render());
					file_put_contents($dir . "/report.html", $p->render());
					
					mkdir($dir . "/members");
					symlink("members", "${dir}/{$obj->nonMemberAccess}");
					file_put_contents($dir . "/members/info.html", $p->render());
				}
			}
			$page .= "<h3>Details saved</h3>";
			if($obj->hasBookingForm)
				$page .= "<h3>A booking form will need to be added manually.</h3>";
			if($obj->hasBookingFormExtra)
				$page .= "<h3>A booking extra form will need to be added manually.</h3>";
		}

		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function eventDelete(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("Events", $id);
			$this->sys->db->deleteObject("Events", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("Events", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=eventDelete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function eventEditInfo($uri){
		$module = $this->sys->modules["content"];
		$page = $module->edit("files/events/" . $this->event . "/" . $this->eventArg);
		return $page;
	}
	
	function eventAddPage($uri){
		$module = $this->sys->modules["content"];
		$page = $module->addPage("files/events/" . $this->event . "/index.html");
		return $page;
	}
	
	function eventAddPage1($uri){
		$module = $this->sys->modules["content"];
		$page = $module->addPage1("files/events/" . $this->event . "/index.html");
		return $page;
	}
	
	function eventEditReport($uri){
		$module = $this->sys->modules["content"];
		$page = $module->edit("files/events/" . $this->event . "/report.html");
		return $page;
	}
	
	function save($uri){
		$module = $this->sys->modules["content"];
		return $module->save($uri);
	}

	function eventAddWhatsOn($uri){
		$module = $this->sys->modules["whatsOn"];
		$obj = $this->sys->db->getNewObject("WhatsOn");
		$obj->startDate = $this->eventObj->startDate;
		$obj->endDate = $this->eventObj->endDate;
		$obj->group = "Tandem Club";
		$obj->description = $this->eventObj->title . "<br>" . $this->eventObj->description;
		$obj->web = $this->sys->baseUrl . "/events/" . $this->eventObj->name;
		$page = $module->edit($obj);
	
		return $page;
	}
	
	function bookingTotals($uri){
		$bookingView = new EventBookingForm($this->sys, $this->eventObj);

		if(method_exists($bookingView, "bookingTotals"))
			return $bookingView->bookingTotals($uri);
	
		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='get'");
		$f->appendNode("h3", "", "Event Totals");
		
		$bookings = 0;
		$adults = 0;
		$children = 0;
		$infants = 0;
		$campingAdults = 0;
		$campingChildren = 0;
		$campingUnits = 0;
		$due["Pounds"] = 0;
		$due["Euros"] = 0;
		$paid["Pounds"] = 0;
		$paid["Euros"] = 0;

		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		foreach($data as $obj){
			$camping = 0;
			
			switch($obj->accommodation){
			case "Camping":
			case "Caravan":
			case "MotorHome":
				$camping = 1;
				break;
			}

			for($n = 0; $n < 6; $n++){
				if($obj->{"name$n"} != ""){
					$age = $obj->{"age$n"};
					if($age > 18){
						$adults++;
					}
					else if($age > 1){
						$children++;
					}
					else {
						$infants++;
					}
					if(camping){
						if($age > 6){
							$campingAdults++;
						}
						else if($age > 1){
							$campingChildren++;
						}
					}
				}
			}
			
			if($camping){
				$campingUnits++;
			}
			
			$due["{$obj->currency}"] += $obj->total - $obj->discount;
			$paid["{$obj->currency}"] += $obj->paid;
			
			$bookings++;
		}
		
		$people = $adults + $children + $infants;

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Value");

		$t->appendTableInput("Bookings", "", "view", $bookings, ""); 
		$t->appendTableInput("People", "", "view", $people, ""); 
		$t->appendTableInput("Adults", "", "view", $adults, ""); 
		$t->appendTableInput("Children", "", "view", $children, ""); 
		$t->appendTableInput("Infants", "", "view", $infants, ""); 
		$t->appendTableInput("CampingChildren", "", "view", $campingChildren, ""); 
		$t->appendTableInput("CampingAdults", "", "view", $campingAdults, ""); 
		$t->appendTableInput("CampingUnits", "", "view", $campingUnits, ""); 
		$t->appendTableInput("Due Pounds", "", "view", $due["Pounds"], ""); 
		$t->appendTableInput("Paid Pounds", "", "view", $paid["Pounds"], ""); 
		$t->appendTableInput("Due Euros", "", "view", $due["Euros"], ""); 
		$t->appendTableInput("Paid Euros", "", "view", $paid["Euros"], ""); 

		
		return $f->render();
	}

	function bookingList($uri){
		$obj = new BObjEventBooking();
		$fields = array("id", "name0", "email", "accommodation", "adults", "children", "infants", "campingAdults", "campingChildren", "total", "paid", "paymentId", "addedDate");

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='get'");
		
		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Action");
		foreach($fields as $field){
			$r->appendNode("th", "", ucfirst($field));
		}
		
		foreach($data as $obj){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "", "");
			$c->appendNode("a", "href='?cmd=bookingView&amp;id={$obj->id}' title='View Info Item'", "V");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=bookingEdit&amp;id={$obj->id}' title='Edit Info Item'", "E");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=bookingDelete&amp;id={$obj->id}' title='Delete Info Item'", "D");
			foreach($fields as $field){
				$c = &$r->appendNode("td", "", "");
				if($field == "email")
					$c->appendNode("a", "href='?cmd=email&amp;id={$obj->id}'", $obj->$field);
				elseif($field == "paymentId")
					$c->appendNode("a", "href='/payments?cmd=view&id={$obj->paymentId}'", $obj->$field);
				else
					$c->appendText($obj->$field);
			}
		}

		return $f->render();
	}

	function bookingListExtra($uri){
		$obj = new BObjEventBookingExtra();
		$fields = array("id", "bookingId", "name", "email", "paymentCurrency", "total", "paymentAmountPaid", "modifiedDate");

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='get'");
		
		$data = $this->sys->db->getObjectList($this->bookingTableExtra, "", "`deleted` != 1", "", "");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Action");
		foreach($fields as $field){
			$r->appendNode("th", "", ucfirst($field));
		}
		
		foreach($data as $obj){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "", "");
//			$c->appendNode("a", "href='?cmd=bookingExtraView&amp;id={$obj->id}' title='View Item'", "V");
//			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=bookingExtraEdit&amp;id={$obj->id}' title='Edit Item'", "E");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=bookingExtraDelete&amp;id={$obj->id}' title='Delete Item'", "D");
			foreach($fields as $field){
				$c = &$r->appendNode("td", "", "");
				if($field == "email")
					$c->appendNode("a", "href='mailto:{$obj->email}'", $obj->$field);
				else
					$c->appendText($obj->$field);
			}
		}

		return $f->render();
	}

	function bookingListFull($uri){
		$obj = new BObjEventBooking();
		$fields = $obj->getMemberNames();

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='get'");
		
		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Action");
		foreach($fields as $field){
			$r->appendNode("th", "", ucfirst($field));
		}
		
		foreach($data as $obj){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "", "");
			$c->appendNode("a", "href='?cmd=bookingView&amp;id={$obj->id}' title='View Info Item'", "V");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=bookingEdit&amp;id={$obj->id}' title='Edit Info Item'", "E");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=bookingDelete&amp;id={$obj->id}' title='Delete Info Item'", "D");
			foreach($fields as $field){
				$c = &$r->appendNode("td", "", "");
				$c->appendText($obj->$field);
			}
		}

		return $f->render();
	}
	
	function bookingListRaw($uri){
		$p = new BHtmlNode("html");
		$p->appendNode("head");
		$b = &$p->appendNode("body");

		$obj = new BObjEventBooking();
		$fields = $obj->getMemberNames();
		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		$t = &$p->appendNode("table", "cellpadding='0' cellspacing='0' border='1' style='empty-cells: show;'");

		$r = &$t->appendNode("tr");
		foreach($fields as $field){
			$r->appendNode("th", "", ucfirst($field));
		}
		
		foreach($data as $obj){
			$r = &$t->appendNode("tr");
			foreach($fields as $field){
				$c = &$r->appendNode("td", "", "");
				$c->appendText($obj->$field);
			}
		}
		
		print($p->render());

		return null;
	}

	function bookingListSpreadsheet($uri){
		$obj = new BObjEventBooking();
		$fields = $obj->getMemberNames();

		$data = $this->sys->db->getObjectList($this->bookingTable, "", "`deleted` != 1", "", "");

		header('Content-Type: text/csv; charset=utf-8');
		header('Content-Disposition: attachment; filename=data.csv');

		$line = "";
		foreach($fields as $field){
			if($line != "")
				$line .= ",";
			$v = ucfirst($field);
			$line .= "{$v}";
		}
		print("{$line}\n");
		
		
		foreach($data as $obj){
			$line = "";
			foreach($obj->getMembers() as $m){
				if($line != "")
					$line .= ",";
				if($m->type == "String"){
					$line .= '"' . addressOneLine($obj->{"{$m->name}"}) . '"';
				}
				else {
					$line .= $obj->{"{$m->name}"};
				}
			}
			print("{$line}\n");
		}

		return null;
	}

	function bookingListExtraSpreadsheet($uri){
		$obj = new BObjEventBookingExtra();
		$fields = $obj->getMemberNames();

		$data = $this->sys->db->getObjectList($this->bookingTable . Extra, "", "`deleted` != 1", "", "");

		header('Content-Type: text/csv; charset=utf-8');
		header('Content-Disposition: attachment; filename=data.csv');

		$line = "";
		foreach($fields as $field){
			if($line != "")
				$line .= ",";
			$v = ucfirst($field);
			$line .= "{$v}";
		}
		print("{$line}\n");
		
		
		foreach($data as $obj){
			$line = "";
			foreach($obj->getMembers() as $m){
				if($line != "")
					$line .= ",";
				if($m->type == "String"){
					$line .= '"' . addressOneLine($obj->{"{$m->name}"}) . '"';
				}
				else {
					$line .= $obj->{"{$m->name}"};
				}
			}
			print("{$line}\n");
		}

		return null;
	}

	function bookingView($uri){
		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject($this->bookingTable, filterToInt($_GET["id"]));
		}
		else {
			return "No Object";
		}
		
		$bookingView = new EventBookingForm($this->sys, $this->eventObj);
		
		return $bookingView->view($obj);
	}

	function bookingEdit($uri){
		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject($this->bookingTable, filterToInt($_GET["id"]));
		}
		else {
			return "No Object";
		}
		
		$bookingView = new EventBookingForm($this->sys, $this->eventObj);
		
		return $bookingView->form($obj);
	}

	function bookingExtraEdit($uri){
		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject($this->bookingTableExtra, filterToInt($_GET["id"]));
		}
		else {
			return "No Object";
		}
		
		$bookingView = new EventBookingFormExtra($this->sys, $this->eventObj);
		
		return $bookingView->form($obj);
	}

	function bookingDelete($uri){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject($this->bookingTable, $id);
			$obj->deleted = 1;
			$this->sys->db->putObject($this->bookingTable, $obj);
//			$this->sys->db->deleteObject("Events", $id);
			$page = "<h3>Booking {$obj->id} for {$obj->name0} has been marked as Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject($this->bookingTable, $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=bookingDelete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete booking {$obj->id} for {$obj->name0} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}

	function bookingExtraDelete($uri){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject($this->bookingTableExtra, $id);
			$obj->deleted = 1;
			$this->sys->db->putObject($this->bookingTableExtra, $obj);
//			$this->sys->db->deleteObject($this->bookingTableExtra, $id);
			$page = "<h3>BookingExtras {$obj->id} for {$obj->name} has been marked as Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject($this->bookingTableExtra, $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=bookingExtraDelete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete booking extra {$obj->id} for {$obj->name} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}

	function getEventName($uri){
		if(strpos($uri, "/")){
			$event = substr($uri, strpos($uri, "/") + 1);
			if(strpos($event, "/"))
				$event = substr($event, 0, strpos($uri, "/") + 1);
		}
		else
			$event = "";
		return $event;
	}

	function parseUri($uri){
		parent::parseUri($uri);
		
		$this->event = "";
		$this->eventArg = "";

		if(strpos($uri, "/")){
			$this->event = substr($uri, strpos($uri, "/") + 1);
			$this->event = filterPath($this->event, $this->sys->baseDir);
			if(strpos($this->event, "/")){
				$this->eventArg = substr($this->event, strpos($this->event, "/") + 1);
				$this->event = substr($this->event, 0, strpos($this->event, "/"));
			}
		}
	}
	
	function email($uri){
		if(isset($_POST["sendEmail"])){
			$bookingForm = new EventBookingForm($this->sys, $this->eventObj);

			$files = array();
			$f = new BHtmlNode("dev");

			$emailAddresses = "";
			if($_POST["id"]){
				$obj = $this->sys->db->getObject($this->bookingTable, filterToInt($_POST["id"]));
				$emailAddresses = $obj->email;

				// Add booking form
				if($_POST["bookingForm"]){
					$h = new BHtmlNode("html");
					$b = &$h->appendNode("head");
					$b = &$h->appendNode("body", "style='background-color: #CCDDDD;'");
					$b->appendNode("style", "type='text/css'", "table { empty-cells: show; }");
					$b->appendText($bookingForm->view($obj, 1));
					
					$fname = tempnam("/tmp", "BookingForm.html");
					$file = fopen($fname, "w");
					fwrite($file, $h->render());
					fclose($file);
					$files[] = new EmailFileAttachment("BookingForm.html", "text/html", $fname);
				}
			}
			else {
				// Get email addresses
				$data = $this->sys->db->getObjectList($this->bookingTable, "email", "`email` != '' and `deleted` != 1", "", "");
				foreach($data as $e){
					if($emailAddresses != "")
						$emailAddresses .= ",{$e->email}";
					else
						$emailAddresses .= $e->email;
				}
			}

			foreach($_FILES as $file){
				if($file["name"] != ""){
					$files[] = new EmailFileAttachment($file["name"], $file["type"], $file["tmp_name"]);
				}
  			}
			
			// Send email
			$email = explode("@", $this->sys->siteEmailAddress);
			$this->sys->siteEmailAddress = $bookingForm->name . "@" . $email[1];

			$f->appendNode("p", "", "EmailAddresses: {$emailAddresses}");
			$f->appendNode("p", "", "From: {$this->sys->siteEmailAddress}");
			$r = 0;
			
			$ret = emailList($emailAddresses, $_POST["subject"], "<html><head></head><body>".$_POST["message"]."</body></html>", 1, $files);
			$f->appendNode("h3", "", "Email has been sent");
			$f->appendText($ret);
		}
		else {
			if(isset($_GET["id"]))
				$id = filterToInt($_GET["id"]);
			else
				$id = 0;
				
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$t->appendTableInput("Subject", "subject", "text", "TandemClub: " . ucfirst($this->event) . ": ", "width:500px"); 
			$t->appendTableInput("Message", "message", "textareaHtml", "", "height:300px; width:400px", array("height"=>"400px")); 
			$t->appendTableInput("Attach Booking Form", "bookingForm", "checkbox", "", ""); 
			$t->appendTableInput("Attachment0", "attach0", "file", "", ""); 
			$t->appendTableInput("Attachment1", "attach1", "file", "", ""); 
			$t->appendTableInput("Attachment2", "attach2", "file", "", ""); 
			$t->appendTableInput("Attachment3", "attach3", "file", "", ""); 
			$f->appendNode("input", "type='hidden' name='id' value='{$id}'");
			$f->appendNode("input", "type='hidden' name='MAX_FILE_SIZE' value='3000000'");
			$f->appendNode("input", "type='submit' name='sendEmail' value='Send Email' style='background-color: lightgreen; font-size: 150%;' onclick='this.value=\"Sending ...\";'");
		}

		return $f->render();
	}
	
	function emailAll($uri){
		if(isset($_POST["sendEmail"])){
			$files = array();
			$f = new BHtmlNode("dev");

			foreach($_FILES as $file){
				if($file["name"] != ""){
					$files[] = new EmailFileAttachment($file["name"], $file["type"], $file["tmp_name"]);
				}
  			}
			
			// Get email addresses
			$data = $this->sys->db->getObjectList($this->bookingTable, "email", "`email` != '' and `deleted` != 1", "", "");
			$emailAddresses = "";
			foreach($data as $e){
				if($emailAddresses != "")
					$emailAddresses .= ",{$e->email}";
				else
					$emailAddresses .= $e->email;
			}

			// Send email
			$bookingForm = new EventBookingForm($this->sys, $this->eventObj);
			$email = explode("@", $this->sys->siteEmailAddress);
			$this->sys->siteEmailAddress = $bookingForm->name . "@" . $email[1];

			$f->appendNode("p", "", "EmailAddresses: {$emailAddresses}");
			$f->appendNode("p", "", "From: {$this->sys->siteEmailAddress}");
			$r = 0;
			
			$ret = emailList($emailAddresses, $_POST["subject"], "<html><head></head><body>".$_POST["message"]."</body></html>", 1, $files);
			$f->appendNode("h3", "", "Email has been sent");
			$f->appendText($ret);
		}
		else {
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$t->appendTableInput("Subject", "subject", "text", "TandemClub: " . ucfirst($this->event) . ": ", "width:500px"); 
			$t->appendTableInput("Message", "message", "textareaHtml", "", "height:300px; width:400px", array("height"=>"400px")); 
			$t->appendTableInput("Attachment0", "attach0", "file", "", ""); 
			$t->appendTableInput("Attachment1", "attach1", "file", "", ""); 
			$t->appendTableInput("Attachment2", "attach2", "file", "", ""); 
			$t->appendTableInput("Attachment3", "attach3", "file", "", ""); 
			$f->appendNode("input", "type='hidden' name='MAX_FILE_SIZE' value='3000000'");
			$f->appendNode("input", "type='submit' name='sendEmail' value='Send Email' style='background-color: lightgreen; font-size: 150%;' onclick='this.value=\"Sending ...\";'");
		}

		return $f->render();
	}

	function createTable($uri){
		$res = "";
		if($this->sys->db->getTableInfo($this->bookingTable)){
			$res .= "Booking table already exists</br>";
		}
		else {
			$obj = new BObjEventBooking();
			$res .= $this->sys->modules["management"]->databaseCreateTableForObject($this->bookingTable, $obj, 0);
			$res .= "Created database table: " . $this->bookingTable . "</br>";
		}

		if($this->eventObj->hasBookingFormExtra){
			if($this->sys->db->getTableInfo($this->bookingTableExtra)){
				$res .= "Booking extra table already exists</br>";
			}
			else {
				$obj = new BObjEventBookingExtra();
				$res .= $this->sys->modules["management"]->databaseCreateTableForObject($this->bookingTableExtra, $obj, 0);
				$res .= "Created database table: " . $this->bookingTableExtra . "</br>";
			}
		}
		return $res;
	}


	function createTable1($uri){
		$res = "";
		if($this->sys->db->getTableInfo($this->bookingTable)){
			$res .= "Booking table already exists</br>";
		}
		else {
			$obj = new BObjEventBooking();
			if($obj->dbTableFields() == ""){
				$res .= "Error: No database fields defined for table: " . $this->bookingTable . "</br>";
			}
			else {
				$this->sys->db->createTable($this->bookingTable, $obj->dbTableFields());
			}

			$obj->dbTableSetup($this->bookingTable);
			$res .= "Created database table: " . $this->bookingTable . "</br>";
		}

		if($this->eventObj->hasBookingFormExtra){
			if($this->sys->db->getTableInfo($this->bookingTableExtra)){
				$res .= "Booking extra table already exists</br>";
			}
			else {
				$obj = new BObjEventBookingExtra();
				if($obj->dbTableFields() == ""){
					$res .= "Error: No database fields defined for table: " . $this->bookingTableExtra . "</br>";
				}
				else {
					$this->sys->db->createTable($this->bookingTableExtra, $obj->dbTableFields());
				}

				$obj->dbTableSetup($this->bookingTableExtra);
				$res .= "Created database table: " . $this->bookingTableExtra . "</br>";
			}
		}
		return $res;
	}

	function payComplete($pay){
//		debug_log("Event: payComplete {$pay->reference}");
		$ra = explode(" ", $pay->reference);
		$eventName = $ra[0];
		$bookingId = $ra[1];
		$isExtra = 0;
		
		$eventNameBase = $eventName;
		$pos = strpos($eventName, "Extra");
		if($pos !== False){
			$eventNameBase = substr($eventName, 0, $pos);
			$isExtra = 1;
		}
		
		$list = $this->sys->db->getObjectList("Events", "", "`name` = '{$eventNameBase}'", "", "");
		if(count($list) != 1){
			return "Event booking update: Error reference incorrect: {$pay->reference}";
		}

		$bookingTable = "EventBooking" . ucfirst($eventName);
		$booking = $this->sys->db->getObject($bookingTable, $bookingId);
		if($booking){
			if($isExtra){
//				debug_log("Event: payComplete extras update: {$bookingTable}");
				$booking->paymentId = $pay->id;
				$booking->paymentDate = $pay->datePaid;
				$booking->paymentAmountPaid = $pay->amount;
				$this->sys->db->putObject($bookingTable, $booking);
			}
			else {
//				debug_log("Event: payComplete update: {$bookingTable}\n");
				$booking->paymentId = $pay->id;
				$booking->paid = $pay->amount;
				$this->sys->db->putObject($bookingTable, $booking);
			}
			return "Event booking updated";
		}
		else {
			return "Event booking update: Error reference incorrect: {$pay->reference}";
		}
	}
};

?>
