<?php
/*******************************************************************************
 *	ForSale.php	ForSale Module
 *			T.Barnaby,	BEAM Ltd,	2017-12-29
 *******************************************************************************
 * Testing
 *	Make sure the Nochex account has test transcations enabled by logging in and setting it.
 *	The Payments class has a test variable that then uses this.
 *	Set the "test" variable of the Membership class to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for Full payment test accessing Nochex, but no actual payments being made. Also send emails before payment.
 *	Set to 2 for Completely local testing (No Nochex interaction).
 *	Set to 3 for no payments at all.
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjForSaleEntry extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("userId", "Int", "Number"),
			new BMember("enabled", "Bool"),
			new BMember("sold", "Bool"),
			new BMember("deleted", "Bool"),
			new BMember("name", "String"),
			new BMember("emailAddress", "String"),
			new BMember("telephone", "String"),
			new BMember("location", "String"),
			new BMember("price", "Float", "Float"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("description", "StringHtml"),
			new BMember("picturePresent", "Bool"),
			new BMember("dateAdded", "DateTime"),
			new BMember("dateEnd", "DateTime"),
			new BMember("paymentId", "Int"),
			new BMember("paymentDate", "Date"),
			new BMember("paymentAmount", "Float", "", "", "decimal(8,2)"),
			new BMember("paymentAmountPaid", "Float", "", "", "decimal(8,2)"),
			));
	}
};

class ForSale extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "forSale";
		$this->title = "ForSale";
		$this->info = "ForSale module. This module allows the management of ForSale items.";
		$this->payments = $this->sys->modules["payments"];
		$this->priceMin = 100.0;
		$this->priceMember = 8.0;
		$this->priceNonMember = 16.0;
		$this->test = 0;
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("ForSaleEntrys" => "BObjForSaleEntry"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminForSale"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "forSaleList";
		
		// Create the directory			
		$dir = $this->sys->baseDir . "/files/forSale";
		if(!file_exists($dir)){
			mkdir($dir);
		}
		
		if(!$this->test)
			$this->sys->page->leftObjectInsert($this->getPageBlockRandom());
	}
	
	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("adminEmailList", "StringList", "", "Administration email addresses", "List of administration email addresses");
		return $configs;
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminForSale");
	}

	function checkAccessEntry($obj){
		return $this->checkAccess() || ($this->sys->login->checkLogin() && ($obj->userId == $this->sys->login->id));
	}
	
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("ForSale", "/{$this->name}", "", array("ForSale List" => "/{$this->name}", "ForSale Add" => "/{$this->name}?cmd=add"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->sys->page->aditemsShow = 1;
		$this->parseUri($uri);

		if($this->test){
			$this->sys->status = "<h1><font color='red'>System is in test mode: {$this->test}. No payments will be taken.</font></h1>\n";
		}

		if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "renew":			return $this->renew($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "sold":			return $this->sold($uri);		break;
			case "editList":		return $this->editList($uri);		break;
			case "configEdit":		return $this->configEdit($uri);		break;
			case "clean":			return $this->clean($uri);		break;
			case "statistics":		return $this->statistics($uri);		break;
			case "journalList":		return $this->journalList($uri);	break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else if($this->sys->login->checkLogin()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "renew":			return $this->renew($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "sold":			return $this->sold($uri);		break;
			case "statistics":		return $this->statistics($uri);		break;
			case "journalList":		return $this->journalList($uri);	break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else {
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "journalList":		return $this->journalList($uri);	break;
			default: 			return $this->viewList($uri);		break;
			}
		}
	}

	function getPageBlock($uri){
		$this->parseUri($uri);

		$help = $this->sys->baseUrl . "/modules/forSale/info.html";

		$m = array();
		$m[] = new BMenu("List All For Sale", "", "href='/forSale?cmd=viewList'");
		if($this->sys->login->checkLogin())
			$m[] = new BMenu("Add For Sale Item", "", "href='?cmd=add'");
		$m[] = new BMenu("Help", "",		"href='{$help}'");
		if($this->checkAccess()){
			$m[] = new BMenu("<b>Admin</b>", "", "");
			$m[] = new BMenu("Edit For Sale Items", "", "href='?cmd=editList'");
			$m[] = new BMenu("Configuration", "",	"href='?cmd=configEdit'");
			$m[] = new BMenu("Statistics", "",	"href='?cmd=statistics'");
			$m[] = new BMenu("Journal List", "",	"href='?cmd=journalList'");
			//$m[] = new BMenu("Clean", "",	"href='?cmd=clean'");
		}
		$o = new BMenu("For Sale", "", "", $m);

		return $o;
	}

	function getRandom(){
		// Getlist of picture ID's available
		$dateNow = new BDate("now");
		$where = "(`enabled` = '1' and `deleted` = '0' and `sold` = '0' and `dateEnd` >= '{$dateNow->get()}' and `paymentAmount` = `paymentAmountPaid`)";
		$l = $this->sys->db->queryObjs("ForSaleEntrys", "select id from {$this->sys->databasePrefix}ForSaleEntrys where {$where}");

		if(count($l)){
			// Select a random one		
			$id = $l[rand() % count($l)]->id;
			return $this->sys->db->getObject("ForSaleEntrys", $id);
		}
		else {
			return 0;
		}
	}

	function getPageBlockRandom(){
		$o = new BHtmlNode("div", "");
		if($obj = $this->getRandom()){
			$o->title = "Forsale";
			if($obj->picturePresent)
				$o->appendNode("a", "href='/forSale?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/files/forSale/picture-{$obj->id}-thumb.jpg'");
			else
				$o->appendNode("a", "href='/forSale?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/modules/forSale/TandemForSale-thumb.png'");

			$o->appendNode("div", "class=ForSaleEntryHeader")->appendNode("a", "href='/forSale?cmd=view&id={$obj->id}'", $obj->title);
			//if($obj->location != "")
			//	$c->appendNode("b", "", "Location: {$obj->location}")->appendNode("/br");
			//$c->appendNode("b", "", "Price: £{$obj->price}")->appendNode("/br");
			$o->appendNode("a", "href='/forSale?cmd=viewList'", "List All For Sale");
		}
		return $o;
	}


	function viewList($uri){
		$dateNow = new BDate("now");
		$page = "<h2>Tandems For Sale and other bits</h2>\n";
		$page .= "<p>You can add items for sale using the 'Add For Sale Item' link. You must have a website login account to do this.";
		$page .= " TC Members get an account by default (email us if you don't have this <a href='mailto:{$this->sys->siteEmailAdmin}'>{$this->sys->siteEmailAdmin}</a>). Non members can create an account using the 'Create New Account' link in the left hand menu.";
		$page .= " You can enable/disable an email to be sent to you when new adverts are published. The 'Send Forsale Emails' option in your User Preferences allows this.";
		$page .= " There is a charge for items above £{$this->priceMin} paid for using a credit or debit card.";
		$page .= " If you set this to 0, then the item will be listed as \"Free to a good home\".";
		$page .= " Always be carefull of fraudsters.";
		$page .= " See our help information <a href='/modules/forSale/info.html'>here</a> for more information including safe selling.";
		$page .= "</p>";

		//$where = "(`enabled` = '1' and `dateEnd` >= '{$dateNow->get()}' and `paymentAmountPaid` >= `paymentAmount` and `deleted` != '1' and `sold` != '1')";
		$where = "(`enabled` = '1' and `dateEnd` >= '{$dateNow->get()}' and `deleted` != '1' and `sold` != '1')";
		if(isset($_GET["start"]))
			$start = intval($_GET["start"]);
			
		$pager = new BHtmlPager(32, $this->sys->db->getObjectNumber("ForSaleEntrys", ""));

		$h = new BHtmlNode("table", "class=ForSale");
		$list = $this->sys->db->getObjectList("ForSaleEntrys", "", $where, "!dateAdded", $pager->getRange());
		foreach($list as $obj){
			$notPaid = 0;
			if($obj->paymentAmount > $obj->paymentAmountPaid){
				if($this->checkAccessEntry($obj)){
					$notPaid = 1;
				}
				else {
					continue;
				}
			}
		
			$dateEnd = new DateTime("$obj->dateEnd");
			$dateEnd = $dateEnd->format("l jS F Y");
			
			$r = &$h->appendNode("tr");
			$c = &$r->appendNode("td");
			if($obj->picturePresent){
				$c->appendNode("a", "href='/forSale?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/files/forSale/picture-{$obj->id}-thumb.jpg'");
			}
			else {
				$c->appendNode("a", "href='/forSale?cmd=view&id={$obj->id}'")->appendNode("img", "src='{$this->sys->baseUrl}/modules/forSale/TandemForSale-thumb.png'");
			}
			$c = &$r->appendNode("td");
			$c->appendNode("div", "class=ForSaleEntryHeader")->appendNode("a", "href='/forSale?cmd=view&id={$obj->id}'", $obj->title);
			if($notPaid){
				$c->appendNode("div", "class=error", "This advert has yet to be paid for so will not be visible to others. Please edit it as needed and you will then be taken through the payment process.");
			}
			
			if($obj->category != "")
				$c->appendNode("span", "class=ForSaleEntryLine", "Category: {$obj->category}");
			if($obj->location != "")
				$c->appendNode("b", "", "Location: {$obj->location}")->appendNode("/br");
			//$c->appendNode("b", "", "Listing ends on: {$dateEnd}")->appendNode("/br");
			if($obj->price)
				$c->appendNode("b", "", "Price: £{$obj->price}")->appendNode("/br");
			else
				$c->appendNode("b", "", "Price: Free to a good home")->appendNode("/br");

			//$c->appendNode("span", "class=ForSaleEntryLine", $obj->description);
		}
		$page .= $h->render();

		return $page;
	}

	function journalList($uri){
		$dateNow = new BDate("now");
		$where = "(`enabled` = '1' and `dateEnd` >= '{$dateNow->get()}' and `paymentAmountPaid` >= `paymentAmount` and `deleted` != '1' and `sold` != '1' and `price` >= '100')";
			
		$h = new BHtmlNode("table", "class=ForSale width='100%'");
		$list = $this->sys->db->getObjectList("ForSaleEntrys", "", $where, "!dateAdded");
		foreach($list as $obj){
			$dateEnd = new DateTime("$obj->dateEnd");
			$dateEnd = $dateEnd->format("l jS F Y");
			
			$r = &$h->appendNode("tr");
			$c = &$r->appendNode("td");
			if($obj->picturePresent){
				$c->appendNode("img", "style='max-width: 100%;' src='{$this->sys->baseUrl}/files/forSale/picture-{$obj->id}.jpg'");
				//$c->appendNode("img", "width='250px' src='{$this->sys->baseUrl}/files/forSale/picture-{$obj->id}.jpg'");
			}
			$c = &$r->appendNode("td");
			$c->appendNode("b", "", $obj->title);
			$c->appendNode("/br");
			if($obj->category != "")
				$c->appendNode("span", "class=ForSaleEntryLine", "Category: {$obj->category}");
			if($obj->location != "")
				$c->appendNode("b", "", "Location: {$obj->location}")->appendNode("/br");
			//$c->appendNode("b", "", "Listing ends on: {$dateEnd}")->appendNode("/br");
			if($obj->price)
				$c->appendNode("b", "", "Price: £{$obj->price}")->appendNode("/br");
			else
				$c->appendNode("b", "", "Price: Free to a good home")->appendNode("/br");

			if($obj->emailAddress != "")
				$c->appendNode("b", "", "Email: ")->appendNode("a", "href='mailto:{$obj->emailAddress}'", "{$obj->emailAddress}")->appendNode("/br");
			if($obj->telephone != "")
				$c->appendNode("b", "", "Telephone: {$obj->telephone}")->appendNode("/br");

			$c->appendNode("span", "class=ForSaleEntryLine", $obj->description);
		}
		$page .= $h->render();

		print $page;
		return Null;
	}

	function editList($uri){
		$page = "<h2>For Sale</h2>\n";
		$dateNow = new BDate("now");

		if(isset($_GET["start"]))
			$start = intval($_GET["start"]);
		
		$where = "";	
		//$where = "`dateEnd` >= '{$dateNow->get()}'";

		$sel = new BSelect("select", "Select", "?cmd=editList", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("period", "dateAdded", "Period", "period", "select", array("", "Last Year", "Last 6 Months", "Last 2 Months", "Last Month", "Last Week")),
			new BViewTypeSelect("startDate", "dateAdded", "Startdate", "greaterThan", "date"),
			new BViewTypeSelect("endDate", "dateAdded", "EndDate", "lessThanEqual", "date"),
			new BViewTypeSelect("enabled", "", "", "equal", "tristate"),
			new BViewTypeSelect("sold", "", "", "equal", "tristate"),
			new BViewTypeSelect("deleted", "", "", "equal", "tristate"),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			));

		$sel->getObj()->period = "Last Month";
		$sel->getObj()->deleted = 0;
		$sel->pagerSet(0, 50, 0);
		$sel->update();

		$where = $sel->getSelectString();

		$view = new BViewList("list", "selectAction");
		$view->setView(array(
			new BViewType("id", "ID"),
			new BViewType("userId", "UserId"),
			new BViewType("enabled", "Enabled"),
			new BViewType("deleted", "Deleted"),
			new BViewType("sold", "Sold"),
			new BViewType("emailAddress", "Email"),
			new BViewType("price", "Price"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category"),
			new BViewType("picturePresent", "Picture"),
			new BViewType("dateAdded", "Date Published", "datetime"),
			new BViewType("dateEnd", "Date End", "datetime"),
			//new BViewType("description", "Description"),
			new BViewType("paymentId", "PaymentId"),
			new BViewType("paymentDate", "PaymentDate"),
			new BViewType("paymentAmount", "Payment"),
			new BViewType("paymentAmountPaid", "Paid")
			));
			
		$list = $this->sys->db->getObjectList("ForSaleEntrys", "", $where, "!dateAdded", $sel->pagerGetLimitString());
		$totalNumber = $this->sys->db->getObjectNumber("ForSaleEntrys", $where);
		$sel->pagerSet(0, 50, $totalNumber);
		$view->setList($list, 1);

		$page = $page . $sel->getHtml();
		$page = $page . $view->getHtml();

		return $page;
	}

	function view($uri){
		// Information
		$page = new BHtmlNode("div");
		$div = &$page->appendNode("div", "class=ForSale style='float:left'");

		if(isset($_GET["id"]) && $obj = $this->sys->db->getObject("ForSaleEntrys", filterToInt($_GET["id"]))){
			$notPaid = 0;
			if($obj->paymentAmount > $obj->paymentAmountPaid){
				if($this->checkAccessEntry($obj)){
					$notPaid = 1;
				}
			}

			if($obj->enabled && !$obj->deleted && ($notPaid || ($obj->paymentAmountPaid >= $obj->paymentAmount))){
				$dateEnd = new DateTime("$obj->dateEnd");
				$dateEnd = $dateEnd->format("l jS F Y");

				$div->appendNode("h2", "", "For Sale: $obj->title");
				if($notPaid){
					$div->appendNode("div", "class=error", "This advert has yet to be paid for so will not be visible to others.  Please edit it as needed and you will then be taken through the payment process.");
				}
				$div->appendNode("b", "", "By: {$obj->name}")->appendNode("/br");
				if($obj->emailAddress != "")
					$div->appendNode("b", "", "Email: ")->appendNode("a", "href='mailto:{$obj->emailAddress}'", "{$obj->emailAddress}")->appendNode("/br");
				if($obj->telephone != "")
					$div->appendNode("b", "", "Telephone: {$obj->telephone}")->appendNode("/br");
				if($obj->location != "")
					$div->appendNode("b", "", "Location: {$obj->location}")->appendNode("/br");
				$div->appendNode("b", "", "Listing ends on: {$dateEnd}")->appendNode("/br");
				if($obj->price)
					$div->appendNode("b", "", "Price: £{$obj->price}")->appendNode("/br");
				else
					$div->appendNode("b", "", "Price: Free to a good home")->appendNode("/br");
				$div->appendNode("p", "", $obj->description);
				if($obj->picturePresent){
					$div->appendNode("img", "src='{$this->sys->baseUrl}/files/forSale/picture-{$obj->id}.jpg'");
				}

				if($this->checkAccessEntry($obj)){
					$buttons = $div->appendNode("div");
					$buttons->appendNode("button", "type='button' name='but1' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=sold&id={$obj->id}\";'", "Sold");
					$buttons->appendNode("button", "type='button' name='but2' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=edit&id={$obj->id}\";'", "Edit");
					$buttons->appendNode("button", "type='button' name='but3' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=renew&id={$obj->id}\";'", "Renew");
					$buttons->appendNode("button", "type='button' name='but4' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"?cmd=delete&id={$obj->id}\";'", "Delete");
				}
			}
			else {
				$div->appendNode("h2", "", "Advert no longer available");
			}
		}

		return $page->render();
	}
	
	function edit($uri){
		$page = "";
		$edit = 0;	

		if(is_object($uri)){
			$page = "<h2>Edit ForSale</h2>";
			$obj = $uri;
			$edit = 1;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit ForSale</h2>";
			$obj = $this->sys->db->getObject("ForSaleEntrys", filterToInt($_GET["id"]));
			$edit = 1;
		}
		else {
			$dateNow = new BDate("now");
			$obj = $this->sys->db->getNewObject("ForSaleEntrys");
			$obj->dateAdded = $dateNow->get();
			$dateNow->timeStamp += 62 * 24 * 3600;
			$obj->dateEnd = $dateNow->get();
			$obj->price = $this->priceMin;
			$obj->paymentAmount = $this->priceNonMember;

			if($this->sys->user){
				$obj->userId = $this->sys->user->id;
				$obj->enabled = 1;
				$obj->name = $this->sys->user->name;
				$obj->location = $this->sys->user->city;
				$obj->emailAddress = $this->sys->user->emailAddress;
				$obj->telephone = $this->sys->user->telephoneHome;
				if($this->sys->user->membershipId)
					$obj->paymentAmount = $this->priceMember;
			}

			$page = "<h2>Add a For Sale item</h2>\n";
			$page .= "<p>The advert will appear for 2 months. There is a charge for items above £100 paid for using a credit or debit card.";
			$page .= " Please enter a realistic price for the item in pounds stirling.";
			$page .= " If you set this to 0, then the item will be listed as \"Free to a good home\".";
			$page .= " You will be taken through the payment system once you save the advert details.";
			$page .= " The charge is £{$this->priceMember} for Tandem Club members and £{$this->priceNonMember} for others.";
			$page .= " You can provide a picture if wanted. You can edit the advert later if needed.";
			$page .= "</p>";
		}

		$view = new BEditObj("ForSale", "ForSale Information", "?cmd=save", "");
		if($this->checkAccess()){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("title", "Item"),
				new BViewType("enabled", "", "checkbox"),
				new BViewType("deleted", "", "checkbox"),
				new BViewType("sold", "", "checkbox"),
				new BViewType("price", "Price (just number)"),
				new BViewType("name"),
				new BViewType("emailAddress"),
				new BViewType("telephone"),
				new BViewType("location"),
				//new BViewType("category", "Category", "select", categoryGetList("ForSale")),
				new BViewType("description", "Description", "textareaHtml", "", 500, 280),
				//new BViewType("description", "Description", "text"),
				new BViewType("dateAdded"),
				new BViewType("dateEnd"),
				new BViewType("paymentId"),
				new BViewType("paymentDate"),
				new BViewType("paymentAmount"),
				new BViewType("paymentAmountPaid"),
				new BViewType("picturePresent", "", "checkbox"),
				new BViewType("picture", "", "pictureUploaderLineMedium"),
				));
		}
		else {
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("enabled", "", "hidden"),
				new BViewType("deleted", "", "hidden"),
				new BViewType("sold", "", "hidden"),
				new BViewType("title"),
				new BViewType("price", "Price (just number)"),
				new BViewType("name"),
				new BViewType("emailAddress"),
				new BViewType("telephone"),
				new BViewType("location"),
				//new BViewType("category", "Category", "select", categoryGetList("ForSale")),
				new BViewType("description", "Description", "textareaHtml", "", 500, 280),
				new BViewType("dateAdded", "", "hidden"),
				new BViewType("dateEnd", "", "hidden"),
				new BViewType("paymentId", "", "hidden"),
				new BViewType("paymentDate", "", "hidden"),
				new BViewType("paymentAmount", "", "hidden"),
				new BViewType("paymentAmountPaid", "", "hidden"),
				new BViewType("picturePresent", "", "hidden"),
				new BViewType("picture", "", "pictureUploaderLineMedium"),
				));
		}
		
		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save($uri){
		$page = "";
		$editing = 0;

		//print_r($_GET);	print_r($_POST); print_r($_FILES);
		//BDatabase::$debug = 1;
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjForSaleEntry();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			if(floatVal($obj->price) >= $this->priceMin){
				if($this->sys->user && $this->sys->user->membershipId){
					$paymentAmount = $this->priceMember;
				}
				else {
					$paymentAmount = $this->priceNonMember;
				}
				if($paymentAmount > $obj->paymentAmountPaid){
					$obj->paymentAmount = $paymentAmount;
				}
			}
			else {
				$obj->paymentAmount = 0.0;
			}
			
			if((count($_FILES) == 1) && isset($_FILES["picture"])){
				$obj->picturePresent = 1;
			}
			
			if($obj->id){
				$this->sys->db->putObject("ForSaleEntrys", $obj);
				$editing = 1;
			}
			else {
				$this->sys->db->appendObject("ForSaleEntrys", $obj);
				$this->statisticsAdd("total", 1);
			}

			// Upload any pictures present
			if((count($_FILES) == 1) && isset($_FILES["picture"])){
				$file = $_FILES["picture"];
				//debug_log("uploadPictures: Do file: {$file["tmp_name"]}\n");

				// Add picture
				$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
				pictureSaveDir($file["tmp_name"], $ext, "/files/forSale", $obj->id, 192);
			}

			$page .= "<h3>ForSale details saved</h3>";

			if($obj->paymentAmount > $obj->paymentAmountPaid){
				$page .= "This advert will cost: £{$obj->paymentAmount} and will last for 2 months on our website. ";

				$h = new BHtmlNode("div");
				$payment = $this->payments->payGetNew();
				$payment->type = "card";
				$payment->source = "forSale";
				$payment->reference = $obj->id;
				$payment->name = $obj->name;
				$payment->address = $this->sys->user->postalAddress;
				$payment->city = $this->sys->user->city;
				$payment->postcode = $this->sys->user->postcode;
				$payment->country = $this->sys->user->country;
				$payment->telephone = $obj->telephone;
				$payment->emailAddress = $obj->emailAddress;
				$payment->amount = $obj->paymentAmount;
				$payment->currency = "Pounds";
				$payment->title = $obj->title;
				$payment->test = $this->test;

				if($this->test == 3){
					$h->appendNode("p", "", "<b>*** Warning: Payments disabled as in test mode 3 ***</b>");
				}
				else {
					$payment->type = "card_membership";
					$h->appendText($this->payments->payDo($payment));
				}
				// Set paymentId
				$obj->paymentId = $payment->id;
				$this->sys->db->putObject("ForSaleEntrys", $obj);

				$page .= $h->render();

				$page .= "<p>Always be carefull of fraudsters.";
				$page .= " See our help information <a href='/modules/forSale/info.html'>here</a> for more information including safe selling.</p>";
				
				if($this->test == 2){
					$page .= "Test2 emails sent";
					$this->sendEmails($obj);
				}
			}

			if($editing)
				$subject = "{$this->sys->siteTitle}: Admin: For Sale Item Edited";
			else
				$subject = "{$this->sys->siteTitle}: Admin: For Sale Item Added";
			$message = $subject . "\n";
			$message .= "Name:        {$obj->name}\n";
			$message .= "Email:       {$obj->emailAddress}\n";
			$message .= "Price:       {$obj->price}\n";
			$message .= "View: {$this->sys->baseUrl}/forSale?cmd=view&id={$obj->id}\n";
			$message .= "Edit: {$this->sys->baseUrl}/forSale?cmd=edit&id={$obj->id}\n";

			if($this->test)
				emailList("terry@beam.ltd.uk", $subject, $message);
			else
				emailList(configGet("forSale.adminEmailList"), $subject, $message);
		}

		return $page;
	}
	
	function payComplete($pay){
//		debug_log("Event: payComplete {$pay->reference}");
		$id = intval($pay->reference);
		
		$obj = $this->sys->db->getObject("ForSaleEntrys", $id);
		if($obj){
//			debug_log("Event: payComplete update: {$bookingTable}\n");
			$obj->paymentId = $pay->id;
			$obj->paymentDate = $pay->datePaid;
			$obj->paymentAmountPaid = $pay->amount;
			$this->sys->db->putObject("ForSaleEntrys", $obj);

			// Send emails to all who are interested
			if(!$editing)
				$this->sendEmails($obj);

			return "ForSale booking updated";
		}
		else {
			return "ForSale update: Error reference incorrect: {$pay->reference}";
		}
	}

	function renew($uri){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("ForSaleEntrys", $id);

		// Check role
		if(!$this->checkAccessEntry($obj)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to renew this item."));
			exit();
		}

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$dateEnd = new BDate($obj->dateEnd);
			$dateEnd->timeStamp += 62 * 24 * 3600;
			$obj->dateEnd = $dateEnd->get();

			$h = new BHtmlNode("div");
			if($obj->paymentAmount){
				$payment = $this->payments->payGetNew();
				$payment->type = "card";
				$payment->source = "forSale";
				$payment->reference = $obj->id;
				$payment->name = $obj->name;
				$payment->address = $this->sys->user->postalAddress;
				$payment->city = $this->sys->user->city;
				$payment->postcode = $this->sys->user->postcode;
				$payment->country = $this->sys->user->country;
				$payment->telephone = $obj->telephone;
				$payment->emailAddress = $obj->emailAddress;
				$payment->amount = $obj->paymentAmount;
				$payment->currency = "Pounds";
				$payment->title = $obj->title;
				$payment->test = $this->test;

				if($this->test == 3){
					$h->appendNode("p", "", "<b>*** Warning: Payments disabled as in test mode 3 ***</b>");
				}
				else {
					$payment->type = "card_membership";
					$h->appendText($this->payments->payDo($payment));
				}
				// Set paymentId
				$obj->paymentId = $payment->id;
			}
			$this->sys->db->putObject("ForSaleEntrys", $obj);

			$page .= $h->render();
			$page .= "<h3>{$obj->title} will be renewed for an extra 2 months when paid if this is needed.</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=renew&id={$id}&doIt=1' method='post'");
			$page = "<h2>Renew a For Sale item</h2>\n";
			$page .= "<p>The advert will appear for an extra 2 months. There is a charge for items above £100 paid for using a credit or debit card.";
			$page .= " You will be taken through the payment system once you agree to renew.";
			$page .= " The charge is £{$this->priceMember} for Tandem Club members and £{$this->priceNonMember} for others.";
			$page .= " You can provide a picture if wanted. You can edit the advert later if needed.";
			$page .= "</p>";
			$f->appendText($page);
			$f->appendNode("h3", "", "Do you really want to renew for an extra 2 months {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Renew'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
		}
			
		return $page;
	}
	
	function sold($uri){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("ForSaleEntrys", $id);

		// Check role
		if(!$this->checkAccessEntry($obj)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to modify this item."));
			exit();
		}
		$obj->sold = 1;
		$this->sys->db->putObject("ForSaleEntrys", $obj);
		$this->statisticsAdd("sold", 1);
			
		return "For sale item set to sold.";
	}

	function delete($uri){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("ForSaleEntrys", $id);

		// Check role
		if(!$this->checkAccessEntry($obj)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete this item."));
			exit();
		}

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("ForSaleEntrys", $id);
			
			$filePath = $this->sys->baseDir . "/files/forSale/picture-{$id}";
			if(file_exists($filePath . ".jpg"))
				unlink($filePath . ".jpg");
			if(file_exists($filePath . "-thumb.jpg"))
				unlink($filePath . "-thumb.jpg");

			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"/forSale\"'>Back to list</button>";
		}
		else {
			$obj->deleted = 1;
			$this->sys->db->putObject("ForSaleEntrys", $obj);

			$page = "For sale item deleted";
		}
			
		return $page;
	}

	function sendEmails($obj){
		if($obj){
			// Send an email to everyone interested in receiving emails
			$subject = "{$this->sys->siteTitle}: Forsale post: {$obj->title}";
			
			$message = $obj->title . "</br>\n";
			if($obj->location != "")
				$message .= "Location: {$obj->location}</br>\n";
			$message .= "Price: £{$obj->price}</br>\n";
			$message .= "{$obj->description}</br>\n\n";
			$message .= "To the Forsale item: <a href='{$this->sys->baseUrl}/forSale?cmd=view&id={$obj->id}'>{$this->sys->baseUrl}/forSale?cmd=view&id={$obj->id}</a></br>\n\n";
			if($obj->emailAddress != "")
				$message .= "Contact email: <a href='mailto:{$obj->emailAddress}'>{$obj->emailAddress}</a>\n\n";
			
			$objList = $this->sys->db->getObjectList("User", "emailAddress", "`forsaleEmailSend` = '1'", "");

			$emailAddressList = array();
			foreach($objList as $o){
				$emailAddressList[] = $o->emailAddress;
			}
			
			$noreply = preg_replace("/^.*@/", "roreply@", $this->sys->siteEmailAddress);
			if($this->test)
				return emailListFromReply($this->sys->siteEmailAddress, $noreply, "terry@beam.ltd.uk", $subject, $message, 1);
			else
				return emailList($emailAddressList, $subject, $message, 1);
		}
	}


	function parseUri($uri){
		parent::parseUri($uri);
	}
	
	function clean(){
		$date = new DateTime("now");
		$date->sub(new DateInterval("P1Y"));
		$dateStr = $date->format("Y-m-d H:i:s");
		
		print "Date: {$dateStr}<br>";
		
		$where = "`dateAdded` < '{$dateStr}'";
		$list = $this->sys->db->getObjectList("ForSaleEntrys", "id", $where);
		foreach($list as $obj){
			print "{$obj->id}<br>";
			$this->sys->db->deleteObject("ForSaleEntrys", $obj->id);
			
			$filePath = $this->sys->baseDir . "/files/forSale/picture-{$obj->id}";
			if(file_exists($filePath . ".jpg"))
				unlink($filePath . ".jpg");
			if(file_exists($filePath . "-thumb.jpg"))
				unlink($filePath . "-thumb.jpg");
		}
	}
	
	function statisticsAdd($item, $value){
		$this->sys->modules["statistics"]->counterAddNow($this->name, $item, $value);
		
		// Total
		$dateNow = new BDate("now");
		$where = "(`enabled` = '1' and `dateEnd` >= '{$dateNow->get()}' and `paymentAmount` = `paymentAmountPaid` and `deleted` != '1' and `sold` != '1')";
		$value = $this->sys->db->getObjectNumber("ForSaleEntrys", $where);
		$this->sys->modules["statistics"]->counterSetNow($this->name, "current", $value);
	}

	function statistics(){

		$f = new BHtmlNode("div");
		$f->appendNode("h3", "", "ForSale Statistics");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Item");
		$r->appendNode("th", "align='left'", "Value");
		
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Current forSale");
		$dateNow = new BDate("now");
		$where = "(`enabled` = '1' and `dateEnd` >= '{$dateNow->get()}' and `paymentAmount` = `paymentAmountPaid` and `deleted` != '1' and `sold` != '1')";
		$value = $this->sys->db->getObjectNumber("ForSaleEntrys", $where);
		$r->appendNode("td", "", $value);

		// Statistics counters
		$f->appendNode("h3", "", "ForSale Statistics Counters");

		$date = new BDate("now");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Year");
		$r->appendNode("th", "align='left'", "Month");
		$r->appendNode("th", "align='left'", "Item");
		$r->appendNode("th", "align='left'", "Number");
		
//		$list = $this->sys->modules["statistics"]->counterGetAll($date->getYear(), "%", $this->name, "%");
		$list = $this->sys->modules["statistics"]->counterGetAll("%", "%", $this->name, "%");
		foreach($list as $obj){
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $obj->year);
			$r->appendNode("td", "", $obj->month);
			$r->appendNode("td", "", $obj->item);
			$r->appendNode("td", "", $obj->number);
		}

		return $f->render();
	}
};

function forSaleList($num=3){
	global	$sys;
	
	$where = "";

	$objList = $sys->db->getObjectList("ForSaleEntrys", "", $where, "!dateAdded", "{$num}");

	$html = new BHtmlNode("div", "id='news'");

	foreach($objList as $obj){
		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateTimeString($obj->dateAdded));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/forSale?cmd=view&id={$obj->id}'", $obj->title);
//		$d1->appendNode("div", "class='NewsSummaryCategory'", "Category: General");
//		$d1->appendNode("div", "class='NewsSummaryAuthor'", "Posted by: terry");
		$d1->appendNode("div", "class='NewsSummaryContent'", $obj->description);
	}

	return $html->render();
}

?>
