<?php
/*******************************************************************************
 *	Forum.php	Forum Module
 *			T.Barnaby,	BEAM Ltd,	2014-11-09
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjForum extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("name", "String", "StringNoSpaces"),
			new BMember("title", "String"),
			new BMember("type", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "String"),
			new BMember("blog", "Int"),
			new BMember("order", "Int"),
			new BMember("dateAdded", "DateTime"),
			));
	}
};

class BObjForumEntry extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("parentId", "Int", "Number"),
			new BMember("grandparentId", "Int", "Number"),
			new BMember("forum", "String"),
			new BMember("userId", "Int", "Number"),
			new BMember("userName", "String"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "StringHtml"),
			new BMember("picturePresent", "Bool"),
			new BMember("date", "DateTime"),
			new BMember("dateChildren", "DateTime"),
			new BMember("dateAdded", "DateTime"),
			new BMember("emailAddress", "String"),
			new BMember("oldAdmin", "String"),
			));
	}
};

class Forum extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "forum";
		$this->title = "Forum";
		$this->info = "Forum module. This module allows the management of Forum items.";
		$this->group = "";
		$this->objId = 0;
		$this->test = 0;
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("Forums" => "BObjForum"));
		$this->sys->db->addObjectClasses(array("ForumEntrys" => "BObjForumEntry"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminForum"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "forumList";
		
		// Create the directory			
		$dir = $this->sys->baseDir . "/files/forum";
		if(!file_exists($dir)){
			mkdir($dir);
		}
	}

	function sessionRead(){
		if(isset($_SESSION["forum.objId"]))
			$this->objId = $_SESSION["forum.objId"];
	}
	function sessionWrite(){
		$_SESSION["forum.objId"] = $this->objId;
	}
	
	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminForum");
	}

	function checkAccessEntry($obj){
		return $this->checkAccess() || ($this->sys->login->checkLogin() && ($obj->userId == $this->sys->login->id));
	}
	
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Forum", "/{$this->name}", "", array("Forum List" => "/{$this->name}", "Forum Add" => "/{$this->name}?cmd=add&group={$this->group}"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->sys->page->aditemsShow = 1;
		$this->parseUri($uri);
		$this->sessionRead();

		if($this->test){
			$this->sys->status = "<h1><font color='red'>System is in test mode: {$this->test}. Added entries will be lost.</font></h1>\n";
		}

		if($this->checkAccess()){
			switch($this->cmd){
			case "view":			$r = $this->view($uri);		break;
			case "viewList":		$r = $this->viewList($uri);	break;
			case "editList":		$r = $this->editList($uri);	break;
			case "add":			$r = $this->edit($uri);		break;
			case "edit":			$r = $this->edit($uri);		break;
			case "save":			$r = $this->save($uri);		break;
			case "delete":			$r = $this->delete($uri);	break;
			case "entryReply":		$r = $this->entryReply($uri);	break;
			case "entryAdd":		$r = $this->entryEdit($uri);	break;
			case "entryEdit":		$r = $this->entryEdit($uri);	break;
			case "entryAppend":		$r = $this->entryAppend($uri);	break;
			case "entrySave":		$r = $this->entrySave($uri);	break;
			case "entryDelete":		$r = $this->entryDelete($uri);	break;
			default: 			$r = $this->view($uri);		break;
			}
		}
		else if($this->sys->login->checkLogin()){
			switch($this->cmd){
			case "view":			$r = $this->view($uri);		break;
			case "viewList":		$r = $this->viewList($uri);	break;
			case "entryReply":		$r = $this->entryReply($uri);	break;
			case "entryAdd":		$r = $this->entryEdit($uri);	break;
			case "entryEdit":		$r = $this->entryEdit($uri);	break;
			case "entryAppend":		$r = $this->entryAppend($uri);	break;
			case "entrySave":		$r = $this->entrySave($uri);	break;
			case "entryDelete":		$r = $this->entryDelete($uri);	break;
			default: 			$r = $this->view($uri);		break;
			}
		}
		else {
			switch($this->cmd){
			case "view":			$r = $this->view($uri);		break;
			case "viewList":		$r = $this->viewList($uri);	break;
			default: 			$r = $this->view($uri);		break;
			}
		}

		$this->sessionWrite();
		
		return $r;
	}

	function getPageBlock($uri){
		$this->parseUri($uri);
		if($this->forumName != ""){
			$forum = $this->sys->db->getObjectFirst("Forums", "", "`name` = '$this->forumName'");
			$type = $forum->type;
		}
		else {
			$type = "all";
		}


		$help = $this->sys->baseUrl . "/modules/forum/info.html";

		$m = array();
		if($this->sys->login->checkLogin()){
			if(($type == "forum") && ($this->forumName != "All")){
				$m[] = new BMenu("List Discussions", "", "href='?cmd=view'");
				$m[] = new BMenu("Start new discussion", "", "onmouseover='this.href = \"?cmd=entryAdd\"'");
				$m[] = new BMenu("Post to this discussion", "", "onmouseover='this.href = \"?cmd=entryAppend\"'");
			}
		}
		$m[] = new BMenu("<b>Forums</b>", "", "");
		$m[] = new BMenu("List All Forums", "", "href='/forum?cmd=viewList'");
		if($this->checkAccess()){
			$m[] = new BMenu("Edit forums", "", "href='?cmd=editList'");
			$m[] = new BMenu("Add a new Forum", "", "href='?cmd=add'");
		}
		$m[] = new BMenu("Help", "",		"href='{$help}'");
		$o = new BMenu("Forum", "", "", $m);

		return $o;
	}

	function viewList($uri){
		$page = "<h2>Forums</h2>\n";

		if(isset($_GET["start"]))
			$start = intval($_GET["start"]);
			
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->group) . "'";
		else
			$where = "";
		
		$pager = new BHtmlPager(32, $this->sys->db->getObjectNumber("Forums", ""));

		$h = new BHtmlNode("div", "class=Forum");
		$list = $this->sys->db->getObjectList("Forums", "", "", "!order,name", $pager->getRange());
		foreach($list as $obj){
			$line = &$h->appendNode("div", "class=ForumEntry");
			$line->appendNode("span", "class=ForumEntryHeader")->appendNode("a", "href='/forum/{$obj->name}'", $obj->title);
			//$line->appendNode("span", "class=ForumEntryLine", "Type: {$obj->type}");
			if($obj->group != "")
				$line->appendNode("span", "class=ForumEntryLine", "Group: {$obj->group}");
			if($obj->category != "")
				$line->appendNode("span", "class=ForumEntryLine", "Category: {$obj->category}");
			$line->appendNode("span", "class=ForumEntryLine", $obj->description);
		}
		$page .= $h->render();

		$page .= $pager->getHtml("?cmd=viewList");

		return $page;
	}

	function editList($uri){
		$page = "<h2>Forums</h2>\n";

		if(isset($_GET["start"]))
			$start = intval($_GET["start"]);
			
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->group) . "'";
		else
			$where = "";
		
		$pager = new BHtmlPager(32, $this->sys->db->getObjectNumber("Forums", ""));

		$view = new BViewDatabase("list", "selectAction");
		$view->setView(array(
			new BViewType("id", "ID"),
			new BViewType("name", "Name"),
			new BViewType("title", "Title"),
			new BViewType("type", "Type"),
			new BViewType("category", "Category"),
			new BViewType("group", "Group"),
			new BViewType("order", "Order"),
			new BViewType("dateAdded", "Date Published", "datetime"),
			new BViewType("description", "Description")
			));

		$view->setSelection($where);
		$view->setLimit($pager->getRange());

		if($view->order == "")
			$view->setOrder("!order,name");

		$view->setTable("Forums");
		$page = $page . $view->getHtml();

		$page .= $pager->getHtml("?cmd=editList");

		return $page;
	}

	function edit($uri){
		if(is_object($uri)){
			$obj = $uri;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Forum</h2>";
			$obj = $this->sys->db->getObject("Forums", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Forums");
			$obj->group = $this->group;
			$obj->dateAdded = strftime("%Y-%m-%dT%H:%M:%S");
		}

		$page = "<h2>Edit Forum</h2>";

		$view = new BEditObj("forum", "Forum Information", "?cmd=save&group={$this->group}", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("name", "Name"),
			new BViewType("title", "Title"),
			new BViewType("type", "", "select", array("forum")),
			new BViewType("category", "Category", "select", categoryGetList("forum")),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("description", "Description", "text"),
			new BViewType("blog", "Blog", "checkbox"),
			new BViewType("order", "Order"),
			new BViewType("dateAdded", "", "hidden"),
			));

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save($uri){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjForum();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			if(($obj->id == 0) && $this->sys->db->getObjectNumber("Forums", "`name` = '{$obj->name}'")){
				$this->sys->status = "<span class='editError'>Error: a forum with that name already exists</span>";
				return $this->edit($obj);
			}

			if($obj->id)
				$this->sys->db->putObject("Forums", $obj);
			else
				$this->sys->db->appendObject("Forums", $obj);

			// Create the directory			
			$dir = $this->sys->baseDir . "/files/forum/" . $obj->name;
			if(!file_exists($dir)){
				mkdir($dir);
			}

		}

		$page .= "<h3>Forum details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($uri){
		// Check role
		if(!$this->checkAccess()){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("Forums", $id);

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("Forums", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
		}
			
		return $page;
	}
	
	function view($uri){
		if($this->forumName == "")
			return $this->viewList($uri);

		$forum = $this->sys->db->getObjectFirst("Forums", "", "`name` = '$this->forumName'");
		if($forum == Null){
			return "No such forum";
		}

		// Information
		if(isset($_GET["search"]))
			$search = $_GET["search"];
		else
			$search = "";
			
		$page = new BHtmlNode("form", "name='forums' action='' method='get'");

		$div = &$page->appendNode("div", "class=Block style='float:left'");
		$div->appendNode("h1", "", $forum->title);
		$div->appendNode("div", "", $forum->description);
		$div->appendNode("div", "", "To add a response or post a new message you need to have an account and be logged in. You can get emails sent to you by enabling the Send Forum Emails tick box in your user preferences. See our help information <a href='/modules/forum/info.html'>here</a> for more information.");

//		$div = &$page->appendNode("div", "class=Forum style='clear:both; float: left'");

		if(isset($_GET["id"])){
			$div->appendNode("div", "", "You can add to this discussion using the entry box at the bottom of this page, when logged in.");
			$id = intval($_GET["id"]);
			$obj = $this->sys->db->getObject("ForumEntrys", $id);
//			$div->appendNode("h1", "", $forum->title . ": " . $obj->title);
			$div->appendNode("h1", "", $obj->title);
			$div->AppendText($this->entryViewObj($forum, $obj, 1));
		}
		else {
//			$div->appendNode("h1", "", $forum->title);
			$pager = new BHtmlPager(32);
			if($this->forumName == "All")
				$where = "`parentId` = '0'";
			else
				$where = "`forum` = '" . dbEscapeString($this->forumName) . "' and `parentId` = '0'";
			
			if($search)
				$where .= " and ( `title` like '%{$search}%' or `description` like '%{$search}%' )";

			$order = "!dateChildren,!id";
			$pager->setTotal($this->sys->db->getObjectNumber("ForumEntrys", $where));
			$list = $this->sys->db->getObjectList("ForumEntrys", "", $where, $order, $pager->getRange());

			$div1 = $div->appendNode("div", "style='margin: 3px;'");
			$searchLine = $div1->appendNode("span");
			$searchLine->appendNode("input", "name='search' size='40' value='{$search}'");
			$searchLine->appendNode("input", "type='submit' value='Search'");

			foreach($list as $obj){
				if($obj->userId)
					$userName = $this->sys->db->getObject("User", $obj->userId)->name;
				else
					$userName = $obj->userName;
				$comments = 1 + intval($this->sys->db->getObjectNumber("ForumEntrys", "`grandparentId` = '{$obj->id}'"));
				$date = explode(" ", $obj->date);
				$date = $date[0];
				$entry = &$div->appendNode("div", "class=ForumEntry");
				$entry->appendNode("a", "class=ForumEntryHeader href='?id={$obj->id}'", $obj->title);
				$line = &$entry->appendNode("div");
				$line->appendNode("span", "class=ForumEntryLine", "Comments: {$comments}");
				$line->appendNode("span", "class=ForumEntryLine", "Started by: {$userName}");
				$line->appendNode("span", "class=ForumEntryLine", $obj->date);
			}
			$div->appendNode("div", "class=ForumEntry", $pager->getHtml("?cmd=view"));
		}

		return $page->render();
	}
	
	function entryViewObj($forum, $obj, $children){
//		$div = new BHtmlNode("div", "class=Forum style='clear:both; float: left'");
		$div = new BHtmlNode("div", "style='clear:both; float: left'");

		if($obj->userId)
			$userName = $this->sys->db->getObject("User", $obj->userId)->name;
		else
			$userName = $obj->userName;

		$date = explode(" ", $obj->date);
		$date = $date[0];
		$entry = &$div->appendNode("div", "class=ForumEntry");
		$entry->appendNode("span", "class=ForumEntryHeader", $userName);
		$entry->appendNode("span", "class=ForumEntryHeader", $obj->date);
		if($this->checkAccessEntry($obj)){
			$entry->appendNode("span", "class=ForumEntryHeader")->appendNode("a", "href='?cmd=entryEdit&id={$obj->id}' title='Edit Item'", "Edit");
			$entry->appendNode("span", "class=ForumEntryHeader")->appendNode("a", "href='?cmd=entryDelete&id={$obj->id}' title='Delete Item'", "Delete");
			$entry->appendNode("span", "class=ForumEntryHeader")->appendNode("a", "href='?cmd=entryReply&id={$obj->id}' title='Post Response'", "Post Response");
		}
		$entry->appendNode("div", "", $obj->description);
		if($obj->picturePresent)
			$entry->appendNode("a", "href='{$this->sys->baseUrl}/forum/{$obj->forum}?id={$id}'")->appendNode("img", "src='{$this->sys->baseUrl}/files/forum/{$obj->forum}/picture-{$obj->id}.jpg'");

		if($children){
			$where = "`parentId` = '{$obj->id}'";
			$order = "date,id";
//			$list = $this->sys->db->getObjectList("ForumEntrys", "", $where, $order, $pager->getRange());
			$list = $this->sys->db->getObjectList("ForumEntrys", "", $where, $order);

			foreach($list as $cobj){
				$div->appendText($this->entryViewObj($forum, $cobj, 1));
			}
		}

		$this->objId = $obj->id;
		
		if($obj->parentId == 0){
		// Provide ability to add an entry
		if($this->sys->login->checkLogin()){
			unset($_GET["id"]);
			$_GET["parentId"] = $obj->id;
			$div->appendNode("div", "style='float: none; clear: both;'", $this->entryEdit(0, $obj->forum));
		}
		}
		return $div->render();
	}

	function entryReply($uri){
		if($this->forumName == "")
			return $this->viewList($uri);

		$forum = $this->sys->db->getObjectFirst("Forums", "", "`name` = '$this->forumName'");
		if($forum == Null){
			return "No such forum";
		}

		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject("ForumEntrys", filterToInt($_GET["id"]));
		}
		else {
			return "No entry to reply tosuch forum";
		}
		
		$page = new BHtmlNode("div");
		$pager = new BHtmlPager(32);
		$page->appendText($this->entryViewObj($forum, $obj, 0));
//		$pager->setTotal($this->sys->db->getObjectNumber("ForumEntrys", $where));

		// Provide ability to add an entry
		if($this->sys->login->checkLogin()){
			unset($_GET["id"]);
			$_GET["parentId"] = $obj->id;
			$_GET["grandparentId"] = $obj->grandparentId;
			$page->appendNode("div", "style='float: none; clear: both;'", $this->entryEdit(0, $obj->forum));
		}
		return $page->render();
	}

	function entryEdit($uri, $forum = ""){
		if(is_object($uri)){
			$obj = $uri;
			$title = "Forum Edit Entry";
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Forum Entry</h2>";
			$obj = $this->sys->db->getObject("ForumEntrys", filterToInt($_GET["id"]));
			$title = "Forum Edit Entry";
		}
		else {
			$title = "Forum Append Entry";
			if($this->forumName != ""){
				$obj = $this->sys->db->getNewObject("ForumEntrys");
				if(isset($_GET["parentId"]))
					$obj->parentId = intval($_GET["parentId"]);
				if(isset($_GET["grandparentId"]))
					$obj->grandparentId = intval($_GET["grandparentId"]);
				if($obj->grandparentId == 0)
					$obj->grandparentId = $obj->parentId;

				if($forum != "")
					$obj->forum = $forum;
				else
					$obj->forum = $this->forumName;
				$obj->userId = $this->sys->login->id;
				$obj->group = $this->group;
				$obj->dateAdded = strftime("%Y-%m-%dT%H:%M:%S");
				$obj->date = strftime("%Y-%m-%dT%H:%M:%S");
				$obj->dateChildren = strftime("%Y-%m-%dT%H:%M:%S");
			}
			else {
				return "Need to select a Forum to add items to";
			}
		}
		
		// Check if allowed to edit
		if($obj->id){
			if($obj->parentId){
				if(!$this->checkAccessEntry($obj)){
					$this->sys->pageDisplayError(new BError(1, "You are not authorised to edit this item."));
					exit();
				}
			}
			else {
				if(!$this->checkAccess()){
					$this->sys->pageDisplayError(new BError(1, "You are not authorised to edit this item."));
					exit();
				}
			}
		}

		$this->sys->workDir = "/forum/" . $this->forumName;
		$forum = $this->sys->db->getObjectFirst("Forums", "", "`name` = '$obj->forum'");

		$page = "";

		$view = new BEditObj("forum", $title, "?cmd=entrySave&group={$this->group}", "");
		if($obj->parentId){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("dateAdded", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("forum", "Forum", "hidden"),
				new BViewType("date", "Date", "hidden"),
				new BViewType("dateChildren", "Date", "hidden"),
				new BViewType("title", "Title", "hidden"),
				new BViewType("category", "Category", "hidden"),
				new BViewType("group", "Group", "hidden"),
				new BViewType("description", "Text", "textareaHtmlPics", "", "550", 300),
				new BViewType("picturePresent", "", "checkbox"),
				new BViewType("picture", "", "pictureUploaderLineMedium"),
				));
		}
		else {
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("dateAdded", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("forum", "Forum", "hidden"),
				new BViewType("dateChildren", "Date", "hidden"),
				new BViewType("date", "Date", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("category", "Category", "select", categoryGetList("forum")),
				new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
				new BViewType("description", "Description", "textareaHtmlPics", "", "550", 300),
				new BViewType("picturePresent", "", "checkbox"),
				new BViewType("picture", "", "pictureUploaderLineMedium"),
				));
		}

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function entryAppend($uri){
		// Provide ability to append an entry to an existing discussion
		if($this->sys->login->checkLogin() && $this->objId){
			unset($_GET["id"]);
			$_GET["parentId"] = $this->objId;
			return $this->entryEdit(0);
		}
		else {
			return "No discussion to append to";
		}
	}

	function entrySave($uri){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjForumEntry();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			if($obj->id){
				$editing = 1;
				$this->sys->db->putObject("ForumEntrys", $obj);
			}
			else {
				$editing = 0;
				$this->sys->db->appendObject("ForumEntrys", $obj);
			}

			// Upload any pictures present
			if((count($_FILES) == 1) && isset($_FILES["picture"])){
				$file = $_FILES["picture"];
				$filepath = "/files/forum/" . $obj->forum;
				//debug_log("uploadPictures: Do file: {$file["tmp_name"]}\n");

				// Add picture
				$ext = strtolower(pathinfo($file["name"], PATHINFO_EXTENSION));
				pictureSaveDir($file["tmp_name"], $ext, $filepath, $obj->id);
				$filepath .= "/picture-" . strval($obj->id) . ".jpg";

				$obj->picturePresent = 1;
				$this->sys->db->putObject("ForumEntrys", $obj);
			}

			$page .= "<h3>Forum details saved</h3>";
			if($obj->parentId)
				$_GET["parentId"] = $obj->parentId;
			else
				$_GET["parentId"] = $obj->id;
			if($obj->grandparentId)
				$_GET["grandparentId"] = $obj->grandparentId;
			else
				$_GET["parentId"] = $obj->id;

			//$page .= $this->view($uri);
			if($obj->grandparentId)
				$objGp = $this->sys->db->getObject("ForumEntrys", $obj->grandparentId);
			else
				$objGp = $obj;

			$this->updateDates($obj);

			$_GET["id"] = $objGp->id;
			//$page .= $this->entryViewObj(Null, $objGp, 1);
			$page .= $this->view(Null);

			if(! $editing){
				// Send emails to all who are interested		
				$this->sendEmails($objGp, $obj);
			}
		}
		else {
			$page .= "<h3>Forum details nothing to save</h3>";
		}

		return $page;
	}
	
	function updateDates($obj){
		if($obj->parentId){
			$objParent = $this->sys->db->getObject("ForumEntrys", $obj->parentId);
			$objParent->dateChildren = $obj->date;
			$this->sys->db->putObject("ForumEntrys", $objParent);
			
			if($objParent->parentId)
				$this->updateDates($objParent);
		}
	}

	function entryDelete($uri){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("ForumEntrys", $id);
		
		// Check if allowed to delete
		if($obj->parentId){
			if(!$this->checkAccessEntry($obj)){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete this item."));
				exit();
			}
		}
		else {
			if(!$this->checkAccess()){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete this item."));
				exit();
			}
		}

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("ForumEntrys", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=entryDelete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete forum entry ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
		}
			
		return $page;
	}
	
	function sendEmails($objGp, $obj){
		if($obj){
			// Send an email to everyone interested in receiving forum emails
			$subject = "{$this->sys->siteTitle}: Forum post: {$objGp->title}";
			$message = "{$obj->description}\n\n";
			$message .= "To the Forum: <a href='{$this->sys->baseUrl}/forum/{$obj->forum}?id={$objGp->id}'>{$this->sys->baseUrl}/forum/{$obj->forum}?id={$objGp->id}</a>\n";
			$message .= "\n";
			$message .= "You can disable receiving these emails in your website accounts User Preferences\n";
			
			if($this->test){
				$emailAddressList = array();
				$emailAddressList[] = $this->sys->user->emailAddress;
			}
			else {
				$objList = $this->sys->db->getObjectList("User", "emailAddress", "`forumEmailSend` = '1'", "");

				$emailAddressList = array();
				foreach($objList as $o){
					$emailAddressList[] = $o->emailAddress;
				}
			}
			return emailList($emailAddressList, $subject, $message, 1);
		}
	}

	function parseUri($uri){
		parent::parseUri($uri);
		
		$this->forumName = "";
		$this->arg = "";

		if(strpos($uri, "/")){
			$this->forumName = substr($uri, strpos($uri, "/") + 1);
			$this->forumName = filterPath($this->forumName, $this->sys->baseDir);
			if(strpos($this->forumName, "/")){
				$this->arg = substr($this->forumName, strpos($this->group, "/") + 1);
				$this->forumName = substr($this->forumName, 0, strpos($this->group, "/"));
			}
			$this->forumName = dbEscapeString($this->forumName);
		}
	}
};

function forumList($forum="", $num=3){
	global	$sys;
	
	if($forum){
		$objList = $sys->db->getObjectList("Forums", "", "`name` = '{$forum}'");
		if(count($objList))
			$where = "`forum`='{$objList[0]->id}'";
		else
			$where = "";
	}
	else {
		$forum = "All";
		$where = "";
	}

	$objList = $sys->db->getObjectList("ForumEntrys", "", $where, "!date", "{$num}");

	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/forum/{$forum}'")->appendNode("h2", "", "Forum: " . $forum);

	foreach($objList as $obj){
		if($obj->grandparentId){
			$title = $sys->db->getObject("ForumEntrys", $obj->grandparentId)->title;
			$id = $obj->grandparentId;
		}
		else {
			$title = $obj->title;
			$id = $obj->id;
		}
		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->dateAdded));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/forum/{$obj->forum}?id={$id}'", $title);
//		$d1->appendNode("div", "class='NewsSummaryCategory'", "Category: General");
//		$d1->appendNode("div", "class='NewsSummaryAuthor'", "Posted by: terry");
		$d1->appendNode("div", "class='NewsSummaryContent'", substr($obj->description, 0, 100));
		if($obj->picturePresent)
			$d1->appendNode("a", "href='{$sys->baseUrl}/forum/{$obj->forum}?id={$id}'")->appendNode("img", "src='{$sys->baseUrl}/files/forum/{$obj->forum}/picture-{$obj->id}-thumb.jpg'");
	}

	return $html->render();
}

?>
