<?php
/*******************************************************************************
 *	forumEmail.php		Forum Email Module
 *				T.Barnaby,	BEAM Ltd,	2008-01-31
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";
require_once "BObj.php";

global	$sys;
$sys->db->addObjectClasses(array("ForumEmail" => "BObjForumEmail"));
$sys->db->addObjectClasses(array("ForumEmailEntry" => "BObjForumEmailEntry"));

class BObjForumEmail extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("name", "String"),
			new BMember("title", "String"),
			new BMember("userIds", "String"),
			new BMember("dateAdded", "DateTime", "date", array('format' => '%Y-%n-%j')),
			));
	}
};
class BObjForumEmailEntry extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("forum", "String"),
			new BMember("title", "String"),
			new BMember("type", "String"),
			new BMember("content", "String"),
			new BMember("filePath", "String"),
			new BMember("userEmail", "String"),
			new BMember("userName", "String"),
			new BMember("dateAdded", "DateTime", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

class Email {
	function __construct($content = ""){
		$this->content = $content;
	}
	
	function set($content){
		$this->content = $content;
	}
	
	function getHeaderField($name){
		$b = strpos($this->content, "\n\n");
		if($b === False)
			$b = strpos($this->content, "\r\n\r\n");
		if($b !== False){
			$lines = explode("\n", substr($this->content, 0, $b));
			for($i = 0; $i < count($lines); $i++) {
				if(preg_match("/^{$name}: (.*)/", $lines[$i], $matches))
					return $matches[1];
			}
		}	

		return "";
	}
	
	function getBody(){
		$b = strpos($this->content, "\n\n");
		if($b === False)
			$b = strpos($this->content, "\r\n\r\n");
		if($b !== False){
			return substr($this->content, $b+2, strlen($this->content));
		}	

		return "";
	}
	
	function getFrom(){
		$e = $this->getHeaderField("From");
		if($e != ""){
			if(preg_match("/.*<(.*)>/", $e, $matches))
				return $matches[1];
		}
		return $e;
	}

	var	$content;
}


class ForumEmail extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "forumEmail";
		$this->title = "ForumEmail";
		$this->info = "ForumEmail Module";
	}
	
	function getMenu(){
		return new BMenu("Forum", "/{$this->name}", "", array("Forum List" => "/{$this->name}", "Forum Add" => "/{$this->name}?cmd=add"));
	}

	function getPage($uri){
		$this->parseUri($uri);

		switch($this->cmd){
		case "add":			return "Forum Add";			break;
		case "forumList":		return $this->viewForumList($uri);	break;
		default: 			return $this->viewList($uri);		break;
		}
	}

	function getPageBlock($uri){
		return new BMenu("Forum", "", "", array(
			new BMenu("List", "", "href='?cmd=viewList'"),
			));
	}

	function view($uri){
		return "Forum";
	}
	
	function viewList(){
		$forums = $this->sys->db->getObjectList("ForumEmail", "", "", "Title");
		
		$page = "<h2>Forums</h2>\n";

		$h = new BHtmlNode("table", "border=1");

		foreach($forums as $forum){
			$r = &$h->appendNode("tr");
			$r->appendNode("td")->appendNode("a", "href=?cmd=forumList&forum={$forum->name}", $forum->title);
			$r->appendNode("td", "", "{$forum->dateAdded}");
		}

		$page = $page . $h->render();

		return $page;
	}

	function viewForumList(){
		$forum = dbEscapeString(filterString($_GET["forum"]));
		if(isset($_GET["pos"]))
			$pos = intval($_GET["pos"]);
		else
			$pos = 0;

		$number = 10;
		$posNext = $pos + $number;
		$posPrev = $pos - $number;
		if($posPrev < 0)
			$posPrev = 0;
		
		$forums = $this->sys->db->getObjectList("ForumEmail", "", "`name`='{$forum}'", "Title");
		$forumName = $forums[0]->name;
		$forumTitle = $forums[0]->title;
		
		$page = "<h2>Forum {$forumTitle}</h2>\n";

		$h = new BHtmlNode("form", "name='listForm' method='get'");
		$h->appendNode("input", "type='hidden' name='pos'");
		$h->appendText("Location: {$pos}: ");
		$h->appendNode("a", "href='javascript: document.listForm.pos.value = {$posPrev}; document.listForm.submit();'", "Prev");
		$h->appendText(" - ");
		$h->appendNode("a", "href='javascript: document.listForm.pos.value = {$posNext}; document.listForm.submit();'", "Next");

		$t = &$h->appendNode("table", "border=1 width=100%");

		$list = $this->sys->db->getObjectList("ForumEmailEntry", "", "`forum`='{$forumName}'", "!dateAdded", "$pos,$number");

		foreach($list as $obj){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "width=200px valign=top bgcolor='#F0F0F0'");
			$c->appendNode("div", "", "Date: " . $obj->dateAdded);
			$c->appendNode("div", "", "User: " . $obj->userName);
			$c->appendNode("div", "", "From: " . $obj->userEmail);

			$c = &$r->appendNode("td", "valign = top");
			$c->appendNode("div", "", "<strong>{$obj->title}</strong>");
			$c->appendNode("hr");
			if($this->sys->browser == "MSIE"){
				if(strlen($this->emailGetBody($obj)) > 512)
					$d = &$c->appendNode("div", "style='overflow:auto; height:300px'");
				else
					$d = &$c->appendNode("div");
			}
			else
				$d = &$c->appendNode("div", "style='overflow:auto; max-height:300px'");

			$d->appendText("<pre>{$this->emailGetBody($obj)}</pre>");
		}

		$h->appendText("Location: {$pos}: ");
		$h->appendNode("a", "href='javascript: document.listForm.pos.value = {$posPrev}; document.listForm.submit();'", "Prev");
		$h->appendText(" - ");
		$h->appendNode("a", "href='javascript: document.listForm.pos.value = {$posNext}; document.listForm.submit();'", "Next");

		$page = $page . $h->render();

		return $page;
	}

	function emailGetBody($obj){
		$email = new EMail(file_get_contents($this->sys->baseDir . "/" . $obj->filePath));
		
		return $email->getBody();
	}
	
	function emailProcess(){
		$forums = $this->sys->db->getObjectList("ForumEmail");
		
		foreach($forums as $forum){
			$email = $this->sys->siteName . "." . $forum->name;
			
//			print "DoEmail: $email\n";

			// Process email list
			$imap = imap_open("{localhost/norsh/novalidate-cert}INBOX", $email, "beam00");
			if(! $imap){
				print "Error not connected";
				exit(1);
			}

			$num = imap_num_msg($imap);
//			print "Number $num\n";

			for($i = 0; $i < $num; $i++){
				$header = imap_headerinfo($imap, 1);
				$h = explode("\n", imap_fetchheader($imap, 1));
				$headerAll = array();
				foreach($h as $line) {
					// separate name and value
					preg_match("#^([^:]*): (.*)#i", $line, $arg);
					if(count($arg)){
						$headerAll[$arg[1]] = $arg[2];
					}
				}

//				print "Header: {$header->from} {$header->subject}\n";
//				print_r($headerAll);
				
				// Check for mail loop
				if(!isset($headerAll["WebSysLoop"])){
				
					$emails = array();
					$userIds = explode(",", $forum->userIds);
//					print "Send emails to: {$userIds}";
					foreach($userIds as $userId){
						$user = $this->sys->db->getObject("User", $userId);
						if($user->emailAddress != "")
							$emails[] = $user->emailAddress;
					}

					$from = $header->from[0]->mailbox . "@" . $header->from[0]->host;
					$content = imap_body($imap, 1);

					$obj = $this->sys->db->getNewObject("ForumEmailEntry");
					$obj = $this->sys->db->appendObject("ForumEmailEntry", $obj);

					$obj->forum = $forum->name;
					$obj->type = "email";
					$obj->title = $header->subject;
					$obj->userEmail = $from;
					$obj->userName = "";
					$obj->content = "";
					$obj->filePath = "files/emails/email-{$obj->id}.eml";
					$obj->dateAdded = strftime("%Y-%m-%dT%H:%M:%S");

					$this->sys->db->putObject("ForumEmailEntry", $obj);
					file_put_contents($this->sys->baseDir . "/" . $obj->filePath, imap_fetchheader($imap, 1) . $content);

					$fromAddress = $this->sys->siteName . "." . $forum->name;

//					$emails = array("terry@beam.ltd.uk");		// For testing

					foreach($emails as $e){
						// Convert headers to a dictionary list
						$headerList = array();
						$hl = explode("\n", imap_fetchheader($imap, 1));
						$l = "";
						foreach($hl as $h){
							$h = trim($h, "\r\n");
							if(preg_match("/^(.*): (.*)/", $h, $m)){
								$headerList[$m[1]] = $m[2];
								$l = $m[1];
							}
							else if($h != ""){
								$headerList[$l] .= "\r\n" . $h;
							}
						}

						// Modify the header
						unset($headerList["To"]);
						unset($headerList["to"]);
						unset($headerList["CC"]);
						$headerList["From"] = $from;
						$headerList["Sender"] = $fromAddress;
						$headerList["Return-Path"] = $fromAddress;
						$headerList["Reply-To"] = $fromAddress;
						$headerList["WebSysLoop"] = "1";

						$headers = "";
						foreach($headerList as $k => $v){
							$headers .= "{$k}: $v\r\n";
						}

//						print "HeaderText:"; print $headers; print "<br>\n";
//						print "Send email to: {$e} {$header->subject}";
//						exit(1);
						mail($e, $header->subject, $content, $headers, "-f {$fromAddress}");
						
						// Max rate 1 email per second
						sleep(1);
					}
				}

				imap_delete($imap, 1);
				imap_expunge($imap);
			}
			imap_close($imap);
		}
	}
	
	// Old code
	function emailAdd($content){
		$email = new Email($content);

		$obj = $this->sys->db->getNewObject("ForumEmailEntry");
		$obj = $this->sys->db->appendObject("ForumEmailEntry", $obj);
		
		$obj->forum = "bds";
		$obj->type = "email";
		$obj->title = $email->getHeaderField("Subject");
		$obj->userEmail = $email->getFrom();
		$obj->userName = "";
		$obj->content = "";
		$obj->filePath = "files/emails/email-{$obj->id}.eml";
		$obj->dateAdded = strftime("%Y-%m-%dT%H:%M:%S");

		$this->sys->db->putObject("ForumEmailEntry", $obj);
		
		file_put_contents($this->sys->baseDir . "/" . $obj->filePath, $content);
	}
	
};

function forumEmailAdd($content){
	global	$sys;
	
	$module = new ForumEmail($sys);
	
	$module->emailAdd($content);
}

function forumEmailProcess(){
	global	$sys;
	
	$module = new ForumEmail($sys);
	
	$module->emailProcess();
}
?>
