<?php
/*******************************************************************************
 *	GroupsTc.php	Groups Module for Tandem club
 *			T.Barnaby,	BEAM Ltd,	2011-12-07
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjGroup extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("name", "String", "stringNoSpaces"),
			new BMember("title", "String", "string"),
			new BMember("description", "String"),
			new BMember("postCodes", "String"),
			new BMember("organiserId", "Int"),
			new BMember("web", "String"),
			));
	}
};

class BObjGroupLocation extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("postcode", "String", "string"),
			new BMember("group", "String", "string"),
			));
	}
	function dbTableFields(){
		return "(
			`id` int NOT NULL auto_increment,
			`postcode` text,
			`group` text,
			PRIMARY KEY  (`id`)
			)";
	}
};

function addSpacesToList($str){
	$ret = "";
	
	for($i = 0; $i < strlen($str); $i++){
		if($str[$i] == ",")
			$ret .= ", ";
		else
			$ret .= $str[$i];
	}
	
	return $ret;
}

class GroupsTc extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "groupsTc";
		$this->title = "Regional Groups";
		$this->info = "Groups module. This module allows the management of Groups.";
		$this->group = "";
		$this->arg = "";
		
		// Add Objects
		$sys->db->addObjectClasses(array("Groups" => "BObjGroup"));
		$sys->db->addObjectClasses(array("GroupLocations" => "BObjGroupLocation"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminGroups"));

		if($this->sys->db->tableExists("Groups")){
			// Get list of group names
			$list = $this->sys->db->getObjectList("Groups", "name", "", "name", "");
			$groups = array();
			$this->sys->groups = array();
			foreach($list as $obj){
				$this->sys->groups[] = $obj->name;
				$groups[] = "adminGroup" . $obj->name;
			}
			$this->sys->modules["user"]->appendRoles($groups);
		}
		
		// Add HTML functions
		$this->sys->htmlFuncs[] = "groupContact";

	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin,adminGroups,adminGroup{$this->group}");
	}

	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Groups", "/{$this->name}", "", array("Groups List" => "/{$this->name}", "Groups Add" => "/{$this->name}?cmd=groupAdd"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);
		$_GET['group'] = $this->group;

		$page = "";
		
		if($this->checkAccess()){
			if($this->group == ""){
				switch($this->cmd){
				case "groupView":	$page = $this->groupView($uri); break;
				case "groupEdit":	$page = $this->groupEdit($uri); break;
				case "groupSave":	$page = $this->groupSave($uri); break;
				case "groupDelete":	$page = $this->groupDelete($uri); break;
				case "updatePostcodes":	$page = $this->updatePostcodes(); break;
				default:		$page = $this->groupList($uri); break;
				}
			}
			else {
				switch($this->cmd){
				case "groupList":	$page = $this->groupList($uri); 	break;
				case "groupEditInfo":	$page = $this->groupEditInfo($uri);	break;
				case "groupAddPage":	$page = $this->groupAddPage($uri);	break;
				case "addPage1":	$page = $this->groupAddPage1($uri);	break;
				case "groupAddDir":	$page = $this->groupAddDir($uri);	break;
				case "addDir1":		$page = $this->groupAddDir1($uri);	break;
				case "save":		$page = $this->groupEditInfoSave($uri);	break;
				default:
					$module = $this->sys->modules["content"];
					if($this->arg == "")
						$page = $module->getPage("files/groups/" . $this->group . "/index.html");
					else
						$page = $module->getPage("files/groups/" . $this->group . "/" . $this->arg);
					break;
				}
			}
		}
		else {
			if($this->group == ""){
				switch($this->cmd){
				case "groupView":	$page = $this->groupView($uri); break;
				default:		$page = $this->groupList($uri); break;
				}
			}
			else {
				switch($this->cmd){
				case "groupList":	$page = $this->groupList($uri); 	break;
				default:		
					$module = $this->sys->modules["content"];
					if($this->arg == "")
						$page = $module->getPage("files/groups/" . $this->group . "/index.html");
					else
						$page = $module->getPage("files/groups/" . $this->group . "/" . $this->arg);
					break;
				}
			}
		}

		return $page;
	}

	function getPageBlock($uri){
		$this->parseUri($uri);
		$_GET['group'] = $this->group;

		if($this->checkAccess()){
			$help = "/files/help/groupsTc.html";

			if($this->group == ""){
				$o = new BMenu("Groups", "", "", array(
					new BMenu("List All", "", "href='?cmd=groupList'"),
					new BMenu("", "", ""),
					new BMenu("Group Add", "", "href='?cmd=groupEdit'"),
					new BMenu("", "", ""),
					new BMenu("UpdatePostcodes", "",		"href='?cmd=updatePostcodes'"),
					new BMenu("Help", "",		"href='{$help}'"),
					));
			}
			else {
				if($this->cmd == "groupEditInfo"){
					$m = array();
					$m[] = new BMenu("Add Page", "javascript:popupAddPage();");
					$o = new BMenu("Group", "", "", $m);
				}
				else {
					$m = array();
					$m[] = new BMenu("Edit page", "", "href='?cmd=groupEditInfo'");
					$m[] = new BMenu("Add page", "", "href='?cmd=groupAddPage'");
					$m[] = new BMenu("Add directory", "", "href='?cmd=groupAddDir'");
					$m[] = new BMenu("Edit news", "", "href='/news?group={$this->group}'");
					$m[] = new BMenu("Edit What's On", "", "href='/whatsOn?cmd=viewList&group={$this->group}'");
					$m[] = new BMenu("Edit showcases", "", "href='/showcase?group={$this->group}'");
					$m[] = new BMenu("Edit documents", "", "href='/document?cmd=viewList&group={$this->group}'");
					if($this->checkAccess("viewMembership"))
						$m[] = new BMenu("Membership", "", "href='/membership?cmd=viewList&group={$this->group}'");
//					$m[] = new BMenu("Bugs/Todo", "",	"href='/info'");
					$m[] = new BMenu("Help", "",		"href='{$help}'");
					$o = new BMenu("Group", "", "", $m);
				}
			}
		}
		else {
			if($this->group == ""){
				$o = new BMenu("Groups", "", "", array(
					new BMenu("List All", "", "href='{$this->sys->baseUrl}/groups?cmd=groupList'"),
					));
			}
			else {
				$o = new BMenu("Group", "", "", array(
					new BMenu("List All", "", "href='/groups?cmd=groupList'"),
					new BMenu("News", "", "href='/news?group={$this->group}'"),
					new BMenu("Showcases", "", "href='/showcase?group={$this->group}'"),
					new BMenu("Documents", "", "href='/document?cmd=viewList&group={$this->group}'"),
					new BMenu("Whats On", "", "href='/whatsOn?group={$this->group}'"),
					));
			}
		}
		return $o;
	}

	function groupList($arg){
		$order = "name";
		
		if($this->checkAccess()){
			$view = new BViewDatabase("group", "action");
			$view->setView(array(
				new BViewType("id"),
				new BViewType("title", "Title", "link", array("\$obj->title", $this->sys->baseUrl ."/groups/\$obj->name/index.html")),
				new BViewType("postCodes"),
				new BViewType("organiserName", "Name"),
				new BViewType("organiserTel", "Telephone"),
				new BViewType("organiserEmail", "Email", "email", array("TandemClub: ")),
				new BViewType("description"),
				));

			$view->setOrder($order);
			$view->setTable("Groups");
			
			foreach($view->getList() as $obj){
				if($obj->organiserId){
					$user = $this->sys->db->getObject("User", $obj->organiserId);
					$obj->organiserName = $user->name;
					$obj->organiserTel = $user->telephoneHome;
					$obj->organiserEmail = $user->emailAddress;
				}
				else {
					$obj->organiserName = "";
					$obj->organiserTel = "";
					$obj->organiserEmail = "";
				}
			}
			
			$p = new BHtmlNode("div");
			$p->appendNode("h2", "", $this->title);
			$t = &$p->appendNode("p", "", "RO's: Please contact <a href='mailto:webgroup@tandem-club.org.uk'>The Tandem Club Web Group</a> to get your region added and to get a website login account so you can edit your regions pages and add/edit What's On etc. ");
			$t->appendText("There some help available <a href='/files/help/groupsTc.html'>here</a>");
			$page = $p->render() . $view->getHtml();
		}
		else {
			$p = new BHtmlNode("div");
			$p->appendNode("h2", "", $this->title);
			$t = &$p->appendNode("p", "", "RO's: Please contact <a href='mailto:webgroup@tandem-club.org.uk'>The Tandem Club Web Group</a> to get your region added and to get a website login account so you can edit your regions pages and add/edit What's On etc. ");
			$t->appendText("There some help available <a href='/files/help/groupsTc.html'>here</a>");

			$t = &$p->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");
			
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "class='heading'", $this->title);
			$r->appendNode("th", "class='heading'", "PostCodes");
			$r->appendNode("th", "class='heading'", "Name");
			$r->appendNode("th", "class='heading'", "Telephone");
			$r->appendNode("th", "class='heading'", "Email");
			$r->appendNode("th", "class='heading'", "Description");

			$list = $this->sys->db->getObjectList("Groups", "", "", $order, "");

			foreach($list as $obj){
				if($obj->organiserId){
					$user = $this->sys->db->getObject("User", $obj->organiserId);
					$obj->organiserName = $user->name;
					$obj->organiserTel = $user->telephoneHome;
					$obj->organiserEmail = $user->emailAddress;
				}
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "")->appendNode("a", "href='{$this->sys->baseUrl}/groups/{$obj->name}/index.html'", $obj->title);
				$r->appendNode("td", "", addSpacesToList($obj->postCodes));
				if($obj->organiserId){			
					$r->appendNode("td", "", $obj->organiserName);
					$r->appendNode("td", "", $obj->organiserTel);
					$r->appendNode("td", "", "<a href='mailto:{$obj->organiserEmail}?subject=TandemClub%3A%20'>{$obj->organiserEmail}</a>");
				}
				else {
					$r->appendNode("td", "", "");
					$r->appendNode("td", "", "");
					$r->appendNode("td", "", "");
				}
				$r->appendNode("td", "", $obj->description);
			}
			$page = $p->render();
		}
		return $page;
	}
	
	function groupView($arg){
		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject("Groups", filterToInt($_GET["id"]));
			header("Location: {$this->sys->baseUrl}/groups/{$obj->name}/index.html");
			return;
		}
		else if($this->group){
			header("Location: {$this->sys->baseUrl}/groups/{$this->group}/index.html");
			return;
		}
		else {
			return "No group selected";
		}
	}
	
	function groupEdit($arg){
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Group</h2>";
			$obj = $this->sys->db->getObject("Groups", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Groups");
		}

		$userInfo = [];
		$users = $this->sys->modules["user"]->getUserList("");
		$userInfo[] = new BNameValue("0", "Vacant");
		foreach($users as $u){
			$userInfo[] = new BNameValue(strval($u->id), "{$u->id}, {$u->name}, {$u->postcode}");
		}
		$view = new BEditObj("group", "Group Information", "?cmd=groupSave", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("name"),
			new BViewType("title"),
			new BViewType("description"),
			new BViewType("postCodes"),
			new BViewType("organiserId", "Organiser", "selectNameValue", $userInfo),
			new BViewType("web"),
			new BViewType("updateMembership", "Update All Membership Groups", "checkbox"),
			));

		$view->setHelp("groupName", "Enter the regional groups name.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");
		$view->setHelp("updateMembership", "If checked will update all groups in the membership database.");

		$view->setObj($obj);
		return $view->getHtml();
	}
	
	function groupSave(){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjGroup();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->groupEdit($obj);
			}

			if($obj->id){
				$this->sys->db->putObject("Groups", $obj);
			}
			else {
				$this->sys->db->appendObject("Groups", $obj);
				$dir = $this->sys->baseDir . "/files/groups/" . $obj->name;

				if(!file_exists($dir . "/index.html")){
					mkdir($dir);

					$h = new BHtmlNode("html");
					$h->appendNode("head");
					$b = &$h->appendNode("body");
					$b->appendNode("h1", "", $obj->title);
					$b->appendNode("script", "language='php'", "groupContact()");

					$t = &$b->appendNode("table", "border='0' cellpadding='0' cellspacing='0' width='100%'");
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "valign='top'", "<script language='php'>newsList()</script>");
					$r->appendNode("td", "valign='top'", "<script language='php'>showcaseGroup()</script>");
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan=2", "<script language='php'>whatsOnList()</script>");
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan='2'", "<script language='php'>documentList()</script>");
					file_put_contents($dir . "/index.html", $h->render());
				}
			}
			if($_POST['updateMembership'] == "1")
				$this->updatePostcodes();
		}

		$page .= "<h3>Details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function groupDelete(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("Groups", $id);
			$this->sys->db->deleteObject("Groups", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("Groups", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=groupDelete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function groupEditInfo($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->edit("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}

	function groupEditInfoSave($uri){
		$module = $this->sys->modules["content"];
		$page = $module->save($uri);
		return $page;
	}

	function groupAddPage($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addPage("files/groups/" . $this->group . "/" . $this->arg);
		$page .= "groupAddPage: For {$this->group}<br>";
		return $page;
	}

	function groupAddPage1($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addPage1("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}

	function groupAddDir($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addDirectory("files/groups/" . $this->group . "/" . $this->arg);
		$page .= "groupAddDir: For {$this->group}<br>";
		return $page;
	}

	function groupAddDir1($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addDirectory1("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}
	
	function getPostcodes($obj){
		$pc = array();
		$list = $this->sys->db->getObjectList("GroupLocations", "", "`group`='{$obj->name}'", "", "");
//		$list = $this->sys->db->getObjectList("GroupLocations", "", "`group`='{$obj->title}'", "", "");
		foreach($list as $i){
			$pc[] = $i->postcode;
		}
		return join(",", $pc);
	}

	function setPostcodes(&$obj){
		$pcl = array_map('trim', explode(",", $obj->postCodes));
		$this->sys->db->query("delete from {$this->sys->databasePrefix}GroupLocations where `group`='{$obj->name}'");
		$this->sys->db->query("delete from {$this->sys->databasePrefix}GroupLocations where `group`='{$obj->title}'");
		foreach($pcl as $i){
			$l = new BObjGroupLocation();
			$l->postcode = $i;
			$l->group = $obj->name;
//			$l->group = $obj->title;
			$this->sys->db->appendObject("GroupLocations", $l);
		}
	}
	
	function getGroupForPostcode($postcode){
		if((strlen($postcode) > 1) && !ctype_alpha($postcode[1]))
			$postcode = substr(strtoupper($postcode), 0, 1);
		else
			$postcode = substr(strtoupper($postcode), 0, 2);

		$list = $this->sys->db->getObjectList("GroupLocations", "", "`postcode`='{$postcode}'", "", "");
		if(count($list))
			return $list[0]->group;
		else
			return Null;
	}
	
	function updatePostcodes(){
		$this->sys->db->query("truncate table {$this->sys->databasePrefix}GroupLocations");
		$list = $this->sys->db->getObjectList("Groups", "", "", "", "");
		foreach($list as $obj){
			$this->setPostcodes($obj);
		}

		// Update membership groups
		if($module = $this->sys->modules["membership"]){
			$module->groupsUpdate();
		}

		return "Updated ok. All membership records have been changed as needed.";
	}

	function parseUri($uri){
		parent::parseUri($uri);

		$this->group = "";
		$this->arg = "";

		if(strpos($uri, "/")){
			$this->group = substr($uri, strpos($uri, "/") + 1);
			$this->group = filterPath($this->group, $this->sys->baseDir);
			if(strpos($this->group, "/")){
				$this->arg = substr($this->group, strpos($this->group, "/") + 1);
				$this->group = substr($this->group, 0, strpos($this->group, "/"));
			}
		}
	}
};

function groupContact($group){
	global	$sys;

	if($group){
		$where = "`name`='{$group}'";
	}
	elseif(isset($_GET['group'])){
		$group = dbEscapeString(filterString($_GET['group']));
		$where = "`name`='{$group}'";
	}
	else {
		$where = "";
	}

	$str = "";

	$list = $sys->db->getObjectList("Groups", "", $where, "", "");
	if(count($list)){
		$userId = $list[0]->organiserId;
		if($userId){
			$user = $sys->db->getObject("User", $userId);
			$str .= "<b>Organisers Contact:</b> {$user->name}";
			if($user->telephoneHome)
				$str .=	 ", <b>Tel:</b> {$user->telephoneHome}";
			if($user->emailAddress)
				$str .=	", <b>Email:</b> <a href='mailto:{$user->emailAddress}'>{$user->emailAddress}</a><br>\n";
		}
	}

	return $str;
}

function groupContactEmail($group){
	global	$sys;

	if($group == "")
		return "";
		
	$where = "`name`='{$group}'";
	$str = "";

	$list = $sys->db->getObjectList("Groups", "", $where, "", "");
	if(count($list)){
		$userId = $list[0]->organiserId;
		if($userId){
			$user = $sys->db->getObject("User", $userId);
			$str = $user->emailAddress;
		}
	}

	return $str;
}

?>
