<?php
/*******************************************************************************
 *	GroupsWiki.php	GroupsWiki Module
 *			T.Barnaby,	BEAM Ltd,	2012-09-27
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("Groups" => "BObjGroups"));

class BObjGroups extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("name", "String", "stringNoSpaces"),
			new BMember("title", "String"),
			new BMember("description", "String"),
			new BMember("postCodes", "String"),
			new BMember("organiserId", "Int"),
			new BMember("web", "String"),
			));
	}
};

class GroupsWiki extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "groups";
		$this->title = "Wiki Groups";
		$this->info = "GroupsWiki module. This module allows the management of Wiki Groups.";
		$this->group = "";
		$this->arg = "";

		if($this->sys->db->tableExists("Groups")){
			// Get list of group names
			$list = $this->sys->db->getObjectList("Groups", "name", "", "name", "");
			$groups = array();
			$this->sys->groups = array();
			foreach($list as $obj){
				$this->sys->groups[] = $obj->name;
				$groups[] = "adminGroup" . $obj->name;
			}
			$this->sys->modules["user"]->appendRoles($groups);
		}

		// Create the directory			
		$dir = $this->sys->baseDir . "/files/groups";
		if(!file_exists($dir)){
			mkdir($dir);
		}
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin,adminGroup{$this->group}");
	}

	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Groups", "/{$this->name}", "", array("Groups List" => "/{$this->name}", "Groups Add" => "/{$this->name}?cmd=groupAdd"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);
		$_GET['group'] = $this->group;
		
//		print "GroupsWiki: Group: '{$this->group}' Arg: '{$this->arg}'<br>";

		$page = "";
		
		if($this->checkAccess()){
			if($this->group == ""){
				switch($this->cmd){
				case "viewContent":	$page = $this->viewContent($uri); break;
				case "groupView":	$page = $this->groupView($uri); break;
				case "groupEdit":	$page = $this->groupEdit($uri); break;
				case "groupSave":	$page = $this->groupSave($uri); break;
				case "groupDelete":	$page = $this->groupDelete($uri); break;
				default:		$page = $this->groupList($uri); break;
				}
			}
			else {
				switch($this->cmd){
				case "viewContent":	$page = $this->viewContent($uri);	break;
				case "groupList":	$page = $this->groupList($uri); 	break;
				case "groupEditInfo":	$page = $this->groupEditInfo($uri);	break;
				case "groupAddPage":	$page = $this->groupAddPage($uri);	break;
				case "addPage1":	$page = $this->groupAddPage1($uri);	break;
				case "groupAddDir":	$page = $this->groupAddDir($uri);	break;
				case "addDir1":		$page = $this->groupAddDir1($uri);	break;
				case "save":		$page = $this->groupEditInfoSave($uri);	break;
				case "addProject":	$page = $this->addProject($uri);	break;
				default:
					$uri = "files/groups/" . $this->group . "/" . $this->arg;
					if(is_dir($uri)){
						header("Location: {$this->sys->baseUrl}/{$uri}");
					}
					else {
						$module = $this->sys->modules["content"];
						if($this->arg == "")
							$page .= $module->getPage("files/groups/" . $this->group . "/index.html");
						else
							$page .= $module->getPage("files/groups/" . $this->group . "/" . $this->arg);
						break;
					}
				}
			}
		}
		else {
			if($this->group == ""){
				switch($this->cmd){
				case "viewContent":	$page = $this->viewContent($uri);	break;
				case "groupView":	$page = $this->groupView($uri); break;
				default:		$page = $this->groupList($uri); break;
				}
			}
			else {
				switch($this->cmd){
				case "viewContent":	$page = $this->viewContent($uri);	break;
				case "groupList":	$page = $this->groupList($uri); 	break;
				default:		
					$uri = "files/groups/" . $this->group . "/" . $this->arg;
					if(is_dir($uri)){
						header("Location: {$this->sys->baseUrl}/{$uri}");
					}
					else {
						$module = $this->sys->modules["content"];
						if($this->arg == "")
							$page = $module->getPage("files/groups/" . $this->group . "/index.html");
						else
							$page = $module->getPage("files/groups/" . $this->group . "/" . $this->arg);
					}
					break;
				}
			}
		}

		return $page;
	}

	function getPageBlock($uri){
		$this->parseUri($uri);
		$_GET['group'] = $this->group;

		if($this->checkAccess()){
			$help = $this->sys->baseUrl . "/modules/groups/info.html";

			if($this->group == ""){
				$o = new BMenu("Groups", "", "", array(
					new BMenu("List All", "", "href='?cmd=groupList'"),
					new BMenu("", "", ""),
					new BMenu("Group Add", "", "href='?cmd=groupEdit'"),
					new BMenu("", "", ""),
					new BMenu("Help", "",		"href='{$help}'"),
					));
			}
			else {
				if($this->cmd == groupEditInfo){
					$m = array();
					$m[] = new BMenu("Add Page", "javascript:popupAddPage();");
					$o = new BMenu("Group", "", "", $m);
				}
				else {
					$m = array();
					$m[] = new BMenu("Group Home page", "", "href='/groups/{$this->group}/index.html'");
					$m[] = new BMenu("View Content", "", "href='?cmd=viewContent'");
					$m[] = new BMenu("Edit page", "", "href='?cmd=groupEditInfo'");
					$m[] = new BMenu("Add page", "", "href='?cmd=groupAddPage'");
					$m[] = new BMenu("Add directory", "", "href='?cmd=groupAddDir'");
					$m[] = new BMenu("Upload File", "?cmd=groupUploadFile", "");
					$m[] = new BMenu("Add Project", "", "href='?cmd=addProject'");
					$m[] = new BMenu("Edit news", "", "href='/news?group={$this->group}'");
					$m[] = new BMenu("Edit showcases", "", "href='/showcase?group={$this->group}'");
					$m[] = new BMenu("Edit documents", "", "href='/document?cmd=viewList&group={$this->group}'");
					$m[] = new BMenu("Help", "",		"href='{$help}'");
					$o = new BMenu("Group", "", "", $m);
				}
			}
		}
		else {
			if($this->group == ""){
				$o = new BMenu("Groups", "", "", array(
					new BMenu("List All", "", "href='{$this->sys->baseUrl}/groups?cmd=groupList'"),
					));
			}
			else {
				$o = new BMenu("Group", "", "", array(
					new BMenu("Group Home page", "", "href='/groups/{$this->group}/index.html'"),
					new BMenu("View Content", "", "href='?cmd=viewContent'"),
					new BMenu("List All", "", "href='/groups?cmd=groupList'"),
					new BMenu("News", "", "href='/news?group={$this->group}'"),
					new BMenu("Showcases", "", "href='/showcase?group={$this->group}'"),
					new BMenu("Documents", "", "href='/document?cmd=viewList&group={$this->group}'"),
					));
			}
		}
		return $o;
	}

	function groupList($arg){
		$order = "name";
		
		if($this->checkAccess()){
			$view = new BViewDatabase("group", "action");
			$view->setView(array(
				new BViewType("id"),
				new BViewType("name"),
				new BViewType("title", "Title", "link", array("\$obj->title", $this->sys->baseUrl ."/groups/\$obj->name/index.html")),
				new BViewType("description"),
				));

			$view->setOrder($order);
			$view->setTable("Groups");
			
			$page = "<h2>{$this->title}</h2>";
			$page .= $view->getHtml();
		}
		else {
			$p = new BHtmlNode("div");
			$p->appendNode("h2", "", $this->title);

			$t = &$p->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");
			
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "style='background-color:yellow;'", $this->title);
			$r->appendNode("th", "style='background-color:yellow;'", "Description");

			$list = $this->sys->db->getObjectList("Groups", "", "", $order, "");

			foreach($list as $group){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "")->appendNode("a", "href='{$this->sys->baseUrl}/groups/{$group->name}/index.html'", $group->title);
				$r->appendNode("td", "", $group->description);
			}
			$page = $p->render();
		}
		return $page;
	}
	
	function groupView($arg){
		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject("Groups", filterToInt($_GET["id"]));
			header("Location: {$this->sys->baseUrl}/groups/{$obj->name}/index.html");
		}
		else if($this->group){
			header("Location: {$this->sys->baseUrl}/groups/{$this->group}/index.html");
			return;
		}
		else {
			return "No group selected";
		}
	}

	function viewContent($uri){
		if(isset($_GET["id"])){
			$obj = $this->sys->db->getObject("Groups", filterToInt($_GET["id"]));
			$file = "{$this->sys->baseDir}/files/groups/{$obj->name}/index.html";
		}
		else if($this->group){
			$file = "{$this->sys->baseDir}/files/groups/{$this->group}/{$this->arg}";
		}
		else {
			return "No group selected";
		}

		if(file_exists($file)){
			if (ob_get_level()) ob_end_clean();
			readfile($file);
			return null;
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Page: $uri: not found"));
			return "";
		}

	}

	function groupEdit($arg){
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Group</h2>";
			$obj = $this->sys->db->getObject("Groups", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Groups");
		}

		$view = new BEditObj("group", "Group Information", "?cmd=groupSave", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("name"),
			new BViewType("title"),
			new BViewType("description"),
			new BViewType("postCodes"),
			new BViewType("organiserId"),
			new BViewType("web"),
			));

		$view->setHelp("groupName", "Enter the groups name.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		return $view->getHtml();
	}
	
	function groupSave(){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjGroups();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->groupEdit($obj);
			}

			if($obj->id){
				$this->sys->db->putObject("Groups", $obj);
			}
			else {
				$this->sys->db->appendObject("Groups", $obj);
				$dir = $this->sys->baseDir . "/files/groups/" . $obj->name;

				if(!file_exists($dir . "/index.html")){
					mkdir($dir);

					$h = new BHtmlNode("html");
					$h->appendNode("head");
					$b = &$h->appendNode("body");
					$b->appendNode("h1", "", $obj->title);
					$t = &$b->appendNode("table", "border='0' cellpadding='0' cellspacing='0' width='100%'");
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan='2'", "<script language='php'>documentList()</script>");
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "valign='top'", "<script language='php'>newsList()</script>");
					$r->appendNode("td", "valign='top'", "<script language='php'>showcaseGroup()</script>");
					file_put_contents($dir . "/index.html", $h->render());
				}
			}
		}

		$page .= "<h3>Details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function groupDelete(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("Groups", $id);
			$this->sys->db->deleteObject("Groups", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("Groups", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=groupDelete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function groupEditInfo($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->edit("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}

	function groupEditInfoSave($uri){
		$module = $this->sys->modules["content"];
		$page = $module->save($uri);
		return $page;
	}

	function groupAddPage($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addPage("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}
	function groupAddPage1($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addPage1("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}

	function groupAddDir($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addDirectory("files/groups/" . $this->group . "/" . $this->arg);
		$page .= "groupAddDir: For {$this->group}<br>";
		return $page;
	}

	function groupAddDir1($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addDirectory1("files/groups/" . $this->group . "/" . $this->arg);
		return $page;
	}

	function addProject($uri){
		$_SESSION["content.templateFile"] = "/modules/groupsWiki/templateProject.html";
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->addDirectory("files/groups/" . $this->group . "/" . $this->arg);
		$page .= "addProject: For {$this->group}<br>";
		return $page;
	}

	function groupUploadFile($uri){
		$module = $this->sys->modules["content"];
		if($this->arg == "")
			$this->arg = "index.html";
		$page = $module->editFile(0);
		return $page;
	}

	function parseUri($uri){
		parent::parseUri($uri);
		
		$this->group = "";
		$this->arg = "";

		if(strpos($uri, "/")){
			$this->group = substr($uri, strpos($uri, "/") + 1);
			$this->group = filterPath($this->group, $this->sys->baseDir);
			if(strpos($this->group, "/")){
				$this->arg = substr($this->group, strpos($this->group, "/") + 1);
				$this->group = substr($this->group, 0, strpos($this->group, "/"));
			}
		}
	}
};

?>
