<?php
/*******************************************************************************
 *	Info.php	Info Module
 *			T.Barnaby,	BEAM Ltd,	2008-09-02
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";


class BObjInfo extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("grandparentId", "Int", "Number"),
			new BMember("parentId", "Int", "Number"),
			new BMember("reference", "String"),
			new BMember("completed", "Bool"),
			new BMember("state", "String"),
			new BMember("title", "String"),
			new BMember("type", "String"),
			new BMember("userName", "String"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("description", "StringHtml"),
			new BMember("lastUpdate", "DateTime", "Datetime", Null, "timestamp", "default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP"),
			
			new BMember("category", "String"),
			new BMember("priority", "Int", "Number"),
			new BMember("version", "String"),
			new BMember("versionDone", "String"),
			new BMember("manDays", "Float"),

			new BMember("attachment", "String"),

			));
	}
};

class Info extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "info";
		$this->title = "Info";
		$this->info = "Info module. This module allows the management of Info items.";
		$this->category = "";
		$this->priority = 1;
		$this->objId = 0;
		$this->test = 0;
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("Info" => "BObjInfo"));
		
		// Initialise categories if needed
		if(configGet("info.states") == "")
			configSet("info", "states", "infoNeeded,test,info,duplicate,leave,notabug,pending,done");
		if(configGet("info.statesCompleted") == "")
			configSet("info", "statesCompleted", "info,duplicate,leave,notabug,done");
		
		$this->categoryList = stringListToArray(configGet("info.categories"));
		$this->stateList = stringListToArray(configGet("info.states"));
	}

	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("categories", "StringList", "", "Category List", "Category list (comma separated)");
		$configs[] = new BObjConfig("categoryDefault", "StringList", "", "Default Category", "Default Category");
		$configs[] = new BObjConfig("states", "StringList", "", "State List", "State list (comma separated)");
		$configs[] = new BObjConfig("statesCompleted", "StringList", "", "Completed State List", "Completed State list (comma separated)");
		$configs[] = new BObjConfig("emailList", "StringList", "", "Email List", "Email list for interested parties (comma separated)");
		return $configs;
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin");
	}
		
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Info", "/{$this->name}", "", array("Info List" => "/{$this->name}", "Info Add" => "/{$this->name}?cmd=infoAdd"));
		}
		else {
			return Null;
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$help = $this->sys->baseUrl . "/modules/info/info.html";
			$o = new BMenu("Info", "", "", array(
				new BMenu("List",		"",	"href='?cmd=infoList'"),
				new BMenu("List Formatted",	"",	"href='?cmd=infoListHtml'"),
				new BMenu("Add New Item",		"",	"href='?cmd=infoAdd'"),
				new BMenu("<b>Item functions</b>", "", ""),
				new BMenu("Append to Item",		"",	"href='?cmd=infoAppend'"),
				new BMenu("Mark item as done",		"",	"href='?cmd=infoDone'"),
				new BMenu("<b>Management</b>", "", ""),
				new BMenu("Configuration", 		"",	"href='?cmd=configEdit'"),
//				new BMenu("Fixup database",		"",	"href='?cmd=fixup'"),
				new BMenu("", "", ""),
				new BMenu("Help",			"",	"href='{$help}'"),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}
	
	function getPage($uri){
		$args = $_GET;

		if(isset($args['cmd']))
			$cmd = filterString($args['cmd']);
		else
			$cmd = "";
		
		$this->sessionRead();

		switch($cmd){
		case 'infoAdd':		$r = $this->infoEdit($args); break;
		case 'infoAppend':	$r = $this->infoAppend($args); break;
		case 'infoEdit':	$r = $this->infoEdit($args); break;
		case 'infoView':	$r = $this->infoView($args); break;
		case 'infoSave':	$r = $this->infoSave($args); break;
		case 'infoDelete':	$r = $this->infoDelete($args); break;
		case 'infoDone':	$r = $this->infoDone($args); break;
		case 'infoListHtml':	$r = $this->infoListHtml($args); break;
		case "configEdit":	$r = $this->configEdit($uri); break;
		case 'fixup':		$r = $this->fixup($args); break;
		case 'infoList':
		default:
					$r = $this->infoList($args);
					break;
		}

		$this->sessionWrite();
		return $r;
	}
	

	function getObject($id){
		$obj = $this->sys->db->getObject("Info", $id);
		if($obj->grandparentId){
			$gobj = $this->sys->db->getObject("Info", $obj->grandparentId);
			$obj->reference = $gobj->reference;
			$obj->category = $gobj->category;
			$obj->priority = $gobj->priority;
			$obj->version = $gobj->version;
			$obj->versionDone = $gobj->versionDone;
			$obj->manDays = $gobj->manDays;
		}
		return $obj;
	}
	
	function putObject($obj){
		if($obj->id){
			$this->sys->db->putObject("Info", $obj);
		}
		else {
			$obj = $this->sys->db->appendObject("Info", $obj);
		}
		if($obj->grandparentId){
			$gobj = $this->sys->db->getObject("Info", $obj->grandparentId);
			$gobj->reference = $obj->reference;
			$gobj->category = $obj->category;
			$gobj->priority = $obj->priority;
			$gobj->version = $obj->version;
			$gobj->versionDone = $obj->versionDone;
			$gobj->manDays = $obj->manDays;
			$this->sys->db->putObject("Info", $gobj);
		}
	}

	function getObjectList($table, $fields = "", $selection = "", $order = "", $limit = ""){
		$list = $this->sys->db->getObjectList($table, $fields, $selection, $order, $limit);
		
		foreach($list as &$obj){
			if($obj->grandparentId){
				$obj = $this->getObject($obj->id);
			}
		}
		
		return $list;
	}
	
	function sessionRead(){
		if(isset($_SESSION["info.objId"]))
			$this->objId = $_SESSION["info.objId"];
	}
	function sessionWrite(){
		$_SESSION["info.objId"] = $this->objId;
	}
	
	function infoList($args){
		$page = "";

		$sel = new BSelect("select", "Select", "", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("category", "", "", "likeAfterStart", "select", array_merge(array(""), $this->categoryList)),
			new BViewTypeSelect("completed", "", "", "equal", "tristate"),
//			new BViewTypeSelect("state", "", "", "equal", "select", array_merge(array("", "!done"), $this->stateList)),
			new BViewTypeSelect("state", "", "", "equal", "select", array_merge(array(""), $this->stateList)),
			new BViewTypeSelect("search", "", "", "", "text", ["width" => 300]),
			new BViewTypeSelect("order", "", "", "order", "select", array("!priority,category,id", "category,!priority,id", "!lastUpdate,category,!priority,id", "!dateAdded,!id", "id", "!id", "reference,id", "!reference,id")),
			new BViewTypeSelect("priority", "", "", "greaterThanEqual", "select", array(0, 1, 2, 3, 4, 5)),
			new BViewTypeSelect("fill0", "", "", "", "empty"),
			new BViewTypeSelect("reference", "", "", "likeAfterStart", "text", ["width" => 300]),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			));

		$sel->getObj()->category = configGet("info.categoryDefault");
		$sel->getObj()->completed = 0;
//		$sel->getObj()->state = "!done";
		$sel->getObj()->priority = 1;
		$sel->getObj()->order = "!priority,category,id";
		$sel->update();
		
		$_SESSION["completed"] = $sel->getObj()->completed;
		$_SESSION["state"] = $sel->getObj()->state;
		$_SESSION["category"] = $sel->getObj()->category;
		$_SESSION["priority"] = $sel->getObj()->priority;
		$_SESSION["order"] = $sel->getObj()->order;
		$_SESSION["srearch"] = $sel->getObj()->search;
		$_SESSION["reference"] = $sel->getObj()->reference;
		
		$where = "(parentId = 0) and " . $sel->getSelectString();
		if($sel->getObj()->search){
			$search = $sel->getObj()->search;
			if($where)
				$where .= " and ";
			$where .= "(`title` REGEXP '.*{$search}.*' or `description` REGEXP '.*{$search}.*')";
		}
		
		$totalNumber = $this->sys->db->getObjectNumber("Info", $where);
		$sel->pagerSet(0, 50, $totalNumber);

//		print "Sel: {$where}<br>";
//		print "Selection:"; print_r($sel->getObj()); print "<br>";

		$view = new BViewList("info", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("reference", "Ref"),
			new BViewType("category"),
			new BViewType("dateAdded", "Date Added", "date"),
			new BViewType("completed"),
			new BViewType("state"),
			new BViewType("priority"),
			new BViewType("manDays"),
			new BViewType("lastUpdate", "Last Update", "date"),
			new BViewType("userName"),
			new BViewType("title", "Title", "link", array("\$obj->id: \$obj->title", "?cmd=infoView&id={\$obj->id}")),
			));
			
		$list = $this->sys->db->getObjectList("Info", "", $where, $sel->getObj()->order, $sel->pagerGetLimitString());
		$totalNumber = $this->sys->db->getObjectNumber("Info", $where);
		$sel->pagerSet(0, 50, $totalNumber);
		$view->setList($list, 1);

		$this->objId = 0;

		$page = $page . $sel->getHtml();
		$page = $page . $view->getHtml();

		return $page;
	}

	function infoViewObjHtml($h, $obj, $n){
		if($obj->parentId == 0){
			$h->appendNode("h3", "", implode(".", $n) . ". " . $obj->title);
			$t = &$h->appendNode("div", "style='margin-left: 40px;'");
		}
		else {
			$h->appendNode("h4", "", implode(".", $n) . ". " . $obj->title);
			$t = &$h->appendNode("div", "style='margin-left: 40px;'");
		}

		$t->appendNode("b", "", "ID: ");	$t->appendText($obj->id);
		$t->appendNode("b", "", ", Reference: ");	$t->appendText($obj->reference);
		$t->appendNode("b", "", ", Category: ");	$t->appendText($obj->category);
		$t->appendNode("b", "", ", DateAdded: ");	$t->appendText($obj->dateAdded);
		$t->appendNode("b", "", ", UserName: ");	$t->appendText($obj->userName);
		$t->appendNode("br");
		$t->appendNode("b", "", "Priority: ");	$t->appendText($obj->priority);
		$t->appendNode("b", "", ", Version: ");	$t->appendText($obj->version);
		$t->appendNode("b", "", ", VersionDone: ");	$t->appendText($obj->versionDone);
		$t->appendNode("b", "", ", Completed: ");	$t->appendText($obj->completed);
		$t->appendNode("b", "", ", State: ");	$t->appendText($obj->state);
		$t->appendNode("b", "", ", ManDays: ");	$t->appendText($obj->manDays);
		$t->appendNode("b", "", ", LastUpdate: ");	$t->appendText($obj->lastUpdate);
		$t->appendNode("br");
		$t->appendText($obj->description);
		
		if($obj->attachment){
			$t->appendNode("br");
			$t->appendNode("b", "", "Attachment: "); $t->appendText($obj->attachment);
		}

		$items = $this->getObjectList("Info", "", "`parentId` = '{$obj->id}'", "", "");
		if(count($items)){
			$n[] = 1;
			foreach($items as $item){
				$this->infoViewObjHtml($t, $item, $n);
				$n[count($n) - 1]++;
			}
		}
	}
	
	function infoListHtml($args){
		$select = "";

		if(isset($_GET["reference"]))
			$this->reference = filterString($_GET["reference"]);
		else
			$this->reference = "";

		if(isset($_GET["category"]))
			$this->category = filterString($_GET["category"]);
		else
			$this->category = "";

		if(isset($_GET["completed"]))
			$this->completed = filterInt($_GET["completed"]);
		else
			$this->state = "!done";
		
		if(isset($_GET["state"]))
			$this->state = filterString($_GET["state"]);
		else
			$this->state = "!done";
		
		if(isset($_GET["priority"]))
			$this->priority = intval($_GET["priority"]);
		else
			$this->priority = "";

		if(isset($_GET["order"]))
			$this->order = filterStringParamList($_GET["order"]);
		else
			$this->order = "category,!priority,id";

		if(isset($_GET["search"]))
			$this->search = filterString($_GET["search"]);
		else
			$this->search = "";

		if(isset($_SESSION["reference"]))
			$this->reference = $_SESSION["reference"];
		if(isset($_SESSION["category"]))
			$this->category = $_SESSION["category"];
		if(isset($_SESSION["completed"]))
			$this->completed = $_SESSION["completed"];
		if(isset($_SESSION["state"]))
			$this->state = $_SESSION["state"];
		if(isset($_SESSION["priority"]))
			$this->priority = $_SESSION["priority"];
		if(isset($_SESSION["order"]))
			$this->order = $_SESSION["order"];

		$fields = array("id", "reference", "category", "dateAdded", "completed", "state", "priority", "manDays", "userName", "title");

		$h = new BHtmlNode("html");
		$b = &$h->appendNode("body");
		$b->appendNode("h1", "style='text-align:center;'", "{$this->sys->siteTitle} Information");

		$select = "`parentId` = '0' and `category` REGEXP '^{$this->category}.*$'";
		if($this->completed != "")
			$select .= " and `completed` = '{$this->completed}'";

		if($this->state != ""){
			if($this->state == "!done")
				$select .= " and `state` != 'done'";
			else
				$select .= " and `state` = '{$this->state}'";
		}

		if($this->priority != ""){
			$select .= " and `priority` >= '{$this->priority}'";
		}
		else {
			$select .= "";
		}

		if($this->reference != ""){
			$select .= " and (`reference` like '{$this->reference}%')";
		}

		if($this->search != ""){
			$select .= " and (`title` REGEXP '.*{$this->search}.*' or `description` REGEXP '.*{$this->search}.*')";
		}

//		print "Sel: {$select}<br>";
//		BDatabase::$debug = 1;
		$data = $this->getObjectList("Info", "", $select, $this->order);

		$n = 1;
		foreach($data as $obj){
			$this->infoViewObjHtml($b, $obj, array($n));
			$n = $n + 1;
		}
		
		print $h->render();

		$this->objId = 0;

		return null;
	}

	function infoViewObj($h, $obj, $n = 1){
//		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='0' style='border:1px solid black;'");
//		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='0' style='width:800px;'");
		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='0'");

		if($obj->parentId == 0){
			$r = &$t->appendNode("tr", "style='background:#B9DAEC'");
			$c = &$r->appendNode("td", "colspan=99", $obj->title);
		}

		$r = &$t->appendNode("tr", "style='background:#C6E9FC'");
		$c = &$r->appendNode("td", "", "");
		if($obj->parentId){
			$id = $this->findGrandparent($obj);
			$c->appendNode("a", "href='?cmd=infoView&id={$id}' title='Goto top most parent Item'", "P");
			$c->appendText("&nbsp");
		}
		$c->appendNode("a", "href='?cmd=infoEdit&id={$obj->id}' title='Edit Info Item'", "E");
		$c->appendText("&nbsp");
		$c->appendNode("a", "href='?cmd=infoAppend&id={$obj->id}' title='Add Sub Item'", "A");
		$c->appendText("&nbsp");
		$c->appendNode("a", "href='?cmd=infoDelete&id={$obj->id}' title='Delete Info Item'", "D");
		
		$r->appendNode("td", "", "{$obj->id}");
		$r->appendNode("td", "", "{$obj->reference}");
		$r->appendNode("td", "", "{$obj->category}");
		$r->appendNode("td", "", "{$obj->dateAdded}");
		$r->appendNode("td", "", "{$obj->userName}");
		$r->appendNode("td", "", "P: {$obj->priority}");
		$r->appendNode("td", "", "V: {$obj->version}");
		$r->appendNode("td", "", "VD: {$obj->versionDone}");
		$r->appendNode("td", "", "C: {$obj->completed}");
		$r->appendNode("td", "", "S: {$obj->state}");
		$r->appendNode("td", "", "M: {$obj->manDays}");
		$r->appendNode("td", "", "T: {$obj->lastUpdate}");

		$r = &$t->appendNode("tr");
//		$r->appendNode("td", "colspan=99 style='border:1px solid black;'", $obj->description);
		$r->appendNode("td", "colspan=99", $obj->description);
		
		if($obj->attachment){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "colspan=99");
			$c->appendNode("b", "", "Attachment: ");
			$fileurl = urlPath("/files/module_info/{$obj->attachment}");
			$c->appendNode("a", "href='{$fileurl}'", "$obj->attachment");
		}

		$items = $this->getObjectList("Info", "", "`parentId` = '{$obj->id}'", "", "");
		if(count($items)){
			$did = "d" . strval($n);
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("th", "align='left'", "Sub Items");
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "colspan=99 style='border:1px solid black;'");
//			$c = &$r->appendNode("td", "colspan=99");
			if(0){
				if($obj->parentId == 0)
					$d = &$c->appendNode("div", "id='{$did}' style='height:550px; overflow: auto; margin-left:10px'");
				else
					$d = &$c->appendNode("div", "id='{$did}' style='overflow: auto; margin-left:10px'");
			}
			else {
				$d = &$c->appendNode("div", "id='{$did}'");
			}

			foreach($items as $item){
				$this->infoViewObj($d, $item, $n + 1);
			}
			$c->appendNode("script", "type='text/javascript'", "var obj = document.getElementById('{$did}'); obj.scrollTop = obj.scrollHeight - obj.clientHeight;");
		}
		if(0){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "colspan=99");
			$c->appendNode("a", "href='?cmd=infoEdit&id={$obj->id}'", "Edit");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=infoAppend&id={$obj->id}'", "AddSubItem");
			$c->appendText("&nbsp");
			$c->appendNode("a", "href='?cmd=infoDelete&id={$obj->id}'", "Delete");
		}
	}
	
	function infoView($args){
		$page = "";

		if(isset($args["id"])){
			$obj = $this->getObject(filterToInt($args["id"]));
		}
		else {
			return "No Object";
		}
		
		$this->objId = $obj->id;

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='' method='post'");
		$f->appendNode("input", "type='hidden' name='id' value='{$obj->id}'");

		$this->infoViewObj($f, $obj);
//		$f->appendNode("a", "href='?cmd=infoAppend&id={$obj->id}'", "Add Comment");

		$page = $page . $f->render();
		
		// Add an item
		$objNew = $this->sys->db->getNewObject("Info");
		$objNew->parentId = $obj->id;
		if($obj->grandparentId)
			$objNew->grandparentId = $obj->grandparentId;
		else
			$objNew->grandparentId = $obj->id;

		$objNew->completed = 0;
		$objNew->title = $obj->title;
		$objNew->reference = $obj->reference;
		$objNew->category = $obj->category;
		$objNew->priority = $obj->priority;
		$objNew->version = $obj->version;
		$objNew->versionDone = $obj->versionDone;
		$objNew->manDays = $obj->manDays;
		$objNew->dateAdded = strftime("%Y-%m-%d");
		if(isset($_SESSION["category"]))
			$objNew->category = $_SESSION["category"];

		$page .= "<h3>Append an item</h3>";
		$page .= $this->infoEditObj($objNew, 1);

		return $page;
	}

	function infoEditObj($obj, $appendInfo = 0){
		$page = "";
		
		// Add view
		if(0 && $obj->parentId){
			$h = new BHtmlNode("div");
			$parentObj = $this->sys->db->getObject("Info", $obj->parentId);
			$this->infoViewObj($h, $parentObj);
			$page .= $h->render();
		}
		
		$view = new BEditObj("form_info", "Edit info", "?cmd=infoSave", "");
		if($appendInfo){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("dateAdded", "", "hidden"),

				new BViewType("title", "Title", "hidden", "", 600),
				new BViewType("reference", "Reference", "hidden", ""),
				new BViewType("category", "Category", "hidden", array_merge(array(""), $this->categoryList)),
				new BViewType("priority", "Priority", "hidden", array_merge(array(""), array(0, 1, 2, 3, 4, 5))),
				new BViewType("version", "Version", "hidden"),
				new BViewType("versionDone", "Version Done", "hidden"),
				new BViewType("manDays", "Man Days", "hidden"),

				new BViewType("head1", "This items fields", "heading"),
				new BViewType("completed", "Completed", "hidden"),
				new BViewType("state", "State", "select", array_merge(array(""), $this->stateList)),
				new BViewType("description", "Description", "textareaHtmlPics", "", 600, 200),
				new BViewType("attachment", "", "hidden"),
				new BViewType("attachmentFile", "Attach File", "pictureUploaderLineMedium"),

				new BViewType("head1", "Management", "heading"),
				new BViewType("userName", "", "hidden"),
				new BViewType("markStateDone", "Mark all done", "checkbox"),
				new BViewType("markState", "Mark all to state", "checkbox"),
				new BViewType("emailSend", "Email originator", "checkbox"),
				new BViewType("emailSendAll", "Email all", "checkbox"),
			));
		}
		else {
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("grandparentId", "", "hidden"),
				new BViewType("parentId", "", "hidden"),
				new BViewType("dateAdded", "", "hidden"),

				new BViewType("title", "Title", "text", "", 600),
				new BViewType("reference", "Reference", "text", ""),
				new BViewType("category", "Category", "select", array_merge(array(""), $this->categoryList)),
				new BViewType("priority", "Priority", "select", array_merge(array(""), array(0, 1, 2, 3, 4, 5))),
				new BViewType("version", "Version"),
				new BViewType("versionDone", "Version Done"),
				new BViewType("manDays", "Man Days"),

				new BViewType("head1", "This items fields", "heading"),
				new BViewType("completed", "Completed", "checkbox"),
				new BViewType("state", "State", "select", array_merge(array(""), $this->stateList)),
				new BViewType("description", "Description", "textareaHtmlPics", "", 600, 200),
				new BViewType("attachment", "", "hidden"),
				new BViewType("attachmentFile", "Attach File", "pictureUploaderLineMedium"),

				new BViewType("head1", "Management", "heading"),
				new BViewType("userName", "", "view"),
				new BViewType("markStateDone", "Mark all done", "checkbox"),
				new BViewType("markState", "Mark all to state", "checkbox"),
				new BViewType("emailSend", "Email originator", "checkbox"),
				new BViewType("emailSendAll", "Email all", "checkbox"),
			));
		}
		
		$obj->emailSend = 1;
		$obj->emailSendAll = 0;
		$view->setObj($obj);
		$page .= $view->getHtml();

		return $page;
	}

	function infoEdit($args){
		if(isset($args["id"])){
			$obj = $this->getObject(filterToInt($args["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Info");
			$obj->completed = 0;
			$obj->priority = 1;
			$obj->dateAdded = strftime("%Y-%m-%d");
			if(isset($_SESSION["category"]))
				$obj->category = $_SESSION["category"];
		}
		
		$this->objId = $obj->id;

		return $this->infoEditObj($obj);
	}

	function infoAppend($args){
		if(isset($args["id"])){
			$id = filterToInt($args["id"]);
		}
		elseif($this->objId){
			$id = $this->objId;
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No item to append to."));
			exit();
		}

		$o = $this->getObject($id);
		$obj = $this->sys->db->getNewObject("Info");
		if($o->grandparentId)
			$obj->grandparentId = $o->grandparentId;
		else
			$obj->grandparentId = $o->id;
		$obj->parentId = $o->id;
		$obj->completed = 0;
		$obj->title = $o->title;
		$obj->reference = $o->reference;
		$obj->category = $o->category;
		$obj->priority = $o->priority;
		$obj->version = $o->version;
		$obj->versionDone = $o->versionDone;
		$obj->manDays = $o->manDays;
		$obj->dateAdded = strftime("%Y-%m-%d");
		
		return $this->infoEditObj($obj);
	}

	function findGrandparent($obj){
		while($obj->parentId){
			$obj = $this->sys->db->getObject("Info", $obj->parentId);
		}
		return $obj->id;
	}
	
	function updateLastUpdate($id){
		$obj = $this->sys->db->getObject("Info", $id);
		$obj->lastUpdate = strftime("%Y-%m-%dT%H:%M:%S");
		$this->sys->db->query("update {$this->sys->databasePrefix}Info set `lastUpdate` = '{$obj->lastUpdate}' where `id` = '{$obj->id}'");
		if($obj->parentId)
			$this->updateLastUpdate($obj->parentId);
	}
	
	function infoMarkState($id, $state){
//		print "MarkObject: {$id}<br>";
		$obj = $this->sys->db->getObject("Info", $id);
		$obj->completed = 1;
		$obj->state = $state;
		$this->sys->db->putObject("Info", $obj);
		
		if($obj->parentId){
			$objList = $this->sys->db->getObjectList("Info", "id", "`id` = '{$obj->parentId}'");
			foreach($objList as $ids){
				$this->infoMarkState($ids->id, $state);
			}
		}
	}
	
	function infoDone($args){
		$page = "";	

		if($this->objId){
			$this->infoMarkState($this->objId, "done");
			$obj = $this->sys->db->getObject("Info", $this->objId);
			$page .= "Info item: '{$obj->title}' and its parents have been marked as done.";
		}
		else {
			$page .= "No object is selected.";
		}
		
		$page .= "<br><a href='?cmd=infoViewList'>Return to list</a>";
		return $page;
	}
	
	function infoSave($args){
		$page = "";
		$attachmentPresent = 0;
		
		//print "Save: "; print_r($_GET); print_r($_POST); print_r($_FILES);

		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjInfo();
			$obj->set($_POST);
			if($obj->id == 0){
				$obj->dateAdded = strftime("%Y-%m-%d");
				$obj->userName = $this->sys->user->name;
			}
			
			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->infoEditObj($obj);
			}

			if((count($_FILES) == 1) && isset($_FILES["attachmentFile"])){
				$attachmentPresent = 1;
			}

//			BDatabase::$debug = 1;
			$this->putObject($obj);
			
			if($attachmentPresent){
				$filename = "{$obj->id}_" . $_FILES["attachmentFile"]["name"];
				$obj->attachment = $filename;

				print "Save file: {$filename } {$obj->id}<br>";

				// Move file
				if(!file_exists(filePathAbs("/files/module_info"))){
					mkdir(filePathAbs("/files/module_info"));
				}
				if(!move_uploaded_file($_FILES["attachmentFile"]["tmp_name"], filePathAbs("/files/module_info/{$filename}")))
					$page .= "Move Error";

				$this->putObject($obj);
			}
			

			if(isset($_POST['markState']) and ($_POST['markState'] == "1")){
				//print "Set objects as done: {$obj->id}<br>";
				$this->infoMarkState($obj->id, $obj->state);
			}
			
			if(isset($_POST['markStateDone']) and ($_POST['markStateDone'] == "1")){
				//print "Set objects as done: {$obj->id}<br>";
				$this->infoMarkState($obj->id, "done");
			}

			$emailSubject = "[{$this->sys->siteTitle}]: Info Changed: {$obj->category}: {$obj->title}";
			if($obj->parentId){
				$this->updateLastUpdate($obj->parentId);
				if(isset($_POST['emailSend']) and ($_POST['emailSend'] == "1")){
					// Send an email to owner of this item
					$grandparentObj = $this->sys->db->getObject("Info", $obj->grandparentId);

					$mesg = "{$obj->title}\n\n{$obj->description}\n\n<br>------------------------------------------------<br>\n";
					$mesg .= "More information is available at: <a href='{$this->sys->baseUrl}/info?cmd=infoView&id={$obj->grandparentId}'>{$this->sys->baseUrl}/info?cmd=infoView&id={$obj->grandparentId}</a>";
					$page .= "Sent email to: {$grandparentObj->userName}<br>";
					if(!$this->test)
						emailUsers($grandparentObj->userName, $emailSubject, $mesg, 1);
				}
			}

			if(isset($_POST['emailSendAll']) and ($_POST['emailSendAll'] == "1")){
				// Send an email to all interested users
				$emailList = configGet("info.emailList");
				$mesg = "{$obj->title}\n\n{$obj->description}\n\n<br>------------------------------------------------<br>\n";
				$mesg .= "More information is available at: <a href='{$this->sys->baseUrl}/info?cmd=infoView&id={$obj->id}'>{$this->sys->baseUrl}/info?cmd=infoView&id={$obj->id}</a>";
				$page .= "Sent emails to {$emailList}<br>";
				if(!$this->test){
					emailList($emailList, $emailSubject, $mesg, 1);
				}
			}
		}

		if(intval($obj->grandparentId) > 0){
			$page .= "Info details saved<br><a href='?cmd=infoView&id={$obj->grandparentId}'>Return to item</a>";
		}
		else {
			$page .= "Info details saved<br><a href='?cmd=infoViewList'>Return to list</a>";
		}
		
		return $page;
	}
	
	function infoDeleteObj($obj){
		// Delete all sub objects
		$items = $this->sys->db->getObjectList("Info", "", "`parentId` = '{$obj->id}'", "", "");
		foreach($items as $item){
			$page .= "Delete SubInfo: {$item->id}<br>\n";
			$this->infoDeleteObj($item);
		}

		if($obj->attachment){
			$filePath = filePathAbs("/files/module_info/{$obj->attachment}");
			if(file_exists($filePath))
				unlink($filePath);
		}
		$this->sys->db->deleteObject("Info", $obj->id);

	}

	function infoDelete($args){
		$page = "";

		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if(isset($args["id"]) && ($args["id"] != "0")){
			$obj = $this->sys->db->getObject("Info", filterToInt($args["id"]));
		}
		else {
			return "No object selected";
		}

		$obj = $this->sys->db->getObject("Info", filterToInt($args["id"]));

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			// Delete all sub objects
			$this->infoDeleteObj($obj);
			$page = "<h3>Info: {$obj->id} Titled: {$obj->title} has been Deleted</h3>";
		}
		else {
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=infoDelete&id={$obj->id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete info item '{$obj->id}' of '{$obj->title}' and all of its sub items ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete Info'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");
			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}
		
		return $page;
	}

	function fixup(){
		$changed = 0;
		$statesCompleted = stringListToArray(configGet("info.statesCompleted"));

		$categories = [];
		for($pos = 0; ; $pos += 100){
			$list = $this->sys->db->getObjectList("Info", "", "", "", "{$pos},100");
			if(!count($list))
				break;

			foreach($list as $obj){
				$changed = 0;
				if($obj->parentId != 0){
					$obj->grandparentId = $this->findGrandparent($obj);
					$changed = 1;
				}

				if(in_array($obj->state, $statesCompleted)){
					if($obj->completed == 0){
						$obj->completed = 1;
						$changed = 1;
					}
				}
				else {
					if($obj->completed == 1){
						$obj->completed = 0;
						$changed = 1;
					}
				}
					
				$categories[$obj->category] = 1;
				
				if($changed)
					$this->sys->db->putObjectRaw("Info", $obj);
			}
		}
		
		if(configGet("info.categories") == "")
			configSet("info", "categories", stringListFromArray(array_keys($categories)));

		return "Database fixed";
	}
};

?>
