<?php
/*******************************************************************************
 *	Journals.php		Tests form input module
 *				T.Barnaby,	BEAM Ltd,	2016-02-09
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";

class BObjJournal extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Int"),
			new BMember("dateAdded", "DateTime", "Datetime"),
			new BMember("date", "Date", "Date"),
			new BMember("title", "String"),
			new BMember("pdf", "String"),
			new BMember("mp3_all", "String"),
			new BMember("mp3_1", "String"),
			new BMember("mp3_2", "String"),
			new BMember("mp3_3", "String"),
			new BMember("mp3_4", "String"),
			new BMember("mp3_5", "String"),
			new BMember("mp3_6", "String"),
			new BMember("mp3_7", "String"),
			new BMember("mp3_8", "String"),
			new BMember("mp3_9", "String"),
			new BMember("mp3_10", "String"),
			new BMember("mp3_11", "String"),
			new BMember("mp3_12", "String"),
			new BMember("mp3_13", "String"),
			new BMember("mp3_14", "String"),
			new BMember("mp3_15", "String"),
			new BMember("mp3_16", "String"),
			new BMember("mp3_17", "String"),
			new BMember("mp3_18", "String"),
			new BMember("mp3_19", "String"),
			new BMember("mp3_20", "String"),
			new BMember("mp3_21", "String"),
			new BMember("mp3_22", "String"),
			new BMember("mp3_23", "String"),
			new BMember("mp3_24", "String"),
			new BMember("mp3_25", "String"),
		));
	}
};

function filterFilename($str){
	return str_replace(" ", "_", $str);
}

class Journals extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "journals";
		$this->title = "Journals";
		$this->info = "Journals Module";
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("Journals" => "BObjJournal"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminJournals"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "journalsLatest";
	}
	
	function checkAccess(){
		return $this->sys->login->checkLogin("admin,adminJournals");
	}
		

	// Standard interface
	function getMenu(){
		return new BMenu("Journals", "/{$this->name}", "", array(
			"Demo List" => "/{$this->name}",
			));
	}

	function getPage($uri){
		$page = "";
		
		$this->parseUri($uri);

		// Check if file access	
		if(strlen($uri) > (strlen($this->name) + 1)){
			if(is_dir(filePathAbs($uri))){
				$module = $this->sys->modules["contentDir"];
			}
			else {
				$module = $this->sys->modules["content"];
			}
			return $module->getPage($uri);
		}

		if($this->checkAccess()){
			switch($this->cmd){
			case "editList":	$page = $this->editList($uri); break;
			case "view":		$page = $this->view($uri); break;
			case "add":		$page = $this->edit($uri); break;
			case "edit":		$page = $this->edit($uri); break;
			case "save":		$page = $this->save($uri); break;
			default: 		$page = $this->viewList($uri); break;
			}
		}
		else {
			$page = $this->viewList($uri);
		}
		
		return $page;
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$help = "/modules/journals/info.html";

			$o = new BMenu("Journals", "", "", array(
				new BMenu("ViewList", "",	"href='/{$this->name}'"),
				new BMenu("Edit Journals", "",	"href='/{$this->name}?cmd=editList'"),
				new BMenu("Add Journal", "",	"href='/{$this->name}?cmd=add'"),
				new BMenu("Help", "",		"href='{$help}'"),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}
	
	function editList(){
		$page = "<h2>Journals Edit List</h2>\n";
		
//		print "Form: "; print_r($_POST); print "<br>\n";
//		BDatabase::$debug = 1;

		$view = new BViewDatabase("list", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("dateAdded", "DateAdded", "datetime"),
			new BViewType("date"),
			new BViewType("title"),
			));

		$view->setOrder("!dateAdded");
		$view->setTable("Journals");
		$page = $page . $view->getHtml();
		
		return $page;
	}
	
	function viewList($uri){
		$p = new BHtmlNode("div");
		$p->appendNode("h2", "", "Journals List");

		$t = &$p->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$list = $this->sys->db->getObjectList("Journals", "", "", "!date", "");

		$col = 0;
		foreach($list as $obj){
			$baseUrl = "/" . $uri . $obj->id;
			$date = new BDate($obj->date);
			$title = "Journal - " . BDate::monthName($date->getMonth()) . " " . $date->getYear();
			
			if($col == 0){
				$r = &$t->appendNode("tr");
				$col = 1;
			}
			else {
				$col = 0;
			}
			$r->appendNode("td")->appendNode("a", "href='{$baseUrl}/{$obj->pdf}'")->appendNode("img", "src='{$baseUrl}/icon.png'");
			$c = &$r->appendNode("td", "style='vertical-align:top;'");
			$c->appendNode("h2", "", $title);
			$c->appendNode("a", "href='{$baseUrl}/{$obj->pdf}'", "PDF File");
			$c->appendNode("br");
			if($obj->mp3_all){
				$c->appendNode("a", "href='{$baseUrl}/{$obj->mp3_all}'", "MP3 Audio Complete Journal");
				$c->appendNode("br");
				$c->appendNode("a", "href='{$baseUrl}'", "MP3 Audio Individual Tracks");
				$c->appendNode("br");
				
			}
			if(0){
				for($n = 1; $n < 21; $n++){
					$item = "mp3_{$n}";
					if($obj->{$item}){
						$filename = $obj->{$item};
						$c->appendNode("a", "href='{$baseUrl}/{$filename}'", "MP3 Audio Track: {$filename}");
						$c->appendNode("br");
					}
				}
			}
		}

		$p->appendNode("br");
		$p->appendNode("a", "href='../tcj'", "Old Journals");		
		$page = $p->render();
		return $page;
	}
	
	function view($arg){
		$page = "<h2>View Journals</h2>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$obj = $this->sys->db->getObject("Journals", filterInt($_GET["id"]));
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		
		$view = new BViewObj("journal", "Journal");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("dateAdded", "DateAdded", "datetime"),
			new BViewType("date"),
			new BViewType("title"),
			new BViewType("pdf"),
			new BViewType("mp3_all"),
			new BViewType("mp3_1"),
			new BViewType("mp3_2"),
			new BViewType("mp3_3"),
			new BViewType("mp3_4"),
			new BViewType("mp3_5"),
			new BViewType("mp3_6"),
			new BViewType("mp3_7"),
			new BViewType("mp3_8"),
			new BViewType("mp3_9"),
			new BViewType("mp3_10"),
			new BViewType("mp3_11"),
			new BViewType("mp3_12"),
			new BViewType("mp3_13"),
			new BViewType("mp3_14"),
			new BViewType("mp3_15"),
			new BViewType("mp3_16"),
			new BViewType("mp3_17"),
			new BViewType("mp3_18"),
			new BViewType("mp3_19"),
			new BViewType("mp3_20"),
			new BViewType("mp3_21"),
			new BViewType("mp3_22"),
			new BViewType("mp3_23"),
			new BViewType("mp3_24"),
			new BViewType("mp3_25"),
			));

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		return $page;
	}

	function edit($arg, $payDo = 0){
		$args = "";
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			if(filterInt($_GET["id"]) == 0)
				return "No record id 0.";
			$page = "<h2>Edit Journal</h2>";
			$obj = $this->sys->db->getObject("Journals", filterInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Journals");
			$obj->id = 0;
			$obj->dateAdded = strftime("%Y-%m-%d %H:%M:%S");
			$obj->date = strftime("%Y-%m-%d");
			$obj->title = "Journal";
		}
		
		$obj->sendEmail = 0;
		
		$view = new BEditObj("journal", "Journal", "?cmd=save", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("dateAdded", "DateAdded", "hidden"),
			new BViewType("date", "Date", "date"),
			new BViewType("title"),

			new BViewType("pdf", "Journal PDF", "fileEdit"),
			new BViewType("mp3_1", "MP3 track 1", "fileEdit"),
			new BViewType("mp3_2", "MP3 track 2", "fileEdit"),
			new BViewType("mp3_3", "MP3 track 3", "fileEdit"),
			new BViewType("mp3_4", "MP3 track 4", "fileEdit"),
			new BViewType("mp3_5", "MP3 track 5", "fileEdit"),
			new BViewType("mp3_6", "MP3 track 6", "fileEdit"),
			new BViewType("mp3_7", "MP3 track 7", "fileEdit"),
			new BViewType("mp3_8", "MP3 track 8", "fileEdit"),
			new BViewType("mp3_9", "MP3 track 9", "fileEdit"),
			new BViewType("mp3_10", "MP3 track 10", "fileEdit"),
			new BViewType("mp3_11", "MP3 track 11", "fileEdit"),
			new BViewType("mp3_12", "MP3 track 12", "fileEdit"),
			new BViewType("mp3_13", "MP3 track 13", "fileEdit"),
			new BViewType("mp3_14", "MP3 track 14", "fileEdit"),
			new BViewType("mp3_15", "MP3 track 15", "fileEdit"),
			new BViewType("mp3_16", "MP3 track 16", "fileEdit"),
			new BViewType("mp3_17", "MP3 track 17", "fileEdit"),
			new BViewType("mp3_18", "MP3 track 18", "fileEdit"),
			new BViewType("mp3_19", "MP3 track 19", "fileEdit"),
			new BViewType("mp3_20", "MP3 track 20", "fileEdit"),
			new BViewType("mp3_21", "MP3 track 21", "fileEdit"),
			new BViewType("mp3_22", "MP3 track 22", "fileEdit"),
			new BViewType("mp3_23", "MP3 track 23", "fileEdit"),
			new BViewType("mp3_24", "MP3 track 24", "fileEdit"),
			new BViewType("mp3_25", "MP3 track 25", "fileEdit"),
			new BViewType("sendEmail", "Send email to all interested", "checkbox"),
			));

		$view->setObj($obj);
		$page .= "Please use short and meaningfull file names with track number encoded early on so that the files will list in track order. ";
		$page .= "This allows screen reader software to be used more easily.<br>";
		$page .= "Setting the \"Send email to all interested\" checkbox tells the system to send an email to all interested people with a link to the MP3 and PDF versions. ";
		$page .= "TC Members with the \"Journal sent by email\" option set in their membership record will receive these. ";
		$page .= $view->getHtml();
		return $page;
	}
	
	function save($uri){
		$page = "";

		if(0){
			print "Journals::save: {$uri}<br>";
			$post = "";
			foreach($_POST as $n => $v) $post .= $n . " => " . $v . ",";
			debug_log("Journals::save: Post: $post");
			$files = "";
			foreach($_FILES as $f) $files .= $f["name"] . ",";
			debug_log("Journals::save: Files: $files");
			print "Post:</br>\n"; foreach($_POST as $n => $v) print $n . " => " . $v . "</br>\n";
			print_r($_FILES); print "<br>";
			print "Files:</br>\n"; foreach($_FILES as $f) if($f["name"]) print $f["name"] . "</br>\n";
		}

		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjJournal();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
			
			foreach($_FILES as $item => $file){
				if($file["name"] != ""){
					if($file["error"])
						return $this->sys->pageDisplayError(new BError(1, "File upload error: " . $file["name"] . ": " . fileErrorString($file["error"])));

					$item = substr($item, 5);
					if($item == "pdf")
						$file["name"] = "journal.pdf";
					else
						$file["name"] = filterFilename($file["name"]);
//					print "DoFile: " . $file["name"] .", " . $file["type"] .", " . $file["tmp_name"] .", " . "<br>\n";
					
					$obj->{$item} = $file["name"];
				}
  			}

			if($obj->id){
				$this->sys->db->putObject("Journals", $obj);
			}
			else {
				$this->sys->db->appendObject("Journals", $obj);
			}

			$baseDir = $this->sys->baseDir . "/" . $uri;
			if(!file_exists($baseDir))
				mkdir($baseDir);
			$baseDir = $this->sys->baseDir . "/" . $uri . "/" . $obj->id;
			if(!file_exists($baseDir))
				mkdir($baseDir);

			foreach($_FILES as $item => $file){
				if($file["name"] != ""){
					if($item == "file_pdf")
						$file["name"] = "journal.pdf";
					else
						$file["name"] = filterFilename($file["name"]);

					$dest = $baseDir . "/" . $file["name"];
//					print "mv: " . $file["tmp_name"] . " to " . $dest . "<br>\n";
					move_uploaded_file($file["tmp_name"], $dest);
					
					if($item == "file_pdf"){
						$filename = $file["name"];
						system("(cd {$baseDir}; convert -geometry 100 {$filename}[0] icon.png)");
					}
				}
  			}

			if(count($_FILES)){
				if($obj->mp3_1){
					$fileNames = "";
					for($n = 1; $n < 12; $n += 1){
						$item = "mp3_{$n}";
						if($obj->{$item})
							$fileNames .= $obj->{$item} . " ";
					}
//					print "Run: cat {$fileNames} > journal.mp3<br>";
					system("(cd {$baseDir}; cat {$fileNames} > journal.mp3)");
					$obj->mp3_all = "journal.mp3";
					$this->sys->db->putObject("Journals", $obj);
				}
			}
			
			if($_POST["sendEmail"] == "1"){
				$password = makeRandomPassword();

				if(!file_exists($this->sys->baseDir . "/files/tmp"))
					mkdir($this->sys->baseDir . "/files/tmp");

				// Remove old journal data
				system("rm {$this->sys->baseDir}/files/tmp/journal_*");
				
				// Make temporary copies
				copy($baseDir . "/journal.pdf", $this->sys->baseDir . "/files/tmp/journal_{$password}.pdf");
				copy($baseDir . "/journal.mp3", $this->sys->baseDir . "/files/tmp/journal_{$password}.mp3");
				
				$page .= "Sending journal emails<br>";
				$page .= $this->sendEmails($password);
			}
			
			$page .= "<h3>Details saved</h3>";
		}

		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";

		return $page;
	}
	
	function sendEmails($password){
		if(array_key_exists("membership", $this->sys->modules)){
			$emailList = $this->sys->modules["membership"]->emailGetJournalSendList();
			
			$subject = "{$this->sys->siteTitle}: New journal is available";
			$message = "A new Tandem Club Journal is available.\n\n";
			$message .= "Audio MP3 Journal: {$this->sys->baseUrl}/files/tmp/journal_{$password}.mp3\n\n";
			$message .= "Viewable PDF Journal: {$this->sys->baseUrl}/files/tmp/journal_{$password}.pdf\n\n";
			$message .= "You can access the MP3 tracks individually by logging into the website and accessing them: {$this->sys->baseUrl}\n\n";
			$message .= "If you no longer wish to receive these emails please contact: webgroup@tandem-club.org.uk\n\n";
			$message .= "If you no longer wish to receive the journal in CD form also please contact: webgroup@tandem-club.org.uk\n\n";
			
			return emailList($emailList, $subject, $message, 0);
		}
	}
	
	function delete(){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("Journals", $id);
			$page = "<h3>Journals: {$id} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("Journals", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete formtest: {$obj->id} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function getLatest(){
		$list = $this->sys->db->getObjectList("Journals", "", "", "!date", "1");
		
		$obj = $list[0];
		$baseUrl = "/" . $this->name . "/" . $obj->id;
		$date = new BDate($obj->date);
		$title = "Journal - " . BDate::monthName($date->getMonth()) . " " . $date->getYear();
		
		$html = new BHtmlNode("div", "id='news'");
		$html->appendNode("a", "href='{$this->sys->baseUrl}/{$this->name}'")->appendNode("h2", "", "Journals");
		$d1 = &$html->appendNode("div", "class='NewsSummary'");

		$div = &$d1->appendNode("div", "style='float: left;'");
		$div->appendNode("a", "href='{$baseUrl}/{$obj->pdf}'")->appendNode("img", "src='{$baseUrl}/icon.png'");
		$div = &$d1->appendNode("div", "style='float: left; padding: 4px;'");
		$div->appendText($title);
		$div->appendNode("br");
		$div->appendNode("a", "href='{$baseUrl}/{$obj->pdf}'", "PDF File");
		$div->appendNode("br");
		if($obj->mp3_all){
			$div->appendNode("a", "href='{$baseUrl}/{$obj->mp3_all}'", "MP3 Audio Complete Journal");
			$div->appendNode("br");
			$div->appendNode("a", "href='{$baseUrl}'", "MP3 Audio Individual Tracks");
			$div->appendNode("br");
		}

		return $html->render();
	}
};

function journalsLatest(){
	global	$sys;

	if($mod = $sys->moduleFindType("Journals")){
		return $mod->getLatest();
	}
	else {
		return "No journals module found";
	}
}
?>
