<?php
/*******************************************************************************
 *	Link.php	Link Module
 *			T.Barnaby,	BEAM Ltd,	2008-03-15
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("Link" => "BObjLink"));

class BObjLink extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("description", "String"),
			new BMember("url", "String"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("icon", "String"),
			));
	}
};

class Link extends BModule {
	// Basic Interface routines
	function __construct($sys){
		parent::__construct($sys);
		$this->name = "link";
		$this->title = "Links";
		$this->info = "Links module. This module allows the management of Link items.";
	}

	function getMenu(){
		if($this->sys->login->checkLogin("admin")){
			return new BMenu("Links", "/{$this->name}", "", array("Link List" => "/{$this->name}", "Link Add" => "/{$this->name}?cmd=add"));
		}
		else {
			return new BMenu("Links", "/{$this->name}", "", array("Link List" => "/{$this->name}"));
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->sys->login->checkLogin("admin")){
			switch($this->cmd){
			case "view":			return $this->view(filterToInt($_GET["id"]));	break;
			case "add":			return $this->edit("0");		break;
			case "edit":			return $this->edit(filterToInt($_GET["id"]));	break;
			case "save":			return $this->save();			break;
			case "delete":			return $this->delete(filterToInt($_GET["id"]));	break;
			case "deleteDo":		return $this->deleteDo(filterToInt($_GET["id"]));	break;
			default: 			return $this->viewList();		break;
			}
		}
		else {
			return $this->viewList();
		}
	}

	function getPageBlock($uri){
		if($this->sys->login->checkLogin("admin")){
			$o = new BMenu("Link", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList'"),
				new BMenu("", "", ""),
//				new BMenu("View", "",	"onmouseover='this.href = \"?cmd=view&id=\" + document.ws.list.objId'"),
				new BMenu("Add", "",	"onmouseover='this.href = \"?cmd=add\"'"),
//				new BMenu("Edit", "",	"onmouseover='this.href = \"?cmd=edit&id=\" + document.ws.list.objId'"),
//				new BMenu("Delete", "",	"onmouseover='this.href = \"?cmd=delete&id=\" + document.ws.list.objId'"),
				new BMenu("", "", ""),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}

	function viewList(){
		if($this->sys->login->checkLogin("admin")){
			$page = "<h2>Link</h2>\n";

			$view = new BViewDatabase("list", "action");
			$view->setView(array(
				new BViewType("title", "Title"),
				new BViewType("category", "Category"),
				new BViewType("description", "Description"),
				new BViewType("url", "Url", "code", 'return "<a href=\"{$obj->url}\">{$obj->url}</a>";'),
				new BViewType("dateAdded", "Date Published", "date"),
				new BViewType("icon", "Icon", "code", 'return "<img alt=\"none\" src=\"../{$obj->icon}\">";')
				));

//			$view->setLimit(20);

			if($view->order == "")
				$view->setOrder("!dateAdded");

			$view->setTable("Link");
			$page = $page . $view->getHtml();

			$page .= "<div class='help'>";
			$page .= "This lists all of the users of the system. ";
			$page .= "In the left hand bar there are links to the functions you can perform.";
			$page .= "</div>";

			return $page;
		}
		else {
			$html = new BHtmlNode("div");
			$t = &$html->appendNode("table", "cellspacing=05");
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td");
			$c->appendNode("h1", "", "Useful Internet Links");

			$list = $this->sys->db->getObjectList("Link", "", "`category` != 'kidsZone'", "id");
			$len = count($list);

			$t = &$html->appendNode("table", "cellspacing=5");
			$col = 0;
			foreach($list as $obj){
				$r = &$t->appendNode("tr");
				$c = &$r->appendNode("td");
				$c->appendNode("a", "href='{$obj->url}'")->appendNode("img", "src='{$obj->icon}'");
				$c = &$r->appendNode("td");
				$c->appendNode("a", "href='{$obj->url}'")->appendText($obj->title);
				$c = &$r->appendNode("td");
				$c->appendText($obj->description);
			}

			return $html->render();
		}
	}

	function view($id){
		$obj = $this->sys->db->getObject("Link", filterToInt($_GET["id"]));

		header("Location: {$obj->url}");
	}

	function edit($arg){
		$page = "<h2>Edit Link</h2>";

		if($arg == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else if(is_object($arg)){
			$obj = $arg;
		}
		else if($arg == 0){
			$obj = $this->sys->db->getNewObject("Link");
			$obj->dateAdded = strftime("%Y-%m-%d");
		}
		else {
			$obj = $this->sys->db->getObject("Link", $arg);
		}

		$view = new BEditObj("Link", "Link Information", "?cmd=save", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("icon", "", "hidden"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category", "select", categoryGetList("link")),
			new BViewType("description", "Description", "textarea", "", 300, 80),
			new BViewType("url", "Url"),
			new BViewType("dateAdded", "Date Added", hidden),
			new BViewType("MAX_FILE_SIZE", "", "hidden"),
			new BViewType("iconFile", "Icon File", "file"),
			));

		$obj->MAX_FILE_SIZE = 30000;

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc");
		$view->setHelp("url", "Enter the Internet address for this link. (eg. www.myschool1.org.uk)");
		$view->setHelp("iconFile", "Optionaly choose a picture for use with the link. Will be rescaled to 128x96");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		
		return $page;
	}

	function save(){
//		print "Save: "; print_r($_POST);

		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjLink();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			if(file_exists($_FILES["iconFile"]["tmp_name"])){
				unlink($this->sys->baseDir . "/files/icons/icon-{$obj->id}.jpg");
				$s = getimagesize($_FILES["iconFile"]["tmp_name"]);
				$pathInfo = pathinfo($_FILES["iconFile"]["name"]);
				switch($pathInfo["extension"]){
				case "jpeg":
				case "jpg":	$image = imagecreatefromjpeg($_FILES["iconFile"]["tmp_name"]);	break;
				case "gif":	$image = imagecreatefromgif($_FILES["iconFile"]["tmp_name"]);	break;
				case "png":	$image = imagecreatefrompng($_FILES["iconFile"]["tmp_name"]);	break;
				}
				$pic = imagecreatetruecolor(128, 96);
				imagecopyresized($pic, $image, 0, 0, 0, 0, 128, 96, $s[0], $s[1]);
				imagejpeg($pic, $this->sys->baseDir . "/files/icons/icon-{$obj->id}.jpg");
				unlink($_FILES["iconFile"]["tmp_name"]);

				$obj->icon = "files/icons/icon-{$obj->id}.jpg";
			}
			else if($obj->icon == ""){
				$obj->icon = "images/site_icon.gif";
			}

			unset($obj->iconFile);

			if($obj->id)
				$this->sys->db->putObject("Link", $obj);
			else
				$this->sys->db->appendObject("Link", $obj);
		}

		$page = "<h3>Link details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("Link", $id);

		$m = new BHtmlNode("div", "name='dialogQuestion'");
		$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteDo&id={$id}' method='post'");
		$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

		$f->appendNode("input", "type='submit' name='arg' value='Delete'");
		$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

		$page = $f->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		
		return $page;
	}

	function deleteDo($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("Link", $id);

		// Now delete the object itself
		$this->sys->db->deleteObject("Link", $id);

		$page = "";
		$page .= "<h3>Link {$obj->title} has been Deleted</h3>";	
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}
};

?>
