<?php
/*******************************************************************************
 *	Management.php	Management Module
 *				T.Barnaby,	BEAM Ltd,	2009-03-13
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BObj.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";

class BObjLog extends BObj {
	function __construct(){
		$m = array();

		$m[] = new BMember("id", "Int", "Number", "", "int", "NOT NULL auto_increment");
		$m[] = new BMember("time", "DateTime");
		$m[] = new BMember("type", "String");
		$m[] = new BMember("module", "String");
		$m[] = new BMember("description", "String");

		parent::__construct($m);
	}
};

class BObjTopMenu extends BObj {
	function __construct(){
		$m = array();

		$m[] = new BMember("menu", "String");

		parent::__construct($m);
	}
};

class Management extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "management";
		$this->title = "Management";
		$this->info = "Management Module";
		
		// Add Objects
		$sys->db->addObjectClasses(array("Log" => "BObjLog"));
	}

	function sessionRead(){
//		if(isset($_SESSION["info.objId"]))
//			$this->objId = $_SESSION["info.objId"];
	}
	function sessionWrite(){
//		$_SESSION["category"] = $this->category;
//		$_SESSION["info.objId"] = $this->objId;
	}
	
	function checkAccess(){
		return ($this->sys->login->checkLogin("admin"));
	}

	// Standard interface
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Management", "/{$this->name}", "", array(
				"Management List" => "/{$this->name}",
				"System Logs" => "/{$this->name}?cmd=logList",
				"Menu Edit" => "/{$this->name}?cmd=menuEdit",
				"<b>Database</b>" => "",
				"Database Backup" => "/{$this->name}?cmd=databaseBackup",
				"Database Tables Create/Update" => "/{$this->name}?cmd=databaseCreate",
				"Database Edit" => "/{$this->name}?cmd=databaseEdit",
				));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		$this->sessionRead();
		
		if($this->checkAccess() || $this->sys->login->checkLoginHttp("admin")){
			switch($this->cmd){
			case "logList":			$r = $this->logList($uri);		break;
			case "menuEdit":		$r = $this->menuEdit($uri);		break;
			case "menuSave":		$r = $this->menuSave($uri);		break;
			case "databaseBackup":		$r = $this->databaseBackup($uri);	break;
			case "databaseCreate":		$r = $this->databaseCreate($uri);	break;
			case "databaseEdit":		$r = $this->databaseEdit($uri);		break;
			default: 			$r = $this->mainPage($uri);		break;
			}
		}
		else {
			$this->sys->pageDisplayError(new BError(403, "UNAUTHORIZED"));
		}

		$this->sessionWrite();
		return $r;
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			return new BMenu("Management", "", "", array(
				new BMenu("System Logs", "", "href='?cmd=logList'"),
				new BMenu("Menu Edit", "", "href='?cmd=menuEdit'"),
				"<b>Database</b>" => "",
				new BMenu("Database Backup", "", "href='?cmd=databaseBackup'"),
				new BMenu("Database Create", "", "href='?cmd=databaseCreate'"),
				new BMenu("Database Edit", "", "href='?cmd=databaseEdit'"),
				));
		}
		else {
			return Null;
		}
	}
	
	function mainPage($uri){
		$page = "<h2>Management</h2>";
		return $page;
	}

	function logList($uri){
		if($this->checkAccess()){
			$page = new BHtmlNode("div");
			$pager = new BHtmlPager(32);
			$list = $this->sys->db->getObjectList("Log", "", "", "!time", $pager->getRange());
			
			$t = &$page->appendNode("table", "border=1");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Time");
			$r->appendNode("th", "", "Type");
			$r->appendNode("th", "", "Module");
			$r->appendNode("th", "", "Description");
			
			foreach($list as $obj){
				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", $obj->time);
				$r->appendNode("td", "", $obj->type);
				$r->appendNode("td", "", $obj->module);
				$r->appendNode("td", "", $obj->description);
			}
			$page->appendNode("div", "class=ForumEntry", $pager->getHtml("?cmd=logList"));

//			$this->logAdd("notice", "membership", "A test entry");
//			$this->sys->logAdd("notice", "membership", "A test entry");

			return $page->render();
		}
		else {
			return "Error: you are not allowed to access this.";
		}
		
	}
	
	function menuEdit($arg){
		$page = "<h2>Website main menu editor</h2>\n";
		$page .= "<div class='help'>";
		$page .= "There is one menu entry per line. Menu name is hierarchical using '/' characters as separators and should be terminaled with the ':' character. ";
		$page .= "The top menu is named MenuTop so most menu entries should start with 'MenuTop/'. ";
		$page .= "TAB's can be used to indent to the URL that the menu entry links to.";
		$page .= "</div>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else {
			$obj = new BObjTopMenu();
			$obj->menu = file_get_contents("{$this->sys->baseDir}/files/menu.conf");
		}
		
		$view = new BEditObj("menu", "Main menu", "?cmd=menuSave", "");
		$editors = array(
			new BViewType("menu", "Menu", "textarea", "", 700, 400),
			);

		$view->setView($editors);

		$view->setHelp("menu", "Enter the websites main menu.");
		$view->setObj($obj);
		$page .= $view->getHtml();
		
		$page .= "<script>var o = document.getElementById('menu');";
		$page .= "o.onkeydown = function(e){ if(e.keyCode==9 || e.which==9){ e.preventDefault();";
		$page .= "var s = this.selectionStart; this.value = this.value.substring(0,this.selectionStart) + '\t' + this.value.substring(this.selectionEnd);";
 		$page .= "this.selectionEnd = s+1; }}</script>";
		
		return $page;
	}
	
	function menuSave($uri){
		$page = "";

		#print "Post: "; print_r($_POST); print "<br>";
		
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjTopMenu();
			$obj->set($_POST);
			
			file_put_contents("{$this->sys->baseDir}/files/menu.conf", $obj->menu);
			$this->sys->createMenuBar();
			
			$page .= "<h2>Menu updated<h2>";
		}
		return $page;
	}
	
	function logAdd($type, $module, $description){
		$obj = $this->sys->db->getNewObject("Log");
		$obj->time = strftime("%Y-%m-%d %H:%M:%S");
		$obj->type = $type;
		$obj->module = $module;
		$obj->description = $description;
		
		$this->sys->db->appendObject("Log", $obj);
	}

	function databaseBackup($uri){
		$tables = array();
		$tablesData = array();
		$page = "";

		// Get table structure
		$rows = $this->sys->db->query("show tables like '" . $this->sys->databasePrefix . "%'");
		foreach($rows as $row){
			$tables[] = $row[0];
			if($row[0] != ($this->sys->databasePrefix . "Statistics"))
				$tablesData[] = $row[0];
		}
		
		// Dump Tables
		$sqlCmd = "mysqldump -h {$this->sys->databaseHost} -u{$this->sys->databaseUser} -p{$this->sys->databasePassword} -c -d {$this->sys->databaseName} " . implode(" ", $tables);
		exec($sqlCmd, $res);

		foreach($res as $line){
			print "{$line}\n";
		}

		// Dump Data
		print "\n\n";
		print "-- Data\n\n";
		unset($res);

		$sqlCmd = "mysqldump -h {$this->sys->databaseHost} -u{$this->sys->databaseUser} -p{$this->sys->databasePassword} -c -t {$this->sys->databaseName} " . implode(" ", $tablesData);
		exec($sqlCmd, $res);

		foreach($res as $line){
			print "{$line}\n";
		}
		
		return Null;
	}

	function databaseCreateTableForObject($table, $obj, $force){
		$ret = "";
	
		// Check if table exists
		$ti = $this->sys->db->getTableInfo($table);

		if(0){
			$ret .= "TableInfo: " . print_r($ti, 1) . "</br>";
			$ret .= "TableFields0: {$obj->dbTableFields()}<br>";
		}

		if(!$ti or $force){
			// Create a new table
			if($obj->dbTableFields() == ""){
				$ret .= "Warning: No database fields defined for Object: {$c}<br>";
			}
			else {
				$this->sys->db->createTable($table, $obj->dbTableFields());
			}

			$obj->dbTableSetup($table);
		}
		else {
			// Add new columns if needed
			$colsNeeded = array();
			foreach($obj->getMembers() as $m){
				$colsNeeded[] = $m->name;
			}
			$colsPresent = array();
			foreach($ti as $v){
				$colsPresent[] = $v[0];
			}
//			$ret .= "Needed: " . print_r($colsNeeded, 1) . "</br>";
//			$ret .= "Present: " . print_r($colsPresent, 1) . "</br>";
			foreach($colsNeeded as $col){
				if(!in_array($col, $colsPresent)){
					$table = $this->sys->databasePrefix . $table;
					$m = $obj->getMembers();
					$m = $m[$col];
					$sql = "alter table `{$table}` add column `{$m->name}` {$m->dbType} {$m->dbArgs}";
					$ret .= "{$c}: Add column: {$col}</br>";
//						$page .= "Sql: '{$sql}': NOT DONE.</br>";
					$this->sys->db->query($sql);
				}
			}
		}
		return $ret;
	}
	
	function databaseCreate($uri){
		$force = 0;
		$page = "<h3>Management</h3>";

		$objClasses = $this->sys->db->getObjectClasses();
		foreach($this->sys->db->getObjectClasses() as $k => $c){
			$page .= "Check: Obj: {$k} => {$c}<br>";

			$obj = new $c($this->sys);

			// Check if table exists
			$ti = $this->sys->db->getTableInfo($k);

			if(0){
				$page .= "TableInfo: " . print_r($ti, 1) . "</br>";
				$page .= "TableFields0: {$obj->dbTableFields()}<br>";
			}
			
			if(!$ti or $force){
				// Create a new table
				if($obj->dbTableFields() == ""){
					$page .= "Warning: No database fields defined for Object: {$c}<br>";
				}
				else {
					$page .= "Create Table {$k}<br>";
//					$this->sys->db->dropTable($k);
					$this->sys->db->createTable($k, $obj->dbTableFields());
				}

				$obj->dbTableSetup($k);
			}
			else {
				// Add new columns if needed
				$colsNeeded = array();
				foreach($obj->getMembers() as $m){
					$colsNeeded[] = $m->name;
				}
				$colsPresent = array();
				foreach($ti as $v){
					$colsPresent[] = $v[0];
				}
//				$page .= "Needed: " . print_r($colsNeeded, 1) . "</br>";
//				$page .= "Present: " . print_r($colsPresent, 1) . "</br>";
				foreach($colsNeeded as $col){
					if(!in_array($col, $colsPresent)){
						$table = $this->sys->databasePrefix . $k;
						$m = $obj->getMembers();
						$m = $m[$col];
						$sql = "alter table `{$table}` add column `{$m->name}` {$m->dbType} {$m->dbArgs}";
						$page .= "{$c}: Add column: {$col}</br>";
//						$page .= "Sql: '{$sql}': NOT DONE.</br>";
						$this->sys->db->query($sql);
					}
				}
			}
		}
		return $page;
	}
	
	function databaseEdit($uri){
		$page = "<h3>Database edit</h3>";
		
		//include("/usr/share/phpMyAdmin/index.php");
		//$page = shell_exec("(cd /usr/share/phpMyAdmin; php /usr/share/phpMyAdmin/index.php)");
		$page .= "Not Implemented yet ..."; 

		return $page;
	}
	
	function install(){
		return $this->databaseCreate("");
	}
	
	// Old functions
	function databaseBackup1($uri){
		$page = "";

		// Get table structure
		$rows = $this->sys->db->query("show tables");
		foreach($rows as $row){
			$tableInfo = $this->sys->db->query("show create table {$row[0]}");
			$page .= $tableInfo[0][1] . ";\n";
		}
		
		// Get Data

		print $page;

		return Null;
	}
};

?>
