<?php
/*******************************************************************************
 *	maps.php		Maps Module
 *				T.Barnaby,	BEAM Ltd,	2014-04-07
 *******************************************************************************
 *
 * Uses: Code-point Open. OS psotcode database
 *	https://www.ordnancesurvey.co.uk/opendatadownload/products.html
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";
require_once "phpcord-2.3.php";

class BObjMapLocation  extends BObj {
	function __construct(){
		$m = array();
		$m[] = new BMember("id", "Int", "Number");
		$m[] = new BMember("postcode", "String");
		$m[] = new BMember("type", "String");
		$m[] = new BMember("eastling", "String");
		$m[] = new BMember("northling", "String");
		$m[] = new BMember("longatude", "String");
		$m[] = new BMember("latitude", "String");
		parent::__construct($m);
	}
}

class BObjMapCountry  extends BObj {
	function __construct(){
		$m = array();
		$m[] = new BMember("id", "Int", "Number");
		$m[] = new BMember("code", "String");
		$m[] = new BMember("name", "String");
		parent::__construct($m);
	}
}

class Maps extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "maps";
		$this->title = "Maps";
		$this->info = "Maps Module";
		$this->apiKey = "F69015E3E42E1A40E0430B6CA40A99D2";		// https://tandem-club-org.uk
//		$this->apiKey = "F8F5D0AC810C6C82E0430C6CA40AD853";
		
		// Add database objects
		$sys->db->addObjectClasses(array("MapCountrys" => "BObjMapCountry"));
		$sys->db->addObjectClasses(array("MapLocations" => "BObjMapLocation"));
		
		// Add roles
//		$this->sys->modules["user"]->appendRoles(array("adminMaps"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "mapsGet";

		$this->adminInit();
	}
	
	// Standard interface
	function getMenu(){
		return new BMenu("Maps", "/{$this->name}", "", array(
			"Maps List" => "/{$this->name}",
			"Maps Reload" => "/{$this->name}?cmd=reload",
			"Maps Test" => "/{$this->name}?cmd=test",
			));
	}

	function getPage($uri){
		$this->parseUri($uri);

		switch($this->cmd){
		case 'reload': 			return $this->reload();		break;
		case 'test': 			return $this->test();		break;
		default: 			return $this->view($uri);	break;
		}
	}

	function getPageBlock($uri){
		$o = new BMenu("Maps", "", "", array(
			new BMenu("Maps Reload", "", "href='?cmd=reload'"),
			new BMenu("Maps Test", "", "href='?cmd=test'"),
			));
		return $o;
	}
	
	function view(){
		$page = "<h1>Maps</h1>";
		$loc = $this->getLocation("BS");
		$page .= $this->getMapOS($loc->eastling, $loc->northling, 7, "100%", 800);
		return $page;
	}
	
	function reload(){
		// Initialise MapLocations data
		if($this->sys->db->tableExists("MapLocations")){
			if($fd = fopen($this->sys->baseDir . "/modules/maps/postcodes.csv", "r")){
				$this->sys->db->truncateTable("MapLocations");
				while($data = fgetcsv($fd, 1000, ",")){
					$loc = new BObjMapLocation();
					$loc->postcode = $data[0];
					$loc->type = $data[1];
					$loc->eastling = $data[2];
					$loc->northling = $data[3];

					$osref = new OSRef($loc->eastling, $loc->northling);
					$latlong = $osref->toLatLng();
					$loc->longatude = $latlong->lng;
					$loc->latitude = $latlong->lat;

					$this->sys->db->appendObject("MapLocations", $loc);
				}
				fclose($fd);
			}
		}
		if($this->sys->db->tableExists("MapCountrys")){
			if($fd = fopen($this->sys->baseDir . "/modules/maps/countryCodes.csv", "r")){
				$this->sys->db->truncateTable("MapCountrys");
				while($data = fgetcsv($fd, 1000, ",")){
					$obj = new BObjMapCountry();
					$obj->code = $data[0];
					$obj->name = $data[1];

					$this->sys->db->appendObject("MapCountrys", $obj);
				}
				fclose($fd);
			}
		}
		return "Reloaded";
	}
	
	function adminInit(){
		// Initialise MapLocations data
		if($this->sys->db->tableExists("MapLocations")){
			$number = $this->sys->db->getObjectNumber("MapLocations");
			if($number == 0){
				if($fd = fopen($this->sys->baseDir . "/modules/maps/postcodes.csv", "r")){
					while($data = fgetcsv($fd, 1000, ",")){
						$loc = new BObjMapLocation();
						$loc->postcode = $data[0];
						$loc->type = $data[1];
						$loc->eastling = $data[2];
						$loc->northling = $data[3];

						$osref = new OSRef($loc->eastling, $loc->northling);
						$latlong = $osref->toLatLng();
						$loc->longatude = $latlong->lng;
						$loc->latitude = $latlong->lat;

						$this->sys->db->appendObject("MapLocations", $loc);
					}
					fclose($fd);
				}
			}
		}
		if($this->sys->db->tableExists("MapCountrys")){
			$number = $this->sys->db->getObjectNumber("MapCountrys");
			if($number == 0){
				if($fd = fopen($this->sys->baseDir . "/modules/maps/countryCodes.csv", "r")){
					while($data = fgetcsv($fd, 1000, ",")){
						$obj = new BObjMapCountry();
						$obj->code = $data[0];
						$obj->name = $data[1];

						$this->sys->db->appendObject("MapCountrys", $obj);
					}
					fclose($fd);
				}
			}
		}
	}
	
	function getCountryList(){
		$l = array();
		$l[] = "United Kingdom";
		$list = $this->sys->db->getObjectList("MapCountrys", "`name`", "", "name", "");
		foreach($list as $i){
			$l[] = $i->name;
		}
		return $l;
	}
	
	function getCountryCode($country){
		$country = dbEscapeString($country);
		$list = $this->sys->db->getObjectList("MapCountrys", "`code`", "`name`='{$country}'", "", "1");
		
		if(count($list))
			return $list[0]->code;
		else
			return "";
	}
	
	function getLocation($postcode){
		if(0){
			if((count($postcode) > 1) && !ctype_alpha($postcode[1]))
				$postcode = substr(strtoupper($postcode), 0, 1);
			else
				$postcode = substr(strtoupper($postcode), 0, 2);
		}
		
		$postcode = dbEscapeString($postcode);	
		$list = $this->sys->db->getObjectList("MapLocations", "", "`postcode`='{$postcode}'", "", "");
		if(count($list))
			return $list[0];
		else
			return Null;
	}

	function getPostcode($mapLocation){
	}

	function getLocations($mapLocation, $distance){
	}

	function getPostcodeList($type = "pcregion"){
		$type = dbEscapeString($type);
		$postcodes = array();
		$list = $this->sys->db->getObjectList("MapLocations", "`postcode`", "`type`='{$type}'", "postcode", "");

		foreach($list as $i){
			$postcodes[] = $i->postcode;
		}
		return $postcodes;
	}
	
	function getMap($location, $scale, $width = 500, $height = 300){
		$loc = explode(",", $location);
//		print "getMap: '{$location}' '$loc' Num: " . count($loc) . "</br>";
		if(count($loc) == 2){
			return $this->getMapOS($loc[0], $loc[1], $scale, $width, $height);
		}
		else {
			$loc = $this->getLocation($location);
			if($loc)
				return $this->getMapOS($loc->eastling, $loc->northling, $scale, $width, $height);
			else
				return "Map location not valid";
		}
	}
	
	function getMapOS($east, $west, $scale, $width = 500, $height = 300){
		$name = "map_" . strval(rand());
		$page = "";
//		$page .= "Location: $east, $west</br>\n";
		$page .= "<script type='text/javascript' src='https://openspace.ordnancesurvey.co.uk/osmapapi/openspace.js?key={$this->apiKey}'></script>\n";
		$page .= "<div id='{$name}' style='width: {$width}px; height: ${height}px; border: 1px solid black;'></div>\n";
		$page .= "<script type='text/javascript'>var osMap; function showMap_{$name}(){ osMap = new OpenSpace.Map('{$name}'); osMap.setCenter(new OpenSpace.MapPoint({$east}, {$west}), ${scale}); var markers = new OpenLayers.Layer.Markers('Markers'); osMap.addLayer(markers); var pos = new OpenSpace.MapPoint({$east}, {$west}); var marker = new OpenLayers.Marker(pos); markers.addMarker(marker); }</script>\n";
		$page .= "<script type='text/javascript'>showMap_{$name}();</script>\n";
		return $page;
	}
	
	function test(){
		$page = "";
		$page .= "<p>Postcode: BS</p>";
		$page .= $this->getMap("BS", 7);
		$page .= "<p>Postcode: BS37</p>";
		$page .= $this->getMap("BS37", 7);
		return $page;
	}
};

function mapsGet($location, $scale, $width=500, $height=300){
	global	$sys;

	$location = trim($location, '"');
//	print "mapsGet: $location, $scale, $width, $height</br>\n";
	$html = new BHtmlNode("div", "id='maps'");
	if($maps = $sys->modules["maps"]){
		$html->appendText($maps->getMap($location, $scale, $width, $height));
	}
	return $html->render();
}

?>
