<?php
/*******************************************************************************
 *	Membership.php	Membership Module
 *			T.Barnaby,	BEAM Ltd,	2013-11-26
 *******************************************************************************
 *
 * Testing
 *	Make sure the Nochex account has test transcations enabled by logging in and setting it.
 *	The Payments class has a test variable that then uses this.
 *	Set the "test" variable of the Membership class to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for Full payment test accessing Nochex, but no actual payments being made.
 *	Set to 2 for Completely local testing (No Nochex interaction).
 *	Set to 3 for no payments at all.
 *
 * Emails Testing:
 *	Set the "testEmail" variable of the Membership class to one of the following:
 *	Set to 0 for Normal operation.
 *	Set to 1 for just send emails to current user's email address
 *	
 */
require_once "BModule.php";
require_once "BObj.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";
require_once "tcpdf.php";

// Output debug information on membershi[p object to debug log file
function membership_debug_obj($title, $obj){
	$str = $title;
	$str .= "\n";
	$str .= "\tFull: id: {$obj->id}, forename: {$obj->forename}, surname: {$obj->surname}, membershipNumber: {$obj->membershipNumber}\n";
	$n = 0;
	foreach($obj->jointList as $j){
		$str .= "\tJoint: {$n}, id: {$j->id}, forename: {$j->forename}, surname: {$j->surname}, membershipNumber: {$j->membershipNumber}\n";
		$n++;
	}
	debug_log($str);
}

class BObjMembership extends BObj {
	function __construct(){
		$m = array();

		$m[] = new BMember("id", "Int", "Number", "", "int", "NOT NULL auto_increment");
		$m[] = new BMember("fullId", "Int", "Number");
		$m[] = new BMember("pending", "Int", "Number");
		$m[] = new BMember("status", "String");
		$m[] = new BMember("membershipType", "String");
		$m[] = new BMember("membershipNumber", "String");
		$m[] = new BMember("joinedDate", "Date");
		$m[] = new BMember("group", "String");

		$m[] = new BMember("prefix", "String");
		$m[] = new BMember("forename", "String");
		$m[] = new BMember("surname", "String");
		$m[] = new BMember("dateOfBirth", "Date", "date", array('format' => '%Y-%n-%j'));
		$m[] = new BMember("family", "Bool");

		$m[] = new BMember("address", "String");
		$m[] = new BMember("city", "String");
		$m[] = new BMember("postcode", "String");
		$m[] = new BMember("country", "String");
		$m[] = new BMember("emailAddress", "String");
		$m[] = new BMember("telephoneHome", "String");
		$m[] = new BMember("telephoneMobile", "String");
		$m[] = new BMember("audioCd", "Int");
		$m[] = new BMember("journalByPost", "Int");
		$m[] = new BMember("journalByEmail", "Int");
		$m[] = new BMember("emailSend", "Int");
		$m[] = new BMember("notes", "String");

		$m[] = new BMember("renewalDate", "Date");
		$m[] = new BMember("renewalSent", "Int");

		$m[] = new BMember("paymentType", "String");
		$m[] = new BMember("paymentCurrency", "String");
		$m[] = new BMember("paymentId", "Int");
		$m[] = new BMember("paymentDate", "Date");
		$m[] = new BMember("paymentAmount", "Float", "", "", "decimal(8,2)");
		$m[] = new BMember("paymentAmountPaid", "Float", "", "", "decimal(8,2)");

		$m[] = new BMember("notesAdmin", "String");
		$m[] = new BMember("attractedFrom", "String");
		$m[] = new BMember("whyResigned", "String");
		$m[] = new BMember("modifiedDate", "Date");
		$m[] = new BMember("modifiedUserName", "String");
		$m[] = new BMember("renewPassword", "String");
		$m[] = new BMember("updatePassword", "String");
		$m[] = new BMember("userId", "Int");

		$m[] = new BMember("gdprAcceptance", "Int");

		parent::__construct($m);
	}
};

function getAddressString($obj){
	$str = "{$obj->prefix} {$obj->forename} {$obj->surname}<br/>";
	for($i = 0; $i < strlen($obj->address); $i++){
		if($obj->address[$i] == "\n")
			$str .= "<br/>";
		else
			$str .= $obj->address[$i];
	}
	if($obj->city != "")
		$str .= "<br/>{$obj->city}";
	$str .= "<br/>{$obj->postcode}";
	if($obj->country != "")
		$str .= "<br/>{$obj->country}";
	return $str;
} 

function addressLines($str, $n){
	$str = str_replace("\r", '', $str);
	$l = explode("\n", $str);
	$l = array_slice($l, 0, $n);
	while(count($l) < $n)
		$l[] = "";
	return $l;
}

function jointCompareMembershipNumber($a, $b){
	$a = $a->membershipNumber;
	$b = $b->membershipNumber;
	preg_match('/\d+/', $a, $m);
	if(count($m))
		$a = intval($m[0]);
	else
		$a = PHP_INT_MAX;
	preg_match('/\d+/', $b, $m);
	if(count($m))
		$b = intval($m[0]);
	else
		$b = PHP_INT_MAX;

	if($a == $b) {
	        return 0;
	}
	return ($a < $b) ? -1 : 1;
}

function quoteForCsv($str){
	return str_replace("\"", "\"\"", $str);
}

class MembershipTc extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "membership";
		$this->title = "Membership";
		$this->info = "Membership module";
		$this->membershipTypes = array("1 year", "5 year");
		$this->membershipTypesAdmin = array("1 year", "5 year", "Honorary", "Complimentary", "Special");
		$this->statusTypes = array("new", "member", "renew", "renewDone", "renewForm", "lapsed", "leave");
		$this->chequeAddressJoin = configGet("membership.chequeAddressJoin");
		$this->chequeAddressRenew = configGet("membership.chequeAddressRenew");
		$this->numJoint = 6;
		$this->test = 0;
		$this->testEmails = 0;

		// Add Objects
		$sys->db->addObjectClasses(array("Membership" => "BObjMembership"));
		$sys->db->addObjectClasses(array("MembershipOld" => "BObjMembership"));
		
		// Add roles
		$this->sys->modules["user"]->appendRoles(array("viewMembership"));
		$this->sys->modules["user"]->appendRoles(array("adminMembership"));

		$this->formNotesJoin = <<< End_Of_Text
			<p>Full membership is available on an annual basis or on a commuted one for five years.
			Joint Membership is available to anyone who rides regularly with a Full Member; at least one applicant must take out Full Membership.
			The first Joint Member is enrolled free. Family membership, which is also free, is open to all dependants of the Full Member aged 17 or under
			residing at the same address, check the family checkbox if this is the case. There is a nominal charge for other Joint Members.
			Joint Members are entitled to all the benefits of the Club, except a copy of the Journal.
			Journals are automatically sent by Airmail to members in all countries outside of the UK. There is no additional charge for this.
			</p>
			<ol>
			<li>Please complete this form and then use the "Join" or "Renew" button at the end as appropriate.
			Make sure you choose the payment type (card, standingOrder, bacs or cheque) and currency type you will be using to pay.</li>
			<li>Add the Full members name first and then any joint members.</li>
			<li>Dates of birth only need to be entered (changed from the default) for people below the age of 18.</li>
			<li>Check the family checkbox for each joint member who is in the same family, residing at the same address.</li>
			<li>Please provide us with your email address if possible. This allows us to automatically send you renewal notices and thus saves us a lot of time.</li>
			<li>If you are renewing manually (ie not from an email link) please enter your membership number, if known, in the "Member Num" field.</li>

			<li>Please then pay in one of the following ways:<br>
			<b>For UK members</b>
			<ul>
			<li>For yearly membership you can pay using a credit or debit card. Click on the "Pay on Credit or Debit Card" button that is displayed after the "Join" or "Renew" button is pressed and you will
			be taken to a secure payments page. Please follow the instructions to complete payment. This is the easiest and best method for us.</li>
			<li>You can pay in Pounds using BACS (direct bank transfer) to the Tandem Clubs account.</li>
			<li>You can pay by Standing Order to the Tandem Clubs account but only for 1 years subscriptions.</li>
			<li>You can post a cheque in Pounds for total amount to {$this->chequeAddressJoin}, with your membership number written on the reverse, payable to: "The Tandem Club".</li>
			</ul>

			<b>For International members</b>
			<ul>
			<li>For yearly membership you can pay using a credit or debit card in pounds stirling. Click on the "Pay on Credit or Debit Card" button that is displayed after the "Join" or "Renew" button is pressed and you will
			be taken to a secure payments page. Please follow the instructions to complete payment. This is the easiest and best method for us.</li>
			<li>You can pay in Euros by direct bank transfer to the Tandem Club's European bank.</li>
			<li>You can pay in Pounds or Euros by posting a cheque to {$this->chequeAddressJoin}.</li>
			<li>You can post a cheque in your local currency for total amount plus the equivalent of 6 GBP to cover our bank charges to {$this->chequeAddressJoin}.</li>
			</ul>
			
			<li>Data privacy: The information you provide on this form will be added to the Club database and used by the Club for purposes only in connection with the running of the Club,
			which includes communicating by post, telephone and email. It will never be disclosed for marketing purposes. The data will be available to committee members and other members
			when it is needed to facilitate the running of the Club and provide membership benefits to you. Please note that photos and videos may be taken during Club rides and events and
			used on the Club website, in the Tandem Club Journal or in Tandem Club publicity material. You have a right to complain to the ICO if you believe there is a problem with the
			Club's handling of your data. Further details are given in the Club's Data Privacy Policy, available on the Club website or on request from the Club Secretary,
			<a href='mailto:secretary@tandem-club.org.uk'>secretary@tandem-club.org.uk</a>. Please indicate on the form below that you have read and accept these terms.</li>
			
			<li>Please use the membership number provided after joining as the BACS or Standing Order payment reference and on the back of cheques.
			The booking reference, account number details and address to post cheques will be given after you join/renew.</li>
			<li>By default you will be given a website login account. This will enable you to access Tandem Club members information and book for events in a simpler fashion.
			If you don't want this please un-check the "Add website account" option.</li>
			<li>The "Information by email" tick box is solely to allow you to receive emails from the club such as runs lists etc. No external organisation will given your email address</li>
			<li>Note that the TAB key can be used to move to the next field</li>
			<li>If you have any problems with the membership form or system please email:
			<a href="mailto:webgroup@tandem-club.org.uk">webgroup@tandem-club.org.uk</a></li>
			</ol>
End_Of_Text;
		$this->formNotesRenew = <<< End_Of_Text
			<p>Full membership is available on an annual basis or on a commuted one for five years.
			Joint Membership is available to anyone who rides regularly with a Full Member; at least one applicant must take out Full Membership.
			The first Joint Member is enrolled free. Family membership, which is also free, is open to all dependants of the Full Member aged 17 or under
			residing at the same address, check the family checkbox if this is the case. There is a nominal charge for other Joint Members.
			Joint Members are entitled to all the benefits of the Club, except a copy of the Journal.
			Journals are automatically sent by Airmail to members in all countries outside of the UK. There is no additional charge for this.
			</p>
			<ol>
			<li>Please complete this form and then use the "Join" or "Renew" button at the end as appropriate.
			Make sure you choose the payment type (card, standingOrder, bacs or cheque) and currency type you will be using to pay.</li>
			<li>Add the Full members name first and then any joint members.</li>
			<li>Dates of birth only need to be entered (changed from the default) for people below the age of 18.</li>
			<li>Check the family checkbox for each joint member who is in the same family, residing at the same address.</li>
			<li>Please provide us with your email address if possible. This allows us to automatically send you renewal notices and thus saves us a lot of time.</li>
			<li>If you are renewing manually (ie not from an email link) please enter your membership number, if known, in the "Member Num" field.</li>
			<li>If you change your membership by adding or removing members please would you take a moment to check that any Standing Order is still correct.</li>

			<li>Please then pay in one of the following ways:<br>
			<b>For UK members</b>
			<ul>
			<li>For yearly membership you can pay using a credit or debit card. Click on the "Pay on Credit or Debit Card" button that is displayed after the "Join" or "Renew" button is pressed and you will
			be taken to a secure payments page. Please follow the instructions to complete payment. This is the easiest and best method for us.</li>
			<li>You can pay in Pounds using BACS (direct bank transfer) to the Tandem Clubs account.</li>
			<li>You can pay by Standing Order to the Tandem Clubs account but only for 1 years subscriptions.</li>
			<li>You can post a cheque in Pounds for total amount to {$this->chequeAddressRenew}, with your membership number written on the reverse, payable to: "The Tandem Club".</li>
			</ul>

			<b>For International members</b>
			<ul>
			<li>For yearly membership you can pay using a credit or debit card in pounds stirling. Click on the "Pay on Credit or Debit Card" button that is displayed after the "Join" or "Renew" button is pressed and you will
			be taken to a secure payments page. Please follow the instructions to complete payment. This is the easiest and best method for us.</li>
			<li>You can pay in Euros by direct bank transfer to the Tandem Club's European bank.</li>
			<li>You can pay in Pounds or Euros by posting a cheque to {$this->chequeAddressRenew}.</li>
			<li>You can post a cheque in your local currency for total amount plus the equivalent of 6 GBP to cover our bank charges to {$this->chequeAddressRenew}.</li>
			</ul>
			
			<li>Data privacy: The information you provide on this form will be added to the Club database and used by the Club for purposes only in connection with the running of the Club,
			which includes communicating by post, telephone and email. It will never be disclosed for marketing purposes. The data will be available to committee members and other members
			when it is needed to facilitate the running of the Club and provide membership benefits to you. Please note that photos and videos may be taken during Club rides and events and
			used on the Club website, in the Tandem Club Journal or in Tandem Club publicity material. You have a right to complain to the ICO if you believe there is a problem with the
			Club's handling of your data. Further details are given in the Club's Data Privacy Policy, available on the Club website or on request from the Club Secretary,
			<a href='mailto:secretary@tandem-club.org.uk'>secretary@tandem-club.org.uk</a>. Please indicate on the form below that you have read and accept these terms.</li>
			
			<li>Please use the membership number provided after joining as the BACS or Standing Order payment reference and on the back of cheques.
			The booking reference, account number details and address to post cheques will be given after you join/renew.</li>
			<li>By default you will be given a website login account. This will enable you to access Tandem Club members information and book for events in a simpler fashion.
			If you don't want this please un-check the "Add website account" option.</li>
			<li>The "Information by email" tick box is solely to allow you to receive emails from the club such as runs lists etc. No external organisation will given your email address</li>
			<li>If you are here from a renew email and wish to leave the club, please use the "Leave the club" button at the end of the form. Please fill in the "Why resigned" field for our information.</li>
			<li>Note that the TAB key can be used to move to the next field</li>
			<li>If you have any problems with the membership form or system please email:
			<a href="mailto:webgroup@tandem-club.org.uk">webgroup@tandem-club.org.uk</a></li>
			</ol>
End_Of_Text;
	}
	
	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("nextMembershipNumber", "Number", "", "Next membership number", "Next membership number");
		$configs[] = new BObjConfig("adminEmailList", "StringList", "", "Administration email addresses", "List of administration email addresses");
		$configs[] = new BObjConfig("newMemberEmailList", "StringList", "", "Administration email addresses for new members", "List of administration email addresses");
		$configs[] = new BObjConfig("chequeAddressJoin", "String", "", "Cheque address Joining", "The name and address to send cheques when joining");
		$configs[] = new BObjConfig("chequeAddressRenew", "String", "", "Cheque address Renewal", "The name and address to send cheques when renewing");
		return $configs;
	}

	function checkAccess(){
		if($this->sys->login->checkLogin("adminMembership")){
			return 2;
		}
		else if($this->sys->login->checkLogin("viewMembership")){
			return 1;
		}
		else {
			return 0;
		}		
	}
	
	// Admin Interface
	function getMenu(){
		if($this->checkAccess() == 2){
			return new BMenu("Membership", "/{$this->name}", "", array(
				"Membership List" => "/{$this->name}",
				"New Member" => "/{$this->name}?cmd=memberNew",
				"Renew Member" => "/{$this->name}?cmd=memberRenew",
				"Renew Member From" => "/{$this->name}?cmd=memberRenewForm",
				"Full list (CSV)" => "/{$this->name}?cmd=getList?type=all",
				"RO: Groups Members (CSV)" => "/{$this->name}?cmd=getList?type=groupAll",
				"RO: Recent Changes (CSV)" => "/{$this->name}?cmd=getList?type=groupChanges",
				"RO: Email Group" => "/{$this->name}?cmd=emailGroup",
				"Journal: Addresses (CSV)" => "/{$this->name}?cmd=getList?type=journalAddresses",
				"Journal: Renewal Postal (CSV)" => "/{$this->name}?cmd=getList?type=renewalNoticesPostal",
				"Journal: Renewal All (CSV)" => "/{$this->name}?cmd=getList?type=renewalNotices",
				"Journal: New members (CSV)" => "/{$this->name}?cmd=getList?type=newMembers",
				"Journal: New members (TXT)" => "/{$this->name}?cmd=newMembers",
				"Statistics" => "/{$this->name}?cmd=statistics",
				"Delete Non Renewals" => "/{$this->name}?cmd=deleteNonRenewals",
				));
		}
		else if($this->checkAccess() == 1){
			return new BMenu("Membership", "/{$this->name}", "", array(
				"Membership List" => "/{$this->name}",
				"Full list (CSV)" => "/{$this->name}?cmd=getList?type=all",
				"RO: Groups Members (CSV)" => "/{$this->name}?cmd=getList?type=groupAll",
				"RO: Recent Changes (CSV)" => "/{$this->name}?cmd=getList?type=groupChanges",
				"RO: Email Group" => "/{$this->name}?cmd=emailGroup",
				"Journal: Addresses (CSV)" => "/{$this->name}?cmd=getList?type=journalAddresses",
				"Journal: Renewal Postal (CSV)" => "/{$this->name}?cmd=getList?type=renewalNoticesPostal",
				"Journal: Renewal All (CSV)" => "/{$this->name}?cmd=getList?type=renewalNotices",
				"Journal: New members (CSV)" => "/{$this->name}?cmd=getList?type=newMembers",
				"Journal: New members (TXT)" => "/{$this->name}?cmd=newMembers",
				"Statistics" => "/{$this->name}?cmd=statistics",
				));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->test || $this->testEmails){
//			print "<h1><font color='red'>System is in test mode: testPayments: {$this->test} testEmails: {$this->testEmails}</font></h1>\n";
			$this->sys->status = "<h1><font color='red'>System is in test mode: testPayments: {$this->test} testEmails: {$this->testEmails}</font></h1>\n";
		}

		// Special renewal
		if(isset($_GET["renew"])){
			$getRenew = filterString($_GET["renew"]);
			
			$l = explode(",", $getRenew);
			if(count($l) != 2){
				return "Error: Renew format is id,password";
			}
			$this->cmd = $_GET["cmd"] = "memberRenew";
			$_GET["id"] = $l[0];
			$_GET["renewPassword"] = $l[1];
		}

		if($this->checkAccess() == 2){
			switch($this->cmd){
			case "configEdit":		return $this->configEdit($uri);		break;
			case "viewList":		return $this->viewList($uri);		break;
			case "view":			return $this->view($uri);		break;
			case "viewCommand":		return $this->viewCommand($uri);	break;
			case "add":			return $this->edit($uri, 1, 0);		break;
			case "edit":			return $this->edit($uri, 1, 0);		break;
			case "save":			return $this->save($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "archiveView":		return $this->view($uri, 1);		break;
			case "archiveEdit":		return "Not implemented";		break;
			case "archiveDelete":		return "Not implemented";		break;
			case "editRaw":			return $this->editRaw($uri);		break;
			case "saveRaw":			return $this->saveRaw($uri);		break;

			case "memberNew":		return $this->memberNew($uri);		break;
			case "memberRenew":		return $this->memberRenew($uri);	break;
			case "memberRenewForm":		return $this->memberRenewForm($uri);	break;
			case "getList":			return $this->getList($uri);		break;
			case "newMembers":		return $this->newMembers($uri);		break;
			case "emailAll":		return $this->emailAll($uri);		break;
			case "emailList":		return $this->emailList($uri);		break;
			case "emailGroup":		return $this->emailGroup($uri);		break;
			case "renewalSendEmails":	return $this->renewalSendEmails($uri);	break;
			case "renewalForms":		return $this->renewalForms($uri);	break;
			case "renewalFormsPdf":		return $this->renewalFormsPdf($uri);	break;
			case "reminderForms":		return $this->reminderForms($uri);	break;
			case "reminderFormsPdf":	return $this->reminderFormsPdf($uri);	break;
			case "addWebAccount":		return $this->addWebAccount($uri);	break;
			case "addWebAccountAll":	return $this->addWebAccountAll($uri);	break;
			case "linkWebAccountAll":	return $this->linkWebAccountAll($uri);	break;
			case "statistics":		return $this->statistics($uri);		break;
			case "help":			return $this->help($uri);		break;
			case "deleteNonRenewals":	return $this->deleteNonRenewals($uri);	break;
			
			case "test":			return $this->test($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else if($this->checkAccess() == 1){
			switch($this->cmd){
			case "viewList":		return $this->viewList($uri);		break;
			case "view":			return $this->view($uri);		break;
			case "memberNew":		return $this->memberNew($uri);		break;
			case "memberRenew":		return $this->memberRenew($uri);	break;
			case "memberRenewForm":		return $this->memberRenewForm($uri);	break;
			case "getList":			return $this->getList($uri);		break;
			case "newMembers":		return $this->newMembers($uri);		break;
			case "emailAll":		return $this->emailAll($uri);		break;
			case "emailList":		return $this->emailList($uri);		break;
			case "emailGroup":		return $this->emailGroup($uri);		break;
			case "statistics":		return $this->statistics($uri);		break;
			case "help":			return $this->help($uri);		break;
			default: 			return $this->viewList($uri);		break;
			}
		}
		else {
			// Non admin users
			switch($this->cmd){
			case "memberNew":		return $this->memberNew($uri);		break;
			case "memberRenew":		return $this->memberRenew($uri);	break;
			case "memberRenewForm":		return $this->memberRenewForm($uri);	break;
			case "save":			return $this->save($uri);		break;
			case "renewalSendEmails":	return $this->renewalSendEmails($uri);	break;
			case "help":			return $this->help($uri);		break;
			default: 			$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Membership")); break;
			}
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess() == 2){
			$o = new BMenu("Membership", "", "", array(
				new BMenu("List", 	"","href='?cmd=viewList'"),
				new BMenu("New Member",	"", "href='?cmd=memberNew'"),
				new BMenu("Member Renew Form",	"", "href='?cmd=memberRenewForm'"),
				new BMenu("<b>Downloads</b>", "", ""),
				new BMenu("Full list", "", "href='?cmd=getList&amp;type=all'"),
				new BMenu("Email all", "", "href='?cmd=emailAll'"),
				new BMenu("Reminder Forms", "", "href='?cmd=reminderForms'"),
				new BMenu("RO: Groups Members", "", "href='?cmd=getList&amp;type=groupAll'"),
				new BMenu("RO: Recent Changes", "", "href='?cmd=getList&amp;type=groupChanges'"),
				new BMenu("RO: Email group", "", "href='?cmd=emailGroup'"),
				new BMenu("JO: Addresses", "", "href='?cmd=getList&amp;type=journalAddresses'"),
				new BMenu("JO: Renewal Postal", "", "href='?cmd=getList&amp;type=renewalNoticesPostal'"),
				new BMenu("JO: Renewal All", "", "href='?cmd=getList&amp;type=renewalNotices'"),
				new BMenu("JO: Renewal Forms", "", "href='?cmd=renewalForms'"),
				new BMenu("JO: New members", "", "href='?cmd=getList&amp;type=newMembers'"),
				new BMenu("JO: New members txt", "", "href='?cmd=newMembers'"),
				new BMenu("<b>Management</b>", "", ""),
				new BMenu("Statistics", "",	"href='?cmd=statistics'"),
				new BMenu("Configuration", "",	"href='?cmd=configEdit'"),
				new BMenu("Delete Non Renewals", "", "href='?cmd=deleteNonRenewals'"),
				new BMenu("<b>Management Special</b>", "", ""),
				new BMenu("Send Renewal Emails", "",	"href='?cmd=renewalSendEmails'"),
				//new BMenu("Add all website accounts", "",	"href='?cmd=addWebAccountAll'"),
				new BMenu("Link all website accounts", "",	"href='?cmd=linkWebAccountAll'"),
				new BMenu("Help", "",		"href='?cmd=help'"),
				));
		}
		else if($this->checkAccess() == 1){
			$o = new BMenu("Membership", "", "", array(
				new BMenu("List", 	"","href='?cmd=viewList'"),
				new BMenu("New Member",	"", "href='?cmd=memberNew'"),
				new BMenu("Member Renew Form",	"", "href='?cmd=memberRenewForm'"),
				new BMenu("<b>Downloads</b>", "", ""),
				new BMenu("RO: Groups Members", "", "href='?cmd=getList&amp;type=groupAll'"),
				new BMenu("RO: Recent Changes", "", "href='?cmd=getList&amp;type=groupChanges'"),
				new BMenu("RO: Email group", "", "href='?cmd=emailGroup'"),
				new BMenu("JO: Addresses", "", "href='?cmd=getList&amp;type=journalAddresses'"),
				new BMenu("JO: Renewal Postal", "", "href='?cmd=getList&amp;type=renewalNoticesPostal'"),
				new BMenu("JO: Renewal All", "", "href='?cmd=getList&amp;type=renewalNotices'"),
				new BMenu("JO: Renewal Forms", "", "href='?cmd=renewalForms'"),
				new BMenu("JO: New members", "", "href='?cmd=getList&amp;type=newMembers'"),
				new BMenu("JO: New members txt", "", "href='?cmd=newMembers'"),
				new BMenu("", "", ""),
				new BMenu("Statistics", "",	"href='?cmd=statistics'"),
				new BMenu("Help", "",		"href='?cmd=help'"),
				));
		}
		else {
			$o = new BMenu("Membership", "", "", array(
				new BMenu("New Member",	"", "href='?cmd=memberNew'"),
				new BMenu("Member Renew Form",	"", "href='?cmd=memberRenewForm'"),
				new BMenu("Renew Member",	"", "href='?cmd=memberRenew'"),
				));
		}
		return $o;
	}

	// Get a member record with joint members records if required
	function recordGet($table, $id, $withJoint = 0){
		$obj = $this->sys->db->getObject($table, $id);
		if($obj && $withJoint){
			$obj->jointList = $this->sys->db->getObjectList($table, "", "`fullId`='{$obj->id}'", "", "");
			usort($obj->jointList, "jointCompareMembershipNumber");
		}
		return $obj;
	}

	// Save a member record with joint members records
	function recordSave($table, $obj, $withJoint = 0){
		$o = clone $obj;
		unset($o->jointList);
		if($o->id)
			$this->sys->db->putObject($table, $o);
		else
			$this->sys->db->appendObject($table, $o);
		
		$obj->id = $o->id;

		if($withJoint && count($obj->jointList)){
			foreach($obj->jointList as &$joint){
				$joint->fullId = $o->id;
				if($joint->id)
					$this->sys->db->putObject($table, $joint);
				else if(trim($joint->forename) != "")
					$this->sys->db->appendObject($table, $joint);
			}
		}
	}

	// Save a member record with joint members records
	function recordDelete($table, $id, $withJoint = 0){
		$table = $this->sys->databasePrefix . $table;
		if($withJoint)
			$cmd = "delete from `{$table}` where `id` = '{$id}' or `fullId` = '{$id}'";
		else
			$cmd = "delete from `{$table}` where `id` = '{$id}'";

		$this->sys->db->query($cmd);
	}
	
	function main($uri){
		$page = "<h2>Membership</h2>\n";
		$page .= "<p>Please use the functions available in the left hand menu.</p>\n";

		return $page;	
	}
	
	function help($uri){
		$help = $this->sys->baseDir . "/modules/membershipTc/info.html";
		
		return file_get_contents($help);
	}


	function viewList($uri){
		$listSize = 40;
		
		if(isset($_GET["pos"]))
			$pos = filterToInt($_GET["pos"]);
		else
			$pos = 0;
		
		if(!isset($_GET["sel_group"]) && isset($_GET["group"]))
			$_GET["sel_group"] = $_GET["group"];

		$number = 10;
		$posNext = $pos + $number;
		$posPrev = $pos - $number;
		if($posPrev < 0)
			$posPrev = 0;

		$page = "<h2>Membership</h2>\n";
		
		if(0){
			$post = "";
			foreach($_POST as $n => $v) $post .= $n . " => " . $v . ",";
			print "MembershipTc::viewList: POST: $post<br>\n";
			debug_log("MembershipTc::viewList: POST: $post");
			$get = "";
			foreach($_GET as $n => $v) $get .= $n . " => " . $v . ",";
			print "MembershipTc::viewList: GET: $get<br>\n";
			debug_log("MembershipTc::viewList: GET: $get");
//			BDatabase::$debug = 1;
		}
		
		$sel = new BSelect("select", "Select", "", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("pending", "", "", "equal", "tristate"),
			new BViewTypeSelect("joinedDate", "", "New", "last2Months", "tristate"),
			new BViewTypeSelect("status", "", "", "equal", "select", array_merge(array(""), $this->statusTypes)),
			new BViewTypeSelect("renewalDate", "", "RenewalDate", "period", "select", array("", "Last 2 Months", "Last Month", "To Now", "To Next Month", "To Next Two Months", "To Next Three Months", "Next Month", "Over 1 Months ago", "Over 2 Months ago", "Over 3 Months ago")),
//			new BViewTypeSelect("renewalDate", "", "RenewalDate", "like"),
			new BViewTypeSelect("modifiedDate", "", "", "period", "select", array("", "Last 2 Months", "Last Month", "To Now", "To Next Month", "Next Month")),
			new BViewTypeSelect("forename", "", "Forename", "likeAfterStart"),
			new BViewTypeSelect("surname", "", "Surname", "likeAfterStart"),
			new BViewTypeSelect("address", "", "Address", "like"),
			new BViewTypeSelect("postcode", "", "Postcode", "postcode"),
			
			new BViewTypeSelect("membershipType", "", "", "equal", "select", array_merge(array(""), $this->membershipTypesAdmin)),
			new BViewTypeSelect("membershipNumber", "", "", "like"),
			new BViewTypeSelect("group", "", "Group", "like", "select", array_merge(array(""), $this->groupsList())),

			new BViewTypeSelect("fullId", "", "Joint", "isset", "tristate"),
			new BViewTypeSelect("archive", "", "", "", "checkbox"),
			new BViewTypeSelect("audioCd", "", "", "equal", "tristate"),
			
			new BViewTypeSelect("emailAddress", "", "Email", "like"),
			new BViewTypeSelect("paymentType", "", "", "equal", "select", array("", "card", "bacs", "cheque", "standingOrder", "complimentary")),
//			new BViewTypeSelect("fill0", "", "", "", "empty"),
			
			new BViewTypeSelect("order", "", "", "order", "select", array("surname,id", "renewalDate", "!renewalDate", "status", "membershipNumber", "!membershipNumber", "postcode", "group", "memberShipType", "id", "!id", "!modifiedDate")),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			));

		$sel->addExtraButton("download", "Download CSV Basic");
		$sel->addExtraButton("downloadAll", "Download CSV All Fields");
		$sel->addExtraButton("emailAll", "Email this selection");
		$sel->addExtraButton("emailFromMe", "Email from me");
		$sel->getObj()->pending = 1;
		$sel->getObj()->archive = 0;
		$sel->getObj()->fullId = 0;
		$sel->pagerSet(0, $listSize, 0);
		$sel->update();
		$where = $sel->getSelectString();
		if($sel->getObj()->joinedDate == 1){
			$where .= " and `status` != 'renewForm'";
		}
		$order = $sel->getOrder();
		if($order == "")
			$order = "surname,id";

//		print "SelString: {$where} Order: {$order}<br>\n";

		if(isset($_POST["download"]) || isset($_GET["download"])){
			if(($sel->getObj()->fullId == 1) || (strval($sel->getObj()->fullId) == ""))
				return $this->getListCsv($sel->getObj()->archive, $where, "id,fullId,status,membershipType,membershipNumber,prefix,forename,surname,emailAddress,address,city,postcode,group");
			else
				return $this->getListCsv($sel->getObj()->archive, $where, "id,status,membershipType,membershipNumber,prefix,forename,surname,emailAddress,address,city,postcode,group");
		}
		else if(isset($_POST["downloadAll"]) || isset($_GET["downloadAll"])){
			return $this->getListCsv($sel->getObj()->archive, $where, "");
		}
		else if(isset($_POST["emailFromMe"]) || isset($_GET["emailFromMe"])){
			$emailAddress = "";
			$emailAddresses = "";
			$data = array(0);
			for($pos = 0; count($data); $pos += 100){
				if($archive)
					$data = $this->sys->db->getObjectList("MembershipOld", "emailAddress", $where . " and `emailAddress` != '' and `emailSend`='1'", "", "{$pos},100");
				else
					$data = $this->sys->db->getObjectList("Membership", "emailAddress", $where . " and `emailAddress` != '' and `emailSend`='1'", "", "{$pos},100");

				foreach($data as $e){
					if($emailAddress == "")
						$emailAddress = $e->emailAddress;
					else if($emailAddresses != "")
						$emailAddresses .= ";{$e->emailAddress}";
					else
						$emailAddresses .= $e->emailAddress;
				}
			}
			$cmd = "mailto:{$emailAddress}?subject=TandemClub: &bcc=" . $emailAddresses;
			header("Location: {$cmd}");
			die();
		}
		else if(isset($_POST["emailAll"]) || isset($_GET["emailAll"])){
			$emailAddresses = "";
			$data = array(0);
			for($pos = 0; count($data); $pos += 100){
				if($archive)
					$data = $this->sys->db->getObjectList("MembershipOld", "emailAddress", $where . " and `emailAddress` != '' and `emailSend`='1'", "", "{$pos},100");
				else
					$data = $this->sys->db->getObjectList("Membership", "emailAddress", $where . " and `emailAddress` != '' and `emailSend`='1'", "", "{$pos},100");

				foreach($data as $e){
					if($emailAddresses != "")
						$emailAddresses .= ",{$e->emailAddress}";
					else
						$emailAddresses .= $e->emailAddress;
				}
			}
			return $this->emailList($uri, $emailAddresses);
		}
		else {
			$view = new BViewDatabase("list", "action");
			if($sel->getObj()->fullId == 1){
				$view->setView(array(
					new BViewType("id"),
					new BViewType("fullId"),
					new BViewType("pending"),
					new BViewType("status"),
					new BViewType("membershipType", "MemType"),
					new BViewType("membershipNumber", "MemNumber"),
					new BViewType("forename", "Forename"),
					new BViewType("surname", "Surname"),
					new BViewType("postcode"),
					new BViewType("emailAddress", "", "link", array("\$obj->emailAddress", "mailto:\$obj->emailAddress")),
					new BViewType("telephoneHome"),
					new BViewType("group"),
					new BViewType("notes"),
					));
			}
			else if(strval($sel->getObj()->fullId) == ""){
				$view->setView(array(
					new BViewType("id"),
					new BViewType("fullId"),
					new BViewType("pending"),
					new BViewType("status"),
					new BViewType("membershipType", "MemType"),
					new BViewType("membershipNumber", "MemNumber"),
					new BViewType("forename", "Forename"),
					new BViewType("surname", "Surname"),
					new BViewType("postcode"),
					new BViewType("emailAddress", "", "link", array("\$obj->emailAddress", "mailto:\$obj->emailAddress")),
					new BViewType("telephoneHome"),
					new BViewType("group"),
					new BViewType("renewalDate"),
					new BViewType("paymentId", "", "link", array("\$obj->paymentId", "/payments?cmd=edit&amp;id=\$obj->paymentId")),
					new BViewType("paymentAmountPaid", "Paid"),
					new BViewType("notes"),
					));
			}
			else{
				$view->setView(array(
					new BViewType("id"),
					new BViewType("pending"),
					new BViewType("status"),
					new BViewType("membershipType", "Mem Type"),
					new BViewType("membershipNumber", "Mem Number"),
					new BViewType("forename", "Forename"),
					new BViewType("surname", "Surname"),
					new BViewType("postcode"),
					new BViewType("emailAddress", "", "link", array("\$obj->emailAddress", "mailto:\$obj->emailAddress")),
					new BViewType("telephoneHome"),
					new BViewType("group"),
					new BViewType("renewalDate"),
					new BViewType("paymentId", "", "link", array("\$obj->paymentId", "/payments?cmd=edit&amp;id=\$obj->paymentId")),
					new BViewType("paymentAmountPaid", "Paid"),
					new BViewType("notes"),
					));
			}

			$view->setSelection($where);
			$view->setLimit($sel->pagerGetLimitString());
			
			if($sel->getObj()->archive){
				$view->setName("archive");
				$view->setOrder($order);
				$totalNumber = $this->sys->db->getObjectNumber("MembershipOld", $where);
				$view->setTable("MembershipOld");
			}
			else {
				$view->setOrder($order);
				$totalNumber = $this->sys->db->getObjectNumber("Membership", $where);
				$view->setTable("Membership");
			}

			$sel->pagerSet(0, $listSize, $totalNumber);

			$page .= $sel->getHtml();
			$page .= $view->getHtml();
			return $page;
		}
	}

	function getList($uri){
		if(isset($_GET["type"]))
			$type = filterString($_GET["type"]);
		else
			$type = "all";
			
		if($type == "all"){
			$fields = "id,fullId,status,prefix,forename,surname,emailAddress,address,city,postcode,country,group";
			$select = "";
		}
		else if($type == "groupAll"){
			if($this->sys->user && ($this->sys->user->group != "")){
				$group = $this->sys->user->group;

				$fields = "id,status,prefix,forename,surname,emailAddress,address,city,postcode,country";
				$select = "`fullId`='0' and `group`='$group'";
			}
			else {
				return "User does not belong to a group";
			}
		}
		else if($type == "groupChanges"){
			if($this->sys->user && ($this->sys->user->group != "")){
				$group = $this->sys->user->group;

				$startDate = new BDate("now");
				$startDate->addSeconds(-86400 * 3 * 31);
				$endDate = new BDate("now");
				$fields = "id,status,forename,surname,emailAddress,address,city,postcode";
				$select = "((`fullId`='0') and (`group` = '$group') and (`modifiedDate` >= '{$startDate->get()}') and (`modifiedDate` <= '{$endDate->get()}'))";
			}
			else {
				return "User does not belong to a group";
			}
		}
		else if($type == "journalAddresses"){
			$fields = "id,status,membershipNumber,forename,surname,emailAddress,address,city,postcode,country";
			$select = "`fullId`='0' and `journalByPost`='1'";
		}
		else if($type == "renewalNotices"){
			$startDate = new BDate("1970-01-01");
			$endDate = new BDate("now");
			$endDate->addSeconds(86400 * 3 * 31);
			$fields = "id,status,forename,surname,emailAddress,address,city,postcode,country,renewalDate";
			$select = "((`fullId`='0') and (`renewalDate` >= '{$startDate->get()}') and (`renewalDate` <= '{$endDate->get()}'))";
		}
		else if($type == "renewalNoticesPostal"){
			$startDate = new BDate("1970-01-01");
			$endDate = new BDate("now");
			$endDate->addSeconds(86400 * 3 * 31);
			$fields = "id,status,forename,surname,emailAddress,address,city,postcode,country,renewalDate";
			$select = "((`fullId`='0') and (`emailAddress` = '') and (`renewalDate` >= '{$startDate->get()}') and (`renewalDate` <= '{$endDate->get()}'))";
		}
		else if($type == "newMembers"){
			$date = new BDate("now");
			$date->addSeconds(-86400 * 2 * 31);
			$fields = "id,status,forename,surname,emailAddress,address,city,postcode,country";
			$select = "`fullId`='0' and `status` != 'renewForm' and `joinedDate` >= '{$date->get()}'";
		}
		else {
			return "Not implemented as yet ...";
		}

		return $this->getListCsv(0, $select, $fields);
	}
	
	function newMembers($url){
		$page = "";
		$date = new BDate("now");
		$date->addSeconds(-86400 * 2 * 31);
		$select = "`fullId`='0' and `status` != 'renewForm' and `joinedDate` >= '{$date->get()}'";
		$data = $this->sys->db->getObjectList("Membership", "id", $select, "", "");
		foreach($data as $obj){
			$obj = $this->recordGet("Membership", $obj->id, 1);
			
			$str = ucfirst(strtolower($obj->forename));
			$surname = ucfirst(strtolower($obj->surname));
			$place = $obj->city;
			if(($obj->country != "UK") && ($obj->country != "United Kingdom"))
				$place .= " " . $obj->country;

			for($i = 0; $i < count($obj->jointList); $i++){
				if($i == (count($obj->jointList)) - 1)
					$sep = " and ";
				else
					$sep = ", ";
					
				if(ucfirst(strtolower($obj->jointList[$i]->surname)) == $surname){
					$str .= $sep . ucfirst(strtolower($obj->jointList[$i]->forename));
				}
				else {
					$str .= " " . $surname . $sep . ucfirst(strtolower($obj->jointList[$i]->forename));
				}
				$surname = ucfirst(strtolower($obj->jointList[$i]->surname));
			}
			$str .= " " . $surname;
			if($place != "")
				$str .= " from <b>{$place}</b>";
			$page .= "{$str}</br>";
		}
		return $page;
	}

	function getListCsv($archive, $select, $fields){
		$addressExpand = 1;
		$fieldsSql = "";
		if($fields != ""){
			$fieldsSql = array();
			foreach(explode(",", $fields) as $f){
				$fieldsSql[] = "`" . $f . "`";
			}
			$fieldsSql = join(",", $fieldsSql);
		}

		// Create header info
		if($archive)
			$data = $this->sys->db->getObjectList("MembershipOld", $fieldsSql, $select, "", "1");
		else
			$data = $this->sys->db->getObjectList("Membership", $fieldsSql, $select, "", "1");


		if(($fields == "") && count($data)){
			$fields = join(",", array_keys($data[0]->getMembers()));
		}
		
		header('Content-Type: text/csv; charset=utf-8');
		header('Content-Disposition: attachment; filename=data.csv');
		
		// List the fields
		$line = "";
		foreach(explode(",", $fields) as $n){
			if($line != "")
				$line .= ",";
			if($addressExpand && ($n == "address")){
				$line .= "Address1,Address2,Address3,Address4,Address5";
			}
			else {
				$line .= ucfirst($n);
			}
		}
		print("{$line}\n");

		// Loop over data
		for($pos = 0; count($data); $pos += 100){
			if($archive)
				$data = $this->sys->db->getObjectList("MembershipOld", $fieldsSql, $select, "", "{$pos},100");
			else
				$data = $this->sys->db->getObjectList("Membership", $fieldsSql, $select, "", "{$pos},100");

			if(count($data)){
				foreach($data as $obj){
					$line = "";
					foreach(explode(",", $fields) as $n){
						if($addressExpand && ($n == "address")){
//							if($line != "")
//								$line .= ",";
//							$line .= '"' . trim(quoteForCsv($obj->{"{$n}"})) . '"';
							$al = addressLines($obj->{"{$n}"}, 5);
							foreach($al as $a){
								$line .= ',"' . trim(quoteForCsv($a)) . '"';
							}
						}
						else {
							if($line != "")
								$line .= ",";
							$line .= '"' . trim(quoteForCsv($obj->{"{$n}"})) . '"';
						}
					}
					print("{$line}\n");
				}
			}
		}

		return null;
	}

	function view($uri, $archive = 0){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$id = filterToInt($_GET["id"]);
			if($archive)
				$obj = $this->recordGet("MembershipOld", $id, 1);
			else
				$obj = $this->recordGet("Membership", $id, 1);
		}

		$page = "<h2>Membership View</h2>";
		
		$h = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='?cmd=viewCommand' method='post'");
		$h->appendNode("h3", "", "Names of people");
		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Prefix");
		$r->appendNode("th", "", "Forename");
		$r->appendNode("th", "", "Surname");
		$r->appendNode("th", "", "Member Num");
		$r->appendNode("th", "", "Date of Birth (If under 18)");
		$r->appendNode("th", "", "Family");

		$r = &$t->appendNode("tr");
		$r->appendNode("td", "", "Name");
		$r->appendNode("td", "", $obj->prefix);
		$r->appendNode("td", "", $obj->forename);
		$r->appendNode("td", "", $obj->surname);
		$r->appendNode("td", "", $obj->membershipNumber);
		$r->appendNode("td", "", $obj->dateOfBirth);
		$r->appendNode("td", "", $obj->family);

		if($obj->fullId == 0){
			for($n = 0; ($n < $this->numJoint) && ($n < count($obj->jointList)); $n++){
				if($n == 0){
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan='8'", "<b>First Joint member is free</b>");
				}
				else if($n == 1){
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan='8'", "<b>Further joint members. Family members under 18 and living at home address are free</b>");
				}

				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", "Joint " . ($n + 1));
				if(count($obj->jointList) > $n){
					$r->appendNode("td", "", $obj->jointList[$n]->prefix);
					$r->appendNode("td", "", $obj->jointList[$n]->forename);
					$r->appendNode("td", "", $obj->jointList[$n]->surname);
					$r->appendNode("td", "", $obj->jointList[$n]->membershipNumber);
					$r->appendNode("td", "", $obj->jointList[$n]->dateOfBirth);
					$r->appendNode("td", "", $obj->jointList[$n]->family);
				}
			}
		}
		
		$h->appendNode("h3", "", "Contact");

		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("Postal Address (first lines)", "", "view_textarea", $obj->address, "height:50px; width:400px"); 
		$t->appendTableInput("City or Town", "", "view", $obj->city); 
		$t->appendTableInput("Postcode", "", "view", $obj->postcode); 
		$t->appendTableInput("Country", "", "view", $obj->country);
		$r = &$t->appendNode("tr"); $r->appendNode("th", "align='left'", "Email Address"); $r->appendNode("td")->appendNode("a", "href='mailto:{$obj->emailAddress}'", $obj->emailAddress); 
		$t->appendTableInput("Home Telephone", "", "view", $obj->telephoneHome); 
		$t->appendTableInput("Mobile Telephone", "", "view", $obj->telephoneMobile); 

		if($obj->fullId == 0){
			$t->appendTableInput("Journal sent by email", "", "view", $obj->journalByEmail); 
			$t->appendTableInput("Journal on a audio CD", "", "view", $obj->audioCd); 
			$t->appendTableInput("Journal sent by post", "", "view", $obj->journalByPost); 
			$t->appendTableInput("Information by email", "", "view", $obj->emailSend); 
		}
		$t->appendTableInput("Notes", "", "view_textarea", $obj->notes, "height:100px; width:400px"); 

		$h->appendNode("h3", "", "Membership");
		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		if($obj->fullId == 0){
			$t->appendTableInput("TC Membership Type", "membershipType", "view", $obj->membershipType); 
		}
		$t->appendTableInput("TC Membership Number", "membershipNumber", "view", $obj->membershipNumber, "width:300px;"); 

		if($obj->fullId == 0){
			$t->appendTableInput("Payment Type", "paymentType", "view", $obj->paymentType); 
			$t->appendTableInput("Payment Currency", "paymentCurrency", "view", $obj->paymentCurrency); 
			$t->appendTableInput("Payment Amount", "paymentAmount", "view", $obj->paymentAmount);
			$t->appendTableInput("Attracted From", "attractedFrom", "view", $obj->attractedFrom); 
			$t->appendTableInput("GDPR Accepted", "gdprAccepted", "view", $obj->gdprAcceptance); 
		}
		
		$h->appendNode("h3", "", "Admin");
		$t = &$h->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("Pending", "pending", "view", $obj->pending, "width:300px;"); 
		if($obj->fullId == 0){
			$t->appendTableInput("Status", "status", "view", $obj->status, "", $this->statusTypes); 
			$t->appendTableInput("JoinedDate", "joinedDate", "view", $obj->joinedDate, "width:300px;"); 
			$t->appendTableInput("Group", "group", "view", $obj->group); 
		}
		$t->appendTableInput("NotesAdmin", "notesAdmin", "view_textarea", $obj->notesAdmin, "height:100px; width:400px;"); 
		if($obj->fullId == 0){
			$t->appendTableInput("PaymentId", "paymentId", "view", $obj->paymentId, "width:300px;"); 
			$t->appendTableInput("PaymentDate", "paymentDate", "view", $obj->paymentDate, "width:300px;"); 
			$t->appendTableInput("PaymentAmountPaid", "paymentAmountPaid", "view", $obj->paymentAmountPaid, "width:300px;"); 
			$t->appendTableInput("RenewalDate (YYYY-MM-DD)", "renewalDate", "view", $obj->renewalDate, "width:300px;"); 
			$t->appendTableInput("RenewalSent", "renewalSent", "view", $obj->renewalSent, "width:300px;"); 
			$t->appendTableInput("Why Resigned", "whyResigned", "view", $obj->whyResigned, "width:300px;");
			$t->appendTableInput("Website userId", "userId", "view", $obj->userId, "width:300px;");
		}
		$t->appendTableInput("ModifiedDate", "modifiedDate", "view", $obj->modifiedDate, "width:300px;"); 
		$t->appendTableInput("ModifiedUserName", "modifiedUserName", "view", $obj->modifiedUserName, "width:300px;"); 

		$t->appendTableInput("RenewPassword", "renewPassword", "view", $obj->renewPassword, "width:300px;"); 
		$t->appendTableInput("UpdatePassword", "updatePassword", "view", $obj->updatePassword, "width:300px;"); 

		// Buttons
		$h->appendNode("input", "type='hidden' name='id' value='{$obj->id}'");
		$t = &$h->appendNode("table");
		$r = &$t->appendNode("tr");
		$r->appendNode("td")->appendNode("input", "type='submit' name='reinstate' style='background-color: lightgreen; font-size: 200%;' value='Reinstate Member' onclick='this.value=\"Reinstating, please wait...\";'");
		$r->appendNode("td")->appendText("Reinstates member and joint members to membership database.");

		$page = $page . $h->render();

		return $page;
	}

	function viewCommand($arg){
//		print "viewCommand<br>";
//		print "Form: " . print_r($_POST, 1) . "<br>\n";

		$page = "";
		if(isset($_POST["reinstate"])){
			if(isset($_POST["id"])){
				$obj = $this->recordGet("MembershipOld", filterToInt($_POST["id"]), 1);
			}
			else {
				$this->sys->pageDisplayError(new BError(1, "You need to supply a record id to reinstate."));
				return "";
			}
			
			// Check if exists
			if($obj == Null){
				$this->sys->pageDisplayError(new BError(1, "You need to supply a valid archive record id to reinstate."));
				return "";
			}

//			$joint0 = $obj->jointList[0];
//			print "Reinstate: {$obj->forename} {$obj->membershipNumber} Joint1: {$joint0->membershipNumber}<br>";
			$obj->id = 0;
			$obj->pending = 1;
			$obj->status = "renew";
			foreach($obj->jointList as $j){
				$j->id = 0;
			}

			$date = new BDate("now");
			$obj->modifiedDate = $date->get();
			if($this->sys->user)
				$obj->modifiedUserName = $this->sys->user->userName;

			$this->recordSave("Membership", $obj, 1);
			$page = "Reinstated {$obj->forename} {$obj->surname} membership record with state renew\n";
		}
		return $page;
	}

	function editRaw($arg){
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Raw Edit of Membership Record</h2>";
			$obj = $this->sys->db->getObject("Membership", filterToInt($_GET["id"]));
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "You need to supply a record id to edit."));
			return "";
		}
		
		$view = new BEditObj("member", "Membership record", "?cmd=saveRaw", "");
		$view->setView(array(
			new BViewType("id", "Id"),
			new BViewType("fullId"),
			new BViewType("pending", "Pending", "checkbox"),
			new BViewType("status", "", "select", array("new", "member", "renew", "renewDone", "renewForm", "lapsed", "leave")),
			new BViewType("membershipType", "", "select", array("1 year", "5 year", "Honorary", "Complimentary", "Special")),
			new BViewType("membershipNumber"),
			new BViewType("joinedDate", "", "date"),
			new BViewType("group"),
	
			new BViewType("prefix", "Prefix", "select", array("", "Mr", "Mrs", "Ms", "Miss", "Dr", "Prof", "Rev", "Sir")),
			new BViewType("forename", "Forename"),
			new BViewType("surname", "Surname"),
			new BViewType("dateOfBirth", "", "date"),
			new BViewType("family", "", "checkbox"),
			
			new BViewType("address", "Address", "textarea", "", 400, 50),
			new BViewType("city"),
			new BViewType("postcode"),
			new BViewType("country"),
			new BViewType("emailAddress"),
			new BViewType("telephoneHome"),
			new BViewType("telephoneMobile"),
			new BViewType("journalByEmail", "", "checkbox"),
			new BViewType("audioCd", "", "checkbox"),
			new BViewType("journalByPost", "", "checkbox"),
			new BViewType("emailSend", "", "checkbox"),
			new BViewType("notes", "", "textarea", "", 400, 100),
			
			new BViewType("renewalDate", "", "date"),
			new BViewType("renewalSent"),
			
			new BViewType("paymentType", "", "select", array("card", "bacs", "cheque", "standingOrder", "complimentary")),
			new BViewType("paymentCurrency", "", "select", array("Pounds", "Euros")),
			new BViewType("paymentId"),
			new BViewType("paymentDate", "", "date"),
			new BViewType("paymentAmount"),
			new BViewType("paymentAmountPaid"),
			
			new BViewType("notesAdmin", "", "textarea", "", 400, 100),
			new BViewType("attractedFrom"),
			new BViewType("whyResigned"),
			new BViewType("userId"),
			new BViewType("modifiedDate", "", "view"),
			new BViewType("modifiedUserName", "", "view"),
			new BViewType("renewPassword"),
			new BViewType("updatePassword"),
			));

//		$view->setHelp("groupName", "Enter the regional groups name.");

		$view->setObj($obj);
		
		$page = "<h1><font color='red'>Edit the raw membership record. For special purposes only, dangerous to use ...</font></h1>";
		$page .= $view->getHtml();

		return $page;
	}
	
	function saveRaw(){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjMembership();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->groupEdit($obj);
			}
			
			$date = new BDate("now");
			$obj->modifiedDate = $date->get();
			if($this->sys->user)
				$obj->modifiedUserName = $this->sys->user->userName;

			if($obj->id){
				$this->sys->db->putObject("Membership", $obj);
			}
			else {
				$this->sys->db->appendObject("Membership", $obj);
			}
			
			$this->webUserUpdate($obj);
		}

		$page .= "<h3>Details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}


	function edit($arg, $admin = 0, $renewForm = 0){
		if(is_object($arg)){
			$obj = $arg;
			if($obj->status != "new")
				$editing = 1;
		}
		else if(isset($_GET["id"])){
			$obj = $this->recordGet("Membership", filterToInt($_GET["id"]), 1);
			$editing = 1;
		}
		else {
			$obj = $this->sys->db->getNewObject("Membership");
			$obj->jointList = array();
			$editing = 0;

			$obj->country = "United Kingdom";

			if($this->test && $this->sys->user){
				$a = explode(" ", $this->sys->user->name);
				if(count($a) == 2){
					$fn = $a[0];
					$sn = $a[1];
				}
				$obj->name = $this->sys->user->name;
				$obj->forename = $fn;
				$obj->surname = $sn;
				$obj->dateOfBirth = "1900-01-01";
				$obj->address = $this->sys->user->postalAddress;
				$obj->city = $this->sys->user->city;
				$obj->postcode = $this->sys->user->postcode;
				$obj->country = $this->sys->user->country;
				$obj->emailAddress = $this->sys->user->emailAddress;
				$obj->telephoneHome = $this->sys->user->telephoneHome;
				$obj->telephoneMobile = $this->sys->user->telephoneMobile;
			}
			$obj->journalByPost = 1;
			$obj->emailSend = 1;
			$obj->paymentType = "card";
			$obj->paymentAmount = "0.00";
		}

		// Set an update validation password
		$obj->updatePassword = makeRandomPassword();
		if($obj->id){
			$this->recordSave("Membership", $obj, 0);
		}

		if($admin){
			if($editing)
				$page = "<h2>Edit Membership</h2>";
			else
				$page = "<h2>New Membership</h2>";
		}
		else if($renewForm){
			$page = "<h2>Renew Membership</h2>";
			$page .= "<div style='color: red; font-size: 150%;'>Please only use this manual renew form if you don't have a suitable emailed or posted renew WEB link as those methods are easier for us to process</div>";
			$obj->status = "renewForm";
		}
		else {
			if($editing){
				$page = "<h2>Renew Membership</h2>";
				if($obj->status != "renewDone")
					$obj->status = "renew";
				if($obj->membershipType == "Complimentary"){
					$obj->membershipType = "1 year";
					$obj->paymentType = "card";
				}
			}
			else {
				$page = "<h2>New Membership</h2>";
				$obj->status = "new";
			}
			$obj->pending = 1;
		}
		
		if(!isset($obj->addAccount))
			$obj->addAccount = 1;

		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='?cmd=save' method='post'");

		if(!$admin){
			if($renewForm || $editing)
				$f->appendText($this->formNotesRenew);
			else
				$f->appendText($this->formNotesJoin);
		}

		$f->appendNode("input", "type='hidden' name='id' value='{$obj->id}'");

		$f->appendNode("h3", "", "Names of people");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "", "Item");
		$r->appendNode("th", "", "Prefix");
		$r->appendNode("th", "", "Forename");
		$r->appendNode("th", "", "Surname");
		if($admin || $renewForm || ($obj->membershipNumber != ""))
			$r->appendNode("th", "", "Member Num");
		$r->appendNode("th", "", "Date of Birth (If under 18)");
		$r->appendNode("th", "", "Family");
		if($editing)
			$r->appendNode("th", "", "Delete");

		// For joint editing add info on full member
		if($obj->fullId != 0){
			$fullObj = $this->recordGet("Membership", $obj->fullId, 0);

			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", "Full members Name");
			$r->appendTableRowInput("", "prefix", "view", $fullObj->prefix, "");
			$r->appendTableRowInput("", "forename", "view", $fullObj->forename, "width: 100px;");
			$r->appendTableRowInput("", "surname", "view", $fullObj->surname, "width: 100px;");
			$r->appendTableRowInput("", "membershipNumber", "view", $fullObj->membershipNumber, "width: 75px;");
			$r->appendTableRowInput("", "dateOfBirth", "view", $fullObj->dateOfBirth);
			$r->appendNode("td", "colspan='2'")->appendNode("a", "href='?cmd=edit&amp;id={$obj->fullId}'", "Edit full");
		}


		$r = &$t->appendNode("tr");
		$r->appendNode("td", "", "Name");
		$r->appendTableRowInput("", "prefix", "select", $obj->prefix, "", array("", "Mr", "Mrs", "Ms", "Miss", "Dr", "Prof", "Rev", "Sir"));
		$r->appendTableRowInput("", "forename", "text", $obj->forename, "width: 100px;");
		$r->appendTableRowInput("", "surname", "text", $obj->surname, "width: 100px;");
		if($admin || $renewForm || ($obj->membershipNumber != "")){
			if($renewForm || $admin)
				$r->appendTableRowInput("", "membershipNumber", "text", $obj->membershipNumber, "width: 75px;");
			else
				$r->appendTableRowInput("", "membershipNumber", "view", $obj->membershipNumber, "width: 75px;");
		}
		else {
			$f->appendNode("input", "type='hidden' name='membershipNumber' value='{$obj->membershipNumber}'");
		}

			
		$r->appendTableRowInput("", "dateOfBirth", "date", $obj->dateOfBirth);
		$r->appendNode("td");
		if($editing)
			$r->appendTableRowInput("", "full_delete", "checkbox", 0, "", "", "calc();");
		else
			$f->appendNode("input", "type='hidden' name='full_delete' id='full_delete' value='0'");

		if($obj->fullId == 0){
			for($n = 0; $n < $this->numJoint; $n++){
				if($n == 0){
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan='8'", "<b>First Joint member is free</b>");
				}
				else if($n == 1){
					$r = &$t->appendNode("tr");
					$r->appendNode("td", "colspan='8'", "<b>Further joint members. Family members under 18 and living at home address are free</b>");
				}

				$r = &$t->appendNode("tr");
				$r->appendNode("td", "", "Joint " . ($n + 1));
				if(count($obj->jointList) > $n){
					$r->appendTableRowInput("", "joint{$n}_prefix", "select", $obj->jointList[$n]->prefix, "", array("", "Mr", "Mrs", "Ms", "Miss", "Dr", "Prof", "Rev", "Sir"));
					$r->appendTableRowInput("", "joint{$n}_forename", "text", $obj->jointList[$n]->forename, "width: 100px;");
					$r->appendTableRowInput("", "joint{$n}_surname", "text", $obj->jointList[$n]->surname, "width: 100px;", "", "calc();");
					if($admin || $renewForm || ($obj->membershipNumber != "")){
						if($renewForm || $admin)
							$r->appendTableRowInput("", "joint{$n}_membershipNumber", "text", $obj->jointList[$n]->membershipNumber, "width: 75px;");
						else
							$r->appendTableRowInput("", "joint{$n}_membershipNumber", "view", $obj->jointList[$n]->membershipNumber, "width: 75px;");
					}
					else {
						$f->appendNode("input", "type='hidden' name='joint{$n}_membershipNumber' value='{$obj->jointList[$n]->membershipNumber}'");
					}
					$r->appendTableRowInput("", "joint{$n}_dateOfBirth", "date", $obj->jointList[$n]->dateOfBirth, "", "", "calc();");
					$r->appendTableRowInput("", "joint{$n}_family", "checkbox", $obj->jointList[$n]->family, "", "", "calc();");
					if($editing)
						$r->appendTableRowInput("", "joint{$n}_delete", "checkbox", 0, "", "", "calc();");
					else
						$f->appendNode("input", "type='hidden' name='joint{$n}_delete' id='joint{$n}_delete' value='0'");
				}
				else {
					$r->appendTableRowInput("", "joint{$n}_prefix", "select", "", "", array("", "Mr", "Mrs", "Ms", "Miss", "Dr", "Prof", "Rev", "Sir"));
					$r->appendTableRowInput("", "joint{$n}_forename", "text", "", "width: 100px;");
					$r->appendTableRowInput("", "joint{$n}_surname", "text", "", "width: 100px;", "", "calc();");

					if($admin || $renewForm || ($obj->membershipNumber != "")){
						if($renewForm || $admin)
							$r->appendTableRowInput("", "joint{$n}_membershipNumber", "text", "", "width: 75px;");
						else
							$r->appendTableRowInput("", "joint{$n}_membershipNumber", "view", "", "width: 75px;");
					}
					else {
						$f->appendNode("input", "type='hidden' name='joint{$n}_membershipNumber' value=''");
					}
					$r->appendTableRowInput("", "joint{$n}_dateOfBirth", "date", "0000-01-01", "", "", "calc();");
					$r->appendTableRowInput("", "joint{$n}_family", "checkbox", 0, "", "", "calc();");
					if($editing)
						$r->appendTableRowInput("", "joint{$n}_delete", "checkbox", 0, "", "", "calc();");
					else
						$f->appendNode("input", "type='hidden' name='joint{$n}_delete' id='joint{$n}_delete' value='0'");
				}
			}
		}
		
		$f->appendNode("h3", "", "Contact");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		$t->appendTableInput("Postal Address (first lines)", "address", "textarea", $obj->address, "height:50px; width:400px"); 
		$t->appendTableInput("City or Town", "city", "text", $obj->city, "width:300px;"); 
		$t->appendTableInput("Postcode", "postcode", "text", $obj->postcode, "width:300px;"); 
		$t->appendTableInput("Country", "country", "select", $obj->country, "", $this->countryList(), "calc();"); 
		$t->appendTableInput("Email Address", "emailAddress", "text", $obj->emailAddress, "width:300px;"); 
		$t->appendTableInput("Home Telephone (with code)", "telephoneHome", "text", $obj->telephoneHome, "width:300px;"); 
		$t->appendTableInput("Mobile Telephone (with code)", "telephoneMobile", "text", $obj->telephoneMobile, "width:300px;"); 
		if($obj->fullId == 0){
			$t->appendTableInput("Journal sent by email", "journalByEmail", "checkbox", $obj->journalByEmail, "", "", "calc();");
			$t->appendTableInput("Journal on a audio CD", "audioCd", "checkbox", $obj->audioCd, "", "", "calc();");
			$t->appendTableInput("Journal sent by post", "journalByPost", "checkbox", $obj->journalByPost, "", "", "calc();");
			$t->appendTableInput("Information by email (club info/renewals only)", "emailSend", "checkbox", $obj->emailSend, "", "", "calc();");
		}
		$t->appendTableInput("Notes", "notes", "textarea", $obj->notes, "height:100px; width:400px;"); 

		$f->appendNode("h3", "", "Membership");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");

		if($obj->fullId == 0){
			if($this->checkAccess() == 2)
				$t->appendTableInput("TC Membership Type", "membershipType", "select", $obj->membershipType, "", $this->membershipTypesAdmin, "calc(); membershipTypeChange();"); 
			else
				$t->appendTableInput("TC Membership Type", "membershipType", "select", $obj->membershipType, "", $this->membershipTypes, "calc();"); 
		}

		if($obj->fullId == 0){
			if($this->checkAccess() == 2)
				$t->appendTableInput("Payment Type", "paymentType", "select", $obj->paymentType, "", array("card", "bacs", "cheque", "standingOrder", "complimentary"), "calc();"); 
			else
				$t->appendTableInput("Payment Type", "paymentType", "select", $obj->paymentType, "", array("card", "bacs", "cheque", "standingOrder"), "calc();"); 
			$t->appendTableInput("Payment Currency", "paymentCurrency", "select", $obj->paymentCurrency, "", array("Pounds", "Euros"), "calc();"); 
			$t->appendTableInput("Payment Amount", "paymentAmount", "text", $obj->paymentAmount);
			if($renewForm){
				$f->appendNode("input", "type='hidden' name='attractedFrom' value='{$obj->v}'");
			}
			else {
				$t->appendTableInput("Attracted From", "attractedFrom", "selectCombobox", $obj->attractedFrom, "", array("", "Was a TC Member", "TC Member", "Cyclist", "Cycling Magazine", "TC Promotion", "Internet Search", "Other")); 
			}
		}
		
		if($admin && $editing){
			$f->appendNode("h3", "", "Admin");
			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$t->appendTableInput("Pending", "pending", "checkbox", $obj->pending, "width:300px;"); 
			if($obj->fullId == 0){
				$t->appendTableInput("Status", "status", "select", $obj->status, "", $this->statusTypes); 
				$t->appendTableInput("JoinedDate", "joinedDate", "view", $obj->joinedDate, "width:300px;"); 
				$f->appendNode("input", "type='hidden' name='joinedDate' value='{$obj->joinedDate}'");
				$t->appendTableInput("Group", "group", "select", $obj->group, "", array_merge(array(""), $this->groupsList())); 
			}
			$t->appendTableInput("NotesAdmin", "notesAdmin", "textarea", $obj->notesAdmin, "height:100px; width:400px;"); 
			if($obj->fullId == 0){
				$t->appendTableInput("PaymentId", "paymentId", "view", $obj->paymentId, "width:300px;"); 
				$f->appendNode("input", "type='hidden' name='paymentId' value='{$obj->paymentId}'");
				$t->appendTableInput("PaymentDate", "paymentDate", "view", $obj->paymentDate, "width:300px;"); 
				$f->appendNode("input", "type='hidden' name='paymentDate' value='{$obj->paymentDate}'");
				$t->appendTableInput("PaymentAmountPaid", "paymentAmountPaid", "view", $obj->paymentAmountPaid, "width:300px;"); 
				$f->appendNode("input", "type='hidden' name='paymentAmountPaid' value='{$obj->paymentAmountPaid}'");
				$t->appendTableInput("RenewalDate (YYYY-MM-DD)", "renewalDate", "text", $obj->renewalDate, "width:300px;"); 
				$t->appendTableInput("RenewalSent", "renewalSent", "text", $obj->renewalSent, "width:300px;"); 
				$t->appendTableInput("Why Resigned", "whyResigned", "text", $obj->whyResigned, "width:300px;");
				$t->appendTableInput("Website userId", "userId", "text", $obj->userId, "width:300px;");
			}
			$t->appendTableInput("ModifiedDate", "modifiedDate", "view", $obj->modifiedDate, "width:300px;"); 
			$t->appendTableInput("ModifiedUserName", "modifiedUserName", "view", $obj->modifiedUserName, "width:300px;"); 
			$t->appendTableInput("I have read the data privacy note above and I consent to it.", "gdprAcceptance", "checkbox", $obj->gdprAcceptance); 
		}
		else {
			$f->appendNode("input", "type='hidden' name='pending' value='{$obj->pending}'");
			$f->appendNode("input", "type='hidden' name='status' value='{$obj->status}'");
			$f->appendNode("input", "type='hidden' name='joinedDate' value='{$obj->joinedDate}'");
			$f->appendNode("input", "type='hidden' name='group' value='{$obj->group}'");
			$f->appendNode("input", "type='hidden' name='notesAdmin' value='{$obj->notesAdmin}'");
			$f->appendNode("input", "type='hidden' name='paymentId' value='{$obj->paymentId}'");
			$f->appendNode("input", "type='hidden' name='paymentDate' value='{$obj->paymentDate}'");
			$f->appendNode("input", "type='hidden' name='paymentAmountPaid' value='{$obj->paymentAmountPaid}'");
			$t->appendTableInput("RenewalDate (YYYY-MM-DD)", "renewalDate", "view", $obj->renewalDate, "width:300px;"); 
			$f->appendNode("input", "type='hidden' name='renewalSent' value='{$obj->renewalSent}'");
			$t->appendTableInput("Why Resigned", "whyResigned", "text", $obj->whyResigned, "width:300px;");
			$f->appendNode("input", "type='hidden' name='userId' value='{$obj->userId}'");
			$f->appendNode("input", "type='hidden' name='modifiedDate' value='{$obj->modifiedDate}'");
			$f->appendNode("input", "type='hidden' name='modifiedUserName' value='{$obj->modifiedUserName}'");
			$t->appendTableInput("Add website account", "addAccount", "checkbox", $obj->addAccount); 
			$t->appendTableInput("I have read the data privacy note above and I consent to it.", "gdprAcceptance", "checkbox", $obj->gdprAcceptance); 
		}

		$f->appendNode("input", "type='hidden' name='renewPassword' value='{$obj->renewPassword}'");
		$f->appendNode("input", "type='hidden' name='updatePassword' value='{$obj->updatePassword}'");
		if($admin){
			$f->appendNode("input", "type='hidden' name='admin' value='1'");
		}
		else {
			$f->appendNode("input", "type='hidden' name='admin' value='0'");
		}
		$f->appendNode("input", "type='hidden' name='editing' value='{$editing}'");

		if($editing){
			if($admin){
				$t = &$f->appendNode("table");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("input", "type='submit' name='save' style='background-color: lightgreen; font-size: 200%;' value='Save Changes' onclick='this.value=\"Saving, please wait...\";'");
				$r->appendNode("td")->appendText("Saves any changes made to the record to the database.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("input", "type='submit' name='sendRenewEmail' style='background-color: lightgreen; font-size: 200%;' value='Send Renew Email' title='Send an email to member with link allowing them to renew and pay' onclick='this.value=\"Sending email, please wait...\";'");
				$r->appendNode("td")->appendText("Send an email to member with link allowing them to view/edit their details, renew (updates renew date etc.) and pay.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("input", "type='submit' name='renewFormProcess' style='background-color: lightgreen; font-size: 200%;' value='RenewForm Process' title='If a renewForm, finds the matching real member form and updates it with the info in the renewForm and deletes the renewForm' onclick='this.value=\"Saving, please wait...\";'");
				$r->appendNode("td")->appendText("If a renewForm, finds the matching real member form (forename,surname,postcode) and updates it with the info in the renewForm and deletes the renewForm record.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("input", "type='submit' name='renew' style='background-color: lightgreen; font-size: 200%;' value='Renew' title='Set member as renewDone with renew date updated and take payment' onclick='this.value=\"Saving, please wait...\";'");
				$r->appendNode("td")->appendText("Set member as renewDone with renew date updated and add a payment record. If paying by credit card take payment over the phone.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("input", "type='submit' name='leave' style='background-color: lightgreen; font-size: 200%;' value='Leave the club'  title='Member leaving the club' onclick='this.value=\"Saving, please wait...\";'");
				$r->appendNode("td")->appendText("Member is leaving the club. Mark record as leaving. The record will need deleting after this.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("input", "type='submit' name='complete' style='background-color: lightgreen; font-size: 200%;' value='Complete new/renew' title='Complete this record by marking as paid (if needed), setting the status to member and clearing the pending flag' onclick='this.value=\"Saving, please wait...\";'");
				$r->appendNode("td")->appendText("Complete this record by marking as paid (if needed), setting the status to member and clearing the pending flag. If renewing make sure RenewalDate is correct. This is normally updated automatically when member submits renewal form or the admin persion has used the Renew function above.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("button", "type='button' name='addWebAccount' style='background-color: lightgreen; font-size: 200%;' title='Add a website login account for this member' onclick='location.href = \"/membership?cmd=addWebAccount&amp;id={$obj->id}\";'", "Add Website Account");
				$r->appendNode("td")->appendText("Add a website login account for this member. They will be sent an email with login information and a temporary password.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("button", "type='button' name='rawEdit' style='background-color: lightgreen; font-size: 200%;' title='Edit raw record entry' onclick='location.href = \"/membership?cmd=editRaw&id={$obj->id}\";'", "Edit  the raw record");
				$r->appendNode("td")->appendText("Edit the raw membership record. For special purposes only, dangerous to use.");
				$r = &$t->appendNode("tr");
				$r->appendNode("td")->appendNode("button", "type='button' name='arg' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"/membership\";'", "Cancel");
				$r->appendNode("td")->appendText("Cancel edits and go back to list.");
			}
			else {
				$f->appendNode("input", "type='submit' name='renew' style='background-color: lightgreen; font-size: 200%;' value='Renew' onclick='this.value=\"Saving, please wait...\";'");
				$f->appendNode("input", "type='submit' name='leave' style='background-color: lightgreen; font-size: 200%;' value='Leave the club'  title='Member leaving the club' onclick='this.value=\"Saving, please wait...\";'");
				$f->appendNode("button", "type='button' name='arg' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"/membership\";'", "Cancel");
			}
		}
		else {
			if($renewForm){
				$f->appendNode("input", "type='submit' name='renew' style='background-color: lightgreen; font-size: 200%;' value='Renew' onclick='this.value=\"Saving, please wait...\";'");
				$f->appendNode("input", "type='submit' name='leave' style='background-color: lightgreen; font-size: 200%;' value='Leave the club' title='Member leaving the club' onclick='this.value=\"Saving, please wait...\";'");
			}
			else {
				$f->appendNode("input", "type='submit' name='save' style='background-color: lightgreen; font-size: 200%;' value='Join' onclick='this.value=\"Saving, please wait...\";'");
			}
			$f->appendNode("button", "type='button' name='arg' style='background-color: lightgreen; font-size: 200%;' onclick='location.href = \"/\";'", "Cancel");
		}

		$page = $page . $f->render();
		
		$javascript = <<< End_Of_Text
function calculateAge(birthDate){
	birthDate = new Date(birthDate);
	now = new Date();

	var years = (now.getFullYear() - birthDate.getFullYear());

	if(now.getMonth() < birthDate.getMonth() || 
	        now.getMonth() == birthDate.getMonth() && now.getDate() < birthDate.getDate()){
	        years--;
	}

	return years;
}

function membershipTypeChange(){
//	alert("membershipType();");
	type = document.getElementById('membershipType');
	paymentType = document.getElementById('paymentType');
	renewalDate = document.getElementById('renewalDate');
	if(type.value == "Complimentary"){
		t = new Date();
//		t.setDate(t.getDate() + 6 * 31);
		t.setFullYear(t.getFullYear() + 1);
		paymentType.value = "complimentary";
		renewalDate.value = t.toISOString().substring(0, 10);
	}
}

function calc(){
//	alert("calc();");
	type = document.getElementById('membershipType');
	currency = document.getElementById('paymentCurrency');
	country = document.getElementById('country');
	audioCd = document.getElementById('audioCd');
	pay = document.getElementById('paymentAmount');
	jointNumber = 0;
	for(j = 1; j < 6; j++){
		age = calculateAge(document.getElementById('joint' + j + '_dateOfBirth').value);
		str = "Age: " + document.getElementById('joint' + j + '_dateOfBirth').value + ", " + String(age);
//		alert(str);
		if((document.getElementById('joint' + j + '_surname').value != "") && (document.getElementById('joint' + j + '_delete').value != 1) && ((document.getElementById('joint' + j + '_family').value != 1) || (age >= 18))){
			jointNumber++;
		}
	}
	
	if((document.getElementById('full_delete').value == 1) && (jointNumber > 0)){
		jointNumber--;
	}
	
	if(currency.value == "Pounds"){
		if(type.value == "1 year"){
			pay.value = "10.00";
			pay.value = (parseFloat(pay.value) + jointNumber * 0.50).toFixed(2);
		}
		else if(type.value == "5 year"){
			pay.value = "40.00";
			pay.value = (parseFloat(pay.value) + jointNumber * 2.00).toFixed(2);
		}
		else if(type.value == "Special"){
		}
		else{
			pay.value = "0.00";
		}
		
		if(country.value != "United Kingdom"){
			if(audioCd.value == "1")
				pay.value = (parseFloat(pay.value) + 15.00).toFixed(2);
		}
	}
	else {
		if(type.value == "1 year"){
			pay.value = "18.00";
			pay.value = (parseFloat(pay.value) + jointNumber * 0.90).toFixed(2);
		}
		else if(type.value == "5 year"){
			pay.value = "72.00";
			pay.value = (parseFloat(pay.value) + jointNumber * 3.60).toFixed(2);
		}
		else if(type.value == "Special"){
		}
		else{
			pay.value = "0.00";
		}

		if(country.value != "United Kingdom"){
			if(audioCd.value == "1")
				pay.value = (parseFloat(pay.value) + 20.00).toFixed(2);
		}
	}
}
calc();
End_Of_Text;

		$page .= "\n<script language='JavaScript'>{$javascript}</script>";
		
		return $page;
	}
	
	function doPayment($obj){
		if($this->test == 3){
			return "<b>*** Warning: Payments disabled as in test mode 3 ***</b>";
		}
		$page = "";
		$paymentModule = $this->sys->modules["payments"];
		$paymentModule->test = $this->test;
		
		$payment = $paymentModule->payGetNew();
		$payment->type = $obj->paymentType;
		$payment->source = "membership";
		$payment->reference = $obj->id;
		$payment->name = $obj->forename . " " . $obj->surname;
		$payment->address = $obj->address;
		$payment->city = $obj->city;
		$payment->postcode = $obj->postcode;
		$payment->country = $obj->country;
		$payment->telephone = $obj->telephoneHome;
		$payment->emailAddress = $obj->emailAddress;
		$payment->amount = $obj->paymentAmount;
		$payment->currency = $obj->paymentCurrency;
		$payment->title = "Membership";
		$payment->test = $this->test;

		if($obj->paymentType == "card"){
			$payment->type = "card_membership";
			$page .= $this->sys->modules["payments"]->payDo($payment);
		}
		else{
			if($obj->paymentType == "cheque"){
				if($payment->currency == "Pounds"){
					$payment->type = "cheque";
				}
				else {
					$payment->type = "cheque_euro";
				}
			}
			else if($obj->paymentType == "bacs"){
				if($payment->currency == "Pounds"){
					$payment->type = "bacs_pounds";
				}
				else {
					$payment->type = "bacs_euro";
				}
			}
			else{
				$payment->type = $obj->paymentType;
			}
			
			if($obj->status == "new")
				$chequeAddress = $this->chequeAddressJoin;
			else
				$chequeAddress = $this->chequeAddressRenew;
			
			$n = new BHtmlNode("div");
			$n->appendNode("h3", "", "Please pay using: {$obj->paymentType}. Please use the reference: 'mem {$obj->membershipNumber}'");
			$r = &$n->appendNode("ol");
			$r->appendNode("li", "", "For people in the UK you can pay in Pounds using direct bank transfer (BACS) to the account: Bank: Lloyds Bank, AccountName: The Tandem Club, Sortcode: 30-91-36, AccountNumber: 00856202, Reference: 'mem {$obj->membershipNumber}'");
			$r->appendNode("li", "", "For people in the UK you can pay in Pounds using Standing Order (1 years subscription only) to the account: Bank: Lloyds Bank, AccountName: The Tandem Club, Sortcode: 30-91-36, AccountNumber: 00856202, Reference: 'mem {$obj->membershipNumber}'.");
			$r->appendNode("li", "", "For people in the UK you can post a cheque in Pounds for total amount, with your membership number '{$obj->membershipNumber}' on the reverse, payable to: \"The Tandem Club\" to: {$chequeAddress}.");
			$r->appendNode("li", "", "For people in the Eurozone pay please pay in Euros by direct bank transfer to the following bank account: 'The Tandem Club - Euro Account' (IBAN) GB05 LOYD 3091 3686 1927 70, (BIC) LOYDGB21178. Please use the reference 'mem {$obj->membershipNumber}'.");
			$r->appendNode("li", "", "For people in the rest of the world pay the equivalent to pounds to the following banks account: Bank Lloyds Bank Bromsgrove Account The Tandem Club (IBAN) GB72 LOYD 3091 3600 8562 02 (BIC) LOYDGB21178. Please use the reference 'mem {$obj->membershipNumber}'.");
			$r->appendNode("li", "", "For people outside the UK you can post a cheque in Pounds or Euros for total amount, with your membership number '{$obj->membershipNumber}' on the reverse, payable to: \"The Tandem Club\" to: {$chequeAddress}. Note there is a 6 GBP (equivalent of) charge for any cheques not written in GBP or Euros.");

			if($obj->paymentType == "standingOrder"){
				$n->appendNode("p", "", "If you change your membership by adding or removing members please would you take a moment to check that any Standing Order is still correct");
				$n->appendNode("p", "", "You can either setup your Standing Order using Internet banking or you can fill in and post this slip to your bank.");
				$n->appendNode("hr");
				$n->appendNode("h3", "", "Standing Order Request - Please send to your bank");
				$n->appendText("Your full name: ........................................... Your Phone Number: .................................<br />\n");
				$n->appendText("Your banks name: ...........................................<br />Sort code: ........................ Account Number: .....................................<br />\n");
				$n->appendText("Recipient: The Tandem Club,Bank: Lloyds Bank, Sort code: 30-91-36, Account Number: 00856202<br />\n");
				$n->appendText("Payment amount: &pound;10.00, Paid Yearly, First payment: {$obj->renewalDate}<br />\n");
				$n->appendText("Payment reference: mem {$obj->membershipNumber}, Payments to be made until further notice<br />\n");
				$n->appendNode("p", "", "Your signature: ........................................... Date: .................................");
			}

			$page .= $n->render();
			
			$this->sys->modules["payments"]->payDo($payment);
			
			if($obj->emailAddress != ""){
				$subject = "{$this->sys->siteTitle}: Membership payment";
				$message = "Your membership has been applied/re-applied for.";
				$message .= " Your membership number is: {$obj->membershipNumber}, please use in all communications with the club.";
				$message .= $n->render();
				emailList($obj->emailAddress, $subject, $message, 1);
			}

		}
		
		// Set paymentId
		$obj->paymentId = $payment->id;
		$this->recordSave("Membership", $obj);

		return $page;
	}

	function payComplete($pay){
//		print "Membership: payComplete {$pay->reference}<br>\n";
		$id = $pay->reference;

		if($id){
			$obj = $this->sys->db->getObject("Membership", $id);
			if($obj){
				$obj->paymentId = $pay->id;
				$obj->paymentDate = $pay->datePaid;
				$obj->paymentAmountPaid = $pay->amount;
				$this->sys->db->putObject("Membership", $obj);
				return "Membership payment updated";
			}
		}
		return "Membership payment: Error reference incorrect: {$pay->reference}";
	}

	function save($uri){
		$page = "";

//		print "save</br>";
//		BDatabase::$debug = 1;
//		print "Form: " . print_r($_POST, 1) . "<br>\n";
//		debug_log("Membership.save: POST: " . print_r($_SERVER, 1) . print_r($_POST, 1));

		if(count($_POST)){
			$date = new BDate("now");

			$admin = isset($_POST["admin"]) && $_POST["admin"];
			$renew = isset($_POST["renew"]) && $_POST["renew"];
			$renewFormProcess = isset($_POST["renewFormProcess"]) && $_POST["renewFormProcess"];
			$leave = isset($_POST["leave"]) && $_POST["leave"];
			$editing = isset($_POST["editing"]) && $_POST["editing"];
			$sendRenewEmail = isset($_POST["sendRenewEmail"]) && $_POST["sendRenewEmail"];
			$complete = isset($_POST["complete"]) && $_POST["complete"];
			
//			print "Save settings: admin: $admin renew: $renew leave: $leave editing: $editing: sendRenewEmail: $sendRenewEmail</br>";
			
			unset($_POST['cmd']);
			if($_POST['id']){
				$obj = $this->recordGet("Membership", filterToInt($_POST['id']), 1);
				for($n = count($obj->jointList); $n < $this->numJoint; $n++){
					$obj->jointList[]  = new BObjMembership();
				}
			}
			else {
				$obj = new BObjMembership();
				$obj->jointList = array();
				for($n = 0; $n < $this->numJoint; $n++){
					$obj->jointList[]  = new BObjMembership();
				}

			}

			// Setup object
			$obj->set($_POST);
//			membership_debug_obj("Membership.save1: Obj:", $obj);

			// Modify a few things
			$obj->forename = trim($obj->forename);
			$obj->surname = trim($obj->surname);

			// Fixup joint members
			for($n = 0; $n < $this->numJoint; $n++){
				if(isset($_POST["joint{$n}_surname"])){
					$obj->jointList[$n]->prefix = filterString($_POST["joint{$n}_prefix"]);
					$obj->jointList[$n]->forename = filterString(trim($_POST["joint{$n}_forename"]));
					$obj->jointList[$n]->surname = filterString(trim($_POST["joint{$n}_surname"]));
					$obj->jointList[$n]->membershipNumber = filterString(trim($_POST["joint{$n}_membershipNumber"]));
					$obj->jointList[$n]->dateOfBirth = filterStringDate($_POST["joint{$n}_dateOfBirth"]);
					$obj->jointList[$n]->family = filterInt($_POST["joint{$n}_family"]);
				}
			}
//			membership_debug_obj("Membership.save2: Obj:", $obj);
			
			// Work around a bug in a version of the Google Chrome web browser
			if(($obj->jointList[5]->forename == $obj->forename) && ($obj->jointList[5]->surname == $obj->surname)){
				$obj->jointList[5]->forename = "";
				$obj->jointList[5]->surname = "";
				membership_debug_obj("Membership.save2a: Duplicate Joint5 name removed:", $obj);
				debug_log("Membership.save: POST: " . print_r($_SERVER, 1) . print_r($_POST, 1));
				emailList("terry@beam.ltd.uk", "TC: same forename1: POST: ", print_r($_SERVER, 1) . print_r($_POST, 1));
			}
			
			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			
			if(($obj->forename == "") || ($obj->surname == "")){
				$this->sys->status = "<span class='editError'>Error in form data. You need to enter your forename and surname.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			
			if((trim($obj->address) == "") && (trim($obj->emailAddress) == "")){
				$this->sys->status = "<span class='editError'>Error in form data. You need to enter a postal address or email address for contact.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			
			if(($obj->paymentType == "standingOrder") && ($obj->membershipType != "1 year")){
				$this->sys->status = "<span class='editError'>Error Standing Orders can only be used for 1 year subscriptions. Please change membership type or payment type.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			
			if(($obj->paymentType == "card") && ($obj->paymentCurrency != "Pounds")){
				$this->sys->status = "<span class='editError'>Error: Sorry we can only process cards in GBP Pounds. Please change payment currency or payment type.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}

			if(0 && ($obj->paymentType == "card") && ($obj->country != "United Kingdom")){
				$this->sys->status = "<span class='editError'>Error: Sorry we can only process cards in GBP Pounds from UK cards. Please change payment currency or payment type.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			if(!$admin && ($obj->paymentType == "card") && ($obj->paymentAmount > 20.0)){
				$this->sys->status = "<span class='editError'>Error: Sorry our card provider only allows payments up to &pound;20.00. Please change payment type or pay for one year.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			if(!$admin && !$obj->gdprAcceptance){
				$this->sys->status = "<span class='editError'>Error: Sorry you need to accept our data protection terms.</span>";
				return $this->edit($obj, $admin, ($obj->status == "renewForm"));
			}
			
			
			// Just send a renew email
			if($sendRenewEmail){
				$this->renewalSendEmail($obj);
				return "Renewal email sent";
			}

			if($renewFormProcess){
				return $this->renewFormProcess($obj);
			}
			
			if($renew){
				// See if we can find an existing record for this member that has filled in a complete renewal form
			}
			
			if($complete){
				// Mark as paid
				if($obj->paymentAmountPaid == 0){
					if($obj->paymentId){
						$this->sys->modules["payments"]->payDone($obj->paymentId);
					}
					$obj->paymentDate = strftime("%Y-%m-%d");
					$obj->paymentAmountPaid = $obj->paymentAmount;
				}
				$obj->status = "member";
				$obj->pending = 0;
			}
			
			if($obj->id){
				$objOld = $this->recordGet("Membership", $obj->id, 1);
				if(!$this->sys->login->checkLogin() && ($objOld->updatePassword != $obj->updatePassword)){
					$this->sys->pageDisplayError(new BError(1, "You need to login using the login form on the left to update Membership info."));
					return "";
				}
				if($objOld->updatePassword != $obj->updatePassword){
					$this->sys->pageDisplayError(new BError(1, "Update validation failed."));
					return "";
				}

				$objOld->id = 0;
				$objOld->pending = 0;
				foreach($objOld->jointList as &$joint)
					$joint->id = 0;
				$this->recordSave("MembershipOld", $objOld, 1);
				$obj->renewPassword = "";
				$obj->updatePassword = "";
			}
			
			// Begin database transaction			
			$this->sys->db->transactionStart();

//			membership_debug_obj("Membership.save3: Obj:", $obj);
			// Delete from the database and object any joint members that have been removed
			for($n = 0, $i = 0; $n < $this->numJoint; $n++){
				if(($_POST["joint{$n}_delete"] == 1) || ($_POST["joint{$n}_surname"] == "")){
					if($i < count($obj->jointList)){
						if($obj->jointList[$i]->id)
							$this->sys->db->deleteObject("Membership", $obj->jointList[$i]->id);
						unset($obj->jointList[$i]);
						$obj->jointList= array_values($obj->jointList);
					}
				}
				else {
					$i++;
				}
			}
			
			// Delete full member if needed
			if($_POST["full_delete"] == 1){
				if(count($obj->jointList) == 0){
					$this->sys->db->transactionEnd(1);
					$this->sys->pageDisplayError(new BError(1, "There are no joint members to set as the new full member."));
					return "";
				}
				
				$obj->prefix = filterString($_POST["joint0_prefix"]);
				$obj->forename = filterString($_POST["joint0_forename"]);
				$obj->surname = filterString($_POST["joint0_surname"]);
				$obj->dateOfBirth = filterStringDate($_POST["joint0_dateOfBirth"]);
				$obj->membershipNumber = filterString($_POST["joint0_membershipNumber"]);

				if($obj->jointList[0]->emailAddress != "")
					$obj->emailAddress = $obj->jointList[0]->emailAddress;
				if($obj->jointList[0]->telephoneHome != "")
					$obj->telephoneHome = $obj->jointList[0]->telephoneHome;
				if($obj->jointList[0]->telephoneMobile != "")
					$obj->telephoneMobile = $obj->jointList[0]->telephoneMobile;
				if($obj->jointList[0]->address != "")
					$obj->address = $obj->jointList[0]->address;
				if($obj->jointList[0]->city != "")
					$obj->city = $obj->jointList[0]->city;
				if($obj->jointList[0]->postcode != "")
					$obj->postcode = $obj->jointList[0]->postcode;

				if($obj->jointList[0]->id)
					$this->sys->db->deleteObject("Membership", $obj->jointList[0]->id);
				array_shift($obj->jointList);
			}

			foreach($obj->jointList as $joint){
				if($joint->forename == $obj->forename){
					$message = "Same forename for {$obj->forename} {$obj->surname}";
					$message .= print_r($_SERVER, 1);
					$message .= print_r($_POST, 1);
					emailList("terry@beam.ltd.uk", "TC: same forename2", $message);
					break;
				}
			}
			
			// Allocate new membership numbers
			if(($obj->id == 0) && ($obj->membershipNumber == "")){
				$obj->membershipNumber = "F" . $this->newMembershipNumber();
			}

			foreach($obj->jointList as &$joint){
				if(($joint->id == 0) && ($joint->membershipNumber == "")){
					$joint->membershipNumber = "J" . $this->newMembershipNumber();
				}
			}

			$date = new BDate("now");
			$obj->modifiedDate = $date->get();
			if($this->sys->user)
				$obj->modifiedUserName = $this->sys->user->userName;

			if($obj->id){
				if(!$admin)
					$obj->pending = 1;
					
				if($leave){
					$obj->status = "leave";
					$obj->paymentType = "";
					$obj->paymentAmount = 0;
					$renew = 0;
				}
				if($renew && ($obj->status != "renewDone")){
					$dateRenewal = new BDate("now");
					$dateRenewal->set($obj->renewalDate);
					if($obj->membershipType == "5 year")
						$dateRenewal->set(array($dateRenewal->getYear() + 5, $dateRenewal->getMonth(), $dateRenewal->getDay()));
					else
						$dateRenewal->set(array($dateRenewal->getYear() + 1, $dateRenewal->getMonth(), $dateRenewal->getDay()));

					$obj->renewalDate = $dateRenewal->get();
					$obj->renewalSent = 0;
					$obj->status = "renewDone";
				}
				
				$this->recordSave("Membership", $obj, 1);

				// Send emails to member
				if($leave && ($obj->emailAddress != "")){
					$subject = "{$this->sys->siteTitle}: Membership";
					$message = "Your membership has been discontinued.";
					$message .= " Sorry to see you go.";
					if($this->testEmails)
						emailList($this->sys->user->emailAddress, $subject, $message);
					else
						emailList($obj->emailAddress, $subject, $message);
				}
				if($renew && ($obj->emailAddress != "")){
					$subject = "{$this->sys->siteTitle}: Membership";
					$message = "Your membership has been renewed.";
					if($obj->membershipNumber != "")
						$message .= " Your membership number is: {$obj->membershipNumber}, please use in all communications with the club.";
					$message .= " Many Thanks.";
					if($this->testEmails)
						emailList($this->sys->user->emailAddress, $subject, $message);
					else
						emailList($obj->emailAddress, $subject, $message);
				}
				
				// Send emails to admin
				if($leave || $renew){
					if($leave){
						$subject = "{$this->sys->siteTitle}: Admin: Member leaving: Number: {$obj->membershipNumber}";
						$message = "{$this->sys->siteTitle}: Admin: Member leaving: Number: {$obj->membershipNumber}\n\n";
						$this->statisticsAdd("leave", 1);
					}
					else {
						$subject = "{$this->sys->siteTitle}: Admin: Member renewal: Number: {$obj->membershipNumber}";
						$message = "{$this->sys->siteTitle}: Admin: Member renewal: Number: {$obj->membershipNumber}\n\n";
						$this->statisticsAdd("renew", 1);
					}
					$message .= "Name:        {$obj->forename} {$obj->surname}\n";
					$message .= "Email:       {$obj->emailAddress}\n";
					$message .= "Postcode:    {$obj->postcode}\n";
					$message .= "PaymentType: {$obj->paymentType}\n\n";
					$message .= "View: {$this->sys->baseUrl}/membership?cmd=view&id={$obj->id}\n";
					$message .= "Edit: {$this->sys->baseUrl}/membership?cmd=edit&id={$obj->id}\n";

					if($this->testEmails){
						emailList($this->sys->user->emailAddress, $subject, $message);
					}
					else {
						emailList(configGet("membership.adminEmailList"), $subject, $message);
						
						if($leave){
							// Send to group officer
							$groupEmail = groupContactEmail($obj->group);
							if($groupEmail != "")
								emailList($groupEmail, $subject, $message);
						}
					}
				}

				$page = "<h3>Membership details saved</h3>";
				if(!$admin){
					if($leave){
						$page .= "<p>Your membership has been discontinued. Sorry to see you go..</p>";
					}
					else {
						$page .= "<p>Your membership has been renewed. Many thanks.</p>";
					}
				}
				else if($renew){
					$page .= "<p>Member renewed, payments need to be made.</p>";
				}
			}
			else {
				$dateRenewal = new BDate("now");
				if($obj->membershipType == "5 year"){
					$dateRenewal->set(array($dateRenewal->getYear() + 5, $dateRenewal->getMonth(), $dateRenewal->getDay()));
				}
				else if($obj->membershipType == "1 year"){
					$dateRenewal->set(array($dateRenewal->getYear() + 1, $dateRenewal->getMonth(), $dateRenewal->getDay()));
				}
				else if($obj->membershipType == "Complimentary"){
//					$dateRenewal->addDays(6 * 31);
					$dateRenewal->set(array($dateRenewal->getYear() + 1, $dateRenewal->getMonth(), $dateRenewal->getDay()));
				}
				else{
					$dateRenewal->set(array($dateRenewal->getYear() + 1, $dateRenewal->getMonth(), $dateRenewal->getDay()));
				}
				
				$obj->pending = 1;
				if(!$renew)
					$obj->status = "new";
				
				$obj->joinedDate = $date->get();
				$obj->renewalDate = $dateRenewal->get();
				$obj->renewalSent = 0;

				// Find regional group
				if($obj->group == "")
					$obj->group = "Unassigned";
				if(($obj->postcode != "") && ($module = $this->sys->modules["groups"])){
					if($group = $module->getGroupForPostcode($obj->postcode))
						$obj->group = $group;
				}

				$s1 = $obj->joinedDate;
				$s2 = $obj->renewalDate;

				$this->recordSave("Membership", $obj, 1);

				// Send emails
				if($obj->emailAddress != ""){
					if($renew){
						$subject = "{$this->sys->siteTitle}: Renew membership";
						$message = "Your renewal of membership has been applied for.";
					}
					else {
						$subject = "{$this->sys->siteTitle}: New membership";
						$message = "Your new membership has been applied for.";
						$message .= " Your membership number is: {$obj->membershipNumber}, please use in all communications with the club.";
						$message .= " You will be contacted by one of the club officials with further information shortly.";
					}
					if($this->testEmails)
						emailList($this->sys->user->emailAddress, $subject, $message);
					else
						emailList($obj->emailAddress, $subject, $message);
				}

				if($renew){
					$subject = "{$this->sys->siteTitle}: Admin: Renew Membership by form";
					$message = "{$this->sys->siteTitle}: Admin: Renew Membership by form\n\n";
					$this->statisticsAdd("renewForm", 1);
				}
				else {
					$subject = "{$this->sys->siteTitle}: Admin: New Membership application: Number: {$obj->membershipNumber}";
					$message = "{$this->sys->siteTitle}: Admin: New Membership application: Number: {$obj->membershipNumber}\n\n";
					$this->statisticsAdd("new", 1);
				}
				$message .= "Name:        {$obj->forename} {$obj->surname}\n";
				$message .= "Email:       {$obj->emailAddress}\n";
				$message .= "Postcode:    {$obj->postcode}\n";
				$message .= "PaymentType: {$obj->paymentType}\n\n";
				$message .= "View: {$this->sys->baseUrl}/membership?cmd=view&id={$obj->id}\n";
				$message .= "Edit: {$this->sys->baseUrl}/membership?cmd=edit&id={$obj->id}\n";

				if($this->testEmails){
					emailList($this->sys->user->emailAddress, $subject, $message);
					$groupEmail = groupContactEmail($obj->group);
//					print "Would send email to RO: {$groupEmail}</br>";
				}
				else {
					emailList(configGet("membership.adminEmailList"), $subject, $message);
					if(!$renew){
						emailList(configGet("membership.newMemberEmailList"), $subject, $message);

						// Send to group officer
						$groupEmail = groupContactEmail($obj->group);
						if($groupEmail != "")
							emailList($groupEmail, $subject, $message);
					}
				}

				$page = "<h3>Membership details saved</h3>";
				if(!$admin){
					$page .= "<p>Your membership has been applied for. One of our club officers will contact you with an information pack shortly. Many Thanks.</p>";
				}
			}
			$this->sys->db->transactionEnd();

			// Add web account if wanted and not present
			if(isset($_POST["addAccount"]) && ($_POST["addAccount"] == "1") && !$leave){
				if(!$this->sys->modules["user"]->checkIfUserExists($obj->emailAddress)){
					$user = new BObjUser();
					$user->name = $obj->forename . " " . $obj->surname;
					$user->userName = $obj->emailAddress;
					$user->userPassword = "";
					$user->postalAddress = $obj->address;
					$user->city = $obj->city;
					$user->postcode = $obj->postcode;
					$user->country = $obj->country;
					$user->group = $obj->group;
					$user->emailSend = $obj->emailSend;
					$user->emailAddress = $obj->emailAddress;
					$user->telephoneHome = $obj->telephoneHome;
					$user->telephoneMobile = $obj->telephoneMobile;
					$user->membershipId = $obj->id;

					$this->sys->modules["user"]->addUser($user, 1);

					$obj->userId = $user->id;
					$this->recordSave("Membership", $obj);

					$page .= "<p>A website login account has been added. You will recieve an email with details.</p>";
				}
				else {
					$page .= "<p>You already have a website login account, so a new one has not been created.</p>";
				}
			}
			$this->webUserUpdate($obj);
			
			if((!$admin || $renew) && !$leave && ($obj->paymentAmount != 0)){
				$page .= $this->doPayment($obj);
			}
			
			$page .= "<button type='button' onclick='location.href = \"/membership\"'>Back to list</button>";

		}
		return $page;
	}

	function renewFormProcess($obj){
//		print "renewFormProcess</br>"; BDatabase::$debug = 1;
		
		if($obj->status != "renewForm"){
			$this->sys->status = "<span class='editError'>Error this is not a renewForm</span>";
			return $this->edit($obj, 1, ($obj->status == "renewForm"));
		}
		
		// Search for matching record
		$select = "`id` != '{$obj->id}' and `status` != 'renewForm' and `forename` = '{$obj->forename}' and `surname` = '{$obj->surname}' and `postcode` = '{$obj->postcode}'";
		$data = $this->sys->db->getObjectList("Membership", "", $select, "", "");

		if(count($data) > 1){
			$num = count($data);
			$this->sys->status = "<span class='editError'>More than one matching entry found ({$num})</span>";
			return $this->edit($obj, 1, ($obj->status == "renewForm"));
		}
		if(count($data) == 0){
			$this->sys->status = "<span class='editError'>No matching entries found. (forname,surname,postcode).</span>";
			return $this->edit($obj, 1, ($obj->status == "renewForm"));
		}
		

		// Copy over data
		$objOld = $this->recordGet("Membership", $data[0]->id, 1);
		
		// Check joints are the same
		foreach($obj->jointList as &$joint){
			// Find info in old joint
			if($joint->forename != ""){
				$found = 0;
				foreach($objOld->jointList as $jointOld){
					if((strtolower($jointOld->forename) == strtolower($joint->forename)) and (strtolower($jointOld->surname) == strtolower($joint->surname))){
						$found = 1;
						break;
					}
				}
				if($found == 0){
					print "Joints not the same</br>\n";
				}
			}
		}
		
		// Backup Old
		$objOld1 = $this->recordGet("Membership", $objOld->id, 1);
		$objOld1->id = 0;
		$objOld1->pending = 0;
		foreach($objOld1->jointList as &$joint)
			$joint->id = 0;
		$this->recordSave("MembershipOld", $objOld1, 1);
		
		// Delete old payment record if present and not paid
		if($objOld1->paymentId != 0){
			$paymentModule = $this->sys->modules["payments"];
			$pay = $this->sys->db->getObject("Payments", $objOld1->paymentId);
			if($pay->paid == 0)
				$this->sys->db->deleteObject("Payments", $objOld1->paymentId);
		}

		// Update current payment record if present with record's id
		if($obj->paymentId != 0){
			$paymentModule = $this->sys->modules["payments"];
			$pay = $this->sys->db->getObject("Payments", $obj->paymentId);
			$pay->reference = $objOld->id;
			$this->sys->db->putObject("Payments", $pay);
		}

		$delId = $obj->id;

		$obj->id = $objOld->id;
		$obj->status = "renewDone";
		$obj->membershipNumber = $objOld->membershipNumber;
		$obj->joinedDate = $objOld->joinedDate;
		$obj->group = $objOld->group;
		if($objOld->notes != "")
			$obj->notes = $objOld->notes . $obj->notes;

		// Renewal date process
		$dateRenewal = new BDate($objOld->renewalDate);
		if($objOld->status != "renewDone"){
			if($obj->membershipType == "5 year")
				$dateRenewal->set(array($dateRenewal->getYear() + 5, $dateRenewal->getMonth(), $dateRenewal->getDay()));
			else
				$dateRenewal->set(array($dateRenewal->getYear() + 1, $dateRenewal->getMonth(), $dateRenewal->getDay()));
		}
		$obj->renewalDate = $dateRenewal->get();
		$obj->renewalSent = 0;

		if($objOld->notesAdmin != "")
			$obj->notesAdmin = $objOld->notesAdmin . $obj->notesAdmin;
		if($objOld->attractedFrom != "")
			$obj->attractedFrom = $objOld->attractedFrom . $obj->attractedFrom;
		$date = new BDate("now");
		$obj->modifiedDate = $date->get();
		if($this->sys->user)
			$obj->modifiedUserName = $this->sys->user->userName;
		$obj->renewPassword = "";
		$obj->updatePassword = "";

		// Update joints and add new ones
		foreach($obj->jointList as &$joint){
			// Find info in old joint
			if($joint->forename != ""){
				$found = 0;
				foreach($objOld->jointList as $jointOld){
					if((strtolower($jointOld->forename) == strtolower($joint->forename)) and (strtolower($jointOld->surname) == strtolower($joint->surname))){
						$joint = $jointOld;
						// print "Set Joint {$joint->id} num {$joint->membershipNumber}</br>";
						$found = 1;
						break;
					}
				}
				if($found == 0){
					// print "Will add joint: {$joint->forename}</br>";
					$joint->membershipNumber = "J" . $this->newMembershipNumber();
				}
			}
		}
		
		// Delete old joints that are no longer present
		foreach($objOld->jointList as $jointOld){
			if($jointOld->forename != ""){
				$found = 0;
				foreach($obj->jointList as $joint){
					if((strtolower($jointOld->forename) == strtolower($joint->forename)) and (strtolower($jointOld->surname) == strtolower($joint->surname))){
						$found = 1;
						break;
					}
				}
				if($found == 0){
					// print "Will delete joint: {$jointOld->forename}</br>";
					$this->recordDelete("Membership", $jointOld->id, 0);
				}
			}
		}
		$this->recordSave("Membership", $obj, 1);
		
		// Backup renewForm
		$objOld = $this->recordGet("Membership", $delId, 1);
		$objOld->id = 0;
		$objOld->status = "renewForm";
		$objOld->pending = 0;
		foreach($objOld->jointList as &$joint)
			$joint->id = 0;
		$this->recordSave("MembershipOld", $objOld, 1);

		// Delete renewForm record from main list
		$this->recordDelete("Membership", $delId, 1);

		return "Complete";
	}
	
	function delete($uri){
		$id = filterToInt($_GET["id"]);
		$page = "";

		// Check role
		if(!$this->sys->login->checkLogin("adminMembership")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete Membership items. Only users with the 'adminMembership' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			// Archive old record
			$objOld = $this->recordGet("Membership", $id, 1);
			
			// Remove website login if it exists
			if(($objOld->emailAddress != "") && array_key_exists("user", $this->sys->modules)){
				$this->sys->modules["user"]->deleteUser($objOld->emailAddress);
				$page .= "<h3>Website account has been Deleted</h3>";	
			}
			
			// Backup old records
			$objOld->id = 0;
			$objOld->pending = 0;
			foreach($objOld->jointList as &$joint)
				$joint->id = 0;
			$this->recordSave("MembershipOld", $objOld, 1);
			$this->recordDelete("Membership", $id, 1);
			$page .= "<h3>Membership record has been Deleted</h3>";	
			$page .= "<button type='button' onclick='location.href = \"/membership\"'>Back to list</button>";
		}
		else {
			$obj = $this->sys->db->getObject("Membership", $id);
			$name = $obj->forename . " " . $obj->surname;

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$name} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"/membership\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function deleteNonRenewals($uri){
		// BDatabase::$debug = 1;
		$page = "";

		// Check role
		if(!$this->sys->login->checkLogin("adminMembership")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete Membership items. Only users with the 'adminMembership' role can perform this."));
			exit();
		}

		$date = new BDate("now");
		$date->addDays(-3 * 31);
		$renewalDate = $date->get();

		$select = "`fullId`='0' and `renewalDate` <= '{$renewalDate}' and (`status` = 'renew' or `status` = 'leave' or `status` = 'lapsed') and `membershipType` != 'Honorary'";
		$count = $this->sys->db->getObjectNumber("Membership", $select);

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			// Archive and delete old records
			$page .= "Will delete {$count} records<br>";
			for($pos = 0; $pos < $count; $pos += 100){
				$data = $this->sys->db->getObjectList("Membership", "id,renewalDate,forename,surname,emailAddress", $select, "surname,id", "0,100");
				foreach($data as $obj){
					$page .= "Deleting: {$obj->id}, {$obj->renewalDate}, {$obj->forename} {$obj->surname}, {$obj->emailAddress}<br>\n";

					// Remove website login if it exists
					if(($obj->emailAddress != "") && array_key_exists("user", $this->sys->modules)){
						$this->sys->modules["user"]->deleteUser($obj->emailAddress);
						$page .= "Website account has been Deleted<br>";	
					}

					// Backup old records
					$objOld = $this->recordGet("Membership", $obj->id, 1);
					$objOld->id = 0;
					$objOld->status = "lapsed";
					$objOld->pending = 0;
					foreach($objOld->jointList as &$joint)
						$joint->id = 0;
					$this->recordSave("MembershipOld", $objOld, 1);
					$this->recordDelete("Membership", $obj->id, 1);
				}
			}

			$page .= "<h3>Membership records have been Deleted</h3>";	
			$page .= "<button type='button' onclick='location.href = \"/membership\"'>Back to list</button>";
		}
		else {
			$page .= "Will delete {$count} records<br>";
			for($pos = 0; $pos < $count; $pos += 100){
				$data = $this->sys->db->getObjectList("Membership", "id,renewalDate,forename,surname,emailAddress,status,membershipType", $select, "renewalDate", "{$pos},100");
				foreach($data as $obj){
					$page .= "Delete: {$obj->id}, {$obj->renewalDate}, {$obj->status}, {$obj->membershipType}, {$obj->forename} {$obj->surname}, {$obj->emailAddress}<br>\n";
				}
			}

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteNonRenewals&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete all these records ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"/membership\";'", "Cancel");

			$page .= $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function renewalSelect($renewalDate){
		$select = "`fullId`='0' and `renewalDate` <= '{$renewalDate}' and `status` != 'renewDone' and `status` != 'renewForm' and `status` != 'lapsed' and `status` != 'leave' and `paymentType` != 'standingOrder' and `emailAddress` != '' and `membershipType` != 'Honorary'";
//		print "Select: {$select}<br>";
		return $select;
	}
	
	function reminderSelect($renewalDate){
		$select = "`fullId`='0' and `renewalDate` <= '{$renewalDate}' and `status` != 'renewDone' and `status` != 'renewForm' and `status` != 'lapsed' and `status` != 'leave' and `paymentType` != 'standingOrder' and (`country` = 'United Kingdom' or `country` = '') and `membershipType` != 'Honorary'";
//		print "Select: {$select}<br>";
		return $select;
	}
	
	function renewalSetStatus($renewalDate){
		$table = $this->sys->databasePrefix . "Membership";
		$cmd = "update `{$table}` set `status` = 'renew', `pending` = '1' where `fullId`='0' and `renewalDate` <= '{$renewalDate}' and `status` = 'member' and `membershipType` != 'Honorary'";
		$this->sys->db->query($cmd);
	}
	
	function renewalSendEmail($obj, $noEmail = 0){
		$obj->status = "renew";
//		if($obj->membershipType == "Complimentary")
//			$obj->membershipType = "1 year";
		$obj->paymentId = 0;
		$obj->paymentDate = "";
		$obj->paymentAmountPaid = 0;
		$obj->renewalSent += 1;
		if($obj->renewPassword == "")
			$obj->renewPassword = makeRandomPassword();
		$this->recordSave("Membership", $obj, 0);

		if(!$noEmail && ($obj->emailAddress != "")){
			$subject = "{$this->sys->siteTitle}: Your membership is due for renewal";
			$message = "Your Tandem Club membership is due for renewal.\n";
			$message .= "Please renew your membership to support the Tandem Club and receive all the benefits of belonging to the club including:\n";
			$message .= "National and International events, local club runs, the TC Journal, extra website information etc.\n";
			$message .= "You can renew or discontinue your membership using the following link:\n\n";
			$message .= "{$this->sys->baseUrl}/membership?cmd=memberRenew&id={$obj->id}&renewPassword={$obj->renewPassword}\n\n";
			$message .= "If you have recently renewed, please disregard this email.\n";
			$message .= "If you wish to leave the club please use the link above or contact the renewals secretary to avoid getting these renewal emails.\n";
			$message .= "Payments using a card or standing order is best for us.\n";
			$message .= "Please contact the renewals secretary (renewals@tandem-club.org.uk) if you have any problems.\n";
			$message .= "\nTandem Club website: https://www.tandem-club.org.uk\n";
			
			if($this->testEmails)
				emailList($this->sys->user->emailAddress, $subject, $message);
			else
				emailList($obj->emailAddress, $subject, $message);
		}
	}
	
	function renewalSendEmails(){
		if($this->sys->user)
			$userName = $this->sys->user->userName;
		else
			$userName = "";

		$date = new BDate("now");
		$date->addDays(10);
		$renewalDate = $date->get();

		$this->renewalSetStatus($renewalDate);
		$data = $this->sys->db->getObjectList("Membership", "", $this->renewalSelect($renewalDate), "", "");
		$count = count($data);

		if($this->checkAccess() == 2){
			if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
				$page = "<h2>Renewals sent</h2>";
				if($this->testEmails)
					$page .= "Emails not actually sent as in test mode</br>";

				$page .= "Sending {$count} renewal email notices<br>\n";

				foreach($data as $obj){
					$page .= "Renew: {$obj->id}, {$obj->renewalDate}, {$obj->forename} {$obj->surname}, {$obj->emailAddress}<br>\n";
					$this->renewalSendEmail($obj, $this->testEmails);
				}

				$this->sys->logAdd("notice", "membership", "RenewalSendEmails sent $count emails");
				$subject = "TandemClub: Admin: Renewal Emails sent";
				$message = "The system has sent {$count} renewal emails.\n\n";
				$message .= "Sent by user: {$userName}.\n\n";
				$message .= $page;
				emailList(configGet("membership.adminEmailList"), $subject, $message, 1);
			}
			else {
				$m = new BHtmlNode("div", "name='dialogQuestion'");
				$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=renewalSendEmails&amp;doIt=1' method='post'");
				$f->appendNode("h3", "", "Do you really want to send {$count} renewal notices ?");

				$f->appendNode("input", "type='submit' name='arg' value='Do It'");
				$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"/membership\";'", "Cancel");

				$page = $f->render();

				foreach($data as $obj){
					$page .= "Renew: {$obj->id}, {$obj->renewalDate}, {$obj->forename} {$obj->surname}, {$obj->emailAddress}<br>\n";
				}
			}
		}
		else {
			if(isset($_GET["cron"]) && $_GET["cron"] == "xxzzyy"){
				foreach($data as $obj){
					$page .= "Renew: {$obj->id}, {$obj->renewalDate}, {$obj->forename} {$obj->surname}, {$obj->emailAddress}<br>\n";
					$this->renewalSendEmail($obj, $this->testEmails);
				}

				$this->sys->logAdd("notice", "membership", "RenewalSendEmails sent $count emails");
				$subject = "TandemClub: Admin: Renewal Emails sent";
				$message = "The system has sent {$count} renewal emails.\n";
				$message .= "Sent by cron system.<br>\n\n";
				$message .= $page;
				emailList(configGet("membership.adminEmailList"), $subject, $message, 1);
			}
			else {
				$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Membership"));
				return NULL;
			}
		}


		return $page;
	}
	
	function renewalForms(){
		$numForms = 50;
		
		$date = new BDate("now");
		$date->addDays(30);
		$renewalDate = $date->get();

		$num = $this->sys->db->getObjectNumber("Membership", $this->renewalSelect($renewalDate));
		$numLinks = intval(($num + $numForms - 1) / $numForms);
		
		$f = new BHtmlNode("div");
		$f->appendNode("h3", "", "Membership Renewal Forms");
		$f->appendText("There are {$num} renewals due");
		
		$l = &$f->appendNode("ul");
		for($i = 0; $i < $numLinks; $i++){
			$start = $i * $numForms;
			$end = $start + $numForms - 1;
			$l->appendNode("li")->appendNode("a", "href=?cmd=renewalFormsPdf&amp;start={$start}", "Forms: {$start} - {$end}");
		}
		
		
		return $f->render();
	}
	
	function renewalFormsPdf(){
		if(isset($_GET["start"]))
			$start = filterToInt($_GET["start"]);
		else
			$start = 0;
			
		$date = new BDate("now");
		$date->addDays(30);
		$renewalDate = $date->get();

		return $this->paperFormsPdf($this->renewalSelect($renewalDate), $start, 50, 0);
	}

	function reminderForms(){
		$numForms = 50;
		
		$date = new BDate("now");
		$renewalDate = $date->get();

		$num = $this->sys->db->getObjectNumber("Membership", $this->reminderSelect($renewalDate));
		$numLinks = intval(($num + $numForms - 1) / $numForms);
		
		$f = new BHtmlNode("div");
		$f->appendNode("h3", "", "Membership Reminder Forms");
		$f->appendText("There are {$num} reminders due");
		
		$l = &$f->appendNode("ul");
		for($i = 0; $i < $numLinks; $i++){
			$start = $i * $numForms;
			$end = $start + $numForms - 1;
			$l->appendNode("li")->appendNode("a", "href=?cmd=reminderFormsPdf&amp;start={$start}", "Forms: {$start} - {$end}");
		}
		
		
		return $f->render();
	}
	
	function reminderFormsPdf(){
		if(isset($_GET["start"]))
			$start = filterToInt($_GET["start"]);
		else
			$start = 0;
			
		$date = new BDate("now");
		$renewalDate = $date->get();

		return $this->paperFormsPdf($this->reminderSelect($renewalDate), $start, 50, 1);
	}

	function paperFormsPdf($select, $start, $number, $reminder = 0){
		$data = $this->sys->db->getObjectList("Membership", "", $select, "", "$start,$number");
//		$c = count($data);
//		print "Number: $c<br/>";

		// create new PDF document
		$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

		// set document information
		$pdf->SetCreator(PDF_CREATOR);
		$pdf->SetAuthor($this->sys->siteTitle);
		if($reminder){
			$pdf->SetTitle("Tandem Club Reminder form");
			$pdf->SetSubject("Tandem Club Reminder form");
			$pdf->SetHeaderData("../../themes/tandemClub/Logo3.png", 30, "Tandem Club", "Membership Reminder", array(0,0,0), array(253, 184, 39));
		}
		else {
			$pdf->SetTitle("Tandem Club Renewal form");
			$pdf->SetSubject("Tandem Club Renewal form");
			$pdf->SetHeaderData("../../themes/tandemClub/Logo3.png", 30, "Tandem Club", "Membership Renewal", array(0,0,0), array(253, 184, 39));
		}

		// set default header data
//		$pdf->setFooterData(array(0,64,0), array(0,64,128));

		// set header and footer fonts
		$pdf->setHeaderFont(Array("times", "", 24));
//		$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));
		$pdf->setPrintFooter(false);

		// set default monospaced font
		$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

		// set margins
		$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT, PDF_MARGIN_BOTTOM);
		$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
		$pdf->SetFooterMargin(0);

		// set auto page breaks
		$pdf->SetAutoPageBreak(TRUE, 0);

		// set image scale factor
		$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

		// set default font subsetting mode
		$pdf->setFontSubsetting(true);

		// Set font
		$pdf->SetFont('times', '', 12, '', true);
		$pdf->setCellHeightRatio(1.05);

		foreach($data as $obj){
			$obj = $this->recordGet("Membership", $obj->id, 1);

			if($obj->renewPassword == ""){
				$obj->renewPassword = makeRandomPassword();
				$this->recordSave("Membership", $obj, 0);
			}

			// Add a page
			$pdf->AddPage();

			// Set some content to print
			$h = new BHtmlNode("div");
			
			$h->appendNode("div");

			$t = &$h->appendNode("table", "border=\"1\" style=\"padding: 30px;\"");
			$r = &$t->appendNode("tr");

			$r->appendNode("td", "", "&nbsp;<br/>" . getAddressString($obj));

			$str = "";
			$str .= "Full: {$obj->membershipNumber}, {$obj->forename} {$obj->surname}<br/>";
			$lc = 1;
			foreach($obj->jointList as $joint){
				$str .= "Joint: {$joint->membershipNumber}, {$joint->forename} {$joint->surname}<br/>";
				$lc++;
			}
			while($lc < 6){
				$str .= "<br />";
				$lc++;
			}
			
			$r->appendNode("td", "", $str);

//			$r = &$t->appendNode("tr");
//			$str = "Other information: Email: {$obj->emailAddress}, Telephone: {$obj->telephoneHome}";
//			$r->appendNode("td", "colspan=\"2\"", $str);
			$h->appendNode("p", "", "Other information: Email: {$obj->emailAddress}, Telephone: {$obj->telephoneHome}");

			$h->appendNode("p", "", "Dear {$obj->forename},\n");
			if($reminder){
				$h->appendText("Your membership was due for renewal on {$obj->renewalDate}. To continue receiving the Tandem Club Journal and other ");
				$h->appendText("benefits of membership, please renew via the Internet link that has been previously emailed to you (if you have supplied us with an email address) without delay or in one of the following ways:\n");
				$h->appendText("Unfortunately it is too late to pay by Standing Order this time around, but you can set up one for next year.\n");
			}
			else {
				$h->appendText("Your membership is due for renewal on {$obj->renewalDate}. To continue receiving the Tandem Club Journal and other ");
				$h->appendText("benefits of membership, please renew via the Internet link that has been previously emailed to you (if you have supplied us with an email address) or in one of the following ways:\n");
			}
			$l = &$h->appendNode("ul");
			$add = configGet("membership.chequeAddressRenew");

			if($reminder){
				$l->appendNode("li", "", "By the Internet (Credit/Debit cards, BACS): Using the URL: {$this->sys->baseUrl}/membership?renew={$obj->id},{$obj->renewPassword}");
			}
			else {
				$l->appendNode("li", "", "By the Internet (Credit/Debit cards, BACS, Standing Order): Using the URL: {$this->sys->baseUrl}/membership?renew={$obj->id},{$obj->renewPassword}");
				$l->appendNode("li", "", "By post using a Standing Order. Please fill in the Standing Order form below and send the Standing Order form to your bank. Please let the Renewals Secretary know that you have paid this way.");
			}		
			$l->appendNode("li", "", "By post using a cheque. Please fill in the details below and send the complete form together with your payment to: Renewals Secretary: {$add}");
			$h->appendNode("p", "", "The Internet based system is easiest for us and allows you to change your details easily. If renewing by post please write any changes to details on the back of this form and fill in your cheque details below.");
			
			$t = &$h->appendNode("table", "border=\"1\"");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "", "Subscription Type");
			$r->appendNode("th", "", "1 Year");
			$r->appendNode("th", "", "5 Years");
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", "Full Member + First Joint Member");
			$r->appendNode("td", "", "&pound;10.00");
			$r->appendNode("td", "", "&pound;40.00");
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", "Family' Members (see below)");
			$r->appendNode("td", "", "&pound;0.00");
			$r->appendNode("td", "", "&pound;0.00");
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", "Other Joint Members");
			$r->appendNode("td", "", "&pound;0.50");
			$r->appendNode("td", "", "&pound;2.00");
			
			$h->appendNode("p", "", "Family membership is open to all dependents of the full member aged 17 or under, and residing at the same address. If any of your joint members qualify for Family membership please add their date of birth against their name at the top of this letter. Please make cheques payable to the Tandem Club.");
			
			$h->appendNode("p", "", "I/we enclose a cheque for &pound;.................... for ...................years membership.");

			if($reminder){
				$date = new BDate($obj->renewalDate);
				$date->addYears(1);
				$soDate = $date->get();
			}
			else {
				$soDate = $obj->renewalDate;
			}
			
			$h->appendNode("hr");
			$h->appendText("&nbsp;<br/>");
			$h->appendNode("b", "", "Standing Order Request - Please send to your bank<br/><br/>");
			$h->appendText("Your full name: ...........................................................................Your Phone Number: .................................<br/>\n");
			$h->appendText("Your banks name: ...........................................................................<br/>\n");
			$h->appendText("Sort code: ........................................ Account Number: .....................................................<br/>\n");
			$h->appendText("Recipient: The Tandem Club,Bank: Lloyds Bank, Sort code: 30-91-36, Account Number: 00856202<br/>\n");
			$h->appendText("Payment amount: &pound;10.00, Paid Yearly, First payment: {$soDate}<br />\n");
			$h->appendText("Payment reference: mem {$obj->membershipNumber}, Payments to be made until further notice<br/><br/>\n");
			$h->appendText("Your signature: ................................................................... Date: .................................");
			
			// Print text using writeHTMLCell()
//			$pdf->writeHTMLCell(0, 0, '', '', $h->render(), 0, 1, 0, true, '', true);
			$pdf->writeHTML($h->render());

			if(0){			
//				return $h->render();
				$pdf->Output('renewal_forms.pdf', 'I'); return "";
			}
		}
		
		$pdf->Output('renewal_forms.pdf', 'I');
		return "";
	}

	function emailAll($uri){
		$emailAddresses = "";
		$data = array(0);
		for($pos = 0; count($data); $pos += 100){
			$data = $this->sys->db->getObjectList("Membership", "emailAddress", "`emailAddress` != '' and `emailSend`='1'", "", "{$pos},100");

			foreach($data as $e){
				if($emailAddresses != "")
					$emailAddresses .= ",{$e->emailAddress}";
				else
					$emailAddresses .= $e->emailAddress;
			}
		}
		return $this->emailList($uri, $emailAddresses);
	}
	
	function emailGroup($uri){
		if($this->sys->user && ($this->sys->user->group != "")){
			$group = $this->sys->user->group;

			$data = $this->sys->db->getObjectList("Membership", "emailAddress", "`group`='$group' and `emailAddress` != '' and  `emailSend`='1'", "", "");

			$emailAddresses = "";
			foreach($data as $e){
				if($emailAddresses != "")
					$emailAddresses .= ",{$e->emailAddress}";
				else
					$emailAddresses .= $e->emailAddress;
			}
			return $this->emailList($uri, $emailAddresses);
		}
		else {
			return "<h3>User does not belong to a group</h3>";
		}
	}
	
	function emailList($uri, $emailAddresses = ""){
		if(isset($_POST["sendEmail"])){
			$files = array();
			$f = new BHtmlNode("dev");
			
			$from = $this->sys->siteEmailAddress;
			$reply = $_POST["reply"];

			$message = "<html><head></head><body>";
			$message .= "Please reply to: <a href='mailto:{$this->sys->user->emailAddress}'>{$this->sys->user->emailAddress}</a><br/>";
			$message .= $_POST["message"];
			$message .= "</body></html>";

			foreach($_FILES as $file){
				if($file["name"] != ""){
					$files[] = new EmailFileAttachment($file["name"], $file["type"], $file["tmp_name"]);
				}
  			}
			
			$emailAddresses = filterString($_POST["emailAddresses"]);
			$emailAddressNum = count(explode(",", $emailAddresses));

			// Send email
			$f->appendNode("p", "", "Number of Email Addresses: {$emailAddressNum}");
			$f->appendNode("p", "", "From: {$from}");
			$f->appendNode("p", "", "ReplyTo: {$reply}");
			$r = 0;

			if($this->testEmails)
				$emailAddresses = $this->sys->user->emailAddress;

			$emailStatus = emailListFromReply($from, $reply, $emailAddresses, filterString($_POST["subject"]), $message, 1, $files);
			$f->appendNode("h3", "", "Emails have been sent");
			$f->appendText($emailStatus);
		}
		else {
			$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='?cmd=emailList' method='post'");

			if($emailAddresses == "")
				return "No email addresses";

			$emailAddressNum = count(explode(",", $emailAddresses));
			$f->appendNode("h3", "", "Sending email to {$emailAddressNum} email addresses");

			$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
			$t->appendTableInput("Subject", "subject", "text", "TandemClub: ", "width:500px"); 
			$t->appendTableInput("Message", "message", "textareaHtml", "", "height:300px; width:400px", array("height"=>"400px")); 
			$t->appendTableInput("Attachment0", "attach0", "file", "", ""); 
			$t->appendTableInput("Attachment1", "attach1", "file", "", ""); 
			$t->appendTableInput("Attachment2", "attach2", "file", "", ""); 
			$t->appendTableInput("Attachment3", "attach3", "file", "", ""); 
			$f->appendNode("input", "type='hidden' name='MAX_FILE_SIZE' value='3000000'");
			$f->appendNode("input", "type='hidden' name='emailAddresses' value='{$emailAddresses}'");
			$f->appendNode("input", "type='hidden' name='reply' value='{$this->sys->user->emailAddress}'");
			$f->appendNode("input", "type='submit' name='sendEmail' value='Send Email' style='background-color: lightgreen; font-size: 150%;' onclick='this.value=\"Sending ...\";'");
		}

		return $f->render();
	}
	
	function emailGetJournalSendList(){
		$emailAddresses = array();
		$data = array(0);
		for($pos = 0; count($data); $pos += 100){
			$data = $this->sys->db->getObjectList("Membership", "emailAddress", "`emailAddress` != '' and `journalByEmail`='1'", "", "{$pos},100");

			foreach($data as $e){
				$emailAddresses[] = $e->emailAddress;
			}
		}
		return $emailAddresses;
	}
	
	function memberNew($uri){
		return $this->edit($uri, 0, 0);
	}

	function memberRenew($uri){
		if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$id = filterToInt($_GET["id"]);
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}

		$obj = $this->recordGet("Membership", filterToInt($id), 1);
		if($obj == Null){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}

		$p = $_GET["renewPassword"];
//		print "RenewPassword: $p ({$obj->renewPassword}) Email: {$obj->emailAddress} ({$this->sys->user->emailAddress})</br>";

		// Check membership database for email match ...
		if(($obj->renewPassword != $_GET["renewPassword"]) && (!isset($this->sys->user) || ($obj->emailAddress != $this->sys->user->emailAddress))){
			$this->sys->pageDisplayError(new BError(1, "RenewPassword does not match our records or you are not logged in"));
			return "";
		}

		return $this->edit($obj, 0, 0);
	}

	function memberRenewForm($uri){
		return $this->edit($uri, 0, 1);
	}


	function newMembershipNumber(){
		$n = configGet("membership.nextMembershipNumber");
		$next = intval($n) + 1;
		configSet("membership", "nextMembershipNumber", $next);
		return $n;
	}
	
	function countryList(){
		$l = array("");
		if($maps = $this->sys->modules["maps"]){
			$l = array_merge($l, $maps->getCountryList());
		}
		return $l;
	}

	function groupsList(){
		$groups = array();
		$list = $this->sys->db->getObjectList("Groups", "", "", "name", "");

		foreach($list as $g){
			$groups[$g->name] = $g->name;
//			$groups[$g->name] = $g->title;
		}
		return $groups;
	}
	
	function groupsUpdate(){
		$pos = 0;
		if($groups = $this->sys->modules["groups"]){
			$list = array(0);
			for($pos = 0; count($list); $pos += 100){
				$list = $this->sys->db->getObjectList("Membership", "", "", "", "{$pos},100");
				foreach($list as $obj){
					if($group = $groups->getGroupForPostcode($obj->postcode)){
						$obj->group = $group;
						$this->sys->db->putObject("Membership", $obj);
					}
				}
			}
		}
	}

	function addWebAccount($uri){
		$page = "<h2>Add web account</h2>";

		if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$id = filterToInt($_GET["id"]);
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No member selected"));
			return "";
		}

		$obj = $this->recordGet("Membership", filterToInt($id), 0);
		
		if($obj->emailAddress == ""){
			$this->sys->pageDisplayError(new BError(1, "Member does not have an email address"));
			return "";
		}
		
		if(!$this->sys->modules["user"]->checkIfUserExists($obj->emailAddress)){
			$user = new BObjUser();
			$user->name = $obj->forename . " " . $obj->surname;
			$user->userName = $obj->emailAddress;
			$user->userPassword = "";
			$user->postalAddress = $obj->address;
			$user->city = $obj->city;
			$user->postcode = $obj->postcode;
			$user->group = $obj->group;
			$user->country = $obj->country;
			$user->emailAddress = $obj->emailAddress;
			$user->emailSend = $obj->emailSend;
			$user->telephoneHome = $obj->telephoneHome;
			$user->telephoneMobile = $obj->telephoneMobile;
			$user->membershipId = $obj->id;

			$page .= $this->sys->modules["user"]->addUser($user, 1) . "<br>";
			
			$obj->userId = $user->id;
			$this->recordSave("Membership", $obj);
			
			$page .= "Added website account for: {$user->name} {$user->emailAddress}.</br>";
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "Member already has website account"));
			return "";
		}
		return $page;
	}

	function addWebAccountAll(){
		$page = "<h2>Add all website accounts</h2>";
		$total = 0;
		$totalNoEmail = 0;

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$page = "<b>Testing. No accounts will actually be added and no emails sent.</b></br>";
			$data = array("0");
			for($pos = 0; count($data); $pos += 100){
				$data = $this->sys->db->queryObjs("Membership", "select * from tcevents_Membership t1 left join tcevents_User t2 on t2.emailAddress = t1.emailAddress where t2.emailAddress is null limit {$pos},100");
//				$data = $this->sys->db->getObjectList("Membership", "", "`fullId` = '0'", "", "{$pos},100");
				foreach($data as $obj){
					if($obj->emailAddress != ""){
						if(!$this->sys->modules["user"]->checkIfUserExists($obj->emailAddress)){
							$user = new BObjUser();
							$user->name = $obj->forename . " " . $obj->surname;
							$user->userName = $obj->emailAddress;
							$user->userPassword = "";
							$user->postalAddress = $obj->address;
							$user->city = $obj->city;
							$user->postcode = $obj->postcode;
							$user->country = $obj->country;
							$user->group = $obj->group;
							$user->emailAddress = $obj->emailAddress;
							$user->emailSend = $obj->emailSend;
							$user->telephoneHome = $obj->telephoneHome;
							$user->telephoneMobile = $obj->telephoneMobile;

//							$this->sys->modules["user"]->addUser($user, 1);
							$page .= "Added website account for: {$user->name} {$user->emailAddress}.</br>";
							$total += 1;
						}
						else {
							$page .= "Already present: UserName: {$obj->emailAddress}.</br>";
						}
					}
					else {
						$totalNoEmail += 1;
					}
				}
			}
			$page .= "Total accounts added: {$total}</br>";
			$page .= "Total without email: {$totalNoEmail}</br>";
		}
		else {
			# Workout how many members with email addresses are not in the website accounts
//			$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `emailAddress` != ''");
			$data = $this->sys->db->query("select count(t1.id) from tcevents_Membership t1 left join tcevents_User t2 on t2.emailAddress = t1.emailAddress where t2.emailAddress is null");
			$value = $data[0][0];
			
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=addWebAccountAll&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to add {$value} website accounts ?");

			$f->appendNode("input", "type='submit' name='arg' value='Do It'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"/membership\";'", "Cancel");

			$page = "<b>Testing. No accounts will actually be added and no emails sent.</b></br>";
			$page .= $f->render();
		}
		return $page;
	}

	function linkWebAccountAll(){
		$page = "<h2>Link all website accounts</h2>";

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$page = "<b>Linking membership and website accounts..</b></br>";

			$data = array("0");
			for($pos = 0; count($data); $pos += 100){
				$data = $this->sys->db->getObjectList("Membership", "", "`fullId` = '0'", "", "{$pos},100");
				foreach($data as $obj){
					if($obj->emailAddress != ""){
						$dataUsers = $this->sys->db->getObjectList("User", "", "`emailAddress` = '{$obj->emailAddress}'", "", "");
						if(count($dataUsers)){
							$user = $dataUsers[0];
							//$page .= "Updating: {$obj->id} {$user->id}<br>";
							if($user->membershipId == 0){
								$page .= "Updating website account: {$obj->id} {$user->id}<br>";
								$user->membershipId = $obj->id;
								$this->sys->db->putObject("User", $user);
							}
							if($obj->userId == 0){
								$page .= "Updating membership account: {$obj->id} {$user->id}<br>";
								$obj->userId = $user->id;
								$this->sys->db->putObject("Membership", $obj);
							}
						}
					}
				}
			}
		}
		else {
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=linkWebAccountAll&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to link all the website accounts with the membership accounts ?");

			$f->appendNode("input", "type='submit' name='arg' value='Do It'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"/membership\";'", "Cancel");

			$page = "";
			$page .= $f->render();
		}
		return $page;
	}
	
	function webUserUpdate($obj){
		// Updates web accounts database with information from membership record update
		$list = $this->sys->db->getObjectList("User", "", "`membershipId` = '{$obj->id}'", "", "");
		if(count($list)){
			$user = $list[0];
//			print("Updating web user record: {$obj->userId}<br>");
			if($obj->emailAddress != "")
				$user->emailAddress = $obj->emailAddress;
			if($obj->telephoneHome != "")
				$user->telephoneHome = $obj->telephoneHome;
			if($obj->telephoneMobile != "")
				$user->telephoneMobile = $obj->telephoneMobile;
			if($obj->address != ""){
				$user->postalAddress = $obj->address;
				$user->city = $obj->city;
				$user->postcode = $obj->postcode;
				$user->country = $obj->country;
			}
			if($obj->group != ""){
				$user->group = $obj->group;
			}
			$this->sys->db->putObject("User", $user);
		}
	}
	
	function userUpdate($user){
		if($user->membershipId){
			$obj = $this->recordGet("Membership", $user->membershipId);
			if($obj){
				if(($user->emailAddress != $obj->emailAddress) ||
					($user->telephoneHome != $obj->telephoneHome) ||
					($user->telephoneMobile != $obj->telephoneMobile) ||
					($user->postalAddress != $obj->address) ||
					($user->city != $obj->city) ||
					($user->postcode != $obj->postcode) ||
					($user->country != $obj->country) ||
					($user->group != $obj->group)){
					
					$subject = "{$this->sys->siteTitle}: Admin: Member updated their details: Number: {$obj->membershipNumber}";
					$message = "{$this->sys->siteTitle}: Admin: Member updated their details: Number: {$obj->membershipNumber}\n\n";

					$message .= "Name:        {$obj->forename} {$obj->surname}\n";
					$message .= "Email:       {$obj->emailAddress}\n";
					$message .= "Postcode:    {$obj->postcode}\n";
					$message .= "PaymentType: {$obj->paymentType}\n\n";
					$message .= "View: {$this->sys->baseUrl}/membership?cmd=view&id={$obj->id}\n";
					$message .= "Edit: {$this->sys->baseUrl}/membership?cmd=edit&id={$obj->id}\n";

					$message .= "\nNew Information\n";
					$message .= "Email:       {$user->emailAddress}\n";
					$message .= "TelephoneHome:    {$user->telephoneHome}\n";
					$message .= "TelephoneMobile:    {$user->telephoneMobile}\n";
					$message .= "Address:    {$user->postalAddress}\n";
					$message .= "City:    {$user->city}\n";
					$message .= "Postcode:    {$user->postcode}\n";
					$message .= "Country:    {$user->country}\n";
					$message .= "Group:    {$user->group}\n";
					$message .= "Groups:    {$user->groups}\n";

					if($this->testEmails){
						emailList($this->sys->user->emailAddress, $subject, $message);
					}
					else {
						emailList(configGet("membership.adminEmailList"), $subject, $message);
					}
				}
			}
		}
	}
	
	function statisticsAdd($item, $value){
		$this->sys->modules["statistics"]->counterAddNow($this->name, $item, $value);
		
		// Total full members
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0'");
		$this->sys->modules["statistics"]->counterSetNow($this->name, "totalFull", $value);
		
		
		// Total joint members
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` != '0'");
		$this->sys->modules["statistics"]->counterSetNow($this->name, "totalJoint",  $value);
	}

	function statistics(){
		$f = new BHtmlNode("div");
		$f->appendNode("h3", "", "Membership Statistics");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Item");
		$r->appendNode("th", "align='left'", "Value");
		
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total members");
		$value = $this->sys->db->getObjectNumber("Membership", "");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total full members");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0'");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total joint members");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` != '0'");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total full members with email");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `emailAddress` != ''");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total full members without email");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `emailAddress` = ''");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total new members waiting for joining");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `status` = 'new'");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total members needing to renew");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `status` = 'renew'");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total renewed waiting for payment complete");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `status` = 'renewDone'");
		$r->appendNode("td", "", $value);

		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Total members leaving");
		$value = $this->sys->db->getObjectNumber("Membership", "`fullId` = '0' and `status` = 'leave'");
		$r->appendNode("td", "", $value);

		// Statistics counters
		$f->appendNode("h3", "", "Membership Statistics Counters");

		$date = new BDate("now");
		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align='left'", "Year");
		$r->appendNode("th", "align='left'", "Month");
		$r->appendNode("th", "align='left'", "Item");
		$r->appendNode("th", "align='left'", "Number");
		
//		$list = $this->sys->modules["statistics"]->counterGetAll($date->getYear(), "%", $this->name, "%");
		$list = $this->sys->modules["statistics"]->counterGetAll("%", "%", $this->name, "%");
		foreach($list as $obj){
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $obj->year);
			$r->appendNode("td", "", $obj->month);
			$r->appendNode("td", "", $obj->item);
			$r->appendNode("td", "", $obj->number);
		}

		return $f->render();
	}
	
	function test(){
		// create new PDF document
		$pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);

		// set document information
		$pdf->SetCreator(PDF_CREATOR);
		$pdf->SetAuthor($sys->siteTitle);
		$pdf->SetTitle("Tandem Club Renewal form");
		$pdf->SetSubject("Tandem Club Renewal form");

		// set default header data
//		$pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, PDF_HEADER_TITLE.' 001', PDF_HEADER_STRING, array(0,64,255), array(253, 184, 39));
		$pdf->SetHeaderData("../../themes/tandemClub/Logo3.png", 30, "Tandem Club", "Membership Renewal", array(0,0,0), array(253, 184, 39));
//		$pdf->setFooterData(array(0,64,0), array(0,64,128));

		// set header and footer fonts
		$pdf->setHeaderFont(Array("times", "", 24));
//		$pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));
		$pdf->setPrintFooter(false);

		// set default monospaced font
		$pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);

		// set margins
		$pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
		$pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
		$pdf->SetFooterMargin(PDF_MARGIN_FOOTER);

		// set auto page breaks
		$pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);

		// set image scale factor
		$pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);

		// set default font subsetting mode
		$pdf->setFontSubsetting(true);

		// Set font
		$pdf->SetFont('times', '', 14, '', true);

		// Add a page
		$pdf->AddPage();

		// set text shadow effect
//		$pdf->setTextShadow(array('enabled'=>true, 'depth_w'=>0.2, 'depth_h'=>0.2, 'color'=>array(196,196,196), 'opacity'=>1, 'blend_mode'=>'Normal'));

		// Set some content to print
		$h = new BHtmlNode("div");
		$r = &$h->appendNode("div");
		$r->appendNode("h3", "", "TestingHeader");

		$t = &$h->appendNode("table", "border=\"1\"");
		$r = &$t->appendNode("tr");
		$r->appendNode("td", "", "Fred Blogs, Kingsmead Cottage,<br/> BS37 6RZ");
		$r->appendNode("td", "", "Membership Number: F12345, Joint: 1");
		$r = &$t->appendNode("tr");
		$r->appendNode("td", "colspan=\"2\"", "Other information");

		$h->appendText("Testing");

		// Print text using writeHTMLCell()
		$pdf->writeHTMLCell(0, 0, '', '', $h->render(), 0, 1, 0, true, '', true);
		$pdf->Output('example_001.pdf', 'I');
		return "";
	}
};


function mesure($var){
    $start = memory_get_usage();
    if(is_string($var)){
        $newValue = $var . '';
    }elseif(is_numeric($var)){
        $newValue = $var + 0;
    }elseif(is_object($var)){
        $newValue = clone $var;
    }elseif(is_array($var)){
        $newValue = array_flip($var);
    }
    return memory_get_usage() - $start;
}


?>
