<?php
/*******************************************************************************
 *	News.php	News Module
 *			T.Barnaby,	BEAM Ltd,	2008-03-15
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";
require_once "BObjPicture.php";

class BObjNews extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("category", "String"),
			new BMember("group", "String"),
			new BMember("description", "StringHtml"),
			new BMember("pictureId", "Int"),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

class News extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "news";
		$this->title = "News";
		$this->info = "News module. This module allows the management of News items.";
		$this->group = "";
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("News" => "BObjNews"));

		// Add HTML functions
		$this->sys->htmlFuncs[] = "newsList";
		$this->sys->htmlFuncs[] = "newsListGroups";
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminGroup" . $this->group) || $this->sys->login->checkLogin("adminPta") || $this->sys->login->checkLogin("adminPupil");
	}
		
	function getMenu(){
		$this->parseUri("");

		if($this->checkAccess()){
			return new BMenu("News", "/{$this->name}", "", array("News List" => "/{$this->name}", "News Add" => "/{$this->name}?cmd=add&amp;group={$this->group}"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view(filterToInt($_GET["id"]));	break;
			case "add":			return $this->edit("0");		break;
			case "edit":			return $this->edit(filterToInt($_GET["id"]));	break;
			case "save":			return $this->save();			break;
			case "delete":			return $this->delete(filterToInt($_GET["id"]));	break;
			case "deleteDo":		return $this->deleteDo(filterToInt($_GET["id"]));	break;
			default: 			return $this->viewList();		break;
			}
		}
		else {
			return $this->viewList();
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$o = new BMenu("News", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList'"),
				new BMenu("", "", ""),
//				new BMenu("View", "",	"onmouseover='this.href = \"?cmd=view&amp;id=\" + document.ws.list.objId'"),
				new BMenu("Add", "",	"onmouseover='this.href = \"?cmd=add&amp;group={$this->group}\"'"),
//				new BMenu("Edit", "",	"onmouseover='this.href = \"?cmd=edit&amp;id=\" + document.ws.list.objId'"),
//				new BMenu("Delete", "",	"onmouseover='this.href = \"?cmd=delete&amp;id=\" + document.ws.list.objId'"),
				new BMenu("", "", ""),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}

	function viewList(){
		$page = "<h2>News</h2>\n";

		if($this->checkAccess()){
			$view = new BViewDatabase("list", "action");
		}
		else {
			$view = new BViewDatabase("list");
		}

		if($this->group){
			$where = "`group`='" . dbEscapeString($this->group) . "'";
			$view->setGroup($this->group);
		}
		else
			$where = "";
		
		$view->setView(array(
			new BViewType("dateAdded", "Date Published", "date"),
			new BViewType("category", "Category"),
			new BViewType("group", "Group"),
			new BViewType("title", "Title"),
			new BViewType("description", "Description")
			));

		$view->setSelection($where);
		$view->setLimit(20);
	
		if($view->order == "")
			$view->setOrder("!dateAdded");

		$view->setTable("News");
		$page = $page . $view->getHtml();

#		$page .= "<div class='help'>";
#		$page .= "This lists all of the users of the system. ";
#		$page .= "In the left hand bar there are links to the functions you can perform.";
#		$page .= "</div>";

		return $page;
	}

	function view($id){
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$obj = $this->sys->db->getObject("News", $id);
		}

		$page = "<h2>News View</h2>";

		$view = new BViewObj("news");
		$view->setView(array(
			new BViewType("title"),
			new BViewType("category"),
			new BViewType("group"),
			new BViewType("description"),
			new BViewType("dateAdded", "Date Published", "date"),
			));
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}

	function edit($arg){
		$page = "<h2>Edit News</h2>";

		if($arg == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else if(is_object($arg)){
			$obj = $arg;
		}
		else if($arg == 0){
			$obj = $this->sys->db->getNewObject("News");
			$obj->group = $this->group;
			$obj->dateAdded = strftime("%Y-%m-%d");
		}
		else {
			$obj = $this->sys->db->getObject("News", $arg);
		}

		if(!isset($obj->sendEmail)) $obj->sendEmail = 0;
		if(!isset($obj->sendTextMessage)) $obj->sendTextMessage = 0;
		

		$view = new BEditObj("news", "News Information", "?cmd=save&amp;group={$this->group}", "");
		$editors = array(
			new BViewType("id", "", "hidden"),
			new BViewType("title", "Title"),
			new BViewType("category", "Category", "select", categoryGetList("news")),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("description", "Description", "textareaHtml", "", 500, 280),
			new BViewType("pictureId", "", "hidden"),
			new BViewType("picture", "", "pictureUploaderLineSmall"),
			new BViewType("dateAdded", ""),
			);

		$editors[] = new BViewType("sendEmail", "Send Emails", "checkbox");

		if($this->sys->smsSend)
			$editors[] = new BViewType("sendTextMessage", "SendTextMessage", "checkbox");
			
		$view->setView($editors);

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("category", "Enter the category by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");
		$view->setHelp("sendEmail", "If checked will send an email to all users.");
		$view->setHelp("sendTextMessage", "If checked will send an SMS text message to all users. Note this service costs per message sent.");

		$view->setObj($obj);
		$page .= $view->getHtml();
		if($this->sys->smsSend)
			$page .= "Note that SMS text messages are limited to 160 characters. The message will be truncated to this length";
		
		return $page;
	}

	function save(){
		global	$sys;

//		debug_log("Save: " . print_r($_POST, 1) . print_r($_FILES, 1) . "\n");

		$page = "";
	
		if(count($_POST)){
			if(isset($_GET['doIt']))
				$doIt = 1;
			else
				$doIt = 0;

			unset($_POST['cmd']);
			$obj = new BObjNews();
			$obj->set($_POST);
			
			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			if(!$doIt && ($_POST['sendTextMessage'] == "1")){
				$sms = $this->sys->modules["sms"];
				if($sms == Null){
					$sys->pageDisplayError(new BError(1, "No SMS module configured"));
					return Null;
				}
				if(! $sms->checkAccess()){
					$sys->pageDisplayError(new BError(1, "You are not authorised to send SMS text messages"));
					return Null;
				}
				$smsNumber = $sms->smsNumber();
				$smsCost = $sms->smsCost("");
				$smsAvailable = $sms->smsAvailable() * $sys->smsCost;
				
				$m = new BHtmlNode("div", "name='dialogQuestion'");
				$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=save&amp;group={$this->group}&amp;doIt=1' method='post'");
				$f->appendNode("h3", "", "Do you really want to to send SMS text messages: this will cost: &pound;{$smsCost} ({$smsNumber} people, &pound;{$smsAvailable} Available) ?");
				foreach($_POST as $k => $p){
					$f->appendNode("input", "type='hidden' name='{$k}' value='{$p}'");
				}
				$f->appendNode("input", "type='submit' name='arg' value='Submit'");
				$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

				$page = $f->render();
				return $page;
			}
			
			// Upload any pictures present
			if((count($_FILES) == 1) && isset($_FILES["picture"])){
				$file = $_FILES["picture"];
//				debug_log("uploadPictures: Do file: {$file["tmp_name"]}\n");

				// Add picture
				$pobj = new BObjPicture();
				$pobj->category = "news";
				$pobj->group = $obj->group;
				$pobj->dateAdded = strftime("%Y-%m-%d");
				$pobj->title = $obj->title;
				$pobj = $this->sys->db->appendObject("Picture", $pobj);
				pictureSave($pobj->id, $file["tmp_name"]);
				$obj->pictureId = $pobj->id;
			}

			if($obj->id)
				$this->sys->db->putObject("News", $obj);
			else
				$this->sys->db->appendObject("News", $obj);

			if($_POST['sendEmail'] == "1"){
				// Send an email to all interested users
				$description = $obj->description;
				
				$description .= "<p>More information is available at: <a href='{$this->sys->baseUrl}/news?cmd=view&id={$obj->id}'>{$this->sys->baseUrl}/news?cmd=view&id={$obj->id}</a></p>";
				$description .= "<p>Sent by the {$this->sys->siteName} system. Please do not reply to this email directly.</p>";
				if($this->sys->user){
					$description .= "<p>Please respond to: <a href='mailto:{$this->sys->user->emailAddress}'>{$this->sys->user->emailAddress}</a></p>";
				}

				$group = $this->group;
				if($obj->group != "")
					$group = $obj->group;

				if($group != "")
					$emailStatus = emailGroup($group, "{$this->sys->siteName}: News: {$obj->title}", $description, 1);
				else
					$emailStatus = emailAll("{$this->sys->siteName}: News: {$obj->title}", $description, 1);
				
				$page .= "<h3>Email message has been sent.</h3>";
				$page .= $emailStatus;
			}

			if($doIt && ($_POST['sendTextMessage'] == "1")){
				// Send a text message to all those with mobile numbers
				$description = htmlToText($obj->description);
				$sms = $this->sys->modules["sms"];
				$r = $sms->sendMessage($obj->title, $description);
				if(is_numeric($r) and ($r == 0))
					$page .= "<h3>Text message has been sent.</h3>";
				else
					$page .= "<h3>{$r}</h3>";
			}

		}

		$page .= "<h3>News details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminGroup" . $this->group)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}
		if($this->group)
			$groupCmd = "&amp;group={$this->group}";
		else
			$groupCmd = "";

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("News", $id);

		$m = new BHtmlNode("div", "name='dialogQuestion'");
		$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteDo&amp;id={$id}{$groupCmd}' method='post'");
		$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

		$f->appendNode("input", "type='submit' name='arg' value='Delete'");
		$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

		$page = $f->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		
		return $page;
	}

	function deleteDo($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminGroup" . $this->group)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("News", $id);

		// Now delete the showcase itself
		$this->sys->db->deleteObject("News", $id);

		$page = "";
		$page .= "<h3>News {$obj->title} has been Deleted</h3>";	
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}
};

function newsList($num=3, $group=""){
	global	$sys;
	
	$title = "News";
	$where = "";

	if(($group == "") && isset($_GET['group']))
		$group = filterString($_GET['group']);
	
	if($group != ""){
		if($group[0] == "!"){
			$group = substr($group, 1);
			$where = "`group`!='$group'";
			$group = "";
		}
		else {
			$where = "`group`='$group'";
		}
		$title = $group . " " . $title;
	}
	$objList = $sys->db->getObjectList("News", "", $where, "!dateAdded", "{$num}");

	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/news?group={$group}'")->appendNode("h2", "", $title);

	foreach($objList as $obj){
		if($obj->pictureId)
			$description = pictureHtml($obj->pictureId, "", 1, "align='left' style='padding-right:5px;'") . $obj->description;
		else
			$description = $obj->description;

		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->dateAdded));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/news?cmd=view&amp;id={$obj->id}'", $obj->title);
//		$d1->appendNode("div", "class='NewsSummaryCategory'", "Category: General");
//		$d1->appendNode("div", "class='NewsSummaryAuthor'", "Posted by: terry");
		$d1->appendNode("div", "class='NewsSummaryContent'", $description);
	}

	return $html->render();
}

function newsListGroups($num=3, $notGroup=""){
	global	$sys;

	$num = intval($num);
	$notGroup = filterString($notGroup);
	
	$where = "";

	if($notGroup != ""){
		if(substr($notGroup, 0, 1) == "!"){
			$notGroup = substr($notGroup, 1);
		}
		$where = "where `group`!='$notGroup'";
	}	
	
	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/news'")->appendNode("h2", "", "Group News");

	$limit = $num * 2;
	$l = $sys->db->queryObjs("News", "select id from {$sys->databasePrefix}News {$where} order by dateAdded desc limit {$limit}");
	if($num > count($l))
		$num = count($l);

	while(count($l) and $num){
		$pos = rand() % count($l);
		if($l[$pos] == null)
			continue;
		$id = $l[$pos]->id;
		$obj = $sys->db->getObject("News", $id);
		$l[$pos] = null;
		$num--;

		if($obj->pictureId)
			$description = pictureHtml($obj->pictureId, "", 1, "align='left' style='padding-right:5px;'") . $obj->description;
		else
			$description = $obj->description;

		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryCategory'")->appendNode("a", "href='{$sys->baseUrl}/groups/{$obj->group}/index.html'", $obj->group);
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->dateAdded));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/news?cmd=view&amp;id={$obj->id}'", $obj->title);
//		$d1->appendNode("div", "class='NewsSummaryAuthor'", "Posted by: terry");
		$d1->appendNode("div", "class='NewsSummaryContent'", $description);
	}

	return $html->render();
}
?>
