<?php
/*******************************************************************************
 *	Payments.php	Payment Module
 *			T.Barnaby,	BEAM Ltd,	2013-11-15
 *******************************************************************************
 *
 * Testing
 *	Make sure the Nochex account has test transcations enabled by logging in and setting it.
 *	The Payments class has a test variable that then uses this.
 *	Set to 1 for Full payment test accessing Nochex, but no actual payemnts being made.
 *	Set to 2 for Completely local testing (No Nochex interaction).
 *
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

function addressToLine($str){
	$r = "";
	
	for($i = 0; $i < strlen($str); $i++){
		if($str[$i] == "\r"){
		}
		else if($str[$i] == "\n"){
			$r .= ", ";
		}
		else {
			$r .= $str[$i];
		}
	}
	return $r;
}

class BObjPayment extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Int"),
			new BMember("dateAdded", "DateTime", "Datetime"),
			new BMember("title", "String"),
			new BMember("type", "String"),
			new BMember("refund", "Int"),
			new BMember("source", "String"),
			new BMember("reference", "String"),
			new BMember("currency", "String"),
			new BMember("amount", "Float", "", "", "decimal(8,2)"),
			new BMember("name", "String"),
			new BMember("emailAddress", "String"),
			new BMember("telephone", "String"),
			new BMember("address", "String"),
			new BMember("city", "String"),
			new BMember("postcode", "String"),
			new BMember("country", "String"),
			new BMember("status", "String"),
			new BMember("secure", "String"),
			new BMember("paid", "Int"),
			new BMember("transactionId", "String"),
			new BMember("complete", "Int"),
			new BMember("test", "Int"),
			new BMember("datePaid", "DateTime", "Datetime"),
		));
	}
};

function http_post($url, $vars) { 
	$u = parse_url($url);
	if(!isset($u["port"])){
		if($u["scheme"] == "https")
			$u["port"] = 443;
		else
			$u["port"] = 80;
	}
	$host = $u["host"];
	$p = $u["path"];
	
	// Get urlencoded vesion of $vars array 
	$urlencoded = ""; 
//	foreach($vars as $k => $v)
//		$urlencoded .= urlencode($k) . "=" . urlencode($v) . "&amp;"; 
	foreach($vars as $k => $v)
		$urlencoded .= urlencode($k) . "=" . urlencode($v) . "&"; 

	$urlencoded = substr($urlencoded, 0, -1);
//	debug_log("Payment Post:: {$urlencoded}");

	$headers = "POST {$p} HTTP/1.0\r\n"
		. "Host: {$host}\r\n" 
		. "Content-Type: application/x-www-form-urlencoded\r\n" 
		. "Content-Length: ". strlen($urlencoded) . "\r\n\r\n";

	$fp = fsockopen($u["host"], $u["port"], $errno, $errstr, 10);
	if(!$fp)
		return "Error: fsockopen failed.\r\nError no: $errno - $errstr";

	fputs($fp, $headers);
	fputs($fp, $urlencoded);  

	$ret = ""; 
	while(!feof($fp))
		$ret .= fgets($fp, 1024);

	fclose($fp);
	return $ret;
} 

class BPaymentNochex {
	// Basic Interface routines
	function __construct(&$sys, &$payments, $type, $test){
		$this->sys = $sys;
		$this->payments = $payments;
		$this->url = "https://secure.nochex.com/";
		$this->apcUrl = "http://www.nochex.com/nochex.dll/apc/apc";
		$this->type = $type;
		$this->merchant_id = "";
		$this->test = $test;
		
		if(array_key_exists($type, $sys->nochex)){
//			print "Set to: " . $sys->nochex[$type];
			$this->merchant_id = $sys->nochex[$type];
		}
	}
	
	function getCardPaymentHtml($obj){
		$baseUrl = $this->sys->baseUrl;
		
		if($maps = $this->sys->modules["maps"]){
			$countryCode = $maps->getCountryCode($obj->country);
		}
		else {
			$countryCode = "";
		}

//		print "SentCountry: '{$obj->country}' '{$countryCode}'</br>"; return "";
		$telephone = str_replace(' ', '', $obj->telephone);
		
		$callbackUrl = $baseUrl . "/payments?cmd=payCallback&type=apc&secure={$obj->secure}&id={$obj->id}";
		$successUrl = $baseUrl . "/payments?cmd=payCallback&type=success_display&secure={$obj->secure}&id={$obj->id}";
		$cancelUrl = $baseUrl . "/payments?cmd=payCallback&type=cancelled&secure={$obj->secure}&id={$obj->id}";
		
		$h = new BHtmlNode("div", "name='paymentsCard'");
		if($this->test == 2)
			$f = &$h->appendNode("form", "name='paymentsCard' action='/payments?cmd=payTest&secure={$obj->secure}' method='post'");
		else
			$f = &$h->appendNode("form", "name='paymentsCard' action='{$this->url}' method='post'");

		$f->appendNode("input", "type='hidden' name='merchant_id' value='{$this->merchant_id}'");
		$f->appendNode("input", "type='hidden' name='amount' value='{$obj->amount}'");
		$f->appendNode("input", "type='hidden' name='order_id' value='{$obj->id}'");
		$f->appendNode("input", "type='hidden' name='description' value='{$obj->title}'");

		$f->appendNode("input", "type='hidden' name='callback_url' value='{$callbackUrl}'");
		$f->appendNode("input", "type='hidden' name='success_url' value='{$successUrl}'");
		$f->appendNode("input", "type='hidden' name='cancel_url' value='{$cancelUrl}'");

		$f->appendNode("input", "type='hidden' name='billing_fullname' value='{$obj->name}'");
		$f->appendNode("input", "type='hidden' name='billing_address' value='{$obj->address}'");
		if($obj->city)
			$f->appendNode("input", "type='hidden' name='billing_city' value='{$obj->city}'");
		$f->appendNode("input", "type='hidden' name='billing_postcode' value='{$obj->postcode}'");
		if($countryCode != "")
			$f->appendNode("input", "type='hidden' name='billing_country' value='{$countryCode}'");
		$f->appendNode("input", "type='hidden' name='email_address' value='{$obj->emailAddress}'");
		$f->appendNode("input", "type='hidden' name='customer_phone_number' value='{$telephone}'");
//		$f->appendNode("input", "type='hidden' name='delivery_postcode' value='{$obj->postcode}'");

		if($this->test){
			$f->appendNode("input", "type='hidden' name='test_transaction' value='100'");
			$f->appendNode("input", "type='hidden' name='test_success_url' value='{$successUrl}'");
		}
		$d1 = &$f->appendNode("div", "style='display:inline-block;'");
		$d1->appendNode("img", "src='/icons/nochex/nochex-small.jpg' align='top'");
		$d2 = &$d1->appendNode("div", "style='display:inline-block;'");
		$d2->appendNode("input", "type='submit' name='submit' value='Pay on Credit or Debit Card' style='background-color: lightgreen; font-size: 120%;'");
		$d2->appendNode("br");
		$d2->appendNode("input", "type='button' name='cancel' value='Cancel' style='background-color: orange; font-size: 120%;' onclick='location.href = \"{$cancelUrl}\";'");

		if(0){
			$str = $h->render();
			debug_log("Payment Post:: {$str}");
		}

		return $h->render();
	}

	function callback($type, $obj){
		$response = http_post($this->apcUrl, $_POST);
		
		debug_log("Payment APC returned: {$response}");

		if(strstr($response, "DECLINED")){
		
			$msg = "APC was not AUTHORISED.\r\n\r\n";
			$status = "declined";
		} 
		else if(strstr($response, "AUTHORISED")){
			$msg = "APC was AUTHORISED.";
			$obj->transactionId = filterString($_POST["transaction_id"]);
			$status = "success";
		}
		else{ 
			$msg = "APC error.";
			$status = "error";
		} 
		debug_log("Payment APC status: $msg");
 
		return $this->payments->payComplete($status, $obj);
	}
};
	
class Payments extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "payments";
		$this->title = "Payments";
		$this->info = "Payments module. This module allows card payemnts.";
		$this->paymentTypes = array("card_membership","card_event","cheque","cheque_euros", "bacs_pounds","bacs_euros","standingOrder", "other");
		$this->paymentStatus = array("added", "cancelled", "declined", "paid", "error");
		$this->sources = array("events", "membership", "forSale", "shop", "adverts", "manual");
		$this->test = 0;
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("Payments" => "BObjPayment"));

		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminPayments,viewPayments"));
	}
	
	function configs(){
		$configs = array();
		$configs[] = new BObjConfig("adminEmailCardsList", "StringList", "", "Admin Email Card List", "Admin email list for card transactions (comma separated)");
		$configs[] = new BObjConfig("adminEmailOthersList", "StringList", "", "Admin Email Others List", "Admin email list for non card transactions (comma separated)");
		return $configs;
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("adminPayments");
	}
		
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Payments", "/{$this->name}", "", array("Payments List" => "/{$this->name}", "Test Payment" => "/{$this->name}/testPayment"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->checkAccess()){
			switch($this->cmd){
			case "configEdit":	$page = $this->configEdit($uri); break;
			case "view":		$page = $this->view($uri); break;
			case "edit":		$page = $this->edit($uri); break;
			case "save":		$page = $this->save($uri); break;
			case "delete":		$page = $this->delete($uri); break;
			case "deleteAllTest":	$page = $this->deleteAllTest($uri); break;
			
			case "makePayment":	$page = $this->makePayment($uri); break;
			case "makePaymentTest":	$page = $this->makePaymentTest($uri); break;
			case "payTest":		$page = $this->payTest($uri); break;
			case "payCallback":	$page = $this->payCallback($uri); break;

			default: 		$page = $this->viewList(); break;
			}
			return $page;
		}
		else if($this->sys->login->checkLogin("viewPayments")){
			switch($this->cmd){
			case "view":		$page = $this->view($uri); break;
			case "makePayment":	$page = $this->makePayment($uri); break;
			case "makePaymentTest":	$page = $this->makePaymentTest($uri); break;
			case "save":		$page = $this->saveFromUser($uri); break;
			case "payCallback":	$page = $this->payCallback($uri); break;
			case "viewList": 	$page = $this->viewList(); break;
			default: 		$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Payments")); break;
			}
			return $page;
		}
		else {
			switch($this->cmd){
			case "makePayment":	$page = $this->makePayment($uri); break;
			case "makePaymentTest":	$page = $this->makePaymentTest($uri); break;
			case "save":		$page = $this->saveFromUser($uri); break;
			case "payCallback":	$page = $this->payCallback($uri); break;
			default: 		$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Payments")); break;
			}
			return $page;
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$help = $this->sys->baseUrl . "/modules/payments/info.html";

			$o = new BMenu("Payments", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList'"),
				new BMenu("Make a payment", "",	"href='?cmd=makePayment'"),
				new BMenu("", "", ""),
				new BMenu("Configuration", "",	"href='?cmd=configEdit'"),
				new BMenu("Delete all test", "",	"href='?cmd=deleteAllTest'"),
				new BMenu("Make a test payment", "",	"href='?cmd=makePaymentTest'"),
				new BMenu("Help", "",		"href='{$help}'"),
				));
		}
		else if($this->sys->login->checkLogin("viewPayments")){
			$help = $this->sys->baseUrl . "/modules/payments/info.html";

			$o = new BMenu("Payments", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList'"),
				new BMenu("Make a payment", "",	"href='?cmd=makePayment'"),
				new BMenu("", "", ""),
				new BMenu("Help", "",		"href='{$help}'"),
				));
		}
		else {
			$o = new BMenu("Payments", "", "", array(
				new BMenu("Make a payment", "",	"href='?cmd=makePayment'"),
				));
		}
		return $o;
	}

	function viewList(){
		$page = "<h2>Payments information</h2>\n";
		
		if(!$this->checkAccess() && !$this->sys->login->checkLogin("viewPayments")){
			$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Payments"));
		}
		
//		print "Form: "; print_r($_POST); print "<br>\n";
		
		$sel = new BSelect("select", "Select", "", 3);
		$sel->setFormMode("get");
		$sel->setView(array(
			new BViewTypeSelect("period", "dateAdded", "Period", "period", "select", array("", "Last Year", "Last 6 Months", "Last 2 Months", "Last Month", "Last Week")),
			new BViewTypeSelect("startDate", "dateAdded", "Startdate", "greaterThan", "date"),
			new BViewTypeSelect("endDate", "dateAdded", "EndDate", "lessThanEqual", "date"),
			new BViewTypeSelect("type", "", "", "equal", "select", array_merge(array(""), $this->paymentTypes)),
			new BViewTypeSelect("refund", "", "", "equal", "tristate"),
			new BViewTypeSelect("source", "", "", "equal", "select", array_merge(array(""), $this->sources)),
			new BViewTypeSelect("reference", "", "", "like", "text", "", 150, 0),
			new BViewTypeSelect("status", "", "", "equal", "select", array_merge(array(""), $this->paymentStatus)),
			new BViewTypeSelect("paid", "", "", "equal", "tristate"),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			new BViewTypeSelect("name", "", "", "like", "text", "", 150, 0),
			));

		$sel->getObj()->period = "Last Month";
		$sel->getObj()->paid = 0;
		$sel->pagerSet(0, 50, 0);
		$sel->update();
		
		$where = $sel->getSelectString();

//		print "Where: {$where}<br>\n";
//		BDatabase::$debug = 1;
		
		$view = new BViewList("list", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("dateAdded", "Date Added", "datetime"),
			new BViewType("datePaid", "Date Paid", "datetime"),
			new BViewType("type", "Type"),
			new BViewType("refund", "Refund"),
			new BViewType("source", "Source"),
			new BViewType("reference", "Reference"),
			new BViewType("name", "Name"),
//			new BViewType("amount", "GBP"),
			new BViewType("amountPounds", "GBP"),
			new BViewType("amountEuros", "EURO"),
			new BViewType("status", "Status"),
			new BViewType("transactionId"),
			new BViewType("paid", "Paid"),
			new BViewType("test", "Test"),
			new BViewType("title", "Title"),
			));
			
		$list = $this->sys->db->getObjectList("Payments", "", $where, "!dateAdded", $sel->pagerGetLimitString());
		$totalNumber = $this->sys->db->getObjectNumber("Payments", $where);
		$sel->pagerSet(0, 50, $totalNumber);

		$amountPounds = 0.0;
		$amountEuros = 0.0;
		foreach($list as $obj){
			if($obj->refund)
				$a = -$obj->amount;
			else
				$a = $obj->amount;

			if($obj->currency == "Euros"){
				$obj->amountPounds = "0.00";
				$obj->amountEuros = $a;
				$amountEuros += $a;
			}
			else {
				$obj->amountPounds = $a;
				$obj->amountEuros = "0.00";
				$amountPounds += $a;
			}
				
		}
		$view->setList($list, 1);

		$l = array_fill(0, 15, "");
		$l[3] = "<b>Total</b>";
		$l[8] = sprintf("%.2f", $amountPounds);
		$l[9] = sprintf("%.2f", $amountEuros);
		$view->addExtraRow($l);

		$page = $page . $sel->getHtml();
		$page = $page . $view->getHtml();

		return $page;
	}
	
	function view($arg){
		$page = "<h2>View Payment</h2>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$obj = $this->sys->db->getObject("Payments", filterToInt($_GET["id"]));
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		
		$view = new BViewObj("payment", "Payment View");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("title", "Title"),
			new BViewType("type", "Type", "select", $this->paymentTypes),
			new BViewType("refund", "Refund", "checkbox"),
			new BViewType("source", "Source", "select", $this->sources),
			new BViewType("reference", "Reference"),
			new BViewType("currency", "Currency", "select", array("Pounds", "Euros")),
			new BViewType("amount", "Amount"),
			new BViewType("name", "Name"),
			new BViewType("address"),
			new BViewType("city"),
			new BViewType("postcode"),
			new BViewType("country"),
			new BViewType("emailAddress", "Email", "emailAddress", array("{$this->sys->siteTitle}: Payment Ref: {$obj->id}: ")),
			new BViewType("telephone"),
			new BViewType("status", "Status", "select", $this->paymentStatus ),
			new BViewType("transactionId"),
			new BViewType("paid", "Paid", "checkbox"),
			new BViewType("test", "Test", "checkbox"),
			new BViewType("dateAdded", "Date Added"),
			new BViewType("datePaid", "Date Paid"),
			));

		$view->setObj($obj);
		return $view->getHtml();
	}

	function makePayment($uri){
		return $this->edit($uri, 1);
	}
	
	function makePaymentTest($uri){
		return $this->edit($uri, 1, 1);
	}
	
	function edit($arg, $payDo = 0, $test = 0){
		$args = "";
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			if(filterToInt($_GET["id"]) == 0)
				return "No payment record id 0.";
			$page = "<h2>Edit Payment</h2>";
			$obj = $this->sys->db->getObject("Payments", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("Payments");
			$obj->id = 0;
			$obj->dateAdded = strftime("%Y-%m-%d %H:%M:%S");
			$obj->status = "added";
			$obj->source = "manual";
			$obj->type = "card_event";
			$obj->currency = "Pounds";
			$obj->country ="United Kingdom";
			if($this->sys->user){
				$obj->name = $this->sys->user->name;
				$obj->address = addressToLine($this->sys->user->postalAddress);
				$obj->city = $this->sys->user->city;
				$obj->postcode = $this->sys->user->postcode;
				$obj->country = $this->sys->user->country;
				$obj->emailAddress =  $this->sys->user->emailAddress;
				$obj->telephone = $this->sys->user->telephoneHome;
			}
		}
		
		if($test || $this->test){
			$obj->test = 1;
		}
		
		if($payDo){
			$view = new BEditObj("payment", "Payment Information", "/payments?cmd=save&amp;payDo=1", "");
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("type", "Type", "select", $this->paymentTypes),
				new BViewType("refund", "Refund", "checkbox"),
				new BViewType("source", "Source", "select", $this->sources),
				new BViewType("reference", "Reference"),
				new BViewType("currency", "Currency", "select", array("Pounds", "Euros")),
				new BViewType("amount", "Amount"),
				new BViewType("name", "Name"),
				new BViewType("address"),
				new BViewType("city"),
				new BViewType("postcode"),
				new BViewType("country", "", "select", $this->countryList()),
				new BViewType("emailAddress", "Email"),
				new BViewType("telephone"),
				new BViewType("status", "", "hidden"),
				new BViewType("dateAdded", "", "hidden"),
				new BViewType("datePaid", "", "hidden"),
				new BViewType("payDo", "", "hidden"),
				new BViewType("test", "", "hidden"),
				));
		}
		else {
			$view = new BEditObj("payment", "Payment Information", "/payments?cmd=save", "");
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("type", "Type", "select", $this->paymentTypes),
				new BViewType("refund", "Refund", "checkbox"),
				new BViewType("source", "Source", "select", $this->sources),
				new BViewType("reference", "Reference"),
				new BViewType("currency", "Currency", "select", array("Pounds", "Euros")),
				new BViewType("amount", "Amount"),
				new BViewType("name", "Name"),
				new BViewType("address"),
				new BViewType("city"),
				new BViewType("postcode"),
				new BViewType("country", "", "select", $this->countryList()),
				new BViewType("emailAddress", "Email"),
				new BViewType("telephone"),
				new BViewType("status", "Status", "select", $this->paymentStatus ),
				new BViewType("transactionId"),
				new BViewType("paid", "Paid", "checkbox"),
				new BViewType("complete", "Complete", "hidden"),
				new BViewType("test", "Test", "checkbox"),
				new BViewType("dateAdded", "Date Added", "view"),
				new BViewType("datePaid", "Date Paid", "view"),
				new BViewType("test", "", "hidden"),
				));
		}

		$view->setHelp("amount", "Enter the amount.");
		$view->setHelp("title", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		return $view->getHtml();
	}
	
	function saveFromUser($uri){
		if(!$this->checkAccess()){
			// Only allow payments and test payments from non admin in users
			if(!isset($_POST["id"]))
				$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Payments"));
			if(filterToInt($_POST["id"]))
				$this->sys->pageDisplayError(new BError(1, "You are not allowed to access Payments"));
		}
		return $this->save($uri);
	}
	
	function save($uri){
		$page = "";
		$edit = 0;
		$payDo = isset($_POST["payDo"]);
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjPayment();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj, $payDo);
			}
			if($obj->amount == 0.00){
				$this->sys->status = "<span class='editError'>Error you neet to enter an amount</span>";
				return $this->edit($obj, $payDo);
			}

			if($obj->id){
				$objOld = $this->sys->db->getObject("Payments", $obj->id);
				$this->sys->db->putObject("Payments", $obj);
				$edit = 1;
				
				if(($objOld->paid == 0) && ($obj->paid == 1)){
					$obj->datePaid = strftime("%Y-%m-%d %H:%M:%S");
					if(array_key_exists($obj->source, $this->sys->modules) && method_exists($this->sys->modules[$obj->source], "payComplete")){
						$page .= "</br>" . $this->sys->modules[$obj->source]->payComplete($obj);
					}
				}
			}
			else {
				$this->sys->db->appendObject("Payments", $obj);
			}

			$page .= "<h3>Details saved</h3>";
			if(!$edit)
				$page .= $this->payDo($obj) . "</br>";
		}
		else {
			$page .= "<h3>Error: Nothing to save</h3>";
		}

		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";

		return $page;
	}

	function delete(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminPayments")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' and 'adminPayments' roles can perform this."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("Payments", $id);
			$this->sys->db->deleteObject("Payments", $id);
			$page = "<h3>Payment: {$obj->id} Titled: {$obj->title} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("Payments", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&amp;id={$id}&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete payment: {$obj->id} titled: {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}

	function deleteAllTest(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminPayments")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' and 'adminPayments' roles can perform this."));
			exit();
		}

		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$table = $this->sys->databasePrefix . "Payments";
			$sql = "delete from `{$table}` where `test` != '0'";
			$obj = $this->sys->db->query($sql);
			$page = "<h3>All test payments have been Deleted</h3>";
		}
		else {
			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteAllTest&amp;doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete all test payments ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}

	function payGetNew(){
		$obj = $this->sys->db->getNewObject("Payments");
		$obj->dateAdded = strftime("%Y-%m-%d %H:%M:%S");
		$obj->status = "added";
		return $obj;
	}

	function payDo($obj){
		global	$sys;
		
		$page = "";

		$obj->secure = makeRandomPassword();
		if($this->test)
			$obj->test = 1;

		if($obj->id){
			$this->sys->db->putObject("Payments", $obj);
		}
		else {
			$this->sys->db->appendObject("Payments", $obj);
		}

		if($obj->refund){
			$page .= "Refund information added. Now please arrange to refund the payment by the method chosen.";
		}
		else {
			if(($obj->type == "card_membership") || ($obj->type == "card_event")){
				$pay = new BPaymentNochex($this->sys, $this, $obj->type, $obj->test);
				$page .= "Now please pay using our secure card payment system.";
				$page .= $pay->getCardPaymentHtml($obj);
			}
			else {
				$page .= "Payment information added. Now please arrange payment by the method chosen.";
			}
		}
		
		return $page;
	}
	
	function payCallback($uri){
		$g = "";
		foreach($_GET as $k => $v){
			$v = filterString($v);
			$g .= "[{$k}] => {$v}, ";
		}
		$p = "";
		foreach($_POST as $k => $v){
			$v = filterString($v);
			$p .= "[{$k}] => {$v}, ";
		}
//		print "PayCallback {$type} Get: {$g} Post: {$p}";
		debug_log("PayCallback: Get: {$g} Post: {$p}");

		if(isset($_GET["type"])){
			$type = filterString($_GET["type"]);
		}
		else {
			return "Error: incorrect parameters";
		}
		
		if(isset($_GET["id"]) && is_numeric($_GET["id"])){
			$obj = $this->sys->db->getObject("Payments", filterToInt($_GET["id"]));
			
			if($obj->secure !=  filterString($_GET["secure"])){
				return "Error: incorrect parameters";
			}
			
			if($type == "apc"){
				$pay = new BPaymentNochex($this->sys, $this, $obj->type, $this->test);
				return $pay->callback($type, $obj);
			}
			else {
				debug_log("PayCallback: call payComplete {$type}");
				return $this->payComplete($type, $obj);
			}
		}
		else {
			return "Error: incorrect parameters";
		}
	}
	
	function payComplete($type, $obj){
		if($obj == null)
			return "Error: Not an object";
		
		$emailSend = 1;
		$page = "<h1>Payment Status</h1>";
		if($type == "success"){
			$obj->datePaid = strftime("%Y-%m-%d %H:%M:%S");
			$obj->status = "paid";
			$obj->paid = 1;
			$page .= "Many thanks {$obj->name} your payment has been made.";
		}
		else if($type == "success_display"){
			// What should happen with this ?
//			$obj->datePaid = strftime("%Y-%m-%d %H:%M:%S");
//			$obj->status = "paid";
//			$obj->paid = 1;
			$emailSend = 1;
			$page .= "Many thanks {$obj->name} your payment has been made.";
		}
		else if($type == "cancelled"){
			$obj->status = "cancelled";
			$obj->paid = 0;
			$page .= "You have cancelled your payment, you will need to pay by other means. Please contact the treasurer.";
		}
		else if($type == "cancelled_display"){
			$emailSend = 0;
			$page .= "You have cancelled your payment, you will need to pay by other means. Please contact the treasurer.";
		}
		else if($type == "declined"){
			$obj->status = "declined";
			$obj->paid = 0;
			$page .= "You payment has beed declined, you will need to pay by other means. Please contact the treasurer.";
		}
		else if($type == "test"){
			$obj->datePaid = strftime("%Y-%m-%d %H:%M:%S");
			$obj->status = "test";
			$obj->paid = 1;
			$page .= "Many thanks {$obj->name} your test payment has been made";
		}
		else {
			$obj->status = "error";
			$obj->paid = 0;
			$page .= "There was an error in payment, you will need to pay by other means. Please contact the treasurer.";
		}

		$this->sys->db->putObject("Payments", $obj);

		// Update module
		//if(($type == "success") || ($type == "success_display")){
		if($type == "success"){
			if(array_key_exists($obj->source, $this->sys->modules) && method_exists($this->sys->modules[$obj->source], "payComplete")){
				$page .= "</br>" . $this->sys->modules[$obj->source]->payComplete($obj);
			}
		}

		if($type != "success_display"){
			// Email
			if($emailSend){
				$subject = "{$this->sys->siteTitle}: Admin: Payment made: Status: {$obj->status} Amount: {$obj->amount}";
				$message = "{$this->sys->siteTitle}: Admin: Payment made\n\n";
				$message .= "Response: {$type}\n";
				$message .= "Title:    {$obj->title}\n";
				$message .= "Name:     {$obj->name}\n";
				$message .= "Type:     {$obj->type}\n";
				$message .= "Status:   {$obj->status}\n";
				$message .= "Currency: {$obj->currency}\n";
				$message .= "Amount:   {$obj->amount}\n\n";
				$message .= "Payment View: {$this->sys->baseUrl}/payments?cmd=view&id={$obj->id}\n";
				$message .= "Payment Edit: {$this->sys->baseUrl}/payments?cmd=edit&id={$obj->id}\n";

				if($obj->test){
					$message .= "\n*** Test Payment ***\n";
					emailList($this->sys->user->emailAddress, $subject, $message);
				}
				else {
					if(substr($obj->type, 0, 5) == "card_"){
						emailList(configGet("payments.adminEmailCardsList"), $subject, $message);
					}
					else {
						emailList(configGet("payments.adminEmailOthersList"), $subject, $message);
					}
				}
			}
		}

		return $page;
	}
	
	function payDone($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminPayments")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to manage payment items. Only users with the 'adminPayments' role can perform this."));
			exit();
		}

		$obj = $this->sys->db->getObject("Payments", $id);
		$obj->datePaid = strftime("%Y-%m-%d %H:%M:%S");
		$obj->status = "paid";
		$obj->paid = 1;
		$this->sys->db->putObject("Payments", $obj);

		// Update module
		if(array_key_exists($obj->source, $this->sys->modules) && method_exists($this->sys->modules[$obj->source], "payComplete")){
			$this->sys->modules[$obj->source]->payComplete($obj);
		}

		return 0;
	}
	
	function payTest($uri){
		// This function used in place of Nochex's URL to test the payment system locally
		//print_r($_GET);	print_r($_POST);

		$h = new BHtmlNode("div");
		$h->appendNode("h1", "", "Payment Test");
		$t = &$h->appendNode("table", "cellpadding='2' cellspacing='2' border='1'");

		foreach($_POST as $k => $v){
			$v = filterString($v);
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $k);
			$r->appendNode("td", "", $v);
		}

		$success_url = filterString($_POST["success_url"]);
		$canceled_url = filterString($_POST["cancel_url"]);
		$apc_url = filterString($_POST["callback_url"]);

//		$declined_url = "?cmd=payCallback&amp;type=declined&amp;id=" . filterString($_POST["order_id"]);
//		$success_do_url = "?cmd=payCallback&amp;type=success&amp;id=" . filterString($_POST["order_id"]);
//		$canceled_do_url = "?cmd=payCallback&amp;type=cancelled&amp;id=" . filterString($_POST["order_id"]);
		$declined_url = "?cmd=payCallback&type=declined&id=" . filterString($_POST["order_id"]);
		$success_do_url = "?cmd=payCallback&type=success&id=" . filterString($_POST["order_id"]);
		$canceled_do_url = "?cmd=payCallback&type=cancelled&id=" . filterString($_POST["order_id"]);

		$f = &$h->appendNode("form", "name='paymentTest' action='?cmd=payCallback' method='post'");

		$v = filterString($_POST["amount"]);
		$f->appendNode("input", "type='hidden' name='amount' value='{$v}'");
		$v = filterString($_POST["order_id"]);
		$f->appendNode("input", "type='hidden' name='order_id' value='{$v}'");
		$f->appendNode("input", "type='hidden' name='info' value=''");
		$f->appendNode("input", "type='hidden' name='transaction_id' value='testxxx'");

		$f->appendNode("input", "type='button' name='success_do' value='Success' onclick='reply(this.form, \"success_do\");'", "");
		$f->appendNode("input", "type='button' name='cancel_do' value='Cancel' onclick='reply(this.form, \"canceled_do\");'", "");
		$f->appendNode("input", "type='button' name='declined' value='Declined' onclick='reply(this.form, \"declined\");'", "");
		$f->appendNode("input", "type='button' name='success' value='Success reply only' onclick='reply(this.form, \"success\");'", "");
		$f->appendNode("input", "type='button' name='canceled' value='Canceled reply only' onclick='reply(this.form, \"canceled\");'", "");
		$f->appendNode("input", "type='button' name='apc' value='Apc' onclick='reply(this.form, \"apc\");'", "");
		$f->appendNode("script", "", "function reply(form, type){ form.info.value = type; if(type == 'success') form.action = '{$success_url}'; else if(type == 'canceled') form.action = '{$canceled_url}'; else if(type == 'apc') form.action = '{$apc_url}'; else if(type == 'declined') form.action = '{$declined_url}'; else if(type == 'success_do') form.action = '{$success_do_url}'; else if(type == 'canceled_do') form.action = '{$canceled_do_url}'; form.submit();}");

		return $h->render();
	}
	
	function countryList(){
		$l = array("");
		if($maps = $this->sys->modules["maps"]){
			$l = array_merge($l, $maps->getCountryList());
		}
		return $l;
	}


};


if(0){
	$sms = new BPayment();

	$r = $sms->getCredits($n);

	print "NumCredits: Res: {$r} Num: {$n}\n";

	$r = $sms->sendMessage("Beam", array("07980502014"), "This is a test message");

	print "Send: Res: {$r}\n";
}

?>
