<?php
/*******************************************************************************
 *	Showcase.php	Showcase Module
 *			T.Barnaby,	BEAM Ltd,	2008-03-15
 *******************************************************************************
 *
 * Database Update 1:
 *	alter table `greenpower_Showcase` add `date` date not null;
 *	update `greenpower_Showcase` set `date` = `dateAdded`;
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";
require_once "BObjPicture.php";

global	$sys;
$sys->db->addObjectClasses(array("Showcase" => "BObjShowcase"));

class BObjShowcase extends BObj {
	function BObjShowcase(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("title", "String"),
			new BMember("group", "String"),
			new BMember("description", "String"),
			new BMember("date", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("dateAdded", "Date", "date", array('format' => '%Y-%n-%j')),
			));
	}
};

function fprintList($fd, $n, $l){
	fprintf($fd, "{$n}: ");
	foreach($l as $k => $v) fprintf($fd, "$k => $v, ");
	fprintf($fd, "<br>\n");
}


class Showcase extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "showcase";
		$this->title = "Showcase";
		$this->info = "Showcase module. This module allows the management of Showcase items.";
		$this->group = "";
		
		// Add HTML functions
		$this->sys->htmlFuncs[] = "showcaseGroup";

	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminGroup" . $this->group);
	}

	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("Showcase", "/{$this->name}", "", array("Showcase List" => "/{$this->name}", "Showcase Add" => "/{$this->name}?cmd=add&group={$this->group}"));
		}
		else {
			return new BMenu("Showcase", "/{$this->name}", "", array("Showcase List" => "/{$this->name}"));
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view(filterToInt($_GET["id"]));	break;
			case "add":			return $this->edit("0");		break;
			case "addPicture":		return $this->addPicture(filterToInt($_GET["id"]));		break;
			case "addPictures":		return $this->addPictures(filterToInt($_GET["id"]));		break;
			case "edit":			return $this->edit(filterToInt($_GET["id"]));	break;
			case "editPictures":		return $this->editPictures(filterToInt($_GET["id"]));	break;
			case "save":			return $this->save();			break;
			case "delete":			return $this->delete(filterToInt($_GET["id"]));	break;
			case "deleteDo":		return $this->deleteDo(filterToInt($_GET["id"]));	break;
			case "uploadPicture":		return $this->uploadPicture(filterToInt($_GET["id"]));	break;
			case "uploadPictures":		return $this->uploadPictures(filterToInt($_GET["id"]));	break;
			case "savePicture":		return $this->savePicture();		break;
			case "deletePicture":		return $this->deletePicture(filterToInt($_GET["id"]));	break;

			default: 			return $this->viewList();		break;
			}
		}
		else {
			switch($this->cmd){
			case "view":			return $this->view(filterToInt($_GET["id"]));	break;
			default: 			return $this->viewList();		break;
			}
		}
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$o = new BMenu("Showcase", "", "", array(
				new BMenu("List", "", "href='?cmd=viewList&group={$this->group}'"),
				new BMenu("", "", ""),
				new BMenu("View", "",	"onmouseover='this.href = \"?cmd=view&group={$this->group}&id=\" + document.ws.list.objId'"),

				new BMenu("Add showcase", "", "onmouseover='this.href = \"?cmd=add&group={$this->group}\"'"),
				new BMenu("Add pictures", "", "onmouseover='this.href = \"?cmd=addPictures&group={$this->group}&id=\" + document.ws.list.objId'"),
				new BMenu("Add single picture", "", "onmouseover='this.href = \"?cmd=addPicture&group={$this->group}&id=\" + document.ws.list.objId'"),
				new BMenu("Edit showcase info", "", "onmouseover='this.href = \"?cmd=edit&group={$this->group}&id=\" + document.ws.list.objId'"),
				new BMenu("Edit showcase pics", "", "onmouseover='this.href = \"?cmd=editPictures&group={$this->group}&id=\" + document.ws.list.objId'"),
				new BMenu("Delete showcase", "", "onmouseover='this.href = \"?cmd=delete&group={$this->group}&id=\" + document.ws.list.objId'"),
				new BMenu("", "", ""),
				));
		}
		else {
			$o = Null;
		}
		return $o;
	}

	function viewList(){
		$page = "";
		
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->group) . "'";
		else
			$where = "";

		if($this->checkAccess()){
			$view = new BViewDatabase("list", "selectAction");
			$view->setGroup($this->group);

			$view->setView(array(
				new BViewType("id", "Picture", "code", 'return pictureHtml(0, "showcase/{$obj->id}", 1);'),
				new BViewType("title", "Title"),
				new BViewType("group", "Group"),
				new BViewType("description", "Description"),
				new BViewType("date", "Date"),
				new BViewType("dateAdded", "Date Published"),
				));

			$view->setSelection($where);
			if($view->order == "")
				$view->setOrder("!date");

			$view->setTable("Showcase");
			$page = $page . $view->getHtml();
		}
		else {
			$list = $this->sys->db->getObjectList("Showcase", "", $where, "!date");

			$view = new BHtmlNode("table");

			foreach($list as $obj){
				$r = &$view->appendNode("tr");
				$c = &$r->appendNode("td");
				$c->appendNode("a", "href='?cmd=view&group={$this->group}&id={$obj->id}'", pictureHtml(0, "showcase/{$obj->id}", 1));
				$c = &$r->appendNode("td");
				$c->appendNode("a", "href='?cmd=view&group={$this->group}&id={$obj->id}'")->appendNode("h3", "", $obj->title);
//				$c = &$r->appendNode("td");
				$c->appendNode("span", "", dateString($obj->date));
				$c->appendNode("span", "style='width: 20px;'", "&nbsp");
				$c->appendNode("span", "", $obj->group);
				$c->appendNode("p", "", $obj->description);
//				$c->appendText($obj->description);
			}
			$page .= $view->render();
		}

		return $page;		
	}

	function view($id){
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}

		$showcase = $this->sys->db->getObject("Showcase", $id);
		if(isset($_GET["pos"]))
			$pos = intval($_GET["pos"]);
		else
			$pos = 0;

		$picList = $this->sys->db->getObjectList("Picture", "", "`category` = 'showcase/{$showcase->id}'", "id");
		$len = count($picList);
		
		if($len == 0)
			return "No pictures in showcase";

		if($pos >= $len)
			$pos = $len - 1;

		$posPrev = $pos - 1;
		$posNext = $pos + 1;

		$pic = $picList[$pos];

		$html = new BHtmlNode("div", "id='showcase-picture'");
		$t = &$html->appendNode("table", "width=100%");

		if(0){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td");
			$c = &$r->appendNode("td", "align='center'");
			$c->appendNode("h1", "", "Showcase - {$showcase->title}");

			// The buttons	
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td");
			if($posPrev < 0)
				$c->appendNode("img", "src='icons/buttons/ButtonPrev.gif'");
			else
				$c->appendNode("a", "href='?cmd=view&group={$this->group}&id={$id}&pos={$posPrev}'")->appendNode("img", "src='icons/buttons/ButtonPrev.gif'");

			$c = &$r->appendNode("td");
			if($posNext >= $len)
				$c->appendNode("img", "src='icons/buttons/ButtonNext.gif'");
			else
				$c->appendNode("a", "href='?cmd=view&group={$this->group}&id={$id}&pos={$posNext}'")->appendNode("img", "src='icons/buttons/ButtonNext.gif'");;
		}
		
		if(0){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td");
			$c = &$r->appendNode("td")->appendNode("center");;
			$c->appendNode("h2", "", $pic->title);
		}


		$r = &$t->appendNode("tr");

		// Add Thumbnails
		$c = &$r->appendNode("td");
		$d = &$c->appendNode("div");

		if($posPrev < 0)
			$d->appendNode("img", "src='icons/buttons/ButtonPrev.gif'");
		else
			$d->appendNode("a", "href='?cmd=view&group={$this->group}&id={$id}&pos={$posPrev}'")->appendNode("img", "src='icons/buttons/ButtonPrev.gif'");

		if($posNext >= $len)
			$d->appendNode("img", "src='icons/buttons/ButtonNext.gif'");
		else
			$d->appendNode("a", "href='?cmd=view&group={$this->group}&id={$id}&pos={$posNext}'")->appendNode("img", "src='icons/buttons/ButtonNext.gif'");;
		
		$s = $pos;
		$e = $pos + 5;
		if($e > count($picList)){
			$e = count($picList);
			$s = $e - 5;
		}
		if($s < 0)
			$s = 0;
		for($i = $s; $i < $e; $i++){
			$p = $picList[$i];
			$l = $c->appendNode("a", "href='?cmd=view&group={$this->group}&id={$showcase->id}&pos={$i}'");
			$l->appendNode("img", "src='{$this->sys->baseUrl}/files/pictures/picture-{$p->id}-thumb.jpg' style='padding-top: 3px' alt='{$p->title}'");
			$c->appendNode("br");
		}

		$c = &$r->appendNode("td", "align='center'");
		$c->appendNode("center")->appendNode("h3", "style='margin-top:0;'", $pic->title);

		$c->appendText(pictureHtml($pic->id, "", 0, "width=640px"));
		$c->appendNode("div", "", $pic->description);

		$r = &$t->appendNode("tr");
		$c = &$r->appendNode("td", "colspan=2");
		$c->appendNode("p", "", $showcase->description);

		$page = $html->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$showcase->id}\";</script>";
		return $page;
	}


	function editObj($obj){
		$page = "<h2>Edit Showcase</h2>";

		$view = new BEditObj("Showcase", "Showcase Information", "?cmd=save&group={$this->group}", "");
		$view->setView(array(new BViewType("id", "", "hidden"),
			new BViewType("title", "Title"),
			new BViewType("group", "Group", "select", array_merge(array(""), $this->sys->groups)),
			new BViewType("description", "Description", "textarea", "", 400, 100),
			new BViewType("date", "Date", "date"),
			new BViewType("dateAdded", "Date Added", "hidden"),
			));

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("description", "Enter an optional description. This can be multiple lines long and contain web addresses etc");

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";

		return $page;
	}

	function edit($id){
		$page = "<h2>Edit Showcase</h2>";

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else if($id == 0){
			$obj = $this->sys->db->getNewObject("Showcase");
			$obj->group = $this->group;
			$obj->date = strftime("%Y-%m-%d");
			$obj->dateAdded = strftime("%Y-%m-%d");
		}
		else {
			$obj = $this->sys->db->getObject("Showcase", $id);
		}

		return $this->editObj($obj);
	}


	function save(){
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjShowcase();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
//				print_r($obj);
				$this->editObj($obj);
				return;
			}

			if($obj->id)
				$this->sys->db->putObject("Showcase", $obj);
			else
				$this->sys->db->appendObject("Showcase", $obj);
		}
		
		$page = "Showcase has been saved";
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		
		return $page;
	}

	function delete($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminGroup" . $this->group)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("Showcase", $id);

		$m = new BHtmlNode("div", "name='dialogQuestion'");
		$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=deleteDo&group={$this->group}&id={$id}' method='post'");
		$f->appendNode("h3", "", "Do you really want to delete showcase {$obj->title} and all of its pictures ?");

		$f->appendNode("input", "type='submit' name='arg' value='Delete Showcase'");
		$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

		$page = $f->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		
		return $page;
	}

	function deleteDo($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminGroup" . $this->group)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}

		$obj = $this->sys->db->getObject("Showcase", $id);

		// First delete all pictures
		$picList = $this->sys->db->getObjectList("Picture", "", "`category` = 'showcase/{$id}'", "");
		foreach($picList as $pic){
	//		printf("Delete Pic: {$pic->id}<br>\n");
			unlink($this->sys->baseDir . "/files/pictures/picture-{$pic->id}.jpg");
			unlink($this->sys->baseDir . "/files/pictures/picture-{$pic->id}-thumb.jpg");
			$this->sys->db->deleteObject("Picture", $pic->id);
		}

		// Now delete the showcase itself
		$this->sys->db->deleteObject("Showcase", $id);

		$page = "";
		$page .= "<h3>Showcase {$obj->title} and all pictures have been Deleted</h3>";	
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		return $page;
	}



	function editPictures($id){
		$page = "<h2>Edit Showcase Pictures</h2>";

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else if($id == 0){
			$obj = $this->sys->db->getNewObject("Showcase");
			$obj->date = strftime("%Y-%m-%d");
			$obj->dateAdded = strftime("%Y-%m-%d");
		}
		else {
			$obj = $this->sys->db->getObject("Showcase", $id);
		}

		$picList = $this->sys->db->getObjectList("Picture", "", "`category` = 'showcase/{$id}'", "id");

		$f = new BHtmlNode("form", "name='pictureEdit' enctype='multipart/form-data' action='?cmd=savePicture&group={$this->group}' method='post'");
		$t = &$f->appendNode("table", "class='BViewList' border='1' width='100%'");
		foreach($picList as $pic){
			$r = &$t->appendNode("tr");
			$c = &$r->appendNode("td", "", pictureHtml($pic->id, "", 1));
			$c = &$r->appendNode("td", "", "Good");

			$n = "good_{$pic->id}";
			if($pic->good)
				$c->appendNode("input", "type='checkbox' name='{$n}_checkbox' value='{$pic->good}' checked onchange='if(this.checked) document.pictureEdit.{$n}.value = 1; else document.pictureEdit.{$n}.value = 0;'");
			else
				$c->appendNode("input", "type='checkbox' name='{$n}_checkbox' value='{$pic->good}' onchange='if(this.checked) document.pictureEdit.{$n}.value = 1; else document.pictureEdit.{$n}.value = 0;'");
			$c->appendNode("input", "type='hidden' name='$n' value='{$pic->good}'");


			$c = &$r->appendNode("td", "", "Title");
			$c->appendNode("input", "type='text' name='title_{$pic->id}' title='Enter the title.'  value='{$pic->title}' size ='35'");
			$c = &$r->appendNode("td", "", "Description");
			$c->appendNode("input", "type='textarea' name='description_{$pic->id}' title='Enter the description.'  value='{$pic->description}' size ='35' style='width: 300px;height: 80px;'");
			$c = &$r->appendNode("td");
			$c->appendNode("a", "href='?cmd=deletePicture&group={$this->group}&id={$pic->id}'", "Delete");
		}
		$f->appendNode("input", "type='submit' name='cmd' value='Save'");
		$f->appendNode("button", "type='button' name='cmd' onclick='location.href = \"?cmd=viewList&group={$this->group}\";'", "Cancel");

		$page .= $f->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";

		return $page;
	}

	function savePicture(){
		$page = "<h2>Picture Update</h2>";
		foreach($_POST as $k => $v){
			$l = explode("_", $k);
			if(count($l) && ($l[0] == "title")){
	//			printf("Title: {$l[1]} : {$v}<br>\n");
				$obj = $this->sys->db->getObject("Picture", $l[1]);
				$obj->title = $v;
				$this->sys->db->putObject("Picture", $obj);
			}
			if(count($l) && ($l[0] == "description")){
	//			printf("Description: {$l[1]} : {$v}<br>\n");
				$obj = $this->sys->db->getObject("Picture", $l[1]);
				$obj->description = $v;
				$this->sys->db->putObject("Picture", $obj);
			}
			if(count($l) && ($l[0] == "good")){
	//			printf("Good: {$l[1]} : {$v}<br>\n");
				$obj = $this->sys->db->getObject("Picture", $l[1]);
				$obj->good = $v;
				$this->sys->db->putObject("Picture", $obj);
			}
		}

	//	print_r($_POST);

		$page .= "Pictures updated";
		return $page;
	}

	function deletePicture($id){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete")){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			return "";
		}

		$page = "<h2>Picture Delete</h2>";

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			unlink($this->sys->baseDir . "/files/pictures/picture-{$id}.jpg");
			unlink($this->sys->baseDir . "/files/pictures/picture-{$id}-thumb.jpg");
			$this->sys->db->deleteObject("Picture", $id);
			$page .= "<h3>Picture {$id} has been Deleted</h3>";	
			return $page;
		}
	}


	function addPicture($id){
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else {
			$obj = $this->sys->db->getObject("Showcase", $id);
		}

		$o = new BObjPicture();
		$o->MAX_FILE_SIZE = 250000000;
		$o->fileName = "";

		$f = new BEditObj("pictureUpload", "Picture Upload", "?cmd=uploadPicture&group={$this->group}&id={$id}", "");
		$f->setSaveButtonTitle("Upload Picture");
		$f->setView(
			array(new BViewType("id", "", "hidden"),
			new BViewType("MAX_FILE_SIZE", "", "hidden"),
			new BViewType("title", "", "text", "", 400, 0),
			new BViewType("description", "Description", "textara", "", 400, 80),
			new BViewType("fileName", "", "fileUploaderLine"),
			));
		$f->setObj($o);

		$page = $f->getHtml();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$id}\";</script>";
		return $page;
	}

	function uploadPicture($id){
		if(0){
			$s = "Testing: Showcase Post: " . print_r($_POST,1 );
			$s .= " Get: " . print_r($_GET, 1);
			$s .= " Files: " . print_r($_FILES, 1);
			$s .= "\n";
			debug_log($s);
		}

		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		else {
			$obj = $this->sys->db->getObject("Showcase", $id);
		}

		foreach($_FILES as $file){
			// Add picture
			$pobj = new BObjPicture();
			$pobj->category = "showcase/{$obj->id}";
			$pobj->group = $obj->group;
			$pobj->dateAdded = strftime("%Y-%m-%d");
			if(isset($_POST["title"]))
				$pobj->title = $_POST["title"];
			if(isset($_POST["description"]))
				$pobj->description = $_POST["description"];
			$pobj = $this->sys->db->appendObject("Picture", $pobj);

			$this->pictureSave($pobj->id, $file["tmp_name"]);
		}

		$page = "The picture has been added.";
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$id}\";</script>";
		return $page;
	}

	function addPictures($id){
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No Showcase selected"));
			return;
		}
		else {
			$obj = $this->sys->db->getObject("Showcase", $id);
		}

		$h = new BHtmlNode("div");
		$h->appendNode("script", "", "new PictureUploader('uploader', '/showcase?cmd=uploadPictures&group={$this->group}&id={$id}', 1024, 768);");

		$page = $h->render();
		$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$id}\";</script>";

		return $page;
	}
	
	function uploadPictures($id){
		if(0){
			$s = "Testing: Showcase::uploadPictures Post: " . print_r($_POST,1 );
			$s .= " Get: " . print_r($_GET, 1);
			$s .= " Files: " . print_r($_FILES, 1);
			$s .= "\n";
			debug_log($s);
		}

		if($id == "undefined"){
			print json_encode(array(error => "No Showcase selected"));
//			$this->sys->pageDisplayError(new BError(1, "No Showcase selected"));
			return Null;
		}
		else {
			$obj = $this->sys->db->getObject("Showcase", $id);
		}
		
		if((count($_FILES) != 1) || (count($_FILES["upload"]["name"]) != 1)){
			print json_encode(array(error => "Incorrect file upload structure"));
			return Null;
		}

		$file = $_FILES["upload"];
//		debug_log("uploadPictures: Do file: {$file["tmp_name"][0]}\n");

		// Add picture
		$pobj = new BObjPicture();
		$pobj->category = "showcase/{$obj->id}";
		$pobj->group = $obj->group;
		$pobj->dateAdded = strftime("%Y-%m-%d");
		if(isset($_POST["title"]))
			$pobj->title = filterString($_POST["title"]);
		if(isset($_POST["description"]))
			$pobj->description = filterString($_POST["description"]);
		$pobj = $this->sys->db->appendObject("Picture", $pobj);

//		debug_log("addshowcase: Id: {$pobj->id} Category: FileName: {$pobj->category} {$file["tmp_name"][0]}\n");
		$this->pictureSave($pobj->id, $file["tmp_name"][0]);

		print json_encode(array(error => "", status => "Uploaded ok"));
		return Null;
	}

	function pictureSave($id, $fileName){
		$size = 1024;
		$thumbSize = 128;
		$s = getimagesize($fileName);
		$aspect = (float)$s[0] / (float)$s[1];
		if($aspect > 1.0){
			$w = $size;
			$h = (int)((float)$w/$aspect);
			$tw = $thumbSize;
			$th = (int)((float)$tw/$aspect);
		}
		else {
			$h = $size;
			$w = (int)((float)$h * $aspect);
			$th = $thumbSize;
			$tw = (int)((float)$th * $aspect);
		}
		$image = imagecreatefromjpeg($fileName);
		$pic = imagecreatetruecolor($w, $h);
		$thumb = imagecreatetruecolor($tw, $th);
		imagecopyresized($pic, $image, 0, 0, 0, 0, $w, $h, $s[0], $s[1]);
		imagecopyresized($thumb, $image, 0, 0, 0, 0, $tw, $th, $s[0], $s[1]);
		imagejpeg($pic, $this->sys->baseDir . "/files/pictures/picture-{$id}.jpg");
		imagejpeg($thumb, $this->sys->baseDir . "/files/pictures/picture-{$id}-thumb.jpg");
	}
};

function showcaseGroup(){
	global	$sys;
	
	if(isset($_GET['group'])){
		$group = filterString($_GET['group']);
		$where = "`group`='{$group}'";
	}
	else{
		$group = "";
		$where = "";
	}
	$objList = $sys->db->getObjectList("Showcase", "", $where, "!date", "2");

	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/showcase?group={$group}'")->appendNode("h2", "", "Showcase");

	foreach($objList as $obj){
		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("a", "href='/showcase?cmd=view&id={$obj->id}'", pictureHtml(0, "showcase/{$obj->id}", 1));
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->date));
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/showcase?cmd=view&id={$obj->id}'", $obj->title);
		$d1->appendNode("div", "class='NewsSummaryContent'", $obj->description);
	}

	return $html->render();
}
?>
