<?php
/*******************************************************************************
 *	solar.php		Solar Module
 *				T.Barnaby,	BEAM Ltd,	2011-09-24
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";
require_once "BDate.php";

require_once "phplot-6.2.0/phplot.php";

global	$sys;
$sys->db->addObjectClasses(array("Solar" => "BObjSolar"));

class BObjSolar extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("DateTime", "DateTime"),
			new BMember("Inverter", "String", "", "", "varchar(10)"),
			new BMember("Serial", "String", "", "", "varchar(40)"),
			new BMember("CurrentPower", "Int"),
			new BMember("ETotalToday", "Float", "", "", "decimal(10,3)"),
			new BMember("PVOutput", "DateTime"),
			new BMember("CHANGETIME", "String", "", "", "timestamp", "default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP"),
			));
	}
	function dbTableFields(){
		return "(
			`id` int NOT NULL auto_increment,
			`DateTime` datetime NOT NULL,
			`Inverter` varchar(10) NOT NULL,
			`Serial` varchar(40) NOT NULL,
			`CurrentPower` int(11),
			`ETotalToday` decimal(10,3),
			`PVOutput` datetime,
			`CHANGETIME` timestamp default CURRENT_TIMESTAMP on update CURRENT_TIMESTAMP,
			PRIMARY KEY  (`id`)
			)";
	}
};

class Solar extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "solar";
		$this->title = "Solar PV";
		$this->info = "Solar PV Panel Generation";
	}
	
	// Standard interface
	function getMenu(){
		return new BMenu("Solar", "/{$this->name}", "", array(
			"Solar List" => "/{$this->name}",
			));
	}

	function getPage($uri){
		$this->parseUri($uri);

		switch($this->cmd){
		default: 			return $this->viewList();	break;
		}
	}

	function getPageBlock($uri){
		$o = new BBlock();
		return $o;
	}
	
	function plot($mon, $startDate, $endDate, $num, &$page){
		$dayPeak = 0.0;
		$dayAverage = 0.0;
		$dayTotal = 0.0;
		$total = 0;
		
		if($mon)
			$page .= "<h3>For Month: " . $startDate->get() . "<h3>";
		else
			$page .= "<h3>For day: " . $startDate->get() . "<h3>";
		$s = $startDate->getDateTime();
		$e = $endDate->getDateTime();
//		$page .= $s . " To " . $e . "<br>";
			
		$list = $this->sys->db->getObjectList("Solar", "", "`DateTime` >= '{$s}' and `DateTime` <= '{$e}'", "DateTime");

		// Add the data
		if(count($list)){
			$xdata = array();
			$ydata = array();
			foreach($list as $v){
				$date = new BDate($v->DateTime, "%Y-%m-%d %H:%M:%S");
				$xdata[] = $date->getSeconds();
				$ydata[] = $v->CurrentPower;

				if($v->CurrentPower > $dayPeak)
					$dayPeak = $v->CurrentPower;
				$dayAverage += $v->CurrentPower;
				$dayTotal += $v->CurrentPower;
			}
			$dayAverage /= count($list);
			$dayTotal /= 10;
			$dayTotal = (float)$list[count($list) - 1]->ETotalToday - (float)$list[0]->ETotalToday;
			$total = (float)$list[count($list) - 1]->ETotalToday;
		}
		else {
			$xdata = array();
			$ydata = array();
			for($x = 0; $x < 100; $x += 1){
				$xdata[] = "07:00:00";
				$xdata[] = $x;
				$ydata[] = 0;
			}
		}
		
//		print "Xdata: {$s}"; print_r($xdata); print "<br>";

		$plot = new PHPlot(800, 350);
		$plot->SetImageBorderType('plain');

		$data = array();
		for($x = 0; $x < count($xdata); $x += 1){
			$data[] = array('', $xdata[$x], $ydata[$x]);
		}
		$plot->SetPlotType('lines');
		$plot->SetDataType('data-data');
		$plot->SetDataValues($data);

		# Main plot title:
		$plot->SetTitle("Power supplied");

		if($mon){
			$plot->SetXLabelType('time', '%d');
			$plot->SetXTimeFormat('%d');
		}
		else {
			$plot->SetXLabelType('time', '%H:%M:%S');
			$plot->SetXTimeFormat('%H:%M:%S');
		}

		# Select a plot area and force ticks to nice values:
		$plot->SetPlotAreaWorld($startDate->getSeconds(), 0, $endDate->getSeconds(), 2600);

		# Draw both grids:
		$plot->SetDrawXGrid(True);
		$plot->SetDrawYGrid(True);

		$output_file = "{$this->sys->baseDir}/files/tmp/solar-" . $num . ".png";
		$plot->SetOutputFile($output_file);
		$plot->SetIsInline(1);
		$plot->DrawGraph();
		$page = $page . "<img src='{$this->sys->baseUrl}/files/tmp/solar-" . $num . ".png'></img>";

//		print "ImageFile: {$output_file}<br>";
	
		if($mon){
			$t = new BHtmlNode("table");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "MonthAverage");
			$r->appendNode("td", "", (int)$dayAverage . " W<br>");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "MonthPeak");
			$r->appendNode("td", "", (int)$dayPeak . " W<br>");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "MonthTotal");
			$r->appendNode("td", "", $dayTotal . " kW/H<br>");
			$page .= $t->render();
		}
		else {
			$t = new BHtmlNode("table");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "DayAverage");
			$r->appendNode("td", "", (int)$dayAverage . " W<br>");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "DayPeak");
			$r->appendNode("td", "", (int)$dayPeak . " W<br>");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "DayTotal");
			$r->appendNode("td", "", $dayTotal . " kW/H<br>");
			$r = &$t->appendNode("tr");
			$r->appendNode("th", "align=left", "Total from Installation");
			$r->appendNode("td", "", $total . " kW/H<br>");
			$page .= $t->render();
		}
		$page .= "<hr/>";
	}
	
	function viewList(){
		$page = "<h2>Solar PV Output: Kingsmead Cottage</h2>\n";

		$t = new BHtmlNode("table", "border=1");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align=left", "Panels");
		$r->appendNode("td", "", "14 x Sharp NU-185R1H");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align=left", "Inverter");
		$r->appendNode("td", "", "Sunnyboy HF 2500");
		$r = &$t->appendNode("tr");
		$r->appendNode("th", "align=left", "Installer");
		$r->appendNode("td", "", "Southern Solar");
		$page .= $t->render();
		
		$page .= "<br/>";

		// Output daily graphs
		$startDate = new BDate("now");
		for($d = 1; $d < 31; $d += 1){
			$startDate->setTime(7,0,0);
			$endDate = clone $startDate;
			$endDate->setTime(21,0,0);

			$this->plot(0, $startDate, $endDate, $d, $page);
			$startDate->timeStamp -= 86400;
		}

		// Output monthly graph
		$date = new BDate("now");
		$startDate = new BDate("now");
		$endDate = new BDate("now");
		
		$startDate->set(array($date->getYear(), $date->getMonth(), 1));
		$endDate->set(array($date->getYear(), $date->getMonth() + 1, 1));
		$this->plot(1, $startDate, $endDate, 0, $page);

		return $page;
	}
};

?>
