<?php
/*******************************************************************************
 *	Statistics.php	Statistics Module
 *			T.Barnaby,	BEAM Ltd,	2008-01-09
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;

class BObjStatistics extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("dateAdded", "DateTime", "date", array('format' => '%Y-%n-%j')),
			new BMember("uri", "String"),
			new BMember("fromIp", "String"),
			new BMember("fromHost", "String"),
			new BMember("fromPage", "String"),
			new BMember("userAgent", "String"),
			));
	}
};

class BObjStatisticsCounter extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("year", "Int", "Number"),
			new BMember("month", "Int", "Number"),
			new BMember("module", "String"),
			new BMember("item", "String"),
			new BMember("number", "Float"),
			));
	}
};

global	$searchEngines;
$searchEngines = array(
	'google' => 'http://www.google.com/bot.html',
	'msn' => 'http://search.msn.com/msnbot.htm', 
	'yahoo' => 'http://help.yahoo.com/help/us/ysearch/slurp', 
	'WebAlta' => 'http://www.webalta.net/ru/about_webmaster.html',
	'cuil' => 'http://www.cuil.com/twiceler/robot.html',
	'gigabot' => 'http://www.gigablast.com/spider.html',
	'yeti' => 'http://help.naver.com/robots'
);


class Statistics extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "statistics";
		$this->title = "Statistics";
		$this->info = "Statistics module";

		// Add Objects
		$this->sys->db->addObjectClasses(array("Statistics" => "BObjStatistics"));
		$this->sys->db->addObjectClasses(array("StatisticsCounters" => "BObjStatisticsCounter"));
	}
	
	function checkAccess(){
		if($this->sys->login->checkLogin("admin")){
			return 1;
		}
		else {
			return 0;
		}		
	}
	
	// Admin Interface
	function getMenu(){
		return new BMenu("Statistics", "/{$this->name}", "", array("Statistics List" => "/{$this->name}",
			 "Statistics List Unique" => "/{$this->name}?cmd=viewUnique",
			 "Statistics List All" => "/{$this->name}?cmd=viewListAll"));
	}

	function getPage($uri){
		$this->parseUri($uri);

		switch($this->cmd){
		case "viewUnique":		return $this->viewList(2);		break;
		case "viewListAll":		return $this->viewList(1);		break;
		case "clean":			return $this->clean();			break;
		case "viewCounters":		return $this->counterView();		break;
		default: 			return $this->viewList();		break;
		}
	}

	function getPageBlock($uri){
		return new BMenu("Statistics", "", "", array(
			new BMenu("AwStats", "", "href='/awstats/awstats.pl'"),
			new BMenu("List", "", "href='?cmd=viewList'"),
			new BMenu("ListUnique", "", "href='?cmd=viewUnique'"),
			new BMenu("List All", "", "href='?cmd=viewListAll'"),
			new BMenu("Clean List", "", "href='?cmd=clean'"),
			new BMenu("List Counters", "", "href='?cmd=viewCounters'"),
			));
	}

	function clean(){
		$page = "<h2>Statistics Clean List</h2>\n";
		$sql = "delete from {$this->sys->databasePrefix}Statistics where DATE_SUB(CURDATE(),INTERVAL 30 DAY) > dateAdded";
		$this->sys->db->query($sql);

		return $page;
	}

	function viewList($all = 0){
		if($all == 1){
			$page = "<h2>Statistics List All</h2>\n";

			$view = new BViewDatabase("stats");
	//		$view->setView(array(
	//			new BViewType("name"),
	//			new BViewType("enabled"),
	//			new BViewType("emailAddress", "Email Address"),
	//			new BViewType("telephoneHome", "Home Telephone"),
	//			new BViewType("roles")));

			if($view->order == "")
				$view->setOrder("!id");

			$view->setTable("Statistics");

			$page = $page . $view->getHtml();
		}
		else if($all == 2){
			$page = "<h2>Statistics List All</h2>\n";

//			BDatabase::$debug = 1;
			$list1 = $this->sys->db->getObjectList("Statistics", "distinct `fromIp`", "", "!dateAdded");
			$list = array();
			foreach($list1 as $i){
				$list2 = $this->sys->db->getObjectList("Statistics", "*", "`fromIp` = '{$i->fromIp}'", "!dateAdded", "1");
				$list[] = $list2[0];
			}

			$view = new BViewList("stats");
	//		$view->setView(array(
	//			new BViewType("name"),
	//			new BViewType("enabled"),
	//			new BViewType("emailAddress", "Email Address"),
	//			new BViewType("telephoneHome", "Home Telephone"),
	//			new BViewType("roles")));

			$view->setOrder("!dateAdded");
			$view->setList($list);

			$page = $page . $view->getHtml();
		}
		else {
			$page = "<h2>Statistics</h2>\n";

			$num = $this->sys->db->getObjectNumber("Statistics");
			$t = new BHtmlNode("table", "border=1 class='BViewList'");
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", "Total Number of page accesses");
			$r->appendNode("td", "", $num);

			$page .= $t->render();
		}

		$page .= "<div class='help'>";
		$page .= "This lists statistics of web accesses from the system. ";
		$page .= "In the left hand bar there are links to the functions you can perform.";
		$page .= "</div>";
		
		return $page;
	}
	
	function counterView(){
		$date = new BDate("now");
		
		$page = "<h2>Statistics View Counters for year</h2>\n";

		$view = new BViewDatabase("StatisticsCounters");
		$view->setOrder("year,month,module,item");
		$view->setSelection("`year`='{$date->getYear()}'");

		$view->setTable("StatisticsCounters");

		$page = $page . $view->getHtml();

		$page .= "<div class='help'>";
		$page .= "This lists statistics of all counters for the last year. ";
		$page .= "</div>";
		
		return $page;
	}

	function counterSet($year, $month, $module, $item, $number){
		$sel = "`year`='{$year}' and `month`='{$month}' and `module`='{$module}' and `item`='{$item}'";

		$obj = $this->sys->db->getObjectFirst("StatisticsCounters", "", $sel);
		if($obj == Null){
			$obj = $this->sys->db->getNewObject("StatisticsCounters");
			$obj->year = $year;
			$obj->month = $month;
			$obj->module = $module;
			$obj->item = $item;
			$obj->number = $number;
			$this->sys->db->appendObject("StatisticsCounters", $obj);
		}
		else {
			$obj->number = floatval($number);
			$this->sys->db->putObject("StatisticsCounters", $obj);
		}
	}

	function counterSetNow($module, $item, $number){
		$date = new BDate("now");
		$this->counterSet($date->getYear(), $date->getMonth(), $module, $item, $number);
	}

	function counterAdd($year, $month, $module, $item, $number){
		$sel = "`year`='{$year}' and `month`='{$month}' and `module`='{$module}' and `item`='{$item}'";

		$obj = $this->sys->db->getObjectFirst("StatisticsCounters", "", $sel);
		if($obj == Null){
			$obj = $this->sys->db->getNewObject("StatisticsCounters");
			$obj->year = $year;
			$obj->month = $month;
			$obj->module = $module;
			$obj->item = $item;
			$obj->number = $number;
			$this->sys->db->appendObject("StatisticsCounters", $obj);
		}
		else {
			$obj->number += floatval($number);
			$this->sys->db->putObject("StatisticsCounters", $obj);
		}
	}

	function counterAddNow($module, $item, $number){
		$date = new BDate("now");
		$this->counterAdd($date->getYear(), $date->getMonth(), $module, $item, $number);
	}

	function counterGetAll($year, $month, $module, $item){
		$sel = "`year` like '{$year}' and `month` like '{$month}' and `module` like '{$module}' and `item` like '{$item}'";
//		$order = "module,item,year,month";
		$order = "!year,!month,module,item";
		return $this->sys->db->getObjectList("StatisticsCounters", "", $sel, $order);
	}
};

function statisticsAdd(){
	global	$sys;
	global	$searchEngines;

	// Ignore search engines 
	$agent = strtolower($_SERVER['HTTP_USER_AGENT']);
	foreach($searchEngines as $name => $searchEngine){
		if(stripos($agent, $searchEngine) !== false)
			return;
	}

	$obj = $sys->db->getNewObject("Statistics");
	
//	printf("NewObjId: %d\n", {$obj->id});
	
	$obj->dateAdded = strftime("%Y-%m-%dT%H:%M:%S");
        if(isset($_SERVER["HTTP_REFERER"])){
		$obj->fromPage = $_SERVER["HTTP_REFERER"];
        }

        $obj->uri = $_SERVER['REQUEST_URI'];
        $obj->fromHost = gethostbyaddr($_SERVER['REMOTE_ADDR']);
	$obj->fromIp = $_SERVER["REMOTE_ADDR"];
	$obj->userAgent = $_SERVER["HTTP_USER_AGENT"];

        // Add an entry into the database
	$sys->db->appendObject("Statistics", $obj);
}

function statisticsClean(){
	global	$sys;
	
	$sql = "delete from {$sys->databasePrefix}Statistics where DATE_SUB(CURDATE(),INTERVAL 90 DAY) > dateAdded";
	$sys->db->query($sql);
}

?>
