<?php
/*******************************************************************************
 *	testForm.php		Tests form input module
 *				T.Barnaby,	BEAM Ltd,	2016-02-09
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BEdit.php";

class BObjTestForm extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Int"),
			new BMember("dateAdded", "DateTime", "Datetime"),
			new BMember("bool", "Bool"),
			new BMember("int", "Int"),
			new BMember("float", "Float"),
			new BMember("date", "Date"),
			new BMember("dateTime", "DateTime"),
			new BMember("string", "String"),
			new BMember("stringHtml", "StringHtml"),
		));
	}
};


class TestForm extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "testForm";
		$this->title = "TestForm";
		$this->info = "TestForm Module";
		
		// Add Objects
		$this->sys->db->addObjectClasses(array("TestForm" => "BObjTestForm"));
	}
	
	// Standard interface
	function getMenu(){
		return new BMenu("TestForm", "/{$this->name}", "", array(
			"Demo List" => "/{$this->name}",
			));
	}

	function getPage($uri){
		$page = "";
		
		$this->parseUri($uri);

		switch($this->cmd){
		case "view":		$page = $this->view($uri); break;
		case "edit":		$page = $this->edit($uri); break;
		case "save":		$page = $this->save($uri); break;
		default: 		$page = $this->viewList($uri); break;
		}
		
		return $page;
	}

	function getPageBlock($uri){
		$o = new BMenu("TestForm", "", "", array(
			new BMenu("List", "",	"href='?cmd=viewList'"),
			new BMenu("View", "",	"href='?cmd=view'"),
			new BMenu("Edit", "",	"href='?cmd=edit'"),
			new BMenu("Delete", "",	"href='?cmd=delete'"),
			));
		return $o;
	}
	
	function viewList(){
		$page = "<h2>TestForm viewList</h2>\n";
		
//		print "Form: "; print_r($_POST); print "<br>\n";
//		BDatabase::$debug = 1;

		$view = new BViewDatabase("list", "action");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("dateAdded", "DateAdded", "datetime"),
			new BViewType("bool", "Bool"),
			new BViewType("int", "Int"),
			new BViewType("float", "Float"),
			new BViewType("date", "Date"),
			new BViewType("dateTime", "DateTime"),
			new BViewType("string", "String"),
			new BViewType("stringHtml", "StringHtml"),
			));

		$view->setOrder("!dateAdded");
		$view->setTable("TestForm");
		$page = $page . $view->getHtml();
		
		return $page;
	}
	
	function view($arg){
		$page = "<h2>View TestForm</h2>";

		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			$obj = $this->sys->db->getObject("TestForm", filterInt($_GET["id"]));
		}
		else {
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		
		$view = new BViewObj("testForm", "TestForm");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("dateAdded", "DateAdded", "datetime"),
			new BViewType("bool", "Bool"),
			new BViewType("int", "Int"),
			new BViewType("float", "Float"),
			new BViewType("date", "Date"),
			new BViewType("dateTime", "DateTime"),
			new BViewType("string", "String"),
			new BViewType("stringHtml", "StringHtml"),
			));

		$view->setObj($obj);
		$page = $page . $view->getHtml();
		return $page;
	}

	function edit($arg, $payDo = 0){
		$args = "";
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"]) &&  is_numeric($_GET["id"])){
			if(filterInt($_GET["id"]) == 0)
				return "No record id 0.";
			$page = "<h2>Edit TestForm</h2>";
			$obj = $this->sys->db->getObject("TestForm", filterInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("TestForm");
			$obj->id = 0;
			$obj->dateAdded = strftime("%Y-%m-%d %H:%M:%S");
		}
		
		$view = new BEditObj("testForm", "TestForm", "?cmd=save", "");
		$view->setView(array(
			new BViewType("id"),
			new BViewType("dateAdded", "DateAdded", "datetime"),
			new BViewType("bool", "Bool", "checkbox"),
			new BViewType("int", "Int"),
			new BViewType("float", "Float"),
			new BViewType("date", "Date", "date"),
			new BViewType("dateTime", "DateTime", "string"),
			new BViewType("string", "String", "textarea", "", 300, 80),
			new BViewType("stringHtml", "StringHtml", "textareaHtml", "", 400, 200),
//			new BViewType("stringHtml", "StringHtml", "textareaHtmlFull", "", 400, 200),
			));

		$view->setObj($obj);
		$page .= $view->getHtml();
		return $page;
	}
	
	function save(){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjTestForm();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			if($obj->id){
				$this->sys->db->putObject("TestForm", $obj);
			}
			else {
				$this->sys->db->appendObject("TestForm", $obj);
			}

			$page .= "<h3>Details saved</h3>";
		}

		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";

		return $page;
	}

	function delete(){
		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("TestForm", $id);
			$page = "<h3>TestForm: {$id} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("TestForm", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete formtest: {$obj->id} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
};

?>
