<?php
/*******************************************************************************
 *	Vwt.php	VWT Module
 *			T.Barnaby,	BEAM Ltd,	2014-11-16
 *******************************************************************************
 *
 *	
 */
require_once "BModule.php";
require_once "BObj.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

class BObjVwtQueue extends BObj {
	function __construct(){
		$m = array();

		$m[] = new BMember("id", "Int", "Number", "", "int", "NOT NULL auto_increment");
		$m[] = new BMember("date", "DateTime");
		$m[] = new BMember("userId", "Int");
		$m[] = new BMember("group", "String");
		$m[] = new BMember("title", "String");
		$m[] = new BMember("public", "Bool");
		$m[] = new BMember("description", "StringHtml");
		$m[] = new BMember("status", "String");
		$m[] = new BMember("resolution", "Int");
		$m[] = new BMember("height", "Float");
		$m[] = new BMember("speed", "Float");
		$m[] = new BMember("scale", "Float");
		$m[] = new BMember("rotateX", "Float");
		$m[] = new BMember("rotateY", "Float");
		$m[] = new BMember("rotateZ", "Float");
		$m[] = new BMember("fileName", "String");
		$m[] = new BMember("error", "String");
		$m[] = new BMember("version", "String");
		$m[] = new BMember("area", "Float");
		$m[] = new BMember("cd", "Float");
		$m[] = new BMember("cd_area", "Float");
		$m[] = new BMember("results", "StringHtml");
		$m[] = new BMember("processingTime", "Int");

		parent::__construct($m);
	}
};

class Vwt extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "vwt";
		$this->title = "VWT";
		$this->info = "VWT module";

		// Add Objects
		$sys->db->addObjectClasses(array("VwtQueue" => "BObjVwtQueue"));
		
		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminVwt"));
	}
	
	function configs(){
		$configs = array();
//		$configs[] = new BObjConfig("nextMembershipNumber", "Number", "", "Next membership number", "Next membership number");
		return $configs;
	}

	function checkAccess(){
		return $this->sys->login->checkLogin();
	}
	
	function checkAccessAdmin(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminVwt");
	}
	
	function checkFileAccess($uri){
		if(!strncmp($uri, "vwt/cases/", 10)){
			return 0;
		}
		if(!strcmp($uri, "vwt")){
			return 0;
		}
		
		return 1;
	}

	
	// Admin Interface
	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("VWT", "/{$this->name}", "", array(
				"List" => "/{$this->name}",
				));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
//		print "Vwt::getPage: '{$uri}'<br/>"; return "";

		// Check if file access
		if($this->checkFileAccess($uri)){
			$module = $this->sys->modules["content"];
			return $module->getPage($uri);
		}

		$this->parseUri($uri);

		if($this->checkAccessAdmin()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "edit1":			return $this->edit1($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "delete":			return $this->delete($uri);		break;
			case "help":			return $this->help($uri);		break;
			case "fix":			return $this->fix($uri);		break;
			default: 			return $this->viewDefault($uri);	break;
			}
		}
		else if($this->checkAccess()){
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "add":			return $this->edit($uri);		break;
			case "edit":			return $this->edit($uri);		break;
			case "save":			return $this->save($uri);		break;
			case "help":			return $this->help($uri);		break;
			default: 			return $this->viewDefault($uri);	break;
			}
		}
		else {
			// Non admin users
			switch($this->cmd){
			case "view":			return $this->view($uri);		break;
			case "help":			return $this->help($uri);		break;
			default: 			return $this->viewDefault($uri);	break;
			}
		}
	}

	function getPageBlock($uri){
		// Check if file access
		if($this->checkFileAccess($uri)){
			$module = $this->sys->modules["content"];
			return $module->getPageBlock($uri);
		}

		if($this->checkAccess()){
			$o = new BMenu("VWT", "", "", array(
				new BMenu("List", 	"",	"href='/vwt'"),
				new BMenu("Add new Simulation",	"",	"href='?cmd=add'"),
				new BMenu("Help", "",		"href='/vwt/vwtHelp.html'"),
				));
		}
		else {
			$o = new BMenu("VWT", "", "", array(
				new BMenu("List", 	"","href='/vwt'"),
				new BMenu("Help", "",		"href='/vwt/vwtHelp.html'"),
				));
		}
		return $o;
	}

	function caseFileView($path){
//		print "fileView: {$path}<br>";
		
		// Get id
		$data = sscanf($path,"vwt/cases/%d/%s");
		if(count($data) != 2){
			$this->sys->pageDisplayError(new BError(1, "Unknown file path."));
			exit();
		}
		$id = $data[0];
		$file = $data[1];
		
//		print "Id: {$id} File: {$file}<br/>";
//		return "";

		$obj = $this->sys->db->getObject("VwtQueue", $id);

		// Validate view
		if(!$this->checkAccess() && ($obj->public == 0)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to view this item."));
			exit();
		}

		if(!$this->checkAccessAdmin()){
			if(($obj->public == 0) && ($obj->userId != $this->sys->user->id) && ($obj->group != $this->sys->user->group)){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to view this item."));
				exit();
			}
		}

		// Return file
		$module = $this->sys->modules["content"];
		return $module->viewFile("../process/cases/{$id}/{$file}");
	}
	
	function wakeServer(){
		system("wol -h 192.168.201.255 00:25:B3:E6:A9:18 > /dev/null");
	}

	function viewDefault($uri){
		if(!strncmp($uri, "vwt/cases/", 10)){
			// Return case file
			header("Cache-Control: no-cache, no-store, must-revalidate");
			return $this->caseFileView($uri);
		}
		if(strcmp($uri, "vwt")){
			// Return file
			$module = $this->sys->modules["content"];
			return $module->viewFile(filePathLocal($uri));
		}
		
		$listSize = 40;
		$where = "";

		$sel = new BSelect("select", "Select", "", 3);
		$sel->setView(array(
			new BViewTypeSelect("public", "", "", "equal", "tristate"),
			new BViewTypeSelect("group", "", "Group", "like", "select", array_merge(array(""), $this->groupsList())),
			new BViewTypeSelect("userId", "", "Mine", "userId", "tristate"),
			new BViewTypeSelect("pager", "", "Page", "pager", "pager"),
			new BViewTypeSelect("order", "", "", "order", "select", array("!date", "cd_area", "cd", "area")),
			));

		$sel->pagerSet(0, $listSize, 0);
		$sel->getObj()->order = "!date";
		
		if($this->checkAccessAdmin()){
			$page = "<h2>VWT Processing Runs for all</h2>\n";
		}
		else {
			if($this->checkAccess()){
				$page = "<h2>VWT Processing Runs for {$this->sys->user->name} of group {$this->sys->user->group}</h2>\n";
				$where .= "(`public` = '1' or `userId` = '{$this->sys->user->id}' or `group` = '{$this->sys->user->group}')";
				$sel->getObj()->mine = 1;
			}
			else {
				$page = "<h2>VWT Processing Runs for all public</h2>\n";
				$where .= "(`public` = '1')";
			}
		}
		$sel->postUpdate();

		if($sel->getSelectString() != ""){
			if($where != "")
				$where .= " and ";
			
			$where .= $sel->getSelectString();
		}
		
//		print "Select: {$where}  Order: {$sel->getOrder()}<br>";
//		BDatabase::$debug = 1;
		
		$totalQueued = $this->sys->db->getObjectNumber("VwtQueue", "`status` = 'queued'");
		$totalNumber = $this->sys->db->getObjectNumber("VwtQueue", $where);
		$sel->pagerSet(0, $listSize, $totalNumber);
		
		$h = new BHtmlNode("table", "class=Vwt border=1");
		$r = &$h->appendNode("tr");
		$r->appendNode("th", "", "Icon");
		if($this->checkAccess())
			$r->appendNode("th", "", "Action");
		$r->appendNode("th", "", "Id");
		$r->appendNode("th", "", "Date");
		$r->appendNode("th", "", "Status");
		$r->appendNode("th", "", "Public");
		$r->appendNode("th", "", "Who/Team");
		$r->appendNode("th", "", "Title");
		$r->appendNode("th", "", "Info");
		$r->appendNode("th", "", "Results");
		$r->appendNode("th", "", "Links");

		$list = $this->sys->db->getObjectList("VwtQueue", "", $where, $sel->getOrder(), $sel->pagerGetLimitString());
		foreach($list as $obj){
			$r = &$h->appendNode("tr");
			
			$icon = "/vwt/cases/{$obj->id}/object_icon.png";
			$r->appendNode("td")->appendNode("img", "src='{$icon}'");
			
			if($this->checkAccess()){
				$c = &$r->AppendNode("td");
				$c->appendNode("a", "href='?cmd=view&id={$obj->id}' title='View Item'", "V");
				$c->appendText("&nbsp");
				$c->appendNode("a", "href='?cmd=edit&id={$obj->id}' title='Edit Item'", "E");
				$c->appendText("&nbsp");
				$c->appendNode("a", "href='?cmd=delete&id={$obj->id}' title='Delete Item'", "D");
			}

			$r->appendNode("td", "", $obj->id);
			$datetime = explode(" ", $obj->date);
			$r->appendNode("td", "", $datetime[0] . "<br>" . $datetime[1]);
			$r->appendNode("td", "", $obj->status);
			$r->appendNode("td", "", $obj->public);
			$r->appendNode("td", "", $this->sys->getUserInfo($obj->userId)->name . "<br/>" . $obj->group);
			$r->appendNode("td", "style='max-width: 200px'")->appendNode("a", "href=?cmd=view&id={$obj->id}", $obj->title);
			
			$str = "File: {$obj->fileName}<br/>\n";
			$str .= "Version: {$obj->version}<br/>\n";
			$str .= "Resolution: {$obj->resolution}<br/>\n";
			$str .= "Height: {$obj->height}<br/>\n";
			$str .= "Speed: {$obj->speed}<br/>\n";
			$str .= "Rotate: X: {$obj->rotateX} Y: {$obj->rotateY} Z: {$obj->rotateZ} <br/>\n";
			$r->appendNode("td", "style='max-width: 200px'", $str);
			
			if($obj->status == "error"){
				$r->appendNode("td", "", $obj->error);
			}
			else if($obj->status == "finished"){
				$str = "Cd x Area: {$obj->cd_area}<br/>\n";
				$str .= "Cd: {$obj->cd}<br/>\n";
				$str .= "Area: {$obj->area}<br/>\n";
				$str .= str_replace("\n", "<br/>\n", $obj->results);
				$r->appendNode("td", "", $str);
			}
			else {
				$r->appendNode("td", "", "");
			}
			$links = &$r->appendNode("td");
			$links->appendNode("a", "href='/vwt/cases/{$obj->id}/file.stl'", "Car Design STL<br>");
			$links->appendNode("a", "href='/vwt/cases/{$obj->id}/log.txt'", "Processing log<br>");
			$links->appendNode("a", "href='/vwt/cases/{$obj->id}/forces.csv'", "Forces<br>");
			$links->appendNode("a", "href='/vwt/cases/{$obj->id}/forces.png'", "Forces Graph<br>");
			$links->appendNode("a", "href='/vwt/cases/{$obj->id}/vtk.zip'", "Results vtk.zip<br>");
		}
		$page .= $sel->getHtml();
		$page .= "Total queued: {$totalQueued}<br/>\n";
		$page .= $h->render();

#		$page .= "<div class='help'>";
#		$page .= "This lists all of the users of the system. ";
#		$page .= "In the left hand bar there are links to the functions you can perform.";
#		$page .= "</div>";

		return $page;
	}

	function edit($uri){
		if(is_object($uri)){
			$page = "<h2>Edit VWT Simulation</h2>";
			$obj = $uri;
		}
		else if(isset($_GET["id"]) && (filterToInt($_GET["id"]) != 0)){
			$page = "<h2>Edit VWT Simulation</h2>";
			$obj = $this->sys->db->getObject("VwtQueue", filterToInt($_GET["id"]));
		}
		else {
			$page = "<h2>Queue new VWT simulation</h2>";
			$obj = $this->sys->db->getNewObject("VwtQueue");
			$obj->date = strftime("%Y-%m-%d %H:%M:%S");
			$obj->userId = $this->sys->user->id;
			$obj->group = $this->sys->user->group;
			$obj->resolution = 1000000;
			$obj->height = 0.02;
			$obj->speed = 46.0;
			$obj->scale = 1.0;
			$obj->rotateX = 0.0;
			$obj->rotateY = 0.0;
			$obj->rotateZ = 0.0;
			$obj->status = "added";
		}
		
		// Validate edit
		if(!$this->checkAccess()){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to edit this item."));
			exit();
		}

		if(!$this->checkAccessAdmin()){
			if($obj->userId != $this->sys->user->id){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to edit this item."));
				exit();
			}
		}
		
		if(isset($_POST["upload"]) || isset($_POST["change"]) || isset($_POST["process"])){
			$obj = new BObjVwtQueue();
			$obj->set($_POST);
			
			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}
		
			if(isset($_POST["upload"])){
				// Validate that the file is a .stl file
				if(isset($_FILES["fileName"]) and ($_FILES["fileName"]["name"] != "")){
					$pi = pathinfo($_FILES["fileName"]["name"]);
					if(strtolower($pi['extension']) != "stl"){
						$this->sys->status = "<span class='editError'>Error the design file must be a *.stl file</span>";
						return $this->edit($obj);
					}
					$obj->fileName = $_FILES["fileName"]["name"];
				}
				else {
					print "Error";
					unset($_POST["upload"]);
					$this->sys->status = "<span class='editError'>Error you need to provide a *.stl design file</span>";
					return $this->edit($obj);
				}

				$obj->status = "uploaded";
				$obj->cd_area = "";
				$obj->area = "";
				$obj->cd = "";
				$obj->results = "";
				$obj->version = "";
				$obj->error = "";

				if($obj->id)
					$this->sys->db->putObject("VwtQueue", $obj);
				else
					$this->sys->db->appendObject("VwtQueue", $obj);

				// Create the case directory and move the STL file there
				$dir =  $this->sys->baseDir . "/files/cases/{$obj->id}";
//				$page .= "Directory: {$dir}<br>";
				if(!file_exists($dir)){
					if(!mkdir($dir, 0777))
						$page .= "Create directory Error";

					chmod($dir, 0777);
				}

				if(!move_uploaded_file($_FILES["fileName"]["tmp_name"], $dir . "/file.stl")){
					$page .= "Move Error";
				}
				
				system("(cd /data/vwt/process; export DISPLAY=:1; ./vwtProcess -u {$obj->id}) > /dev/null");
			}
			else if(isset($_POST["change"])){
				$objOrig = $this->sys->db->getObject("VwtQueue", $obj->id);
				if(($obj->resolution != $objOrig->resolution) || ($obj->height != $objOrig->height) || ($obj->speed != $objOrig->speed) ||
					($obj->scale != $objOrig->scale) || 
					($obj->rotateX != $objOrig->rotateX) || ($obj->rotateY != $objOrig->rotateY) || ($obj->rotateZ != $objOrig->rotateZ)){
					$obj->status = "uploaded";
					$obj->cd_area = "";
					$obj->area = "";
					$obj->cd = "";
					$obj->results = "";
					$obj->version = "";
					$obj->error = "";
				}

				if($obj->id)
					$this->sys->db->putObject("VwtQueue", $obj);
				else
					$this->sys->db->appendObject("VwtQueue", $obj);

				system("(cd /data/vwt/process; export DISPLAY=:1; ./vwtProcess -u {$obj->id}) > /dev/null");
			}
			else if(isset($_POST["process"])){
//				print "Process<br>";

				$obj->status = "queued";
				$obj->cd_area = "";
				$obj->area = "";
				$obj->cd = "";
				$obj->results = "";
				$obj->version = "";
				$obj->error = "";

				if($obj->id)
					$this->sys->db->putObject("VwtQueue", $obj);
				else
					$this->sys->db->appendObject("VwtQueue", $obj);
				
				$page = "Simulation has been queued";
				$page .= "<button type='button' name='cancel' style='background-color: lightgreen; font-size: 150%;' onclick='location.href = \"/vwt\";'>Go to List</button>";
				$this->wakeServer();
				return $page;
			}
		}
		
		$f = new BHtmlNode("form", "id='{$this->name}' name='{$this->name}' enctype='multipart/form-data' action='?cmd=edit&id={$obj->id}' method='post'");
		$f->appendNode("input", "type='hidden' name='id' value='{$obj->id}'");
		$f->appendNode("input", "type='hidden' name='date' value='{$obj->date}'");
		$f->appendNode("input", "type='hidden' name='userId' value='{$obj->userId}'");
		$f->appendNode("input", "type='hidden' name='group' value='{$obj->group}'");
		$f->appendNode("input", "type='hidden' name='version' value='{$obj->version}'");
		$f->appendNode("input", "type='hidden' name='cd_area' value='{$obj->cd_area}'");
		$f->appendNode("input", "type='hidden' name='cd' value='{$obj->cd}'");
		$f->appendNode("input", "type='hidden' name='area' value='{$obj->area}'");
		$f->appendNode("input", "type='hidden' name='results' value='{$obj->results}'");
		$f->appendNode("input", "type='hidden' name='processingTime' value='{$obj->processingTime}'");

		$t = &$f->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1' width='100%'");
		if($obj->id)
			$t->appendTableInput("Status", "status", "select", $obj->status, "", array("added","uploaded","queued","running","error","finished"));
		else
			$f->appendNode("input", "type='hidden' name='status' value='{$obj->status}'");
		$t->appendTableInput("Title", "title", "text", $obj->title);
		$t->appendTableInput("FileName (*.stl)", "fileName", "file", $obj->fileName);
		$t->appendTableInput("Public", "public", "checkbox", $obj->public);
		$t->appendTableInput("Resolution", "resolution", "select", $obj->resolution, "", array("100000", "200000", "500000", "1000000","2000000"));
		$t->appendTableInput("Height", "height", "text", $obj->height);
		$t->appendTableInput("Speed", "speed", "speed", $obj->speed);
		$t->appendTableInput("Scale", "scale", "text", $obj->scale);
		$t->appendTableInput("RotateX", "rotateX", "text", $obj->rotateX);
		$t->appendTableInput("RotateY", "rotateY", "text", $obj->rotateY);
		$t->appendTableInput("RotateZ", "rotateZ", "text", $obj->rotateZ);
		$t->appendTableInput("Description", "description", "textareaHtml", $obj->description, "", array("height"=>"100px"));

		$f->appendNode("input", "type='submit' name='upload' style='background-color: lightgreen; font-size: 150%;' value='Upload file' onclick='this.value=\"Uploading, please wait...\";'");
		$f->appendNode("input", "type='submit' name='change' style='background-color: lightgreen; font-size: 150%;' value='Change parameters' onclick='this.value=\"Changing, please wait...\";'");
		$f->appendNode("input", "type='submit' name='process' style='background-color: lightgreen; font-size: 150%;' value='Process in VWT' onclick='this.value=\"Adding to queue, please wait...\";'");
		$f->appendNode("button", "type='button' name='cancel' style='background-color: lightgreen; font-size: 150%;' onclick='location.href = \"/\";'", "Cancel");

		$info = <<< End_Of_Text
			Fill in the title and select a suitable STL format file in the FileName fields.
			Optional fields are:<br>
			<ul>
			<li>Public: Make this simulation public for others to see</li>
			<li>Height: This is the height in meters of the very bottom of the design (wheels ?) from the road surface</li>
			<li>Speed: This is the VWT windspeed in km/H</li>
			<li>Scale: Scale model by factor</li>
			<li>RotateX: Rotate model about X-Axis by given number of degrees</li>
			<li>RotateY: Rotate model about Y-Axis by given number of degrees</li>
			<li>RotateZ: Rotate model about Z-Axis by given number of degrees</li>
			<li>Description: Model/Simulation description</li>
			</ul>
			Once done click on the <b>Upload file</b> button to upload the file.<br>
			Check the orientation of the model and change parameters as needed using the <b>Change parameters</b> button.<br>
			Then click on the <b>Process in VWT</b> button to queue the model for VWT processing.<br>
			See the help information at: <a href='/files/vwt/vwtHelp.html'>vwtHelp.html</a> for more information.<br>
End_Of_Text;
		
		$h = new BHtmlNode("table");
		$r = &$h->appendNode("tr");
		$r->appendNode("td", "", $f->render());
		$c = &$r->appendNode("td", "valign=top");
		$c->appendNode("div", "class=Block", $info);
		$icon = "/vwt/cases/{$obj->id}/object.png";
		$c->appendNode("div", "class=Block")->appendNode("img", "src='{$icon}'");;
		
		$page = $page . $h->render();
		
		return $page;
	}

	
	function edit1($uri){
		if(is_object($uri)){
			$page = "<h2>Edit VWT Simulation</h2>";
			$obj = $uri;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit VWT Simulation</h2>";
			$obj = $this->sys->db->getObject("VwtQueue", filterToInt($_GET["id"]));
		}
		else {
			$page = "<h2>Queue new VWT simulation</h2>";
			$obj = $this->sys->db->getNewObject("VwtQueue");
			$obj->date = strftime("%Y-%m-%d %H:%M:%S");
			$obj->userId = $this->sys->user->id;
			$obj->group = $this->sys->user->group;
			$obj->resolution = 1000000;
			$obj->height = 0.02;
			$obj->speed = 46.0;
			$obj->scale = 1.0;
			$obj->rotateX = 0.0;
			$obj->rotateY = 0.0;
			$obj->rotateZ = 0.0;
			$obj->status = "queued";
		}
		
		// Validate edit
		if(!$this->checkAccess()){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to edit this item."));
			exit();
		}

		if(!$this->checkAccessAdmin()){
			if($obj->userId != $this->sys->user->id){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to edit this item."));
				exit();
			}
		}

		$view = new BEditObj("vwt", "VWT Simulation", "?cmd=save", "");
		if($obj->id){
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("date", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("group", "", "hidden"),
				new BViewType("status", "Status", "select", array("queued","running","error","finished")),
				new BViewType("title", "Title"),
				new BViewType("fileName", "FileName (*.stl)", "file"),
				new BViewType("public", "Public", "checkbox"),
				new BViewType("resolution", "Resolution", "select", array("100000", "200000", "500000", "1000000","2000000")),
				new BViewType("height", "Height"),
				new BViewType("speed", "Speed"),
				new BViewType("scale", "Scale"),
				new BViewType("rotateX", "RotateX"),
				new BViewType("rotateY", "RotateY"),
				new BViewType("rotateZ", "RotateZ"),
				new BViewType("description", "Description", "textareaHtml", "", 400, 150),
				new BViewType("version", "", "hidden"),
				new BViewType("cd_area", "", "hidden"),
				new BViewType("area", "", "hidden"),
				new BViewType("cd", "", "hidden"),
				new BViewType("results", "", "hidden"),
				new BViewType("processingTime", "", "hidden"),
				));
		}
		else {
			$view->setView(array(
				new BViewType("id", "", "hidden"),
				new BViewType("date", "", "hidden"),
				new BViewType("userId", "", "hidden"),
				new BViewType("group", "", "hidden"),
				new BViewType("status", "", "hidden"),
				new BViewType("title", "Title"),
				new BViewType("fileName", "FileName (*.stl)", "file"),
				new BViewType("public", "Public", "checkbox"),
				new BViewType("resolution", "Resolution", "select", array("100000", "200000", "500000", "1000000","2000000")),
				new BViewType("height", "Height (m)"),
				new BViewType("speed", "Speed (km/H)"),
				new BViewType("scale", "Scale (number)"),
				new BViewType("rotateX", "RotateX (degrees)"),
				new BViewType("rotateY", "RotateY (degrees)"),
				new BViewType("rotateZ", "RotateZ (degrees)"),
				new BViewType("description", "Description", "textareaHtml", "", 400, 150),
				new BViewType("version", "", "hidden"),
				new BViewType("cd_area", "", "hidden"),
				new BViewType("area", "", "hidden"),
				new BViewType("cd", "", "hidden"),
				new BViewType("results", "", "hidden"),
				new BViewType("processingTime", "", "hidden"),
				));
		}

		$view->setHelp("title", "Enter the title.");
		$view->setHelp("resolution", "Enter the resolution by selecting one from the list.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		
		$info = <<< End_Of_Text
			Fill in the title and select a suitable STL format file in the FileName fields.
			Once done click on the <b>Save</b> button to upload the file and start the simulation.<br>
			Optional fields are:<br>
			<ul>
			<li>Public: Make this simulation public for others to see</li>
			<li>Height: This is the height in meters of the very bottom of the design (wheels ?) from the road surface</li>
			<li>Speed: This is the VWT windspeed in km/H</li>
			<li>Scale: Scale model by factor</li>
			<li>RotateX: Rotate model about X-Axis by given number of degrees</li>
			<li>RotateY: Rotate model about Y-Axis by given number of degrees</li>
			<li>RotateZ: Rotate model about Z-Axis by given number of degrees</li>
			<li>Description: Model/Simulation description</li>
			</ul>
			See the help information at: <a href='/files/vwt/vwtHelp.html'>vwtHelp.html</a> for more information.<br>
			If editing an entry the status can be set to 'queued' to re-run the simulation.
End_Of_Text;
		
		$h = new BHtmlNode("table");
		$r = &$h->appendNode("tr");
		$r->appendNode("td", "", $view->getHtml());
		$r->appendNode("td", "valign=top")->appendNode("div", "class=Block", $info);
		
		$page = $page . $h->render();
		
		return $page;
	}

	function save($uri){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjVwtQueue();
			$obj->set($_POST);
			
			if($obj->id)
				$objOrig = $this->sys->db->getObject("VwtQueue", $obj->id);


			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			// Validate that the file is a .stl file
			if(isset($_FILES["fileName"]) and ($_FILES["fileName"]["name"] != "")){
				$pi = pathinfo($_FILES["fileName"]["name"]);
				if(strtolower($pi['extension']) != "stl"){
					$this->sys->status = "<span class='editError'>Error the design file must be a *.stl file</span>";
					return $this->edit($obj);
				}
				$obj->fileName = $_FILES["fileName"]["name"];
			}
			else if($obj->id == 0){
				$this->sys->status = "<span class='editError'>Error you need to provide a *.stl design file</span>";
				return $this->edit($obj);
			}

			if($obj->id && ($obj->fileName == "")){
				$obj->fileName = $objOrig->fileName;
			}
			
			if($obj->status == "queued"){
				$obj->area = "";
				$obj->cd = "";
				$obj->results = "";
				$obj->version = "";
				$obj->error = "";
			}
			
			if($obj->id)
				$this->sys->db->putObject("VwtQueue", $obj);
			else
				$this->sys->db->appendObject("VwtQueue", $obj);

			// Create the case directory and move the STL file there
			$dir =  $this->sys->baseDir . "/files/cases/{$obj->id}";
//			$page .= "Directory: {$dir}<br>";
			if(!file_exists($dir)){
				if(!mkdir($dir, 0777))
					$page .= "Create directory Error";

				chmod($dir, 0777);
			}
				
			if(isset($_FILES["fileName"]) and ($_FILES["fileName"]["name"] != "")){
//				$obj->fileNameOriginal = $_FILES["fileName"]["name"];
//				$pi = pathinfo($obj->fileNameOriginal);
//				$obj->type = strtolower($pi['extension']);

				if(!move_uploaded_file($_FILES["fileName"]["tmp_name"], $dir . "/file.stl")){
					$page .= "Move Error";
				}
			}
			
			$this->wakeServer();
		}

		$page .= "<h3>VwtQuery details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete($uri){
		// Validate delete
		if(!$this->checkAccess()){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete this item."));
			exit();
		}

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		$obj = $this->sys->db->getObject("VwtQueue", $id);

		if(!$this->checkAccessAdmin()){
			if($obj->userId != $this->sys->user->id){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete this item."));
				exit();
			}
		}


		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$this->sys->db->deleteObject("VwtQueue", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
			$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		}
		else {

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&id={$id}&doIt=1' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
		}
			
		return $page;
	}

	function view($uri){
		if($uri == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$id = filterToInt($_GET["id"]);
			$obj = $this->sys->db->getObject("VwtQueue", $id);
		}

		// Validate view
		if(!$this->checkAccess() && ($obj->public == 0)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to view this item."));
			exit();
		}

		if(!$this->checkAccessAdmin()){
			if(($obj->public == 0) && ($obj->userId != $this->sys->user->id) && ($obj->group != $this->sys->user->group)){
				$this->sys->pageDisplayError(new BError(1, "You are not authorised to view this item."));
				exit();
			}
		}

		$page = "<h2>VWT Simulation view</h2>";

		$view = new BViewObj("vwt");
		$view->setView(array(
			new BViewType("id", "ID"),
			new BViewType("date", "Date", "date"),
			new BViewType("userId", "UserId"),
			new BViewType("group", "Group"),
			new BViewType("title", "Title"),
			new BViewType("status", "Status"),
			new BViewType("version", "Version"),
			new BViewType("resolution", "Resolution"),
			new BViewType("height", "Height"),
			new BViewType("speed", "Speed"),
			new BViewType("scale", "Scale"),
			new BViewType("rotateX", "RotateX"),
			new BViewType("rotateY", "RotateY"),
			new BViewType("rotateZ", "RotateZ"),
			new BViewType("fileName", "FileName"),
			new BViewType("description", "Description"),
			new BViewType("error", ""),
			new BViewType("version", ""),
			new BViewType("area", ""),
			new BViewType("cd", ""),
			new BViewType("results", ""),
			new BViewType("processingTime", ""),
			));
		$view->setObj($obj);
		$page = $page . $view->getHtml();

		return $page;
	}
	
	function help($uri){
		return $this->sys->modules["content"]->getPage("files/vwt/vwtHelp.html");

//		$help = $this->sys->baseDir . "/modules/vwt/info.html";
//		return file_get_contents($help);
	}
	
	function fix($uri){
		$page = "Fix up";
		
		$list = $this->sys->db->getObjectList("VwtQueue", "", "", "", "");
		foreach($list as $obj){
			$obj->cd_area = floatval($obj->cd) * floatval($obj->area);
			$this->sys->db->putObject("VwtQueue", $obj);
		}

		return $page;
	}
	
	function groupsList(){
		$groups = array();
		$list = $this->sys->db->getObjectList("Groups", "", "", "name", "");

		foreach($list as $g){
			$groups[$g->name] = $g->name;
//			$groups[$g->name] = $g->title;
		}
		return $groups;
	}
};

?>
