<?php
/*******************************************************************************
 *	WhatsOn.php	WhatsOn Module
 *			T.Barnaby,	BEAM Ltd,	2011-12-07
 *******************************************************************************
 */
require_once "BModule.php";
require_once "BBlock.php";
require_once "BView.php";
require_once "BDate.php";
require_once "BEdit.php";
require_once "BDate.php";

global	$sys;
$sys->db->addObjectClasses(array("WhatsOn" => "BObjWhatsOn"));

function groupGetTitle($name){
	global	$sys;

	$list = $sys->db->getObjectList("Groups", "title", "`name`='" . $name . "'", "", "");
	if(count($list))
		return $list[0]->title;
	else
		return "";
}

function removeParagraphs($str){
	 $str = preg_replace('/<p*>/', ' ', $str); 
	 $str = preg_replace('/<\/p*>/', ' ', $str);
	 $str = preg_replace('/ map /', ' ', $str);
	 return $str;
}

function removeMapText($str){
	 $str = preg_replace('/ map /', ' ', $str);
	 return $str;
}

class BObjWhatsOn extends BObj {
	function __construct(){
		parent::__construct(array(
			new BMember("id", "Int", "Number"),
			new BMember("startDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("endDate", "Date", "date", array('format' => '%Y-%n-%j')),
			new BMember("group", "String"),
			new BMember("startTime", "String"),
			new BMember("startPlace", "String"),
//			new BMember("rideLength", "Int", "number", array("min" => "0", "max" => "400")),
			new BMember("rideLength", "Int", "string", array("format" => "0-9")),
			new BMember("rideEase", "String"),
			new BMember("description", "StringHtml"),
			new BMember("who", "String"),
			new BMember("tel", "String"),
			new BMember("web", "String"),
			new BMember("startLocation", "String"),
			new BMember("gpxFile", "String"),
			new BMember("email", "String"),
			new BMember("addedDate", "Date"),
			new BMember("addedUserName", "String"),
			));
	}
};

class Map {
	function __construct(){
	}	

	function render($width, $height, $location){
//		$src = "http://maps.google.co.uk/maps?hl=en&ie=UTF8&ll=51.523271,-2.356224&spn=0.085125,0.228481&t=m&z=13&vpsrc=6&output=embed";
		$src = "http://maps.google.co.uk/maps?hl=en&ie=UTF8&ll={$location}&t=m&z=13&vpsrc=6&output=embed";

		$h = new BHtmlNode("div");
		$h->appendNode("iframe", "width='{$width}' height='{$height}' frameborder='0' scrolling='no' marginheight='0' marginwidth='0' src='{$str}'");
		$h->appendNode("small")->appendNode("a", "href='$src' style='color:#0000FF;text-align:left'", "View Larger Map");
		return $h->render();
	}
}

class WhatsOn extends BModule {
	// Basic Interface routines
	function __construct(&$sys){
		parent::__construct($sys);
		$this->name = "whatsOn";
		$this->title = "WhatsOn";
		$this->info = "WhatsOn module. This module allows the management of WhatsOn.";
		$this->group = "";
		
		if(isset($_GET['group'])){
			$this->group = filterString($_GET['group']);
			$this->groupTitle = groupGetTitle($this->group);
		}

		// Add HTML functions
		$this->sys->htmlFuncs[] = "whatsOnList";
		
		// Add roles
		$this->sys->modules["user"]->appendRoles(array("adminWhatsOn"));
	}

	function checkAccess(){
		return $this->sys->login->checkLogin("admin") || $this->sys->login->checkLogin("adminWhatsOn") || $this->sys->login->checkLogin("adminGroup" . $this->group);
	}

	function getMenu(){
		if($this->checkAccess()){
			return new BMenu("WhatsOn", "/{$this->name}", "", array("WhatsOn List" => "/{$this->name}", "WhatsOn Add" => "/{$this->name}?cmd=infoAdd", "Booking form" => "/tc/modules/events/bookingForm.php"));
		}
		else {
			return Null;
		}
	}

	function getPage($uri){
		$this->parseUri($uri);

		$page = "";
		
		switch($this->cmd){
		case "view":	$page = $this->view($uri); break;
		case "edit":	$page = $this->edit($uri); break;
		case "save":	$page = $this->save($uri); break;
		case "delete":	$page = $this->delete($uri); break;
		case "magList":	$page = $this->viewByDate($uri, 1); break;
		case "magList2":	$page = $this->viewByDateMag($uri); break;
		case "magList3":	$page = $this->viewByDateMag3($uri); break;
		default:	$page = $this->viewList($uri); break;
		}

		return $page;
	}

	function getPageBlock($uri){
		if($this->checkAccess()){
			$help = $this->sys->baseUrl . "/modules/events/info.html";
			
			$o = new BMenu("WhatsOn", "", "", array(
				new BMenu("List By Date", "", "href='?cmd=viewList&order=startDate,group'"),
				new BMenu("List By Group", "", "href='?cmd=viewList&order=group,startDate'"),
				new BMenu("List By ID", "", "href='?cmd=viewList&order=id'"),
				new BMenu("Journal List", "", "href='?cmd=magList'"),
				new BMenu("Journal List2", "", "href='?cmd=magList2'"),
				new BMenu("Journal List3", "", "href='?cmd=magList3'"),
				new BMenu("", "", ""),
				new BMenu("Event Add", "",	"href='?cmd=edit&group={$this->group}'"),
				new BMenu("", "", ""),
				new BMenu("Help", "",		"href='{$help}'"),
				));
		}
		else {
			$o = new BMenu("WhatsOn", "", "", array(
				new BMenu("List By Date", "", "href='?cmd=viewList&order=startDate,group'"),
				new BMenu("List By Group", "", "href='?cmd=viewList&order=group,startDate'"),
				new BMenu("Journal List", "", "href='?cmd=magList'"),
				new BMenu("Journal List2", "", "href='?cmd=magList2'"),
				new BMenu("Journal List3", "", "href='?cmd=magList3'"),
				));
		}
		return $o;
	}
	
	function viewList($arg){
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->groupTitle) . "'" . "and `startDate` >= CURDATE()";
		else
			$where = "`startDate` >= CURDATE()";

		if(isset($_GET["order"]))
			$order = filterStringParamList($_GET["order"]);
		else
			$order = "startDate";
		
		if($this->checkAccess()){
			$view = new BViewDatabase("list", "action");
			$view->setGroup($this->group);
			$view->setView(array(
				new BViewType("id"),
				new BViewType("group"),
				new BViewType("startDate"),
				new BViewType("rideLength"),
				new BViewType("rideEase"),
				new BViewType("who"),
				));

			$view->setSelection($where);
			if($view->order == "")
				$view->setOrder($order);

			$view->setTable("WhatsOn");
			
			$page = "<h2>Tandem Club Whats On</h2>";
			$page .= $view->getHtml();
		}
		else {
			if(preg_match("/^group.*/", $order)){
				$page = $this->viewByGroup($arg);
			}
			else {
				$page = $this->viewByDate($arg);
			}
		}
		return $page;
	}
	
	function viewByDate($arg, $mag = 0){
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->groupTitle) . "'" . "and `startDate` >= CURDATE()";
		else
			$where = "`startDate` >= CURDATE()";

		$order = "startDate";
		
		$p = new BHtmlNode("div");
		$p->appendNode("h2", "", "Tandem Club Whats On");
		$p->appendNode("p", "", "Regional organisers and other administrators can add entries to the What's On system.");
		$p->appendNode("p", "", "If you participate in any ride organised by or in the name of The Tandem Club, you do so entirely at your own risk. The Club, its officers and ride organisers accept no responsibility for any loss, damage to property, accident, injury or death that may occur, however caused, whether by negligence, deliberate act or otherwise. These rides are organised for members of the Tandem Club, please join the club.");

		if($mag)
			$t = &$p->appendNode("table", "class='BView' cellpadding='0' cellspacing='0'  border='1' bordercolor='#107296'");
		else
			$t = &$p->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$curDate = new BDate("2000-01-01");
		$list = $this->sys->db->getObjectList("WhatsOn", "", $where, $order, "");

		foreach($list as $event){
			$sd = new BDate($event->startDate);
			$ed = new BDate($event->endDate);
			if($sd > $curDate){
				$str = $sd->getDateTime("%A %e %B %Y");
				if($mag)
					$t->appendNode("tr")->appendNode("th", "colspan='2' style='text-align:left;background-color:#107296;color:white;'", $str);
				else
					$t->appendNode("tr")->appendNode("th", "colspan='2' class='heading'", $str);
				$curDate = $sd;
			}
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $event->group);

			$str = "";			
			if($event->endDate && ($event->endDate > $event->startDate))
				$str .= "Until " . $ed->getDateTime("%A %e %B %Y") . ". ";

			if($event->startPlace)
				$str .= "Starting from {$event->startPlace} ";
			if($event->startTime){
				if($event->startPlace)
					$str .= "at {$event->startTime}";
				else
					$str .= "At {$event->startTime}";
			}
			$str .= ". ";

			if($event->rideLength)
				$str .= "{$event->rideLength} miles";
			if($event->rideEase){
				if($event->rideLength)
					$str .= ", ";
				if($mag){
					switch($event->rideEase){
					case "Easy":	$str .= "<span style='color:red'>E</span>";	break;
					case "Average":	$str .= "<span style='color:red'>Av</span>";	break;
					case "Hard":	$str .= "<span style='color:red'>H</span>";	break;
					}
				}
				else {
					$str .= "{$event->rideEase} ";
				}
			}
			if($event->rideLength || $event->rideEase)
				$str .= ". ";

			if($event->description)
				$str .= removeParagraphs("{$event->description} ");

			$s = "";
			if($event->who)
				$s .= "{$event->who}";
			if($event->tel){
				if($s) $s .= ", ";
				if(!$mag) $s .= "Tel: ";
				$s .= "<b>{$event->tel}</b>";
			}
			if($event->email){
				if($s) $s .= ", ";
				if(!$mag) $s .= "Email: ";
				$s .= "<a href='mailto:{$event->email}'>{$event->email}</a>";
			}
			if($event->web){
				if($s) $s .= ", ";
				if(!$mag) $s .= "Web: ";
				if(strncmp($event->web, "http://", 7))
					$s .= "<a href='http://{$event->web}'>{$event->web}</a>";
				else
					$s .= "<a href='{$event->web}'>{$event->web}</a>";
			}
			if($s) $s .= ", ";
			$s .= "ID: {$event->id}";
			$str .= $s;

			if(0){
				$map = new Map();
				$str .= $map->render(200, 150, "51.523271,-2.356224");
			}
			$r->appendNode("td", "", $str);

		}
		$page = $p->render();
		return $page;
	}

	function viewByDateMag($arg){
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->groupTitle) . "'" . "and `startDate` >= CURDATE()";
		else
			$where = "`startDate` >= CURDATE()";

		$order = "startDate";
		
		$p = new BHtmlNode("div");
		$p->appendNode("h2", "", "Tandem Club Whats On");
		$p->appendNode("p", "", "Regional organisers and other administrators can add entries to the What's On system.");
		$p->appendNode("p", "", "If you participate in any ride organised by or in the name of The Tandem Club, you do so entirely at your own risk. The Club, its officers and ride organisers accept no responsibility for any loss, damage to property, accident, injury or death that may occur, however caused, whether by negligence, deliberate act or otherwise. These rides are organised for members of the Tandem Club, please join the club.");

		$t = &$p->appendNode("table", "class='BView' cellpadding='0' cellspacing='0'  border='1' bordercolor='#107296'");

		$curDate = new BDate("2000-01-01");
		$list = $this->sys->db->getObjectList("WhatsOn", "", $where, $order, "");

		foreach($list as $event){
			$sd = new BDate($event->startDate);
			$ed = new BDate($event->endDate);
			if($sd > $curDate){
				$str = $sd->getDateTime("%A %e %B %Y");
				$t->appendNode("tr")->appendNode("th", "colspan='2' style='text-align:left;background-color:#107296;color:white;'", $str);
				$curDate = $sd;
			}
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "style='white-space:nowrap;'", $event->group);

			$str = "";			
			if($event->endDate && ($event->endDate > $event->startDate))
				$str .= "Until " . $ed->getDateTime("%A %e %B %Y") . ". ";

			if($event->startPlace)
				$str .= "Start: {$event->startPlace} ";
			if($event->startTime){
				if($event->startPlace)
					$str .= "at {$event->startTime}";
				else
					$str .= "At {$event->startTime}";
			}
			$str .= ". ";

			if($event->rideLength)
				$str .= "{$event->rideLength} miles";
			if($event->rideEase){
				if($event->rideLength)
					$str .= ", ";

				switch($event->rideEase){
				case "Easy":	$str .= "<span style='color:green'>E</span>";	break;
				case "Average":	$str .= "<span style='color:orange'>A</span>";	break;
				case "Hard":	$str .= "<span style='color:red'>H</span>";	break;
				}
			}
			if($event->rideLength || $event->rideEase)
				$str .= ". ";

			if($event->description)
				$str .= removeMapText(htmlToText(removeParagraphs("{$event->description} ")));

			$s = "";
			if($event->who)
				$s .= "<b>{$event->who}</b>";
			if($event->tel){
				if($s) $s .= ", ";
				$s .= "<b>{$event->tel}</b>";
			}
			if($event->email){
				if($s) $s .= ", ";
				$s .= "<a href='mailto:{$event->email}'>{$event->email}</a>";
			}
			if($event->web){
				if($s) $s .= ", ";
				if(strncmp($event->web, "http://", 7))
					$s .= "<a href='http://{$event->web}'>{$event->web}</a>";
				else
					$s .= "<a href='{$event->web}'>{$event->web}</a>";
			}

			$str .= $s;

			if(0){
				$map = new Map();
				$str .= $map->render(200, 150, "51.523271,-2.356224");
			}
			$r->appendNode("td", "", $str);

		}
		$page = $p->render();
		return $page;
	}

	function viewByDateMag3($arg){
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->groupTitle) . "'" . "and `startDate` >= CURDATE()";
		else
			$where = "`startDate` >= CURDATE()";

		$order = "startDate";
		
		$p = new BHtmlNode("div");
		$p->appendNode("h2", "", "Tandem Club Whats On");
		$p->appendNode("p", "", "Regional organisers and other administrators can add entries to the What's On system.");
		$p->appendNode("p", "", "If you participate in any ride organised by or in the name of The Tandem Club, you do so entirely at your own risk. The Club, its officers and ride organisers accept no responsibility for any loss, damage to property, accident, injury or death that may occur, however caused, whether by negligence, deliberate act or otherwise. These rides are organised for members of the Tandem Club, please join the club.");

		$t = &$p->appendNode("table", "class='BView' cellpadding='0' cellspacing='0'  border='1' bordercolor='#107296'");

		$curDate = new BDate("2000-01-01");
		$list = $this->sys->db->getObjectList("WhatsOn", "", $where, $order, "");

		foreach($list as $event){
			$sd = new BDate($event->startDate);
			$ed = new BDate($event->endDate);
			if($sd > $curDate){
				$str = $sd->getDateTime("%A %e %B %Y");
				$t->appendNode("tr")->appendNode("th", "colspan='2' style='text-align:left;background-color:#107296;color:white;'", $str);
				$curDate = $sd;
			}
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "style='white-space:nowrap;'", $event->group);

			$str = "";			
			if($event->endDate && ($event->endDate > $event->startDate))
				$str .= "Until " . $ed->getDateTime("%A %e %B %Y") . ". ";

			if($event->startPlace)
				$str .= "Start: {$event->startPlace} ";
			if($event->startTime){
				if($event->startPlace)
					$str .= "at {$event->startTime}";
				else
					$str .= "At {$event->startTime}";
			}
			$str .= ". ";

			if($event->rideLength)
				$str .= "{$event->rideLength} miles";
			if($event->rideEase){
				if($event->rideLength)
					$str .= ", ";

				switch($event->rideEase){
				case "Easy":	$str .= "<span style='color:green'>E</span>";	break;
				case "Average":	$str .= "<span style='color:orange'>A</span>";	break;
				case "Hard":	$str .= "<span style='color:red'>H</span>";	break;
				}
			}
			if($event->rideLength || $event->rideEase)
				$str .= ". ";

			if($event->description)
				$str .= removeMapText(htmlToText(removeParagraphs("{$event->description} ")));

			$s = "";
			if($event->who)
				$s .= "<b>{$event->who}</b>";
			if($event->tel){
				if($s) $s .= ", ";
				$s .= "<b>{$event->tel}</b>";
			}
			if($event->email){
				if($s) $s .= ", ";
				$s .= "<a href='mailto:{$event->email}'>{$event->email}</a>";
			}
			if($event->web){
				if($s) $s .= ", ";
				if(strncmp($event->web, "http://", 7))
					$s .= "<a href='http://{$event->web}'>{$event->web}</a>";
				else
					$s .= "<a href='{$event->web}'>{$event->web}</a>";
			}

			$str .= $s;

			if(0){
				$map = new Map();
				$str .= $map->render(200, 150, "51.523271,-2.356224");
			}
			$r->appendNode("td", "", $str);

		}
		$page = $p->render();
		
		header("Content-type: application/vnd.ms-word");
		header("Content-Disposition: attachment; Filename=JournalWhatsOn.doc");
		print $page;

		return Null;
	}

	function viewByGroup($arg){
		if($this->group)
			$where = "`group`='" . dbEscapeString($this->groupTitle) . "'" . "and `startDate` >= CURDATE()";
		else
			$where = "`startDate` >= CURDATE()";

		$order = "group,startDate";
		
		$p = new BHtmlNode("div");
		$p->appendNode("h2", "", "Tandem Club Whats On");
		$p->appendNode("p", "", "Regional organisers and other administartors can add entries to the What's On system.");
		$p->appendNode("p", "", "If you participate in any ride organised by or in the name of The Tandem Club, you do so entirely at your own risk. The Club, its officers and ride organisers accept no responsibility for any loss, damage to property, accident, injury or death that may occur, however caused, whether by negligence, deliberate act or otherwise.");

		$t = &$p->appendNode("table", "class='BView' cellpadding='2' cellspacing='2' border='1'");

		$curDate = new BDate("2000-01-01");
		$curGroup = "";
		$list = $this->sys->db->getObjectList("WhatsOn", "", $where, $order, "");

		foreach($list as $event){
			$sd = new BDate($event->startDate);
			$ed = new BDate($event->endDate);
			if($event->group != $curGroup){
				$t->appendNode("tr")->appendNode("th", "colspan='2'  class='heading'", $event->group);
				$curGroup = $event->group;
			}
			$r = &$t->appendNode("tr");
			$r->appendNode("td", "", $sd->getDateTime("%A %e %B %Y"));

			$str = "";			
			if($event->endDate && ($event->endDate > $event->startDate))
				$str .= "Until " . $ed->getDateTime("%A %e %B %Y") . ". ";

			if($event->startPlace)
				$str .= "Starting from {$event->startPlace} ";
			if($event->startTime){
				if($event->startPlace)
					$str .= "at {$event->startTime}";
				else
					$str .= "At {$event->startTime}";
			}
			$str .= ". ";

			if($event->rideLength)
				$str .= "{$event->rideLength} miles ";
			if($event->rideEase)
				$str .= "{$event->rideEase} ";
			if($event->rideLength || $event->rideEase)
				$str .= ". ";

			if($event->description)
				$str .= removeParagraphs("{$event->description}. ");
			
			$s = "";
			if($event->who)
				$s .= "{$event->who}";
			if($event->tel){
				if($s) $s .= ", ";
				$s .= "Tel: <b>{$event->tel}</b>";
			}
			if($event->email){
				if($s) $s .= ", ";
				$s .= "Email: <a href='mailto:{$event->email}'>{$event->email}</a>";
			}
			if($event->web){
				if($s) $s .= ", ";
				if(strncmp($event->web, "http://", 7))
					$s .= "<a href='http://{$event->web}'>{$event->web}</a>";
				else
					$s .= "<a href='{$event->web}'>{$event->web}</a>";
			}
			if($s) $s .= ", ";
			$s .= "ID: {$event->id}";
			$str .= $s;

			if(0){
				$map = new Map();
				$str .= $map->render(200, 150, "51.523271,-2.356224");
			}

			$r->appendNode("td", "", $str);
		}
		$page = $p->render();
		return $page;
	}

	function view($arg){
		$id = filterToInt($_GET["id"]);
		if($id == "undefined"){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return "";
		}
		else {
			$event = $this->sys->db->getObject("WhatsOn", $id);
		}

		$str = "<b>{$event->group}</b>:";


		$sd = new BDate($event->startDate);
		$ed = new BDate($event->endDate);
		$str .= " " . $sd->getDateTime("%A %e %B %Y");
		if($event->endDate)
			$str .= " to " . $ed->getDateTime("%A %e %B %Y");
		$str .= ".";
		
		if($event->startPlace)
			$str .= " Starting from {$event->startPlace}";
		if($event->startTime)
			if($event->startPlace)
				$str .= " at {$event->startTime}";
			else
				$str .= "At {$event->startTime}";
		if($event->startPlace || $event->startTime)
			$str .= ".";

		if($event->rideLength)
			$str .= " {$event->rideLength} miles";
		if($event->rideEase)
			$str .= " {$event->rideEase}";
		if($event->rideLength || $event->rideEase)
			$str .= ".";

		$str .= " {$event->description} ";

		$s = "";
		if($event->who)
			$s .= "{$event->who}";
		if($event->tel){
			if($s) $s .= ", ";
			$s .= "Tel: <b>{$event->tel}</b>";
		}
		if($event->email){
			if($s) $s .= ", ";
			$s .= "Email: <a href='mailto:{$event->email}'>{$event->email}</a>";
		}
		if($event->web){
			if($s) $s .= ", ";
			$s .= "Web: <a href='{$event->web}'>{$event->web}</a>";
		}
		if($s) $s .= ", ";
		$s .= "ID: {$event->id}";
		$str .= $s;

		return $str;
	}
	
	function edit($arg){
		if(is_object($arg)){
			$obj = $arg;
		}
		else if(isset($_GET["id"])){
			$page = "<h2>Edit Event</h2>";
			$obj = $this->sys->db->getObject("WhatsOn", filterToInt($_GET["id"]));
		}
		else {
			$obj = $this->sys->db->getNewObject("WhatsOn");
		}
		
		if($this->group){
			$groups = array($this->groupTitle);
		}
		else {
//			$groups = categoryGetList("TcGroups");
			$groups = $this->groupsList();
		}

		$view = new BEditObj("event", "Event Information", "/whatsOn?cmd=save", "");
		$view->setView(array(
			new BViewType("id", "", "hidden"),
			new BViewType("group", "Group", "select", $groups),
			new BViewType("startDate", "StartDate", "date"),
			new BViewType("endDate", "EndDate (If over multiple days)", "date"),
			new BViewType("startTime", "StartTime (09:00)"),
			new BViewType("startPlace", "StartPlace"),
			new BViewType("rideLength", "RideLength (Miles)"),
			new BViewType("rideEase", "RideEase", "select", array("", "Easy", "Average", "Hard")),
			new BViewType("description", "Description", "textareaHtml", "", 500, 280),
			new BViewType("who", "Contact (Name/names)"),
			new BViewType("tel", "Telephone"),
			new BViewType("email", "Email"),
			new BViewType("web", "Web"),
			new BViewType("addedDate", "", "hidden"),
			new BViewType("addedUserName", "", "hidden"),
			));

		$view->setHelp("group", "Enter the group.");
		$view->setHelp("description", "Enter a description. This can be multiple lines long and contain web addresses etc.");

		$view->setObj($obj);
		return $view->getHtml();
	}
	
	function save(){
		$page = "";
	
		if(count($_POST)){
			unset($_POST['cmd']);
			$obj = new BObjWhatsOn();
			$obj->set($_POST);

			if($err = $obj->validate()){
				$items = implode(",", array_keys($err));
				$this->sys->status = "<span class='editError'>Error in form data in fields: $items</span>";
				return $this->edit($obj);
			}

			if($obj->id){
				$this->sys->db->putObject("WhatsOn", $obj);
			}
			else {
				$this->sys->db->appendObject("WhatsOn", $obj);
			}
		}

		$page .= "<h3>Details saved</h3>";
		$page .= "<button type='button' onclick='location.href = \"javascript:history.go(-2)\"'>Back to list</button>";
		return $page;
	}

	function delete(){
		// Check role
		if(!$this->sys->login->checkLogin("adminDelete") && !$this->sys->login->checkLogin("adminGroup" . $this->group)){
			$this->sys->pageDisplayError(new BError(1, "You are not authorised to delete items. Only users with the 'adminDelete' role can perform this."));
			exit();
		}
		if($this->group)
			$groupCmd = "&group={$this->group}";
		else
			$groupCmd = "";

		if(!isset($_GET["id"])){
			$this->sys->pageDisplayError(new BError(1, "No object selected"));
			return;
		}
		$id = filterToInt($_GET["id"]);
		
		if(isset($_GET["doIt"]) && $_GET["doIt"] == "1"){
			$obj = $this->sys->db->getObject("WhatsOn", $id);
			$this->sys->db->deleteObject("WhatsOn", $id);
			$page = "<h3>{$obj->title} has been Deleted</h3>";
		}
		else {
			$obj = $this->sys->db->getObject("WhatsOn", $id);

			$m = new BHtmlNode("div", "name='dialogQuestion'");
			$f = &$m->appendNode("form", "name='dialogQuestion' action='?cmd=delete&id={$id}&doIt=1{$groupCmd}' method='post'");
			$f->appendNode("h3", "", "Do you really want to delete {$obj->title} ?");

			$f->appendNode("input", "type='submit' name='arg' value='Delete'");
			$f->appendNode("button", "type='button' name='arg' onclick='location.href = \"javascript:history.go(-1)\";'", "Cancel");

			$page = $f->render();
			$page .= "<script language='JavaScript'>document.ws.list = {}; document.ws.list.objId = \"{$obj->id}\";</script>";
		}

		return $page;
	}
	
	function groupsList(){
		$groups = array();
		$list = $this->sys->db->getObjectList("Groups", "", "", "title", "");

		foreach($list as $g){
			$groups[$g->name] = $g->title;
//			$groups[$g->name] = $g->name;
		}
		return $groups;
	}
};

function whatsOnList(){
	global	$sys;
	
	if(isset($_GET['group'])){
		$group = filterString($_GET['group']);
		$groupTitle = groupGetTitle($group);
		$where = "`group`='{$groupTitle}'";
		$where .= " and `startDate` >= CURDATE()";
	}
	else {
		$group = "";
		$groupTitle = "";
		$where = "`startDate` >= CURDATE()";
	}

	$objList = $sys->db->getObjectList("WhatsOn", "", $where, "startDate", "3");

	$html = new BHtmlNode("div", "id='news'");
	$html->appendNode("a", "href='{$sys->baseUrl}/whatsOn?group={$group}'")->appendNode("h2", "", "What's On");

	foreach($objList as $obj){
		$title = "";
		if($obj->startPlace)
			$title .= "Starting from: " . $obj->startPlace;
		if($obj->startTime)
			if($obj->startPlace)
				$title .= ", at " . $obj->startTime;
			else
				$title .= "At " . $obj->startTime;
		if($obj->rideLength)
			$title .= ", " . $obj->rideLength . " miles";
		if($obj->rideEase)
			$title .= ", " . $obj->rideEase;

		$description = $obj->description;
		$description .= " Contact: " . $obj->who;
		if($obj->tel)
			$description .= " Tel: " . $obj->tel;
		if($obj->email)
			$description .= " Email: <a href='mailto:{$obj->email}'>{$obj->email}</a>";
		
		$d1 = &$html->appendNode("div", "class='NewsSummary'");
		$d1->appendNode("div", "class='NewsSummaryPostdate'", dateString($obj->startDate) . " " . $obj->group);
		$d1->appendNode("div", "class='NewsSummaryLink'")->appendNode("a", "href='{$sys->baseUrl}/whatsOn?cmd=view&id={$obj->id}'", $title);
//		$d1->appendNode("div", "class='NewsSummaryCategory'", "Category: General");
//		$d1->appendNode("div", "class='NewsSummaryAuthor'", "Posted by: terry");
		$d1->appendNode("div", "class='NewsSummaryContent'", removeParagraphs($description));
	}

	return $html->render();
}
?>
